// Keep track of the status of all quests here
// The available quests are determined from quests.json which is downloaded from the server
Quests = function () {
    var CONTROLLER_NAME = "quests";
    var ACTION_CHECK_VERSION = "checkVersion";
    var ACTION_GET_DOLLAR = "getDollar";
    var ACTION_GET_IN_GAME_PROGRESS = "getInGameProgress";
    var ACTION_UPDATE_IN_GAME_PROGRESS = "updateInGameProgress";
    var ACTION_GET_CROSS_GAME_PROGRESS = "getCrossGameProgress";
    var ACTION_UPDATE_CROSS_GAME_PROGRESS = "updateCrossGameProgress";
    var ACTION_FETCH_RUNTIMES = "fetchRuntimes";
    var questsController = null;

    var QUESTS_BASE_URL = "https://playsimple.co/misc_web_server/quests/";

    var VERSION_CHECK_INTERVAL = 3600;
    var DOLLAR_SYNC_INTERVAL = 86400;
    var IN_GAME_SYNC_INTERVAL = 600;
    var CROSS_GAME_SYNC_INTERVAL = 180;
    var RUNTIME_FETCH_INTERVAL = 3600;

    var USE_COMPRESSED_DATA = 0;
    var USE_DECOMPRESSED_DATA = 1;
    var IN_GAME_STR = "inGame";
    var CROSS_GAME_STR = "crossGame";
    var CROSS_GAME_DECOMPRESSED_STR = "crossGameDecompressed";

    var DATA_SHARING_INDEX_GAME_ID = 0;
    var DATA_SHARING_INDEX_REF_ID = 1;
    var DATA_SHARING_INDEX_VERSION = 2;
    var DATA_SHARING_INDEX_CREATED_TS = 3;
    var DATA_SHARING_INDEX_UPDATED_TS = 4;
    var DATA_SHARING_INDEX_PUZZLE_COMPLETES = 5;
    var DATA_SHARING_INDEX_ADV_ID = 6;
    var DATA_SHARING_INDEX_COIN_WALLET = 7;
    var DATA_SHARING_INDEX_PAYER_FLAG = 8;
    var DATA_SHARING_INDEX_FB_FLAG = 9;


    /////////////////////////////////////////////////
    // The values as loaded from the downloaded jsons
    var curQuests = null;
    var allQuests = null;
    var CUR_QUESTS_FILE = "cur_quests.json";
    var ALL_QUESTS_FILE = "all_quests.json";
    /////////////////////////////////////////////////

    /////////////////////////////////////////////
    // Stored in User's quest data on client side
    var curQuestsVersion = "0";
    var allQuestsVersion = "0";
    var dollarValue = 150; // 1 Dollar is equivalent to how many coins
    var lastVersionFetch = 0;
    var lastDollarFetch = 0;
    var lastInGameFetch = 0;
    var lastInGameUpdate = 0;
    var lastCrossGameFetch = 0;
    var lastCrossGameUpdate = 0;
    var inGameLastUpdate = 0;
    var crossGameLastUpdate = 0;
    var inGameLastLocalUpdate = 0; // Update this with current timestamp whenever any quest status is updated on client.
    var crossGameLastLocalUpdate = 0;
    var inGameProgress = {};
    var crossGameProgress = {};
    /////////////////////////////////////////////
    var activeQuests = 0;
    var completedQuests = 0;
    var claimedQuests = 0;
    var buyScreenList = [];
    var homeScreenList = [];
    var buyScreenQuestMap = [];
    var adWallQuests = {};
    var adWallQuestsMap = {};
    var homeScreenQuestMap = [];
    var statusForControl = 0;
    var questsFirstLoadTimestamp;
    var crossGameDataAvailable = 0;
    var installAndDoSomethingQuests = [];
    var inGameDataAvailable = 0;
    var payerStatusForDailyQuests = 0;
    var lastDateAppOpened = 0;
    var lastDailyQuestsMap = {};
    var currentDailyQuestMap = {};
    var questsToBeAnimated = [];
    var halfProgressShow = {};
    var dailyQuestsShouldBeReset = 0;
    var cohortDaysTracking = 0;
    var firstSyncHasHappened = 0;
    var allGamesDataMap = {};
    var sortedQuestsMap = {};
    var crossGameRelatedQuests = null;
    var lastRuntimeFetch = 0;
    var lastUpdateCrossGame = 0;
    var questRuntimes = {};

    var xpromoPopupSeenInDay = 0;
    var lastDayOfUser = 0;
    var xpromoLastSurfacedTimestamp = 0;
    var xpromoSurfacingButtonLastClicked = 0;
    var lastRuntimeUpdate = 0;
    var shouldIncrementOnClick = 1;
    // * it is an array of params of funtion "redirectToStoreWithTracking"
    // * it will be used for calling redirect functionm after acknowledge from adjust click tracking is received
    var redirectDataForAdjust = [];
    var cgdqProgress = "";

    // these variable are for watch to earn xpromo surfacing popup
    var xpromoW2ePopupViews = 0;
    var xpromoW2ePopupClicks = 0;
    var xpromoW2eGameCounter = 0;
    var xpromoW2eQuestActive = null;


    var questCenterSeen = false;
    var questCenterSeenTimestamp = null;
    var questCenterLastSeenTimestamp = null;
    var questCenterLastOpenedTimestamp = null; // Timestamp representing when the QC was last opened
    var questAnimation = false;
    var questCenterLastScreen = null;
    var questCenterTrackingScreen = null;
    var questsFirstLoadTime = 0;
    var lastCrossQuestTouched = "";
    var claimQuestFtue = true;
    var questDescription = null;
    var questCompleteId = null;
    var iMessageFtueShownCount = 0;
    var curQuestVersion = 0;
    var questFtue = false;
    var fireTrackingForEachQuestMap = {};

    var dailyQuestLocked = 0;
    var lastCrossGameProgressString = "";
    var lastInGameProgressString = "";
    var dailyQuestsResetFlag = false; // Flag to skip updateAllQuests call, as it is repeatedly called while resetting all quest values
    var experimentsToChangeOnDailyBasis = null; // Status for experiments that needs to be updated only once a day
    var dailyQuestsEligibleToBeShown = {}; // this map will contain daily quest id as key, and 1/0 as value if it shown or not.

    var optimizeAdIdForLimitAdTracking = true; // Flag to use optimized ad id for LAT users. Ensures that current players continue using the existing flow and the optimization is only for new players.
    var optimizedAdIdDataSharing = null; // This id is set after fetching the shared data, based on the oldest installed game adv id. It is used when the "optimizeAdIdForLimitAdTracking" is set to true.

    var isInitialized = false;
    var flutterFileBackedUp = false;


    this.init = function () {

        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.flutterFileSetup();
        } else {
            pspnFlutterObj.cocosFileSetup();
        }

    };

    this.initContinued = function() {
        // The Quest Server is the same as login server
        questsController = new Controller(constantsObj.LOGIN_SERVER_REST, constantsObj.LOGIN_SERVER_REST);
        if (utilObj.isStaging()) {
            if (!utilObj.isWeb()) {
                QUESTS_BASE_URL = "https://playsimple.in/misc_web_server/quests/";
            }
            VERSION_CHECK_INTERVAL = 15;
            IN_GAME_SYNC_INTERVAL = 30;
            CROSS_GAME_SYNC_INTERVAL = 30;
            DOLLAR_SYNC_INTERVAL = VERSION_CHECK_INTERVAL;
            RUNTIME_FETCH_INTERVAL = 30;
        }

        load();
        if (questsObj.getPayerStatusForDailyQuests() == undefined) {
            questsObj.setPayerStatusForDailyQuests(0);
        }
        if (questsObj.getLastDateAppOpened() == undefined) {
            questsObj.setLastDateAppOpened(0);
        }
        if (firstSyncHasHappened == undefined) {
            firstSyncHasHappened = 0;
        }
    };

    this.getBuyScreenList = function () {
        return buyScreenList;
    };

    this.getXpromoW2ePopupViews = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("xpromoW2ePopupViews");
        }
        return xpromoW2ePopupViews;
    };
    this.setXpromoW2ePopupViews = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("xpromoW2ePopupViews", val);
            return;
        }
        xpromoW2ePopupViews = val;
    };
    this.getXpromoW2ePopupClicks = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("xpromoW2ePopupClicks");
        }
        return xpromoW2ePopupClicks;
    };
    this.setXpromoW2ePopupClicks = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("xpromoW2ePopupClicks", val);
            return;
        }
        xpromoW2ePopupClicks = val;
    };

    this.getXpromoW2eGameCounter = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("xpromoW2eGameCounter");
        }
        return xpromoW2eGameCounter;
    };

    this.setXpromoW2eGameCounter = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("xpromoW2eGameCounter", val);
            return;
        }
        xpromoW2eGameCounter = val;
    };

    this.getXpromoW2eQuestActive = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("xpromoW2eQuestActive");
        }
        return xpromoW2eQuestActive;
    };

    this.setXpromoW2eQuestActive = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("xpromoW2eQuestActive", val);
            return;
        }
        xpromoW2eQuestActive = val;
    };

    this.getLastDayOfUser = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("lastDayOfUser");
        }
        return lastDayOfUser;
    };

    this.setLastDayOfUser = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("lastDayOfUser", val);
            return;
        }
        lastDayOfUser = val;
    };

    this.getXpromoLastSurfacedTimestamp = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("xpromoLastSurfacedTimestamp");
        }
        return xpromoLastSurfacedTimestamp;
    };

    this.setXpromoLastSurfacedTimestamp = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("xpromoLastSurfacedTimestamp", val);
            return;
        }
        xpromoLastSurfacedTimestamp = val;
    };

    this.getXpromoSurfacingButtonLastClicked = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("xpromoSurfacingButtonLastClicked");
        }
        return xpromoSurfacingButtonLastClicked;
    };

    this.setXpromoSurfacingButtonLastClicked = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("xpromoSurfacingButtonLastClicked", val);
            return;
        }
        xpromoSurfacingButtonLastClicked = val;
    };

    this.getXpromoPopupSeenInDay = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getW2eFeatureData("xpromoPopupSeenInDay");
        }
        return xpromoPopupSeenInDay;
    };

    this.setXpromoPopupSeenInDay = function (val) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.setW2eFeatureData("xpromoPopupSeenInDay", val);
            return;
        }
        xpromoPopupSeenInDay = val;
    };

    this.getAdWallQuests = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return pspnFlutterObj.getAdwallQuests();
        }
        return adWallQuests;
    };

    this.getShouldIncrementOnClick = function () {
        return shouldIncrementOnClick;
    };

    this.setShouldIncrementOnClick = function (val) {
        shouldIncrementOnClick = val;
    };

    this.getQuestAnimation = function () {
        return questAnimation;
    };
    this.setQuestAnimation = function (val) {
        questAnimation = val;
    };

    this.getRedirectDataForAdjust = function () {
        return redirectDataForAdjust;
    };

    // Saved and loaded from Game Data
    this.setQuestCenterSeen = function (status) {
        questCenterSeen = status;
    };
    // Saved and loaded from Game Data
    this.getQuestCenterSeen = function () {
        return questCenterSeen;
    };

    // Saved and loaded from Game Data
    this.setQuestCenterSeenTimestamp = function (time) {
        questCenterSeenTimestamp = time;
    };

    // Saved and loaded from Game Data
    this.getQuestCenterSeenTimestamp = function () {
        return questCenterSeenTimestamp;
    };

    // Saved and loaded from Game Data
    this.setQuestCenterLastSeenTimestamp = function (time) {
        questCenterLastSeenTimestamp = time;
    };

    // Saved and loaded from Game Data
    this.getQuestCenterLastSeenTimestamp = function () {
        return questCenterLastSeenTimestamp;
    };

    // Saved and loaded from Game Data
    this.getQuestCenterLastScreen = function () {
        return questCenterLastScreen;
    };

    // Saved and loaded from Game Data
    this.setQuestCenterLastScreen = function (num) {
        questCenterLastScreen = num;
    };

    // Saved and loaded from Game Data
    this.getQuestCenterTrackingScreen = function () {
        return questCenterTrackingScreen;
    };

    // Saved and loaded from Game Data
    this.setQuestCenterTrackingScreen = function (num) {
        questCenterTrackingScreen = num;
    };

    // Saved and loaded from Game Data
    this.getQuestCenterLastOpenedTS = function() {
        return questCenterLastOpenedTimestamp;
    };

    // Saved and loaded from Game Data
    this.setQuestCenterLastOpenedTS = function(time) {
        questCenterLastOpenedTimestamp = time;
    };

    // Saved and loaded from Game Data - Probably not needed
    this.getQuestsFirstLoadTime = function () {
        return userObj.getCreated();
    };

    // Saved and loaded from Game Data - Probably not needed
    this.setQuestsFirstLoadTime = function () {
        questsFirstLoadTime = userObj.getCreated()
    };

    this.getLastCrossQuestTouched = function () {
        return lastCrossQuestTouched;
    };

    this.setLastCrossQuestTouched = function (val) {
        lastCrossQuestTouched = val;
    };

    this.getClaimQuestFtue = function () {
        return claimQuestFtue;
    };

    this.setClaimQuestFtue = function (val) {
        claimQuestFtue = val;
    };

    this.getQuestDescription = function () {
        return questDescription;
    };

    this.setQuestDescription = function (val) {
        questDescription = val;
    };

    this.getQuestCompleteId = function () {
        return questCompleteId;
    };

    this.setQuestCompleteId = function (val) {
        questCompleteId = val;
    };

    this.getQuestAnimation = function () {
        return questAnimation;
    };
    this.setQuestAnimation = function (val) {
        questAnimation = val;
    };

    this.getcurQuestVersion = function () {
        return curQuestVersion;
    };

    this.getiMessageFtueShownCount = function () {
        return iMessageFtueShownCount;
    };

    this.setiMessageFtueShownCount = function (val) {
        iMessageFtueShownCount = val;
    };

    this.setcurQuestVersion = function (x) {
        curQuestVersion = x;
    };

    this.getQuestFtue = function () {
        return questFtue;
    };

    this.setQuestFtue = function (val) {
        questFtue = val;
    };

    this.getLastRuntimeFetch = function () {
        return lastRuntimeFetch;
    };

    this.setLastRuntimeFetch = function (val) {
        lastRuntimeFetch = val;
    };

    this.getLastRuntimeUpdate = function () {
        return lastRuntimeUpdate;
    };

    this.setLastRuntimeUpdate = function (val) {
        lastRuntimeUpdate = val;
    };

    this.getDailyQuestLocked = function () {
        return dailyQuestLocked;
    };

    this.setDailyQuestLocked = function (val) {
        dailyQuestLocked = dailyQuestLocked || val;
    };

    this.getFireTrackingForEachQuestMap = function (trackId) {
        return fireTrackingForEachQuestMap[trackId];
    };

    this.setFireTrackingForEachQuestMap = function (trackId) {
        fireTrackingForEachQuestMap[trackId] = true;
    };

    this.resetFireTrackingForEachQuestMap = function () {
        fireTrackingForEachQuestMap = {};
    };

    this.getLastCrossGameProgressString = function () {
        return lastCrossGameProgressString;
    };

    this.setLastCrossGameProgressString = function (val) {
        lastCrossGameProgressString = val;
    };

    this.getLastInGameProgressString = function () {
        return lastInGameProgressString;
    };

    this.setLastInGameProgressString = function (val) {
        lastInGameProgressString = val;
    };

    /**
     * experiments are set on daily basis, all the experiments for which are stored in xpromoConstantsObj.EXP_TO_CHANGE_ON_DAILY_BASIS
     */
    this.setExperimentsToChangeOnDailyBasis = function () {
        // changing experiments only on day change - ensure this happens only on the first session of the day (since this flow would be active once new build is updated during the day)
        var d = new Date();
        if (xpromoUtilObj.isWebStaging()) {
            d = new Date(utilObj.getCurrentTimestampMs());
        }
        var curDate = d.getDate() + d.getMonth() + d.getYear();
        if(curDate == questsObj.getLastDateAppOpened()) {
            return;
        }

        var experimentMap = epmObj.getExperimentsMap();
        if (experimentMap != utilObj.getStringifiedJson({})) {
            experimentsToChangeOnDailyBasis = {};
            for (var i = 0; i < xpromoConstantsObj.EXP_TO_CHANGE_ON_DAILY_BASIS.length; i++) {
                var expName = xpromoConstantsObj.EXP_TO_CHANGE_ON_DAILY_BASIS[i];
                if (epmObj.doesExpExist(expName)) {
                    experimentsToChangeOnDailyBasis[expName] = epmObj.getChosenVariant(expName);
                }
            }
        }
    };

    /**
     * fetching variant of experiments which are saved in experimentsToChangeOnDailyBasis
     * @param expName pass the experiment name to get variant
     * @returns {int} return variant of experiment
     */
    this.getExperimentsToChangeOnDailyBasis = function (expName) {
        if (expName != undefined && (experimentsToChangeOnDailyBasis == null || experimentsToChangeOnDailyBasis[expName] == undefined)) {
            questsObj.setExperimentsToChangeOnDailyBasis();
        }
        if (expName == undefined || experimentsToChangeOnDailyBasis == null || experimentsToChangeOnDailyBasis[expName] == undefined) {
            return constantsObj.EXP_VAR_CONTROL;
        }
        return experimentsToChangeOnDailyBasis[expName];
    };

    this.loadLocal = function (gameData) {
        questCenterSeen = gameData.questCenterSeen;
        questCenterSeenTimestamp = gameData.questCenterSeenTimestamp;
        questCenterLastSeenTimestamp = gameData.questCenterLastSeenTimestamp;
        if (gameData.questAnimation != undefined) {
            questAnimation = gameData.questAnimation;
        }
        if (gameData.questCenterLastScreen != undefined) {
            questCenterLastScreen = gameData.questCenterLastScreen;
        }
        if (gameData.questCenterTrackingScreen != undefined) {
            questCenterTrackingScreen = gameData.questCenterTrackingScreen;
        }
        if (gameData.questsFirstLoadTime != undefined) {
            questsFirstLoadTime = gameData.questsFirstLoadTime;
        }
        if (gameData.lastCrossQuestTouched != undefined) {
            lastCrossQuestTouched = gameData.lastCrossQuestTouched;
        }
        if (gameData.claimQuestFtue != undefined) {
            claimQuestFtue = gameData.claimQuestFtue;
        }
        if (gameData.questDescription != undefined) {
            questDescription = gameData.questDescription;
        }
        if (gameData.questCompleteId != undefined) {
            questCompleteId = gameData.questCompleteId;
        }
        if (gameData.iMessageFtueShownCount != undefined) {
            iMessageFtueShownCount = gameData.iMessageFtueShownCount;
        }
        if (gameData.curQuestVersion != undefined) {
            curQuestVersion = gameData.curQuestVersion;
        }
        if (gameData.questFtue != undefined) {
            questFtue = gameData.questFtue;
        }
        if (gameData.questCenterLastOpenedTimestamp != undefined) {
            questCenterLastOpenedTimestamp = gameData.questCenterLastOpenedTimestamp;
        }
    };

    this.getXPromoGameObj = function (gameData) {
        gameData.questCenterSeen = questCenterSeen;
        gameData.questCenterSeenTimestamp = questCenterSeenTimestamp;
        gameData.questCenterLastSeenTimestamp = questCenterLastSeenTimestamp;
        gameData.questCenterLastOpenedTimestamp = questCenterLastOpenedTimestamp;
        gameData.questAnimation = questAnimation;
        gameData.questCenterLastScreen = questCenterLastScreen;
        gameData.questCenterTrackingScreen = questCenterTrackingScreen;
        gameData.questsFirstLoadTime = questsFirstLoadTime;
        gameData.lastCrossQuestTouched = lastCrossQuestTouched;
        gameData.claimQuestFtue = claimQuestFtue;
        gameData.questDescription = questDescription;
        gameData.questCompleteId = questCompleteId;
        gameData.iMessageFtueShownCount = iMessageFtueShownCount;
        gameData.curQuestVersion = curQuestVersion;
        gameData.questFtue = questFtue;

        return gameData;
    };

    this.getGameObjForSync = function (gameData) {
        gameData.questCenterSeen = questCenterSeen + "";
        gameData.questCenterSeenTimestamp = questCenterSeenTimestamp + "";
        gameData.questCenterLastSeenTimestamp = questCenterLastSeenTimestamp + "";
        gameData.questCenterLastOpenedTimestamp = questCenterLastOpenedTimestamp + "";
        return gameData;

    };

    this.resetRedirectDataForAdjust = function () {
        redirectDataForAdjust = [];
    };

    function load() {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        var questsData = ls.getItem(constantsObj.QUESTS_DATA);
        if (questsData == null || questsData == "" || questsData == undefined) {
            return false;
        }

        var quests = JSON.parse(questsData);
        xpromoUtilObj.loadFromQuests(quests);
        curQuestsVersion = quests.curQuestsVersion;
        allQuestsVersion = quests.allQuestsVersion;
        dollarValue = quests.dollarValue;
        inGameProgress = quests.inGameProgress;
        crossGameProgress = quests.crossGameProgress;
        lastVersionFetch = quests.lastVersionFetch;
        lastDollarFetch = quests.lastDollarFetch;
        lastInGameFetch = quests.lastInGameFetch;
        lastInGameUpdate = quests.lastInGameUpdate;
        lastCrossGameFetch = quests.lastCrossGameFetch;
        lastCrossGameUpdate = quests.lastCrossGameUpdate;
        inGameLastUpdate = quests.inGameLastUpdate;
        crossGameLastUpdate = quests.crossGameLastUpdate;
        inGameLastLocalUpdate = quests.inGameLastLocalUpdate;
        crossGameLastLocalUpdate = quests.crossGameLastLocalUpdate;
        payerStatusForDailyQuests = quests.payerStatusForDailyQuests;
        lastDateAppOpened = quests.lastDateAppOpened;
        halfProgressShow = quests.halfProgressShow;
        dailyQuestsShouldBeReset = quests.dailyQuestsShouldBeReset;
        firstSyncHasHappened = quests.firstSyncHasHappened;
        crossGameRelatedQuests = quests.crossGameRelatedQuests;
        xpromoW2ePopupClicks = quests.xpromoW2ePopupClicks;
        xpromoW2ePopupViews = quests.xpromoW2ePopupViews;
        xpromoW2eGameCounter = quests.xpromoW2eGameCounter;
        sortedQuestsMap = quests.sortedQuestsMap;
        lastRuntimeFetch = quests.lastRuntimeFetch;
        lastRuntimeUpdate = quests.lastRuntimeUpdate;
        lastDayOfUser = quests.lastDayOfUser;
        xpromoLastSurfacedTimestamp = quests.xpromoLastSurfacedTimestamp;
        xpromoSurfacingButtonLastClicked = quests.xpromoSurfacingButtonLastClicked;
        xpromoPopupSeenInDay = quests.xpromoPopupSeenInDay;

        xpromoW2eQuestActive = quests.xpromoW2eQuestActive;
        allGamesDataMap = quests.allGamesDataMap;
        lastCrossGameProgressString = quests.lastCrossGameProgressString;
        lastInGameProgressString = quests.lastInGameProgressString;
        experimentsToChangeOnDailyBasis = quests.experimentsToChangeOnDailyBasis;

        if (quests.fireTrackingForEachQuestMap != undefined) {
            fireTrackingForEachQuestMap = quests.fireTrackingForEachQuestMap;
        }

        if (lastRuntimeFetch == undefined) {
            lastRuntimeFetch = 0;
        }

        if (lastRuntimeUpdate == undefined) {
            lastRuntimeUpdate = 0;
        }

        if (lastDayOfUser == undefined) {
            lastDayOfUser = 0;
        }

        if (xpromoLastSurfacedTimestamp == undefined) {
            xpromoLastSurfacedTimestamp = 0;
        }

        if (xpromoSurfacingButtonLastClicked == undefined) {
            xpromoSurfacingButtonLastClicked = 0;
        }

        if (xpromoPopupSeenInDay == undefined) {
            xpromoPopupSeenInDay = 0;
        }

        if (sortedQuestsMap == undefined) {
            sortedQuestsMap = {};
        }

        if (crossGameRelatedQuests == undefined) {
            crossGameRelatedQuests = null;
        }
        if (xpromoW2ePopupClicks == undefined) {
            xpromoW2ePopupClicks = 0;
        }
        if (xpromoW2ePopupViews == undefined) {
            xpromoW2ePopupViews = 0;
        }
        if (xpromoW2eGameCounter == undefined) {
            xpromoW2eGameCounter = 0;
        }

        if (xpromoW2eQuestActive == undefined) {
            xpromoW2eQuestActive = null;
        }

        if (allGamesDataMap == undefined) {
            allGamesDataMap = {};
        }

        if (lastCrossGameProgressString == undefined) {
            lastCrossGameProgressString = "";
        }

        if (lastInGameProgressString == undefined) {
            lastInGameProgressString = "";
        }

        if (experimentsToChangeOnDailyBasis == undefined) {
            experimentsToChangeOnDailyBasis = {};
        }

        if(quests.optimizeAdIdForLimitAdTracking != undefined) {
            optimizeAdIdForLimitAdTracking = quests.optimizeAdIdForLimitAdTracking;
        } else {
            optimizeAdIdForLimitAdTracking = false;
        }

        if(quests.optimizedAdIdDataSharing != undefined) {
            optimizedAdIdDataSharing = quests.optimizedAdIdDataSharing;
        }

        var curQuestsData = ls.getItem(CUR_QUESTS_FILE);
        var allQuestsData = ls.getItem(ALL_QUESTS_FILE);
        // loggerObj.debugLog("this is unsortedQuest arr length 1 ", Math.random());
        try {
            if (allQuestsData != null && allQuestsData != "") {
                allQuests = JSON.parse(allQuestsData);
            }

            if (curQuestsData != null && curQuestsData != "") {
                curQuests = JSON.parse(curQuestsData);

                dailyQuestsObj.stringifyStreakQuests(curQuests);

                // Incase the game is being opened after a day change, this would be true for the first session of the game
                dailyQuestsObj.tryUpdateOldQuestData();
                dailyQuestsObj.mergeOlderQuestsIfEligible(curQuests);
            }


        } catch (e) {
            //loggerObj.errorLog("Invalid json stored locally");
        }

        return true;
    }

    this.save = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.save();
            return;
        }

        var quests = {};
        quests = xpromoUtilObj.saveQuestsData(quests);
        quests.curQuestsVersion = curQuestsVersion;
        quests.allQuestsVersion = allQuestsVersion;
        quests.dollarValue = dollarValue;
        quests.inGameProgress = inGameProgress;
        quests.crossGameProgress = crossGameProgress;
        quests.lastVersionFetch = lastVersionFetch;
        quests.lastDollarFetch = lastDollarFetch;
        quests.lastInGameFetch = lastInGameFetch;
        quests.lastInGameUpdate = lastInGameUpdate;
        quests.lastCrossGameFetch = lastCrossGameFetch;
        quests.lastCrossGameUpdate = lastCrossGameUpdate;
        quests.inGameLastUpdate = inGameLastUpdate;
        quests.crossGameLastUpdate = crossGameLastUpdate;
        quests.inGameLastLocalUpdate = inGameLastLocalUpdate;
        quests.crossGameLastLocalUpdate = crossGameLastLocalUpdate;
        quests.payerStatusForDailyQuests = payerStatusForDailyQuests;
        quests.lastDateAppOpened = lastDateAppOpened;
        quests.halfProgressShow = halfProgressShow;
        quests.dailyQuestsShouldBeReset = dailyQuestsShouldBeReset;
        quests.firstSyncHasHappened = firstSyncHasHappened;
        quests.fireTrackingForEachQuestMap = fireTrackingForEachQuestMap;
        quests.crossGameRelatedQuests = crossGameRelatedQuests;
        quests.sortedQuestsMap = sortedQuestsMap;
        quests.xpromoW2ePopupClicks = xpromoW2ePopupClicks;
        quests.xpromoW2eGameCounter = xpromoW2eGameCounter;
        quests.xpromoW2ePopupViews = xpromoW2ePopupViews;
        quests.lastRuntimeFetch = lastRuntimeFetch;
        quests.lastRuntimeUpdate = lastRuntimeUpdate;
        quests.lastDayOfUser = lastDayOfUser;
        quests.xpromoLastSurfacedTimestamp = xpromoLastSurfacedTimestamp;
        quests.xpromoSurfacingButtonLastClicked = xpromoSurfacingButtonLastClicked;
        quests.xpromoPopupSeenInDay = xpromoPopupSeenInDay;
        quests.xpromoW2eQuestActive = xpromoW2eQuestActive;
        quests.allGamesDataMap = allGamesDataMap;
        quests.lastCrossGameProgressString = lastCrossGameProgressString;
        quests.lastInGameProgressString = lastInGameProgressString;
        quests.experimentsToChangeOnDailyBasis = experimentsToChangeOnDailyBasis;
        quests.optimizeAdIdForLimitAdTracking = optimizeAdIdForLimitAdTracking;
        quests.optimizedAdIdDataSharing = optimizedAdIdDataSharing;

        ls.setItem(constantsObj.QUESTS_DATA, utilObj.getStringifiedJson(quests));
    };

    this.resetInGameQuestData = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.resetInGameQuestData();
            return;
        }

        inGameProgress = {};
        lastInGameFetch = 0;
        lastInGameUpdate = 0;
        inGameLastUpdate = 0;
        inGameLastLocalUpdate = 0;
        halfProgressShow = {};
        dailyQuestsShouldBeReset = 0;
        firstSyncHasHappened = 0;
        socialQuestObj.resetQuestsData();
        crossGameRelatedQuests = null;
        sortedQuestsMap = {};
        curQuestsVersion = "0";
        lastVersionFetch = 0;
        crossGameDailyQuestsObj.resetDailyQuestData();
    };
    this.syncToServer = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        var curTimestamp = utilObj.getCurrentTimestamp();
        if (curTimestamp - lastVersionFetch >= VERSION_CHECK_INTERVAL) {
            questsObj.checkQuestsVersion();
            lastVersionFetch = curTimestamp;
        }
        if (curTimestamp - lastDollarFetch >= DOLLAR_SYNC_INTERVAL) {
            questsObj.checkDollarValue();
            lastDollarFetch = curTimestamp;
        }
        if (curTimestamp - lastRuntimeFetch >= RUNTIME_FETCH_INTERVAL) {
            questsObj.fetchRuntimes();
            lastRuntimeFetch = curTimestamp;
        }
        if (curQuests != null && allQuests != null) {
            if (curTimestamp - lastInGameFetch >= IN_GAME_SYNC_INTERVAL) {
                questsObj.getInGameProgress();
                lastInGameFetch = curTimestamp;
            }
            if (curTimestamp - lastCrossGameFetch >= CROSS_GAME_SYNC_INTERVAL || crossGameDataAvailable == 0) {
                questsObj.getCrossGameProgress();
                lastCrossGameFetch = curTimestamp;
            }
        }
    };

    this.respHandler = function (action, data, success) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        var curTimestamp = utilObj.getCurrentTimestamp();
        if (action == ACTION_CHECK_VERSION) {
            questsObj.handleCheckVersion(curTimestamp, data, success);
        } else if (action == ACTION_GET_DOLLAR) {
            questsObj.handleDollarValueFetch(curTimestamp, data, success);
        } else if (action == ACTION_GET_IN_GAME_PROGRESS) {
            questsObj.handleGetInGameProgress(curTimestamp, data, success);
        } else if (action == ACTION_UPDATE_IN_GAME_PROGRESS) {
            questsObj.handleUpdateInGameProgress(curTimestamp, data, success);
        } else if (action == ACTION_GET_CROSS_GAME_PROGRESS) {
            questsObj.handleGetCrossGameProgress(curTimestamp, data, success);
        } else if (action == ACTION_UPDATE_CROSS_GAME_PROGRESS) {
            questsObj.handleUpdateCrossGameProgress(curTimestamp, data, success);
        } else if (action == ACTION_FETCH_RUNTIMES) {
            xpromoQuestRuntimesObj.handleFetchRuntimes(curTimestamp, data, success);
        }
        questsObj.save();
    };

    this.handleCheckVersion = function (curTimestamp, data, success) {
        lastVersionFetch = curTimestamp;
        if (success == false) {
            return;
        }
        // If the current quest version on prod is not the same as the one currently stored, refetch the quest info jsons
        //loggerObj.debugLog("Quest version info:", utilObj.getStringifiedJson(data));
        if (data.v != questsObj.getcurQuestVersion() || curQuests == null) {
            // Go ahead and download the updated quest jsons
            questsObj.setcurQuestVersion(data.v);
            questsObj.downloadCurQuestsInfo(data.v);
            questsObj.setQuestCenterSeen(false);
            questsObj.setQuestCenterSeenTimestamp(null);
        }
        if (data.v != allQuestsVersion || allQuests == null) {
            // Go ahead and download the updated quest jsons
            questsObj.downloadAllQuestsInfo(data.v);
        }
    };

    this.handleUpdateCrossGameProgress = function (curTimestamp, data, success) {
        lastCrossGameUpdate = curTimestamp;
        if (success == false) {
            return;
        }
    };

    this.handleDollarValueFetch = function (curTimestamp, data, success) {
        lastDollarFetch = curTimestamp;
        if (success == false) {
            return;
        }
        dollarValue = parseInt(data.d);
    };

    this.handleGetInGameProgress = function (curTimestamp, data, success) {
        // Update the initial progress values as fetched from server for this refid
        // This needs to be done after decoding.
        lastInGameFetch = curTimestamp;
        inGameDataAvailable = 1;
        if (success == false) {
            return;
        }
        if (data == null) {
            // This means that server doesn't have any new data for game progress.
            // Check if there's any additional data on client. If yes, mark serverUpdateNeeded
        } else {
            // Server has some new data for game progress. Merge the data with the client side data correctly.
            // If client had some additional data compared to server, mark serverUpdateNeeded
            questsObj.mergeInGameProgress(data.q);
        }

        questsObj.updateAllQuests();

        if (inGameLastLocalUpdate > lastInGameUpdate) {
            // Something was updated on client after the last fetch. So client has updated data compared to server.
            questsObj.updateInGameProgress();
        }
    };

    this.handleUpdateInGameProgress = function (curTimestamp, data, success) {
        lastInGameUpdate = curTimestamp;
        if (success == false) {
            return;
        }
    };

    this.handleGetCrossGameProgress = function (curTimestamp, data, success) {
        lastCrossGameFetch = curTimestamp;
        questsObj.setCrossGameDataAvailable(1);
        if (success == false) {
            return;
        }
        // Update the initial progress values as fetched from server for this advertiserId
        // This needs to be done after decoding.
        if (data == null) {
            // This means that server doesn't have any new data for game progress.
            // Check if there's any additional data on client. If yes, mark serverUpdateNeeded
        } else {
            // Server has some new data for game progress. Merge the data with the client side data correctly.
            // If client had some additional data compared to server, mark serverUpdateNeeded
            questsObj.mergeCrossGameProgress(data.q);
        }

        //loggerObj.debugLog("Updating all quests in cross game");
        questsObj.updateAllQuests();
        firstSyncHasHappened = 1;
        if (crossGameDataAvailable == 1 && (crossGameLastLocalUpdate > lastCrossGameUpdate)) {
            // Something was updated on client after the last fetch. So client has updated data compared to server.
            questsObj.updateCrossGameProgress();
        }
    };

    this.fetchRuntimes = function () {
        // loggerObj.debugLog("this is fetch runtimes");
        var requestData = {};
        requestData.env = constantsObj.ENV;
        if (lastRuntimeUpdate == undefined) {
            lastRuntimeUpdate = 0;
        }
        requestData.lastUpdate = lastRuntimeUpdate.toString();
        questsController.sendHttpRequest(CONTROLLER_NAME, ACTION_FETCH_RUNTIMES, requestData);
    };

    this.mergeInGameProgress = function (questsData, isDataCompressed) {
        // Merge the in game progress received from server with that on the client.
        var serverQuestsData = JSON.parse(questsData);
        inGameLastUpdate = serverQuestsData.l;
        serverQuestsData = questsObj.decodeGameProgress(serverQuestsData.q, IN_GAME_STR);
        // TODO:
        // Do the merge correctly. This may lead to some quests getting completed as well.
        var finalInGameProgress = serverQuestsData;
        // Some entries may be present in server and some in client, and some are common.
        for (var questId in inGameProgress) {
            var questDetails = allQuests[questId];
            if (questDetails == undefined) {
                continue;
            }
            var questInfo = inGameProgress[questId];
            var serverQuestInfo = finalInGameProgress[questId];
            if (serverQuestInfo == undefined) {
                // Server data didn't have any info about this quest.
                finalInGameProgress[questId] = questInfo;
            } else {
                var questInfoFinal = questInfo;
                if (serverQuestInfo.status == questInfo.status) {
                    // While Merging a Quest of type Level Complete
                    // If Start level on server is more than client , then keep server quest Info
                    if (questInfo.status == xpromoConstantsObj.QUEST_STARTED) {
                        if (questDetails.qt == xpromoConstantsObj.QUEST_TYPE_LEVEL_COMPLETE_IN_TIME) {
                            var startLevelServer = serverQuestInfo.params[0];
                            var startLevelClient = questInfo.params[0];
                            // will always take the data which has lower start level
                            if (startLevelServer > startLevelClient) {
                                questInfoFinal = questInfo;
                            } else {
                                questInfoFinal = serverQuestInfo;
                            }
                        } else if (questDetails.qt == xpromoConstantsObj.QUEST_TYPE_PLAY_CONSECUTIVE_DAYS) {
                            questInfoFinal = serverQuestInfo;
                        }
                    }
                    // If status is same, then client would be having the latest info. Take that.
                    finalInGameProgress[questId] = questInfoFinal;
                } else {
                    var finalQuestInfo = questInfo;
                    if (serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                        if (questInfo.status == xpromoConstantsObj.QUEST_STARTED) {
                            finalQuestInfo = serverQuestInfo;
                        }
                    }
                    if (serverQuestInfo.status == xpromoConstantsObj.QUEST_CLAIMED) {
                        if (questInfo.status == xpromoConstantsObj.QUEST_STARTED || questInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                            finalQuestInfo = serverQuestInfo;
                        }
                    }
                    finalInGameProgress[questId] = finalQuestInfo;
                }
            }
        }

        inGameProgress = finalInGameProgress;
    };

    this.mergeCrossGameProgress = function (questsData, comingFromContentProvider, lastUpdate) {
        // Merge the cross game progress received from server with that on the client.
        if (curQuests == null || allQuests == null || questsData == "" || questsData == null) {
            return;
        }
        questsObj.getAvailableCrossGameQuests();
        var serverQuestsData;
        if (comingFromContentProvider != 1) {
            var serverQuestsDataParse = JSON.parse(questsData);
            crossGameLastUpdate = serverQuestsDataParse.l;
            if (xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_DECOMPRESS_SAVE) == USE_DECOMPRESSED_DATA && (serverQuestsDataParse.d != undefined && serverQuestsDataParse.d != "")) {
                serverQuestsData = questsObj.decodeGameProgress(serverQuestsDataParse.d, CROSS_GAME_DECOMPRESSED_STR);
            } else {
                serverQuestsData = questsObj.decodeGameProgress(serverQuestsDataParse.q);
            }
            questsObj.setAllGamesData(serverQuestsDataParse.agd);
        } else {
            serverQuestsData = questsObj.decodeGameProgress(questsData, CROSS_GAME_DECOMPRESSED_STR);
        }
        // TODO:
        // Do the merge correctly. This may lead to some quests getting completed as well.
        //loggerObj.debugLog("Merging:", utilObj.getStringifiedJson(serverQuestsData), utilObj.getStringifiedJson(inGameProgress));
        var finalCrossGameProgress = crossGameProgress;
        // Some entries may be present in server and some in client, and some are common.
        for (var questId in serverQuestsData) {
            var questDetails = allQuests[questId];
            if (questDetails == undefined) {
                continue;
            }
            var questInfo = finalCrossGameProgress[questId];
            var serverQuestInfo = serverQuestsData[questId];
            if (questInfo == undefined) {
                // Client didn't have any info about this quest.
                //loggerObj.debugLog("10. Cross Game Engagement InMergeCrossGameProgress no data present on Client -> ", utilObj.getStringifiedJson(serverQuestsData));


                finalCrossGameProgress[questId] = serverQuestInfo;
                //loggerObj.debugLog("this is quest details zzzz", questDetails != undefined , crossGameProgress[questId] == undefined, firstSyncHasHappened, serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED, questId);

                if (questDetails != undefined && serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED && firstSyncHasHappened == 0) {
                    finalCrossGameProgress[questId].status = xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START;
                }

                //loggerObj.debugLog("this is final cross game ", utilObj.getStringifiedJson(serverQuestInfo), utilObj.getStringifiedJson(finalCrossGameProgress));
                // Is this an install kind of quest?
                if (questDetails != undefined && (questDetails.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME || questDetails.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME_BUYSCREEN) && questDetails.gh == constantsObj.GAME_ID) {
                    if (finalCrossGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED) {
                        finalCrossGameProgress[questId].status = xpromoConstantsObj.QUEST_COMPLETED;
                        crossGameProgress[questId] = finalCrossGameProgress[questId];
                        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                        //loggerObj.debugLog("Cross game completed case - 1");
                    }
                }
            } else {
                // Data for the quest present in both server and client
                if (serverQuestInfo.status == questInfo.status) {
                    // If status is same, then client would be having the latest info. That has been taken.
                    if (curQuests[questId] != undefined && curQuests[questId].gh == constantsObj.GAME_ID) {
                        if (curQuests[questId] != undefined && curQuests[questId].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
                            if (questInfo.params != undefined && questInfo.params[3] < serverQuestInfo.params[3]) {
                                finalCrossGameProgress[questId] = serverQuestInfo;
                            } else {
                                finalCrossGameProgress[questId] = questInfo;
                            }
                        } else {
                            finalCrossGameProgress[questId] = questInfo;
                        }

                    } else {
                        if (curQuests[questId] != undefined && curQuests[questId].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
                            if (questInfo.params != undefined && questInfo.params[3] < serverQuestInfo.params[3]) {
                                finalCrossGameProgress[questId] = serverQuestInfo;
                            } else {
                                finalCrossGameProgress[questId] = questInfo;
                            }
                        } else {
                            finalCrossGameProgress[questId] = serverQuestInfo;
                        }
                    }
                } else {
                    // handling install type quest here
                    var nonMergingQuestList = questRotationObj.getNonMergingQuestList();
                    if (questDetails.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME || questDetails.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME_BUYSCREEN) {
                        // Is this the game in which quest is gonna be completed?
                        if (questDetails.gh == constantsObj.GAME_ID) {
                            if (serverQuestInfo.status == xpromoConstantsObj.QUEST_STARTED) {
                                serverQuestInfo.status = xpromoConstantsObj.QUEST_COMPLETED;
                            }
                        } else {
                            // Is this one of the games in which the quest is shown?
                            for (var q in questDetails.gs) {
                                var gid = questDetails.gs[q];
                                if (gid == constantsObj.GAME_ID) {
                                    if (serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START && finalCrossGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED) {
                                        serverQuestInfo.status = xpromoConstantsObj.QUEST_COMPLETED;
                                        if(xPromoGameSpecificObj.isUserInteractionTrackingEnabled()) {
                                            machineLearningDataObj.resetDayXpromoInstalled();
                                        }
                                    }
                                }
                            }
                        }
                        if (questInfo.status == xpromoConstantsObj.QUEST_CLAIMED || questInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                            if (serverQuestInfo.status == xpromoConstantsObj.QUEST_CLAIMED) {
                                questInfo.status = xpromoConstantsObj.QUEST_CLAIMED;
                            } else {
                                serverQuestInfo = questInfo;
                            }
                        }
                        if (serverQuestInfo.status == xpromoConstantsObj.QUEST_NOT_STARTED && questInfo.status == xpromoConstantsObj.QUEST_STARTED) {
                            serverQuestInfo.status = xpromoConstantsObj.QUEST_STARTED;
                        }

                        if ((serverQuestInfo.status == xpromoConstantsObj.QUEST_NOT_STARTED || serverQuestInfo.status == xpromoConstantsObj.QUEST_STARTED) && questInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                            serverQuestInfo.status = xpromoConstantsObj.QUEST_COMPLETED;
                        }
                        if (nonMergingQuestList[questId] == 1 && serverQuestInfo.status == xpromoConstantsObj.QUEST_STARTED) {
                            if (comingFromContentProvider != 1) {
                                serverQuestInfo.status = xpromoConstantsObj.QUEST_NOT_STARTED;
                                nonMergingQuestList[questId] = 0;

                            } else {
                                //should correct the flow
                                continue;
                            }
                        }

                        if (questDetails.gh != constantsObj.GAME_ID) {
                            for (var q in questDetails.gs) {
                                var gid = questDetails.gs[q];
                                if (gid == constantsObj.GAME_ID) {
                                    var incentivizedElsewhere = xpromoUtilObj.isGameAlreadyMarkedForRewarding(questDetails.gh, xpromoConstantsObj.TRACK_QUEST_CENTER);
                                    if(incentivizedElsewhere && serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                                        serverQuestInfo.status = xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START;
                                    }
                                }
                            }
                        }

                        finalCrossGameProgress[questId].params = serverQuestInfo.params;
                        finalCrossGameProgress[questId].id = serverQuestInfo.id;
                        crossGameProgress[questId].params = finalCrossGameProgress[questId].params;
                        crossGameProgress[questId].id = finalCrossGameProgress[questId].id;

                        var str = questsObj.getLastCrossQuestTouched();
                        if (str != "") {
                            var elm = str.split("_");
                            if (elm[0] == "push" && serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                                if (questId == elm[1]) {
                                    statusForControl = 2;
                                }
                            }
                        }
                        questsObj.updateCrossGameQuestStatus(questId, serverQuestInfo.status);
                    } else {
                        // TODO: handle cross game engagement quests here
                        var finalQuestInfo = questInfo;
                        if (curQuests[questId] != undefined && curQuests[questId].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) == 0) {
                            if(questDetails.gh == constantsObj.GAME_ID) {
                                if((serverQuestInfo.status == xpromoConstantsObj.QUEST_STARTED) && (questInfo.status == xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START)) {
                                    finalQuestInfo = serverQuestInfo;
                                    if(questInfo.params != undefined) {
                                        finalQuestInfo.params = questInfo.params;
                                    }
                                    questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_STARTED);
                                }
                            } else {
                                if ((serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START) && (questInfo.status == xpromoConstantsObj.QUEST_NOT_STARTED)) {
                                    finalQuestInfo = serverQuestInfo;
                                    questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START);
                                } else if ((serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START) && (questInfo.status == xpromoConstantsObj.QUEST_STARTED)) {
                                    for (var q in questDetails.gs) {
                                        var gid = questDetails.gs[q];
                                        if (gid == constantsObj.GAME_ID && serverQuestInfo.params != undefined) {
                                            finalQuestInfo.params = serverQuestInfo.params;
                                        }
                                    }
                                }

                                for (var q in questDetails.gs) {
                                    var gid = questDetails.gs[q];
                                    if (gid == constantsObj.GAME_ID) {
                                        var incentivizedElsewhere = xpromoUtilObj.isGameAlreadyMarkedForRewarding(questDetails.gh, xpromoConstantsObj.TRACK_QUEST_CENTER);
                                        if(incentivizedElsewhere && serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                                            serverQuestInfo.status = xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START;
                                            questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START);
                                        }
                                    }
                                }
                            }
                        }

                        if (serverQuestInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                            if (questInfo.status == xpromoConstantsObj.QUEST_STARTED || questInfo.status == xpromoConstantsObj.QUEST_NOT_STARTED) {
                                if (curQuests[questId] != undefined && curQuests[questId].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0 && questInfo != undefined && questInfo.params != undefined && parseInt(serverQuestInfo.params[3]) < parseInt(questInfo.params[3])) {
                                    finalQuestInfo = questInfo;
                                    questsObj.updateCrossGameQuestStatus(questId, questInfo.status);

                                } else {
                                    finalQuestInfo = serverQuestInfo;
                                    questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);

                                }
                            }
                        }
                        if (serverQuestInfo.status == xpromoConstantsObj.QUEST_CLAIMED) {
                            if (questInfo.status == xpromoConstantsObj.QUEST_STARTED || questInfo.status == xpromoConstantsObj.QUEST_COMPLETED || questInfo.status == xpromoConstantsObj.QUEST_NOT_STARTED) {
                                finalQuestInfo = serverQuestInfo;
                                questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_CLAIMED);
                            }
                        }
                        if (curQuests[questId] != undefined && curQuests[questId].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0 && curQuests[questId].gh == constantsObj.GAME_ID) {
                            if (parseInt(serverQuestInfo.params[3]) > parseInt(questInfo.params[3])) {
                                finalQuestInfo = serverQuestInfo;

                            }
                            if (serverQuestInfo.status == xpromoConstantsObj.QUEST_STARTED && questInfo.status == xpromoConstantsObj.QUEST_NOT_STARTED) {
                                finalQuestInfo = serverQuestInfo;

                            }
                        }

                        finalCrossGameProgress[questId] = finalQuestInfo;

                    }
                }
            }
        }

        if (statusForControl == 2) {
            //loggerObj.debugLog("grant happened for push");
            var str = questsObj.getLastCrossQuestTouched();
            var elm = str.split("_");
            var data = allQuests[elm[1]];
            var cashReward = 0;

            if (data != undefined) {
                cashReward = questsObj.getRewardValueForQuest(data);
            }

            // TODO: Fire addCash with correct tracking.
            machineLearningDataObj.incCrossGameQuestsEarns();
            var isSuccessful = economyObj.addCash(cashReward, constantsObj.TRACK_EARN, "push", "");
            xPromoGameSpecificObj.fireEarnTracking(false, xpromoConstantsObj.TRACK_PUSH, constantsObj.TRACK_COINS, xPromoGameSpecificObj.getCashEquivalent() + "", cashReward + "", "", isSuccessful);
            questsObj.updateCrossGameQuestStatus(elm[1], xpromoConstantsObj.QUEST_CLAIMED);
            questsObj.setLastCrossQuestTouched("");
            if (cashReward > 0) {
                utilObj.showMessage(cashReward + " " + "coins added as reward");
            }
            statusForControl = 0;
        }

        crossGameProgress = finalCrossGameProgress;
    };

    this.updateCrossGameStatus = function (questId, linkId) {
        questsObj.updateCrossGameFromPushNotif(questId, xpromoConstantsObj.QUEST_STARTED, linkId);

        if ((!questsObj.isQuestShownInGame(questId)) && xPromoGameSpecificObj.getPushNotifGrant()) {
            questsObj.setLastCrossQuestTouched("push_" + questId);
        } else {
            questsObj.setLastCrossQuestTouched("");
        }
    };

    this.delayedUpdateAllQuests = function(sender) {
        questsObj.updateAllQuests();
    };

    this.updateAllQuests = function (questType) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.updateAllQuests(questType);
            return;
        }

        if (curQuests == null || dailyQuestsResetFlag == true) {
            return;
        }
        utilObj.getCurrentTimestamp();
        // This call is mainly to ensure only the status of available quests is being updated.
        questsObj.getAvailableInGameQuests(questType);
        questsObj.getAvailableCrossGameQuests(questType);
        // * updating all in game quests
        questsObj.updateAllInGameQuests();

        // * updating all cross game quests
        questsObj.updateAllCrossGameQuests();

        // * updating counts of active , completed and claimed quests here
        questsObj.updateQuestStatusCounts();
    };

    this.updateAllInGameQuests = function () {
        var userCurLevel = xPromoGameSpecificObj.getLevelForQuests();
        for (var questId in inGameProgress) {
            var questInfo = inGameProgress[questId];
            if (questInfo.status == xpromoConstantsObj.QUEST_COMPLETED
                || questInfo.status == xpromoConstantsObj.QUEST_CLAIMED
                || questInfo.status == xpromoConstantsObj.QUEST_FAILED) {
                // These are already complete one way or another
                continue;
            }
            var questDetails = curQuests[questId];
            if (questDetails != undefined) {
                var questDetailsType = questDetails.qt;
                if (dailyQuestsObj.isDailyQuestOrStreakQuest(questDetailsType)) {
                    questDetailsType = xpromoConstantsObj.QUEST_TYPE_DAILY_QUESTS;
                }
                switch (questDetailsType) {
                    case xpromoConstantsObj.QUEST_TYPE_LEVEL_COMPLETE:
                        if (userCurLevel >= parseInt(questDetails.pa)) {
                            questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_VIP_LEVEL_COMPLETE:
                        var vipCurLevel = xPromoGameSpecificObj.getVIPPuzzlesCompleted();
                        var vipLevelRequired = parseInt(questDetails.pa);
                        var vipStartLevel = parseInt(questInfo.params[0]);
                        if(vipCurLevel >= (vipLevelRequired + vipStartLevel)) {
                            questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_SECONDARY_QUEST_LEVEL:
                        if (userCurLevel >= parseInt(questDetails.pa) + parseInt(questInfo.params[0])) {
                            questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_LEVEL_COMPLETE_IN_TIME:
                        var allParams = questDetails.pa.split(":");
                        var questLevelsRequired = parseInt(allParams[0]);
                        if (userCurLevel >= questLevelsRequired + parseInt(questInfo.params[0])) {
                            questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                        }
                        break;
                }
            }
        }
    };

    this.updateAllCrossGameQuests = function () {
        var userCurLevel = xPromoGameSpecificObj.getLevelForQuests();
        if (crossGameDataAvailable == 1) {
            var allCrossGame = questsObj.getAllCurCrossGameQuests();
            for (var i in allCrossGame) {
                var questData = allCrossGame[i];
                var questId = questData.id;
                // Need to update progress for any install quests
                if ((questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME || questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME_BUYSCREEN || questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) && questData.gh == constantsObj.GAME_ID) {
                    if (crossGameProgress[questId] == undefined) {
                        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START);
                    } else {
                        if (crossGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED && questData.qt != xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) {
                            questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                            if(xPromoGameSpecificObj.isUserInteractionTrackingEnabled()) {
                                machineLearningDataObj.resetDayXpromoInstalled();
                            }
                        }
                    }
                }
            }

            for (var questId in crossGameProgress) {
                var questInfo = crossGameProgress[questId];
                if (questInfo.status == xpromoConstantsObj.QUEST_COMPLETED
                    || questInfo.status == xpromoConstantsObj.QUEST_CLAIMED
                    || questInfo.status == xpromoConstantsObj.QUEST_FAILED) {
                    // These are already complete one way or another
                    continue;
                }
                var questData = curQuests[questId];
                if (questData != undefined) {
                    var questId = questData.id;
                    if (questData.qt == xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_COMPLETE && questData.gh == constantsObj.GAME_ID) {
                        var tempParams = questData.pa.split(":");
                        if (questInfo.status == xpromoConstantsObj.QUEST_STARTED && userCurLevel >= parseInt(tempParams[3])) {
                            questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                        }
                    }
                }
            }
        }
        crossGameDailyQuestsObj.updateCGDQOptUnlockStatus();
    };

    this.decodeGameProgress = function (progressData, src) {
        var decodedData = null;
        var dataDecoded = null;
        try {
            decodedData = decodeURIComponent(progressData);
            dataDecoded = utilObj.decompressText(decodedData);
        } catch (e) {
            trackObj.trackCounter("debug", "log", "decodeError", "", "", "", "", "", "");
        }
        if (src == undefined || src == CROSS_GAME_STR || ((src == IN_GAME_STR || src == CROSS_GAME_DECOMPRESSED_STR) && (dataDecoded != null && dataDecoded.indexOf(";") == 0))) {
            decodedData = dataDecoded;
        }
        var questSplitter = ";", questInfoSplitter = ":";
        if (!(dataDecoded != null && dataDecoded.indexOf(";") == 0) && (src == IN_GAME_STR || src == CROSS_GAME_DECOMPRESSED_STR)) { //used indexof in place of contains for web compatibility on chrome
            questSplitter = "^", questInfoSplitter = "~";
        }
        var decodedProgressData = {};
        // loggerObj.debugLog("this is decompressed log");
        if (decodedData == null || decodedData == "") {
            return decodedProgressData;
        }
        decodedData = decodedData.split(questSplitter);
        for (var i in decodedData) {
            if (decodedData[i].length <= 0) {
                continue;
            }
            var questData = decodedData[i].split(questInfoSplitter);
            var questId = questData[0];
            var questEntry = {
                'id': questId,
                'status': questData[1],
                'params': [],
            };
            for (var j = 2; j < questData.length; j++) {
                questEntry.params.push(questData[j]);
            }
            decodedProgressData[questId] = questEntry;
        }
        return decodedProgressData;
    };

    this.encodeGameProgress = function (progressData, src) {
        //<quest_id>:<status>:<param1>:<param2>....;<quest_id>:<status>:<param1>....;<quest_id>:<status>....;
        // Some quests will have additional params. Some will not. Individual quest data is separated by ;
        // How many params are present for a quest, this needs to be in the json config for the quest.
        var questSplitter = ";", questInfoSplitter = ":";
        if (src == IN_GAME_STR || src == CROSS_GAME_DECOMPRESSED_STR) {
            questSplitter = "^", questInfoSplitter = "~";
        }
        var encodedData = "";
        for (var i in progressData) {
            var questInfo = progressData[i];
            encodedData += questInfo.id + questInfoSplitter + questInfo.status;
            if (questInfo.params == undefined || questInfo.params == null) {
                encodedData += questInfoSplitter;
            } else {
                for (var qp = 0; qp < questInfo.params.length; qp++) {
                    encodedData += questInfoSplitter + questInfo.params[qp];
                }
            }
            encodedData += questSplitter;
        }
        // Also use utilObj.compressText and utilObj.sanitizeStringForRest();
        if (src != IN_GAME_STR && src != CROSS_GAME_DECOMPRESSED_STR) {
            encodedData = utilObj.compressText(encodedData);
        }
        encodedData = utilObj.sanitizeStringForRest(encodedData);

        return encodedData;
    };

    this.checkQuestsVersion = function () {
        questsController.sendHttpRequest(CONTROLLER_NAME, ACTION_CHECK_VERSION, {env: constantsObj.ENV});
    };

    this.checkDollarValue = function () {
        questsController.sendHttpRequest(CONTROLLER_NAME, ACTION_GET_DOLLAR, {
            env: constantsObj.ENV,
            gid: constantsObj.GAME_ID + ""
        });
    };

    this.downloadCurQuestsInfo = function (version) {
        var curQuestsUrl = QUESTS_BASE_URL + constantsObj.ENV + "/v" + version + "/cur_quests.json";
        utilObj.downloadNormalFile(curQuestsUrl, questsObj.downloadCurQuestsComplete, version);
    };

    this.downloadCurQuestsComplete = function (questsData, version) {
        curQuestsVersion = version;
        try {
            curQuests = JSON.parse(questsData);
            dailyQuestsObj.tryUpdateOldQuestData();

            dailyQuestsObj.mergeOlderQuestsIfEligible(curQuests);
            questsObj.sortQuests(curQuests, "curQuests", true);
        } catch (e) {
            //loggerObj.debugLog("Error in Parsing curQuests data");
            return;
        }
        ls.setItem(CUR_QUESTS_FILE, questsData);
        //loggerObj.debugLog("Cur quests json:", curQuestsVersion, utilObj.getStringifiedJson(curQuests));
    };

    this.downloadAllQuestsInfo = function (version) {
        var allQuestsUrl = QUESTS_BASE_URL + constantsObj.ENV + "/v" + version + "/all_quests.json";
        utilObj.downloadNormalFile(allQuestsUrl, questsObj.downloadAllQuestsComplete, version);
    };

    this.downloadAllQuestsComplete = function (questsData, version) {
        allQuestsVersion = version;
        try {
            allQuests = JSON.parse(questsData);
        } catch (e) {
            //loggerObj.debugLog("Error in Parsing allQuests data");
            return;
        }
        ls.setItem(ALL_QUESTS_FILE, questsData);
        //loggerObj.debugLog("All quests json:", allQuestsVersion, utilObj.getStringifiedJson(allQuests));
    };

    this.getInGameProgress = function () {
        var data = {};
        data.refId = userObj.getRefId();
        if (data.refId == "" || data.refId == null || data.refId == undefined) {
            return;
        }
        data.env = constantsObj.ENV;
        data.gid = constantsObj.GAME_ID + "";
        data.lastUpdate = inGameLastUpdate + "";
        questsController.sendHttpRequest(CONTROLLER_NAME, ACTION_GET_IN_GAME_PROGRESS, data);

    };

    this.resetInGameDataAvailable = function () {
        inGameDataAvailable = 0;
    };

    this.updateInGameProgress = function (force) {
        if (inGameDataAvailable == 0 && force == undefined) {
            return;
        }
        var data = {};
        data.refId = userObj.getRefId();
        if (data.refId == "" || data.refId == null || data.refId == undefined) {
            return;
        }
        var encodedProgress = questsObj.encodeGameProgress(inGameProgress, IN_GAME_STR);
        data.env = constantsObj.ENV;
        data.gid = constantsObj.GAME_ID + "";
        data.progress = encodedProgress;
        if (encodedProgress == questsObj.getLastInGameProgressString()) {
            return;
        }
        questsObj.setLastInGameProgressString(encodedProgress);
        // loggerObj.debugLog("this is update in game progress ", utilObj.getStringifiedJson(data));
        questsController.sendHttpRequest(CONTROLLER_NAME, ACTION_UPDATE_IN_GAME_PROGRESS, data);
    };

    this.getCrossGameProgress = function () {
        if(!questsObj.allowCrossGameCalls()) {
            return;
        }

        var data = {};
        data.advertiserId = questsObj.getOptimizedAdvertiserId();
        if (data.advertiserId == "" || data.advertiserId == null || data.advertiserId == undefined) {
            return;
        }
        data.env = constantsObj.ENV;
        //loggerObj.debugLog("The adv id send is ", data.advertiserId);
        data.lastUpdate = crossGameLastUpdate + "";
        questsController.sendHttpRequest(CONTROLLER_NAME, ACTION_GET_CROSS_GAME_PROGRESS, data);
    };

    this.updateCrossGameProgress = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        if (crossGameDataAvailable == 0) {
            return;
        }

        if(!questsObj.allowCrossGameCalls()) {
            return;
        }

        var allGamesData = questsObj.encodeAllGameData(allGamesDataMap);
        //loggerObj.debugLog("Updating cross game progress:", utilObj.getStringifiedJson(crossGameProgress));
        var data = {};
        data.advertiserId = questsObj.getOptimizedAdvertiserId();
        if (data.advertiserId == "" || data.advertiserId == null || data.advertiserId == undefined) {
            return;
        }
        data.env = constantsObj.ENV;
        if (xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_DECOMPRESS_SAVE) == USE_COMPRESSED_DATA) {
            data.progress = questsObj.encodeGameProgress(crossGameProgress);
        }
        data.allGamesData = allGamesData;
        var progressDecompressed = questsObj.encodeGameProgress(crossGameProgress, CROSS_GAME_DECOMPRESSED_STR);
        data.progressDecompressed = progressDecompressed;
        // loggerObj.debugLog("this is cross game update ", utilObj.getStringifiedJson(data));
        if (progressDecompressed == questsObj.getLastCrossGameProgressString()) {
            return;
        }
        questsObj.setLastCrossGameProgressString(progressDecompressed);
        questsController.sendHttpRequest(CONTROLLER_NAME, ACTION_UPDATE_CROSS_GAME_PROGRESS, data);


    };

    this.getDecompressedProgress = function () {
        return questsObj.encodeGameProgress(crossGameProgress, CROSS_GAME_DECOMPRESSED_STR);
    };

    // For marking quests as started, completed, etc
    this.updateInGameQuestStatus = function (questId, status) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        lastSortingDoneAt = 0;

        if (inGameProgress[questId] == undefined) {
            inGameProgress[questId] = {
                'id': questId,
                'status': status
            };
        }
        if (inGameProgress[questId].status == status || inGameProgress[questId].status == xpromoConstantsObj.QUEST_CLAIMED) {//&& !questsObj.getQuestAnimation()) {
            return;
        }
        inGameProgress[questId].status = status;

        var shouldShowToaster = 0;
        var sortedQuestList = questsObj.getSortedQuestList();
        for (var quest in sortedQuestList) {
            if (sortedQuestList[quest].id == questId
                || (allQuests != null && curQuests != null && dailyQuestsObj.isDailyQuestOrStreakQuest(sortedQuestList[quest].qt) && allQuests[sortedQuestList[quest].id].qt == allQuests[questId].qt)) {
                shouldShowToaster = 1;
                break;
            }
        }

        var questInfo = curQuests[questId];
        if (questInfo == undefined || questInfo == null) {
            return;
        }
        if (curQuests[questId] != undefined && dailyQuestsObj.isDailyQuestOrStreakQuest(curQuests[questId].qt)) {
            crossGameDailyQuestsObj.setLastQuestCompletedDaily(1);
        } else {
            crossGameDailyQuestsObj.setLastQuestCompletedDaily(0);
        }

        if (status == xpromoConstantsObj.QUEST_CLAIMED && curQuests[questId] != undefined && !dailyQuestsObj.isDailyQuestOrStreakQuest(curQuests[questId].qt)) {
            questsObj.setQuestFtue(true);
            if (questId == questsObj.getQuestCompleteId()) {
                questsObj.setQuestAnimation(false);
            }
        }
        if (status == xpromoConstantsObj.QUEST_COMPLETED && shouldShowToaster == 1) {

            if (curQuests[questId] != null && dailyQuestsObj.isDailyQuestOrStreakQuest(curQuests[questId].qt) && xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_DAILY_QUEST_SWITCH_RUNTIMES) == xpromoConstantsObj.VARIANT_COMPLETE_X_LEVELS) {
                crossGameDailyQuestsObj.setDailyQuestsCompleted(1 + crossGameDailyQuestsObj.getDailyQuestsCompleted());
            }

            questsObj.setQuestAnimation(true);
            questsObj.setQuestDescription(questInfo.d);
            questsObj.setQuestCompleteId(questId);
            questsToBeAnimated.push(questId);
            if (curScreen == constantsObj.SCREEN_GAME) {
                xpromoAnimationObj.showDailyQuestCompleteToaster(gameLayer);
            } else if (questConfigObj.isLinearProgressionAvailable() && curScreen == constantsObj.SCREEN_LINEAR_PROGRESSION) {
                xpromoAnimationObj.showDailyQuestCompleteToaster(linearProgressionLayer);
            } else if (xPromoGameSpecificObj.isDigraphAvailable() && curScreen == constantsObj.SCREEN_DIGRAPH_GAME) {
                xpromoAnimationObj.showDailyQuestCompleteToaster(digraphGameLayer);
            } else if (xPromoGameSpecificObj.isDigraphAvailable() && curScreen == constantsObj.SCREEN_DIGRAPH_LINEAR_PROGRESSION) {
                xpromoAnimationObj.showDailyQuestCompleteToaster(digraphLLPLayer);
            } else if (xPromoGameSpecificObj.isDailyPuzzleAvailable() && curScreen == constantsObj.SCREEN_SPELLING_BEE) {
                xpromoAnimationObj.showDailyQuestCompleteToaster(dailySpellingBeeLayer);
            }

        }
        questsObj.dailyQuestsCompleted();
        if (questsObj.getQuestAnimation() && !xPromoGameSpecificObj.getIsPuzCompFtueShown() && xPromoGameSpecificObj.shouldShowQuestAnimation()) {
            utilObj.delayedCallback(gameLayer, questsObj.delayAnimation, 0.5, null, false);
        }
        if (curQuests[questId] != undefined && inGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED) {
            if(dailyQuestsObj.isDailyQuestOrStreakQuest(curQuests[questId].qt)) {
                trackObj.trackCounter(constantsObj.TRACK_DIALOG, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(),
                    xpromoConstantsObj.TRACK_QUEST_CENTER, questsObj.getDailyQuestTrackingStr(questId), questId + "", constantsObj.TRACK_COMPLETE, "", "");
            } else {
                trackObj.trackCounter(constantsObj.TRACK_DIALOG, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "", constantsObj.TRACK_COMPLETE, "", "", "");
            }
        }
        inGameLastLocalUpdate = utilObj.getCurrentTimestamp();
        questsObj.updateQuestStatusCounts();
        //loggerObj.debugLog("quest Status for quest Id is ", questId, inGameProgress[questId].status, status);
    };

    this.delayAnimation = function () {
        if(curScreen == constantsObj.SCREEN_GAME) {
            gameLayer.showQuestAnimation(questsObj.getQuestDescription());
        } else if (xPromoGameSpecificObj.isDailyPuzzleAvailable() && curScreen == constantsObj.SCREEN_SPELLING_BEE) {
            dailySpellingBeeLayer.showQuestAnimation(questsObj.getQuestDescription())
        }
    };

    this.updateCrossGameQuestStatus = function (questId, status) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        var shouldShowToaster = 0;

        if (crossGameProgress[questId] == undefined) {
            crossGameProgress[questId] = {
                'id': questId,
                'status': status,
            }
        } else {
            if (crossGameProgress[questId].status == status) {
                return;
            }
            crossGameProgress[questId].status = status;
        }

        var adwallQuests = questsObj.getAdWallQuests();

        // not updating related quests for adwall quests
        // TODO: will have to change for xpromo after w2e feature
        for (var quest in adwallQuests) {
            if (adwallQuests[quest].id == questId) {
                return;
            }
        }

        if (!questsObj.isQuestShownInGame(questId)) {
            return;
        }

        var sortedQuestList = questsObj.getSortedQuestList();
        for (var quest in sortedQuestList) {
            if (sortedQuestList[quest].id == questId) {
                shouldShowToaster = 1;
                break;
            }
        }

        if (curQuests == null || allQuests == null || curQuests[questId] == undefined || allQuests[questId] == undefined) {
            return;
        }

        if (curQuests[questId] != undefined && curQuests[questId].isAdwallQuest == 1) {
            shouldShowToaster = 0;
        }

        // Not returning from this flow if the quest is to be marked as claimed. Let the BFS run in that scenario.
        if (curQuests[questId].isAdwallQuest != 1 && shouldShowToaster == 0 && status != xpromoConstantsObj.QUEST_CLAIMED) {
            return;
        }

        var shouldUpdate = false;

        for (var gs = 0; gs < allQuests[questId]["gs"].length; gs++) {
            if (allQuests[questId]["gs"][gs] == constantsObj.GAME_ID) {
                shouldUpdate = true;
            }
        }
        if (questId == "nil" || status == undefined || status.length > 1 || curQuests[questId] == undefined) {
            return;
        }

        // * updating related quest
        // * implemented a bfs for updating all related quests
        var visitedQuest = {};
        var questQueue = [];
        questQueue.push(questId);
        while (questQueue.length > 0) {
            var questInQueue = questQueue.pop();
            if (visitedQuest[questInQueue] == 1) {
                continue;
            }
            visitedQuest[questInQueue] = 1;
            var questDetails = allQuests[questInQueue];
            if (questDetails == undefined) {
                continue;
            }

            if (crossGameProgress[questInQueue] == undefined) {
                crossGameProgress[questInQueue] = {
                    'id': questInQueue,
                    'params': []
                };
            }
            if (crossGameRelatedQuests == null) {
                return;
            }
            var relatedQuests = crossGameRelatedQuests[questInQueue];
            if (relatedQuests != undefined) {
                for (var re = 0; re < relatedQuests.length; re++) {
                    if (relatedQuests[re].length == 0) {
                        continue;
                    }
                    if (visitedQuest[relatedQuests[re]] != 1) {
                        questQueue.push(relatedQuests[re]);
                    }
                }
            }

            if (curQuests[questInQueue] == undefined) {
                if (status == xpromoConstantsObj.QUEST_COMPLETED || status == xpromoConstantsObj.QUEST_CLAIMED || status == xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START) {
                    crossGameProgress[questInQueue].status = xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START;
                } else {
                    crossGameProgress[questInQueue].status = status;
                }
            } else {
                crossGameProgress[questInQueue].status = status;
            }
        }

        crossGameProgress[questId].status = status;
        var questInfo = curQuests[questId];
        if (questInfo == undefined || questInfo == null) {
            return;
        }
        if (questInfo.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME || questInfo.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME_BUYSCREEN) {
            if (crossGameProgress[questId].params != undefined && (questInfo.b == xpromoConstantsObj.QUEST_BUCKET_BUY_SCREEN || questInfo.b == xpromoConstantsObj.QUEST_HOME_SCREEN)) {
                crossGameProgress[questId].params[0] = constantsObj.GAME_ID;
            } else {
                crossGameProgress[questId].params = [constantsObj.GAME_ID];
            }
        }

        if (status == xpromoConstantsObj.QUEST_CLAIMED) {
            questsObj.setQuestFtue(true);
        }

        if (status == xpromoConstantsObj.QUEST_COMPLETED && questsObj.isQuestShownInGame(questId) && shouldShowToaster == 1) {
            questsObj.setQuestAnimation(true);
            questsObj.setQuestDescription(questInfo.d);
            if (dailyQuestsObj.isDailyQuestOrStreakQuest(curQuests[questId].qt)) {
                questsObj.setQuestCompleteId(questId);
            }
            questsObj.dailyQuestsCompleted();
            questsToBeAnimated.push(questId);
            if (curScreen == constantsObj.SCREEN_GAME) {
                xpromoAnimationObj.showDailyQuestCompleteToaster(gameLayer);
            } else if (questConfigObj.isLinearProgressionAvailable() && curScreen == constantsObj.SCREEN_LINEAR_PROGRESSION) {
                xpromoAnimationObj.showDailyQuestCompleteToaster(linearProgressionLayer);
            }
        }
        if (questsObj.getQuestAnimation() && !xPromoGameSpecificObj.getIsPuzCompFtueShown() && xPromoGameSpecificObj.shouldShowQuestAnimation()) {
            utilObj.delayedCallback(gameLayer, questsObj.delayAnimation, 0.5, null, false);
        }

        crossGameLastLocalUpdate = utilObj.getCurrentTimestamp();
        if (status == xpromoConstantsObj.QUEST_COMPLETED) {
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId, xpromoConstantsObj.trackQuestStatus[status], "", "", "");
        }
        questsObj.updateCrossGameProgress();
        questsObj.updateQuestStatusCounts();
    };

    this.addQuestsToBeAnimated = function (questId) {
        questsToBeAnimated.push(questId);
        if (curScreen == constantsObj.SCREEN_GAME) {
            xpromoAnimationObj.showDailyQuestCompleteToaster(gameLayer);
        } else if (questConfigObj.isLinearProgressionAvailable() && curScreen == constantsObj.SCREEN_LINEAR_PROGRESSION) {
            xpromoAnimationObj.showDailyQuestCompleteToaster(linearProgressionLayer);
        } else if (xPromoGameSpecificObj.isDigraphAvailable() && curScreen == constantsObj.SCREEN_DIGRAPH_GAME) {
            xpromoAnimationObj.showDailyQuestCompleteToaster(digraphGameLayer);
        } else if (xPromoGameSpecificObj.isDigraphAvailable() && curScreen == constantsObj.SCREEN_DIGRAPH_LINEAR_PROGRESSION) {
            xpromoAnimationObj.showDailyQuestCompleteToaster(digraphLLPLayer);
        } else if (xPromoGameSpecificObj.isDailyPuzzleAvailable() && curScreen == constantsObj.SCREEN_SPELLING_BEE) {
            xpromoAnimationObj.showDailyQuestCompleteToaster(dailySpellingBeeLayer);
        }
        questsObj.updateQuestStatusCounts();
    };

    this.getQuestTrackingInfo = function () {
        var questTrackingStr = "";
        questTrackingStr += activeQuests + "_" + completedQuests + "_" + claimedQuests;
        return questTrackingStr;
    };

    this.updateQuestStatusCounts = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.updateQuestStatusCounts();
            return;
        }

        if (curQuests == null || allQuests == null || !xPromoGameSpecificObj.isQuestCenterUnlocked()) {
            return;
        }
        activeQuests = 0;
        completedQuests = 0;
        claimedQuests = 0;

        var availableQuestsMap = {};
        var sortedQuestList = questsObj.getSortedQuestList();
        for (var quest in sortedQuestList) {
            availableQuestsMap[sortedQuestList[quest].id] = 1;
        }

        // * incrementing count for loop back engagement quest
        var CGDQCompleteTime = crossGameDailyQuestsObj.getCrossGameDailyQuestCompleteTimestamp();
        if (crossGameDailyQuestsObj.getCrossGameDailyQuestCompletedData() != null && crossGameDailyQuestsObj.getCrossGameDailyQuestCompletedData() != undefined && CGDQCompleteTime != 0 && utilObj.getCurrentTimestamp() - CGDQCompleteTime >= xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_LOOP_BACK_TOASTER)) {
            completedQuests += 1;
        }

        // * counting all in game quests
        for (var questId in inGameProgress) {
            var questInfo = inGameProgress[questId];
            if (questInfo != undefined && questInfo != null && curQuests[questId] != undefined && availableQuestsMap[questId] != undefined) {
                if (questInfo.status == xpromoConstantsObj.QUEST_STARTED) {
                    activeQuests += 1;
                }
                if (questInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                    completedQuests += 1;
                }
                if (questInfo.status == xpromoConstantsObj.QUEST_CLAIMED) {
                    claimedQuests += 1;
                }
            }
        }

        // * counting all cross game quests
        for (var questId in crossGameProgress) {
            var questInfo = crossGameProgress[questId];
            if (questInfo != undefined && questInfo != null && questsObj.isQuestShownInGame(questId) && curQuests[questId] != undefined && availableQuestsMap[questId] != undefined) {
                if (questInfo.status == xpromoConstantsObj.QUEST_STARTED) {
                    activeQuests += 1;
                }
                if (questInfo.status == xpromoConstantsObj.QUEST_COMPLETED) {
                    completedQuests += 1;
                }
                if (questInfo.status == xpromoConstantsObj.QUEST_CLAIMED) {
                    claimedQuests += 1;
                }
            }
        }
        // * adding count for related quest
        var curRequest = requestsObj.getCurrentActiveRequest()[0];
        if (curRequest != undefined && curRequest != null) {
            if (curRequest.status == xpromoConstantsObj.REQUEST_STARTED || curRequest.status == xpromoConstantsObj.REQUEST_PLAYED) {
                activeQuests += 1;
            }
            if (curRequest.status == xpromoConstantsObj.REQUEST_COMPLETED) {
                completedQuests += 1;
            }
        }

        var layersToChange = [gameLayer];
        var llpLayersToChange = [];

        if(xPromoGameSpecificObj.isSettingsLayerEnabled()) {
            layersToChange.push(settingsLayer);
        }

        if(questConfigObj.isLinearProgressionAvailable()) {
            llpLayersToChange.push(linearProgressionLayer);
        }

        if(xPromoGameSpecificObj.isDigraphAvailable()) {
            layersToChange.push(digraphGameLayer);
            llpLayersToChange.push(digraphLLPLayer);
        }

        if(xPromoGameSpecificObj.isDailyPuzzleAvailable()) {
            layersToChange.push(dailySpellingBeeLayer);
        }

        if(xPromoGameSpecificObj.showQuestNotifOnHomeScreen()) {
            layersToChange.push(homeLayer);
        }

        // * changing jewel notif as per number of completed quests
        if (questsObj.getCompletedQuests() > 0 && !questsObj.showTagOnQuestCenterBubble()) {
            questsObj.showQuestsCompletedJewelNotif(layersToChange, llpLayersToChange);
            if (xPromoGameSpecificObj.isInButtonFillVariant()) {
                xPromoGameSpecificObj.showFilledQuestButton(true);
            }
        } else if (userObj.isUserCreated() || (completedQuests == 0 && questsObj.getQuestCenterSeen() && !questsObj.showTagOnQuestCenterBubble())) {
            questsObj.hideQuestCompletedJewelNotif(layersToChange, llpLayersToChange);
            if (xPromoGameSpecificObj.isInButtonFillVariant()) {
                xPromoGameSpecificObj.showFilledQuestButton(false);
            }
        } else {
            if (questsObj.getQuestCenterSeenTimestamp() == null) {
                questsObj.setQuestCenterSeenTimestamp(utilObj.getCurrentTimestamp());
            }
            questsObj.showNewTagJewelnotif(layersToChange, llpLayersToChange);
            if (xPromoGameSpecificObj.isInButtonFillVariant()) {
                xPromoGameSpecificObj.showFilledQuestButton(true);
            }
        }
    };

    this.updateCrossGameFromPushNotif = function (questId, status, linkId) {
        if (curQuests != undefined && curQuests[questId] != undefined && curQuests[questId].status == xpromoConstantsObj.QUEST_NOT_STARTED) {
            questsObj.updateCrossGameQuestStatus(questId, status);
        } else if (xPromoGameSpecificObj.shouldUpdateJewelNotif()){
            if (questsObj.getCompletedQuests() > 0) {
                if (gameObj.getQuestCenterFill()) {

                } else {
                    gameObj.checkAndUpdateTrackingNotif();
                }
                utilObj.setVisible(gameLayer.questFillIcon, true);
                gameObj.setQuestCenterFill(true);
                utilObj.setVisible(gameLayer.questIcon, false);
                utilObj.setVisible(gameLayer.questCenterButtonTrans, true)
                utilObj.setVisible(gameLayer.questCenterButton, false);
            }
        }
    };

    this.redirectToPlaystore = function (name) {
        var shareLink = questsObj.getGameShareLinkForNotif(name);
        utilObj.openUrl(shareLink);
    };

    this.getGameShareLinkForNotif = function (game) {
        var msg1 = "https://playsimple.in/" + game + "?g=%GROUP%&l=%LABEL%";
        msg1 = msg1.replace("%GROUP%", "quest_notif_" + constantsObj.GAME_ID);
        var label = constantsObj.GAME_ID + ":" + userObj.getRefId();
        msg1 = msg1.replace("%LABEL%", label);
        return msg1;
    };

    this.getCompletedQuests = function () {
        return completedQuests;
    };

    this.setCompletedQuests = function (val) {
        completedQuests = val;
    };

    this.getClaimedQuests = function () {
        return claimedQuests;
    };

    this.setClaimedQuests = function (val) {
        claimedQuests = val;
    };

    this.getActiveQuests = function () {
        return activeQuests;
    };

    this.setActiveQuests = function (val) {
        activeQuests = val;
    };

    this.claimInGameQuestReward = function (questId) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        if (!utilObj.isOnline()) {
            utilObj.showMessage(stringsObj.not_online);
            return;
        }
        // Check the status of the questId.
        // It should be marked as completed and no other state.
        var questData = curQuests[questId];
        if (questData == undefined || questData == null) {
            return;
        }
        if (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_COMPLETED || questData == undefined) {
            utilObj.showMessage(stringsObj.error_generic);
            return;
        }

        var cashReward = questsObj.getRewardValueForQuest(questData);
        machineLearningDataObj.incInGameQuestsEarns();
        // TODO: Fire addCash with correct tracking.
        var success = economyObj.addCash(cashReward, constantsObj.TRACK_EARN, xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "");
        xPromoGameSpecificObj.fireEarnTracking(false, constantsObj.TRACK_QUEST_CENTER, constantsObj.TRACK_COINS, xPromoGameSpecificObj.getCashEquivalent() + "", cashReward + "", questId + "", success);
        if(xPromoGameSpecificObj.isUserInteractionTrackingEnabled()) {
            machineLearningDataObj.updateQuestCenterEarns(cashReward);
        }

        // If claim can be done, then make a request to the server (after updating status as claimed temporarily)
        // When success response is received from server (meaning server now thinks reward is claimed), grant the reward.
        // Can consider allowing the grant without waiting for server response - opens it up to exploit for a small percentage of users.
        questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_CLAIMED);
        if (curQuests[questId].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_SECONDARY_QUEST_LEVEL) == 0) {
            crossGameDailyQuestsObj.setSecondaryQuestClaimed(1);
        }
        questsObj.getInGameProgress(); //This fetches latest data and merges it, and then updates the data on server.
    };

    this.claimCrossGameQuestReward = function (params) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        var questId = params.questId;
        if (!utilObj.isOnline()) {
            utilObj.showMessage(stringsObj.not_online);
            return;
        }

        // Check the status of the questId.
        // It should be marked as completed and no other state.
        var questData = curQuests[questId];
        if (questData == null || questData == undefined) {
            return;
        }
        if (crossGameProgress[questId] == undefined || crossGameProgress[questId].status != xpromoConstantsObj.QUEST_COMPLETED || questData == undefined) {
            utilObj.showMessage(stringsObj.error_generic);
            return;
        }

        var cashReward = questsObj.getRewardValueForQuest(questData);
        // TODO: Fire addCash with correct tracking.
        trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId, constantsObj.TRACK_CLAIM, "", "", "");
        machineLearningDataObj.incCrossGameQuestsEarns();
        if (curQuests[questId].isSocialQuest == 1) {
            socialQuestObj.setChangeFriendInSocialQuest(1);
        }
        var isSuccessful = economyObj.addCash(cashReward, constantsObj.TRACK_EARN, xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "");
        xPromoGameSpecificObj.fireEarnTracking(false, constantsObj.TRACK_QUEST_CENTER, constantsObj.TRACK_COINS, xPromoGameSpecificObj.getCashEquivalent() + "", cashReward + "", questId + "", isSuccessful);
        if(xPromoGameSpecificObj.isUserInteractionTrackingEnabled()) {
            machineLearningDataObj.updateQuestCenterEarns(cashReward);
        }

        // If claim can be done, then make a request to the server (after updating status as claimed temporarily)
        // When success response is received from server (meaning server now thinks reward is claimed), grant the reward.
        // Can consider allowing the grant without waiting for server response - opens it up to exploit for a small percentage of users.
        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_CLAIMED);
        questsObj.getCrossGameProgress(); //This fetches latest data and merges it, and then updates the data on server.

        // * updating progress for cgdq on claiming
        if (params.questType.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
            crossGameDailyQuestsObj.setCGDQClaimed(1);
            crossGameDailyQuestsObj.updateCrossGameDailyQuestProgress(params.questId);
        }
    };

    this.isQuestInGame = function (questId) {
        if (allQuests != null) {
            var questData = allQuests[questId];
            if (questData != undefined) {
                if (questData.cg == 1) {
                    return false;
                } else {
                    return true;
                }
            }
        }
        return constantsObj.NOT_FOUND;
    };

    this.isQuestShownInGame = function (questId) {
        if (allQuests != null) {
            var questData = allQuests[questId];
            if (questData != undefined) {
                for (var i in questData.gs) {
                    if (questData.gs[i] == constantsObj.GAME_ID) {
                        return true;
                    }
                }
                return false;
            }
        }
        return false;
    };

    this.isQuestAvailableInCountry = function (countryList) {
        if (countryList.length == 0 || countryList[0] == "") {
            return true;
        }
        var country = utilObj.getCountry();
        var matchFound = false;
        var othersFound = false;
        var allFound = false;
        for (var i in countryList) {
            if (countryList[i] == country) {
                matchFound = true;
                break;
            } else if (countryList[i] == "others") {
                othersFound = true;
            } else if (countryList[i] == "others") {
                allFound = true;
                break;
            }
        }

        if (matchFound || allFound) {
            return true;
        }

        if (!matchFound && othersFound) {
            return true;
        }

        return false;
    };

    this.isQuestAvailableOnClient = function (clients) {
        if (utilObj.isStaging() && utilObj.isWeb()) {
            return true;
        }
        if (clients.length == 0) {
            return true;
        }
        var client = utilObj.getTrackingClient();
        var matchFound = false;
        for (var i in clients) {
            if (client == clients[i]) {
                matchFound = true;
                break;
            }
        }
        return matchFound;
    };

    this.getAvailableInGameQuests = function (questType) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return [];
        }

        var availableQuests = [];
        currentDailyQuestMap = {};
        lastDailyQuestsMap = {};
        dailyQuestsEligibleToBeShown = {};
        if (curQuests == null || allQuests == null || !xPromoGameSpecificObj.isQuestCenterUnlocked()) {
            return availableQuests;
        }

        var candidateForBucket = {};
        var candidatesInBucket = {};
        var vipLevelQuestCandidates = [];
        var curTimestamp = utilObj.getCurrentTimestamp();
        var userCurLevel = xPromoGameSpecificObj.getLevelForQuests();
        var consLogins = parseInt(xPromoGameSpecificObj.getConsecutiveLogins());
        if (questsObj.getQuestsFirstLoadTime() == 0) {
            questsObj.setQuestsFirstLoadTime();
        }
        var curQuestsArr = questsObj.sortQuests(curQuests, "curQuests");

        if(questType == undefined || dailyQuestsObj.isDailyQuestOrStreakQuest(questType)) {
            dailyQuestsObj.resetQuestLevelMap();
        }

        for (var quest in curQuestsArr) {
            var questId = curQuestsArr[quest].id;
            var questData = curQuests[questId];

            if(questData == undefined) {
                // trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER, "questData-undefined", questId + "", questsObj.getcurQuestVersion() + "", "", "", "", "");
                continue;
            }

            if (questType != undefined) {
                if (questType == xpromoConstantsObj.QUEST_TYPE_PUZZLE_COMPLETE) {
                    if (questData.qt != xpromoConstantsObj.QUEST_TYPE_LEVEL_COMPLETE && questData.qt != xpromoConstantsObj.QUEST_TYPE_LEVEL_COMPLETE_IN_TIME) {
                        continue;

                    }
                } else if (questType == xpromoConstantsObj.QUEST_TYPE_VIP_LEVEL_COMPLETE && questData.qt != xpromoConstantsObj.QUEST_TYPE_VIP_LEVEL_COMPLETE) {
                    continue;
                }
            }
            if (questsObj.isQuestInGame(questId)
                && questData.gh == constantsObj.GAME_ID
                && allQuests[questId] != undefined && curQuests[questId] != undefined
                && questsObj.isQuestAvailableInCountry(questData.col)
                && questsObj.isQuestAvailableOnClient(questData.cl)
                && (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED
                    || dailyQuestsObj.isDailyQuestOrStreakQuest(questData.qt)
                    || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_SECONDARY_QUEST_LEVEL) == 0)) {
                var questBucket = questData.b;
                var questPriority = questData.p;
                questData.id = questId;
                if (!questsObj.isEligibleForQuest(questData)) {
                    continue;
                }
                if (candidatesInBucket[questBucket] == undefined) {
                    candidatesInBucket[questBucket] = [];
                }
                switch (questData.qt) {
                    case xpromoConstantsObj.QUEST_TYPE_LEVEL_COMPLETE:
                        var questLevelRequired = parseInt(questData.pa);
                        var lastChosenRequiredLevel = constantsObj.NOT_FOUND;
                        if (candidatesInBucket[questBucket].length > 0) {
                            // TODO - Need to check if this bucket ever goes above one!!
                            lastChosenRequiredLevel = parseInt(candidatesInBucket[questBucket].pa);
                        }
                        //loggerObj.debugLog("1.Progress status:", utilObj.getStringifiedJson(inGameProgress[questId]), questLevelRequired, userCurLevel, questId);
                        if (questLevelRequired > userCurLevel
                            || (inGameProgress[questId] != undefined && inGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED)
                            || (inGameProgress[questId] != undefined && inGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED)) {
                            // This is still an open quest
                            if (lastChosenRequiredLevel == constantsObj.NOT_FOUND || lastChosenRequiredLevel > questLevelRequired) {
                                var questStatus = xpromoConstantsObj.QUEST_STARTED;
                                if (inGameProgress[questId] != undefined) {
                                    questStatus = inGameProgress[questId].status;
                                }
                                questData.status = questStatus;
                                questData.curProgress = userCurLevel;
                                questData.reqProgress = questLevelRequired;
                                questData.timeLimit = 0;
                                questData.params = [userCurLevel];
                                candidatesInBucket[questBucket].push(questData);
                            }
                        }
                        break;

                    case xpromoConstantsObj.QUEST_TYPE_VIP_LEVEL_COMPLETE:
                        if (userCurLevel >= xPromoGameSpecificObj.getVIPPuzzleUnlockLevel() && // Unlock after N puzzle completes, AND
                            candidatesInBucket[questBucket].length == 0) { // Bucket should not have any quests at present

                            var subscriptionStatus = xPromoGameSpecificObj.checkSubscriptionStatus();

                            //Should be shown if its undefined and meets the subscriber / non-subscriber requirements - creating new quest is of lower priority and is added to the end of the list
                            if(inGameProgress[questId] == undefined &&
                                ((questData.pf == 1 &&  subscriptionStatus == constantsObj.SUBSCRIPTION_ACTIVE) ||
                                    (questData.pf == 0 && subscriptionStatus == constantsObj.SUBSCRIPTION_INACTIVE))) {

                                vipLevelQuestCandidates.push(questData);

                            // Should be shown if the quest is already started or completed, this is of higher priority than starting a new quest and is added to the beginning of the array.
                            } else if (inGameProgress[questId] != undefined &&
                                (inGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED ||
                                    inGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED)) {

                                vipLevelQuestCandidates.unshift(questData);

                            }
                        }

                        break;

                    // this quest will be shown only after cross game daily quest is completed
                    case xpromoConstantsObj.QUEST_TYPE_SECONDARY_QUEST_LEVEL:
                        if (questData.qt == xpromoConstantsObj.QUEST_TYPE_SECONDARY_QUEST_LEVEL && crossGameDailyQuestsObj.getCrossGameDailyQuestCompletedFlag() == 0) {
                            continue;
                        }
                        var questLevelsRequired = parseInt(questData.pa);
                        var lastChosenRequiredLevels = constantsObj.NOT_FOUND;
                        if (candidatesInBucket[questBucket].length > 0) {
                            lastChosenRequiredLevels = parseInt(candidatesInBucket[questBucket].pa);
                        }

                        // This is still an open quest
                        if (lastChosenRequiredLevels == constantsObj.NOT_FOUND || lastChosenRequiredLevels > questLevelsRequired) {
                            var questStatus = xpromoConstantsObj.QUEST_STARTED;
                            // var timeLimit = parseInt(allParams[1]) * constantsObj.SECONDS_IN_HOUR;
                            // var startTime = curTimestamp;
                            var startLevel = userCurLevel;
                            if (inGameProgress[questId] != undefined) {
                                startLevel = inGameProgress[questId].params[0];
                                questStatus = inGameProgress[questId].status;
                            }
                            questData.status = questStatus;
                            questData.curProgress = userCurLevel - startLevel;
                            questData.reqProgress = questLevelsRequired;
                            questData.params = [startLevel];
                            questData.engagementType = xpromoConstantsObj.TRACK_SECONDARY_QUEST;
                            if (userCurLevel >= startLevel + questLevelsRequired) {
                                questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                            }
                            // for retaining questz: inorder to get tick on secondary quest we do following for var 1 and 2(inside daily quest hood)
                            if (!(questStatus == xpromoConstantsObj.QUEST_CLAIMED && crossGameDailyQuestsObj.getCurrentCGDQExp() == constantsObj.EXP_VAR_3)) {
                                candidatesInBucket[questBucket].push(questData);
                                //candidateForBucket[questBucket] = questData;
                            }
                        }
                        break;

                    case xpromoConstantsObj.QUEST_TYPE_LEVEL_COMPLETE_IN_TIME:
                        var allParams = questData.pa.split(":");
                        var questLevelsRequired = parseInt(allParams[0]);
                        var lastChosenRequiredLevels = constantsObj.NOT_FOUND;
                        if (candidatesInBucket[questBucket].length > 0 && candidatesInBucket[questBucket].pa != undefined) {
                            var oldParams = candidatesInBucket[questBucket].pa.split(":");
                            lastChosenRequiredLevels = parseInt(oldParams[0]);
                        }
                        if (inGameProgress[questId] == undefined || inGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED || inGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED) {
                            // This is still an open quest
                            if (lastChosenRequiredLevels == constantsObj.NOT_FOUND || lastChosenRequiredLevels > questLevelsRequired) {
                                var questStatus = xpromoConstantsObj.QUEST_STARTED;
                                var timeLimit = parseInt(allParams[1]) * constantsObj.SECONDS_IN_HOUR;
                                var startTime = curTimestamp;
                                var startLevel = userCurLevel;
                                if (inGameProgress[questId] != undefined) {
                                    if (inGameProgress[questId].status != xpromoConstantsObj.QUEST_COMPLETED && (curTimestamp - inGameProgress[questId].params[1]) >= timeLimit) {
                                        // User couldn't complete this quest in time.
                                        // Restart it for beginning.
                                        var userCurProgress = userCurLevel - inGameProgress[questId].params[0];
                                        trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getTrackingBucketId(), gameObj.getTrackingPuzId(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "", constantsObj.TRACK_FAILED, userCurProgress + "", "1", "");
                                        trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getTrackingBucketId(), gameObj.getTrackingPuzId(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "", constantsObj.TRACK_START, "0", "1", "");
                                        inGameProgress[questId].status = xpromoConstantsObj.QUEST_STARTED;
                                        inGameProgress[questId].params = [userCurLevel, curTimestamp];
                                        inGameLastLocalUpdate = curTimestamp;
                                        if (userCurProgress >= questLevelsRequired) {
                                            questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                                        }
                                    } else {
                                        startTime = inGameProgress[questId].params[1];
                                        startLevel = inGameProgress[questId].params[0];
                                    }
                                    questStatus = inGameProgress[questId].status;
                                }
                                questData.status = questStatus;
                                questData.curProgress = userCurLevel - startLevel;
                                questData.reqProgress = questLevelsRequired;
                                questData.timeLimit = timeLimit;
                                questData.params = [startLevel, startTime];
                                candidatesInBucket[questBucket].push(questData);
                                //candidateForBucket[questBucket] = questData;
                            }
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_PLAY_CONSECUTIVE_DAYS:
                        var consDaysRequired = parseInt(questData.pa);
                        var lastChosenConsDays = constantsObj.NOT_FOUND;
                        if (candidatesInBucket[questBucket].length > 0) {
                            lastChosenConsDays = parseInt(candidatesInBucket[questBucket].pa);
                        }
                        if (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
                            if (lastChosenConsDays == constantsObj.NOT_FOUND || lastChosenConsDays > consDaysRequired) {
                                var questStatus = xpromoConstantsObj.QUEST_STARTED;
                                var consDaysNeededForCompletion = parseInt(consLogins + consDaysRequired - 1);
                                if (inGameProgress[questId] != undefined) {
                                    questStatus = inGameProgress[questId].status;
                                    var originalConsLogins = parseInt(inGameProgress[questId].params[0]);
                                    var originalConsLoginsNeeded = parseInt(inGameProgress[questId].params[2]);
                                    if (originalConsLogins > consLogins) {
                                        // Resetting as consLogins value has become lower => that user didn't come for some days.
                                        trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getTrackingBucketId(), gameObj.getTrackingPuzId(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "", constantsObj.TRACK_FAILED, "", "1", "");
                                        trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getTrackingBucketId(), gameObj.getTrackingPuzId(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "", constantsObj.TRACK_START, "", "1", "");
                                        questData.params = [consLogins, 0, consDaysNeededForCompletion];
                                        inGameProgress[questId].params = questData.params;
                                    } else if (originalConsLogins == consLogins) {
                                        // Nothing has changed since last update
                                        questData.params = inGameProgress[questId].params;
                                    } else {
                                        // User has come for additional day.
                                        //loggerObj.debugLog("Marking the cons logins - ", consLogins, originalConsLoginsNeeded);
                                        if (consLogins >= originalConsLoginsNeeded) {
                                            questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                                            //loggerObj.debugLog("Marking the cons logins quest as completed");
                                        } else {
                                            // Some progress has been made for quest
                                        }
                                        questData.params = [consLogins, 0, originalConsLoginsNeeded];
                                        inGameProgress[questId].params = questData.params;
                                        inGameLastLocalUpdate = curTimestamp;
                                    }
                                } else {
                                    // Compute consDaysNeededForCompletion - this is different from consDaysRequired as user may already have some consLogins when the quest starts.
                                    questData.params = [consLogins, 0, consDaysNeededForCompletion];
                                }
                                questData.curProgress = consDaysRequired - (questData.params[2] - questData.params[0]);
                                questData.reqProgress = consDaysRequired;
                                questData.status = questStatus;
                                questData.timeLimit = 0;
                                candidatesInBucket[questBucket].push(questData);
                                //candidateForBucket[questBucket] = questData;
                            }
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_FB_CONNECT:
                        // This quest is not to be stored in user quest data.
                        // Since claim for it is also not done separately.
                        if (!fbWrapperObj.isLoggedIn()) {
                            var lastChosenPriority = constantsObj.NOT_FOUND;
                            // if (candidateForBucket[questBucket] != undefined) {
                            //     lastChosenPriority = candidateForBucket[questBucket].p;
                            // }
                            //if (lastChosenPriority == constantsObj.NOT_FOUND || lastChosenPriority > questPriority) {
                            questData.curProgress = 0;
                            questData.reqProgress = 0;
                            questData.status = xpromoConstantsObj.QUEST_STARTED;
                            questData.params = [];
                            questData.timeLimit = 0;
                            candidatesInBucket[questBucket].push(questData);
                            //candidateForBucket[questBucket] = questData;
                            //}
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_FRIENDS:
                        // This quest is not to be stored in user quest data.
                        // Since claim for it is also not done separately.
                        var lastChosenPriority = constantsObj.NOT_FOUND;
                        // if (candidateForBucket[questBucket] != undefined) {
                        //     lastChosenPriority = candidateForBucket[questBucket].p;
                        // }
                        //if (lastChosenPriority == constantsObj.NOT_FOUND || lastChosenPriority > questPriority) {
                        questData.curProgress = 0;
                        questData.reqProgress = 0;
                        questData.status = xpromoConstantsObj.QUEST_STARTED;
                        questData.params = [];
                        questData.timeLimit = 0;
                        candidatesInBucket[questBucket].push(questData);
                        //candidateForBucket[questBucket] = questData;
                        //}

                        break;
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_IMESSAGE_APP_PERSISTENT:
                        // This quest is not to be stored in user quest data.
                        // Since claim for it is also not done separately.
                        var lastChosenPriority = constantsObj.NOT_FOUND;
                        // if (candidateForBucket[questBucket] != undefined) {
                        //     lastChosenPriority = candidateForBucket[questBucket].p;
                        // }
                        //if (lastChosenPriority == constantsObj.NOT_FOUND || lastChosenPriority > questPriority) {
                        questData.curProgress = 0;
                        questData.reqProgress = 0;
                        questData.status = xpromoConstantsObj.QUEST_STARTED;
                        questData.params = [];
                        questData.timeLimit = 0;
                        candidatesInBucket[questBucket].push(questData);
                        //candidateForBucket[questBucket] = questData;
                        //loggerObj.debugLog("pushing imessageApp persistent quest into list of quests");
                        //}
                        break;
                    // case xpromoConstantsObj.QUEST_TYPE_RATE_US:
                    //     var curAppVersion = utilObj.getVersionCode();
                    //     var lastRatedVersion = gameObj.getLastRatedVersion();
                    //     //loggerObj.debugLog("curAppversion , lastRatedVersion, ingameProgress ", curAppVersion, lastRatedVersion, utilObj.getStringifiedJson(inGameProgress[questId]));
                    //     if (curAppVersion == lastRatedVersion && inGameProgress[questId] != undefined && inGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED) {
                    //         questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                    //     }
                    //     if (curAppVersion != lastRatedVersion ||
                    //         (inGameProgress[questId] != undefined && inGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED)) {
                    //         var lastChosenPriority = constantsObj.NOT_FOUND;
                    //         // if (candidateForBucket[questBucket] != undefined) {
                    //         //     lastChosenPriority = candidateForBucket[questBucket].p;
                    //         // }
                    //         if (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
                    //             //if (lastChosenPriority == constantsObj.NOT_FOUND || lastChosenPriority > questPriority) {
                    //             questData.curProgress = 0;
                    //             questData.reqProgress = 0;
                    //             var questStatus = xpromoConstantsObj.QUEST_STARTED;
                    //             if (inGameProgress[questId] != undefined) {
                    //                 questStatus = inGameProgress[questId].status;
                    //             }
                    //             questData.status = questStatus;
                    //             questData.params = [];
                    //             questData.timeLimit = 0;
                    //             candidatesInBucket[questBucket].push(questData);
                    //             //candidateForBucket[questBucket] = questData;
                    //             //}
                    //         }
                    //     }
                    //     break;
                    case xpromoConstantsObj.QUEST_TYPE_FB_LIKE:
                    case xpromoConstantsObj.QUEST_TYPE_FOLLOW_TWITTER:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_WHATSAPP:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_TWITTER:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_FB_MESSENGER:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_INSTAGRAM:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_GOOGLE_PLUS:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_MESSAGE:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_EMAIL:
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_CONTACT_BOOK:
                    case xpromoConstantsObj.QUEST_TYPE_RATE_US:
                        if (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
                            questData.curProgress = 0;
                            questData.reqProgress = 0;
                            var questStatus = xpromoConstantsObj.QUEST_STARTED;
                            if (inGameProgress[questId] != undefined) {
                                questStatus = inGameProgress[questId].status;
                            }
                            questData.status = questStatus;
                            questData.params = [];
                            questData.timeLimit = 0;
                            candidatesInBucket[questBucket].push(questData);
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_INVITE_IMESSAGE_APP:
                        if (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
                            questData.curProgress = 0;
                            questData.reqProgress = 0;
                            var questStatus = xpromoConstantsObj.QUEST_STARTED;
                            if (inGameProgress[questId] != undefined) {
                                questStatus = inGameProgress[questId].status;
                            }
                            questData.status = questStatus;
                            questData.params = [];
                            questData.timeLimit = 0;
                            candidatesInBucket[questBucket].push(questData);
                        }
                        break;
                    case xpromoConstantsObj.QUEST_TYPE_WATCH_VIDEOS:
                    case xpromoConstantsObj.QUEST_TYPE_SPIN_SPINNER:
                    case xpromoConstantsObj.QUEST_TYPE_USE_HINT:
                    case xpromoConstantsObj.QUEST_TYPE_PURCHASE_COINS:
                    case xpromoConstantsObj.QUEST_TYPE_OPEN_DICTIONARY:
                    case xpromoConstantsObj.QUEST_TYPE_OPEN_SUBSCIRIPTION:
                    case xpromoConstantsObj.QUEST_TYPE_USE_HINT_BURST:
                    case xpromoConstantsObj.QUEST_TYPE_USE_HINT_TARGET:
                    case xpromoConstantsObj.QUEST_TYPE_PLAY_DAILY_PUZZLE:
                    case xpromoConstantsObj.QUEST_TYPE_CLICK_DICTIONARY_AUDIO:
                    case xpromoConstantsObj.QUEST_TYPE_PURCHASE_PREMIUM_AVATAR:
                        // loggerObj.debugLog("this is subs ", questData.qt, gameObj.shouldShowSubscriptionQuest());
                        if (questData.qt == xpromoConstantsObj.QUEST_TYPE_OPEN_SUBSCIRIPTION) {
                            if (!xPromoGameSpecificObj.shouldShowSubscriptionQuest()) {   //  && (inGameProgress[questId] == undefined || inGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED) && gameObj.getIsJoiningBonusGiven() == false
                                continue;
                            } else {
                                questData.d = allQuests[questId].d;
                                questData.d = questData.d.replace("%value%", runtimeObj.getSubscriptionJoiningBonus(2));
                                questData.re = runtimeObj.getSubscriptionJoiningBonus(2) / questsObj.getDollarValue();
                            }
                        }

                        if(questData.qt == xpromoConstantsObj.QUEST_TYPE_PURCHASE_PREMIUM_AVATAR) {
                           if(!xPromoGameSpecificObj.isProfileEnabled()) {
                               continue;
                           }
                        }

                        if(questData.qt == xpromoConstantsObj.QUEST_TYPE_CLICK_DICTIONARY_AUDIO && !xPromoGameSpecificObj.isAudioQuestEnabled()) {
                            continue;
                        }

                        if (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
                            if (inGameProgress[questId] == undefined && questData.qt == xpromoConstantsObj.QUEST_TYPE_OPEN_SUBSCIRIPTION) {
                                trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getTrackingBucketId(), gameObj.getTrackingPuzId(),
                                    xpromoConstantsObj.TRACK_QUEST_CENTER, questId + "", constantsObj.TRACK_START, "", "", "");
                            }
                            var questStatus = xpromoConstantsObj.QUEST_STARTED;
                            var numActionsTaken = 0;
                            var reqActions = parseInt(questData.pa);
                            if (inGameProgress[questId] != undefined) {
                                numActionsTaken = inGameProgress[questId].params[0];
                                if (numActionsTaken >= reqActions) {
                                    questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                                }
                                questStatus = inGameProgress[questId].status;
                            }
                            questData.curProgress = numActionsTaken;
                            questData.reqProgress = reqActions;
                            questData.status = questStatus;
                            questData.params = [numActionsTaken];
                            questData.timeLimit = 0;
                            candidatesInBucket[questBucket].push(questData);
                        }
                        break;
                    case dailyQuestsObj.isDailyQuestOrStreakQuestEnabled(questData.qt):
                        dailyQuestsObj.showInGameDailyQuests(questData, lastDailyQuestsMap, candidatesInBucket, currentDailyQuestMap, dailyQuestsEligibleToBeShown);
                        break;
                }
            }
        }

        // Helper method for initializing VIP Quests!
        questsObj.populateVipQuest(candidatesInBucket, vipLevelQuestCandidates);

        for (var qb in lastDailyQuestsMap) {
            if (candidatesInBucket[qb] != undefined && candidatesInBucket[qb].length == 0) {
                candidatesInBucket[qb].push(lastDailyQuestsMap[qb]);
            }
            /*if (candidateForBucket[qb] == undefined) {
                candidateForBucket[qb] = lastDailyQuestsMap[qb];
            }*/
        }
        // questsObj.updateInGameProgress();
        dailyQuestsShouldBeReset = 0;
        var questsFirstLoadTimestamp = questsObj.getQuestsFirstLoadTime();

        candidateForBucket = questRotationObj.getCandidateForBucket(candidatesInBucket);

        for (var bucketId in candidateForBucket) {
            var questData = candidateForBucket[bucketId];
            var questId = questData.id;
            if (inGameProgress[questId] == undefined
                && !questsObj.isPersistentQuest(questData)) {
                var questEligibleAfter = 0;
                if (questData.ea != undefined) {
                    questEligibleAfter = parseInt(questData.ea) * constantsObj.SECONDS_IN_DAY;
                }
                if (curTimestamp - questsFirstLoadTimestamp < questEligibleAfter) {
                    continue;
                }
                // Need to mark this as started
                inGameProgress[questId] = {
                    'id': questId,
                    'status': xpromoConstantsObj.QUEST_STARTED,
                    'params': questData.params,
                };

                inGameLastLocalUpdate = curTimestamp;
            } else if (inGameProgress[questId] != undefined && dailyQuestsObj.isDailyQuestOrStreakQuest(questData.qt)) {
                inGameProgress[questId].params = questData.params;
            }
            availableQuests.push(questData);
        }
        //loggerObj.debugLog("Candidate for bucket:", utilObj.getStringifiedJson(candidateForBucket));
        return availableQuests;
    };

    /**
     * Helper method to initialize a VIP Quest from a set of available quests
     * This method needs to be provided with the custom prioritized list.
     * @param candidatesInBucket - Reference to the candidatesInBucket Map
     * @param vipLevelQuestCandidates - Reference to the vip level quests those are eligible for population
     */
    this.populateVipQuest = function (candidatesInBucket, vipLevelQuestCandidates) {
        if(vipLevelQuestCandidates.length > 0) {

            // Lets pick the first eligible quest - we have already prioritized active quests over new quests
            var questData = vipLevelQuestCandidates[0];
            var questId = questData.id;
            var questBucket = questData.b;

            if(candidatesInBucket[questBucket] != undefined) {
                var vipLevelRequired = parseInt(questData.pa);
                var vipPuzCompletes = Math.max(0, xPromoGameSpecificObj.getVIPPuzzlesCompleted());
                var questStatus = xpromoConstantsObj.QUEST_STARTED;
                var vipPuzzlesStart = vipPuzCompletes;

                if (inGameProgress[questId] != undefined) {
                    questStatus = inGameProgress[questId].status;
                    vipPuzzlesStart = inGameProgress[questId].params[0];
                }

                questData.status = questStatus;
                questData.curProgress = vipPuzCompletes - vipPuzzlesStart;
                questData.reqProgress = vipLevelRequired;
                questData.timeLimit = 0;
                questData.params = [vipPuzzlesStart];
                candidatesInBucket[questBucket].push(questData);
            }
        }
    };

    this.isPersistentQuest = function (questData) {
        if (questData == null) {
            return false;
        }
        return (questData.qt == xpromoConstantsObj.QUEST_TYPE_FB_CONNECT || questData.qt == xpromoConstantsObj.QUEST_TYPE_INVITE_FRIENDS || questData.qt == xpromoConstantsObj.QUEST_TYPE_INVITE_IMESSAGE_APP_PERSISTENT);
    };

    // * making an adjacency list for all related quests
    this.makeRelatedQuests = function (force) {
        if (crossGameRelatedQuests != null && Object.keys(crossGameRelatedQuests).length > 0 && force != true) {
            return;
        }
        crossGameRelatedQuests = {};
        for (var q in allQuests) {
            var availableQuest = allQuests[q];
            if ((availableQuest["qt"].indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) == 0 || availableQuest["qt"].indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_ACTION) == 0 || availableQuest["qt"].indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_TIME) == 0) && (availableQuest.gh == constantsObj.GAME_ID)) {
                installAndDoSomethingQuests.push(availableQuest["id"]);
                if (crossGameProgress[q] == undefined && crossGameDataAvailable == 1) {
                    // loggerObj.debugLog("this is install time quest ", q);
                    crossGameProgress[q] = {
                        'id': q,
                        'status': xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START,
                    }
                } else if (curQuests[q] == undefined && crossGameDataAvailable == 1) {
                    crossGameProgress[q].status = xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START;
                }
            }

            if (availableQuest.b == xpromoConstantsObj.QUEST_BUCKET_BUY_SCREEN || availableQuest.b == xpromoConstantsObj.QUEST_HOME_SCREEN) {
                continue;
            }
            var relatedQuests = availableQuest["r"];
            if (relatedQuests != undefined && relatedQuests != "0") {
                relatedQuests = relatedQuests.split(":");
                for (var i = 0; i < relatedQuests.length; i++) {
                    if (allQuests[relatedQuests[i]] == undefined || allQuests[relatedQuests[i]].b == xpromoConstantsObj.QUEST_BUCKET_BUY_SCREEN || allQuests[relatedQuests[i]].b == xpromoConstantsObj.QUEST_HOME_SCREEN) {
                        continue;
                    }
                    if (crossGameRelatedQuests[q] == undefined) {
                        crossGameRelatedQuests[q] = [];
                    }
                    if (crossGameRelatedQuests[relatedQuests[i]] == undefined) {
                        crossGameRelatedQuests[relatedQuests[i]] = [];
                    }
                    crossGameRelatedQuests[q].push(relatedQuests[i]);
                    crossGameRelatedQuests[relatedQuests[i]].push(q);
                }
            }
        }
    };

    this.getAvailableCrossGameQuests = function (questType) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return [];
        }

        crossGameDailyQuestsObj.setDailyQuestToBeShown(null);
        var availableQuests = [];
        if (curQuests == null || utilObj.isLimitAdTracking()) {
            return availableQuests;
        }
        questsObj.makeRelatedQuests();

        // fetching requests
        var currentActiveRequest = requestsObj.getCurrentActiveRequest()[0];
        if (currentActiveRequest != undefined) {
            currentActiveRequest.r = "";
        }

        // candidateForBucket stores the quest data for a bucket which is going to be shown in the game
        var candidatesInBucket = {};  // stores all the quests for corresponding bucket id
        var minLevelRequired = [];

        // host Game bucket is for marking the required quests in a bucket for the host game which are not going to be shown in the game
        var hostGameBucket = {};
        var candidateForBucket = {};
        var curTimestamp = utilObj.getCurrentTimestamp();
        if (questsObj.getQuestsFirstLoadTime() == 0) {
            questsObj.setQuestsFirstLoadTime();
        }
        var questsFirstLoadTimestamp = questsObj.getQuestsFirstLoadTime();

        var curQuestsArr = questsObj.sortQuests(curQuests, "curQuests");

        for (var quest in curQuestsArr) {
            var questId = curQuestsArr[quest].id;
            var questData = curQuests[questId];
            if (questData == undefined) {
                continue;
            }
            if (questType != undefined) {
                if (questType == xpromoConstantsObj.QUEST_TYPE_PUZZLE_COMPLETE) {
                    if (questData.qt != xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_COMPLETE) {
                        continue;
                    }
                } else if (questType == xpromoConstantsObj.QUEST_TYPE_VIP_LEVEL_COMPLETE && questData.qt != xpromoConstantsObj.QUEST_TYPE_VIP_LEVEL_COMPLETE) {
                    continue;
                }
            }

            if (questData == undefined) {
                continue;
            }
            var questBucket = questData.b;
            var questPriority = questData.p;
            questData.id = questId;
            if (questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME_BUYSCREEN && adWallQuestsMap[questData.gh] == undefined && questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_CROSS_GAME && questData.gs[0] == xPromoGameSpecificObj.getAdwallGameShownId()) {
                if (utilObj.isWeb() || (utilObj.isIos() && questData["cl"][0] == 1) || (utilObj.isAndroid() && questData["cl"][0] == 0)) {
                    if(!utilObj.isAmazonDevice() || (xPromoGameSpecificObj.getAmazonAvailableGames()).indexOf(questData.gh) >= 0) {
                        adWallQuestsMap[questData.gh] = true;
                        adWallQuests[questData.p] = questData;
                        var gameIcon = "icon_" + questData.gh + ".png";
                        spritesArr[gameIcon] = xpromoUtilObj.getAbsFilePath(gameIcon);
                        sprite_topush.push(spritesArr[gameIcon]);
                        questData.isAdwallQuest = 1;
                    }
                }
            }

            // not showing install quest if request is active for that game
            // changing request map here | adding related quest for request
            if (currentActiveRequest != undefined && questData.gh == currentActiveRequest.gh && questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME) {
                currentActiveRequest.r += questId + ":";
                continue;
            }

            var timeForQuestToBeActivated = 0;
            if (questData.ea != undefined) {
                timeForQuestToBeActivated = parseInt(questData.ea) * constantsObj.SECONDS_IN_DAY;
            }
            if (!questsObj.isQuestInGame(questId)
                && (questsObj.isQuestShownInGame(questId) || questData.gh == constantsObj.GAME_ID)
                && questsObj.isQuestAvailableInCountry(questData.col)
                && questsObj.isQuestAvailableOnClient(questData.cl)
                && (crossGameProgress[questId] == undefined || !(crossGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START))
                && curTimestamp - questsFirstLoadTimestamp > timeForQuestToBeActivated) {
                //TODO: If not the game host, for quest type levelComplete , just update the quest Progress from crossGameProgress Fetched from the server
                for (var i = 0; i < xpromoConstantsObj.QUEST_TYPES.length; i++) {
                    if (questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPES[i]) == 0) {
                        questData.qt = xpromoConstantsObj.QUEST_TYPES[i];
                    }
                }

                if (questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
                    questData.qt = xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ;
                }
                if (candidatesInBucket[questBucket] == undefined) {
                    candidatesInBucket[questBucket] = [];
                }
                switch (questData.qt) {
                    case xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME:
                        var allParams = questData.pa.split(":");
                        var appName = allParams[0];
                        var itunesId = allParams[3];
                        var fbid = allParams[4];
                        if (allParams[5] != 1 && utilObj.isLimitTrackingWithoutAdjustId()) {
                            continue;
                        }
                        if (utilObj.isIos()) {
                            appName = fbid;
                        }
                        var questStatus = xpromoConstantsObj.QUEST_NOT_STARTED;
                        if (crossGameProgress[questId] != undefined) {
                            questStatus = crossGameProgress[questId].status;
                        }
                        questData.status = questStatus;
                        if (xpromoUtilObj.isAppInstalled(appName) && questStatus != xpromoConstantsObj.QUEST_COMPLETED && questStatus != xpromoConstantsObj.QUEST_STARTED) {
                            // do not show install quests if the app is installed already and its questStatus is either completed or started
                        } else if (questStatus == xpromoConstantsObj.QUEST_NOT_STARTED || questStatus == xpromoConstantsObj.QUEST_STARTED
                            || questStatus == xpromoConstantsObj.QUEST_COMPLETED
                            || (!xpromoUtilObj.isAppInstalled(appName) && questStatus != xpromoConstantsObj.QUEST_CLAIMED && questStatus != xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START)) {
                            //var shareLink = questsObj.getGameShareLink(allParams[2]);
                            if (itunesId != undefined && itunesId != null) {
                                questData.params = [itunesId];
                            } else {
                                questData.params = [];
                            }
                            var gameIcon = "icon_" + questData.gh + ".png";
                            if (!utilObj.isWeb() && !utilObj.fileExists(gameIcon)) {
                                //loggerObj.debugLog("Downloading game Icon for gameId : ", gameIcon);
                                xPromoGameSpecificObj.downloadFileV1(constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/icons/" + gameIcon, gameIcon);
                            }
                            if (!questsObj.isQuestShownInGame(questId)) {
                                hostGameBucket[questBucket] = questData;
                            } else {
                                candidatesInBucket[questBucket].push(questData);
                            }
                        }
                        break;

                    case xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_COMPLETE:
                    case xpromoConstantsObj.QUEST_TYPE_INSTALL_AND:
                        var allParams = questData.pa.split(":");
                        var appName = allParams[0];
                        var fbid = "";
                        var shareLink = questsObj.getGameShareLink(allParams[2]);

                        if (allParams[6] != 1 && utilObj.isLimitTrackingWithoutAdjustId()) {
                            continue;
                        }
                        if (allParams.length > 5) {
                            fbid = allParams[5];
                        }
                        var isAppInstalled = false;
                        if (utilObj.isAndroid()) {
                            isAppInstalled = xpromoUtilObj.isAppInstalled(appName);
                        } else if (utilObj.isIos()) {
                            isAppInstalled = xpromoUtilObj.isAppInstalled(fbid);
                        }

                        if (utilObj.isIos()) {
                            // TODO: Doesn't currently work correctly for ios as deeplinks are not correct.
                            appName = allParams[1];
                        }

                        if (isAppInstalled && (crossGameProgress[questId] == undefined  || crossGameProgress[questId] == xpromoConstantsObj.QUEST_NOT_STARTED) && questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) {
                            continue;
                        }

                        var questLevelRequired = parseInt(allParams[3]);
                        minLevelRequired = [];
                        var userCurLevel = questsObj.getRelevantDataCount(questId);
                        if (userCurLevel == undefined) {
                            userCurLevel = 0;
                        }

                        if (crossGameProgress[questId] == undefined || crossGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
                            // if the game host is same as Game Id , then the quest can be either shown in the game or not
                            if (questData.gh == constantsObj.GAME_ID) {
                                var allParams = questData.pa.split(":");
                                var questLevelRequired = parseInt(allParams[3]);
                                if (questLevelRequired > userCurLevel
                                    || (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED)
                                    || (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED)) {
                                    var questStatus = xpromoConstantsObj.QUEST_NOT_STARTED;
                                    if (questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_AND && (crossGameProgress[questId] == undefined  || crossGameProgress[questId] == xpromoConstantsObj.QUEST_NOT_STARTED) && crossGameDataAvailable == 1) {
                                        questData.status = xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START;
                                        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START);
                                        questsObj.changeStatusOfAllInstallQuests(questId);
                                    } else if (questData.qt == xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_COMPLETE) {
                                        questStatus = xpromoConstantsObj.QUEST_STARTED;
                                    }
                                    if (crossGameProgress[questId] != undefined) {
                                        questStatus = crossGameProgress[questId].status;
                                    }
                                    if (questStatus == xpromoConstantsObj.QUEST_STARTED && userCurLevel >= parseInt(questLevelRequired)) {
                                        questStatus = xpromoConstantsObj.QUEST_COMPLETED;
                                        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                                    }
                                    questData.status = questStatus;
                                    questData.curProgress = userCurLevel;
                                    questData.reqProgress = questLevelRequired;
                                    questData.timeLimit = 0;
                                    questData.params = [userCurLevel];
                                    if (questsObj.isQuestShownInGame(questId)) {
                                        candidatesInBucket[questBucket].push(questData);
                                    } else {
                                        hostGameBucket[questBucket] = questData;
                                    }
                                } else if (userCurLevel >= questLevelRequired && (crossGameProgress[questId] == undefined  || crossGameProgress[questId] == xpromoConstantsObj.QUEST_NOT_STARTED) && crossGameDataAvailable == 1) {
                                    crossGameProgress[questId] = {
                                        'id': questId,
                                        'status': xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START
                                    }
                                }
                            } else {
                                if (candidatesInBucket[questBucket].length == 0 || questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) {
                                    /*||
                                    (candidateForBucket[questBucket].status == xpromoConstantsObj.QUEST_NOT_STARTED && candidateForBucket[questBucket].qt != xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME &&
                                        (crossGameProgress[questId] != undefined || crossGameProgress[questId] == undefined /*&& lastChosenPriority > questPriority))) {*/
                                    //Todo - check whether removal of this valid or not
                                    var hostGame = 0;
                                    if (questData.qt != xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) {
                                        hostGame = parseInt(questData.gh);
                                        if (minLevelRequired[hostGame] == undefined) {
                                            minLevelRequired[hostGame] = 0;
                                        }
                                        if (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_CLAIMED && crossGameProgress[questId].params.length > 0) {
                                            if (parseInt(crossGameProgress[questId].params[0]) > minLevelRequired[hostGame]) {
                                                minLevelRequired[hostGame] = parseInt(crossGameProgress[questId].params[0]);
                                            }
                                        }
                                    }
                                    var tempQData = curQuests[questId];
                                    hostGame = parseInt(tempQData.gh);
                                    if (minLevelRequired[hostGame] == undefined || minLevelRequired[hostGame] == null) {
                                        minLevelRequired[hostGame] = 0;
                                    }
                                    if (hostGame != 0 && questLevelRequired > minLevelRequired[hostGame]) {
                                        var currProgress = 0;
                                        var questStatus = xpromoConstantsObj.QUEST_NOT_STARTED;
                                        if (crossGameProgress[questId] != undefined) {
                                            if (crossGameProgress[questId].params != undefined && crossGameProgress[questId].params.length > 0 && !isNaN(parseInt(crossGameProgress[questId].params[0]))) {
                                                currProgress = parseInt(crossGameProgress[questId].params[0]);
                                            }
                                            questStatus = crossGameProgress[questId].status;
                                        }
                                        questData.status = questStatus;
                                        questData.curProgress = currProgress;
                                        questData.reqProgress = questLevelRequired;
                                        questData.timeLimit = 0;
                                        questData.shareLink = shareLink;
                                        questData.params = [currProgress];
                                        candidatesInBucket[questBucket].push(questData);
                                    }
                                } else {
                                    candidatesInBucket[questBucket].push(questData);
                                }
                            }
                        }
                        break;

                    case xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_IN_TIME:
                    case xpromoConstantsObj.QUEST_TYPE_INSTALL_TIME:
                        var allParams = questData.pa.split(":");
                        var appName = allParams[0];
                        var fbid = "";
                        var shareLink = questsObj.getGameShareLink(allParams[2]);
                        if (allParams[7] != 1 && utilObj.isLimitTrackingWithoutAdjustId()) {
                            continue;
                        }
                        if (allParams.length > 6) {
                            fbid = allParams[6];
                        }
                        var isAppInstalled = false;
                        if (utilObj.isAndroid()) {
                            isAppInstalled = xpromoUtilObj.isAppInstalled(appName);
                        } else if (utilObj.isIos()) {
                            isAppInstalled = xpromoUtilObj.isAppInstalled(fbid);
                        }
                        if (utilObj.isIos()) {
                            // TODO: Doesn't currently work correctly for ios as deeplinks are not correct.
                            appName = allParams[1];
                        }

                        if (isAppInstalled && crossGameProgress[questId] == undefined && questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_TIME) {
                            continue;
                        }

                        var questLevelRequired = parseInt(allParams[3]);
                        var questTime = parseInt(allParams[4]) * constantsObj.SECONDS_IN_HOUR;
                        minLevelRequired = [];
                        var startTime = curTimestamp;
                        var userCurLevel = questsObj.getRelevantDataCount(questId);
                        if (userCurLevel == undefined) {
                            userCurLevel = 0;
                        }

                        if (crossGameProgress[questId] != undefined && curTimestamp - crossGameProgress[questId].params[1] > questTime && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED) {
                            continue;
                        }

                        if ((crossGameProgress[questId] == undefined || crossGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED)) {
                            var startTime = curTimestamp;
                            if (crossGameProgress[questId] != undefined && crossGameProgress[questId].params != undefined && crossGameProgress[questId].params.length > 0) {
                                startTime = crossGameProgress[questId].params[1];
                            }
                            // if the game host is same as Game Id , then the quest can be either shown in the game or not
                            if (questData.gh == constantsObj.GAME_ID) {
                                var allParams = questData.pa.split(":");
                                var questLevelRequired = parseInt(allParams[3]);
                                var lastChosenRequiredLevel = constantsObj.NOT_FOUND;
                                // if the quest is not shown in the game , then maintain the progress in hostGameBucket
                                if (!questsObj.isQuestShownInGame(questId)) {
                                    if (hostGameBucket[questBucket] != undefined) {
                                        lastChosenRequiredLevel = parseInt(hostGameBucket[questBucket].pa);
                                    }
                                } else {
                                    // if the quest is shown in the game , maintain the progress in candidateForBucket
                                    if (candidatesInBucket[questBucket].length > 0) {
                                        lastChosenRequiredLevel = parseInt(candidatesInBucket[questBucket].pa);
                                    }
                                }
                                if (questLevelRequired > userCurLevel
                                    || (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED)
                                    || (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED)) {

                                } else if (userCurLevel >= questLevelRequired && crossGameProgress[questId] == undefined && crossGameDataAvailable == 1) {
                                    crossGameProgress[questId] = {
                                        'id': questId,
                                        'status': xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START
                                    }
                                }
                            } else {
                                if (candidatesInBucket[questBucket].length == 0) { /*||
                                    (candidateForBucket[questBucket] != undefined && candidateForBucket[questBucket].status == xpromoConstantsObj.QUEST_NOT_STARTED && candidateForBucket[questBucket].qt != xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME &&
                                        (crossGameProgress[questId] != undefined || crossGameProgress[questId] == undefined /*&& lastChosenPriority > questPriority))){*/
                                    var hostGame = 0;
                                    if (questData.qt != xpromoConstantsObj.QUEST_TYPE_INSTALL_TIME) {
                                        hostGame = parseInt(questData.gh);
                                        if (minLevelRequired[hostGame] == undefined) {
                                            minLevelRequired[hostGame] = 0;
                                        }
                                        if (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_CLAIMED && crossGameProgress[questId].params.length > 0) {
                                            if (parseInt(crossGameProgress[questId].params[0]) > minLevelRequired[hostGame]) {
                                                minLevelRequired[hostGame] = parseInt(crossGameProgress[questId].params[0]);
                                            }
                                        }
                                    }
                                    var tempQData = curQuests[questId];
                                    hostGame = parseInt(tempQData.gh);
                                    if (minLevelRequired[hostGame] == undefined || minLevelRequired[hostGame] == null) {
                                        minLevelRequired[hostGame] = 0;
                                    }
                                    if (hostGame != 0 && questLevelRequired > minLevelRequired[hostGame]) {
                                        var currProgress = 0;
                                        var questStatus = xpromoConstantsObj.QUEST_NOT_STARTED;
                                        if (crossGameProgress[questId] != undefined) {
                                            if (crossGameProgress[questId].params != undefined && crossGameProgress[questId].params.length > 0) {
                                                currProgress = parseInt(crossGameProgress[questId].params[0]);
                                            }
                                            questStatus = crossGameProgress[questId].status;
                                        }
                                        questData.status = questStatus;
                                        questData.curProgress = currProgress;
                                        questData.reqProgress = questLevelRequired;
                                        questData.timeLimit = questTime;
                                        questData.shareLink = shareLink;
                                        questData.params = [currProgress, startTime];
                                        candidatesInBucket[questBucket].push(questData);
                                    }
                                } else {
                                    candidatesInBucket[questBucket].push(questData);
                                }
                            }
                        }
                        break;

                    case xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_ACTION:
                    case xpromoConstantsObj.QUEST_TYPE_INSTALL_ACTION:
                        var allParams = questData.pa.split(":");
                        var questLevelRequired = parseInt(allParams[3]);
                        var appName = allParams[0];
                        var fbid = "";
                        var shareLink = questsObj.getGameShareLink(allParams[2]);
                        if (allParams[6] != 1 && utilObj.isLimitTrackingWithoutAdjustId()) {
                            continue;
                        }
                        if (allParams.length > 5) {
                            fbid = allParams[5];
                        }
                        var isAppInstalled = false;
                        if (utilObj.isAndroid()) {
                            isAppInstalled = xpromoUtilObj.isAppInstalled(appName);
                        } else if (utilObj.isIos()) {
                            isAppInstalled = xpromoUtilObj.isAppInstalled(fbid);
                        }
                        if (utilObj.isIos()) {
                            // TODO: Doesn't currently work correctly for ios as deeplinks are not correct.
                            appName = allParams[1];
                        }
                        if (isAppInstalled && crossGameProgress[questId] == undefined && questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_ACTION) {
                            continue;
                        }
                        minLevelRequired = [];
                        var userCurLevel = questsObj.getRelevantDataCount(questId);
                        if (userCurLevel == undefined) {
                            userCurLevel = 0;
                        }

                        if (crossGameProgress[questId] == undefined || crossGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
                            // if the game host is same as Game Id , then the quest can be either shown in the game or not
                            if (questData.gh == constantsObj.GAME_ID) {
                                var allParams = questData.pa.split(":");
                                var questLevelRequired = parseInt(allParams[3]);
                                var lastChosenRequiredLevel = constantsObj.NOT_FOUND;
                                // if the quest is not shown in the game , then maintain the progress in hostGameBucket
                                if (!questsObj.isQuestShownInGame(questId)) {
                                    if (hostGameBucket[questBucket] != undefined) {
                                        lastChosenRequiredLevel = parseInt(hostGameBucket[questBucket].pa);
                                    }
                                } else {
                                    // if the quest is shown in the game , maintain the progress in candidateForBucket
                                    if (candidatesInBucket[questBucket].length > 0) {
                                        lastChosenRequiredLevel = parseInt(candidatesInBucket[questBucket].pa);
                                    }
                                }
                                if (questLevelRequired > userCurLevel
                                    || (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED)
                                    || (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED)) {

                                } else if (userCurLevel >= questLevelRequired && crossGameProgress[questId] == undefined && crossGameDataAvailable == 1) {
                                    crossGameProgress[questId] = {
                                        'id': questId,
                                        'status': xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START
                                    }
                                }
                            } else {
                                if (candidatesInBucket[questBucket].length == 0) { /*candidateForBucket[questBucket] == undefined ||
                                    (candidateForBucket[questBucket] != undefined && candidateForBucket[questBucket].status == xpromoConstantsObj.QUEST_NOT_STARTED && candidateForBucket[questBucket].qt != xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME &&
                                        (crossGameProgress[questId] != undefined || crossGameProgress[questId] == undefined && lastChosenPriority > questPriority))*/
                                    var hostGame = 0;
                                    if (questData.qt != xpromoConstantsObj.QUEST_TYPE_INSTALL_ACTION) {
                                        hostGame = parseInt(questData.gh);
                                        if (minLevelRequired[hostGame] == undefined) {
                                            minLevelRequired[hostGame] = 0;
                                        }
                                        if (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_CLAIMED) {
                                            if (parseInt(crossGameProgress[questId].params[0]) > minLevelRequired[hostGame]) {
                                                minLevelRequired[hostGame] = parseInt(crossGameProgress[questId].params[0]);
                                            }
                                        }
                                    }
                                    var tempQData = curQuests[questId];
                                    hostGame = parseInt(tempQData.gh);
                                    if (minLevelRequired[hostGame] == undefined || minLevelRequired[hostGame] == null) {
                                        minLevelRequired[hostGame] = 0;
                                    }
                                    if (hostGame != 0 && questLevelRequired > minLevelRequired[hostGame]) {
                                        var currProgress = 0;
                                        var questStatus = xpromoConstantsObj.QUEST_NOT_STARTED;
                                        if (crossGameProgress[questId] != undefined) {
                                            if (crossGameProgress[questId].params != undefined && crossGameProgress[questId].params.length > 0) {
                                                currProgress = parseInt(crossGameProgress[questId].params[0]);
                                            }
                                            questStatus = crossGameProgress[questId].status;
                                        }
                                        questData.status = questStatus;
                                        questData.curProgress = currProgress;
                                        questData.reqProgress = questLevelRequired;
                                        questData.timeLimit = 0;
                                        questData.shareLink = shareLink;
                                        questData.params = [currProgress];
                                        candidatesInBucket[questBucket].push(questData);
                                    }
                                } else {
                                    candidatesInBucket[questBucket].push(questData);
                                }
                            }
                        }
                        break;

                    case xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ:
                        crossGameDailyQuestsObj.showCrossGameDailyQuest(questId, questData, candidatesInBucket, hostGameBucket, crossGameDataAvailable);
                        break;

                }
            }
        }

        candidateForBucket = questRotationObj.getCandidateForBucket(candidatesInBucket);


        if (crossGameDailyQuestsObj.getDailyQuestToBeShown() == null || crossGameDailyQuestsObj.getDailyQuestToBeShown() == undefined) {
            crossGameDailyQuestsObj.resetCrossGameDailyQuestMap();
        }

        // for quests shown in game
        var socialQuest = 0, shouldShowFriend = 1;
        for (var bucketId in candidateForBucket) {
            var questData = candidateForBucket[bucketId];
            // questData = questsObj.changeQuestDataForIos(questData);
            var questId = questData.id;

            if (crossGameProgress[questId] == undefined) {
                // Need to mark this as started
                if (questData != undefined && questData.status != xpromoConstantsObj.QUEST_NOT_STARTED && questData.b != xpromoConstantsObj.QUEST_BUCKET_BUY_SCREEN && questData.b != xpromoConstantsObj.QUEST_HOME_SCREEN) {
                    var questEligibleAfter = 0;
                    if (questData.ea != undefined) {
                        questEligibleAfter = parseInt(questData.ea) * constantsObj.SECONDS_IN_DAY;
                    }
                    if (curTimestamp - questsFirstLoadTimestamp < questEligibleAfter) {
                        continue;
                    }
                    crossGameProgress[questId] = {
                        'id': questId,
                        'status': xpromoConstantsObj.QUEST_STARTED,
                        'params': questData.params,
                    };
                    crossGameLastLocalUpdate = curTimestamp;
                }
            }

            // push the quest in the list of available cross game quests only if it is shown in the game
            if (questsObj.isQuestShownInGame(questId) && questData.b != xpromoConstantsObj.QUEST_BUCKET_BUY_SCREEN && questData.b != xpromoConstantsObj.QUEST_HOME_SCREEN) {
                var socialQuestReturnValues = socialQuestObj.edittingDataForSocialQuest(questData, socialQuest, shouldShowFriend);
                socialQuest = socialQuestReturnValues[0];
                shouldShowFriend = socialQuestReturnValues[1];
                availableQuests.push(questData);
            }
        }

        for (var newQuests in buyScreenList) {
            var questData = buyScreenList[newQuests];
            var questId = questData.id;
            if (crossGameProgress[questId] == undefined) {
                // Need to mark this as started
                if (questData != undefined && questData.status != xpromoConstantsObj.QUEST_NOT_STARTED) {
                    crossGameProgress[questId] = {
                        'id': questId,
                        'status': xpromoConstantsObj.QUEST_STARTED,
                        'params': [questData.params],
                    };
                    crossGameLastLocalUpdate = curTimestamp;
                }
            }
        }

        // for cross game quests of host game which are not shown in the host game , just mark the progress in crossGameProgress
        for (var bucketId in hostGameBucket) {
            var questData = hostGameBucket[bucketId];
            var questId = questData.id;
            if (crossGameProgress[questId] == undefined && questData.status != xpromoConstantsObj.QUEST_NOT_STARTED && questData.b != xpromoConstantsObj.QUEST_BUCKET_BUY_SCREEN && questData.b != xpromoConstantsObj.QUEST_HOME_SCREEN) {
                // Need to mark this as started
                if ((questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) == 0 || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_ACTION) == 0 || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_TIME) == 0)
                    && crossGameProgress[questId] == undefined && crossGameDataAvailable == 1) {
                    crossGameProgress[questId] = {
                        'id': questId,
                        'status': xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START,
                        'params': questData.params,
                    };
                    questsObj.changeStatusOfAllInstallQuests(questId);
                } else {
                    crossGameProgress[questId] = {
                        'id': questId,
                        'status': xpromoConstantsObj.QUEST_STARTED,
                        'params': questData.params,
                    };
                }
                questsObj.changeStatusOfAllInstallQuests(questId);
                crossGameLastLocalUpdate = curTimestamp;
            } else if (crossGameProgress[questId] != undefined && questData.b != xpromoConstantsObj.QUEST_BUCKET_BUY_SCREEN && questData.b != xpromoConstantsObj.QUEST_HOME_SCREEN) {
                crossGameProgress[questId].params = questData.params;
                crossGameLastLocalUpdate = curTimestamp;
            }

            if (questsObj.isQuestShownInGame(questId)) {
                availableQuests.push(questData);
            }
        }
        // questsObj.updateCrossGameProgress();
        return availableQuests;
    };

    this.makeRelatedQuestsSeen = function (questId, questsSeen) {
        if (crossGameRelatedQuests == undefined || crossGameRelatedQuests == null) {
            return;
        }
        var questQueue = [];
        questQueue.push(questId);
        while (questQueue.length > 0) {
            var curQuest = questQueue.pop();
            questsSeen[curQuest] = 1;
            if (crossGameRelatedQuests[curQuest] != undefined) {
                for (var j = 0; j < crossGameRelatedQuests[curQuest].length; j++) {
                    if (questsSeen[crossGameRelatedQuests[curQuest][j]] == undefined) {
                        questQueue.push(crossGameRelatedQuests[curQuest][j]);
                    }
                }
            }
        }
    };

    this.makeRelatedQuestsUnseen = function (questId, questsSeen) {
        if (crossGameRelatedQuests == undefined || crossGameRelatedQuests == null) {
            return;
        }
        var questQueue = [];
        questQueue.push(questId);
        while (questQueue.length > 0) {
            var curQuest = questQueue.pop();
            questsSeen[curQuest] = undefined;
            if (crossGameRelatedQuests[curQuest] != undefined) {
                for (var j = 0; j < crossGameRelatedQuests[curQuest].length; j++) {
                    if (questsSeen[crossGameRelatedQuests[curQuest][j]] == 1) {
                        questQueue.push(crossGameRelatedQuests[curQuest][j]);
                    }
                }
            }
        }
    };

    this.getAllRelatedQuest = function (questId) {
        var questArr = [], questQueue = [], relatedQuestsSeen = {};
        questQueue.push(questId);
        while (questQueue.length > 0) {
            var curQuest = questQueue.pop();
            relatedQuestsSeen[curQuest] = 1;
            if (crossGameRelatedQuests[curQuest] != undefined) {
                for (var j = 0; j < crossGameRelatedQuests[curQuest].length; j++) {
                    if (relatedQuestsSeen[crossGameRelatedQuests[curQuest][j]] == undefined) {
                        questQueue.push(crossGameRelatedQuests[curQuest][j]);
                        questArr.push(crossGameRelatedQuests[curQuest][j]);
                    }
                }
            }
        }
        return questArr;
    };

    this.getAllCurCrossGameQuests = function () {
        var curCrossGameQuests = [];
        if (curQuests == null) {
            return curCrossGameQuests;
        }
        for (var questId in curQuests) {
            var questData = curQuests[questId];
            questData.id = questId;
            if (questsObj.isQuestInGame(questId)) {
                continue;
            }
            curCrossGameQuests.push(questData);
        }
        return curCrossGameQuests;
    };

    this.getSortedQuestList = function () {
        // This will consist of both current and in game quests
        var questList = [];
        if (lastInGameFetch == 0 || lastCrossGameFetch == 0) {
            return questList;
        }
        // fetching requests data and then concatenating it with cross game and in game quests
        var requests = requestsObj.getCurrentActiveRequest();
        var inGame = questsObj.getAvailableInGameQuests();
        var crossGame = questsObj.getAvailableCrossGameQuests();

        // loggerObj.debugLog("this is unique quests cross", utilObj.getStringifiedJson(crossGame));
        // removing related quest
        var questsArr = [], questsSeen = {}, questQueue = [];
        for (var i = 0; i < crossGame.length; i++) {
            questQueue.push(crossGame[i].id);
            while (questQueue.length > 0) {
                var curQuest = questQueue.pop();
                if (questsSeen[curQuest] == undefined) {
                    questsArr.push(crossGame[i]);
                }
                questsSeen[curQuest] = 1;
                if (crossGameRelatedQuests[curQuest] != undefined) {
                    for (var j = 0; j < crossGameRelatedQuests[curQuest].length; j++) {
                        if (questsSeen[crossGameRelatedQuests[curQuest][j]] == undefined) {
                            questQueue.push(crossGameRelatedQuests[curQuest][j]);
                            questsSeen[crossGameRelatedQuests[curQuest][j]] = 1;
                        }
                    }
                }
            }
        }

        // sending an additional param as tracking id
        var questPos = 1;
        for (var i = 0; i < questsArr.length; i++) {
            if (questsArr[i].isSocialQuest == 1) {
                questsArr[i].trackingId = questPos;
                //the above tracking will be later updated based on bot or friend being shown
            } else {
                questsArr[i].trackingId = xpromoConstantsObj.TRACK_QUEST + "_" + questPos;
            }
            questPos += 1;
        }

        // loggerObj.debugLog("this is unique quests ", utilObj.getStringifiedJson(questsArr));

        var unsortedQuestList = inGame.concat(questsArr);
        unsortedQuestList = unsortedQuestList.concat(requests);
        // Now sort all of these
        unsortedQuestList.sort(function (a, b) {
            return a.p - b.p;
        });

        unsortedQuestList.sort(function (a, b) {
            return a.b - b.b;
        });

        unsortedQuestList.sort(function (a, b) {
            return a.qp - b.qp;
        });
        var dailyQuests = [], nonDailyQuests = [];
        for (var i = 0; i < unsortedQuestList.length; i++) {
            if (unsortedQuestList[i] == constantsObj.NOT_FOUND) {
                continue;
            }
            if (dailyQuestsObj.isDailyQuestOrStreakQuest(unsortedQuestList[i].qt)) {
                dailyQuests.push(unsortedQuestList[i]);
                questsObj.setDailyQuestLocked(1);
            } else {
                nonDailyQuests.push(unsortedQuestList[i]);
            }
        }

        var sortedQuestList = dailyQuests.concat(nonDailyQuests);
        var expXpromoDq = crossGameDailyQuestsObj.getCurrentCGDQExp();
        var crossGameDq = null;
        var crossGameDqInd = -1;
        for (var i = 0; i < sortedQuestList.length; i++) {
            if (sortedQuestList[i] != undefined) {
                if (sortedQuestList[i].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
                    if (expXpromoDq == constantsObj.EXP_VAR_CONTROL) {
                        sortedQuestList.splice(i, 1);
                        break;
                    }
                    crossGameDq = sortedQuestList[i];
                    crossGameDqInd = i;
                    break;
                } else if (crossGameDailyQuestsObj.getCrossGameDailyQuestCompletedFlag() == 1 && sortedQuestList[i].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_SECONDARY_QUEST_LEVEL) == 0) {
                    crossGameDq = sortedQuestList[i];
                    crossGameDqInd = i;
                    break;
                }
            }
        }
        if (expXpromoDq == constantsObj.EXP_VAR_1) {
            if ((crossGameDq != undefined || crossGameDq != null) && sortedQuestList.length >= 3) {
                var lastDailyQuestIndex = 2;
                for (var i = 0; i < sortedQuestList.length; i++) {
                    if (sortedQuestList[i] != undefined && dailyQuestsObj.isDailyQuestOrStreakQuest(sortedQuestList[i].qt)) {
                        lastDailyQuestIndex = i;
                    }
                }
                sortedQuestList[lastDailyQuestIndex] = crossGameDq;
                sortedQuestList.splice(crossGameDqInd, 1);
            }
        }
        return sortedQuestList;
    };

    var sortedQuestList = null;
    var lastSortingDoneAt = 0;
    this.incActionForQuestType = function (questType) {

        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.incActionForQuestType(questType);
            return;
        }

        var curTimestamp = utilObj.getCurrentTimestamp();
        var questList = questsObj.getSortedQuestList();
        for (var questIndex in questList) {
            var questData = questList[questIndex];
            //loggerObj.debugLog("questData currently is ", utilObj.getStringifiedJson(questData));
            if (questData.qt == questType) {
                var questCompleted = 0;
                var questId = questData.id;
                inGameProgress[questId].params[0] += 1;
                inGameLastLocalUpdate = curTimestamp;
                if (questData.pa != undefined && questData.pa != null) {
                    if (inGameProgress[questId].params[0] >= parseInt(questData.pa)) {
                        questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                        questCompleted = 1;
                    }
                } else {
                    questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
                    questCompleted = 1;
                }
                // if (questCompleted == 1) {
                //     trackObj.trackCounter(constantsObj.TRACK_DIALOG, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(), xpromoConstantsObj.TRACK_QUEST_CENTER, questId, constantsObj.TRACK_COMPLETE, "", "", "");
                // }
            }
        }
    };

    this.getDollarValue = function () {
        return dollarValue;
    };

    this.changeStatusOfAllInstallQuests = function (questId) {
        if (crossGameDataAvailable == 0) {
            return;
        }
        for (var i = 0; i < installAndDoSomethingQuests.length; i++) {
            if (installAndDoSomethingQuests[i] != questId) {
                if (crossGameProgress[installAndDoSomethingQuests[i]] == undefined) {
                    if (installAndDoSomethingQuests[i] == undefined) {
                        continue;
                    }
                    crossGameProgress[installAndDoSomethingQuests[i]] = {
                        'id': installAndDoSomethingQuests[i],
                        'status': xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START,
                    }
                }
            }
        }
    };

    this.getGameShareLink = function (game, screenName, campaignName) {
        var msg1 = "https://playsimple.in/" + game + "?g=%GROUP%&l=%LABEL%";
        if (screenName == constantsObj.SCREEN_MORE_GAMES_LAYER || screenName == constantsObj.SCREEN_SOFT_LAUNCH) {
            msg1 = msg1.replace("%GROUP%", "adwall");
            msg1 = msg1 + "&cgn=" + campaignName;
        } else {
            msg1 = msg1.replace("%GROUP%", "quest_center_" + constantsObj.GAME_ID);
        }
        var label = constantsObj.GAME_ID + ":" + userObj.getRefId();
        msg1 = msg1.replace("%LABEL%", label);
        return msg1;
    };

    this.getRelevantDataCount = function (questId) {
        if (allQuests[questId] == undefined) {
            return 0;
        }
        var questType = allQuests[questId].qt;
        if (questType == xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_ACTION_USE_HINT || questType == xpromoConstantsObj.QUEST_TYPE_INSTALL_ACTION_USE_HINTS) {
            //loggerObj.debugLog("this is hint used count ", xPromoGameSpecificObj.getHintsUsedCount());
            return xPromoGameSpecificObj.getHintsUsedCount();
        } else if ((questType == xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_IN_TIME_PUZZLES || questType == xpromoConstantsObj.QUEST_TYPE_INSTALL_TIME_PUZZLES)
            || (questType == xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_COMPLETE || questType == xpromoConstantsObj.QUEST_TYPE_INSTALL_AND_COMPLETE_PUZZLES)) {
            return xPromoGameSpecificObj.getLevelForQuests();
        } else {
            return 0;
        }
    };

    this.getPayerStatusForDailyQuests = function () {
        return payerStatusForDailyQuests;
    };

    this.setPayerStatusForDailyQuests = function (val) {
        payerStatusForDailyQuests = val;
    };

    this.getLastDateAppOpened = function () {
        return lastDateAppOpened;
    };

    this.setLastDateAppOpened = function (val) {
        lastDateAppOpened = val;
    };

    this.resetDailyQuestsProgress = function () {
        questsToBeAnimated = [];
        dailyQuestsShouldBeReset = 1;
        dailyQuestsResetFlag = true;
        var dailyQuestsMap = questConfigObj.getDailyQuestsConfigMap();

        for (var questType in dailyQuestsMap) {
            if (questConfigObj.isDailyQuestConfigInJson(questType)) {
                var functionName = questConfigObj.getDailyQuestConfigData(questType, xpromoConstantsObj.RESET_FUNCTION_NAME);
                var functionArgs = questConfigObj.getDailyQuestConfigData(questType, xpromoConstantsObj.ARGUMENTS_FOR_RESET_FUNCTION);
                eval(functionName).apply(this, functionArgs);
            }
        }

        dailyQuestsResetFlag = false;
        questsObj.updateAllQuests();

        for (var quest in inGameProgress) {
            if (curQuests != undefined && curQuests[quest] != undefined && curQuests[quest] != null && (dailyQuestsObj.isDailyQuestOrStreakQuest(curQuests[quest].qt) || curQuests[quest].qt == xpromoConstantsObj.QUEST_TYPE_SECONDARY_QUEST_LEVEL)) {
                // trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "stage3", curQuests[quest].id, "", "", "", "", "");
                delete inGameProgress[quest];
            }
        }

        if (questsObj.getPayerStatusForDailyQuests() == 1 || xPromoGameSpecificObj.getIsPayerUser()) {
            questsObj.setPayerStatusForDailyQuests(2);
        }
        questsObj.updateInGameProgress(true);
        // trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "stage4", "", "", "", "", "", "");
    };

    //this function checks whether a new day has begun
    this.shouldResetDailyQuests = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.shouldResetDailyQuests();
            return;
        }

        var d = new Date();
        if (xpromoUtilObj.isWebStaging()) {
            d = new Date(utilObj.getCurrentTimestampMs());
        }
        var curDate = d.getDate() + d.getMonth() + d.getYear();
        // if it is new date reset progress and other variables
        // trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "stage1", "", "", "", "", "", "");
        if (curDate != questsObj.getLastDateAppOpened()) {
            // trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "stage2", "", "", "", "", "", "");

            xPromoGameSpecificObj.resetDQPersonalizedValues();
            questsObj.resetDailyQuestsProgress();
            xpromoW2eObj.resetVariables();
            crossGameDailyQuestsObj.resetVariablesOnDailyBasis();
            dailyQuestsObj.resetDailyQuestReplacementItems();

            // Incase the game open during the change of a day (daily quest expiry)
            dailyQuestsObj.tryUpdateOldQuestData();

            questFlagsObj.getMultiplierForDailyQuests();
            questsObj.setLastDateAppOpened(curDate);
        }
    };

    this.getStatusOfQuest = function (questId) {
        if (crossGameProgress[questId] != undefined && crossGameProgress[questId].status != undefined) {
            return crossGameProgress[questId].status;
        } else {
            return xpromoConstantsObj.QUEST_NOT_STARTED;
        }
    };

    this.grantVideoQuestReward = function () {
        for (var questId in inGameProgress) {
            if (curQuests[questId] != undefined && inGameProgress[questId] != undefined) {
                if (curQuests[questId].qt == xpromoConstantsObj.QUEST_TYPE_DAILY_QUESTS_WATCH_TO_EARN && inGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED) {
                    questsObj.claimInGameQuestReward(questId);
                }
            }
        }
    };

    this.getCurrentDailyQuestMap = function () {
        return currentDailyQuestMap;
    };

    this.getQuestToBeAnimated = function () {
        return questsToBeAnimated;
    };

    this.getCurQuests = function () {
        return curQuests;
    };

    this.getRewardValueForQuest = function (curQuest) {
        if (curQuest != undefined && curQuest != null) {
            if(xPromoGameSpecificObj.isDQPersonalizationEnabled() && dailyQuestsObj.isDailyQuestOrStreakQuest(curQuest.qt)) {
                var dqPayOut = xPromoGameSpecificObj.getDQPayout(curQuest.qt, dailyQuestsObj.getDailyQuestLevel(curQuest.qt, curQuest.id));
                if(dqPayOut != constantsObj.NOT_FOUND && dqPayOut != 0) {
                    return Math.round(dqPayOut);
                }
            }

            if(dailyQuestsObj.isDailyQuestReplacement(curQuest.id)) {
                var payOut = parseInt(xPromoGameSpecificObj.getDQreplacementValue());
                if(payOut != constantsObj.NOT_FOUND && payOut != 0 )
                    return Math.round(payOut);
            }

            return Math.round(curQuest.re * questsObj.getDollarValue());
        }
        return 0;
    };

    this.getCurQuestData = function (questId) {
        var curQuests = questsObj.getCurQuests();
        if(curQuests == undefined || curQuests == null) {
            return null;
        }
        var curQuest = curQuests[questId];
        if (curQuest == undefined) {
            curQuest = requestsObj.getCurrentActiveRequest()[0];
        }
        return curQuest;
    };

    this.getAllQuests = function () {
        return allQuests;
    };

    this.getInGameProgressMap = function () {
        return inGameProgress;
    };

    // * this function evaluates whether to show daily quest notif or not
    this.dailyQuestsCompleted = function () {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.updateDailyQuestsNotif();
            return;
        }

        var quests = questsObj.getCurQuests();
        var count = 0, ReqdQuests = 0, isCGDQAvailable = 0;
        for (var quest in quests) {
            if (dailyQuestsObj.isDailyQuestOrStreakQuest(quests[quest].qt) && quests[quest].gh == constantsObj.GAME_ID && dailyQuestsEligibleToBeShown[quest] == 1) {
                if (quests[quest].pf == 0 && (questsObj.getPayerStatusForDailyQuests() == 0 || questsObj.getPayerStatusForDailyQuests() == 1)) {
                    ReqdQuests++;
                } else if (quests[quest].pf == 1 && questsObj.getPayerStatusForDailyQuests() == 2){
                    ReqdQuests++;
                }
                if (inGameProgress[quest] != undefined && inGameProgress[quest].status == xpromoConstantsObj.QUEST_CLAIMED) {
                    count++;
                }
            }
            if (quests[quest].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0 && questsObj.isQuestShownInGame(quests[quest].id) && questsObj.isQuestAvailableOnClient(quests[quest].cl) && crossGameDailyQuestsObj.getDailyQuestToBeShown() != null) {
                isCGDQAvailable = 1;
            }
        }
        if (isCGDQAvailable == 1 && (crossGameDailyQuestsObj.getCurrentCGDQExp() == constantsObj.EXP_VAR_1 || crossGameDailyQuestsObj.getCurrentCGDQExp() == constantsObj.EXP_VAR_2)) {
            ReqdQuests++;
            // * since cgdq will take a place for 3rd daily quest bucket in experiment 1
            if (crossGameDailyQuestsObj.getCurrentCGDQExp() == constantsObj.EXP_VAR_1) {
                ReqdQuests = ReqdQuests - 3;
            }
            if (crossGameDailyQuestsObj.getCrossGameDailyQuestCompletedFlag() == 1) {
                count++;
            }
        }

        if (!dailyQuestsObj.isDailyQuestsEligibleDays()) {
            crossGameDailyQuestsObj.setDailyQuestUnlockTime(dailyQuestsObj.getDailyQuestUnlockTime());
            crossGameDailyQuestsObj.setShouldShowDailyQuestNotif(false);
            crossGameDailyQuestsObj.setIsDailyQuestCompleted(false);
        } else if (count == ReqdQuests) {
            crossGameDailyQuestsObj.setShouldShowDailyQuestNotif(true);
            crossGameDailyQuestsObj.setIsDailyQuestCompleted(true);
        } else {
            crossGameDailyQuestsObj.setShouldShowDailyQuestNotif(true);
            crossGameDailyQuestsObj.setIsDailyQuestCompleted(false);
        }
    };

    this.shouldToasterBeShown = function (questId) {
        if ((inGameProgress[questId] != undefined && inGameProgress[questId].status != xpromoConstantsObj.QUEST_COMPLETED) || (crossGameProgress[questId] != undefined && crossGameProgress[questId].status != xpromoConstantsObj.QUEST_COMPLETED)) {
            return true;
        } else {
            return false;
        }
    };

    this.makeStringAppropriate = function (stringToChange, stringToReplaceWith, questType) {
        if (stringToReplaceWith == 1 && questConfigObj.isDailyQuestConfigInJson(questType)) {
            var singular = questConfigObj.getDailyQuestConfigData(questType, xpromoConstantsObj.DAILY_QUEST_SINGULAR_NAME);
            var plural = questConfigObj.getDailyQuestConfigData(questType, xpromoConstantsObj.DAILY_QUEST_PLURAL_NAME);
            stringToChange = stringToChange.replace(plural, singular);
        }
        return stringToChange;
    };

    this.getQuestsCountInBucket = function () {
        var questsCount = {};
        if (curQuests == undefined || curQuests == null) {
            return questsCount;
        }
        for (var quest in inGameProgress) {
            if (curQuests[quest] != undefined && inGameProgress[quest] != undefined && inGameProgress[quest] != null && inGameProgress[quest].status == xpromoConstantsObj.QUEST_COMPLETED) {
                if (questsCount[curQuests[quest].b] == undefined) {
                    questsCount[curQuests[quest].b] = 0;
                }
                questsCount[curQuests[quest].b]++;
            }
        }

        for (var quest in crossGameProgress) {
            if (questsObj.isQuestShownInGame(quest) && curQuests[quest] != undefined && crossGameProgress[quest] != undefined && crossGameProgress[quest].status == xpromoConstantsObj.QUEST_COMPLETED) {
                if (questsCount[curQuests[quest].b] == undefined) {
                    questsCount[curQuests[quest].b] = 0;
                }
                questsCount[curQuests[quest].b]++;
            }
        }
        return questsCount;
    };

    this.setCohortDays = function (val) {
        cohortDaysTracking = val;
    };

    this.getCohortDays = function () {
        return cohortDaysTracking;
    };

    this.getDailyQuestTrackingStr = function (questId) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        var curQuests = questsObj.getCurQuests();
        var lvl = curQuests[questId].p;
        lvl = lvl %xpromoConstantsObj.NUMBER_OF_DAILY_QUESTS_IN_BUCKET ? lvl %xpromoConstantsObj.NUMBER_OF_DAILY_QUESTS_IN_BUCKET : xpromoConstantsObj.NUMBER_OF_DAILY_QUESTS_IN_BUCKET;
        var trackStr = "daily_quest";
        if (curQuests[questId].pf != undefined && curQuests[questId].pf != null && curQuests[questId].pf == 1) {
            trackStr += "_payer";
        } else {
            trackStr += "_non_payer"
        }

        trackStr = trackStr + "_D" + questsObj.getCohortDays();
        var questType = curQuests[questId].qt;
        var dq = "";
        if (questConfigObj.isDailyQuestConfigInJson(questType)) {
            dq = questConfigObj.getDailyQuestConfigData(questType, xpromoConstantsObj.TRACKER_NAME);
        }
        trackStr = trackStr + "_" + dq + "_level" + lvl;
        return trackStr;
    };

    this.completedQuestsCount = function () {
        var completedQuestsCount = 0;
        for (var quest in inGameProgress) {
            if (inGameProgress[quest] != undefined && inGameProgress[quest].status == xpromoConstantsObj.QUEST_COMPLETED) {
                completedQuestsCount++;
            }
        }
        for (var quest in crossGameProgress) {
            if (crossGameProgress[quest] != undefined && crossGameProgress[quest].status == xpromoConstantsObj.QUEST_COMPLETED) {
                completedQuestsCount++;
            }
        }
        return completedQuestsCount;
    };

    this.mergeAllGamesDataMap = function () {
        // write all the things you want to share across games
        var isPayer = xPromoGameSpecificObj.getIsPayer() ? 1 : 0;
        var isFbConnected = xPromoGameSpecificObj.isLoggedIn() ? 1 : 0;
        var allGamesDataForCurrentGame = [constantsObj.GAME_ID, userObj.getRefId(), utilObj.getVersionCode(), userObj.getCreated(), xPromoGameSpecificObj.getLastGameOpenTimestamp(), xPromoGameSpecificObj.getPuzCompleted(),
            questsObj.getOptimizedAdvertiserId(), xPromoGameSpecificObj.getCash(), isPayer, isFbConnected];
        if (allGamesDataMap[constantsObj.GAME_ID] == undefined) {
            return allGamesDataForCurrentGame;
        } else {
            allGamesDataForCurrentGame[DATA_SHARING_INDEX_CREATED_TS] = Math.min(allGamesDataForCurrentGame[DATA_SHARING_INDEX_CREATED_TS], allGamesDataMap[constantsObj.GAME_ID][DATA_SHARING_INDEX_CREATED_TS]);
            if (allGamesDataMap[constantsObj.GAME_ID][DATA_SHARING_INDEX_PUZZLE_COMPLETES] == NaN || allGamesDataMap[constantsObj.GAME_ID][DATA_SHARING_INDEX_PUZZLE_COMPLETES] == undefined) {
                allGamesDataMap[constantsObj.GAME_ID][DATA_SHARING_INDEX_PUZZLE_COMPLETES] = 0;
            }
            allGamesDataForCurrentGame[DATA_SHARING_INDEX_PUZZLE_COMPLETES] = Math.max(allGamesDataForCurrentGame[DATA_SHARING_INDEX_PUZZLE_COMPLETES], allGamesDataMap[constantsObj.GAME_ID][DATA_SHARING_INDEX_PUZZLE_COMPLETES]);
        }
        return allGamesDataForCurrentGame;
    };

    this.setAllGamesData = function (allGamesData) {
        if (allGamesData == undefined || allGamesData == "") {
            if(optimizedAdIdDataSharing == null) {
                optimizedAdIdDataSharing = xpromoUtilObj.getAdvertiserId();
                questsObj.trackAdIdAssignment();
            }
            return;
        }
        var allGamesDataArr = allGamesData.split("_");
        var oldestAdIdFromGames = null;
        var oldestInstallTime = -1;
        for (var i = 0; i < allGamesDataArr.length; i++) {
            if (allGamesDataArr[i] != "") {
                var gameData = allGamesDataArr[i];
                gameData = gameData.split(":");
                if (allGamesDataMap[gameData[0]] != undefined) {
                    gameData[DATA_SHARING_INDEX_CREATED_TS] = Math.min(gameData[DATA_SHARING_INDEX_CREATED_TS], allGamesDataMap[gameData[0]][DATA_SHARING_INDEX_CREATED_TS]);
                    if (allGamesDataMap[gameData[0]][DATA_SHARING_INDEX_PUZZLE_COMPLETES] == NaN || allGamesDataMap[gameData[0]][DATA_SHARING_INDEX_PUZZLE_COMPLETES] == undefined) {
                        allGamesDataMap[gameData[0]][DATA_SHARING_INDEX_PUZZLE_COMPLETES] = 0;
                    }
                    if (gameData[DATA_SHARING_INDEX_PUZZLE_COMPLETES] == NaN || gameData[DATA_SHARING_INDEX_PUZZLE_COMPLETES] == undefined) {
                        gameData[DATA_SHARING_INDEX_PUZZLE_COMPLETES] = 0;
                    }
                    gameData[DATA_SHARING_INDEX_PUZZLE_COMPLETES] = Math.max(gameData[DATA_SHARING_INDEX_PUZZLE_COMPLETES], allGamesDataMap[gameData[0]][DATA_SHARING_INDEX_PUZZLE_COMPLETES]);
                }

                /*
                    We need to go through the adIds passed by different games (if they are)
                    Take the adId of the oldest game and save it to local quest data for future usage for crossGameQuests
                    Also use the same adId in data sharing between games
                */
                if(gameData[DATA_SHARING_INDEX_ADV_ID] != undefined) {
                    if(oldestAdIdFromGames == null || (oldestAdIdFromGames != null && oldestInstallTime > parseInt(gameData[DATA_SHARING_INDEX_CREATED_TS]))) {
                        oldestAdIdFromGames = gameData[DATA_SHARING_INDEX_ADV_ID];
                        oldestInstallTime = parseInt(gameData[DATA_SHARING_INDEX_CREATED_TS]);
                    }
                }

                // keeping all the data from other games in the allGamesDataMap in the form of array
                allGamesDataMap[gameData[0]] = gameData;
            }
        }
        // This ensures that the "optimizedAdIdDataSharing" is set at the end of call this method.
        if(optimizedAdIdDataSharing == null) {
            if(oldestAdIdFromGames != null && oldestAdIdFromGames != "") {
                optimizedAdIdDataSharing = oldestAdIdFromGames;
            } else {
                optimizedAdIdDataSharing = xpromoUtilObj.getAdvertiserId();
            }
            questsObj.trackAdIdAssignment();
        }
    };

    this.encodeAllGameData = function (force) {
        if (crossGameDataAvailable == 0 && force != true) {
            return;
        }
        // *  merging progress of server and client
        // * this case will come when user has uninstalled current app and reinstalled again
        var allGamesDataMapForCurrentgame = questsObj.mergeAllGamesDataMap();
        allGamesDataMap[constantsObj.GAME_ID] = allGamesDataMapForCurrentgame;

        var allGamesDataString = "";
        //TODO: we have to choose appropriate block
        for (var gameData in allGamesDataMap) {
            for (var ix = 0; ix < allGamesDataMap[gameData].length; ix++) {
                if (allGamesDataMap[gameData][ix] != "" || allGamesDataMap[gameData][ix] === 0) {
                    allGamesDataString += allGamesDataMap[gameData][ix] + ":";
                }
            }
            allGamesDataString = utilObj.trimLastChar(allGamesDataString);
            allGamesDataString += "_";
        }

        return allGamesDataString;
    };

    this.redirectToStoreWithTracking = function (campaignName, screenName, params, trackingId, responseFromAdjust) {
        var adjustRedirectExp = epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_XPROMO_REDIRECT);
        var redirectExp = epmObj.getChosenVariant(xpromoConstantsObj.EXP_REDIRECT_TO_APP_STORE);
        var link;

        if (utilObj.isAndroid()) {
            if (xpromoUtilObj.isAppInstalled(params["adrAppId"])) {
                utilObj.launchAnotherApp(params["adrAppId"]);
            }
            else if (redirectExp == constantsObj.EXP_VAR_CONTROL) {
                link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
                utilObj.openStore(null, link, screenName);
            } else {
                if (adjustRedirectExp == constantsObj.EXP_VAR_CONTROL || campaignName != xpromoConstantsObj.TRACK_QUEST_CENTER) {

                }
                utilObj.openUrl("market://details?id=" + params["adrAppId"]);
            }
        } else if (utilObj.isIos()) {
            if (xpromoUtilObj.isAppInstalled(params["fbId"])) {
                utilObj.launchAnotherApp(params["fbId"]);
            } else if (redirectExp == constantsObj.EXP_VAR_CONTROL) {
                link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
                utilObj.openStore(null, link, screenName);
            } else if (redirectExp == constantsObj.EXP_VAR_1) {
                utilObj.openUrl("itms-apps://itunes.apple.com/app/id" + params["iosAppId"]);
            } else if (redirectExp == constantsObj.EXP_VAR_2) {
                link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
                utilObj.openStore(params["iosAppId"], link, screenName);
            }
            if (redirectExp == constantsObj.EXP_VAR_1 || redirectExp == constantsObj.EXP_VAR_2) {
                if (adjustRedirectExp == constantsObj.EXP_VAR_CONTROL || campaignName != xpromoConstantsObj.TRACK_QUEST_CENTER) {

                }
            }
        } else if (xpromoUtilObj.isWebStaging()) {
            xpromoUtilObj.openPlayStoreUrl(params["adrAppId"]);
        }
    };

    this.setCrossGameDataAvailable = function (val) {
        crossGameDataAvailable = val;
    };

    this.fireTrackingForEachQuest = function (rowInfo, speciesTrack) {
        var questId = rowInfo["id"];
        var trackingIdInfo = "";
        if (rowInfo["trackingId"] != undefined) {
            trackingIdInfo = rowInfo["trackingId"];
        }
        if(speciesTrack == undefined) {
            speciesTrack = trackingIdInfo;
        }
        trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(), xpromoConstantsObj.TRACK_QUEST_CENTER, rowInfo["id"], constantsObj.TRACK_VIEW, speciesTrack, "", "");
        var trackId = xpromoUtilObj.getAdjustTrackerForGameId(rowInfo["gh"]);
        if (allQuests[questId].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
            xpromoUtilObj.hitAdjustViewTracking(trackId, xpromoConstantsObj.TRACK_QUEST_CENTER_DQ, trackingIdInfo);
        } else {
            xpromoUtilObj.hitAdjustViewTracking(trackId, xpromoConstantsObj.TRACK_QUEST_CENTER, trackingIdInfo);
        }
    };

    this.redirectToPlaystoreOnClick = function (questId, campaignSrc, trackingId) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            return;
        }

        if (campaignSrc == undefined) {
            campaignSrc = xpromoConstantsObj.TRACK_QUEST_CENTER;
        }
        var questData = allQuests[questId];
        if (questData == undefined) {
            return;
        }
        var params = questData.pa.split(":"), redirectionParams;
        if (questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
            redirectionParams = {
                "adrAppId": params[0],
                "gameLink": params[2],
                "gameId": questData.gh,
                "fbId": params[4],
                "iosAppId": params[3]
            };
            questsObj.redirectToStoreWithTracking(campaignSrc, constantsObj.SCREEN_QUEST_CENTER, redirectionParams, trackingId);
        } else if (questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_COMPLETE) == 0
            || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_AND) == 0 || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_ACTION) == 0
            || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_ACTION) == 0) {
            redirectionParams = {
                "adrAppId": params[0],
                "gameLink": params[2],
                "gameId": questData.gh,
                "fbId": params[5],
                "iosAppId": params[4]
            };
            questsObj.redirectToStoreWithTracking(campaignSrc, constantsObj.SCREEN_QUEST_CENTER, redirectionParams, trackingId);
        } else if (questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_LEVEL_IN_TIME) == 0
            || questData.qt.indexOf(xpromoConstantsObj.QUEST_TYPE_INSTALL_TIME) == 0) {
            redirectionParams = {
                "adrAppId": params[0],
                "gameLink": params[2],
                "gameId": questData.gh,
                "fbId": params[6],
                "iosAppId": params[5]
            };
            questsObj.redirectToStoreWithTracking(campaignSrc, constantsObj.SCREEN_QUEST_CENTER, redirectionParams, trackingId);
        } else {
            questsObj.redirectToPlaystore(params[2]);
        }
    };

    this.getRelatedQuests = function () {
        return crossGameRelatedQuests;
    };

    this.getRelatedQuestsForQuestId = function (questId) {
        if (crossGameRelatedQuests == null || crossGameRelatedQuests == undefined) {
            questsObj.makeRelatedQuests();
            if (crossGameRelatedQuests == null || crossGameRelatedQuests == undefined) {
                return null;
            }
        }
        return crossGameRelatedQuests[questId];
    };

    this.sortQuests = function (unsortedQuests, unsortedQuestStr, force) {
        if (sortedQuestsMap[unsortedQuestStr] != undefined && force != true) {
            return sortedQuestsMap[unsortedQuestStr];
        }
        var questsArr = [];

        for (var quest in unsortedQuests) {
            unsortedQuests[quest].id = quest;
            questsArr.push(unsortedQuests[quest]);
        }

        questsArr.sort(function (a, b) {
            return a.p - b.p;
        });

        questsArr.sort(function (a, b) {
            return a.b - b.b;
        });

        questsArr.sort(function (a, b) {
            return a.qp - b.qp;
        });

        sortedQuestsMap[unsortedQuestStr] = questsArr;
        questsObj.makeRelatedQuests(true);
        return questsArr;
    };

    this.appInstalled = function () {
        questsObj.getAvailableCrossGameQuests();
    };

    this.getCrossGameProgressMap = function () {
        return crossGameProgress;
    };

    /**
     * checks whether necessary conditions are met for granting coins for xpromotion
     * follows the same flow for bubble pop popup and w2e popup
     */
    this.checkForXpromoAfterPopupSurfacing = function () {
        gameLayer.xpromoAfterW2eChecksDone = 0;
        if (questsObj.getShouldIncrementOnClick() == 0 && (xPromoGameSpecificObj.isScreenForW2eXpromo(curScreen) || xPromoGameSpecificObj.isScreenForW2eXpromo(lastScreen))) {
            if (xPromoGameSpecificObj.shouldCheckForQuestCompleteOnPopupSurfacing()) {
                questsObj.xpromoAfterPopupSurfacingChecks();
                gameLayer.xpromoAfterW2eChecksDone = 1;
            }
        }
        bubblePopObj.checkAndGrantXpromoReward();
        giftBoxObj.checkAndGrantXpromoReward();
        adWallObj.checkAndGrantXpromoReward();
        adWallObj.checkAndSetInstalledFriendData();
    };

    /**
     * this function checks whether the user has installed an app after clicking on bubble pop xpromo popup or w2e popup in a specified attribution window
     * this function also grants coins for installing another app
     * in case if user is eligible for reward, it will be also change states if related quests to claimed
     */
    this.xpromoAfterPopupSurfacingChecks = function () {
        if ((utilObj.getCurrentTimestamp() - questsObj.getXpromoSurfacingButtonLastClicked()) / (60 * 60) > xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_SURFACING_ATTRIBUTION_WINDOW)) {
            return;
        }
        var xpromoActiveQuest = questsObj.getXpromoW2eQuestActive();
        if (xpromoActiveQuest == null || xpromoActiveQuest == "") {
            return;
        }

        var currentAdwallQuest;
        if(pspnFlutterObj.isFlutterEnabled()) {
            currentAdwallQuest = pspnFlutterObj.getAdwallQuestById(xpromoActiveQuest)
        } else if (allQuests != null) {
            currentAdwallQuest = allQuests[xpromoActiveQuest];
        }

        if (currentAdwallQuest == undefined) {
            return;
        }

        if(xpromoUtilObj.isGameAlreadyMarkedForRewarding(currentAdwallQuest.gh, xpromoConstantsObj.TRACK_W2E_QUEST)) {
            questsObj.setXpromoW2eQuestActive("");
            return;
        }

        var questParams = currentAdwallQuest.pa.split(":");

        utilObj.resetIsAppInstalledMap();
        if (xpromoUtilObj.isAppInstalled(questParams[0])) {
            // checking whether user clicked on xpromo popup play button
            if (questsObj.getShouldIncrementOnClick() == 0) {
                if (popupLayer.popupOpen && (xPromoGameSpecificObj.isScreenForW2eXpromo(lastScreen))) {
                    var layerForAnimation = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(lastScreen);
                    popupLayer.closeWithDefault();
                    questsObj.makeRelatedQuestsForW2eClaimed(xpromoActiveQuest);
                    // TODO - We had changed this to not use delayedCallback, why?
                    utilObj.delayedCallback(layerForAnimation, questsObj.runCoinGrantAnim, 2, null, false);
                } else if (xPromoGameSpecificObj.isScreenForW2eXpromo(curScreen)) {
                    var layerForAnimation = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen);
                    questsObj.makeRelatedQuestsForW2eClaimed(xpromoActiveQuest);
                    questsObj.runCoinGrantAnim(layerForAnimation);
                }
            } else {
                questsObj.makeRelatedQuestsForW2eClaimed(xpromoActiveQuest);
                questsObj.runCoinGrantAnim(xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen));
            }
        }
    };

    this.makeRelatedQuestsForW2eClaimed = function (questId) {

        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.updateRelatedQuestsStatus(questId, xpromoConstantsObj.QUEST_CLAIMED);
            return;
        }

        var questData = (allQuests != undefined && allQuests.hasOwnProperty(questId)) ? allQuests[questId] : undefined;
        if (questData == undefined) {
            return;
        }

        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_CLAIMED);
        var relatedQuests = questData.r.split(":");
        for (var i = 0; i < relatedQuests.length; i++) {
            questsObj.updateCrossGameQuestStatus(relatedQuests[i], xpromoConstantsObj.QUEST_CLAIMED);
        }
    };

    this.runCoinGrantAnim = function (sender) {
        if (sender == undefined) {
            sender = gameLayer;
        }
        var grantCoins = xpromoW2eObj.getMultipliedLastCoinsAdded();
        xPromoGameSpecificObj.canW2eCoinGrant(0, 0, grantCoins, false, sender);

        var w2eExp = epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_XPROMO_W2E);
        var creativeId = xpromoConstantsObj.TRACK_W2E_QUEST + "_" + w2eExp;

        var desc = "";
        var gameHosted = 0;
        var priority = -1;

        var quest;
        var xpromoActiveQuest = questsObj.getXpromoW2eQuestActive();
        if(xpromoActiveQuest != null && xpromoActiveQuest != "") {
            if(pspnFlutterObj.isFlutterEnabled()) {
                quest = pspnFlutterObj.getAdwallQuestById(xpromoActiveQuest);
            } else {
                quest = (allQuests != null) ? allQuests[xpromoActiveQuest] : null;
            }
        }

        if(quest != null) {
            desc = quest.d;
            gameHosted = quest.gh;
            priority = quest.p;
        }

        utilObj.showMessage("Congrats! You've earned " + grantCoins + " coins for installing " + desc);
        trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xpromoActiveQuest, gameHosted, xpromoConstantsObj.TRACK_W2E_QUEST, constantsObj.TRACK_COMPLETE, "", creativeId, "", "");
        // TODO - This was moved out and "grantCoinsAfterDelay" was moved directly here without
        utilObj.delayedCallback(sender, xPromoGameSpecificObj.grantCoinsAfterDelay, 2.0, {cashToGrant: grantCoins, phylum: constantsObj.TRACK_EARN, class: "push", order: "", shouldTrack: false}, false);
        xPromoGameSpecificObj.fireEarnTracking(false, xpromoConstantsObj.TRACK_PUSH, constantsObj.TRACK_COINS, xPromoGameSpecificObj.getCashEquivalent() + "", grantCoins + "", "", true);
        questsObj.setXpromoW2eQuestActive("");
        xpromoW2eObj.resetPriorityPostInstall(priority);
        gameLayer.xpromoAfterW2eChecksDone = 1;
    };

    this.getAllGamesDataMap = function () {
        return allGamesDataMap;
    };

    this.getCGDQProgress = function () {
        cgdqProgress = "";
        for (var quest in crossGameProgress) {
            var questParams = "";
            if (crossGameProgress[quest].params != undefined) {
                for (var i = 0; i < crossGameProgress[quest].params.length; i++) {
                    questParams += crossGameProgress[quest].params[i] + ":";
                }
                questParams = utilObj.trimLastChar(questParams);
                if (curQuests != null && crossGameProgress[quest] != null && crossGameProgress[quest] != undefined && curQuests[quest] != undefined && curQuests[quest].qt.indexOf(xpromoConstantsObj.QUEST_TYPE_CROSS_GAME_DQ) == 0) {
                    cgdqProgress += crossGameProgress[quest].id + ":" + crossGameProgress[quest].status + ":" + questParams + ";";
                }
            }
        }
        return cgdqProgress;
    };


    this.mergeCGDQProgress = function (cgdqProgress) {
        if (cgdqProgress != "") {
            crossGameProgress = questsObj.getCrossGameProgressMap();
            var questProgress = cgdqProgress.split(";");
            for (var i = 0; i < questProgress.length; i++) {
                if (questProgress[i] == "") {
                    continue;
                }
                var singleQuestProgress = questProgress[i].split(":");
                var questparams = singleQuestProgress.splice(2, 4);

                if (crossGameProgress[singleQuestProgress[0]] == undefined) {
                    crossGameProgress[singleQuestProgress[0]] = {
                        "id": singleQuestProgress[0],
                        "status": xpromoConstantsObj.QUEST_NOT_STARTED,
                    }
                }
                if (crossGameProgress[singleQuestProgress[0]].params == undefined) {
                    crossGameProgress[singleQuestProgress[0]].params = [0, 0, 0, -1];
                }

                if (crossGameProgress[singleQuestProgress[0]] != undefined) {
                    if (crossGameProgress[singleQuestProgress[0]].params != undefined && parseInt(questparams[3]) > parseInt(crossGameProgress[singleQuestProgress[0]].params[3])) {
                        crossGameProgress[singleQuestProgress[0]].params = questparams;
                        questsObj.updateCrossGameQuestStatus(singleQuestProgress[0], singleQuestProgress[1]);
                    }
                    if ((crossGameProgress[singleQuestProgress[0]].status == undefined || crossGameProgress[singleQuestProgress[0]].status == xpromoConstantsObj.QUEST_NOT_STARTED) && singleQuestProgress[1] == xpromoConstantsObj.QUEST_STARTED) {
                        questsObj.updateCrossGameQuestStatus(singleQuestProgress[0], singleQuestProgress[1]);
                    }
                }
            }
        }
    };

    this.getEncodedAllGamesDataMap = function () {
        return questsObj.encodeAllGameData(true);
    };

    this.sendTrackingForAppOpen = function (gameId) {
        if (crossGameDailyQuestsObj.getCgdqReceiverActive() != null) {
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, crossGameDailyQuestsObj.getCurrentCGDQExp(), gameId, xpromoConstantsObj.TRACK_CGDQ_RECIEVER, constantsObj.TRACK_APP_OPEN, "", "", "", "");
        }
        if (crossGameDailyQuestsObj.getDailyQuestToBeShown() != null) {
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, crossGameDailyQuestsObj.getCurrentCGDQExp(), gameId, xpromoConstantsObj.TRACK_CGDQ, constantsObj.TRACK_APP_OPEN, "", "", "", "");
        }
    };

    this.isEligibleForQuest = function (questData) {
        if (questData.qt == constantsObj.QUEST_TYPE_SPIN_SPINNER) {
            if (gameObj.isSpinnerAvailable()) {
                return true;
            }
        } else if (questData.qt == xpromoConstantsObj.QUEST_TYPE_USE_HINT_TARGET) {
            if (gameObj.isInPickHintVariant() && gameObj.isPickHintUnlocked()) {
                return true;
            }
        } else if (questData.qt == xpromoConstantsObj.QUEST_TYPE_USE_HINT_BURST) {
            if (gameObj.isInBurstHintVariant() && gameObj.isBurstHintUnlocked()) {
                return true;
            }
        } else if (questData.qt == xpromoConstantsObj.QUEST_TYPE_PLAY_DAILY_PUZZLE) {
            if (dailyPuzzleObj.isDailyPuzzleFeatureUnlocked()) {
                return true;
            }
        } else {
            return true;
        }
        return false;
    };

    this.changeQuestDataForIos = function (questData) {
        if (utilObj.isIos() && questData.cg == 1) {
            var showQuestClaimRuntime = parseInt(xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_CROSS_GAME_QUEST_CLAIM));
            var showQuestIncentiveDaysAfter = parseInt(xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_CROSS_GAME_QUEST_SHOW_DAYS_AFTER));
            var showQuestIncentivePuzzlesAfter = parseInt(xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_CROSS_GAME_QUEST_SHOW_PUZZLES_AFTER));
            var currentTimeStamp = utilObj.getCurrentTimestamp(),
                installTimeStamp = xPromoGameSpecificObj.getInstallTimeStamp();
            var currentDate = new Date(currentTimeStamp * 1000), installDate = new Date(installTimeStamp * 1000);
            currentDate.setHours(0, 0, 0, 0), installDate.setHours(0, 0, 0, 0);
            var currentDateTimeStamp = currentDate.getTime() / 1000,
                installDateTimeStamp = installDate.getTime() / 1000;
            var questId = questData.id;

            // *showing reward as 0 for ios users if he has not exceeded runtime days or puzzle completed are less than request runtime levels
            if ((currentDateTimeStamp - installDateTimeStamp) / constantsObj.SECONDS_IN_DAY < showQuestIncentiveDaysAfter || xPromoGameSpecificObj.getPuzCompleted() < showQuestIncentivePuzzlesAfter) {
                questData.re = 0;
                //* if showQuestClaimRuntime is 0 then remove the quest surfacing on completing quest
                //* if showQuestClaimRuntime is 1 then show reward on quest complete and donot remove quest surfacing
                if (crossGameProgress[questId] != undefined && crossGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED) {
                    if (showQuestClaimRuntime == xpromoConstantsObj.DONOT_SHOW_REWARD_ON_QUEST_COMPLETE) {
                        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_CLAIMED);
                    } else {
                        questData.re = allQuests[questId].re;
                    }
                }
            } else {
                questData.re = allQuests[questId].re;
            }
        }
        return questData;
    };

    this.crossGameProgressMerge = function (cgProgress) {
        questsObj.mergeCrossGameProgress(cgProgress, 1, utilObj.getCurrentTimestamp());
    };
    
    this.executingFunctionsAfterGettingSharedData = function (sharedDataMap) {
        if(pspnFlutterObj.isFlutterEnabled()){
            return;
        }
        try {
            sharedDataMap = JSON.parse(sharedDataMap);
            questsObj.setAllGamesData(sharedDataMap[xpromoConstantsObj.ALL_GAMES_DATA]);
            xpromoUtilObj.markSharedDataFetched();
            questsObj.crossGameProgressMerge(sharedDataMap[xpromoConstantsObj.CROSS_GAME_PROGRESS]);
            var gameId = sharedDataMap[xpromoConstantsObj.GAME_ID_AS_STRING];
            if (utilObj.getCurrentTimestamp() - sharedDataMap[xpromoConstantsObj.LAST_UPDATE] < 5 && gameId != constantsObj.GAME_ID) {
                questsObj.sendTrackingForAppOpen(gameId);
            }
            xpromoUtilObj.actionPostDataSharingFetch();
        } catch (e) {
            // Shared Data Map might be corrupted, lets try to fetch the data from server in that case
            xpromoUtilObj.markSharedDataFetched();
            questsObj.getCrossGameProgress();
        }

        if(xPromoGameSpecificObj.isUserInteractionTrackingEnabled()) {
            machineLearningDataObj.setPSGamesSnapshot();
        }
    };

    this.getCrossGameQuestStatusForId = function (questId) {
        if (crossGameProgress[questId] != undefined && crossGameProgress[questId].status != undefined) {
            return crossGameProgress[questId].status;
        }
        return xpromoConstantsObj.QUEST_NOT_STARTED;
    };

    this.getCrossGameDataAvailable = function () {
        return crossGameDataAvailable;
    };

    /**
     * Utility method to enable layers notifying the number of quest completes
     * @param layersToChange - array of layers to changes
     */
    this.showQuestsCompletedJewelNotif = function (layersToChange, llpLayersToChange) {
        for (var i=0; i<layersToChange.length; i++) {
            if (layersToChange[i] != undefined) {
                utilObj.setVisible(layersToChange[i].newLabelNotifCircleIconQuests, false);
                utilObj.setVisible(layersToChange[i].newLabelNumQuestsLbl, false);
                utilObj.setVisible(layersToChange[i].notifCircleIconQuests, true);
                utilObj.setVisible(layersToChange[i].numQuestsLbl, true);
                utilObj.setVisible(layersToChange[i].notifCircle, true);
                utilObj.setString(layersToChange[i].numQuestsLbl, questsObj.getCompletedQuests() + "");
            }
        }

        for(var i = 0; i < llpLayersToChange.length; i++) {
            utilObj.setVisible(llpLayersToChange[i].notifCircle1, true);
            utilObj.setVisible(llpLayersToChange[i].notifCircle2, true);
            utilObj.setString(llpLayersToChange[i].numQuestsLbl1, questsObj.getCompletedQuests() + "");
            utilObj.setString(llpLayersToChange[i].numQuestsLbl2, questsObj.getCompletedQuests() + "");
            utilObj.setVisible(llpLayersToChange[i].notifCircle, true);
            utilObj.setVisible(llpLayersToChange[i].numQuestsLbl1, true);
            utilObj.setVisible(llpLayersToChange[i].numQuestsLbl2, true);
            utilObj.setVisible(llpLayersToChange[i].newLabelNotifCircleIconQuests, false);
            utilObj.setVisible(llpLayersToChange[i].newLabelNumQuestsLbl, false);
        }
    };

    /**
     * Utility method to disable layers notifying the number of quest completes
     * @param layersToChange - array of layers to changes
     */
    this.hideQuestCompletedJewelNotif = function (layersToChange, llpLayersToChange) {
        for (var i=0; i<layersToChange.length; i++) {
            if (layersToChange[i] != undefined) {
                utilObj.setVisible(layersToChange[i].newLabelNotifCircleIconQuests, false);
                utilObj.setVisible(layersToChange[i].newLabelNumQuestsLbl, false);
                utilObj.setVisible(layersToChange[i].notifCircleIconQuests, false);
                utilObj.setVisible(layersToChange[i].numQuestsLbl, false);
                utilObj.setString(layersToChange[i].numQuestsLbl, questsObj.getCompletedQuests() + "");
                utilObj.setVisible(layersToChange[i].notifCircle, false);
            }
        }

        for(var i = 0; i < llpLayersToChange.length; i++) {
            utilObj.setVisible(llpLayersToChange[i].notifCircle1, false);
            utilObj.setVisible(llpLayersToChange[i].notifCircle2, false);
            utilObj.setString(llpLayersToChange[i].numQuestsLbl1, questsObj.getCompletedQuests() + "");
            utilObj.setString(llpLayersToChange[i].numQuestsLbl2, questsObj.getCompletedQuests() + "");
            utilObj.setVisible(llpLayersToChange[i].notifCircle, false);
            utilObj.setVisible(llpLayersToChange[i].numQuestsLbl1, false);
            utilObj.setVisible(llpLayersToChange[i].numQuestsLbl2, false);
            utilObj.setVisible(llpLayersToChange[i].newLabelNotifCircleIconQuests, false);
            utilObj.setVisible(llpLayersToChange[i].newLabelNumQuestsLbl, false);
        }
    };

    /**
     * Utility method to enable a tag on quest center for new quest availability
     * @param layersToChange - array of layers to changes
     */
    this.showNewTagJewelnotif = function (layersToChange, llpLayersToChange) {
        for (var i=0; i<layersToChange.length; i++) {
            if (layersToChange[i] != undefined) {
                utilObj.setVisible(layersToChange[i].newLabelNotifCircleIconQuests, true);
                utilObj.setVisible(layersToChange[i].newLabelNumQuestsLbl, true);
                utilObj.setVisible(layersToChange[i].notifCircleIconQuests, false);
                utilObj.setVisible(layersToChange[i].numQuestsLbl, false);
                utilObj.setVisible(layersToChange[i].notifCircle, true);
            }
        }

        for(var i = 0; i < llpLayersToChange.length; i++) {
            utilObj.setVisible(llpLayersToChange[i].notifCircle, true);
            utilObj.setVisible(llpLayersToChange[i].notifCircle1, false);
            utilObj.setVisible(llpLayersToChange[i].notifCircle2, false);
            utilObj.setVisible(llpLayersToChange[i].numQuestsLbl1, false);
            utilObj.setVisible(llpLayersToChange[i].numQuestsLbl2, false);
            utilObj.setVisible(llpLayersToChange[i].newLabelNotifCircleIconQuests, true);
            utilObj.setVisible(llpLayersToChange[i].newLabelNumQuestsLbl, true);
        }
    };

    /**
     * Utility method to inform if a new tag needs to be shown - needs to be shown once everyday
     * @returns {boolean}
     */
    this.showTagOnQuestCenterBubble = function() {
        var lastOpened = questsObj.getQuestCenterLastOpenedTS();
        if(lastOpened == undefined || lastOpened == null || lastOpened == "") {
            return true;
        } else if(parseInt(lastOpened) > 0) {
            return (xpromoUtilObj.getCurrentDateInString() != xpromoUtilObj.getTimestampInDateString(parseInt(lastOpened)));
        }
        return false;
    };

    this.makeRelatedQuestsStatusStarted = function (questId) {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.updateRelatedQuestsStatus(questId, xpromoConstantsObj.QUEST_STARTED);
            return;
        }

        if(curQuests != undefined && curQuests[questId] != undefined) {
            var relatedQuests = curQuests[questId].r.split(":");
            var crossGameProgress = questsObj.getCrossGameProgressMap();
            for (var i=0; i<relatedQuests.length; i++) {
                var relatedQuest = relatedQuests[i];
                if (relatedQuest != "" && relatedQuest != undefined && (crossGameProgress[relatedQuest] == undefined || crossGameProgress[relatedQuest].status == xpromoConstantsObj.QUEST_NOT_STARTED)) {
                    questsObj.updateCrossGameQuestStatus(relatedQuest, xpromoConstantsObj.QUEST_STARTED);
                }
            }
        }
    };

    this.getOptimizedAdvertiserId = function () {
        if(optimizeAdIdForLimitAdTracking && optimizedAdIdDataSharing != null) {
            return optimizedAdIdDataSharing;
        }
        return xpromoUtilObj.getAdvertiserId();
    };

    this.allowCrossGameCalls = function () {
        if(optimizeAdIdForLimitAdTracking) {
            return (optimizedAdIdDataSharing != null || xpromoUtilObj.isSharedDataFetchedOnce());
        }
        return true;
    };

    /**
     * Method to extract daily quests and the first xpromo quest for view in the QC Auto pop UI
     * @returns {{dailyQuests: Array, xpromoQuests: Array, dailyQuestTimeLeft: Integer}}
     */
    this.getQuestsForPopupView = function() {
        var result = {
            "dailyQuests": [],
            "xpromoQuests": [],
        };
        var questList = this.getSortedQuestList();
        for (var questIndex in questList) {
            if(questIndex > 3) { // We have a limit for populating 4 quests
                break;
            }

            var questData = questList[questIndex];

            if(questData == undefined) {
                continue;
            }

            var gameIcon = "";

            var gameIconPath = "icon_" + questData.gh + ".png";
            gameIcon = xpromoUtilObj.getAbsFilePath(gameIconPath);
            if (!utilObj.isWeb() && !utilObj.fileExists(gameIconPath)) {
                gameIcon = questConfigObj.getConfigData("icon_game");
                xPromoGameSpecificObj.downloadFileV1(constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/icons/" + gameIconPath, gameIconPath);
            }

            if (xpromoConstantsObj.QUEST_ICON_GAME_MAP[constantsObj.GAME_ID] != undefined && xpromoConstantsObj.QUEST_ICON_GAME_MAP[constantsObj.GAME_ID][questData.qt] != undefined) {
                gameIcon = xpromoConstantsObj.QUEST_ICON_GAME_MAP[constantsObj.GAME_ID][questData.qt];
            } else if (xpromoConstantsObj.QUEST_ICON_MAP[questData.qt] != undefined) {
                gameIcon = xpromoConstantsObj.QUEST_ICON_MAP[questData.qt];
            }


            var rowInfo = {
                "id": questData.id, // Quest ID
                "icon": gameIcon, // Quest Icon
                "qt": questData.qt, // Quest Type
                "description": questData.d, // Description
                "reqProgress": questData.reqProgress, // requiredProgress
                "curProgress": questData.curProgress, // currentProgress
                "status": questData.status, // Status
                "gh": questData.gh, // Game Hosted
                "coinReward": questsObj.getRewardValueForQuest(questData), // Coin Reward
                "timeLimit": questData.timeLimit, // time left for quest to expire, 0 for non-timed quests
                "bucket": questData.b, // Quest Bucket
                "crossGame": questData.cg, // Whether its a cross-game quest
            };

            // We only populate the daily quest or streak quest for in-game quest list
            if(dailyQuestsObj.isDailyQuestOrStreakQuest(questData.qt)) {
                rowInfo["description"] = questData.d.replace("%value%", questData.reqProgress);
                result["dailyQuests"].push(rowInfo);
            } else if (questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME) { // and only install type quests for xpromo quests
                if(questData.isSocialQuest == 1 && questData.originalDesc != undefined) {
                    rowInfo["description"] = questData.originalDesc;
                }
                result["xpromoQuests"].push(rowInfo);
            }

        }
        result["dailyQuestTimeLeft"] = questsObj.getDailyQuestTimeLeftForPopupView();
        return result;
    };

    /**
     * Method to get the time remaining for the daily quest to complete for QC Auto Pop UI
     * @returns {number}
     */
    this.getDailyQuestTimeLeftForPopupView = function() {
        var timeLeft = 0;
        if(questCenterLayer.checkDailyQuestsLocked()) {
            var days = xpromoGameRuntimeObj.getDailyQuestsDaysNonPayer();
            if (xPromoGameSpecificObj.getIsPayerUser()) {
                days = xpromoGameRuntimeObj.getDailyQuestsDaysPayer();
            }

            var installTime = xPromoGameSpecificObj.getInstallTimeStamp();
            var dayCur = new Date(installTime * 1000);
            var dqLockLimit = constantsObj.SECONDS_IN_DAY - (dayCur.getHours() * 60 * 60 + dayCur.getMinutes() * 60 + dayCur.getSeconds()) + (days - 1) * constantsObj.SECONDS_IN_DAY;
            timeLeft = parseInt(installTime) + parseInt(dqLockLimit) - utilObj.getCurrentTimestamp();
        } else {
            timeLeft = xpromoUtilObj.getTimeLeftForDayComplete();
        }
        return timeLeft;
    };

    /**
     * Handler method for clicks on quests outside of quest center
     * currently being called from QC Auto Pop UI
     * @param params - Quest ID and quest related information
     */
    this.handleQuestActionOnPopupView = function(params) {
        if(params == undefined || params.id == undefined) {
            return;
        }

        var questList = this.getSortedQuestList();
        for (var questIndex in questList) {
            var questData = questList[questIndex];
            if(questData != undefined && questData.id == params.id) {
                var actionParams = {
                    "status": questData.status,
                    "qt": questData.qt,
                    "questId": questData.id,
                    "firstParam": null,
                    "itunesId": null,
                    "isCrossGame": (questData.cg == 1) ? true : false,
                    "reward": questsObj.getRewardValueForQuest(questData),
                    "showReward": questCenterLayer.shouldShowReward(questData.qt, questData.id),
                    "isRequest": questData.isRequest,
                    "isSocialQuest": questData.isSocialQuest,
                    "trackingId": questData.trackingId,
                    "gameHosted": questData.gh,
                    "engagementType": questData.engagementType,
                    "curProgress": questData.curProgress,
                    "isLoopbackEngagement": questData.isLoopbackEngagement,
                    "bucket": questData.b,
                    "loopIndex": questIndex,
                    "popupView": true,
                };

                if(dailyQuestsObj.isDailyQuestOrStreakQuest(questData.qt)) {
                    actionParams["description"] = questData.d.replace("%value%", questData.reqProgress);
                } else if (questData.qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME) { // and only install type quests for xpromo quests
                    if(questData.isSocialQuest == 1 && questData.originalDesc != undefined) {
                        actionParams["description"] = questData.originalDesc;
                    }
                }


                if (questData.params != undefined && questData.params != null) {
                    questCenterLayer.paramsMap["firstParam"] = questData.params[0];
                    if (questData.params[1] != null && questData.params[1] != undefined) {
                        questCenterLayer.paramsMap["itunesId"] = questData.params[1];
                    }
                }

                questCenterLayer.questAction(null, actionParams);

                break;
            }
        }
    };

    this.trackAdIdAssignment = function () {
        trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO_ADVID, "", questsObj.getOptimizedAdvertiserId(), "", "", "receiver", "", "", "");
    }
};
