package in.playsimple.common;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;

import com.adjust.sdk.Adjust;
import com.adjust.sdk.AdjustEvent;
import com.facebook.appevents.AppEventsLogger;
import com.google.firebase.analytics.FirebaseAnalytics;
import com.google.firebase.crashlytics.FirebaseCrashlytics;
import com.google.firebase.remoteconfig.FirebaseRemoteConfig;
import com.google.firebase.remoteconfig.FirebaseRemoteConfigSettings;
import com.tapjoy.Tapjoy;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.HashMap;
import java.util.Map;

import in.playsimple.Constants;

public class Analytics {
    private static FirebaseCrashlytics crashlytics = null;
    private static FirebaseAnalytics mFirebaseAnalytics;
    private static FirebaseRemoteConfig mFirebaseRemoteConfig;
    private static Activity activity = null;

    public static void init(Activity act) {
        try {
            mFirebaseAnalytics = FirebaseAnalytics.getInstance(act);
            crashlytics = FirebaseCrashlytics.getInstance();
            Analytics.initializeRemoteConfig();
        } catch (Exception e) {

        }
        activity = act;
    }

    public static void init(Context context) {
        try {
            mFirebaseAnalytics = FirebaseAnalytics.getInstance(context);
            crashlytics = FirebaseCrashlytics.getInstance();
            Analytics.initializeRemoteConfig();
        } catch (Exception e) {

        }
    }

    public static void logException(Exception e) {
        try {
            FirebaseCrashlytics.getInstance().recordException(e);
        } catch (Exception e1) {

        }
    }

    public static void setAppOpen() {
        mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.APP_OPEN, new Bundle());
    }

    public static void setUserId(String userId) {
        crashlytics.setUserId(userId);
        crashlytics.setCustomKey(PSConstants.TRACK_RID, userId);
    }

    public static void setDeviceInfo(String cpuArch, String deviceWidth, String deviceHeight, String deviceModel, String deviceId, String country, String isOnline) {
        crashlytics.setCustomKey(PSConstants.TRACK_CPU_ARCH, cpuArch);
        crashlytics.setCustomKey(PSConstants.TRACK_DEVICE_WIDTH, deviceWidth + "");
        crashlytics.setCustomKey(PSConstants.TRACK_DEVICE_HEIGHT, deviceHeight + "");
        crashlytics.setCustomKey(PSConstants.TRACK_DEVICE_MODEL, deviceModel);
        // crashlytics.setCustomKey(PSConstants.TRACK_DEVICE_ID, deviceId);
        crashlytics.setCustomKey(PSConstants.TRACK_COUNTRY, country);
        crashlytics.setCustomKey(PSConstants.TRACK_ONLINE, isOnline + "");
    }

    public static void screenView(String screenName) {
        crashlytics.setCustomKey(PSConstants.TRACK_SCREEN, screenName);

        Bundle b = new Bundle();
        b.putString(FirebaseAnalytics.Param.ITEM_NAME, screenName);
        mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.VIEW_ITEM, b);
    }

    public static void purchase(boolean success, float price, String currency, String itemName, String itemType, String itemId, String orderId, String data, String signature) {
        if (success) {
            Tapjoy.trackPurchase(itemId, currency, price, null);

            AdjustEvent event = new AdjustEvent(Constants.ADJUST_PURCHASE_TOKEN);
            event.setRevenue(price, currency);
            Adjust.trackEvent(event);

            Bundle b = new Bundle();
            b.putDouble(FirebaseAnalytics.Param.VALUE, price);
            b.putString(FirebaseAnalytics.Param.CURRENCY, currency);
            b.putString(FirebaseAnalytics.Param.TRANSACTION_ID, orderId);

            mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.ECOMMERCE_PURCHASE, b);

            // Facebook
            AppEventsLogger logger = AppEventsLogger.newLogger(activity);
            logger.logPurchase(BigDecimal.valueOf(price), Currency.getInstance(currency));

        }
    }

    public static void purchaseBegin(float price, String currency, String itemName, String itemType, String itemId) {
        Bundle b = new Bundle();
        b.putDouble(FirebaseAnalytics.Param.VALUE, price);
        b.putString(FirebaseAnalytics.Param.CURRENCY, currency);

        mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.BEGIN_CHECKOUT, b);
    }

    public static void updateVirtualCurrency(String item, int val, String trackClass, String order) {
        // Log.i(Constants.TAG, "Updating virtual currency:" + item + ";" + val + ";" + trackClass + ";" + order);
    }

    public static void share(String method, String name, String type, String id) {
        Bundle b = new Bundle();
        b.putString(FirebaseAnalytics.Param.METHOD, method);
        b.putString(FirebaseAnalytics.Param.ITEM_NAME, name);
        b.putString(FirebaseAnalytics.Param.CONTENT_TYPE, type);
        b.putString(FirebaseAnalytics.Param.ITEM_ID, id);
        mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.SHARE, b);
    }

    public static void rating(int rating, String name) {
        Bundle b = new Bundle();
        b.putInt("rate", rating);
        b.putString(FirebaseAnalytics.Param.ITEM_NAME, name);
        mFirebaseAnalytics.logEvent("rating", b);
    }

    public static void signUp(String method, String rid, boolean success) {
        Bundle b = new Bundle();
        b.putString(FirebaseAnalytics.Param.METHOD, method);
        b.putString("rid", rid);
        b.putBoolean("success", success);
        mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.SIGN_UP, b);
    }

    public static void login(String method, String screen, boolean success) {
        Bundle b = new Bundle();
        b.putString("screen", screen);
        b.putBoolean("success", success);
        mFirebaseAnalytics.logEvent(FirebaseAnalytics.Event.LOGIN, b);
    }

    public static void invite(String method, String screen, int count) {
        Bundle b = new Bundle();
        b.putString(FirebaseAnalytics.Param.METHOD, method);
        b.putString("screen", screen);
        b.putInt("count", count);
        mFirebaseAnalytics.logEvent("invite", b);
    }

    public static void levelStart(String level, String bucket, String puzId) {
        // Log.i(Constants.TAG, "Level start:" + level + ";" + bucket + ";" + puzId);
        crashlytics.setCustomKey(PSConstants.TRACK_LEVEL_START, level);

        Bundle b = new Bundle();
        b.putString("level", level);
        mFirebaseAnalytics.logEvent(PSConstants.TRACK_LEVEL_START, b);
    }

    public static void levelEnd(String level, String bucket, String puzId, int score, boolean success) {
        // Log.i(Constants.TAG, "Level end:" + level + ";" + bucket + ";" + puzId);
        crashlytics.setCustomKey(PSConstants.TRACK_LEVEL_END, level);

        Bundle b = new Bundle();
        b.putString("level", level);
        b.putInt("score", score);
        b.putBoolean("success", success);
        mFirebaseAnalytics.logEvent(PSConstants.TRACK_LEVEL_END, b);
    }

    public static void customEvent(String refId, String k, String p, String c, String o, String f, String g, String s, int co, String oppId) {
        String crumb = refId + ";" + k + ";" + p + ";"
                + c + ";" + o + ";" + f + ";" + g + ";" + s + ";" + co + ";" + oppId;
        sendReport(crumb);
    }

    public static void sendReport(String crumb) {
        crashlytics.log(crumb);
    }

    public static void install(String k, String utm_source, String utm_medium, String utm_term,
                               String utm_content, String utm_campaign) {
        if (utm_source == null) {
            return;
        }
        if (utm_medium == null) {
            utm_medium = "";
        }
        if (utm_term == null) {
            utm_term = "";
        }
        if (utm_content == null) {
            utm_content = "";
        }
        if (utm_campaign == null) {
            utm_campaign = "";
        }

        try {
            if (mFirebaseAnalytics == null) {
                // Possibly fixing https://console.firebase.google.com/project/project-702790064579389011/monitoring/app/android:com.littleengine.junglematch/cluster/3503d01a
                Log.i(Constants.TAG, "Null objects while tracking installs");
                return;
            }

            Bundle b = new Bundle();
            b.putString("source", utm_source);
            b.putString("medium", utm_medium);
            b.putString("term", utm_term);
            b.putString("content", utm_content);
            b.putString("campaign", utm_campaign);
            mFirebaseAnalytics.logEvent(k, b);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void userSyncStart(boolean hasRefId) {
        String custom = "SyncStartUser";
        Bundle b = new Bundle();
        b.putBoolean("hasRefId", hasRefId);
        mFirebaseAnalytics.logEvent(custom, b);
    }

    public static void userSyncEnd(boolean hasRefId, String action, boolean success) {
        String custom = "SyncEndUser";

        Bundle b = new Bundle();
        b.putBoolean("hasRefId", hasRefId);
        b.putString("action", action);
        b.putBoolean("success", success);
        mFirebaseAnalytics.logEvent(custom, b);
    }

    public static void refIdMissing(int level) {
        String refIdMissing = "RefIdMissing";
        boolean online = PSUtil.isOnline();

        Bundle b = new Bundle();
        b.putInt("level", level);
        b.putBoolean("online", online);
        mFirebaseAnalytics.logEvent(refIdMissing, b);
    }

    public static void installEvent(String refId , String syncId) {
        SharedPreferences pref = activity.getSharedPreferences(PSConstants.PREFS_NAME, 0);
        String installPref = PSConstants.TRACK_INSTALL + "_" + syncId;
        String trackInfo = "install";
        Map<String, ?> spMap = pref.getAll();
        for (Map.Entry<String, ?> entry : spMap.entrySet()) {
            if (entry.getKey().contains(Constants.ADJUST_CUSTOM_INSTALL_TOKEN)) {
                trackInfo = "logout";
                break;
            }
        }
        if (pref != null && !pref.getBoolean(installPref, false)) {
            AdjustEvent event = new AdjustEvent(Constants.ADJUST_CUSTOM_INSTALL_TOKEN);
            event.addPartnerParameter("refid", refId);
            event.addPartnerParameter("gid", Constants.GAME_ID + "");
            event.addPartnerParameter("cli", Constants.CLIENT + "");
            event.addPartnerParameter("b", PSUtil.getVersionCode() + "");
            event.addPartnerParameter("evty", trackInfo + "");
            event.addPartnerParameter("syncid", syncId);
            Adjust.trackEvent(event);
            Log.i(Constants.TAG, "firing custom install event with refid::" + refId);
            SharedPreferences.Editor editor = pref.edit();
            editor.putBoolean(installPref, true);
            editor.apply();
        }
    }

    public static void incentLevelComplete(String refId, String duration) {
        triggerAdjustCustomEvent(refId, Constants.ADJUST_INCENT_LEVEL_REACHED, duration);
    }

    public static void triggerAdjustCustomEvent(String refId, String eventName, String duration) {
        SharedPreferences pref = activity.getSharedPreferences(PSConstants.PREFS_NAME, 0);
        String incentPref = eventName + "_" + refId;
        if (pref != null && !pref.getBoolean(incentPref, false)) {
            triggerAdjustEvent(refId, eventName, duration);
            SharedPreferences.Editor editor = pref.edit();
            editor.putBoolean(incentPref, true);
            editor.apply();
        }
    }

    public static void triggerAdjustEvent(String refId, String eventName, String duration) {
        AdjustEvent event = new AdjustEvent(eventName);
        event.addPartnerParameter("refid", refId);
        event.addPartnerParameter("gid", Constants.GAME_ID + "");
        event.addPartnerParameter("cli", Constants.CLIENT + "");
        event.addPartnerParameter("b", PSUtil.getVersionCode() + "");
        event.addCallbackParameter("ts", duration);
        event.addPartnerParameter("ts", duration);
        Adjust.trackEvent(event);
        Log.i(Constants.TAG, "firing adjust event with refid::" + refId);
    }

    public static void syncFabricEvent(String eventName, String status) {

    }

    public static void crashEvent(String name, String reason, String version) {
        Track.trackCounter(PSConstants.TRACK_CRASH, name, reason, version, "", "", "", "1", "");
    }

    public static void alarmErrorEvent(String alarmType) {
        Bundle b = new Bundle();
        b.putString("type", alarmType);
        mFirebaseAnalytics.logEvent(PSConstants.TRACK_ALARM +"_" + PSConstants.TRACK_ERROR, b);
    }

    public static void alarmEvent(String alarmType) {
        Bundle b = new Bundle();
        b.putString("type", alarmType);
        mFirebaseAnalytics.logEvent(PSConstants.TRACK_ALARM, b);
    }

    public static void alarmEventFor(String trackDay, String alarmType) {
        Bundle b = new Bundle();
        b.putString("day", trackDay);
        b.putString("type", alarmType);
        mFirebaseAnalytics.logEvent(PSConstants.TRACK_ALARM, b);
    }

    public static void notifSentEventFor(String trackDay, String alarmType, String notifName, String textOrImage) {
        Bundle b = new Bundle();
        b.putString("day", trackDay);
        b.putString("type", alarmType);
        b.putString("name", notifName);
        b.putString("graphic", textOrImage);
        mFirebaseAnalytics.logEvent(PSConstants.TRACK_ALARM + "_sent_", b);
    }

    public static void notifNotSentEventFor(String trackDay, String alarmType, String reason) {
        Bundle b = new Bundle();
        b.putString("day", trackDay);
        b.putString("type", alarmType);
        b.putString("reason", reason);
        mFirebaseAnalytics.logEvent(PSConstants.TRACK_ALARM + "_not_sent_", b);
    }

    public static void bannerEventFor(final String status) {
        Bundle b = new Bundle();
        b.putString("status", status);
        mFirebaseAnalytics.logEvent("BannerAds", b);
    }

    public static void interstitialEventFor(final String status) {
        Bundle b = new Bundle();
        b.putString("status", status);
        mFirebaseAnalytics.logEvent("interstitialAds", b);
    }

    public static void videoEventFor(final String status) {
        Bundle b = new Bundle();
        b.putString("status", status);
        mFirebaseAnalytics.logEvent("videoAds", b);
    }

    public static void offerwallEventFor(final String status, final String offerwallType) {
        Bundle b = new Bundle();
        b.putString("status", status);
        b.putString("offerwallType", offerwallType);
        mFirebaseAnalytics.logEvent("offerwallAds", b);
    }

    public static void sendFirebaseErrorEvents(String eventName, String spriteName) {
        Bundle b = new Bundle();
        b.putString("SpriteName", spriteName);
        mFirebaseAnalytics.logEvent(eventName, b);
    }

    public static void sendFirebasePositionEvents(String eventName, String screenName) {
        Bundle b = new Bundle();
        b.putString("ScreenName", screenName);
        mFirebaseAnalytics.logEvent(eventName, b);
    }

    public static void sendFirebaseScreenErrorEvent(String screen) {
        Bundle b = new Bundle();
        b.putString("Updating", screen);
        mFirebaseAnalytics.logEvent("ScreenUpdateError", b);
    }

    /**
     * sends custom FireBase events with the given event Name and RefId
     *
     * @param eventName
     * @param refId
     */
    public static void sendFirebaseCustomEvent(String eventName, String refId) {
        // Log.i(Constants.TAG, "triggeringEventsLog: FireBase Events Log: Firing FireBase Event with refId -> " + refId + " eventName -> " + eventName);
        Bundle b = new Bundle();
        b.putString("refid", refId);
        b.putInt("gid", Constants.GAME_ID);
        b.putInt("cli", Constants.CLIENT);
        b.putInt("b", PSUtil.getVersionCode());
        mFirebaseAnalytics.logEvent(eventName, b);
    }

    public static void triggerAdsAdjustEvents(String eventName) {
        AdjustEvent event = new AdjustEvent(eventName);
        event.addPartnerParameter("gid", Constants.GAME_ID + "");
        event.addPartnerParameter("cli", Constants.CLIENT + "");
        event.addPartnerParameter("b", PSUtil.getVersionCode() + "");
        Adjust.trackEvent(event);
    }

    public static void setUserProperty(String property, String value) {
        if(mFirebaseAnalytics == null) {
            Log.i(Constants.TAG, "setUserProperty: firebase analytics not initialized");
            return;
        }
        mFirebaseAnalytics.setUserProperty(property, value);
    }

    private static void initializeRemoteConfig() {
        if(mFirebaseRemoteConfig == null) {
            mFirebaseRemoteConfig = FirebaseRemoteConfig.getInstance();
            FirebaseRemoteConfigSettings configSettings = new FirebaseRemoteConfigSettings.Builder()
                    .setMinimumFetchIntervalInSeconds(Constants.HOUR_IN_SECONDS)
                    .build();
            mFirebaseRemoteConfig.setConfigSettingsAsync(configSettings);
            mFirebaseRemoteConfig.setDefaultsAsync(new HashMap<>());
        }
    }

    public static void fetchAndActivateRemoteConfig() {
        if(mFirebaseRemoteConfig == null) {
            Log.i(Constants.TAG, "REMOTE CONFIG fetchAndActivateRemoteConfig: remote config not initialized");
            return;
        }
        mFirebaseRemoteConfig.fetchAndActivate();
    }

    public static String getRemoteConfigValue(String key) {
        if(mFirebaseRemoteConfig == null) {
            Log.i(Constants.TAG, "REMOTE CONFIG fetchConfig: remote config not initialized");
            return "";
        }

        return mFirebaseRemoteConfig.getString(key);
    }
}