package in.playsimple.common;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;

import com.facebook.AccessToken;
import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.FacebookSdk;
import com.facebook.GraphRequest;
import com.facebook.GraphResponse;
import com.facebook.HttpMethod;
import com.facebook.login.LoginManager;
import com.facebook.login.LoginResult;
import com.facebook.share.Sharer;
import com.facebook.share.model.GameRequestContent;
import com.facebook.share.model.ShareContent;
import com.facebook.share.model.ShareLinkContent;
import com.facebook.share.model.ShareMediaContent;
import com.facebook.share.model.SharePhoto;
import com.facebook.share.model.ShareVideo;
import com.facebook.share.widget.GameRequestDialog;
import com.facebook.share.widget.ShareDialog;
import com.google.firebase.crashlytics.FirebaseCrashlytics;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import in.playsimple.Constants;
import in.playsimple.GameSpecific;

public class FacebookWrapper {
    final static String TAG = Constants.TAG + "/FB";
    final static String PUBLISH_ACTIONS = "publish_actions";
    private static Activity activity = null;
    private static final String CALLBACK_FUNCTION = "fbWrapperObj.jsbCallBack";
    private ShareDialog shareDialog;
    private static LoginResult loginResult = null;

    enum LOGIN_RESULT {
        SUCCESS(0),
        CANCEL(1),
        ERROR(2);

        private int value;

        private LOGIN_RESULT(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public final static String ACCESS_TOKEN = "accessToken";
    public final static String GRANTED_PERMISSIONS = "permissions";
    public final static String SUCCESS = "success";
    public final static String API_ID = "apiId";

    public final static String TYPE = "type";
    public final static String TYPE_LOGIN = "login";
    public final static String TYPE_API = "api";
    public final static String TYPE_LOGOUT = "logout";
    public final static String TYPE_LINK_SHARE = "linkShare";
    public final static String TYPE_APP_REQUEST = "appRequest";

    public final static String KEY_POST_ID = "postId";
    public final static String KEY_ERROR = "error";
    public final static String KEY_REQ_ID = "request";
    public final static String KEY_TO = "to";


    private static final int REQ_FRIENDS = 1;
    private static final int REQ_INVITABLE_FRIENDS = 2;
    private static final int REQ_ALL_OPEN = 3;
    private static final int REQ_DELETE_OPEN = 4;
    private static final int REQ_PHOTO_UPLOAD = 5;
    private static final int REQ_LEVELUP_COGS = 6;
    private static final int REQ_SPINNER_COGS = 7;
    private static final int USER_INFO = 8;

    private static CallbackManager callbackManager;
    public GameRequestDialog grd;
    public LoginManager loginManager;

    public static void setActivity(Activity a) {
        activity = a;
    }

    public GameRequestDialog getGrd() {
        return grd;
    }

    public static CallbackManager getCallbackManager() {
        return callbackManager;
    }

    static FacebookWrapper fbWrapperObj;

    private FacebookWrapper() {
        initFbSDK();
    }

    public static FacebookWrapper get() {
        if (activity == null) {
            Log.e(Constants.TAG, "Please set activity b4 init");
            return null;
        }

        if (fbWrapperObj == null) {
            fbWrapperObj = new FacebookWrapper();
        }
        return fbWrapperObj;
    }

    public final static int FBERROR = -2;


    public void initFbSDK() {
        try {
            Log.i(Constants.TAG, "Setting activity in FB sdk:" + activity);
            FacebookWrapper.setActivity(activity);
            FacebookSdk.sdkInitialize(activity);
            callbackManager = CallbackManager.Factory.create();

            grd = new GameRequestDialog(activity);
            grd.registerCallback(callbackManager,
                    new FacebookCallback<GameRequestDialog.Result>() {
                        public void onSuccess(GameRequestDialog.Result result) {
                            JSONObject jObj = new JSONObject();
                            boolean noFriends = false;
                            try {
                                String id = result.getRequestId();
                                if (id == null) {
                                    return;
                                }
                                Log.i(Constants.TAG + "/GRD",
                                        result.getRequestRecipients() + "");
                                Log.i(Constants.TAG + "/GRD",
                                        id + "");
                                jObj.put(TYPE, TYPE_APP_REQUEST);
                                jObj.put(KEY_REQ_ID, id);
                                jObj.put(SUCCESS, LOGIN_RESULT.SUCCESS.getValue());
                                List<String> arr = result
                                        .getRequestRecipients();
                                StringBuilder sb = new StringBuilder();
                                for (String to : arr) {
                                    sb.append(to);
                                    sb.append(",");
                                }
                                String toStr = sb.toString();
                                int toLen = toStr.length();
                                if (toLen <= 0) {
                                    noFriends = true;
                                } else {
                                    toStr = toStr.substring(0, toLen - 1);
                                    jObj.put(KEY_TO, toStr);
                                    Analytics.invite(PSConstants.TRACK_REQUEST, PSConstants.TRACK_ANY, toLen);
                                }
                            } catch (JSONException e) {
                                Analytics.logException(e);
                            }
                            if (noFriends) {
                                PSUtil.showMessage(PSStrings.noFriends);
                            } else {
                                GameSpecific.sendCallBack(CALLBACK_FUNCTION, jObj.toString());
                            }
                        }

                        public void onCancel() {
                            Log.e(Constants.TAG, "error cancel");

                            JSONObject jObj = new JSONObject();
                            try {
                                jObj.put(TYPE, TYPE_APP_REQUEST);
                                jObj.put(KEY_REQ_ID, -2);
                                jObj.put(SUCCESS, LOGIN_RESULT.CANCEL.getValue());
                            } catch (JSONException e) {
                                Analytics.logException(e);
                            }
                            GameSpecific.sendCallBack(CALLBACK_FUNCTION, jObj.toString());
                        }

                        public void onError(FacebookException error) {
                            JSONObject jObj = new JSONObject();
                            try {
                                jObj.put(TYPE, TYPE_APP_REQUEST);
                                jObj.put(KEY_REQ_ID, -2);
                                jObj.put(SUCCESS, LOGIN_RESULT.ERROR.getValue());
                                jObj.put(KEY_ERROR, error.getLocalizedMessage());
                            } catch (JSONException e) {
                                Analytics.logException(e);
                            }
                            GameSpecific.sendCallBack(CALLBACK_FUNCTION, jObj.toString());
                        }
                    }
            );

            shareDialog = new ShareDialog(activity);
            shareDialog.registerCallback(callbackManager, new FacebookCallback<Sharer.Result>() {
                @Override
                public void onSuccess(Sharer.Result result) {
                    Log.d(FacebookWrapper.TAG, "share link success");
                    JSONObject jObj = new JSONObject();
                    try {
                        jObj.put(FacebookWrapper.TYPE, FacebookWrapper.TYPE_LINK_SHARE);
                        jObj.put(FacebookWrapper.KEY_POST_ID, result.getPostId());
                        jObj.put(FacebookWrapper.SUCCESS, LOGIN_RESULT.SUCCESS.getValue());
                    } catch (Exception e) {
                        Analytics.logException(e);
                    }
                    PSUtil.showMessage("Post Shared Successfully!");
                    GameSpecific.sendCallBack(CALLBACK_FUNCTION, jObj.toString());
                }

                @Override
                public void onCancel() {
                    Log.d(FacebookWrapper.TAG, "share link cancel");
                    JSONObject jObj = new JSONObject();
                    try {
                        jObj.put(FacebookWrapper.TYPE, FacebookWrapper.TYPE_LINK_SHARE);
                        jObj.put(FacebookWrapper.SUCCESS, LOGIN_RESULT.CANCEL.getValue());
                    } catch (Exception e) {
                        Analytics.logException(e);
                    }
                    GameSpecific.sendCallBack(CALLBACK_FUNCTION, jObj.toString());
                }

                @Override
                public void onError(FacebookException error) {
                    Log.d(FacebookWrapper.TAG, "share link error");
                    JSONObject jObj = new JSONObject();
                    try {
                        jObj.put(FacebookWrapper.TYPE, FacebookWrapper.TYPE_LINK_SHARE);
                        jObj.put(FacebookWrapper.KEY_ERROR, error.getLocalizedMessage());
                        jObj.put(FacebookWrapper.SUCCESS, LOGIN_RESULT.ERROR.getValue());
                    } catch (Exception e) {
                        Analytics.logException(e);
                    }
                    GameSpecific.sendCallBack(CALLBACK_FUNCTION, jObj.toString());
                }
            });

            loginManager = LoginManager.getInstance();
            loginManager.registerCallback(callbackManager,
                    new FacebookCallback<LoginResult>() {
                        @Override
                        public void onSuccess(
                                LoginResult loginResult) {
                            Log.d(FacebookWrapper.TAG, "login success");
                            FacebookWrapper.loginResult = loginResult;
                            FacebookWrapper.getUserInformation();
                        }

                        @Override
                        public void onCancel() {
                            Log.d(FacebookWrapper.TAG, "login cancel");
                            FacebookWrapper.sendLoginCallBack(
                                    LOGIN_RESULT.CANCEL, null);
                        }

                        @Override
                        public void onError(
                                FacebookException error) {
                            Log.d(FacebookWrapper.TAG, "login error");
                            FacebookWrapper.sendLoginCallBack(
                                    LOGIN_RESULT.ERROR, null);
                        }
                    });

        } catch (Exception e) {
            Analytics.logException(e);
        }
    }

    public static void appRequests(String info) throws JSONException {
        if (fbWrapperObj == null) {
            fbWrapperObj = FacebookWrapper.get();
        }
        JSONObject obj = new JSONObject(info);
        GameRequestContent.Builder builder = new GameRequestContent.Builder();
        Analytics.sendReport("Sending FB Game Requests");
        if (obj.has("message")) {
            builder.setMessage(obj.getString("message"));
        }
        if (obj.has("to")) {
            String to_str = obj.getString("to");
            builder.setTo(to_str);
            Log.d(Constants.TAG, "to = " + to_str.split(",").length + "," + to_str);
        }

        if (obj.has("title")) {
            builder.setTitle(obj.getString("title"));
        }

//		Log.i(Constants.TAG+"/FB", "sendign app requests to "+ obj.getString("to")+":"+obj.getString("title")+":"+obj.getString("message"));
        fbWrapperObj.grd.show(builder.build());
    }


    public static void login(String commaPermissions) {
        Log.d(FacebookWrapper.TAG, "login attempt");
        if (fbWrapperObj == null) {
            fbWrapperObj = FacebookWrapper.get();
        }
        Analytics.sendReport("Attempting FB Login");
        String[] permissionList = commaPermissions.split(",");
        if (commaPermissions.contains(PUBLISH_ACTIONS)) {
            fbWrapperObj.loginManager.logInWithPublishPermissions(activity, Arrays.asList(
                    permissionList));
        } else {
            fbWrapperObj.loginManager.logInWithReadPermissions(activity, Arrays.asList(
                    permissionList));
        }
    }

    public static void sendLoginCallBack(LOGIN_RESULT code, JSONObject data) {
        JSONObject jobj = new JSONObject();
        try {
            jobj.put(FacebookWrapper.TYPE, TYPE_LOGIN);
            Analytics.sendReport("FBLogin Call back" + code);
            if (code == LOGIN_RESULT.SUCCESS && loginResult != null) {
                jobj.put(FacebookWrapper.SUCCESS, 0);
                jobj.put(FacebookWrapper.ACCESS_TOKEN, loginResult.getAccessToken().getToken());
                Set<String> grantedPermissions = loginResult
                        .getRecentlyGrantedPermissions();
                String permissions = "";
                for (String perm : grantedPermissions) {
                    permissions += perm + ",";
                }
                if (permissions.length() > 0) {
                    permissions = permissions.substring(0, permissions.length() - 1);
                }
                jobj.put(FacebookWrapper.GRANTED_PERMISSIONS, permissions);

                if (data != null) {
                    if (data.has("id")) {
                        jobj.put("id", data.get("id"));
                    }
                    if (data.has("name")) {
                        jobj.put("name", data.get("name"));
                    }
                    if (data.has("gender")) {
                        jobj.put("gender", data.get("gender"));
                    }
                    if (data.has("email")) {
                        jobj.put("email", data.get("email"));
                    }
                    if (data.has("token_for_business")) {
                        jobj.put("token_for_business", data.get("token_for_business"));
                    }
                }

            } else {
                jobj.put(FacebookWrapper.SUCCESS, -1);
            }
        } catch (JSONException e) {
            Analytics.logException(e);
        }
        String msg = jobj.toString();
        GameSpecific.sendCallBack(CALLBACK_FUNCTION, msg);
    }

    public static String getAccessToken() {
        String accessToken = "";
        AccessToken currentAccessToken = AccessToken.getCurrentAccessToken();
        if (FacebookWrapper.isFacebookLoggedIn()) {
            if (!currentAccessToken.isExpired()) {
                accessToken = currentAccessToken.getToken();
            } else {
                AccessToken.refreshCurrentAccessTokenAsync(
                        new AccessToken.AccessTokenRefreshCallback() {
                            @Override
                            public void OnTokenRefreshed(AccessToken accessToken) {
                                Log.d(Constants.TAG, "access token refreshed");
                            }

                            @Override
                            public void OnTokenRefreshFailed(FacebookException exception) {
                                Log.d(Constants.TAG, "access token failed to refresh");
                            }
                        });
            }
        } else {
//            TODO user not logged in
        }
        Log.d(FacebookWrapper.TAG, "returning access token = " + accessToken);
        return accessToken;
    }

    public static boolean isFacebookLoggedIn() {
        boolean loggedIn = false;
        try {
            loggedIn = AccessToken.getCurrentAccessToken() != null;
        } catch (Exception e) {
            Analytics.logException(e);
        }
        return loggedIn;
    }


    public static void getPlayingFriends() {
        Bundle params = new Bundle();
        params.putString("fields", "name,picture.width(200).height(200)");
        params.putInt("limit", 200);
        onApi("/me/friends", "GET", params.toString(), REQ_FRIENDS);
    }

    public static void getUserInformation() {
//        Bundle params = new Bundle();
        JSONObject params=new JSONObject();
        try {
            params.put("fields", "id,name,email,gender,token_for_business");
        } catch (JSONException e) {
            e.printStackTrace();
        }
//        params.putString("fields", "name,email,gender,token_for_business");
        onApi("/me", "GET", params.toString(), USER_INFO);
    }


    public static void getInvitableFriends() {
        return;/*
        Bundle params = new Bundle();
        params.putString("fields", "name,picture.width(200).height(200)");
        params.putInt("limit", 5000);
        onApi("/me/invitable_friends", "GET", params.toString(),
                REQ_INVITABLE_FRIENDS);*/
    }


    public static void onApi(String graphPath, String mode, String params, final int apiId) {
        Analytics.sendReport("Getting FB Request " + graphPath);
        AccessToken accessToken = AccessToken.getCurrentAccessToken();
        final String fileName = "friendsData" + apiId + ".json";
        if (accessToken == null) {
            Log.i(Constants.TAG, "DEBUG:: Not making api call as access token is null");
            // Analytics.syncFabricEvent(Constants.TRACK_FB_API, Constants.TRACK_FAIL);
            return;
        }

        HttpMethod httpMode = HttpMethod.GET;
        if (mode != null) {
            switch (mode) {
                case "GET":
                    httpMode = HttpMethod.GET;
                    break;
                case "POST":
                    httpMode = HttpMethod.POST;
                    break;
                case "DELETE":
                    httpMode = HttpMethod.DELETE;
                    break;
            }
        }

        Bundle paramsBundle = new Bundle();
        JSONObject jObj = null;
        try {
            jObj = new JSONObject(params);
        } catch (JSONException e) {
            Log.e(Constants.TAG, "Json exception while parsing in onApi:" + params);
            // Analytics.logException(e);
            // Analytics.syncFabricEvent(Constants.TRACK_FB_API, Constants.TRACK_FAIL);
            return;
        }
        if (jObj != null) {
            Iterator<String> iter = jObj.keys();
            while (iter.hasNext()) {
                String key = iter.next();
                try {
                    paramsBundle.putString(key, jObj.getString(key));
                } catch (JSONException e) {
                    Analytics.logException(e);
                }
            }
        }
        String trackPath = graphPath;
        if (!graphPath.contains("/me")) {
            trackPath = "friend_pic";
        }
        new GraphRequest(
                accessToken,
                graphPath,
                paramsBundle,
                httpMode,
                new GraphRequest.Callback() {
                    public void onCompleted(GraphResponse resp) {
                        final GraphResponse response = resp;
                        activity.runOnUiThread(new Runnable() {
                            @Override
                            public void run() {

//                                Log.d(FacebookWrapper.TAG, response.toString());
                                JSONObject resp = response.getJSONObject();
                                if (resp == null) {
                                    if (apiId == USER_INFO) {
                                        FacebookWrapper.sendLoginCallBack(LOGIN_RESULT.ERROR, null);
                                    }
                                    return;
                                }
                                try {
                                    resp.put(FacebookWrapper.TYPE, FacebookWrapper.TYPE_API);
                                    resp.put(FacebookWrapper.API_ID, apiId);
                                    switch (apiId) {
                                        case REQ_FRIENDS:
                                            Analytics.sendReport("FB Response - Playing Friends");
                                            Friends.updatePlayingFriends(resp);
                                            break;
                                        case REQ_INVITABLE_FRIENDS:
                                            Analytics.sendReport("FB Response - Invitable Friends");
                                            Friends.updateInvitableFriends(resp);
                                            break;
                                        case REQ_ALL_OPEN:
                                            Analytics.sendReport("FB Response - Open Requests");
                                            break;
                                        case REQ_DELETE_OPEN:
                                            Analytics.sendReport("FB Response - Delete Requests");
                                            break;
                                        case USER_INFO:
                                            if (resp.has("error")) {
                                                FacebookWrapper.sendLoginCallBack(LOGIN_RESULT.ERROR, null);
                                            } else {
                                                FacebookWrapper.sendLoginCallBack(LOGIN_RESULT.SUCCESS, resp);
                                            }
                                            return;
                                    }
                                } catch (Exception e) {
                                    Analytics.logException(e);
                                }

                                String msg = "";
                                try {
                                    GameSpecific.saveToLocalStorage(fileName, resp.toString());
                                    JSONObject json = new JSONObject();
                                    json.put("type", "api");
                                    json.put("data", fileName);
                                    json.put("apiId", apiId);
                                    msg = json.toString();
                                } catch (Exception e) {
                                    Analytics.logException(e);
                                }

                                GameSpecific.sendCallBack(CALLBACK_FUNCTION, msg);
                            }
                        });
                    }
                }
        ).executeAsync();
    }

    public static void logout() {
        if (fbWrapperObj == null) {
            fbWrapperObj = FacebookWrapper.get();
        }
        try {
            fbWrapperObj.loginManager.logOut();
        } catch (Exception e) {
            Log.e(Constants.TAG, "Logging out had exception");
            e.printStackTrace();
            Analytics.logException(e);
        }
    }

    public static String getUserID() {
        String userId = "";
        try {
            userId = AccessToken.getCurrentAccessToken().getUserId();
        } catch (Exception e) {
            Analytics.logException(e);
        }
        return userId;
    }

    public static void shareLink(String link) {
        if (fbWrapperObj == null) {
            fbWrapperObj = FacebookWrapper.get();
        }
        if (ShareDialog.canShow(ShareLinkContent.class)) {
            ShareLinkContent content = new ShareLinkContent.Builder()
                    .setContentUrl(Uri.parse(link))
                    .build();
            Analytics.sendReport("FB Share");
            fbWrapperObj.shareDialog.show(content);
        }
    }

    public static void shareVideo() {
        //Bitmap photo1 = BitmapFactory.decodeResource(
        //        context.getResources(), R.drawable.notif_spinner_large);
        /* Working photo share
        Log.i(Constants.TAG, "File dir:" + Util.getFilesDirPath());
        Bitmap photo1 = BitmapFactory.decodeFile(Util.getFilesDirPath() + "/sharePhoto.png");
        SharePhoto sharePhoto1 = new SharePhoto.Builder().setBitmap(photo1).build();

        ShareContent shareContent = new ShareMediaContent.Builder()
                .addMedium(sharePhoto1)
                .build();

        fbWrapperObj.shareDialog.show(shareContent);*/
        if (fbWrapperObj == null) {
            fbWrapperObj = FacebookWrapper.get();
        }
        try {
            ShareVideo shareVideo1 = new ShareVideo.Builder()
                    .setLocalUrl(Uri.fromFile(new File(PSUtil.getFilesDirPath() + "/share_video.mp4")))
                    .build();
            Bitmap photo1 = BitmapFactory.decodeFile(PSUtil.getFilesDirPath() + "/share_photo.png");
            SharePhoto sharePhoto1 = new SharePhoto.Builder()
                    .setBitmap(photo1)
                    .build();
            ShareContent shareContent = new ShareMediaContent.Builder()
                    .addMedium(sharePhoto1)
                    .addMedium(shareVideo1)
                    .build();

            fbWrapperObj.shareDialog.show(shareContent);
        } catch (Exception e) {
            Analytics.logException(e);
            Log.i(Constants.TAG, "Video sharing failed");
            PSUtil.showMessage(PSStrings.errorGeneric);
        }
    }

    public static void sharePhotoOnFb() {
        if (fbWrapperObj == null) {
            fbWrapperObj = FacebookWrapper.get();
        }
        try {
            Bitmap photo1 = BitmapFactory.decodeFile(PSUtil.getFilesDirPath() + "/share_photo.png");
            SharePhoto sharePhoto1 = new SharePhoto.Builder()
                    .setBitmap(photo1)
                    .build();

            ShareContent shareContent = new ShareMediaContent.Builder()
                    .addMedium(sharePhoto1)
                    .build();

            fbWrapperObj.shareDialog.show(shareContent);

        } catch (Exception e) {
            Analytics.logException(e);
            Log.i(Constants.TAG, "Video sharing failed");
            PSUtil.showMessage(PSStrings.errorGeneric);
        }
    }
}

