package in.playsimple.common;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;

import in.playsimple.Constants;
import in.playsimple.GameSpecific;

public class Friends {
    private static Context context = null;
    public static Controller friendsController = null;
    private static Friends friends = null;

    private JSONArray invitableFriends = new JSONArray();
    private JSONArray playingFriends = new JSONArray();
    private JSONArray psFriends = new JSONArray();
    private long lastUpdate = 0;
    private long lastPsUpdate = 0;
    private int lastReqTo = 0;
    private int lastInviteTo = 0;
    private JSONObject sentAtMap = new JSONObject();

    private Friends() {
    }

    public static Friends get() throws Exception {
        if (Friends.friends == null) {
            friends = new Friends();
            friends.load();
        }
        if (friendsController == null) {
            // friendsController = new Controller(Constants.LOGIN_SERVER);
        }
        return friends;
    }

    public static void setContext(Context context) {
        Friends.context = context;
    }

    private boolean load() {
        try {
            String friendsData = GameSpecific.loadFromLocalStorage(Constants.FRIENDS_FILE);
            JSONObject friendsObj = new JSONObject(friendsData);

            // Log.i(Constants.TAG, "friends data:" + friendsData);
            this.playingFriends = friendsObj.getJSONArray("playingFriends");
            this.invitableFriends = friendsObj.getJSONArray("invitableFriends");
            this.lastUpdate = friendsObj.getLong("lastUpdate");
            this.lastReqTo = friendsObj.getInt("lastReqTo");
            this.lastInviteTo = friendsObj.getInt("lastInviteTo");
            this.sentAtMap = friendsObj.getJSONObject("sentAtMap");
            this.psFriends = friendsObj.getJSONArray("psFriends");
            this.lastPsUpdate = friendsObj.getLong("lastPsUpdate");
        } catch (Exception e) {
            // Analytics.logException(e);
            friends.save();
            Log.i(Constants.TAG, "Exception when reading friends data.");
            return false;
        }
        return true;
    }

    public boolean save() {
        JSONObject friendsObj = new JSONObject();
        try {
            friendsObj.put("playingFriends", friends.playingFriends);
            friendsObj.put("invitableFriends", friends.invitableFriends);
            friendsObj.put("psFriends", friends.psFriends);
            friendsObj.put("lastUpdate", friends.lastUpdate);
            friendsObj.put("lastPsUpdate", friends.lastPsUpdate);
            friendsObj.put("lastReqTo", friends.lastReqTo);
            friendsObj.put("lastInviteTo", friends.lastInviteTo);
            friendsObj.put("sentAtMap", friends.sentAtMap);
            GameSpecific.saveToLocalStorage(Constants.FRIENDS_FILE, friendsObj.toString());

            // Log.i(Constants.TAG, "Friends data saved in android as:" +
            // friendsObj.toString());
        } catch (Exception e) {
            Analytics.logException(e);
            return false;
        }
        return true;
    }

    public static void attemptUpdate() {
        if (friends == null) {
            try {
                friends = Friends.get();
            } catch (Exception e) {
                Analytics.logException(e);
                return;
            }
        }
        Log.i(Constants.TAG,
                "DEBUG:: Friends.java attemptUpdate Attempting friend list update");
        long curTimestamp = PSUtil.getCurrentTimestamp();
        if (curTimestamp - friends.lastUpdate > PSConstants.FRIENDS_FETCH_INTERVAL) {
            FacebookWrapper.getPlayingFriends();
        } else {
            Log.i(Constants.TAG, "DEBUG:: Unable to fetch friends right now "
                    + friends.lastUpdate);
        }
        attemptPsFriendsUpdate();
    }

    public static void attemptPsFriendsUpdate() {
        if (friends == null) {
            try {
                friends = Friends.get();
            } catch (Exception e) {
                Analytics.logException(e);
                return;
            }
        }
        Log.i(Constants.TAG,
                "DEBUG:: Friends.java attemptPsFriendsUpdate Attempting PS friends update");
        long curTimestamp = PSUtil.getCurrentTimestamp();
        if (curTimestamp - friends.lastPsUpdate > PSConstants.FRIENDS_FETCH_INTERVAL) {
            Friends.syncPsFriends();
        } else {
            Log.i(Constants.TAG, "DEBUG:: Unable to fetch friends right now ");
        }
    }

    public static void reset() {
        Log.d(Constants.TAG, "DEBUG:: reset friends called");
        friends.invitableFriends = new JSONArray();
        friends.playingFriends = new JSONArray();
        friends.psFriends = new JSONArray();
        friends.lastUpdate = 0;
        friends.lastPsUpdate = 0;
        friends.lastReqTo = 0;
        friends.lastInviteTo = 0;
        friends.sentAtMap = new JSONObject();
    }

    public static void updatePlayingFriends(JSONObject friendsData) {
        try {
            if (friends == null) {
                try {
                    friends = Friends.get();
                } catch (Exception e) {
                    Analytics.logException(e);
                    return;
                }
            }
            friends.load();
            friends.playingFriends = friendsData.getJSONArray("data");
            friends.save();
            FacebookWrapper.getInvitableFriends();
        } catch (Exception e) {
            Analytics.logException(e);
        }
    }

    public static void updateInvitableFriends(JSONObject friendsData) {
        try {
            if (friends == null) {
                friends = Friends.get();
            }

            if (friends == null) {
                return;
            }
            friends.load();
            friends.invitableFriends = friendsData.getJSONArray("data");
            Log.i(Constants.TAG, "Invitable friends:"
                    + friends.invitableFriends.length());
            friends.lastUpdate = PSUtil.getCurrentTimestamp();
            friends.save();

            Log.i(Constants.TAG, "DEBUG:: Friends fetched:"
                    + friends.invitableFriends.length() + ";"
                    + friends.playingFriends.length());
//            Flags flags = Flags.get();
//            flags.setFriendsFetched(true);
//            flags.save(Constants.TRACK_FRIENDS);
//
//            Track.setContext(context);
//            Game.setContext(context);
//            // Game game = Game.get();
//            // If friend image downloads are to be triggered, use native flags
//            // and trigger from js.
        } catch (Exception e) {
            Analytics.logException(e);
        }
    }

    public static int getRandomInvitableIndex() {
        int rand = PSConstants.NOT_FOUND;
        if (friends == null) {
            rand = PSConstants.NO_FB_CONNECT;
            return rand;
        }
        int numFriends = friends.invitableFriends.length();
        if (numFriends == 0) {
            rand = PSConstants.NO_INVITABLE_FRIENDS;
            return rand;
        }
        rand = PSUtil.getRandomIntInRange(0, 10);
        if (rand > numFriends) {
            rand = 0;
        }
        return rand;
    }

    public static int getRandomInvitable(Bitmap[] images, String[] names,
                                         int count) {
        int randIndex = getRandomInvitableIndex();
        if (randIndex == PSConstants.NOT_FOUND
                || randIndex == PSConstants.NO_FB_CONNECT
                || randIndex == PSConstants.NO_INVITABLE_FRIENDS) {
            return randIndex;
        }
        int numFriends = friends.invitableFriends.length();
        try {
            for (int i = 0; i < count; i++) {
                int nextIndex = (randIndex + i) % 10;
                if (nextIndex < numFriends) {
                    randIndex = nextIndex;
                } else {
                    randIndex = 0;
                }
                JSONObject friendInfo = friends.invitableFriends
                        .getJSONObject(randIndex);
                names[i] = friendInfo.getString("name");
                String filePath = PSUtil.getFilesDirPath() + "/" + names[i] + ".jpg";
                images[i] = BitmapFactory.decodeFile(filePath);
                // Log.i(Constants.TAG, "Images missing?" + images[i]);
                if (images[i] == null) {
                    return PSConstants.NO_INVITE_IMAGE;
                }
            }
        } catch (Exception e) {
            Analytics.logException(e);
            randIndex = PSConstants.NOT_FOUND;
        }
        return randIndex;
    }

    public static void syncPsFriends() {
        Log.i(Constants.TAG, "Attempting to sync ps friends");
        try {
            // Get latest info
            User.get();
        } catch (Exception e) {
            Analytics.logException(e);
        }
        HashMap<String, String> map = User.getBaseMap();
        if (map.get("fbAccessToken").equals("") || map.get("refId").equals("")
                || map.get("fbid").equals("")) {
            Log.i(Constants.TAG, "Not yet ready to fetch ps friends");
            return;
        }

        // friendsController.sendRequest("psfriends", "sync", map);
    }

    public static void respHandler(String action, JSONObject data,
                                   boolean success) {
        if (action.equals("sync")) {
            if (success == false) {
                return;
            }
            try {
                String psFriendsStr = data.getString("psfriends");
                JSONObject psFriendsObj = new JSONObject(psFriendsStr);
                friends.psFriends = psFriendsObj.getJSONArray("Friends");
                friends.lastPsUpdate = PSUtil.getCurrentTimestamp();
                friends.save();
                // Log.i(Constants.TAG, "PS friends:" + friends.psFriends);
            } catch (Exception e) {
                Analytics.logException(e);
            }
        }
    }

    public static void downloadTopKInvitableFriends(int k) throws JSONException {
        Log.d(Constants.TAG, "starting download top k friends");
        String downloadString = "";
        for (int ix = 0; ix < k; ix++) {
            JSONObject obj = friends.invitableFriends.getJSONObject(ix);
            String url = obj.getJSONObject("picture").getJSONObject("data")
                    .getString("url");
            downloadString += url + "," + obj.getString("name") + ".jpg;";
            GameSpecific.downloadFile(obj.getString("name"), downloadString, "", "", "");
        }
        Log.d(Constants.TAG, "downloading invitable friends " + downloadString);

    }

    public String getFbPicUrl(String fbid) {
        return PSConstants.GRAPH_API_BASE + "/" + fbid
                + "/picture?type=normal&width=200&height=200";
    }
}