package in.playsimple.common;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.RemoteException;
import android.util.Log;

import com.android.installreferrer.api.InstallReferrerClient;
import com.android.installreferrer.api.InstallReferrerStateListener;
import com.android.installreferrer.api.ReferrerDetails;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import in.playsimple.Constants;

import static android.content.Context.MODE_PRIVATE;

public class InstallReferrer implements InstallReferrerStateListener {
    InstallReferrerClient referrerClient;
    private static Context context;
    private static User user = null;

    public void startConnection(Context context) {
        InstallReferrer.context = context;

        // only if the referrer data is not already fetched, it should be fetched again
        final SharedPreferences prefs = context.getSharedPreferences(PSConstants.PREFS_MISC, MODE_PRIVATE);
        boolean isReferrerDataAlreadyFetched = prefs.getBoolean(PSConstants.PREFS_MISC_INSTALL_REFERRER_KEY, false);

        if (isReferrerDataAlreadyFetched) {
            Log.d(Constants.TAG, "install referrer data: details already fetched");
            return;
        }
        try {
            referrerClient = InstallReferrerClient.newBuilder(context).build();
            referrerClient.startConnection(this);
        } catch (RuntimeException e) {
            Analytics.logException(e);
        }
    }

    @Override
    public void onInstallReferrerSetupFinished(int responseCode) {
        switch (responseCode) {
            case InstallReferrerClient.InstallReferrerResponse.OK:
                try {
                    Log.v(Constants.TAG, "InstallReferrer connected");
                    ReferrerDetails response = referrerClient.getInstallReferrer();
                    handleReferrer(response);
                    referrerClient.endConnection();
                } catch (RemoteException e) {
                    // e.printStackTrace();
                }
                break;
            case InstallReferrerClient.InstallReferrerResponse.FEATURE_NOT_SUPPORTED:
                Log.w(Constants.TAG, "InstallReferrer not supported");
                break;
            case InstallReferrerClient.InstallReferrerResponse.SERVICE_UNAVAILABLE:
                Log.w(Constants.TAG, "Unable to connect to the service");
                break;
            default:
                Log.w(Constants.TAG, "responseCode not found.");
        }
    }

    public void handleReferrer(ReferrerDetails response) {
        String installReferrerEncoded = response.getInstallReferrer();
        long referrerClickTimestampSeconds = response.getReferrerClickTimestampSeconds();
        long installBeginTimestampSeconds = response.getInstallBeginTimestampSeconds();
        final String timestampTracking = referrerClickTimestampSeconds + "_" + installBeginTimestampSeconds;

        String k = PSConstants.TRACK_INSTALL;
        try {
            Controller.setContext(context);
            User.setContext(context);
            Track.setContext(context);
            user = User.get();
            Track.get();

            if (!user.getRefId().equals("")) {
                k = PSConstants.TRACK_REACT;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            String utm_source = "";
            String utm_medium = "";
            String utm_term = "";
            String utm_content = "";
            String utm_campaign = "";

            if (installReferrerEncoded == null) {
                utm_source = "unknown";
            } else {
                String referrer = java.net.URLDecoder.decode(installReferrerEncoded, "UTF-8");
                Log.d(Constants.TAG, "install referrer data: referrer decoded: " + referrer);
                String[] params = referrer.split("&");
                HashMap<String, String> refInfo = new HashMap<String, String>();
                for (int i = 0; i < params.length; i++) {
                    String[] keyValue = params[i].split("=");
                    try {
                        refInfo.put(keyValue[0], keyValue[1]);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                if (refInfo.containsKey("utm_source")) {
                    utm_source = refInfo.get("utm_source");
                } else {
                    utm_source = "unknown";
                }
                if (refInfo.containsKey("utm_medium")) {
                    utm_medium = refInfo.get("utm_medium");
                }
                if (refInfo.containsKey("utm_term")) {
                    utm_term = refInfo.get("utm_term");
                }
                if (refInfo.containsKey("utm_content")) {
                    utm_content = refInfo.get("utm_content");
                }
                if (refInfo.containsKey("utm_campaign")) {
                    utm_campaign = refInfo.get("utm_campaign");
                }
                final String content1 = utm_content;

                // Attempting to auto grant referral bonus
                if (utm_term != null && utm_term.equals("ref")) {
                    if (utm_content != null && !utm_content.equals("")) {
                        int delay = 10 * 1000;
                        if (user.getRefId().equals("")) {
                            delay = 30 * 1000;
                        }
                        final Timer timer = new Timer();
                        timer.schedule(new TimerTask() {
                            @Override
                            public void run() {
                                tryRefCode(user.getRefId(), content1);
                            }
                        }, delay);
                    }
                }
            }

            Analytics.install(k, utm_source, utm_medium, utm_term, utm_content, utm_campaign);
            Log.i(Constants.TAG, "Install info:" + k + ";" + utm_source + ";"
                    + utm_medium + ";" + utm_term + ";" + utm_content + ";"
                    + utm_campaign);
            int delay = 30 * 1000;

            final Context context1 = context;
            final String k_final = k;
            final String utm_source_final = utm_source;
            final String utm_medium_final = utm_medium;
            final String utm_term_final = utm_term;
            final String utm_content_final = utm_content;
            final String utm_campaign_final = utm_campaign;
            final Timer timer = new Timer();
            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    try {
                        user = User.get(); // Making sure user refid is refreshed so tracking can happen
                        Track.setContext(context1);
                        Track track = Track.get();
                        String installerType = PSConstants.TRACK_UNKNOWN_DL;
                        boolean installedFromStore = verifyInstallerId(context);
                        if (installedFromStore) {
                            installerType = PSConstants.TRACK_PLAY_STORE_DL;
                        }
                        if (user.getRefId().equals("")) {
                            Track.trackCounter(k_final, utm_source_final, utm_medium_final, installerType + "",
                                    utm_content_final, utm_campaign_final, timestampTracking, "0", "");
                        } else {
                            Track.trackCounterImmediate(k_final, utm_source_final, utm_medium_final, installerType + "",
                                    utm_content_final, utm_campaign_final, timestampTracking, "0", "");
                        }
                        // track.syncPendingTracking(false, true);
                        Log.i(Constants.TAG, "Install - Attempting pending tracking sync:" + user.getRefId());


                        final SharedPreferences prefs = context.getSharedPreferences(PSConstants.PREFS_MISC, MODE_PRIVATE);
                        prefs.edit()
                                .putBoolean(PSConstants.PREFS_MISC_INSTALL_REFERRER_KEY, true)
                                .apply();
                    } catch (Exception e) {
                        Analytics.logException(e);
                    }
                }
            }, delay);
        } catch (UnsupportedEncodingException e) {
            Analytics.logException(e);
            e.printStackTrace();
        }
    }

    /**
     * Function for determining app installing source, returns true if installed from play store, else false.
     * @param context:  base object
     * @return boolean
     */
    public static boolean verifyInstallerId(Context context) {
        try {
            // A list with valid installers package name
            List<String> validInstallers = new ArrayList<>(Arrays.asList("com.android.vending", "com.google.android.feedback"));
            // The package name of the app that has installed your app
            final String installer = context.getPackageManager().getInstallerPackageName(context.getPackageName());
            // true if your app has been downloaded from Play Store
            return (installer != null && validInstallers.contains(installer));
        }  catch (Exception e) {
            Analytics.logException(e);
        }
        return false;
    }

    private void tryRefCode(String refId, String referrer) {
        if (refId.equals("")) {
            Track.trackCounter(PSConstants.TRACK_INSTALL, "code", "missing", "refid", referrer,
                    "", "", "0", "");
            return;
        }
        Track.trackCounter(PSConstants.TRACK_INSTALL, "code", "validating", "server", referrer,
                "auto", "", "0", "");

        Log.i(Constants.TAG, "Trying refcode:" + referrer);
    }

    @Override
    public void onInstallReferrerServiceDisconnected() {

    }
}
