package in.playsimple.common;

import android.app.Activity;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.NonNull;

import com.google.android.gms.auth.api.Auth;
import com.google.android.gms.auth.api.signin.GoogleSignIn;
import com.google.android.gms.auth.api.signin.GoogleSignInAccount;
import com.google.android.gms.auth.api.signin.GoogleSignInClient;
import com.google.android.gms.auth.api.signin.GoogleSignInOptions;
import com.google.android.gms.auth.api.signin.GoogleSignInResult;
import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthCredential;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.auth.PlayGamesAuthProvider;

import in.playsimple.Constants;
import in.playsimple.GameSpecific;

public class PSLogin {
    private static Activity activity = null;
    private final static int GOOGLE_SIGN_IN = 10001;
    private final static int GOOGLE_PLAY_GAMES_SIGN_IN = 10002;

    public static void init(Activity act) {
        activity = act;
    }

    public static void signInWithGoogle() {
        // Configure Google Sign In
        GoogleSignInOptions gso = new GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                .requestIdToken(GameSpecific.getGoogleSignInClient())
                .requestEmail()
                .build();


        // Build a GoogleSignInClient with the options specified by gso.
        GoogleSignInClient mGoogleSignInClient = GoogleSignIn.getClient(activity, gso);
        Intent signInIntent = mGoogleSignInClient.getSignInIntent();

        activity.startActivityForResult(signInIntent, GOOGLE_SIGN_IN);
    }

    public static void signInWithPlayGames()
    {
        Log.d(Constants.TAG, "signinwithgooglePlaygames");

        GoogleSignInOptions gso = new GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
                .requestServerAuthCode(GameSpecific.getGoogleSignInClient())
                .build();
        GoogleSignInClient GoogleSignInClient = GoogleSignIn.getClient(activity, gso);
//        GoogleSignInClient  GoogleSignInClient = GoogleSignIn.getClient(activity,
//                GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN);
        Intent signInIntent = GoogleSignInClient.getSignInIntent();

        activity.startActivityForResult(signInIntent,GOOGLE_PLAY_GAMES_SIGN_IN);
    }

    public static boolean handleSignInResult(int requestCode, int resultCode, Intent data) {
        boolean result = false;
        switch (requestCode) {
            case GOOGLE_PLAY_GAMES_SIGN_IN:
                result = handlePlayGamesSignInResult(requestCode, resultCode, data);
                break;
            case GOOGLE_SIGN_IN:
                result = handleGmailSignInResult(requestCode, resultCode, data);
                break;
        }
        return result;
    }

    private static boolean handleGmailSignInResult(int requestCode, int resultCode, Intent data) {
        // Result returned from launching the Intent from GoogleSignInApi.getSignInIntent(...);
        Log.i(Constants.TAG, "Google login data - " + data.getAction() + " - " + data.getDataString() + " - " + data.getData() + " - "
                + data.getPackage() + " - " + data.getScheme() + " - " + data.toString() + " - " + data.getExtras());
        Task<GoogleSignInAccount> task = GoogleSignIn.getSignedInAccountFromIntent(data);
        try {
            // Google Sign In was successful, authenticate with Firebase
            GoogleSignInAccount account = task.getResult(ApiException.class);
            Log.i(Constants.TAG, "Google login results: - " + account.getEmail() + " - " + account.getDisplayName() + " - " + account.getPhotoUrl());
            // firebaseAuthWithGoogle(account);
        } catch (ApiException e) {
            // Google Sign In failed, update UI appropriately
            Log.w(Constants.TAG, "Google sign in failed" + e.getStatusCode() + " - " + e.getMessage(), e);
            // ...
            return false;
        }
        return true;
    }

    private static boolean handlePlayGamesSignInResult(int requestCode, int resultCode, Intent data) {
        GoogleSignInResult result = Auth.GoogleSignInApi.getSignInResultFromIntent(data);
        if (result.isSuccess()) {
            // The signed in account is stored in the result.
            Log.d(Constants.TAG, "result success google play games sign in ");
            GoogleSignInAccount signedInAccount = result.getSignInAccount();
            firebaseAuthWithPlayGames(signedInAccount);
        } else {
            Log.d(Constants.TAG, "result fail google play games sign in ");
            String message = result.getStatus().getStatusMessage();
            if (message == null || message.isEmpty()) {
                //message = getString(R.string.signin_other_error);
            }
            return false;
        }
        return true;
    }

    public static void firebaseAuthWithPlayGames(GoogleSignInAccount acct) {
        Log.d(Constants.TAG, "firebaseAuthWithPlayGames:" + acct.getId());

        final FirebaseAuth auth = FirebaseAuth.getInstance();
        AuthCredential credential = PlayGamesAuthProvider.getCredential(acct.getServerAuthCode());
        auth.signInWithCredential(credential)
                .addOnCompleteListener(activity, new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {
                        if (task.isSuccessful()) {
                            // Sign in success, update UI with the signed-in user's information
                            Log.d(Constants.TAG, "signInWithCredential:success");
                            FirebaseUser user = auth.getCurrentUser();
                           // updateUI(user);
                        } else {
                            // If sign in fails, display a message to the user.
                            Log.w(Constants.TAG, "signInWithCredential:failure", task.getException());
//                            Toast.makeText(MainActivity.this, "Authentication failed.",
//                                    Toast.LENGTH_SHORT).show();
//                            updateUI(null);
                        }

                        // ...
                    }
                });
    }

}
