package in.playsimple.common;

import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.media.AudioManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.BatteryManager;
import android.os.Build;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.SystemClock;
import android.provider.Settings;
import android.speech.RecognizerIntent;
import android.telephony.TelephonyManager;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

import com.adjust.sdk.Adjust;
import com.android.volley.RequestQueue;
import com.android.volley.toolbox.HurlStack;
import com.android.volley.toolbox.Volley;
import com.chartboost.sdk.Chartboost;
import com.facebook.CallbackManager;
import com.google.android.gms.ads.identifier.AdvertisingIdClient;
import com.google.firebase.crashlytics.FirebaseCrashlytics;
import com.mopub.common.MoPub;
import com.tapjoy.Tapjoy;

import java.util.ArrayList;
import java.util.List;

import androidx.appcompat.app.AlertDialog;
import in.playsimple.Constants;
import in.playsimple.GameSpecific;
import in.playsimple.common.experimental.AdIntelligence;
import in.playsimple.common.experimental.Speak;
import in.playsimple.common.flutter.FlutterBridge;
import in.playsimple.common.flutter.FlutterViewActivity;
import in.playsimple.common.mopub.PSMopubAds;

import static android.content.Context.BATTERY_SERVICE;

public class PSUtil {
    private static Activity activity = null;
    private static Context context = null;

    private final static String FLUTTER_ROUTE_NAME = "psUtil";

    private static boolean appPaused = false;
    private static RequestQueue mRequestQueue = null;
    private static boolean isMovingToGame = false;

    public static void init(Activity act) {
        activity = act;
        context = act;

        AdIntelligence.init(activity);
        Analytics.init(activity);
        Ads.setActivity(activity);
        Controller.setContext(activity);
        FacebookWrapper.setActivity(activity);
        Experiment.setContext(activity);
        Track.setContext(activity);
        User.setContext(activity);
        Speak.setActivity(activity);
        PSNotifs.init(activity);
        PSLogin.init(activity);
        PSRuntime.init(activity);

        new CountDownTimer(1500, 1500) {
            public void onTick(long millisUntilFinished) {
            }

            public void onFinish() {
                PSMopubAds.init(activity);
                FlutterBridge.setActivity(activity);
                Ads.initializeTapjoy();
                FacebookWrapper.get();
            }
        }.start();

        try {
            Experiment.get();
            Track.get();
        } catch (Exception e1) {
            e1.printStackTrace();
        }

        Analytics.setDeviceInfo(PSUtil.getCpuArch(), PSUtil.getDeviceWidth() + "", PSUtil.getDeviceHeight() + "",
                PSUtil.getDeviceModel(), PSUtil.getDeviceId(), PSUtil.getCountry(), PSUtil.isOnline() + "");
    }

    public static void onCreate(Bundle savedInstanceState) {
        MoPub.onCreate(activity);
        GameSpecific.onCreate();
    }

    public static void onStart() {
        Log.d(Constants.TAG, "activity starting");
        FlutterBridge.sendStateToFlutterModule(FLUTTER_ROUTE_NAME, "onStart");
        MoPub.onStart(activity);
        Tapjoy.onActivityStart(activity);
        GameSpecific.onStart();
    }

    public static void onRestart(Context context) {
        Log.d(Constants.TAG, "activity re-starting");
        if (GameSpecific.shouldGameRestart()) {
            GameSpecific.restartGame(context);
        } else {
            GameSpecific.setActivityBackgroundTime(0);
        }

        FlutterBridge.sendStateToFlutterModule(FLUTTER_ROUTE_NAME, "onRestart");
        MoPub.onRestart(activity);
        GameSpecific.onRestart(context);
    }

    public static void onResume() {
        Log.d(Constants.TAG, "activity resuming");
        appPaused = false;
        isMovingToGame = true;
        FlutterBridge.sendStateToFlutterModule(FLUTTER_ROUTE_NAME, "onResume");
        Chartboost.onResume(activity);
        MoPub.onResume(activity);
        Analytics.setAppOpen();
        GameSpecific.onResume();
        Analytics.sendReport("App resumed");
    }

    public static void onPause() {
        appPaused = true;
        FlutterBridge.sendStateToFlutterModule(FLUTTER_ROUTE_NAME, "onPause");
        Chartboost.onPause(activity);
        MoPub.onPause(activity);
        GameSpecific.onPause();
        Analytics.sendReport("App paused");
    }

    public static void onStop() {
        isMovingToGame = false;
        FlutterBridge.sendStateToFlutterModule(FLUTTER_ROUTE_NAME, "onStop");
        Tapjoy.onActivityStop(activity);
        Chartboost.onStop(activity);
        MoPub.onStop(activity);
        GameSpecific.onStop();
    }

    public static void onDestroy() {
        FlutterBridge.sendStateToFlutterModule(FLUTTER_ROUTE_NAME, "onDestroy");
        PSMopubAds.onDestroy();
        MoPub.onDestroy(activity);
        Chartboost.onDestroy(activity);
        GameSpecific.onDestroy();
    }

    public static void onBackPressed() {
        FlutterBridge.sendStateToFlutterModule(FLUTTER_ROUTE_NAME, "onBackPressed");
        // If an interstitial is on screen, close it.
        MoPub.onBackPressed(activity);
        GameSpecific.onBackPressed();
    }

    public static void onActivityResult(int requestCode, int resultCode, Intent data) {
        boolean handled = GameSpecific.onActivityResult(requestCode, resultCode, data);
        CallbackManager fbCallbackManager = FacebookWrapper.getCallbackManager();
        if (!handled && fbCallbackManager != null && !fbCallbackManager.onActivityResult(requestCode, resultCode, data)) {
            // Try other possibilities
            switch (requestCode) {
                case PSConstants.REQ_CODE_SPEECH_INPUT: {
                    if (resultCode == Activity.RESULT_OK && null != data) {
                        ArrayList<String> result = data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS);
                        Log.i(Constants.TAG, "Result of speech input:" + result.get(0));
                        handled = true;
                    }
                    break;
                }
            }
        }
        if (!handled) {
            handled = PSLogin.handleSignInResult(requestCode, resultCode, data);
        }
    }

    public static void setContext(Context con) {
        context = con;
    }

    public static void handleNotifClick(Bundle b) {
        GameSpecific.handleNotifClick(b);
    }

    public static String getFilesDirPath() {
        return context.getFilesDir().getAbsolutePath();
    }

    public static String getBaseDirPath() {
        return context.getCacheDir().getParent();
    }

    public static String getCacheDirPath() {
        return context.getCacheDir().getAbsolutePath();
    }

    public static String getExternalBaseDirPath() {
        String dir = null;
        try {
            dir = context.getExternalCacheDir().getParent();
        } catch (Exception e) {

        }
        return dir;
    }

    public static boolean getAppPaused() {
        return appPaused;
    }

    public static int getBatteryLevel() {
        int batteryLevel;
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP) {
            BatteryManager batteryManager = (BatteryManager) activity.getSystemService(BATTERY_SERVICE);
            batteryLevel = batteryManager.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY);
        } else {
            Intent intent = new ContextWrapper(activity.getApplicationContext()).
                    registerReceiver(null, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));
            batteryLevel = (intent.getIntExtra(BatteryManager.EXTRA_LEVEL, -1) * 100) /
                    intent.getIntExtra(BatteryManager.EXTRA_SCALE, -1);
        }

        return batteryLevel;
    }

    public static String getAndroidId() {
        // https://developer.android.com/reference/android/provider/Settings.Secure.html#ANDROID_ID
        String androidId = Settings.Secure.getString(activity.getContentResolver(), Settings.Secure.ANDROID_ID);
        Log.i(Constants.TAG, "Android id for user:" + androidId);
        return androidId;
    }

    public static boolean isTablet() {

        DisplayMetrics displayMetrics = new DisplayMetrics();
        activity.getWindowManager().getDefaultDisplay().getMetrics(displayMetrics);

        int width = displayMetrics.widthPixels / displayMetrics.densityDpi;
        int height = displayMetrics.heightPixels / displayMetrics.densityDpi;

        double screenDiagonal = Math.sqrt(width * width + height * height);
        return (screenDiagonal >= 5.7);
    }

    public static RequestQueue getmRequestQueue() {
        if (mRequestQueue == null) {
            mRequestQueue = Volley.newRequestQueue(activity.getApplicationContext(),
                    new HurlStack());
        }
        return mRequestQueue;
    }

    public static void handleNewIntent(Intent intent) {
        String data = intent.getDataString();

        // passing deeplink data to Adjust
        Log.d(Constants.TAG, "Adjust log: the deeplinkg parameters: " + data);
        Adjust.appWillOpenUrl(intent.getData(), context);
    }

    public static boolean isOnline() {
        if (!PSUtil.hasNetworkConnectivity()) {
            // Log.i(Constants.TAG, "Network connectivity missing");
            return false;
        }

        return true;
    }

    public static String getDeviceModel() {
        return android.os.Build.MODEL;
    }

    public static String getCpuArch() {
        return Build.CPU_ABI;
    }

    private static long lastNetworkCheck = 0;
    private static boolean userIsNetworkConnected = true;

    public static boolean hasNetworkConnectivity() {
        long curTimestamp = PSUtil.getCurrentTimestamp();
        if (curTimestamp - lastNetworkCheck < PSConstants.NETWORK_CHECK_INTERVAL) {
            // Log.i(Constants.TAG, "Already did network check:" +
            // userIsNetworkConnected);
            return userIsNetworkConnected;
        }
        if (context == null) {
            Log.i(Constants.TAG, "Context not set in hasNetworkConnectivity");
            return false;
        }
        lastNetworkCheck = curTimestamp;
        ConnectivityManager cm = (ConnectivityManager) context
                .getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo ni = cm.getActiveNetworkInfo();
        if (ni == null) {
            userIsNetworkConnected = false;
        } else {
            userIsNetworkConnected = true;
        }
        return userIsNetworkConnected;
    }

    // Highest possible value is (max - 1)
    public static int getRandomIntInRange(int min, int max) {
        int rand = min + (int) (Math.random() * (max - min));
        return rand;
    }

    public static long getCurrentTimestampMs() {
        return System.currentTimeMillis();
    }

    public static long getCurrentTimestamp() {
        return (PSUtil.getCurrentTimestampMs() / 1000);
    }

    public static int getSystemUpTime() {
        return (int) (SystemClock.elapsedRealtime() / 1000);
    }

    public static boolean isSoundPlayable() {
        boolean isSoundPlayable = true;
        try {
            AudioManager am = (AudioManager) activity
                    .getSystemService(Context.AUDIO_SERVICE);
            switch (am.getRingerMode()) {
                case AudioManager.RINGER_MODE_SILENT:
                    isSoundPlayable = false;
                    break;
                case AudioManager.RINGER_MODE_VIBRATE:
                    isSoundPlayable = false;
                    break;
                case AudioManager.RINGER_MODE_NORMAL:
                    isSoundPlayable = true;
                    break;
            }
        } catch (Exception e) {
            Analytics.logException(e);
        }
        return isSoundPlayable;
    }

    public static boolean isAppInstalled(String pkgName) {
        List<ApplicationInfo> packages;
        PackageManager pm;

        pm = context.getPackageManager();
        packages = pm.getInstalledApplications(0);
        for (ApplicationInfo packageInfo : packages) {
            if (packageInfo.packageName.equals(pkgName))
                return true;
        }
        return false;
    }

    public static void showMessage(final String msg) {
        if (activity == null || activity.isFinishing()) {
            Log.i(Constants.TAG, "Activity not set in showMessage");
            return;
        }
        activity.runOnUiThread(() -> {
            Toast toast = Toast.makeText(activity, msg, Toast.LENGTH_SHORT);
            toast.setGravity(Gravity.CENTER, 0, 0);
            toast.show();
        });
    }

    public static String getDeviceId() {
        if (context == null) {
            Log.i(Constants.TAG, "Context not set in getDeviceId");
            return "";
        }
        String deviceId = Settings.Secure.getString(context.getContentResolver(),
                Settings.Secure.ANDROID_ID);
        // Log.i(Constants.TAG, "Returning device id as:" + deviceId);
        return deviceId;
    }

    public static int getVersionCode() {
        int versionCode = 0;
        if (context == null) {
            Log.i(Constants.TAG, "Context not set in getVersionCode");
            return 0;
        }

        try {
            versionCode = context.getPackageManager().getPackageInfo(
                    context.getPackageName(), 0).versionCode;
        } catch (Exception e) {
            versionCode = 0;
        }
        return versionCode;
    }

    public static String getAdvertiserId() {
        String advertiserId = "";
        try {
            AdvertisingIdClient.Info adInfo = AdvertisingIdClient.getAdvertisingIdInfo(context);
            advertiserId = adInfo.getId();
        } catch (Exception e) {
            // May happen if device doesn't have google play services.
            // Analytics.logException(e);
        }
        // Log.i(Constants.TAG, "Returning advertiserId as:" + advertiserId);
        return advertiserId;
    }

    public static String getAdjustId() {
        String adjustId = "";
        adjustId = Adjust.getAdid();
        return adjustId;
    }


    public static String getOsVersion() {
        return android.os.Build.VERSION.RELEASE;
    }

    public static int getOsCode() {
        return android.os.Build.VERSION.SDK_INT;
    }

    public static int getDeviceWidth() {
        if (activity == null) {
            Log.i(Constants.TAG, "Activity not set in getDeviceWidth");
            return 0;
        }
        DisplayMetrics dm = new DisplayMetrics();
        activity.getWindowManager().getDefaultDisplay().getMetrics(dm);
        return dm.widthPixels;
    }

    public static int getDeviceHeight() {
        if (activity == null) {
            Log.i(Constants.TAG, "Activity not set in getDeviceHeight");
            return 0;
        }
        DisplayMetrics dm = new DisplayMetrics();
        activity.getWindowManager().getDefaultDisplay().getMetrics(dm);
        return dm.heightPixels;
    }

    public static String getBuildVersion() {
        String version = "0";
        if (activity == null) {
            Log.i(Constants.TAG, "Activity not set in contactUs");
            return version;
        }

        try {
            version = activity.getPackageManager().getPackageInfo(
                    activity.getPackageName(), 0).versionName;
        } catch (Exception e) {

        }
        return version;
    }

    public static void showAlert(final String title, final String msg) {
        if (activity == null || activity.isFinishing()) {
            Log.i(Constants.TAG, "Activity not set in showAlert");
            return;
        }
        Log.i(Constants.TAG, "Showing Alert " + title + msg);
        activity.runOnUiThread(new Runnable() {
            public void run() {
                AlertDialog.Builder builder = new AlertDialog.Builder(activity);
                builder.setMessage(msg)
                        .setTitle(title)
                        .setCancelable(true)
                        .setPositiveButton("OK", (dialogInterface, i) -> dialogInterface.dismiss());

                AlertDialog dialog = builder.create();
                dialog.setCanceledOnTouchOutside(false);
                dialog.show();
            }
        });
    }


    public static String getCountry() {
        String country = "";
        if (activity == null) {
            Log.i(Constants.TAG, "Activity not set in getCountry");
            return country;
        }
        try {
            TelephonyManager telephonyManager = (TelephonyManager) activity
                    .getSystemService(Context.TELEPHONY_SERVICE);
            country = telephonyManager.getSimCountryIso();
        } catch (Exception e) {
            Analytics.logException(e);
        }
        return country;
    }

    public static boolean isOnAnyAppActivity() {
        return (isMovingToGame || FlutterViewActivity.isMovingToFlutter);
    }

    public static boolean launchAnotherGame(String pkgName) {
        Intent launchIntent = context.getPackageManager().getLaunchIntentForPackage(pkgName);
        if (launchIntent != null) {
            context.startActivity(launchIntent);//null pointer check in case package name was not found
            return true;
        }
        return false;
    }
}
