package in.playsimple.common;

import android.content.Context;
import android.util.Log;

import com.tapjoy.TJActionRequest;
import com.tapjoy.TJError;
import com.tapjoy.TJGetCurrencyBalanceListener;
import com.tapjoy.TJPlacement;
import com.tapjoy.TJPlacementListener;
import com.tapjoy.Tapjoy;

import java.util.HashMap;

import in.playsimple.Constants;

public class TapjoyContent {

    TJPlacement placement;
    String key;
    static HashMap<String, TapjoyContent> contentMap;

    private TapjoyContent() {

    }

    public void showContent() {
        Log.d(Constants.TAG, "showing tapjoy content");
        this.placement.showContent();
    }

    public static TapjoyContent get(Context context, String contentName) {
        if (contentMap == null) {
            contentMap = new HashMap<String, TapjoyContent>();
        }
        if (!contentMap.containsKey(contentName)) {
            TapjoyContent tjContent = new TapjoyContent(context, contentName);
            contentMap.put(contentName, tjContent);
            tjContent.key = contentName;
            tjContent.requestContent();
        }
        return contentMap.get(contentName);
    }

    public void requestContent() {
        if (!placement.isContentAvailable()) {
            placement.requestContent();
        }
    }

    public boolean isContentReady() {
        return placement.isContentReady();
    }

    public static void checkTapjoyCash() {
        Tapjoy.getCurrencyBalance(new CustomCurrencyBalanceListener());
    }

    private TapjoyContent(Context context, String contentName) {
        TJPlacementListener placementListener = new TJPlacementListener() {
            @Override
            public void onClick(TJPlacement arg0) {
                Log.i(Constants.TAG, "Tapjoy element clicked");
            }

            @Override
            public void onRewardRequest(TJPlacement arg0, TJActionRequest arg1,
                                        String arg2, int arg3) {
                TapjoyContent.checkTapjoyCash();
            }

            @Override
            public void onRequestSuccess(TJPlacement arg0) {
                Log.d(Constants.TAG, "content available succesfull");
                TapjoyContent.checkTapjoyCash();
            }

            @Override
            public void onRequestFailure(TJPlacement arg0, TJError arg1) {
                // TODO Auto-generated method stub
                Log.d(Constants.TAG, "no content available - failure");
            }

            @Override
            public void onPurchaseRequest(TJPlacement arg0,
                                          TJActionRequest arg1, String arg2) {
                // TODO Auto-generated method stub
            }

            @Override
            public void onContentShow(TJPlacement arg0) {

            }

            @Override
            public void onContentReady(TJPlacement arg0) {
                Log.d(Constants.TAG, "Tapjoy content ready ");
                // TODO Auto-generated method stub
            }

            @Override
            public void onContentDismiss(TJPlacement arg0) {
                try {
                    placement.requestContent();
                    Tapjoy.getCurrencyBalance(new CustomCurrencyBalanceListener());
                } catch (Exception e) {
                    Analytics.logException(e);
                }
            }
        };
        placement = new TJPlacement(context, contentName, placementListener);
        placement.requestContent();
    }
}

class CustomCurrencyBalanceListener implements TJGetCurrencyBalanceListener {

    @Override
    public void onGetCurrencyBalanceResponse(String arg0, int arg1) {
        Log.i(Constants.TAG, "callback currency");
        Log.i(Constants.TAG, "received " + arg0 + " total amount = " + arg1);
        Ads.updateTapjoyCash(arg1);
    }

    @Override
    public void onGetCurrencyBalanceResponseFailure(String arg0) {
        Log.i(Constants.TAG, "error " + arg0);
    }
}
