package in.playsimple.common;

import android.content.Context;
import android.util.Log;

import com.google.firebase.crashlytics.FirebaseCrashlytics;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CopyOnWriteArrayList;

import in.playsimple.Constants;
import in.playsimple.GameSpecific;

public class Track {
    public static final String TRACKID = "trackId";
    public static final String DATA = "data";

    public static Track track = null;
    public static Context context = null;
    public static Controller trackController = null;
    public static int uniformError = 0;

    private static User user;
    private long lastSync = 0;
    private static boolean attemptImmediateSync = false;
    private static CopyOnWriteArrayList<JSONObject> trackingData = null;
    private static int trackingIndex = 0;
    private int versionCode = 0;
    private int origin = 0;
    private static HashMap<String, Boolean> trackingMap = new HashMap<>();

    private Track() {
        try {
            user = User.get();
        } catch (Exception e) {
            Log.i(Constants.TAG, "No tracking data yet");
            // Analytics.logException(e);
        }
    }

    public static Track get() throws Exception {
        if (context == null) {
            throw new Exception(
                    "Tracking context needs to be set before getting tracking object.");
        }
        if (track == null) {
            track = new Track();
            track.load();
        }
        return track;
    }

    public static void setContext(Context context) {
        Track.context = context;
    }

    public static boolean attemptSync() {
        return attemptImmediateSync;
    }

    private static void setAttemptSync(boolean sync) {
        attemptImmediateSync = sync;
    }

    public long getLastSync() {
        return lastSync;
    }

    public CopyOnWriteArrayList<JSONObject> getTrackingData() {
        return trackingData;
    }

    public int getNumEntries() {
        return trackingData.size();
    }

    private void load() {
        try {
            String trackData = GameSpecific.loadFromLocalStorage(Constants.NATIVE_TRACK_FILE);
            JSONObject trackingInfo = new JSONObject(trackData);
            JSONArray arr = trackingInfo.getJSONArray("data");
            trackingData = new CopyOnWriteArrayList<JSONObject>();
            for (int ix = 0; ix < arr.length(); ix++) {
                JSONObject jObj = new JSONObject().put(TRACKID,
                        getNextTrackingIndex() + "").put(DATA,
                        arr.getJSONObject(ix));
                trackingData.add(jObj);
            }
            lastSync = trackingInfo.getLong("lastSync");
            if (trackingInfo.has("origin")) {
                origin = trackingInfo.getInt("origin");
            } else {
                origin = PSUtil.getRandomIntInRange(1, PSConstants.TRACK_ORIGIN_MAX);
                Log.i(Constants.TAG, "Creating new origin:" + origin);
            }
            if (trackingInfo.has("trackingIndex")) {
                trackingIndex = trackingInfo.getInt("trackingIndex");
            }
            getVersionCode();
        } catch (Exception e) {
            // Analytics.logException(e);
            Log.i(Constants.TAG, "No tracking data yet.");
            trackingData = new CopyOnWriteArrayList<JSONObject>();
            origin = PSUtil.getRandomIntInRange(1, PSConstants.TRACK_ORIGIN_MAX);
            Log.i(Constants.TAG, "Creating new origin:" + origin);
        }
    }

    public int getVersionCode() {
        if (versionCode == 0) {
            try {
                versionCode = context.getPackageManager().getPackageInfo(
                        context.getPackageName(), 0).versionCode;
            } catch (Exception e) {
                versionCode = 0;
            }
        }
        return versionCode;
    }

    public boolean save() {
        try {
            JSONObject trackingInfo = new JSONObject();
            Iterator<JSONObject> iter = trackingData.iterator();
            JSONArray arr = new JSONArray();
            while (iter.hasNext()) {
                arr.put(iter.next().getJSONObject(DATA));
            }
            trackingInfo.put("data", arr);
            trackingInfo.put("lastSync", lastSync);
            trackingInfo.put("origin", origin);
            trackingInfo.put("trackingIndex", trackingIndex);
            GameSpecific.saveToLocalStorage(Constants.NATIVE_TRACK_FILE,
                    trackingInfo.toString());
            // Log.i(Constants.TAG, "Saved tracking:" +
            // trackingInfo.toString());
        } catch (Exception e) {
            Analytics.logException(e);
            return false;
        }
        return true;
    }

    public static void trackCounterCpp(String k, String p, String c, String o,
                                       String f, String g, String s, String co, String oppId) {
        if (c.startsWith("cocos2d: fullPathForFilename:") || c.startsWith("DEBUG::") || c.startsWith("TRACE::")
                || c.startsWith("INFO::") || c.startsWith("WARNING::") || c.startsWith("ERROR::")
                || c.startsWith("\n") || c.startsWith("Inner ") || c.startsWith("Evaluating ") || c.startsWith("offest ")
                || c.startsWith("ScriptingCore:: evaluateScript fail:") || c.startsWith("In the constructor of")
                || c.startsWith("Number of") || c.startsWith("cocos2d: SpriteBatchNode:") || c.startsWith("jsbindings: finalizing JS")
                || c.startsWith("debug info:") || c.startsWith("basic_object_finalize") || c.startsWith("[WebSocket::init]")
                || c.startsWith("In the") || c.startsWith("create rendererRecreatedListener for GLProgramState") || (c.startsWith("cocos2d: warning: Uniform not found: depth") && uniformError == 1) || c.startsWith("cocos2d: ERROR: Failed to link program")
                || c.startsWith("cocos2d: QuadCommand: resizing index") || c.startsWith("Cocos2d-JS v3.")
                || c.startsWith("JS: <no filename=\"filename\">") || c.startsWith("reload all texture")
                || c.startsWith("Dirty Uniform and Attributes of GLProgramState") || c.startsWith("Response failed, error buffer")
                || c.startsWith("if the index doesn") || c.startsWith("touchesBegan") || c.startsWith("touchesEnded") || c.startsWith("OpenGL") || c.startsWith("debug server") || c.startsWith("cocos_audioengine_focus_change") || c.startsWith("cocos2d: SpriteFrameCache:") || c.startsWith("touchesMoved")
                || c.startsWith("cocos2d: unsupported image format!")) {
            // Log.i(Constants.TAG, "Dropping track counter");
            if (c.startsWith("Response failed, error buffer")) {
                GameSpecific.sendCallBack("utilObj.stopSyncs", "");
            }
        } else {
            if (c.startsWith("cocos2d: warning: Uniform not found: depth")) {
                uniformError = 1;
            }
            // Log.i(Constants.TAG, "Track counter cpp:" + c);
            // Log.i(Constants.TAG, "Not Dropping track counter");
            int index = c.indexOf("assets/");
            if (index != -1) {
                c = c.substring(index + 7);
            }
            trackCounter(k, p, c, o, f, g, s, co, oppId);
            int versionCode = PSUtil.getVersionCode();
            if (versionCode%2 == 0) {
                Analytics.syncFabricEvent("jsError", PSUtil.getBuildVersion() + " " + c);
            } else {
                Analytics.syncFabricEvent("jsErrorStaging", PSUtil.getBuildVersion() + " " + c);
            }

            try{
                int shouldShowDebugOptions = Integer.parseInt(Experiment.getChosenVariant("showJSPopup"));
                if (shouldShowDebugOptions == 1){
                    PSUtil.showAlert("JS Error", c);
                }
            } catch (Exception e){
            }
        }
    }

    private static String sanitizeTrackingFields(String field) {
        field = field.replace(":", "-");
        field = field.replace("\n", "");
        return field;
    }

    public static void trackCounter(String k, String p, String c, String o,
                                    String f, String g, String s, String co, String oppId) {
        try {
            if (k == null || k == "") {
                return;
            }
            if (p == null) {
                p = "";
            }
            if (c == null) {
                c = "";
            }
            if (o == null) {
                o = "";
            }
            if (f == null) {
                f = "";
            }
            if (g == null) {
                g = "";
            }
            if (s == null) {
                s = "";
            }
            if (co == null) {
                co = "0";
            }
            if (oppId == null) {
                oppId = "";
            }

            p = sanitizeTrackingFields(p);
            c = sanitizeTrackingFields(c);
            o = sanitizeTrackingFields(o);
            f = sanitizeTrackingFields(f);
            g = sanitizeTrackingFields(g);
            s = sanitizeTrackingFields(s);
            co = sanitizeTrackingFields(co);
            oppId = sanitizeTrackingFields(oppId);

            String sep = ":";
            final String msg = k + sep + p + sep + c + sep + o + sep + f + sep + g + sep + s + sep + co + sep + oppId;
            long curTimestamp = PSUtil.getCurrentTimestamp();
            String msgWithTs = msg + sep + curTimestamp;
            if (trackingMap.get(msgWithTs) != null) {
                Log.i(Constants.TAG, "Not Sending back:" + msg);
                return;
            }
            trackingMap.put(msgWithTs, true);
            Log.i(Constants.TAG, "Sending back:" + msg);
            if (!GameSpecific.platformInitCompleted()) {
                new Timer().schedule(new TimerTask() {
                    @Override
                    public void run() {
                        GameSpecific.sendCallBack("trackObj.processNativeTracking", msg);
                    }
                }, 20000);
            } else {
                GameSpecific.sendCallBack("trackObj.processNativeTracking", msg);
            }
        } catch (Exception e) {
            Analytics.logException(e);
        }
    }

    public static void trackCounterNative(String k, String p, String c, String o,
                                          String f, String g, String s, String co, String oppId) {
        Log.i(Constants.TAG, "this is force update tracking " + k + " " + p + " " + c + " " + o + " " + f + " " + g + " " + s + " " + co + " " + oppId);
        if (GameSpecific.platformInitCompleted()) {
            trackCounter(k, p, c, o, f, g, s, co, oppId);
            return;
        } else {
            trackCounterImmediate(k, p, c, o, f, g, s, co, oppId);
            return;
        }
    }

    public static void trackCounterImmediate(String k, String p, String c, String o,
                                             String f, String g, String s, String co, String oppId) {
        try {

            String sep = ":";
            final String msg = k + sep + p + sep + c + sep + o + sep + f + sep + g + sep + s + sep + co + sep + oppId;
            long curTimestamp = PSUtil.getCurrentTimestamp();
            String msgWithTs = msg + sep + curTimestamp;
            if (trackingMap.get(msgWithTs) != null) {
                Log.i(Constants.TAG, "Not Sending back:" + msg);
                return;
            }
            trackingMap.put(msgWithTs, true);

            String controller = "track";
            String action = "sync";
            HashMap<String, String> map = new HashMap<>();
            JSONObject data = new JSONObject();
            data.put("c", controller);
            data.put("a", action);

            String rid = "";
            String gps = "";
            JSONObject info = new JSONObject();

            try {
                user = User.get();
                rid = user.getTrackingRefId() + "";
                gps = user.getPsId() + "";
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (k.equals((PSConstants.TRACK_FIRST_LAUNCH)) && rid.isEmpty()) {
                rid = "dummy_refid";
            } else if (rid.isEmpty()) {
                return;
            }
            String trackFieldSeparator = ":";
            String trackingSeparator = "^";
            info.put("tid", "1");
            info.put("gid", Constants.GAME_ID + "");
            info.put("cli", Constants.CLIENT + "");
            info.put("or", "1");
            info.put("rid", rid);
            info.put("gps", gps);
            info.put("tr", rid + trackFieldSeparator + k + trackFieldSeparator + p + trackFieldSeparator
                    + c + trackFieldSeparator + o + trackFieldSeparator + f + trackFieldSeparator
                    + g + trackFieldSeparator
                    + s + trackFieldSeparator
                    + PSUtil.getVersionCode() + trackFieldSeparator
                    + curTimestamp + trackFieldSeparator + oppId + trackFieldSeparator + co + trackFieldSeparator
                    + trackingSeparator);


            data.put("d", info);
            map.put("data", data.toString());

            trackController = new Controller(PSConstants.TRACKING_SERVER);
            trackController.sendHttpRequest(controller, action, map);
        } catch (Exception e) {
            Analytics.logException(e);
        }
    }

    private static int getNextTrackingIndex() {
        trackingIndex += 1;
        trackingIndex = trackingIndex % PSConstants.TRACK_MOD_VAL;
        return trackingIndex;
    }

    public static void gameLoadTracking() {
        if (User.isFirstLaunch) {
            Track.trackCounterImmediate(PSConstants.TRACK_FIRST_LAUNCH, PSConstants.TRACK_SCREEN_LOADED, "", "", PSUtil.getCurrentTimestamp() + "",
                    PSUtil.getOsVersion(), "", "", "");
        }
    }

    public static void respHandler(String action, JSONObject data,
                                   boolean success) {

        if (action.equals("sync")) {

        } else {
            Log.i(Constants.TAG, "Unknown action received from server.Action:"
                    + action + ";Data:" + data);
        }
    }

    public void syncPendingTracking(boolean fromServerAck, boolean force) {
        return;
    }

    public void syncTracking() {
        syncPendingTracking(false, false);
    }

    static String lastNJSTracking = "";
    static int lastNJSTrackingCount;
    public static void loadJsTracking() {
        try {
            String trackData = GameSpecific.loadFromLocalStorage(Constants.JS_TRACK_FILE);
            JSONObject trackingInfo = new JSONObject(trackData);
            if (trackingInfo.has("lastNTracking")) {
                lastNJSTracking = trackingInfo.getString("lastNTracking");
            }
            if (trackingInfo.has("lastNTrackingCount")) {
                lastNJSTrackingCount = trackingInfo.getInt("lastNTrackingCount");
            }
        } catch (Exception e) {
            Log.i(Constants.TAG, "No tracking data yet - js.");
        }
    }

    public static String getLastNJsTracking() {
        loadJsTracking();
        return lastNJSTracking;
    }

}
