package in.playsimple.common;

import android.content.Context;
import android.util.Log;

import org.json.JSONObject;

import java.util.HashMap;

import in.playsimple.Constants;
import in.playsimple.GameSpecific;
import in.playsimple.common.flutter.FlutterBridge;

public class User {
	private static Context context = null;
	private static User user = null;

	private String refId = "";
	private String syncId = "";
	private String trackingRefId = "";
	private String gpsid = "";
	private String xpromoId = "";
	private String name = "";
	private String email = "";
	private String gender = "";
	private String fbid = "";
	private String isPayer = "0";

	private String curDevice = "";
	private String deviceId = "";
	private String deviceHash = "";
	private String infoHash = "";
	private String gameUserInfoHash = "";
	private String xpromoHash = "";
	private String created = "";
	private String chosenName = "";
	private String chosenEmail = "";

	public static boolean isFirstLaunch = false;

	private User() {
	}

	public static User get() throws Exception {
		if (User.context == null) {
			throw new Exception(
					"User context must be initialized before asking for user object.");
		}
		if (User.user == null) {
			user = new User();
		}
		user.load(); // Reloading this just in case some data has been updated
		// from js
		return user;
	}

	public static void setContext(Context context) {
		User.context = context;
	}

	private boolean load() {
		try {
			String userData = GameSpecific.loadFromLocalStorage(Constants.USER_DATA_FILE);
			if (userData == null) {
				isFirstLaunch = true;
			}
			JSONObject userObj = new JSONObject(userData);
			this.refId = userObj.getString("refId");	
			this.name = userObj.getString("name");
			this.email = userObj.getString("email");
			this.gender = userObj.getString("gender");
			this.fbid = userObj.getString("fbid");
			this.curDevice = userObj.getString("curDevice");
			if (userObj.has("deviceId")) {
				this.deviceId = userObj.getString("deviceId");	
			}
			
			this.deviceHash = userObj.getString("deviceHash");
			this.infoHash = userObj.getString("infoHash");
			this.created = userObj.getString("created");
			if (userObj.has("gi")) {
				this.gameUserInfoHash = userObj.getString("gi");	
			}
			if (userObj.has("xh")) {
				this.xpromoHash = userObj.getString("xh");
			}

			if (userObj.has("trackingRefId")) {
				this.trackingRefId = userObj.getString("trackingRefId");
			} else {
				this.trackingRefId = this.refId;
			}
			if (userObj.has("chosenName")) {
				this.chosenName = userObj.getString("chosenName");
			}
			if (userObj.has("chosenEmail")) {
				this.chosenEmail = userObj.getString("chosenEmail");
			}
			if (userObj.has("isPayer")) {
				this.isPayer = userObj.getString("isPayer");
			}
			if (userObj.has("syncId")) {
				this.syncId = userObj.getString("syncId");	
			} else {
				this.syncId = this.refId;
			}
			if (userObj.has("xpromoId")) {
				this.xpromoId = userObj.getString("xpromoId");	
			} else {
				this.xpromoId = this.refId;
			}
			if (userObj.has("gpsid")) {
				this.gpsid = userObj.getString("gpsid");	
			} else {
				this.gpsid = this.trackingRefId;
			}
		} catch (Exception e) {
			// Analytics.logException(e);
			Log.i(Constants.TAG, "No data for user yet.");
			return false;
		}
		return true;
	}

	public String getRefId() {
		if (syncId == null || syncId == "") {
			load();
		}
		return syncId;
	}

	public String getPsId() {
		if (refId == null || refId == "") {
			load();
		}
		return refId;
	}

	public String getSyncId() {
		if (syncId == null || syncId == "") {
			load();
		}
		return syncId;
	}

	public String getTrackingRefId() {
		if (trackingRefId == null || trackingRefId == "") {
			load();
		}
		return trackingRefId;
	}

	public String getXpromoId() {
		if (xpromoId == null || xpromoId == "") {
			load();
		}
		return xpromoId;
	}

	public String getName() {
		return name;
	}

	public String getEmail() {
		return email;
	}

	public String getGender() {
		return gender;
	}

	public String getFbid() {
		return fbid;
	}

	public String getCurDevice() {
		return curDevice;
	}

	public String getDeviceId() {
		return deviceId;
	}

	public String getDeviceHash() {
		return deviceHash;
	}

	public String getInfoHash() {
		return infoHash;
	}

	public String getCreated() {
		return created;
	}

	public String getIsPayer() {
		return isPayer;
	}

	public String getGameUserInfoHash() {
		return gameUserInfoHash;
	}

	public String getXpromoHash() {
		return xpromoHash;
	}

	public static String getPurchaseToken(int quantity) {
		String refId = "unknown";
		if (user != null) {
			refId = user.getRefId();
		}
		return refId + "_" + quantity + "_" + PSUtil.getCurrentTimestamp();
	}

	public static HashMap<String, String> getBaseMap() {
		HashMap<String, String> map = new HashMap<String, String>();
		String fbAccessToken = FacebookWrapper.getAccessToken();
		map.put("refId", user.getPsId());
		map.put("syncId", user.getSyncId());
		map.put("xpromoId", user.getXpromoId());
		map.put("trackingRefId", user.getTrackingRefId());
		map.put("name", user.getName());
		map.put("email", user.getEmail());
		map.put("gender", user.getGender());
		map.put("fbid", user.getFbid());
		map.put("fbAccessToken", fbAccessToken);
		map.put("curDevice", user.getCurDevice());
		map.put("deviceId", user.getDeviceId());
		map.put("infoHash", user.getInfoHash());
		map.put("deviceHash", user.getDeviceHash());
		map.put("gi", user.getGameUserInfoHash());
		map.put("xh", user.getXpromoHash());
		map.put("createdAt", user.getCreated());
		return map;
	}

	public String getChosenName() {
		return chosenName;
	}

	public String getChosenEmail() {
		return chosenEmail;
	}

	public void updateNameEmail(String name, String email) {
		chosenName = name;
		chosenEmail = email;

		JSONObject jObj = new JSONObject();
		try {
			jObj.put("cn", chosenName);
			jObj.put("ce", chosenEmail);
		} catch (Exception e) {

		}
		GameSpecific.sendCallBack("userObj.updateNameEmail", jObj.toString());
	}

	public static void showPrivacy(String theme, String refId, String trackingRefId, String countryCode, String subDivision, String law) {
		JSONObject flutterData = new JSONObject();
		try {
			flutterData.put("route", "privacy");
			flutterData.put("theme", theme);
			flutterData.put("r", refId);
			flutterData.put("tr", trackingRefId);
			flutterData.put("g", Constants.GAME_ID + "");
			flutterData.put("c", Constants.CLIENT + "");
			flutterData.put("cc", countryCode);
			flutterData.put("su", subDivision);
			flutterData.put("law", law);
		} catch (Exception e) {

		}
		FlutterBridge.sendDataToFlutterModule(flutterData.toString(), true, true);
	}

	public void updateChosenCountry(String chosenCountry) {
		JSONObject jObj = new JSONObject();
		try {
			jObj.put("cc", chosenCountry);
		} catch (Exception e) {

		}
		GameSpecific.sendCallBack("userObj.updateChosenCountry", jObj.toString());
	}

	public void updateAgeRange(String cmin, String cmax) {
		JSONObject jObj = new JSONObject();
		try {
			jObj.put("cmin", cmin);
			jObj.put("cmax", cmax);
		} catch (Exception e) {

		}
		GameSpecific.sendCallBack("userObj.updateAgeRange", jObj.toString());
	}

	public void acceptPrivacyPolicy() {
		JSONObject jObj = new JSONObject();
		GameSpecific.sendCallBack("userObj.acceptPrivacyPolicy", jObj.toString());
	}
}