package in.playsimple.common.admob;

import android.app.Activity;
import android.util.DisplayMetrics;
import android.util.Log;
import android.util.TypedValue;
import android.view.Display;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;

import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.AdView;

import org.json.JSONObject;

import java.util.HashMap;

import in.playsimple.AdsGameSpecific;
import in.playsimple.Constants;
import in.playsimple.common.AdUnit;
import in.playsimple.common.PSConstants;
import in.playsimple.common.Track;
import in.playsimple.common.User;
import in.playsimple.common.flutter.FlutterBridge;
import in.playsimple.common.mopub.PSMopubAds;

public class PSAdmobBanners {
    private static Activity activity;

    private static AdView adView;
    private static boolean isBannerLoaded = false;
    private static boolean isInitDone = false;
    private static boolean loadInProgress = false;
    private static boolean shouldShow = false;
    private static boolean adSizeSet = false;

    private static String ADMOB_BANNER_PLACEMENT = "";
    private static int admobBannerVariant = -1;
    private static HashMap<String, AdUnit> adUnitsMap = new HashMap<>();
    private static HashMap<String, String> adUnitToPlacement = new HashMap<>();
    public static void setActivity(Activity act) {
        activity = act;
    }
    public static void initAdUnit(AdUnit adUnit) {
        adUnitsMap.put(adUnit.getName(), adUnit);
    }

    public static void init() {
        updateBannerPlacement();
        Log.i(Constants.TAG, "admob log: initing for banners");

        adView = new AdView(activity);

        RelativeLayout relLayout = new RelativeLayout(activity);
        FrameLayout frameLayout = AdsGameSpecific.getFrameLayout();
        frameLayout.addView(relLayout);

        RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.FILL_PARENT,
                (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 50, activity.getResources().getDisplayMetrics()));

        if (AdsGameSpecific.getIsBannerAtTop()) {
            params.addRule(RelativeLayout.ALIGN_PARENT_TOP);
        } else {
            params.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
        }
        params.addRule(RelativeLayout.CENTER_HORIZONTAL);

        relLayout.addView(adView, params);

        adView.setAdUnitId(ADMOB_BANNER_PLACEMENT);

        adView.setAdListener(new AdListener() {
            @Override
            public void onAdLoaded() {
                // Code to be executed when an ad finishes loading.
                Log.d(Constants.TAG, "admob log: banner: banner loaded: id " + ADMOB_BANNER_PLACEMENT);
                loadInProgress = false;
                isBannerLoaded = true;
                AdUnit adUnit = getAdUnitForAdUnitId(ADMOB_BANNER_PLACEMENT);
//                PSMopubAds.updatePlacementLoadStatus(isBannerLoaded, adUnit);
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_SUCCESS);
            }

            @Override
            public void onAdFailedToLoad(int errorCode) {
                // Code to be executed when an ad request fails.
                Log.d(Constants.TAG, "admob log: banner: banner failed, " + errorCode + " - " + ADMOB_BANNER_PLACEMENT);
                loadInProgress = false;
                isBannerLoaded = false;
                AdUnit adUnit = getAdUnitForAdUnitId(ADMOB_BANNER_PLACEMENT);
                //PSMopubAds.updatePlacementLoadStatus(isBannerLoaded, adUnit);
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_FAIL);
            }

            @Override
            public void onAdOpened() {
                // Code to be executed when an ad opens an overlay that
                // covers the screen.
                Log.d(Constants.TAG, "admob log: banner: banner onAdOpened: id " + ADMOB_BANNER_PLACEMENT);
            }

            @Override
            public void onAdClicked() {
                // Code to be executed when the user clicks on an ad.
                Log.d(Constants.TAG, "admob log: banner: banner onAdClicked: id " + ADMOB_BANNER_PLACEMENT);
                AdUnit adUnit = getAdUnitForAdUnitId(ADMOB_BANNER_PLACEMENT);
                PSMopubAds.updateForAdClick(adUnit);
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_CLICK);
            }

            @Override
            public void onAdLeftApplication() {
                // Code to be executed when the user has left the app.
                Log.d(Constants.TAG, "admob log: banner: banner onAdLeftApplication: id " + ADMOB_BANNER_PLACEMENT);
            }

            @Override
            public void onAdClosed() {
                // Code to be executed when the user is about to return
                // to the app after tapping on an ad.
                Log.d(Constants.TAG, "admob log: banner: banner onAdClosed: id " + ADMOB_BANNER_PLACEMENT);
            }
        });

        adView.setOnPaidEventListener(adValue -> {
            Track.trackCounter(PSConstants.TRACK_ADMOB_ILRD,  PSConstants.TRACK_BANNER, adView.getResponseInfo().getMediationAdapterClassName(),  adValue.getCurrencyCode(), adValue.getPrecisionType() + "", PSMopubAds.screen,adValue.getValueMicros() + "", "", "");

            Log.d("GoogleAds", String.format(
                 "Paid event of value %d in currency %s of precision %s%n from ad network %s.",
                 adValue.getValueMicros(),
                 adValue.getCurrencyCode(),
                 adValue.getPrecisionType(),
                 adView.getResponseInfo().getMediationAdapterClassName())
                 );
        });

        isInitDone = true;
        JSONObject msgInfo = PSMopubAds.getBaseParamsForDartCall("initComplete", null);
        try {
            msgInfo.put("mType", PSMopubAds.MOPUB_AD_TYPE_BANNER);
            msgInfo.put("isPayer", User.get().getIsPayer());
            msgInfo.put("variant", 0);
        } catch (Exception e) {
        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    private static void updateBannerPlacement() {
        ADMOB_BANNER_PLACEMENT = AdsGameSpecific.getAdmobBannerId();
    }

    public static AdUnit getAdUnitForAdUnitId(String adUnitId) {
        try {
            String placement = adUnitToPlacement.get(adUnitId);
            AdUnit adUnit = adUnitsMap.get(placement);
            return adUnit;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static boolean load() {
        Log.d(Constants.TAG, "admob log: banner: calling to load banner");
        AdUnit adUnit = getAdUnitForAdUnitId(ADMOB_BANNER_PLACEMENT);
        if (adView == null) {
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_REQUEST_FAIL);
            Log.d(Constants.TAG, "admob log: banner: admob view is null, so not loading");
            return false;
        }

        activity.runOnUiThread(() -> {
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_REQUEST);
            loadInProgress = true;

            if (!adSizeSet) {
                AdSize adSize = getAdSize();
                adView.setAdSize(adSize);
                adSizeSet = true;
            }
            AdRequest adRequest = new AdRequest.Builder().build();
            adView.loadAd(adRequest);
            if (!shouldShow) {
                adView.setVisibility(View.GONE);
            }

        });
        return true;
    }

    public static void updateAdUnit(AdUnit adUnit) {
        if (!isInitDone) {
            return;
        }
        adUnitToPlacement.put(adUnit.getAdUnitId(), adUnit.getName());
        adUnitsMap.put(adUnit.getName(), adUnit);
        updateBannerPlacement();

        Log.d(Constants.TAG, "admob log: banner: update ad unit");
        load();
    }

    public static void setShouldShowPlacement(boolean val) {
        shouldShow = val;
    }

    public static boolean showPlacement(String screen, String puzzleInfo) {
        Log.d(Constants.TAG, "admob log: banner: coming here to show banner ad");
        final String oldScreen = PSMopubAds.screen;
        PSMopubAds.screen = screen;
        PSMopubAds.puzzleInfo = puzzleInfo;
        setShouldShowPlacement(true);
        if (loadInProgress) {
            Log.i(Constants.TAG, "admob log: banner: load already in progress");
            return false;
        }
        if (!isBannerLoaded) {
            return getBids(0);
        } else if (adView != null) {
            activity.runOnUiThread(() -> {
                adView.setVisibility(View.VISIBLE);
                if (!oldScreen.equals(screen)) {
                    PSMopubAds.sendAdTrackingWithAdUnitId(getAdUnitForAdUnitId(ADMOB_BANNER_PLACEMENT), PSConstants.TRACK_VIEW);
                }
            });
            return true;
        }
        return false;
    }

    public static boolean hidePlacement(String screen, String puzzleInfo) {
        Log.d(Constants.TAG, "admob log: banner: coming here to hide banner ad");
        PSMopubAds.screen = screen;
        PSMopubAds.puzzleInfo = puzzleInfo;
        setShouldShowPlacement(false);
        if (adView != null) {
            activity.runOnUiThread(() -> {
                adView.setVisibility(View.GONE);
                PSMopubAds.sendAdTrackingWithAdUnitId(getAdUnitForAdUnitId(ADMOB_BANNER_PLACEMENT), PSConstants.TRACK_HIDDEN);
            });
            return true;
        }
        // This means banner is already hidden.
        return true;
    }

    public static HashMap<String, AdUnit> getAdUnitsMap() {
        return adUnitsMap;
    }

    public static boolean getBids(int psAdType) {
        if (adView == null) {
            Log.i(Constants.TAG, "admob log: banner: banner view is null");
            return false;
        }
        load();
        return true;
    }

    private static AdSize getAdSize() {
        // Step 2 - Determine the screen width (less decorations) to use for the ad width.
        Display display = activity.getWindowManager().getDefaultDisplay();
        DisplayMetrics outMetrics = new DisplayMetrics();
        display.getMetrics(outMetrics);

        float widthPixels = outMetrics.widthPixels;
        float density = outMetrics.density;

        int adWidth = (int) (widthPixels / density);

        // Step 3 - Get adaptive ad size and return for setting on the ad view.
        return AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(activity, adWidth);
    }

    public static int getAdmobBannerVariant() {
        if (admobBannerVariant != -1) {
            return admobBannerVariant;
        }
        return admobBannerVariant;
//
//        int variant = PSConstants.EXP_VAR_CONTROL;
//        String var = Experiment.getChosenVariant(AdsGameSpecific.EXP_ADMOB_BANNERS);
//        try {
//            variant = Integer.parseInt(var);
//        } catch (Exception e) {
//            switch (var) {
//                case "control":
//                    variant = PSConstants.EXP_VAR_CONTROL;
//                    break;
//                case "var1":
//                    variant = PSConstants.EXP_VAR_1;
//                    break;
//                case "var2":
//                    variant = PSConstants.EXP_VAR_2;
//                    break;
//                default:
//                    break;
//            }
//        }
//        admobBannerVariant = variant;
//        return variant;
    }


    public static boolean areAdmobBannersEnabled() {
//        int variant = PSAdmobBanners.getAdmobBannerVariant();
//        if (variant != PSConstants.EXP_VAR_CONTROL) {
//            Log.d(Constants.TAG, "admob log: areAdmobBannersEnabled true");
//            return true;
//        }
//        Log.d(Constants.TAG, "admob log: areAdmobBannersEnabled false");
        return false;
    }

    public static void onPause() {
        if (adView != null) {
            adView.pause();
        }
    }

    public static void onResume() {
        if (adView != null) {
            adView.resume();
        }
    }

    public static void onDestroy() {
        if (adView != null) {
            adView.destroy();
        }
    }
}
