package in.playsimple.common.experimental;

import android.app.Activity;
import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;

import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import in.playsimple.GameSpecific;
import in.playsimple.common.Analytics;
import in.playsimple.common.PSConstants;
import in.playsimple.common.PSUtil;

public class AdIntelligence {
    private static Activity activity = null;
    private static Context context = null;
    private static JSONObject adData = null;
    private static final String skipDirsStr = "lib,code_cache,no_backup,shared_prefs,databases,app_fiverocks,app_webview,app_textures,app_download_internal,org.chromium.android_webview,volley,.Fabric,WebView";
    static String[] skipDirs;

    public static void init (Activity act) {
        setActivity(act);
        try {
            String rawAdData = GameSpecific.loadFromLocalStorage(PSConstants.AD_INTELLIGENCE_FILE);
            adData = new JSONObject(rawAdData);
        } catch (Exception e) {
            // Analytics.logException(e);
            // Log.i(PSConstants.TAG, "No data for ad intel yet.");
        }
        skipDirs = AdIntelligence.skipDirsStr.split(",");
    }

    public static void setActivity(Activity act) {
        activity = act;
        context = act;
    }

    public static void listParentDirectories() {
        File f = new File(PSUtil.getBaseDirPath());
        File[] files = f.listFiles();
        for (File inFile : files) {
            if (inFile.isDirectory()) {
                // is directory
                Log.i(PSConstants.TAG, "Directory:" + inFile);
            } else {
                Log.i(PSConstants.TAG, "Not Directory:" + inFile);
            }
        }
    }

    public static void listAllFiles() {
        ListFiles lf = new ListFiles();
        lf.execute();
    }

    public static void markAdUploaded(String filePath) {
        try {
            if (adData == null) {
                adData = new JSONObject();
            }
            adData.put(filePath, true);
            GameSpecific.saveToLocalStorage(PSConstants.AD_INTELLIGENCE_FILE,
                    adData.toString());
            // Log.i(PSConstants.TAG, "Saved ad intel info:" + adData.toString());
        } catch (Exception e) {
            Analytics.logException(e);
        }
    }

    public static String uploadFileToServer(File file) {
        try {
            String filePath = file.getAbsolutePath();
            String[] filePathArray = filePath.split("/");
            int dirCount = filePathArray.length;
            String fileName = filePathArray[dirCount - 1];
            String filePathConcat = "";

            // Look into local storage for match
            if (adData != null && adData.has(fileName)) {
                // Log.i(PSConstants.TAG, "This ad has already been uploaded: " + fileName);
                return null;
            }

            // If file name has a .tmp extension, do not upload.
            if (fileName.endsWith(".tmp")) {
                // Log.i(PSConstants.TAG, "This is a temp file: " + fileName);
                return null;
            }

            byte[] fileData = AdIntelligence.readFile(file);
            // Log.i(PSConstants.TAG, "File size:" + file + ";" + fileData.length);
            if (fileData == null || fileData.length <= 0) {
                return null;
            }

            for (int i = dirCount - 2; i >= 0; i--) {
                String dirName = filePathArray[i];
                if (dirName.equals(PSConstants.PACKAGE_NAME)) {
                    break;
                }
                filePathConcat = dirName.replace(".", "").concat(":" + filePathConcat);
            }

            filePathConcat = "android:".concat(filePathConcat);

            // Log.i(PSConstants.TAG, "File info:" + fileName + ";" + filePathConcat);

            // Check if the file needs upload
            String checkUrl = PSConstants.AD_INTEL_SERVER + "check/" + fileName + "/" + filePathConcat;
            URL url = new URL(checkUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            DataOutputStream dos = null;
            conn.setDoInput(true); // Allow Inputs
            conn.setDoOutput(true); // Allow Outputs
            conn.setUseCaches(false); // Don't use a Cached Copy
            conn.setRequestMethod("GET");
            conn.setRequestProperty("Connection", "Keep-Alive");
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            dos = new DataOutputStream(conn.getOutputStream());
            dos.write(1);
            int serverResponseCode = conn.getResponseCode();
            StringBuffer response = new StringBuffer();
            if (serverResponseCode == 200) {
                BufferedReader in = new BufferedReader(new InputStreamReader(
                        conn.getInputStream()));
                String inputLine;

                while ((inputLine = in.readLine()) != null) {
                    response.append(inputLine);
                }
                in.close();
                if (response.toString().equals("1")) {
                    // Log.i(PSConstants.TAG, "This file has already been uploaded to server");
                    AdIntelligence.markAdUploaded(fileName);
                    return null;
                }
            } else {
                // Something odd about file name. Mark it as uploaded.
                AdIntelligence.markAdUploaded(fileName);
                return null;
            }
            // Log.i(PSConstants.TAG, "check response:" + serverResponseCode + ";" + response);

            String uploadUrl = PSConstants.AD_INTEL_SERVER + "upload/" + fileName + "/" + filePathConcat;
            url = new URL(uploadUrl);
            conn = (HttpURLConnection) url.openConnection();
            conn.setDoInput(true); // Allow Inputs
            conn.setDoOutput(true); // Allow Outputs
            conn.setUseCaches(false); // Don't use a Cached Copy
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Connection", "Keep-Alive");
            conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            dos = new DataOutputStream(conn.getOutputStream());
            dos.write(fileData);
            serverResponseCode = conn.getResponseCode();
            if (serverResponseCode == 200) {
                // Log.i(PSConstants.TAG, "Successfully uploaded");
                AdIntelligence.markAdUploaded(fileName);
            } else {
                Log.i(PSConstants.TAG, "Upload failed:" + fileName + ";" + filePathConcat);
            }
            dos.flush();
            dos.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static byte[] readFile(File file) throws IOException {
        // Open file
        RandomAccessFile f = new RandomAccessFile(file, "r");
        try {
            // Get and check length
            long longlength = f.length();
            int length = (int) longlength;
            if (length != longlength)
                throw new IOException("File size >= 2 GB");
            if (length > 10000000) {
                return null;
            }
            // Read file and return data
            byte[] data = new byte[length];
            f.readFully(data);
            return data;
        } finally {
            f.close();
        }
    }
}

class ListFiles extends AsyncTask<String, Void, Boolean> {
    private List<File> inFiles = new ArrayList<>();
    private Queue<File> files = new LinkedList<>();

    @Override
    protected void onPreExecute() {
        // lib,code_cache,no_backup,.Fabric
    }

    protected Boolean doInBackground(String... params) {
        File f = new File(PSUtil.getBaseDirPath());
        files.addAll(Arrays.asList(f.listFiles()));
        String externalBaseDir = PSUtil.getExternalBaseDirPath();
        if (externalBaseDir != null) {
            f = new File(externalBaseDir);
            files.addAll(Arrays.asList(f.listFiles()));
        }
        while (!files.isEmpty()) {
            File file = files.remove();
            String []subDirs = file.getAbsolutePath().split("/");
            String curDir = subDirs[subDirs.length - 1];
            if (file.isDirectory()) {
                boolean shouldSkip = false;
                for (int i = 0; i < AdIntelligence.skipDirs.length; i++) {
                    if (curDir.equals(AdIntelligence.skipDirs[i])) {
                        shouldSkip = true;
                        break;
                    }
                }
                if (!shouldSkip) {
                    files.addAll(Arrays.asList(file.listFiles()));
                    // Log.i(PSConstants.TAG, "Directory:" + file);
                }
            } else {
                // Upload files except in the files directory where we download facebook images and other files
                String prevDir = "";
                if (subDirs.length - 2 >= 0) {
                    prevDir = subDirs[subDirs.length - 2];
                }
                if (!prevDir.equals("files")) {
                    inFiles.add(file);
                    AdIntelligence.uploadFileToServer(file);
                }
            }
        }
        return true;
    }

    protected void onPostExecute(Boolean... result) {

    }
}
