package in.playsimple.common.flutter;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import in.playsimple.Constants;
import in.playsimple.GameSpecific;
import in.playsimple.common.PSUtil;
import in.playsimple.pspn.XPromoUtil;
import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.FlutterEngine;

public class FlutterViewActivity extends FlutterActivity {
    public static boolean isMovingToFlutter = false;
    private static String params;

    // This is the method where you provide your existing FlutterEngine instance.
    @Nullable
    @Override
    public FlutterEngine provideFlutterEngine(@NonNull Context context) {
        Log.i(FlutterBridge.TAG, "About to send to flutter provideFlutterEngine - ");
        return FlutterBridge.flutterEngine;
    }

    @Override
    public void configureFlutterEngine(@NonNull FlutterEngine flutterEngine) {
        Log.i(FlutterBridge.TAG, "About to send to flutter 1 - ");
        if (params != null && params != "") {
            FlutterBridge.sendDataToFlutterModule(params, false);
        }
    }

    public static void startActivity(Context context, String p) {
        params = p;
        Intent intent = new Intent(context, FlutterViewActivity.class);
        context.startActivity(intent);

        Log.i(FlutterBridge.TAG, "About to send to flutter - ");
    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.i(FlutterBridge.TAG, "About to send to flutter start - ");
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.i(FlutterBridge.TAG, "About to send to flutter resume - ");

        isMovingToFlutter = true;
        GameSpecific.sendCallBack("psUtilObj.flutterAppResume", "");
    }

    @Override
    protected void onPause() {
        super.onPause();
        Log.i(FlutterBridge.TAG, "About to send to flutter pause - ");

        GameSpecific.sendCallBack("psUtilObj.flutterAppPause", "");
    }

    @Override
    protected void onStop() {
        super.onStop();
        isMovingToFlutter = false;
        GameSpecific.onStop();
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        if (GameSpecific.shouldGameRestart()) {
            finish();
        } else {
            GameSpecific.setActivityBackgroundTime(0);
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        if (hasFocus) {
            hideSystemUI();
        }

    }

    private void hideSystemUI() {
        // Enables regular immersive mode.
        // For "lean back" mode, remove SYSTEM_UI_FLAG_IMMERSIVE.
        // Or for "sticky immersive," replace it with SYSTEM_UI_FLAG_IMMERSIVE_STICKY
        View decorView = getWindow().getDecorView();
        decorView.setSystemUiVisibility(
                View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                        // Set the content to appear under the system bars so that the
                        // content doesn't resize when the system bars hide and show.
                        | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                        | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                        // Hide the nav bar and status bar
                        | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_FULLSCREEN);
    }

}
