package in.playsimple.common.mopub;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;

import com.adjust.sdk.Adjust;
import com.adjust.sdk.AdjustConfig;
import com.google.firebase.analytics.FirebaseAnalytics;
import com.mopub.common.MoPub;
import com.mopub.common.SdkConfiguration;
import com.mopub.common.SdkInitializationListener;
import com.mopub.common.privacy.ConsentData;
import com.mopub.common.privacy.PersonalInfoManager;
import com.mopub.mobileads.GooglePlayServicesAdapterConfiguration;
import com.mopub.network.ImpressionListener;
import com.mopub.network.ImpressionsEmitter;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import in.playsimple.AdsGameSpecific;
import in.playsimple.Constants;
import in.playsimple.GameSpecific;
import in.playsimple.common.AdUnit;
import in.playsimple.common.Ads;
import in.playsimple.common.PSConstants;
import in.playsimple.common.PSUtil;
import in.playsimple.common.Track;
import in.playsimple.common.admob.PSAdmobBanners;
import in.playsimple.common.flutter.FlutterBridge;
import in.playsimple.common.inMobi.InMobiAdapterConfiguration;
import in.playsimple.common.inMobi.InMobiGDPR;
import in.playsimple.common.smaato.PSSmaato;
import io.flutter.plugin.common.MethodCall;

public class PSMopubAds {
    private static Activity activity;
    private static ImpressionListener mImpressionListener;

    private final static String FLUTTER_ROUTE_NAME = "mopub";

    public final static int PS_AD_TYPE_INTERSTITIAL = 1; // This is only interstitials, but could include Rewarded also in future.
    public final static int PS_AD_TYPE_BANNER = 2;
    public final static int PS_AD_TYPE_REWARDED = 3; // This is rewarded + video interstitials

    public final static int MOPUB_AD_TYPE_INTERSTITIAL = 1; // Interstitials as per Mopub classification
    public final static int MOPUB_AD_TYPE_BANNER = 2;
    public final static int MOPUB_AD_TYPE_REWARDED = 3; // Rewarded videos as per Mopub classification

    public final static String BANNER_PREFIX = "BANNER";

    public static String screen = "";
    public static String puzzleInfo = "";

    private static Boolean mopubInitComplete = false;
    private static Boolean dartUnitsComputed = false;
    private static Boolean consentToBeGranted = false;
    // This is a list of the adUnits in the sequence in which they're present in the various runtimes.
    // The adUnit info in the list will usually be stale in terms of the adUnitId being used - as this is only initialized at app start and never updated after that.
    private static ArrayList<AdUnit> adUnitsInSequence;

    public static void init(Activity activity) {
        if (!Constants.MOPUB_ADS_ENABLED) {
            return;
        }
        PSMopubAds.activity = activity;
        PSAdmobBanners.setActivity(activity);
        PSMopubBanners.setActivity(activity);
        PSMopubInterstitials.setActivity(activity);
        PSMopubRewardedVideos.setActivity(activity);
        AdsGameSpecific.initAdUnitMaps();
        initAdUnitSequence();

        // PSBidMachine.init(activity);

        HashMap<String, String> admobConfig = new HashMap<>();
        admobConfig.put("npa", "0");

        SdkConfiguration sdkConfiguration = new SdkConfiguration.Builder(Constants.MOPUB_VALID_AD_UNIT).
                withMediatedNetworkConfiguration(InMobiAdapterConfiguration.class.getName(), new HashMap<>()).
                withAdditionalNetwork(InMobiAdapterConfiguration.class.getName()).
                withMediatedNetworkConfiguration(GooglePlayServicesAdapterConfiguration.class.getName(), admobConfig).
//                withAdditionalNetwork(BidMachineAdapterConfiguration.class.getName()).
//                withMediatedNetworkConfiguration(BidMachineAdapterConfiguration.class.getName(), PSBidMachine.getBidMachineConfiguration()).
                withLegitimateInterestAllowed(true)
                //.withMediationSettings("MEDIATION_SETTINGS")
                //.withNetworksToInit(networksToInit)
                //.withLogLevel(MoPubLog.LogLevel.DEBUG)
                .build();
        MoPub.initializeSdk(activity, sdkConfiguration, initMopubSdkListener());
//        PSAmazonAds.init(activity);

        mImpressionListener = (adUnitId, impressionData) -> {
            // Log.d(Constants.TAG, "mopub log: impression data received:" + impressionData);
            if (impressionData == null) {
                return;
            }

            try {
                Track.trackCounter(PSConstants.TRACK_MOPUB_ILRD, impressionData.getAdGroupId(), impressionData.getAdGroupName(), impressionData.getAdUnitName(), impressionData.getAdGroupPriority() + "-" + impressionData.getPrecision(), PSMopubAds.screen, impressionData.getPublisherRevenue() + "", "", "");
                PSMopubAds.updateAdUnitAfterImpression(impressionData.getAdGroupPriority(), getAdUnitForAdUnitId(impressionData.getAdUnitId()));

                Adjust.trackAdRevenue(AdjustConfig.AD_REVENUE_MOPUB, impressionData.getJsonRepresentation());

                FirebaseAnalytics mFirebaseAnalytics = FirebaseAnalytics.getInstance(activity);
                Bundle params = new Bundle();
                params.putString("adunit_id", impressionData.getAdUnitId());
                params.putString("adunit_name", impressionData.getAdUnitName());
                params.putString("currency", impressionData.getCurrency());
                params.putString("publisher_revenue", impressionData.getPublisherRevenue() + "");
                params.putString("network_name", impressionData.getNetworkName());
                params.putString("precision", impressionData.getPrecision());

                mFirebaseAnalytics.logEvent("ad_impression_shown", params);

                Log.d(Constants.TAG, "mopub log: impression data received:" + impressionData.getAdGroupId() + " - " + impressionData.getAdGroupName() + " - " + impressionData.getAdUnitName() + " - " + impressionData.getAdGroupPriority() + "-" + impressionData.getPrecision() + " - " + PSMopubAds.screen + " - " + impressionData.getPublisherRevenue() + "");
            } catch (Exception e) {
                e.printStackTrace();
            }
        };
        ImpressionsEmitter.addListener(mImpressionListener);
    }

    public static SdkInitializationListener initMopubSdkListener() {
        return () -> {
            // MoPub SDK initialized.
            // Check if you should show the consent dialog here, and make your ad requests.
            Log.d(Constants.TAG, "mopub log: Ad SDK initialization finished");
            mopubInitComplete = true;
            checkAndInitAdUnits();
        };
    }

    public static void grantMopubConsent() {
        Log.d(Constants.TAG, "CCPA: DATA_PRIVACY: CONSENT: going to call grant consent: ");
        consentToBeGranted = true;
        if (mopubInitComplete) {
            grantAllConsents();
        }
    }

    private static void grantAllConsents() {
        if (!consentToBeGranted) {
            return;
        }
        PSMopubAds.grantConsent();
        // PSBidMachine.grantConsent();
        PSSmaato.grantConsent();
    }

    public static void grantConsent() {
        Log.d(Constants.TAG, "CCPA: DATA_PRIVACY: CONSENT: grant consent getting called on native");
        PersonalInfoManager mPersonalInfoManager = MoPub.getPersonalInformationManager();
        ConsentData consentData = mPersonalInfoManager.getConsentData();
        if (!MoPub.canCollectPersonalInformation()) {
            // Log.i(Constants.TAG, "MOPUBGDPR: grantConsent is called: " + mPersonalInfoManager.shouldShowConsentDialog() + "it is ");
            if (mPersonalInfoManager.shouldShowConsentDialog()) {
//                Log.i(Constants.TAG, "MOPUBGDPR: should show consent is true");
//                Log.i(Constants.TAG, "MOPUBGDPR:" + consentData.getCurrentVendorListLink());
//                Log.i(Constants.TAG, "MOPUBGDPR:" + consentData.getCurrentPrivacyPolicyLink());
                mPersonalInfoManager.grantConsent();
                InMobiGDPR.grantConsent();
            } else {
//                Log.i(Constants.TAG, "MOPUBGDPR: gdpr is false");
            }
        }
        updateCurrentConsentStatus(MoPub.canCollectPersonalInformation());
//        Log.i(Constants.TAG, "MOPUBGDPR: the current status is: " + MoPub.canCollectPersonalInformation() + ".");
    }

    private static void initAdUnitSequence() {
        HashMap<String, AdUnit> adUnits = PSMopubRewardedVideos.getAdUnitsMap();
        adUnits.putAll(PSMopubInterstitials.getAdUnitsMap());
        if (PSAdmobBanners.areAdmobBannersEnabled()) {
            adUnits.putAll(PSAdmobBanners.getAdUnitsMap());
        } else {
            adUnits.putAll(PSMopubBanners.getAdUnitsMap());
        }

        Iterator hmIterator = adUnits.entrySet().iterator();
        int numElements = adUnits.size();
        adUnitsInSequence = new ArrayList<>(numElements);
        for (int i = 0; i < numElements; i++) {
            adUnitsInSequence.add(AdUnit.createFromJSONStr(null));
        }

        while (hmIterator.hasNext()) {
            Map.Entry mapElement = (Map.Entry) hmIterator.next();

            AdUnit adUnit = ((AdUnit) mapElement.getValue());
            adUnitsInSequence.set(adUnit.getSequence(), adUnit);
        }
    }

    // mopubDartConfigLoaded - called from Dart code once initial config data is loaded.
    // Informs game code about the same.
    public static void mopubDartConfigLoaded() {
        Log.i(Constants.TAG, "mopub log: mopubDartConfigLoaded");
        AdsGameSpecific.mopubDartConfigLoaded();
    }

    // updateSessionInfo - This is called from the game code at the start of each session (after inits have been completed correctly)
    // reset - whether to reset the stored data for ad calculations (usually reset at the start of a new day)
    // variant - which variant is the user in for the adUnits experiment. This will determine the adUnits which get used.
    public static void updateSessionInfo(boolean reset, int variant) {
        Log.i(Constants.TAG, "mopub log: updateSessionInfo " + reset + " - " + variant);
        // Make the call to dart code - for computing initial adUnits.
        JSONObject msgInfo = getBaseParamsForDartCall("updateSessionInfo", null);
        try {
            msgInfo.put("reset", reset);
            msgInfo.put("variant", variant);
        } catch (Exception e) {

        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    // mopubDartUnitsComputed - called from Dart code once ad units computation is done.
    public static void mopubDartUnitsComputed(String adUnits) {
        Log.i(Constants.TAG, "mopub log: mopubDartUnitsComputed - " + adUnits);
        updateAdUnits(adUnits);
        dartUnitsComputed = true;
        checkAndInitAdUnits();
    }

    public static void updateAdUnits(String adUnits) {
        try {
            JSONArray adUnitsArray = new JSONArray(adUnits);
            int numElements = adUnitsArray.length();
            for (int i = 0; i < numElements; i++) {
                AdUnit adUnit = adUnitsInSequence.get(i);
                adUnit.setAdUnitId(adUnitsArray.getString(i));

                int mopubAdType = adUnit.getMopubAdType();
                switch (mopubAdType) {
                    case MOPUB_AD_TYPE_BANNER:
                        if (PSAdmobBanners.areAdmobBannersEnabled()) {
                            PSAdmobBanners.updateAdUnit(adUnit);
                        } else {
                            PSMopubBanners.updateAdUnit(adUnit);
                        }
                        break;
                    case MOPUB_AD_TYPE_INTERSTITIAL:
                        PSMopubInterstitials.updateAdUnit(adUnit);
                        break;
                    case MOPUB_AD_TYPE_REWARDED:
                        PSMopubRewardedVideos.updateAdUnit(adUnit);
                        break;
                    default:
                        break;
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            Log.i(Constants.TAG, "mopub log: updateAdUnits - Unable to parse as json - " + adUnits);
        }
    }

    public static void checkAndInitAdUnits() {
        Log.i(Constants.TAG, "mopub log: checkAndInitAdUnits " + dartUnitsComputed + " " + mopubInitComplete);
        if (dartUnitsComputed && mopubInitComplete) {
            activity.runOnUiThread(() -> {
                grantAllConsents();
                PSMopubBanners.init();
                PSMopubInterstitials.init();
                PSMopubRewardedVideos.init();
            });
        }
    }

    public static JSONObject getBaseParamsForDartCall(String action, AdUnit adUnit) {
        JSONObject msgInfo = new JSONObject();
        try {
            msgInfo.put("route", FLUTTER_ROUTE_NAME);
            msgInfo.put("action", action);
            msgInfo.put("isTablet", PSUtil.isTablet());
            if (adUnit != null) {
                msgInfo.put("adUnit", adUnit.getJSONObject());
            }
        } catch (Exception e) {

        }
        return msgInfo;
    }

    public static void updatePlacementLoadStatus(boolean success, AdUnit adUnit) {
        if (adUnit == null) {
            Track.trackCounter(PSConstants.TRACK_DEBUG, PSConstants.TRACK_AD_TRACKING, "placement_load_status", success + "", "ad_unit_null", "", "", "", "");
        }
        // Log.i(Constants.TAG, "mopub log: updatePlacementLoadStatus: " + adUnit.getName() + " - " + success);
        JSONObject msgInfo = getBaseParamsForDartCall("updatePlacementLoadStatus", adUnit);
        try {
            msgInfo.put("success", success);
        } catch (Exception e) {

        }
        if ( AdsGameSpecific.adLoadForced.containsKey(adUnit.getName())) {
            AdsGameSpecific.adLoadForced.put(adUnit.getName(), false);
        }
        AdsGameSpecific.updatePlacementLoadStatus(success, adUnit);
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    public static void updatePlacementStatus(boolean close, int count, AdUnit adUnit) {
        if (adUnit == null) {
            Track.trackCounter(PSConstants.TRACK_DEBUG, PSConstants.TRACK_AD_TRACKING, "placement_status", close + "", "ad_unit_null", "", "", "", "");
        }
        // Log.i(Constants.TAG, "mopub log: updatePlacementStatus: " + adUnit.getName());
        JSONObject msgInfo = getBaseParamsForDartCall("updatePlacementStatus", adUnit);
        try {
            msgInfo.put("close", close);
            msgInfo.put("count", count);
        } catch (Exception e) {

        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    public static void updateForAdClick(AdUnit adUnit) {
        // Log.i(Constants.TAG, "mopub log: updating after click: " + adUnit.getName());
        JSONObject msgInfo = getBaseParamsForDartCall("updateForAdClick", adUnit);
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    public static void updateAdUnitAfterImpression(int priority, AdUnit adUnit) {
        // Log.i(Constants.TAG, "mopub log: updating ad unit after impression: " + adUnit.getName());
        JSONObject msgInfo = getBaseParamsForDartCall("updateAdUnitAfterImpression", adUnit);
        try {
            msgInfo.put("priority", priority);
        } catch (Exception e) {

        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
        AdsGameSpecific.afterImpression(adUnit);
    }

    public static void updateForAdView(AdUnit adUnit) {
        // Log.i(Constants.TAG, "mopub log: updating after view: " + adUnit.getName());
        AdsGameSpecific.afterView(adUnit);
    }

    public static AdUnit getAdUnitForAdUnitId(String adUnitId) {
        AdUnit adUnit = PSMopubRewardedVideos.getAdUnitForAdUnitId(adUnitId);
        if (adUnit == null) {
            adUnit = PSMopubInterstitials.getAdUnitForAdUnitId(adUnitId);
            if (adUnit == null) {
                if (PSAdmobBanners.areAdmobBannersEnabled()) {
                    adUnit = PSAdmobBanners.getAdUnitForAdUnitId(adUnitId);
                } else {
                    adUnit = PSMopubBanners.getAdUnitForAdUnitId(adUnitId);
                }
            }
        }
        return adUnit;
    }

    public static AdUnit getAdUnitForPlacementName(String placementName) {
        HashMap<String, AdUnit> adUnits = PSMopubRewardedVideos.getAdUnitsMap();
        adUnits.putAll(PSMopubInterstitials.getAdUnitsMap());
        if (PSAdmobBanners.areAdmobBannersEnabled()) {
            adUnits.putAll(PSAdmobBanners.getAdUnitsMap());
        } else {
            adUnits.putAll(PSMopubBanners.getAdUnitsMap());
        }
        return adUnits.get(placementName);
    }

    public static boolean isPlacementAvailable(String placementName) {
        AdUnit adUnit = getAdUnitForPlacementName(placementName);
        if (adUnit == null) {
            return false;
        }
        boolean adUnitAvailable = false;
        switch (adUnit.getMopubAdType()) {
            case MOPUB_AD_TYPE_BANNER:
                adUnitAvailable = true;
                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
                adUnitAvailable = PSMopubInterstitials.isPlacementAvailable(placementName);
                break;
            case MOPUB_AD_TYPE_REWARDED:
                adUnitAvailable = PSMopubRewardedVideos.isPlacementAvailable(placementName);
                break;

        }
        return adUnitAvailable;
    }

    public static boolean isAnyPlacementAvailable(String colonSeparatedPlacementNames) {
        String[] placementNames = colonSeparatedPlacementNames.split(":");
        for (int i = 0; i < placementNames.length; i++) {
            if (isPlacementAvailable(placementNames[i])) {
                return true;
            }
        }
        return false;
    }

    public static boolean showAnyPlacementAvailable(String colonSeparatedPlacementNames, String screen, String puzzleInfo) {
        String[] placementNames = colonSeparatedPlacementNames.split(":");
        for (int i = 0; i < placementNames.length; i++) {
            if (isPlacementAvailable(placementNames[i])) {
                if (showPlacement(placementNames[i], screen, puzzleInfo)) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean showPlacement(String placementName, String screen, String puzzleInfo) {
        AdUnit adUnit = getAdUnitForPlacementName(placementName);
        if (adUnit == null) {
            return false;
        }

        boolean adShown = false;
        switch (adUnit.getMopubAdType()) {
            case MOPUB_AD_TYPE_BANNER:
                if (PSAdmobBanners.areAdmobBannersEnabled()) {
                    adShown = PSAdmobBanners.showPlacement(screen, puzzleInfo);
                } else {
                    adShown = PSMopubBanners.showPlacement(screen, puzzleInfo);
                }

                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
                adShown = PSMopubInterstitials.showPlacement(placementName, screen, puzzleInfo);
                break;
            case MOPUB_AD_TYPE_REWARDED:
                adShown = PSMopubRewardedVideos.showPlacement(placementName, screen, puzzleInfo);
                break;
        }
        return adShown;
    }

    public static boolean hidePlacement(String placementName, String screen, String puzzleInfo) {
        AdUnit adUnit = getAdUnitForPlacementName(placementName);
        if (adUnit == null) {
            // Since hidePlacement can only be done for banners.
            if (PSAdmobBanners.areAdmobBannersEnabled()) {
                PSAdmobBanners.setShouldShowPlacement(false);
            } else {
                PSMopubBanners.setShouldShowPlacement(false);
            }
            return false;
        }

        boolean adHidden = false;
        switch (adUnit.getMopubAdType()) {
            case MOPUB_AD_TYPE_BANNER:
                if (PSAdmobBanners.areAdmobBannersEnabled()) {
                    adHidden = PSAdmobBanners.hidePlacement(screen, puzzleInfo);
                } else {
                    adHidden = PSMopubBanners.hidePlacement(screen, puzzleInfo);
                }
                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
            case MOPUB_AD_TYPE_REWARDED:
                break;
        }
        return adHidden;
    }

    public static void updateMopubIdToUse(String placementName, String adUnitId, String trackC) {
        Log.i(Constants.TAG, "mopub log: mopub priority - setting for ad unit:" + placementName + " - " + adUnitId);

        AdUnit adUnit = getAdUnitForPlacementName(placementName);
        if (adUnit == null) {
            Log.i(Constants.TAG, "mopub log: mopub priority - no match found for adUnitName:" + placementName + " - " + adUnitId);
            return;
        }

        Track.trackCounter(PSConstants.TRACK_AD_TRACKING, adUnit.getTrackP(), trackC, "0", adUnit.getAdUnitId(), adUnit.getTrackG(), adUnitId, "", "");
        adUnit.setAdUnitId(adUnitId);
        switch (adUnit.getMopubAdType()) {
            case MOPUB_AD_TYPE_BANNER:
                if (PSAdmobBanners.areAdmobBannersEnabled()) {
                    PSAdmobBanners.updateAdUnit(adUnit);
                } else {
                    PSMopubBanners.updateAdUnit(adUnit);
                }
                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
                PSMopubInterstitials.updateAdUnit(adUnit);
                break;
            case MOPUB_AD_TYPE_REWARDED:
                PSMopubRewardedVideos.updateAdUnit(adUnit);
                break;
        }
    }

    public static void sendDebugAdTrackingWithAdUnitId(AdUnit adUnit, String c) {
        if (adUnit == null) {
            Log.i(Constants.TAG, "mopub log: BAD LOGIC - adUnit match failed in tracking");
            return;
        }
        Track.trackCounter(PSConstants.TRACK_DEBUG, adUnit.getTrackP(), c, PSMopubAds.screen, PSUtil.isOnline() + "", adUnit.getTrackG(),
                adUnit.getAdUnitId(), "", "");
    }

    public static void sendAdTrackingWithAdUnitId(AdUnit adUnit, String c) {
        if (adUnit == null) {
            Log.i(Constants.TAG, "mopub log: BAD LOGIC - adUnit match failed in tracking");
            return;
        }
        Track.trackCounter(PSConstants.TRACK_AD_TRACKING, adUnit.getTrackP(), c, PSMopubAds.screen, PSUtil.isOnline() + "", adUnit.getTrackG(),
                adUnit.getAdUnitId(), "", "");
    }

    public static void updateAdUnitIdToUse(AdUnit adUnit) {
        switch (adUnit.getMopubAdType()) {
            case MOPUB_AD_TYPE_BANNER:
                if (PSAdmobBanners.areAdmobBannersEnabled()) {
                    PSAdmobBanners.updateAdUnit(adUnit);
                } else {
                    PSMopubBanners.updateAdUnit(adUnit);
                }
                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
                PSMopubInterstitials.updateAdUnit(adUnit);
                break;
            case MOPUB_AD_TYPE_REWARDED:
                PSMopubRewardedVideos.updateAdUnit(adUnit);
                break;
        }
    }

    public static boolean loadMopubAdType(int mopubAdType, int psAdType) {
        boolean res = false;
        switch (mopubAdType) {
            case MOPUB_AD_TYPE_BANNER:
                if (PSAdmobBanners.areAdmobBannersEnabled()) {
                    res = PSAdmobBanners.getBids(psAdType);
                } else {
                    res = PSMopubBanners.getBids(psAdType);
                }
                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
                res = PSMopubInterstitials.getBids(psAdType);
                break;
            case MOPUB_AD_TYPE_REWARDED:
                res = PSMopubRewardedVideos.getBids(psAdType);
                break;
        }
        return res;
    }

    public static boolean handleDartCall(MethodCall call) {
        boolean result = true;
        String action = call.argument("a");
        String adUnitStr = call.argument("adUnit");
        Log.i(Constants.TAG, "mopub log: handle Dart Call - adUnit:" + adUnitStr);
        AdUnit adUnit = AdUnit.createFromJSONStr(adUnitStr);
        switch (action) {
            case "configLoaded":
                mopubDartConfigLoaded();
                break;
            case "unitsComputed":
                mopubDartUnitsComputed(call.argument("adUnits"));
                break;
            case "loadMopubAdType":
                result = loadMopubAdType(Integer.parseInt(call.argument("mType")), 0);
                break;
            case "loadPSAdType":
                result = loadMopubAdType(Integer.parseInt(call.argument("mType")), Integer.parseInt(call.argument("pType")));
                break;
            case "updateAdUnitIdToUse":
                updateAdUnitIdToUse(adUnit);
                break;
            case "isPlacementAvailable":
                result = PSMopubAds.isPlacementAvailable(adUnit.getName());
                break;
            case "retryAdLoad":
                if ( AdsGameSpecific.adLoadForced.containsKey(adUnit.getName())) {
                    AdsGameSpecific.adLoadForced.put(adUnit.getName(), false);
                }
                result = loadMopubAdType(adUnit.getMopubAdType(), adUnit.getPsAdType());
                if (result) {
                    sendAdTrackingWithAdUnitId(adUnit, "load_retry");
                }
                break;
            case "retryAdLoadForced":
                if ( AdsGameSpecific.adLoadForced.containsKey(adUnit.getName())) {
                    AdsGameSpecific.adLoadForced.put(adUnit.getName(), true);
                }
                result = loadMopubAdType(adUnit.getMopubAdType(), adUnit.getPsAdType());
                if (result) {
                    sendAdTrackingWithAdUnitId(adUnit, "load_retry");
                }
                break;
            case "isPlacementPlaying":
                result = isPlacementPlaying(adUnit.getMopubAdType());
                break;
            case "updateMopubConsentStatus":
                result = updateCurrentConsentStatusFromDart(call.argument("optedOut"), call.argument("optReason"));
                break;
            default:
                result = false;
                break;
        }
        return result;
    }

    public static void onDestroy() {
        if (mImpressionListener != null) {
            ImpressionsEmitter.removeListener(mImpressionListener);
            mImpressionListener = null;
        }
    }

    public static boolean isPlacementPlaying(int mopubAdType) {
        boolean res = false;
        switch (mopubAdType) {
            case MOPUB_AD_TYPE_INTERSTITIAL:
                res = PSMopubInterstitials.getIsPlacementPlaying();
                break;
            case MOPUB_AD_TYPE_REWARDED:
                res = PSMopubRewardedVideos.getIsPlacementPlaying();
                break;
        }
        return res;
    }
    
    public static void loadAdForPlacement(String placementName) {
        AdUnit adUnit = getAdUnitForPlacementName(placementName);
        if (adUnit == null) {
            return;
        }
        switch (adUnit.getMopubAdType()) {
            case MOPUB_AD_TYPE_BANNER:
                PSMopubBanners.load();
                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
                PSMopubInterstitials.loadAdForPlacement(adUnit);
                break;
            case MOPUB_AD_TYPE_REWARDED:
                PSMopubRewardedVideos.loadAdForPlacement(adUnit);
                break;
        }
        return;
    }

    public static void loadAdForPlacements(String colonSeparatedPlacementNames) {
        String[] placementNames = colonSeparatedPlacementNames.split(":");
        for (int i = 0; i < placementNames.length; i++) {
            loadAdForPlacement(placementNames[i]);
        }
        return;
    }

    public static void resetRetryCount(int mopubAdType) {
        switch (mopubAdType) {
            case MOPUB_AD_TYPE_BANNER:
                break;
            case MOPUB_AD_TYPE_INTERSTITIAL:
                PSMopubInterstitials.resetRetryCount();
                break;
            case MOPUB_AD_TYPE_REWARDED:
                break;
        }
        return;
    }
    
    private static void updateCurrentConsentStatus(boolean consentStatus) {
//        Log.d(Constants.TAG, "CCPA: DATA_PRIVACY: CONSENT: current status of consent"+ consentStatus);
        GameSpecific.updateCurrentConsentStatusToGame(consentStatus);

        JSONObject msgInfo = new JSONObject();
        try {
            msgInfo.put("route", FLUTTER_ROUTE_NAME);
            msgInfo.put("action", "updateConsentStatus");
            msgInfo.put("status", consentStatus + "");

        } catch (Exception e) {

        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    private static boolean updateCurrentConsentStatusFromDart(String optOut, String chosenReason) {
//        Log.d(Constants.TAG, "CCPA: DATA_PRIVACY: CONSENT: got call from dart to native: "+ optOut +":"+chosenReason);
        boolean hasUserOptedOut = (optOut.equals("true"));
        if(hasUserOptedOut) {
            revokeMopubConsent();
        } else {
            grantMopubConsent();
        }
        GameSpecific.updateUserChosenConsentStatusToGame(hasUserOptedOut, chosenReason);
        return true;
    }

    private static void revokeMopubConsent() {
        Log.d(Constants.TAG, "CCPA: DATA_PRIVACY: CONSENT: got call to revoke consent: ");
        PSMopubAds.revokeConsent();
        PSSmaato.revokeConsent();
    }

    public static void revokeConsent() {
        PersonalInfoManager mPersonalInfoManager = MoPub.getPersonalInformationManager();
        ConsentData consentData = mPersonalInfoManager.getConsentData();
        if (MoPub.canCollectPersonalInformation()) {
//            Log.d(Constants.TAG, "CCPA: DATA_PRIVACY: CONSENT: going to call revoke consent: ");
            mPersonalInfoManager.revokeConsent();
            InMobiGDPR.revokeConsent();
        }
        updateCurrentConsentStatus(MoPub.canCollectPersonalInformation());
    }

}