package in.playsimple.common.mopub;

import android.app.Activity;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;

import com.mopub.mobileads.MoPubErrorCode;
import com.mopub.mobileads.MoPubView;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import in.playsimple.AdsGameSpecific;
import in.playsimple.Constants;
import in.playsimple.common.AdUnit;
import in.playsimple.common.PSConstants;
import in.playsimple.common.Experiment;
import in.playsimple.common.flutter.FlutterBridge;

public class PSMopubBanners {
    private static Activity activity;
    private static MoPubView moPubView;
    private static boolean isBannerLoaded = false;
    private static boolean isInitDone = false;
    private static boolean loadInProgress = false;
    private static boolean shouldShow = false;

    private static String MOPUB_BANNER_PLACEMENT = "";

    // Map of adUnitId -> PlacementName (Placement name is specific to each game and is assumed to be known to both the JS/C# code and the Game Specific Java code)
    private static HashMap<String, String> adUnitToPlacement = new HashMap<>();
    // Map of PlacementName -> AdUnit (which is currently being used for that PlacementName)
    private static HashMap<String, AdUnit> adUnitsMap = new HashMap<>();

    private static String keywords = null;
    private static Map<String, Object> localExtras = null;

    public static void setActivity(Activity act) {
        activity = act;
    }

    public static void initAdUnit(AdUnit adUnit) {
        adUnitsMap.put(adUnit.getName(), adUnit);
    }

    public static void init() {
        updateBannerPlacement();

        moPubView = new MoPubView(activity);
        RelativeLayout relLayout = new RelativeLayout(activity);
        FrameLayout frameLayout = AdsGameSpecific.getFrameLayout();
        frameLayout.addView(relLayout);

        RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
                RelativeLayout.LayoutParams.FILL_PARENT,
                (int) TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, 50, activity.getResources().getDisplayMetrics()));


        if (AdsGameSpecific.getIsBannerAtTop()) {
            params.addRule(RelativeLayout.ALIGN_PARENT_TOP);
        } else {
            params.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
        }
        params.addRule(RelativeLayout.CENTER_HORIZONTAL);

        relLayout.addView(moPubView, params);

        Log.i(Constants.TAG, "mopub log: banner: calling to load banner " + MOPUB_BANNER_PLACEMENT);
        moPubView.setAdUnitId(MOPUB_BANNER_PLACEMENT);
        AdsGameSpecific.setBannerAdListener(moPubView);

        isInitDone = true;
        JSONObject msgInfo = PSMopubAds.getBaseParamsForDartCall("initComplete", null);
        try {
            msgInfo.put("mType", PSMopubAds.MOPUB_AD_TYPE_BANNER);
            msgInfo.put("isPayer", AdsGameSpecific.getIsPayer());
            msgInfo.put("variant", 0);
            msgInfo.put("adExpiryVariant", Integer.parseInt(Experiment.getChosenVariant(AdsGameSpecific.EXP_AD_EXPIRY)));
        } catch (Exception e) {
        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    private static void updateBannerPlacement() {
        Iterator hmIterator = adUnitsMap.entrySet().iterator();
        String firstAdUnit = "";
        while (hmIterator.hasNext()) {
            Map.Entry mapElement = (Map.Entry) hmIterator.next();

            AdUnit adUnit = ((AdUnit) mapElement.getValue());
            adUnitToPlacement.put(adUnit.getAdUnitId(), adUnit.getName());
            if (!adUnit.getName().startsWith("BANNER")) {
                continue;
            }
            firstAdUnit = adUnit.getAdUnitId();
            break;
        }
        MOPUB_BANNER_PLACEMENT = firstAdUnit;
    }

    public static AdUnit getAdUnitForAdUnitId(String adUnitId) {
        try {
            String placement = adUnitToPlacement.get(adUnitId);
            AdUnit adUnit = adUnitsMap.get(placement);
            return adUnit;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static boolean load() {
        Log.d(Constants.TAG, "mopub log: banner: calling to load banner");
        AdUnit adUnit = getAdUnitForAdUnitId(MOPUB_BANNER_PLACEMENT);
        if (moPubView == null) {
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_REQUEST_FAIL);
            Log.d(Constants.TAG, "mopub log: banner: mopub view is null, so not loading");
            return false;
        }

        activity.runOnUiThread(() -> {
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_REQUEST);
            loadInProgress = true;
            if (keywords != null) {
                moPubView.setKeywords(keywords);
            }
            if (localExtras != null) {
                moPubView.setLocalExtras(localExtras);
            }
            moPubView.loadAd();
            if (!shouldShow) {
                moPubView.setVisibility(View.GONE);
            }
        });
        return true;
    }

    public static void updateAdUnit(AdUnit adUnit) {
        if (!isInitDone) {
            return;
        }
        adUnitToPlacement.put(adUnit.getAdUnitId(), adUnit.getName());
        adUnitsMap.put(adUnit.getName(), adUnit);
        updateBannerPlacement();

        Log.d(Constants.TAG, "mopub log: banner: update ad unit");
        load();
    }

    public static void setShouldShowPlacement(boolean val) {
        shouldShow = val;
    }

    public static boolean showPlacement(String screen, String puzzleInfo) {
        Log.d(Constants.TAG, "mopub log: banner: coming here to show banner ad");
        final String oldScreen = PSMopubAds.screen;
        PSMopubAds.screen = screen;
        PSMopubAds.puzzleInfo = puzzleInfo;
        setShouldShowPlacement(true);
        if (loadInProgress) {
            Log.i(Constants.TAG, "mopub log: banner: load already in progress");
            return false;
        }
        if (moPubView == null) {
            Log.i(Constants.TAG, "mopub log: banner: banner view is null");
            return false;
        }
        if (!isBannerLoaded) {
            return getBids(0);
        } else if (moPubView != null) {
            activity.runOnUiThread(() -> {
                moPubView.setVisibility(View.VISIBLE);
                if (!oldScreen.equals(screen)) {
                    PSMopubAds.sendAdTrackingWithAdUnitId(getAdUnitForAdUnitId(MOPUB_BANNER_PLACEMENT), PSConstants.TRACK_VIEW);
                }
            });
            return true;
        }
        return false;
    }

    public static boolean hidePlacement(String screen, String puzzleInfo) {
        Log.d(Constants.TAG, "mopub log: banner: coming here to hide banner ad");
        PSMopubAds.screen = screen;
        PSMopubAds.puzzleInfo = puzzleInfo;
        setShouldShowPlacement(false);
        if (moPubView != null) {
            activity.runOnUiThread(() -> {
                moPubView.setVisibility(View.GONE);
                PSMopubAds.sendAdTrackingWithAdUnitId(getAdUnitForAdUnitId(MOPUB_BANNER_PLACEMENT), PSConstants.TRACK_HIDDEN);
            });
            return true;
        }
        // This means banner is already hidden.
        return true;
    }

    public static HashMap<String, AdUnit> getAdUnitsMap() {
        return adUnitsMap;
    }

    public static boolean getBids(int psAdType) {
        if (moPubView == null) {
            Log.i(Constants.TAG, "mopub log: banner: banner view is null");
            return false;
        }
        if (moPubView != null) {
            moPubView.setAdUnitId(MOPUB_BANNER_PLACEMENT);
        }
//        if (PSBidMachine.isBidMachineEnabled()) {
//            PSBidMachine.getBannerBids(true);
//        } else {
            load();
//        }
        return true;
    }

    // Sent when the banner has successfully retrieved an ad.
    public static void onLoaded(MoPubView banner){
        Log.d(Constants.TAG, "mopub log: banner: banner loaded: id " + MOPUB_BANNER_PLACEMENT);
        loadInProgress = false;
        isBannerLoaded = true;
        AdUnit adUnit = getAdUnitForAdUnitId(MOPUB_BANNER_PLACEMENT);
        PSMopubAds.updatePlacementLoadStatus(isBannerLoaded, adUnit);
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_SUCCESS);

        if (PSMopubBanners.shouldShow) {
            Log.d(Constants.TAG, "mopub log: banner loaded - and being shown: " + MOPUB_BANNER_PLACEMENT);
            PSMopubBanners.showPlacement(PSMopubAds.screen, PSMopubAds.puzzleInfo);
        }

//        if (PSBidMachine.isBidMachineEnabled()) {
//            Log.d(Constants.TAG, "mopub log: banner: banner loaded - refreshing BidMachine request: id " + MOPUB_BANNER_PLACEMENT);
//            PSBidMachine.getBannerBids(false);
//        }
    }

    // Sent when the banner has failed to retrieve an ad. You can use the MoPubErrorCode value to diagnose the cause of failure.
    public static void onFailed(MoPubView banner, MoPubErrorCode errorCode){
        Log.d(Constants.TAG, "mopub log: banner: banner failed, " + errorCode.toString() + " - " + MOPUB_BANNER_PLACEMENT);
        loadInProgress = false;
        isBannerLoaded = false;
        AdUnit adUnit = getAdUnitForAdUnitId(MOPUB_BANNER_PLACEMENT);
        PSMopubAds.updatePlacementLoadStatus(isBannerLoaded, adUnit);
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_FAIL);
    }

    // Sent when the user has tapped on the banner.
    public static void onClicked(MoPubView banner){
        Log.d(Constants.TAG, "mopub log: banner: banner clicked");
        AdUnit adUnit = getAdUnitForAdUnitId(MOPUB_BANNER_PLACEMENT);
        PSMopubAds.updateForAdClick(adUnit);
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_CLICK);
    }

    // Sent when the banner has just taken over the screen.
    public static void onExpanded(MoPubView banner){
        Log.d(Constants.TAG, "mopub log: banner: banner expanded");
    }

    // Sent when an expanded banner has collapsed back to its original size.
    public static void onCollapsed(MoPubView banner){
        Log.d(Constants.TAG, "mopub log: banner: banner collapsed");
    }

    public static void setKeywords(String k) {
        if (moPubView != null) {
            moPubView.setKeywords(k);
            keywords = null;
        } else {
            keywords = k;
        }
        Log.d(Constants.TAG, "mopub log: banner: setting keywords - " + keywords + " - " + k);
    }

    public static void setLocalExtras(Map<String, Object> e) {
        if (moPubView != null) {
            moPubView.setLocalExtras(e);
            localExtras = null;
        } else {
            localExtras = e;
        }
        Log.d(Constants.TAG, "mopub log: banner: setting local extras - " + localExtras + " - " + e);
    }

}
