package in.playsimple.common.mopub;

import android.app.Activity;
import android.util.Log;

import com.mopub.mobileads.MoPubErrorCode;
import com.mopub.mobileads.MoPubInterstitial;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import in.playsimple.AdsGameSpecific;
import in.playsimple.Constants;
import in.playsimple.common.AdUnit;
import in.playsimple.common.Experiment;
import in.playsimple.common.PSConstants;
import in.playsimple.common.Track;
import in.playsimple.common.flutter.FlutterBridge;

public class PSMopubInterstitials {
    private static Activity activity;
    private static boolean isInitDone = false;
    private static int loadInProgressCount[] = new int[4]; // Just keeping it large enough to ensure index values for psAdTypes are covered.
    private static boolean isPlacementPlaying = false;

    // Map of adUnitId -> MoPubInterstitial object
    private static HashMap<String, MoPubInterstitial> mopubInterstitials = new HashMap<>();
    // Map of PlacementName -> Whether to create new MoPubInterstitial object or not
    private static HashMap<String, Boolean> updateInterstitials = new HashMap<>();
    // Map of adUnitId -> PlacementName (Placement name is specific to each game and is assumed to be known to both the JS/C# code and the Game Specific Java code)
    private static HashMap<String, String> adUnitToPlacement = new HashMap<>();
    // Map of PlacementName -> AdUnit (which is currently being used for that PlacementName)
    private static HashMap<String, AdUnit> adUnitsMap = new HashMap<>();

    private static String keywords = null;
    private static Map<String, Object> localExtras = null;

    public static void setActivity(Activity act) {
        activity = act;
    }

    public static void initAdUnit(AdUnit adUnit) {
        adUnitsMap.put(adUnit.getName(), adUnit);
    }

    public static void init() {
        Iterator hmIterator = adUnitsMap.entrySet().iterator();
        while (hmIterator.hasNext()) {
            Map.Entry mapElement = (Map.Entry)hmIterator.next();

            AdUnit adUnit = ((AdUnit)mapElement.getValue());
            initAdUnitHelper(adUnit);
            adUnitToPlacement.put(adUnit.getAdUnitId(), adUnit.getName());
        }

        isInitDone = true;
        JSONObject msgInfo = PSMopubAds.getBaseParamsForDartCall("initComplete", null);
        try {
            msgInfo.put("mType", PSMopubAds.MOPUB_AD_TYPE_INTERSTITIAL);
            msgInfo.put("isPayer", AdsGameSpecific.getIsPayer());
            msgInfo.put("variant", Integer.parseInt(Experiment.getChosenVariant(AdsGameSpecific.EXP_RENDER_RATE_INTERSTITIAL)));
            msgInfo.put("adExpiryVariant", Integer.parseInt(Experiment.getChosenVariant(AdsGameSpecific.EXP_AD_EXPIRY)));
        } catch (Exception e) {
        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    private static void initAdUnitHelper(AdUnit adUnit) {
        MoPubInterstitial interstitial = new MoPubInterstitial(activity, adUnit.getAdUnitId());
        AdsGameSpecific.setInterstitialAdListener(interstitial);

        mopubInterstitials.put(adUnit.getAdUnitId(), interstitial);
        updateInterstitials.put(adUnit.getName(), false);
    }

    public static boolean checkAndLoad(final AdUnit adUnit) {
        Log.d(Constants.TAG, "mopub log: interstitial: is loading " + adUnit.getAdUnitId());
        MoPubInterstitial interstitial = mopubInterstitials.get(adUnit.getAdUnitId());
        if (PSMopubInterstitials.getIsPlacementPlaying() || (interstitial.isReady() && !AdsGameSpecific.adLoadForced.get(adUnit.getName()))) {
            Log.d(Constants.TAG, "mopub log: interstitial: not loading - playing or present:" + adUnit + " - " + PSMopubInterstitials.getIsPlacementPlaying());
            return false;
        }
        loadInProgressCount[adUnit.getPsAdType()]++;
        PSMopubAds.sendDebugAdTrackingWithAdUnitId(adUnit, loadInProgressCount[adUnit.getPsAdType()] + "");
        activity.runOnUiThread(() -> {

            String placementName = adUnit.getName();
            if (updateInterstitials.containsKey(placementName) && updateInterstitials.get(placementName)) {
                initAdUnitHelper(adUnit);
            }
            MoPubInterstitial m = mopubInterstitials.get(adUnit.getAdUnitId());
            if (keywords != null) {
                m.setKeywords(keywords);
            }
            if (localExtras != null) {
                m.setLocalExtras(localExtras);
            }

            m.load();

            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_REQUEST);
        });
        return true;
    }

    public static void load(int psAdType) {
        if (loadInProgressCount[psAdType] > 0) {
            // This check is required here and in getBids.
            Log.i(Constants.TAG, "mopub log: interstitial: in load fn - load already in progress:" + loadInProgressCount[psAdType]);
            return;
        }
        Iterator hmIterator = adUnitsMap.entrySet().iterator();
        while (hmIterator.hasNext()) {
            Map.Entry mapElement = (Map.Entry) hmIterator.next();

            AdUnit adUnit = ((AdUnit) mapElement.getValue());
            if (psAdType == 0 || adUnit.getPsAdType() == psAdType) {
                checkAndLoad(adUnit);
            }
        }
    }

    public static AdUnit getAdUnitForAdUnitId(String adUnitId) {
        try {
            String placementName = adUnitToPlacement.get(adUnitId);
            AdUnit adUnit = adUnitsMap.get(placementName);
            return adUnit;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static AdUnit getAdUnitForInterstitial(MoPubInterstitial interstitial) {
        try {
            Iterator hmIterator = mopubInterstitials.entrySet().iterator();
            while (hmIterator.hasNext()) {
                Map.Entry mapElement = (Map.Entry) hmIterator.next();

                MoPubInterstitial interstitialObj = ((MoPubInterstitial) mapElement.getValue());
                if (interstitialObj.equals(interstitial)) {
                    return getAdUnitForAdUnitId((String)mapElement.getKey());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static MoPubInterstitial getObjectForPlacementName(String placementName) {
        try {
            AdUnit adUnit = adUnitsMap.get(placementName);
            MoPubInterstitial interstitial = mopubInterstitials.get(adUnit.getAdUnitId());
            return interstitial;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static boolean isPlacementAvailable(String placementName) {
        MoPubInterstitial interstitial = getObjectForPlacementName(placementName);
        if (interstitial != null) {
            return interstitial.isReady();
        }
        return false;
    }

    public static boolean showPlacement(String placementName, String screen, String puzzleInfo) {
        PSMopubAds.screen = screen;
        PSMopubAds.puzzleInfo = puzzleInfo;
        if (isPlacementAvailable(placementName)) {
            Log.d(Constants.TAG, "mopub log: interstitial is ready and is shown");
            activity.runOnUiThread(() -> {
                getObjectForPlacementName(placementName).show();
            });
            return true;
        } else {
            // Caching is likely already in progress if `isReady()` is false.
            // Avoid calling `load()` here and instead rely on the callbacks.
        }
        Log.d(Constants.TAG, "mopub log: interstitial is not ready and not shown");
        return false;
    }

    public static void updateAdUnit(AdUnit adUnit) {
        if (!isInitDone) {
            return;
        }
        initAdUnitHelper(adUnit);
        adUnitToPlacement.put(adUnit.getAdUnitId(), adUnit.getName());
        adUnitsMap.put(adUnit.getName(), adUnit);
        // In the next load of the interstitial, the new adUnitId will get used.
    }

    public static HashMap<String, AdUnit> getAdUnitsMap() {
        return adUnitsMap;
    }

    private static int getTotalLoadInProgress() {
        int totalLoading = 0;
        for (int i = 0; i < loadInProgressCount.length; i++) {
            totalLoading += loadInProgressCount[i];
        }
        return totalLoading;
    }

    public static boolean getBids(int psAdType) {
        int loadingCount = getTotalLoadInProgress();
        if (psAdType == 0 && loadingCount > 0) {
            Log.i(Constants.TAG, "mopub log: interstitial: load already in progress:" + loadingCount);
            return false;
        } else if (loadInProgressCount[psAdType] > 0) {
            Log.i(Constants.TAG, "mopub log: interstitial: load already in progress - for psAdType:" + psAdType + " - " + loadInProgressCount[psAdType]);
            return false;
        }
        if (false) {//PSBidMachine.isBidMachineEnabled()) {
            //PSBidMachine.getInterstitialBids(psAdType);
        } else {
            load(psAdType);
        }
        return true;
    }

    public static void onLoaded(MoPubInterstitial interstitial) {
        // The interstitial has been cached and is ready to be shown.
        AdUnit adUnit = getAdUnitForInterstitial(interstitial);
        if (adUnit == null) {
            Track.trackCounter(PSConstants.TRACK_DEBUG, PSConstants.TRACK_AD_TRACKING, PSConstants.TRACK_INTERSTITIAL, "loaded", "ad_unit_null", "", "", "", "");
            return;
        }
        loadInProgressCount[adUnit.getPsAdType()]--;
        Log.i(Constants.TAG, "mopub log: interstitial is loaded:" + loadInProgressCount[adUnit.getPsAdType()] + " - " + adUnit.getAdUnitId());
        String loaded = PSConstants.TRACK_LOAD_SUCCESS_FAIL;
        Boolean isInterstitialReady = interstitial.isReady();
        if (isInterstitialReady) {
            loaded = PSConstants.TRACK_SUCCESS;
        }
        PSMopubAds.updatePlacementLoadStatus(isInterstitialReady, adUnit);
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, loaded);
    }

    public static void onFailed(MoPubInterstitial interstitial, MoPubErrorCode errorCode) {
        // The interstitial has failed to load. Inspect errorCode for additional information.
        AdUnit adUnit = getAdUnitForInterstitial(interstitial);
        if (adUnit == null) {
            Track.trackCounter(PSConstants.TRACK_DEBUG, PSConstants.TRACK_AD_TRACKING, PSConstants.TRACK_INTERSTITIAL, PSConstants.TRACK_FAIL, "ad_unit_null", "", "", "", "");
            return;
        }
        loadInProgressCount[adUnit.getPsAdType()]--;
        Log.i(Constants.TAG, "mopub log: interstitial failed to load:" + loadInProgressCount[adUnit.getPsAdType()] + " - " + errorCode);
        PSMopubAds.updatePlacementLoadStatus(false, adUnit);
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_LOAD_FAIL);
    }

    public static void onShown(MoPubInterstitial interstitial) {
        // The interstitial has been shown. Pause / save state accordingly.
        isPlacementPlaying = true;
        AdUnit adUnit = getAdUnitForInterstitial(interstitial);
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_VIEW);
        PSMopubAds.updateForAdView(adUnit);
    }

    public static void onClicked(MoPubInterstitial interstitial) {
        AdUnit adUnit = getAdUnitForInterstitial(interstitial);
        PSMopubAds.updateForAdClick(adUnit);
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_CLICK);
    }

    public static void onDismissed(MoPubInterstitial interstitial) {
        // The interstitial has being dismissed. Resume / load state accordingly.
        AdUnit adUnit = getAdUnitForInterstitial(interstitial);
        // By the time we get here, impression tracking has happened, and any changes to the adUnitId would have also happened.
        PSMopubAds.updatePlacementStatus(true, 1, adUnit);
        isPlacementPlaying = false;

        if (adUnit == null) {
            AdsGameSpecific.grantVideoReward();
            return;
        }
        if (adUnit.getPsAdType() == PSMopubAds.PS_AD_TYPE_REWARDED) {
            AdsGameSpecific.grantVideoReward();
        } else if (adUnit.getPsAdType() == PSMopubAds.PS_AD_TYPE_INTERSTITIAL) {
            AdsGameSpecific.interstitialAdDismiss();
        }
        PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_CLOSE);

        if (Integer.parseInt(Experiment.getChosenVariant(AdsGameSpecific.EXP_RENDER_RATE_INTERSTITIAL)) == PSConstants.EXP_VAR_CONTROL) {
            getBids(adUnit.getPsAdType());
        }
    }

    public static void loadAdForPlacement(AdUnit adUnit) {
        int placementIndex = adUnit.getPsAdType();
        if (placementIndex <= 0 || placementIndex >= loadInProgressCount.length) {
            return;
        }
        if (loadInProgressCount[placementIndex] > 0) {
            return;
        }
        checkAndLoad(adUnit);
    }

    public static boolean getIsPlacementPlaying() {
        return isPlacementPlaying;
    }

    public static void setKeywords(String k) {
        keywords = k;
    }

    public static void setLocalExtras(Map<String, Object> e) {
        localExtras = e;
    }

    public static void resetRetryCount() {
        JSONObject msgInfo = PSMopubAds.getBaseParamsForDartCall("resetRetryCount", null);
        try {
            msgInfo.put("mType", PSMopubAds.MOPUB_AD_TYPE_INTERSTITIAL);
        } catch (Exception e) {
        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }
}
