package in.playsimple.common.mopub;

import android.app.Activity;
import android.util.Log;

import com.mopub.common.MediationSettings;
import com.mopub.common.MoPubReward;
import com.mopub.mobileads.MoPubErrorCode;
import com.mopub.mobileads.MoPubRewardedVideoListener;
import com.mopub.mobileads.MoPubRewardedVideoManager;
import com.mopub.mobileads.MoPubRewardedVideos;

import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import androidx.annotation.NonNull;
import in.playsimple.AdsGameSpecific;
import in.playsimple.Constants;
import in.playsimple.common.AdUnit;
import in.playsimple.common.Experiment;
import in.playsimple.common.Analytics;
import in.playsimple.common.PSConstants;
import in.playsimple.common.PSUtil;
import in.playsimple.common.flutter.FlutterBridge;

public class PSMopubRewardedVideos {
    private static Activity activity;
    private static boolean isInitDone = false;
    private static int loadInProgressCount = 0;

    private static MoPubRewardedVideoListener rewardedVideoListener;
    public static boolean videoSeenCompletely = false;
    public static long videoStartedTime = 0;
    private static boolean isPlacementPlaying = false;
    private static int loadInProgressArr[] = new int[4];

    // Map of adUnitId -> PlacementName (Placement name is specific to each game and is assumed to be known to both the JS/C# code and the Game Specific Java code)
    private static HashMap<String, String> adUnitToPlacement = new HashMap<>();
    // Map of PlacementName -> AdUnit (which is currently being used for that PlacementName)
    private static HashMap<String, AdUnit> adUnitsMap = new HashMap<>();

    private static String keywords = null;
    private static MediationSettings mediationSettings;

    public static void setActivity(Activity act) {
        activity = act;
    }

    public static void initAdUnit(AdUnit adUnit) {
        adUnitsMap.put(adUnit.getName(), adUnit);
    }

    public static void init() {
        rewardedVideoListener = new MoPubRewardedVideoListener() {
            @Override
            public void onRewardedVideoLoadSuccess(String adUnitId) {
                loadInProgressCount--;
                // Called when the video for the given adUnitId has loaded.
                // At this point you should be able to call MoPubRewardedVideos.showRewardedVideo(String) to show the video.
                if (!isAdUnitInUse(adUnitId)) {
                    Log.d(Constants.TAG, "mopub log: video: load success - of invalid adUnit - " + adUnitId);
                    return;
                }
                AdUnit adUnit = getAdUnitForAdUnitId(adUnitId);
                int placementIndex = AdsGameSpecific.getIndexForPlacementName(adUnit);
                if (placementIndex != -1) {
                    loadInProgressArr[placementIndex]--;
                }
                boolean isVideoLoaded = MoPubRewardedVideos.hasRewardedVideo(adUnitId);
                String videoLoaded = PSConstants.TRACK_SUCCESS;
                if (!isVideoLoaded) {
                    videoLoaded = PSConstants.TRACK_LOAD_SUCCESS_FAIL;
                    Log.d(Constants.TAG, "mopub log: video: load success - resetting to default - " + adUnitId);
                }
                Log.d(Constants.TAG, "mopub log: video: load success - " + adUnitId + " is available " + isVideoLoaded);
                PSMopubAds.updatePlacementLoadStatus(isVideoLoaded, adUnit);
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, videoLoaded);
            }

            @Override
            public void onRewardedVideoLoadFailure(String adUnitId, MoPubErrorCode errorCode) {
                loadInProgressCount--;
                // Called when a video fails to load for the given adUnitId.
                // The provided error code will provide more insight into the reason for the failure to load.
                if (!isAdUnitInUse(adUnitId)) {
                    Log.d(Constants.TAG, "mopub log: video: load failure - of invalid adUnit - " + adUnitId);
                    return;
                }
                Log.d(Constants.TAG, "mopub log: video: load failure - " + adUnitId + " - " + errorCode.toString());
                AdUnit adUnit = getAdUnitForAdUnitId(adUnitId);
                int placementIndex = AdsGameSpecific.getIndexForPlacementName(adUnit);
                if (placementIndex != -1) {
                    loadInProgressArr[placementIndex]--;
                }
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_LOAD_FAIL);
                PSMopubAds.updatePlacementLoadStatus(false, adUnit);
            }

            @Override
            public void onRewardedVideoStarted(String adUnitId) {
                // Called when a rewarded video starts playing.
                videoSeenCompletely = false;
                videoStartedTime = PSUtil.getCurrentTimestamp();
                isPlacementPlaying = true;
//                final SharedPreferences prefs = activity.getSharedPreferences(Constants.PREFS_VIDEO_GRANT, MODE_PRIVATE);
//                prefs.edit()
//                        .putBoolean(Constants.PREFS_VIDEO_COMPLETELY_WATCHED_KEY, false)
//                        .apply();
                Log.d(Constants.TAG, "mopub log: video: started, video seen completely: " + videoSeenCompletely);
                AdUnit adUnit = getAdUnitForAdUnitId(adUnitId);
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_VIEW);
                PSMopubAds.updateForAdView(adUnit);
            }

            @Override
            public void onRewardedVideoPlaybackError(String adUnitId, MoPubErrorCode errorCode) {
                //  Called when there is an error during video playback.
                Log.d(Constants.TAG, "mopub log: video: playback error," + adUnitId + errorCode.toString());
                PSMopubAds.sendAdTrackingWithAdUnitId(getAdUnitForAdUnitId(adUnitId), PSConstants.TRACK_VIEW_FAIL);
            }

            @Override
            public void onRewardedVideoClosed(String adUnitId) {
                // Called when a rewarded video is closed. At this point your application should resume.
                long durationSinceStart = PSUtil.getCurrentTimestamp() - videoStartedTime;
                Log.d(Constants.TAG, "mopub log: video: video closed, " + durationSinceStart);
                if ((videoSeenCompletely && durationSinceStart >= 5) || durationSinceStart > 15) {
                    Log.d(Constants.TAG, "mopub log: video: video seen completely, so granting reward");
                    videoSeenCompletely = false;
                    videoStartedTime = PSUtil.getCurrentTimestamp();
                    AdsGameSpecific.grantVideoReward();
                } else {
                    AdsGameSpecific.rewardedVideoClosed();
                }
                AdUnit adUnit = getAdUnitForAdUnitId(adUnitId);
                int count = checkVideosToLoadCount();
                PSMopubAds.updatePlacementStatus(true, count, adUnit);
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_CLOSE);

                isPlacementPlaying = false;
                getBids(adUnit.getPsAdType());
            }

            @Override
            public void onRewardedVideoCompleted(Set<String> adUnitIds, MoPubReward reward) {
                // Called when a rewarded video is completed and the user should be rewarded.
                // You can query the reward object with boolean isSuccessful(), String getLabel(), and int getAmount().
                videoSeenCompletely = true;
                Log.d(Constants.TAG, "mopub log: updateAdUnitAfterImpression: video completed, video seen completely: " + videoSeenCompletely);
                Iterator<String> itr = adUnitIds.iterator();
                while (itr.hasNext()) {
                    PSMopubAds.sendAdTrackingWithAdUnitId(getAdUnitForAdUnitId(itr.next()), "video_completed");
                }
            }

            @Override
            public void onRewardedVideoClicked(@NonNull String adUnitId) {
                Log.d(Constants.TAG, "mopub log: video: clicked");
                AdUnit adUnit = getAdUnitForAdUnitId(adUnitId);
                PSMopubAds.updateForAdClick(adUnit);
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_CLICK);
            }
        };
        MoPubRewardedVideos.setRewardedVideoListener(rewardedVideoListener);

        Iterator hmIterator = adUnitsMap.entrySet().iterator();
        while (hmIterator.hasNext()) {
            Map.Entry mapElement = (Map.Entry) hmIterator.next();

            AdUnit adUnit = ((AdUnit) mapElement.getValue());
            adUnitToPlacement.put(adUnit.getAdUnitId(), adUnit.getName());
        }

        isInitDone = true;
        JSONObject msgInfo = PSMopubAds.getBaseParamsForDartCall("initComplete", null);
        try {
            msgInfo.put("mType", PSMopubAds.MOPUB_AD_TYPE_REWARDED);
            msgInfo.put("isPayer", AdsGameSpecific.getIsPayer());
            msgInfo.put("variant", 0);
            msgInfo.put("adExpiryVariant", Integer.parseInt(Experiment.getChosenVariant(AdsGameSpecific.EXP_AD_EXPIRY)));
        } catch (Exception e) {
        }
        FlutterBridge.sendDataToFlutterModule(msgInfo.toString(), false);
    }

    public static boolean isAdUnitInUse(String adUnitId) {
        Iterator hmIterator = adUnitsMap.entrySet().iterator();
        while (hmIterator.hasNext()) {
            Map.Entry mapElement = (Map.Entry) hmIterator.next();
            AdUnit adUnit = ((AdUnit) mapElement.getValue());
            if (adUnitId.equals(adUnit.getAdUnitId())) {
                return true;
            }
        }
        return false;
    }

    public static AdUnit getAdUnitForAdUnitId(String adUnitId) {
        try {
            String placement = adUnitToPlacement.get(adUnitId);
            AdUnit adUnit = adUnitsMap.get(placement);
            return adUnit;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static boolean checkAndLoad(final AdUnit adUnit) {
        try {
            if (PSMopubRewardedVideos.getIsPlacementPlaying() || (MoPubRewardedVideos.hasRewardedVideo(adUnit.getAdUnitId()) && !AdsGameSpecific.adLoadForced.get(adUnit.getName()))) {
                Log.d(Constants.TAG, "mopub log: video: rewarded video not loading - playing or present:" + adUnit.getAdUnitId() + " - " + PSMopubRewardedVideos.getIsPlacementPlaying());
                return false;
            }
            loadInProgressCount++;
            int placementIndex = AdsGameSpecific.getIndexForPlacementName(adUnit);
            if (placementIndex != -1) {
                loadInProgressArr[placementIndex]++;
            }
            PSMopubAds.sendDebugAdTrackingWithAdUnitId(adUnit, loadInProgressCount + "");
            activity.runOnUiThread(() -> {
                if (keywords != null) {
                    MoPubRewardedVideos.loadRewardedVideo(adUnit.getAdUnitId(), new MoPubRewardedVideoManager.RequestParameters(keywords), mediationSettings);
                } else {
                    MoPubRewardedVideos.loadRewardedVideo(adUnit.getAdUnitId());
                }
                Log.d(Constants.TAG, "mopub log: video: loading rewarded video for " + adUnit.getAdUnitId());
                
                PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_REQUEST);
            });
            return true;
        } catch (Exception e) {
            Log.d(Constants.TAG, "mopub log: video: exception when loading video " + adUnit.getAdUnitId());
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_EXCEPTION);
            e.printStackTrace();
        }
        return false;
    }

    public static void load(int psAdType) {
        if (loadInProgressCount > 0) {
            // This check is required here and in getBids.
            Log.d(Constants.TAG, "mopub log: video: in load fn - load already in progress:" + loadInProgressCount);
            return;
        }
        Log.d(Constants.TAG, "mopub log: call checkRewardedPlacementsToLoadAd from load");
        AdsGameSpecific.checkRewardedPlacementsToLoadAd();
//        Iterator hmIterator = adUnitsMap.entrySet().iterator();
//        while (hmIterator.hasNext()) {
//            Map.Entry mapElement = (Map.Entry) hmIterator.next();
//
//            AdUnit adUnit = ((AdUnit) mapElement.getValue());
//            if (psAdType == 0 || adUnit.getPsAdType() == psAdType) {
//                checkAndLoad(adUnit);
//            }
//        }
    }

    public static int checkVideosToLoadCount() {
        int count = 0;
        Iterator hmIterator = adUnitsMap.entrySet().iterator();
        while (hmIterator.hasNext()) {
            Map.Entry mapElement = (Map.Entry) hmIterator.next();

            AdUnit adUnit = ((AdUnit) mapElement.getValue());
            if (!MoPubRewardedVideos.hasRewardedVideo(adUnit.getAdUnitId())) {
                count++;
            }
        }
        Log.d(Constants.TAG, "mopub log: video: checkVideosToLoadCount: " + count);
        return count;
    }

    public static boolean isPlacementAvailable(String placementType) {
        AdUnit adUnit = adUnitsMap.get(placementType);
        try {
            return MoPubRewardedVideos.hasRewardedVideo(adUnit.getAdUnitId());
        } catch (Exception e) {
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_EXCEPTION);
            Log.d(Constants.TAG, "mopub log: video: is video available exception " + e.getMessage());
            e.printStackTrace();
            Analytics.logException(e);
        }
        return false;
    }

    public static boolean showPlacement(String placementType, String screen, String puzzleInfo) {
        PSMopubAds.screen = screen;
        PSMopubAds.puzzleInfo = puzzleInfo;
        AdUnit adUnit = adUnitsMap.get(placementType);
        try {
            if (isPlacementAvailable(placementType)) {
                Log.d(Constants.TAG, "mopub log: video: showing rewarded video for " + placementType);
                activity.runOnUiThread(() -> {
                    try {
                        MoPubRewardedVideos.showRewardedVideo(adUnit.getAdUnitId());
                    } catch (Exception e) {
                        Log.d(Constants.TAG, "mopub log: exception when showing video, " + e.getMessage());
                        e.printStackTrace();
                    }
                });
                return true;
            }
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_VIEW_ERROR);
        } catch (Exception e) {
            PSMopubAds.sendAdTrackingWithAdUnitId(adUnit, PSConstants.TRACK_EXCEPTION);
            Log.d(Constants.TAG, "mopub log: video: exception when showing rewarded video -> " + e.getMessage());
            e.printStackTrace();
            Analytics.logException(e);
        }
        return false;
    }

    public static void updateAdUnit(AdUnit adUnit) {
        if (!isInitDone) {
            return;
        }
        adUnitToPlacement.put(adUnit.getAdUnitId(), adUnit.getName());
        adUnitsMap.put(adUnit.getName(), adUnit);
    }

    public static HashMap<String, AdUnit> getAdUnitsMap() {
        return adUnitsMap;
    }

    public static boolean getBids(int psAdType) {
        if (loadInProgressCount > 0) {
            Log.i(Constants.TAG, "mopub log: video: load already in progress:" + loadInProgressCount);
            return false;
        }
        if (false) {//PSBidMachine.isBidMachineEnabled()) {
            //PSBidMachine.getRewardedBids(psAdType);
        } else {
            load(psAdType);
        }

        return true;
    }

    public static void loadAdForPlacement(AdUnit adUnit) {
        int placementIndex = AdsGameSpecific.getIndexForPlacementName(adUnit);
        Log.i(Constants.TAG, "mopub log: video load call java side:" + placementIndex + " - " + adUnit.getName() + " - " + adUnit.getAdUnitId());
        if (placementIndex == -1) {
            return;
        }
        if (loadInProgressArr[placementIndex] > 0) {
            return;
        }
        checkAndLoad(adUnit);
    }

    public static boolean getIsPlacementPlaying() {
        return isPlacementPlaying;
    }

    public static void setKeywords(String k) {
        keywords = k;
    }

    public static void setMediationSettings(MediationSettings m) {
        mediationSettings = m;
    }
}
