#import "AdIntelligence.h"
#import "Analytics.h"
#import "SqliteHelper.h"
#import "Util.h"

@implementation AdIntelligence

NSArray *skipDirs;
SqliteHelper *database;
NSMutableDictionary *adData;
NSString *rawAdData;
+(void) init {
    NSString* skipDirsStr = @"Caches/images,in.cardgame.solitaire,Snapshots,WebKit,com.apple.WebKit,com.crashlytics.data,io.fabric.sdk.ios.data";
    skipDirs = [skipDirsStr componentsSeparatedByString: @","];
    database = [SqliteHelper get:READWRITE];
    
    rawAdData = [database getData:AD_INTELLIGENCE_FILE];
    if (rawAdData != nil) {
        // NSLog(@"Loaded adIntelligence data: %@", rawAdData);
        adData = [Util stringToJson:rawAdData];
    } else {
        // NSLog(@"Unable to load adIntelligence data");
    }
}

+(void) listParentDirectories {
    // NSLog(@"Listing parent directories");
    NSString *cacheDirectory = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) firstObject];
    NSArray* dirs = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:cacheDirectory
                                                                        error:NULL];
    [dirs enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        NSString *filename = (NSString *)obj;
        NSString *extension = [[filename pathExtension] lowercaseString];
        NSLog(@"File name: %@", filename);
    }];
    
}

+(void) listAllFiles {
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self init];
        
        NSFileManager *fileManager = [NSFileManager defaultManager];
        NSString *cacheDirectory = [NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES) firstObject];
        NSURL *directoryURL = [NSURL URLWithString:[cacheDirectory stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]]; // URL pointing to the directory you want to browse
        NSArray *keys = [NSArray arrayWithObject:NSURLIsDirectoryKey];
        
        NSDirectoryEnumerator *enumerator = [fileManager
                                             enumeratorAtURL:directoryURL
                                             includingPropertiesForKeys:keys
                                             options:0
                                             errorHandler:^(NSURL *url, NSError *error) {
                                                 // Handle the error.
                                                 // Return YES if the enumeration should continue after the error.
                                                 return YES;
                                             }];

        NSUInteger skipDirLen = [skipDirs count];
        for (NSURL *url in enumerator) {
            NSError *error;
            NSNumber *isDirectory = nil;
            if (! [url getResourceValue:&isDirectory forKey:NSURLIsDirectoryKey error:&error]) {
                // handle error
            }
            else if (! [isDirectory boolValue]) {
                // No error and it’s not a directory; do something with the file
                Boolean shouldSkip = false;
                for (int i = 0; i < skipDirLen; i++) {
                    if ([url.absoluteString rangeOfString:skipDirs[i]].location != NSNotFound) {
                        shouldSkip = true;
                        break;
                    }
                }
                if (!shouldSkip) {
                    // NSLog(@"This is a file: %@", url);
                    [self uploadFileToServer:url];
                }
            }
        }
    });
}

+(void) markAdUploaded:(NSString *)filePath {
    if (adData == nil) {
        adData = [[NSMutableDictionary alloc] init];
    }
    [adData setObject:@"1" forKey:filePath];
    NSString *data = [Util JSONToString:adData];
    if (data != nil) {
        NSString* report = [NSString stringWithFormat:@"%@:%d:%d", @"AdUploaded", (int)[data length], (int)[filePath length]];
        [Analytics sendReport:report];
        [database setItem:AD_INTELLIGENCE_FILE value:data];
    }
    // NSLog(@"Saved adIntelligence data: %@", [Util JSONToString:adData]);
}

+(NSString *)uploadFileToServer:(NSURL *)url

{
    /*NSString *fileName = @"help.jpg";
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    
    NSString *documentsDirectory = [paths objectAtIndex:0];
    
    NSString *expectedFilePath = [documentsDirectory stringByAppendingPathComponent:fileName];
    NSLog(@"Expected file path: %@", expectedFilePath);*/
    
    NSString *haystack = url.absoluteString;
    NSString *haystackPrefix = @"file://";
    NSRange needleRange = NSMakeRange(haystackPrefix.length,
                                      haystack.length - haystackPrefix.length);
    NSString *filePath = [haystack substringWithRange:needleRange];
    NSArray *filePathArray = [filePath componentsSeparatedByString: @"/"];
    NSUInteger dirCount = [filePathArray count];
    NSString *fileName = filePathArray[dirCount - 1];
    // NSLog(@"File name from path: %@ - %@", fileName, filePath);
    
    // Replace %20 with space if any
    filePath = [filePath stringByReplacingOccurrencesOfString:@"%20" withString:@" "];
    
    if (adData != nil && [adData objectForKey:fileName] != nil) {
        // NSLog(@"This ad has already been uploaded: %@", fileName);
        return @"";
    }
    
    NSData *file1Data = [[NSData alloc] initWithContentsOfFile:filePath];
    NSUInteger dataLen = [file1Data length];
    if (dataLen == 0) {
        NSLog(@"Couldn't read from file: %@", filePath);
        return @"";
    }
    // NSLog(@"Read from file: %@ %lu", filePath, dataLen);
    
    NSString *filePathConcat = @"";
    for (int i = (int)(dirCount - 2); i >= 1; i--) {
        NSString *dirName = filePathArray[i];
        if ([dirName isEqualToString:@"Caches"]) {
            NSString *prevDirName = filePathArray[i - 1];
            if ([prevDirName isEqualToString:@"Library"]) {
                break;
            }
        }
        filePathConcat = [NSString stringWithFormat:@"%@:%@", dirName, filePathConcat];
    }
    filePathConcat = [NSString stringWithFormat:@"%@:%@", @"ios", filePathConcat];
   
    // Check if the file needs upload
    NSString *checkUrlString = [NSString stringWithFormat:@"%@%@%@/%@", AD_INTEL_SERVER, @"check/", fileName, filePathConcat];
    NSMutableURLRequest *checkRequest = [[NSMutableURLRequest alloc] init];
    [checkRequest setURL:[NSURL URLWithString:checkUrlString]];
    [checkRequest setHTTPMethod:@"GET"];
    NSData *checkReturnData = [NSURLConnection sendSynchronousRequest:checkRequest returningResponse:nil error:nil];
    NSString *checkReturnString = [[NSString alloc] initWithData:checkReturnData encoding:NSUTF8StringEncoding] ;
    // NSLog(@"Return from check - %@", checkReturnString);
    if ([checkReturnString isEqualToString:@"1"]) {
        // This file has already been uploaded to server
        [Analytics sendReport:@"AlreadyPresent"];
        [self markAdUploaded:fileName];
        return checkReturnString;
    }
    
    NSString *urlString = [NSString stringWithFormat:@"%@%@%@/%@", AD_INTEL_SERVER, @"upload/", fileName, filePathConcat];
    // NSLog(@"File path concat: %@, %@", filePathConcat, urlString);
    
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] init];
    
    [request setURL:[NSURL URLWithString:urlString]];
    
    [request setHTTPMethod:@"POST"];
    
    NSMutableData *body = [NSMutableData data];
    
    [body appendData:[NSData dataWithData:file1Data]];
    
    [request setHTTPBody:body];
    
    NSData *returnData = [NSURLConnection sendSynchronousRequest:request returningResponse:nil error:nil];
    
    NSString *returnString = [[NSString alloc] initWithData:returnData encoding:NSUTF8StringEncoding] ;
    
    if ([returnString isEqualToString:@"1"]) {
        // This file has already been uploaded to server
        [Analytics sendReport:@"JustUploaded"];
        [self markAdUploaded:fileName];
        return returnString;
    }
    return returnString;
    
}

@end
