#import "Speak.h"

@implementation Speak
SFSpeechRecognizer *speechRecognizer;
SFSpeechAudioBufferRecognitionRequest *recognitionRequest;
SFSpeechRecognitionTask *recognitionTask;
AVAudioEngine *audioEngine;
AVAudioInputNode *inputNode;

+(void) configSpeechRecognizer {
    speechRecognizer = [[SFSpeechRecognizer alloc] initWithLocale:[NSLocale localeWithLocaleIdentifier:@"en-US"]];
    [SFSpeechRecognizer requestAuthorization:^(SFSpeechRecognizerAuthorizationStatus authStatus) {
        BOOL isButtonEnabled = NO;
        switch (authStatus) {
                case SFSpeechRecognizerAuthorizationStatusAuthorized:
                //User gave access to speech recognition
                NSLog(@"Authorized");
                isButtonEnabled = YES;
                break;
                
                case SFSpeechRecognizerAuthorizationStatusDenied:
                //User denied access to speech recognition
                NSLog(@"SFSpeechRecognizerAuthorizationStatusDenied");
                break;
                
                case SFSpeechRecognizerAuthorizationStatusRestricted:
                //Speech recognition restricted on this device
                NSLog(@"SFSpeechRecognizerAuthorizationStatusRestricted");
                break;
                
                case SFSpeechRecognizerAuthorizationStatusNotDetermined:
                //Speech recognition not yet authorized
                NSLog(@"SFSpeechRecognizerAuthorizationStatusNotDetermined");
                break;
                
            default:
                NSLog(@"Default");
                break;
        }
        
        // http://stackoverflow.com/questions/31951704/this-application-is-modifying-the-autolayout-engine-from-a-background-thread-wh
        //[[NSOperationQueue mainQueue] addOperationWithBlock:^{
        //    _microphoneBtn.enabled = isButtonEnabled;
        //}];
    }];
}

+(void) toggleRecording {
    if ([audioEngine isRunning]) {
        NSLog(@"AUDIO ENGINE IS RUNNING");
        [audioEngine stop];
        [recognitionRequest endAudio];
    } else {
        NSLog(@"AUDIO ENGINE IS WORKING");
        [self startRecording];
    }
}

+(void) stopRecording {
    [audioEngine stop];
    [recognitionRequest endAudio];
}

+(void)startRecording { // http://stackoverflow.com/questions/37821826/continuous-speech-recogn-with-sfspeechrecognizer-ios10-beta
    if ([audioEngine isRunning]) {
        NSLog(@"AUDIO ENGINE IS RUNNING");
        [audioEngine stop];
        [recognitionRequest endAudio];
    }
    
    if (recognitionTask != nil) {
        [recognitionTask cancel];
        recognitionTask = nil;
    }
    
    NSError * outError;
    AVAudioSession *audioSession = [AVAudioSession sharedInstance];
    @try {
        [audioSession setCategory:AVAudioSessionCategoryRecord error:&outError];
        [audioSession setMode:AVAudioSessionModeMeasurement error:&outError];
        [audioSession setActive:true withOptions:AVAudioSessionSetActiveOptionNotifyOthersOnDeactivation  error:&outError];
    } @catch (NSException * e) {
        NSLog(@"audioSession properties weren't set because of an error. %@", e);
    }
    
    recognitionRequest = [[SFSpeechAudioBufferRecognitionRequest alloc] init];
    inputNode = [audioEngine inputNode];
    if (inputNode == nil) {
        NSLog(@"Audio engine has no input node");
    }
    if (recognitionRequest == nil) {
        NSLog(@"Unable to created a SFSpeechAudioBufferRecognitionRequest object");
    }
    recognitionRequest.shouldReportPartialResults = YES;
    recognitionTask = [speechRecognizer recognitionTaskWithRequest:recognitionRequest
                                                       resultHandler:^(SFSpeechRecognitionResult *result, NSError *error) {
                                                           NSLog(@"Something happened?");
                                                           BOOL isFinal = NO;
                                                           if (result != nil) {
                                                               NSLog(@"Transcription text: %@", result.bestTranscription.formattedString);
                                                               isFinal = result.isFinal;
                                                           } else {
                                                               NSLog(@"Result is nil");
                                                           }
                                                           if (error != nil || isFinal) {
                                                               NSLog(@"Error - 1 %@", error.description);
                                                               [audioEngine stop];
                                                               [inputNode removeTapOnBus:0];
                                                               recognitionRequest = nil;
                                                               recognitionTask = nil;
                                                           } else {
                                                               NSLog(@"Error - 2");
                                                           }
                                                       }];
    
    AVAudioFormat *recordingFormat = [inputNode outputFormatForBus:0];
    [inputNode installTapOnBus:0 bufferSize:1024 format:recordingFormat block:^(AVAudioPCMBuffer *buffer, AVAudioTime *when) {
        NSLog(@"I am here");
        [recognitionRequest appendAudioPCMBuffer:buffer];
    }];
    
    [audioEngine prepare];
    
    @try {
        [audioEngine startAndReturnError:&outError];
    } @catch (NSException *e) {
        NSLog(@"audioEngine couldn't start because of an error. %@", e);
    }
}

- (void)speechRecognizer:(SFSpeechRecognizer*)speechRecognizer availabilityDidChange:(BOOL)available {
    NSLog(@"AVAILABLE: %@", (available?@"YES":@"NO"));
}

@end
