//
//  FlutterBridge.m
//
//  Created by Suraj Nalin on 18/10/19.
//

#import "FlutterBridge.h"
#import <FlutterPluginRegistrant/GeneratedPluginRegistrant.h> // Only if you have Flutter Plugins
#import "User.h"
#import "Track.h"
//#import "PSMopubAds.h"
#import "Experiment.h"
#import "Quests.h"
#import "UIView+Toast.h"
#import "PSPrivacy.h"
#import "Util.h"
#import "PSUtil.h"

#define FLUTTER_METHOD_CHANNEL @"in.playsimple.flutter.method"
#define FLUTTER_MESSAGE_CHANNEL @"in.playsimple.flutter.message"

static FlutterBridge *flutterBridge;
static NSMutableArray* criticalQueue;

@implementation FlutterBridge

+(void) init {
    if (flutterBridge == nil) {
        flutterBridge = [[FlutterBridge alloc] init];
    }
    
    // https://flutter.dev/docs/development/platform-integration/platform-channels#example-objc
    // https://medium.com/flutter/flutter-platform-channels-ce7f540a104e
    dispatch_async(dispatch_get_main_queue(), ^{
        flutterBridge.flutterEngine = [[FlutterEngine alloc] initWithName:@"io.flutter" project:nil];
        [flutterBridge.flutterEngine runWithEntrypoint:nil];
        [GeneratedPluginRegistrant registerWithRegistry:flutterBridge.flutterEngine];
        
        NSLog(@"Channels being initialized - flutter init");
        [self initializeChannels];
    });
}

+(void) sendDataToFlutterModule:(NSString*) routeParams isActivity:(Boolean) isActivity isCritical:(Boolean) isCritical {
    if(isCritical) {
        if(flutterBridge == nil || !flutterBridge.channelsInitialized) {
            [FlutterBridge addToCriticalQueue:routeParams isActivity:isActivity atStart:false];
            return;
        }
    }
    [FlutterBridge sendDataToFlutterModule:routeParams isActivity:isActivity];
}

+(void) sendDataToFlutterModule:(NSString*) routeParams isActivity:(Boolean) isActivity {
    NSLog(@"Flutter show attempt - %@ %d", routeParams, isActivity);
    if (!flutterBridge.channelsInitialized) {
        NSLog(@"Channels not yet initialized. Message not sent. - %d %@", isActivity, routeParams);
        return;
    }
    dispatch_async(dispatch_get_main_queue(), ^{
//        [[FIRCrashlytics crashlytics] logWithFormat:@"FlutterBridge: sending data: %d %@", isActivity, routeParams];
        // [FlutterBridge initFlutterInfoBeforeCall];
        [flutterBridge.messageChannel sendMessage:routeParams];
        //        [flutterBridge.messageChannel setMessageHandler:^(id  _Nullable message, FlutterReply  _Nonnull callback) {
        //            // Note: this method is invoked on the UI thread.
        //            // TODO
        //            NSLog(@"Something received on message channel");
        //        }];
        if (isActivity) {
            // The delay in presenting the flutter view has been put deliberately.
            // Without this delay, the app would crash due to the framebuffer not being written correctly before the transition.
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.1 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
                UIViewController *rootViewController = [UIApplication sharedApplication].keyWindow.rootViewController;
                flutterBridge.flutterViewController.modalPresentationStyle = UIModalPresentationFullScreen;
                flutterBridge.flutterViewController.wantsFullScreenLayout = YES;
                [rootViewController presentViewController:flutterBridge.flutterViewController animated:false completion:nil];
            });
        }
    });
}

+(void) sendStateToFlutterModule:(NSString*) controller action:(NSString*) action {
    NSMutableDictionary *msgData = [NSMutableDictionary new];
    [msgData setObject:controller forKey:@"route"];
    [msgData setObject:action forKey:@"action"];
    NSData *msgInfo = [NSJSONSerialization dataWithJSONObject:msgData options:0 error:nil];
    [FlutterBridge sendDataToFlutterModule:[[NSString alloc] initWithData:msgInfo encoding:NSUTF8StringEncoding] isActivity:false];
}

+(bool) initializeChannels {
    if (flutterBridge.flutterEngine == nil) {
        NSLog(@"Channels being initialized before FlutterEngine");
        return false;
    }
    NSLog(@"Attempt to initialize flutter channels");
    if (flutterBridge.channelsInitialized) {
        NSLog(@"Attempt to initialize flutter channels - channels already initialized");
        return true;
    }
    
    [FlutterBridge initFlutterInfoBeforeCall];
    
    flutterBridge.channelsInitialized = true;
    
    NSDictionary *config = @{
        @"route": @"config",
        @"g": [NSString stringWithFormat:@"%d", GAME_ID_VAL],
        @"c": [NSString stringWithFormat:@"%d", CLIENT_ID],
    };
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:config options:0 error:nil];
    [FlutterBridge addToCriticalQueue:[[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding] isActivity:false atStart:true];

    [flutterBridge processCriticalQueue];
    return true;
}

+ (void) initFlutterInfoBeforeCall {
    flutterBridge.flutterViewController = [[FlutterViewController alloc] initWithEngine:flutterBridge.flutterEngine nibName:nil bundle:nil];
    flutterBridge.methodChannel = [FlutterMethodChannel methodChannelWithName:FLUTTER_METHOD_CHANNEL binaryMessenger:flutterBridge.flutterViewController];
    flutterBridge.messageChannel = [FlutterBasicMessageChannel messageChannelWithName:FLUTTER_MESSAGE_CHANNEL binaryMessenger:flutterBridge.flutterViewController codec:[FlutterStringCodec sharedInstance]];
    
    [flutterBridge.methodChannel setMethodCallHandler:^(FlutterMethodCall* call, FlutterResult result) {
                // Note: this method is invoked on the UI thread.
                NSString *method = call.method;
                NSLog(@"flutterIOS - Something received on method channel: %@", method);
//                [[FIRCrashlytics crashlytics] logWithFormat:@"FlutterBridge: called with: %@", method];
                if ([@"initializeCustomView" isEqualToString:call.method]) {
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.1 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
                        UIViewController *rootViewController = [UIApplication sharedApplication].keyWindow.rootViewController;
                        flutterBridge.flutterViewController.modalPresentationStyle = UIModalPresentationFullScreen;
                        flutterBridge.flutterViewController.wantsFullScreenLayout = YES;
                        [rootViewController presentViewController:flutterBridge.flutterViewController animated:false completion:nil];
                    });
                    result(@"1");
                } else if ([@"onPspnModuleInteracted" isEqualToString:call.method]) {
                    NSString* method = call.arguments[@"method"];
                    NSString* args = call.arguments[@"args"];
                    [Util sendJSCallBack:method msg:args];
                    result(@"1");
                } else if ([@"gettingSharedData" isEqualToString:call.method]) {
                    NSString* iosGameIds = call.arguments[@"iosGameIds"];
                    if([Quests respondsToSelector:@selector(gettingSharedData:)]) {
                        [Quests gettingSharedData:iosGameIds];
                    }
                    result(@"1");
                } else if ([@"shareDataBetweenGames" isEqualToString:call.method]) {
                    NSString* allGamesData = call.arguments[@"allGamesData"];
                    NSString* crossGameProgress = call.arguments[@"crossGameProgress"];
                    if([Quests respondsToSelector:@selector(shareDataBetweenGames:crossGame:)]) {
                        [Quests shareDataBetweenGames:allGamesData crossGame:crossGameProgress];
                    }
                    result(@"1");
                } else if ([@"getInstalledApps" isEqualToString:call.method]) {
                    NSString* bundleIds = call.arguments[@"bundleIds"];
                    NSArray * bundles = [bundleIds componentsSeparatedByString:@","];
                    NSString* gameIds = call.arguments[@"gameIds"];
                    NSArray * games = [gameIds componentsSeparatedByString:@","];
                    NSMutableArray *installedIds = [[NSMutableArray alloc] init];
                    for (NSUInteger index = 0; index < bundles.count; index++) {
                        if([PSUtil isAppInstalled:bundles[index]]) {
                             [installedIds addObject: games[index]];
                        }
                    }
                    result(installedIds);
                } else if ([@"launchAnotherApp" isEqualToString:call.method]) {
                    NSString* bundleId = call.arguments[@"bundleId"];
                    [PSUtil launchAnotherApp:bundleId];
                    result(@"1");
                } else if ([@"showNativeToast" isEqualToString:call.method]) {
                    NSString* msg = call.arguments[@"msg"];
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.1 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
                        UIViewController *rootViewController = [UIApplication sharedApplication].keyWindow.rootViewController;
                        while(rootViewController.presentedViewController) {
                            rootViewController = rootViewController.presentedViewController;
                        }
                        UIView *topView = rootViewController.view;
                        [topView makeToast:msg duration:1 position:CSToastPositionCenter];
                    });
                    result(@"1");
                } else if ([@"updateNameEmail" isEqualToString:call.method]) {
                    NSString* chosenName = call.arguments[@"name"];
                    NSString* chosenEmail = call.arguments[@"email"];
                    if (chosenName != nil && chosenEmail != nil) {
                        [User updateNameEmail:chosenName email:chosenEmail];
                    }
                    result(@"1");
                    // result(@(batteryLevel));
    //                result([FlutterError errorWithCode:@"UNAVAILABLE"
    //                message:@"Battery info unavailable"
    //                details:nil]);
    //            } else if ([@"updateChosenCountry" isEqualToString:call.method]) {
    //                NSString* chosenCountry = call.arguments[@"cc"];
    //
    //                NSLog(@"Chosen country - %@", chosenCountry);
    //                [User updateChosenCountry:chosenCountry];
    //                result(@"1");
    //            } else if ([@"updateAgeRange" isEqualToString:call.method]) {
    //                NSString* chosenMinAge = call.arguments[@"cmin"];
    //                NSString* chosenMaxAge = call.arguments[@"cmax"];
    //
    //                NSLog(@"Chosen age range - %@ %@", chosenMinAge, chosenMaxAge);
    //                [User updateAgeRange:chosenMinAge cmax:chosenMaxAge];
    //                result(@"1");
                } else if ([@"acceptPrivacyPolicy" isEqualToString:call.method]) {
                    [User acceptPrivacyPolicy];
                    result(@"1");
                } else if ([@"sendTrackingFlutter" isEqualToString:call.method]) {
                    NSString* track_k = call.arguments[@"k"];
                    NSString* track_p = call.arguments[@"p"];
                    NSString* track_c = call.arguments[@"c"];
                    NSString* track_o = call.arguments[@"o"];
                    NSString* track_f = call.arguments[@"f"];
                    NSString* track_g = call.arguments[@"g"];
                    NSString* track_s = call.arguments[@"s"];
                    NSString* track_co = call.arguments[@"co"];
                    NSString* track_app = call.arguments[@"app"];

                    // These arguments are not part of core_ps_sdk until a 1.0.42
                    if(track_s == nil) {
                        track_s = @"";
                    }
                    if(track_co == nil) {
                        track_co = @"";
                    }
                    if(track_app == nil) {
                        track_app = @"";
                    }

                    [Track trackCounter:track_k p:track_p c:track_c o:track_o f:track_f g:track_g s:track_s co:track_co oppId:track_app];
                    NSLog(@"Sent tracking - %@, %@, %@, %@, %@, %@, %@, %@, %@ ", track_k, track_p, track_c, track_o, track_f, track_g, track_s, track_co, track_app);
                    result(@"1");
                } else if ([@"logFlutterException" isEqualToString:call.method]) {
                    NSString* expMessage = call.arguments[@"e"];
                    NSString* expStack = call.arguments[@"s"];
//                    [[FIRCrashlytics crashlytics] logWithFormat:@"%@ - %@", expMessage, expStack];
                    if([expMessage length] > 100) {
                        expMessage = [expMessage substringToIndex:100];
                    }
                    if([expStack length] > 100) {
                        expStack = [expStack substringToIndex:100];
                    }
                    [Track trackCounter:@"flutter_error" p:expMessage c:expStack o:@"flutter_error" f:@"" g:@"" s:@"" co:@"" oppId:@""];
                    result(@"1");
                } else if ([@"updateQuestNotifs" isEqualToString:call.method]) {
                    long arg1 = [call.arguments[@"dailyQuestUnlockTime"] longValue];
                    Boolean arg2 = [call.arguments[@"shouldShowDailyQuestNotif"] boolValue];
                    Boolean arg3 = [call.arguments[@"isDailyQuestCompleted"] boolValue];
                    Quests *quests = [Quests get];
                    if (quests && [quests respondsToSelector:@selector(saveNotifData:shouldShowDailyQuestNotif:isDailyQuestCompleted:)]) {
                        [quests saveNotifData:arg1 shouldShowDailyQuestNotif:arg2 isDailyQuestCompleted:arg3];
                    }
                    result(@"1");
                } else if ([@"experiment" isEqualToString:call.method]) {
                    NSString* expName = call.arguments[@"name"];
                    NSString* variant = [Experiment getChosenVariant:expName];
                    NSLog(@"flutteriOS: Handling dart call for experiment:%@ %@", expName, variant);
                    result(variant);
                } else if ([@"data_privacy" isEqualToString:call.method]) {
                    NSLog(@"CCPA - going to call send privacy data to game with argument:%@", call.arguments);
                    [PSPrivacy sendDataPrivacyStatesToGame:call.arguments];
                    result(@"1");
                } else {
                    result(FlutterMethodNotImplemented);
                }
            }];
}

+(void) addToCriticalQueue:(NSString*) routeParams isActivity:(Boolean) isActivity atStart:(Boolean) atStart {
    // Add to critical queue
    if(criticalQueue == nil) {
        criticalQueue = [[NSMutableArray alloc] init];
    }
    NSMutableDictionary *queueData = [NSMutableDictionary new];
    [queueData setObject:routeParams forKey:@"params"];
    [queueData setObject:[NSNumber numberWithBool:isActivity] forKey:@"isActivity"];
    if(atStart) {
        [criticalQueue insertObject:queueData atIndex:0];
    } else {
        [criticalQueue addObject:queueData];
    }
}

-(void) processCriticalQueue {
    if(criticalQueue != nil && [criticalQueue count] > 0) {
        if(self.queueTimer == nil) {
            self.queueTimer = [NSTimer timerWithTimeInterval:1 target:self selector:@selector(queueProcessor:) userInfo:nil repeats:YES];
            [[NSRunLoop currentRunLoop] addTimer:self.queueTimer forMode:NSDefaultRunLoopMode];
        }
    }
}

-(void) queueProcessor:(NSTimer *)timer {
    if(criticalQueue != nil && [criticalQueue count] > 0) {
        NSMutableDictionary *queueData = [criticalQueue firstObject];
        NSString *routeParams = queueData[@"params"];
        NSNumber *isActivity = queueData[@"isActivity"];
        [FlutterBridge sendDataToFlutterModule:routeParams isActivity:[isActivity boolValue]];
        [criticalQueue removeObjectAtIndex:0];
    } else {
        [self.queueTimer invalidate];
        self.queueTimer = nil;
    }
}

@end
