//
//  PSLogin.m
//

#import "PSLogin.h"

static PSLogin *psLogin = nil;
static PSASAuthorizationController *asAuthorizationController = nil;

@implementation PSLogin

+ (void) init {
    if (psLogin == nil) {
        psLogin = [[PSLogin alloc] init];
    }
    
    // controller initialized for apple sign in
    if (asAuthorizationController == nil) {
        asAuthorizationController = [[PSASAuthorizationController alloc] init];
    }

//    [GIDSignIn sharedInstance].clientID = [FIRApp defaultApp].options.clientID;
//    [GIDSignIn sharedInstance].delegate = psLogin;
}

+ (BOOL) handleURL:(NSURL *)url {
    BOOL handled = false;
//    handled = [[GIDSignIn sharedInstance] handleURL:url];
    return handled;
}

+ (void) signInWithGoogle {
    NSLog(@"Google Sign In: - Sign in started");
//    [GIDSignIn sharedInstance].presentingViewController = [PSUtil getRootViewController];
//    [[GIDSignIn sharedInstance] signIn];
}

//- (void)signIn:(GIDSignIn *)signIn
//didSignInForUser:(GIDGoogleUser *)user
//     withError:(NSError *)error {
//    NSLog(@"Google Sign In: Sign in callback received: %@ %@ %@", signIn, user, error);
//    if (error == nil) {
//        GIDAuthentication *authentication = user.authentication;
//        FIRAuthCredential *credential =
//        [FIRGoogleAuthProvider credentialWithIDToken:authentication.idToken
//                                     accessToken:authentication.accessToken];
//        NSLog(@"Google Sign In: Sign in callback received - no error: %@ %@ %@", signIn, user, authentication.idToken);
//        [[FIRAuth auth] signInWithCredential:credential
//        completion:^(FIRAuthDataResult * _Nullable authResult,
//                     NSError * _Nullable error) {
//            NSLog(@"Google Sign In: Sign in result: %@ %@", authResult, error);
//            if (authResult == nil) { return; }
//            FIRUser *user = authResult.user;
//            NSLog(@"Google Sign In: Signed in - user data: %@ %@ %d", user.displayName, user.email, user.isEmailVerified);
//        }];
//    } else {
//        NSLog(@"Google Sign In: Sign in callback received - errored out: %@ %@ %@", signIn, user, error);
//    }
//}

//- (void)signIn:(GIDSignIn *)signIn
//didDisconnectWithUser:(GIDGoogleUser *)user
//     withError:(NSError *)error {
//    // Perform any operations when the user disconnects from app here.
//    // ...
//    NSLog(@"Google Sign In: Sign out callback received: %@ %@ %@", signIn, user, error);
//}

// call this fn to start apple sign in flow
+ (void)startAppleSignIn {
    NSLog(@"Apple Sign in: started apple sign in");
    [asAuthorizationController startSignInWithAppleFlow];
}

@end


/**
 Apple sign in functionality code started
*/
@implementation PSASAuthorizationController

// generates nonce string for sending with request
- (NSString *)randomNonce:(NSInteger)length {
  NSAssert(length > 0, @"Expected nonce to have positive length");
  NSString *characterSet = @"0123456789ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijklmnopqrstuvwxyz-._";
  NSMutableString *result = [NSMutableString string];
  NSInteger remainingLength = length;

  while (remainingLength > 0) {
    NSMutableArray *randoms = [NSMutableArray arrayWithCapacity:16];
    for (NSInteger i = 0; i < 16; i++) {
      uint8_t random = 0;
      int errorCode = SecRandomCopyBytes(kSecRandomDefault, 1, &random);
      NSAssert(errorCode == errSecSuccess, @"Unable to generate nonce: OSStatus %i", errorCode);

      [randoms addObject:@(random)];
    }

    for (NSNumber *random in randoms) {
      if (remainingLength == 0) {
        break;
      }

      if (random.unsignedIntValue < characterSet.length) {
        unichar character = [characterSet characterAtIndex:random.unsignedIntValue];
        [result appendFormat:@"%C", character];
        remainingLength--;
      }
    }
  }

  return result;
}

// this fn triggers the request for sign in and sets delegate for callback
- (void)startSignInWithAppleFlow {
  if (@available(iOS 13.0, *)) {
    NSString *nonce = [self randomNonce:32];
    self.currentNonce = nonce;
      
    ASAuthorizationAppleIDProvider *appleIDProvider = [[ASAuthorizationAppleIDProvider alloc] init];
    ASAuthorizationAppleIDRequest *request = [appleIDProvider createRequest];
    request.requestedScopes = @[ASAuthorizationScopeFullName, ASAuthorizationScopeEmail];
    request.nonce = [self stringBySha256HashingString:nonce];

    ASAuthorizationController *authorizationController =
        [[ASAuthorizationController alloc] initWithAuthorizationRequests:@[request]];
    authorizationController.delegate = self;
    authorizationController.presentationContextProvider = self;
    [authorizationController performRequests];
    NSLog(@"Apple Sign in: request sent");
  }
}

// utility fn for the nonce request
- (NSString *)stringBySha256HashingString:(NSString *)input {
  const char *string = [input UTF8String];
  unsigned char result[CC_SHA256_DIGEST_LENGTH];
  CC_SHA256(string, (CC_LONG)strlen(string), result);

  NSMutableString *hashed = [NSMutableString stringWithCapacity:CC_SHA256_DIGEST_LENGTH * 2];
  for (NSInteger i = 0; i < CC_SHA256_DIGEST_LENGTH; i++) {
    [hashed appendFormat:@"%02x", result[i]];
  }
  return hashed;
}

// success delegate - will contain the information like user id and email.
- (void)authorizationController:(ASAuthorizationController *)controller
   didCompleteWithAuthorization:(ASAuthorization *)authorization API_AVAILABLE(ios(13.0)) {
  NSLog(@"Apple Sign in: callback received");
  if ([authorization.credential isKindOfClass:[ASAuthorizationAppleIDCredential class]]) {
    ASAuthorizationAppleIDCredential *appleIDCredential = authorization.credential;
    NSString *rawNonce = self.currentNonce;
    NSLog(@"Apple Sign in: inside authorization %@ %@", appleIDCredential.user, appleIDCredential.email, appleIDCredential.fullName);
    
      
    NSAssert(rawNonce != nil, @"Invalid state: A login callback was received, but no login request was sent.");

    if (appleIDCredential.identityToken == nil) {
      NSLog(@"Unable to fetch identity token.");
      return;
    }

    NSString *idToken = [[NSString alloc] initWithData:appleIDCredential.identityToken
                                              encoding:NSUTF8StringEncoding];
    if (idToken == nil) {
      NSLog(@"Unable to serialize id token from data: %@", appleIDCredential.identityToken);
    }

    // Initialize a Firebase credential.
//    FIROAuthCredential *credential = [FIROAuthProvider credentialWithProviderID:@"apple.com"
//                                                                        IDToken:idToken
//                                                                       rawNonce:rawNonce];

    // Sign in with Firebase.
//    [[FIRAuth auth] signInWithCredential:credential
//                              completion:^(FIRAuthDataResult * _Nullable authResult,
//                                           NSError * _Nullable error) {
//      if (error != nil) {
//        // Error. If error.code == FIRAuthErrorCodeMissingOrInvalidNonce,
//        // make sure you're sending the SHA256-hashed nonce as a hex string
//        // with your request to Apple.
//          NSLog(@"Apple Sign in: fail");
//         [Track trackCounter:TRACK_LOGIN p:@"" c:@"" o:TRACK_APPLE f:TRACK_FAIL g:@"" s:@"" co:@"" oppId:@""];
//        return;
//      }
//        // Sign-in succeeded!
//        NSLog(@"Apple Sign in: success");
//         [Track trackCounter:TRACK_LOGIN p:@"" c:@"" o:TRACK_APPLE f:TRACK_SUCCESS g:@"" s:@"" co:@"" oppId:@""];
//
//        // sending back data to js side on successful login
//        NSString *user = appleIDCredential.user;
//        NSString *email = appleIDCredential.email;
//        email = [self sanitizeString:email];
//
//        // EDIT: fetching full name for the user
//        NSString *firstName = appleIDCredential.fullName.givenName;
//        firstName = [self sanitizeString:firstName];
//        NSString *middleName = appleIDCredential.fullName.middleName;
//        middleName = [self sanitizeString:middleName];
//        NSString *familyName = appleIDCredential.fullName.familyName;
//        familyName = [self sanitizeString:familyName];
//
//        // EDIT: appending name based on null string
//        NSString *fullName = firstName;
//        if (middleName.length != 0) {
//            fullName = [NSString stringWithFormat: @"%@ %@", fullName, middleName];
//        }
//        if (familyName.length != 0) {
//            fullName = [NSString stringWithFormat: @"%@ %@", fullName, familyName];
//        }
//        fullName = [self sanitizeString:fullName];
//
//        NSMutableDictionary* dict = [[NSMutableDictionary alloc] init];
//        [dict setObject:@"apple" forKey:@"type"];
//        [dict setObject:user forKey:@"appleid"];
//        [dict setObject:email forKey:@"emailID"];
//        [dict setObject:fullName forKey:@"fullName"];
//        [GameSpecific triggerLoginSuccessActions:dict];
//    }];
  }
}

// checks for null and empty string
- (NSString*) sanitizeString:(NSString *) str {
    if (str.length == 0) { // when str is nil and empty
        str = @"";
    }
    return str;
}

// fail delegate - will contain the error while signing in
- (void)authorizationController:(ASAuthorizationController *)controller
           didCompleteWithError:(NSError *)error API_AVAILABLE(ios(13.0)) {
  NSLog(@"Sign in with Apple errored: %@", error);
}

/**
   end Apple sign in
*/

@end


