#import "SqliteHelper.h"

@implementation SqliteHelper

static SqliteHelper *helper;
static SqliteHelper *writeHelper;


+(SqliteHelper*) get:(int)mode;{
    SqliteHelper *currDB;
    BOOL success = true;
    if (mode == READONLY) {
        if(helper==nil){
            helper=[[SqliteHelper alloc] init];
            success = [helper initializeDB:mode];
        }
        currDB = helper;
    }else if(mode == READWRITE){
        if(writeHelper==nil){
            writeHelper=[[SqliteHelper alloc] init];
            success= [writeHelper initializeDB:mode];
        }
        currDB = writeHelper;
    }
    
    if (!success) {
        ENDebug(@"ERROR in opening database in mode  =  %d", mode);
        return nil;
    }else
        return currDB;
}

-(BOOL) initializeDB:(int) mode;{
    NSString *docDirectory = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) firstObject];
    int result = -1;
    if (mode == READONLY) {
        NSString *sqLiteDb = [NSString stringWithFormat:@"%@/%@", docDirectory, SQLLITEDB];
        result = sqlite3_open_v2([sqLiteDb UTF8String], &database, SQLITE_OPEN_READONLY, NULL);
    }else{
        NSString *sqLiteDb = [NSString stringWithFormat:@"%@/%@", docDirectory, SQLLITEDB_WRITE];
        result = sqlite3_open([sqLiteDb UTF8String], &database);
    }
    
    BOOL success = false;
    if (result != SQLITE_OK && result != SQLITE_DONE) {
        ENDebug(@"Failed to open database in mode = %d", mode);
        success = false;
    }else{
        ENDebug(@"database opened successfully mode = %d", mode);
        if (mode == READWRITE) {
            NSString *initQuery = [NSString stringWithFormat:@"CREATE TABLE IF NOT EXISTS %@ (key TEXT PRIMARY KEY,value TEXT);", SQLLITE_TABLE];
            sqlite3_stmt *statement;
            if(sqlite3_prepare_v2(database, [initQuery UTF8String], -1, &statement, nil) \
               ==SQLITE_OK) {
                result = sqlite3_step(statement);
                if (result != SQLITE_OK) {
                    success = false;
                }
            }
        }
        success = true;
    }
    return success;
}


-(NSString *)getData: (NSString*) key {
    NSString *data = nil;
    _selectQuery = [NSString stringWithFormat:@"SELECT value from %@ where key \
                    = '%@'", SQLLITE_TABLE,key];
    @try {
        if (sqlite3_prepare_v2(database, [_selectQuery UTF8String], -1, &_stmt_select, nil)\
            == SQLITE_OK) {
            while (sqlite3_step(_stmt_select) == SQLITE_ROW) {
                char *dataChars = (char *) sqlite3_column_text(_stmt_select, 0);
                data = [[NSString alloc] initWithUTF8String:dataChars];
            }
        }
    }
    @catch (NSException *exception) {
        ENError(@"%@", [exception description]);
    }
    return data;
}



-(Boolean) setItem:(NSString*) key value:(NSString*) value;
{
    Boolean  flag = false;
    @try {
        _updateQuery = [NSString stringWithFormat:@"replace into %@ (key,value)values('%@','%@')",\
                        SQLLITE_TABLE, key, value];
        
        if (sqlite3_prepare_v2(database, [_updateQuery UTF8String], -1, &_stmt_update, nil)\
            == SQLITE_OK) {
            flag |= sqlite3_step(_stmt_update);
            if( flag != SQLITE_OK && flag != SQLITE_DONE){
                ENError(@"Error in storing in local storage" );
            }
        }else{
            flag = false;
        }
        ENDebug(@"setItem complete");
    } @catch (NSException *e) {
        ENInfo(@"%@", [e description]);
        flag = false;
    }
    return flag;
}


-(Boolean) removeItem:(NSString*) key;
{
    Boolean flag = false;
    @try {
        _removeQuery = [NSString stringWithFormat:@"delete from %@ where key='%@'", \
                        SQLLITE_TABLE, key];
        if (sqlite3_prepare_v2(database, [_removeQuery UTF8String], -1, &_stmt_remove, nil)\
            == SQLITE_OK) {
            sqlite3_step(_stmt_remove);
            flag = true;
        }else{
            flag = false;
        }
    } @catch (NSException *e) {
        ENDebug(@"%@", [e description]);
        flag = false;
    }
    return flag;
}

-(void) closeConnection;{
    //    sqlite3_close(database);
}

- (void)dealloc
{
    //    sqlite3_close(database);
}

-(void) close;{
    ENError(@"debug");
}

@end
