#import <Foundation/Foundation.h>

#import "User.h"
#import "Constants.h"
#import "Logging.h"
#import "Util.h"
#import "FlutterBridge.h"

@implementation User
User* user;

-(User*) init{
    self = [super init];
    
    self.refId = @"";
    self.syncId = @"";
    self.trackingRefId = @"";
    self.xpromoId = @"";
    self.gpsid = @"";
    self.advertiserId = @"";
    self.name = @"";
    self.email = @"";
    self.gender = @"";
    self.fbid = @"";
    self.curDevice = @"";
    self.deviceId = @"";
    self.deviceHash = @"";
    self.infoHash=@"";
    self.gameUserInfoHash = @"";
    self.xpromoHash = @"";
    self.logoutClicked = false;
    self.countryCode = @"";
    self.subDivision = @"";
    self.city = @"";
    self.installNetwork = @"";
    self.installCampaign = @"";
    self.installAdGroup = @"";
    self.installCreative = @"";
    self.trackerToken = @"";
    self.chosenName = @"";
    self.chosenEmail = @"";
    
    database = [SqliteHelper get:READONLY];
    return self;
}

+(User*) get{
    if(user==nil || [[user refId] isEqualToString:@""]){
        user=[[User alloc] init];
        [user load];
    }
    return user;
}


-(Boolean) load {
    NSString *user_data = [database getData:USER_DATA_FILE];
    if (user_data == nil) {
        _isFirstLaunch = true;
    }
    if(user_data){
        NSError *e = nil;
        NSDictionary *userObj = [Util stringToJson:user_data];
        if(e){
            ENError(@"No data for user yet %@", e.description);
            return false;
        }
        
        self.refId = userObj[@"refId"];
        self.name = userObj[@"name"];
        self.email = userObj[@"email"];
        self.gender = userObj[@"gender"];
        self.fbid = userObj[@"fbid"];
        self.curDevice = userObj[@"curDevice"];
        if([userObj objectForKey:@"deviceId"]!=nil){
            self.deviceId = userObj[@"deviceId"];
        }
        self.infoHash = userObj[@"infoHash"];
        self.deviceHash = userObj[@"deviceHash"];
        if([userObj objectForKey:@"gi"]!=nil){
            self.gameUserInfoHash = userObj[@"gi"];
        }
        if([userObj objectForKey:@"xh"]!=nil){
            self.xpromoHash = userObj[@"xh"];
        }
        self.logoutClicked = false;
        
        if([userObj objectForKey:@"httpConnectionFailure"]!=nil){
            _httpConnectionFailure = [[userObj objectForKey:@"httpConnectionFailure"] intValue];
        }
        
        if([userObj objectForKey:@"trackingRefId"]!=nil){
            self.trackingRefId = [userObj objectForKey:@"trackingRefId"];
        } else {
            self.trackingRefId = self.refId;
        }
        if([userObj objectForKey:@"chosenName"]!=nil){
            self.chosenName = userObj[@"chosenName"];
        }
        if([userObj objectForKey:@"chosenEmail"]!=nil){
            self.chosenEmail = userObj[@"chosenEmail"];
        }
        if([userObj objectForKey:@"syncId"]!=nil){
            self.syncId = [userObj objectForKey:@"syncId"];
        } else {
            self.syncId = self.refId;
        }
        if([userObj objectForKey:@"xpromoId"]!=nil){
            self.xpromoId = [userObj objectForKey:@"xpromoId"];
        } else {
            self.xpromoId = self.refId;
        }
        if([userObj objectForKey:@"gpsid"]!=nil){
            self.gpsid = [userObj objectForKey:@"gpsid"];
        } else {
            self.gpsid = self.trackingRefId;
        }
        NSLog(@"Successfully loaded user data:%@", self.trackingRefId);
        
        return true;
    }else{
        ENInfo(@"No stored data for user yet");
        return false;
    }
}


+(NSString *) getName; {
    return user.name;
}

+(NSString *) getRefId; {
    return user.syncId;
}

+(NSString *) getPsId; {
    return user.refId;
}

+(NSString *) getTrackingRefId; {
    return user.trackingRefId;
}

+(void) updateNameEmail:(NSString*)name email:(NSString*)email {
    NSMutableDictionary* dict = [[NSMutableDictionary alloc] init];
    [dict setObject:name forKey:@"cn"];
    [dict setObject:email forKey:@"ce"];
    
    user.chosenEmail = email;
    user.chosenName = name;
    NSString* msg = [Util JSONToString:dict];
    [Util sendJSCallBack:@"userObj.updateNameEmail" msg:msg];
}

+(void) showPrivacy:(NSString*)theme refId:(NSString*)refId trackingRefId:(NSString*)trackingRefId countryCode:(NSString*)countryCode subDivision:(NSString*)subDivision law:(NSString*)law {
    NSLog(@"flutter - showing privacy flow: %@ %@", theme, countryCode);
    @try {
        NSLog(@"flutter - showing privacy flow 1: %@ %@", refId, trackingRefId);
        NSDictionary *flutterData = @{
                                      @"route": @"privacy",
                                      @"theme": theme,
                                      @"r": refId,
                                      @"tr": trackingRefId,
                                      @"g": [NSString stringWithFormat:@"%d", GAME_ID_VAL],
                                      @"c": [NSString stringWithFormat:@"%d", CLIENT_ID],
                                      @"cc": countryCode,
                                      @"su": subDivision,
                                      @"law": law,
                                      };
        NSError *error;
        NSData *jsonData = [NSJSONSerialization dataWithJSONObject:flutterData options:0 error:&error];
        [FlutterBridge sendDataToFlutterModule:[[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding] isActivity:true isCritical:true];
    } @catch (NSException *exception) {
        NSLog(@"flutter - exception in privacy: %@", exception);
    }
    return;
}

+(void) updateChosenCountry:(NSString*)chosenCountry {
    NSMutableDictionary* dict = [[NSMutableDictionary alloc] init];
    [dict setObject:chosenCountry forKey:@"cc"];
    
    NSString* msg = [Util JSONToString:dict];
    [Util sendJSCallBack:@"userObj.updateChosenCountry" msg:msg];
}

+(void) updateAgeRange:(NSString*)cmin cmax:(NSString*)cmax {
    NSMutableDictionary* dict = [[NSMutableDictionary alloc] init];
    [dict setObject:cmin forKey:@"cmin"];
    [dict setObject:cmax forKey:@"cmax"];
    
    NSString* msg = [Util JSONToString:dict];
    [Util sendJSCallBack:@"userObj.updateAgeRange" msg:msg];
}

+(void) acceptPrivacyPolicy {
    NSMutableDictionary* dict = [[NSMutableDictionary alloc] init];
    
    NSString* msg = [Util JSONToString:dict];
    [Util sendJSCallBack:@"userObj.acceptPrivacyPolicy" msg:msg];
}


@end
