PSRenderRate = function () {
    var wordCount = 0;
    var interstitialLimits = [0, 0, 0, 0, 0];
    var interstitialLoadAttempted = false;
    var PACKAGE_PATH_MOPUB_ANDROID = "in/playsimple/common/mopub/";

    // This is to be called by the game code, to set the limits correctly for the particular puzzle.
    // Should be called whenever user opens up a puzzle, so that we have the right limits for that type of puzzle.
    // Example call: psRenderRateObj.setInterstitialLimits([0, mainWords1, mainWords2, mainWords1, mainWords2]);
    this.setInterstitialLimits = function (limits) {
        for (var i = 0; i < limits.length && i < interstitialLimits.length; i++) {
            interstitialLimits[i] = limits[i];
        }
        interstitialLoadAttempted = false; // This is reset, since it's a new puzzle now.
    };

    // This is to be called by the game code, with the initial number of words solved for that particular puzzle 
    // (since some puzzles may have words in the half solved state)
    this.setWordCount = function (wc) {
        wordCount = wc;
    };

    // Pass the updated counter value whenever the user solves a word - or multiple words get solved in one go because of some hints.
    // Once the value is set, it is compared with the limits and ad load request is sent at the appropriate time.
    this.updateWordCountAndAttemptLoad = function (wc, isInterstitialInCooldown, trackOrder) {
        wordCount = wc;
        if (interstitialLoadAttempted || isInterstitialInCooldown) {
            return;
        }

        var variant = epmObj.getChosenVariant(adsObj.EXP_RENDER_RATE_INTERSTITIAL);
        if (variant == 0) {
            return;
        }
        if (wordCount >= interstitialLimits[variant]) {
            interstitialLoadAttempted = true;
            trackObj.trackCounter(psConstantsObj.TRACK_AD_TRACKING, psConstantsObj.TRACK_INTERSTITIAL, "eligible", trackOrder, utilObj.isOnline(), "mopub", "", 1, "");
            psMopubAdsObj.retryAdLoadForPlacementType(psMopubAdsObj.MOPUB_AD_TYPE_INTERSTITIAL, psMopubAdsObj.PS_AD_TYPE_INTERSTITIAL);
        }
    };

    this.getRenderRateExperimentVariant = function () {
        return epmObj.getChosenVariant(adsObj.EXP_RENDER_RATE_INTERSTITIAL);
    };

    this.getInterstitialLimits = function (index) {
        if (index >= 0 && index < interstitialLimits.length) {
            return interstitialLimits[index];
        } else {
            return 0;
        }
    };

    this.resetRetryCount = function (adType) {
        var variant = epmObj.getChosenVariant(adsObj.EXP_RENDER_RATE_INTERSTITIAL);
        if (variant == 0) {
            return;
        }
        if (utilObj.isAndroid()) {
            psUtilObj.makeNativeCall(psConstantsObj.PACKAGE_PATH_MOPUB_ANDROID + "PSMopubAds", "resetRetryCount", "(I)V", adType);
        } else if (utilObj.isIos()) {
            psUtilObj.makeNativeCall("PSMopubAds", "resetRetryCount:", adType);
        }
    };
};