FbRequest = function () {
    var reqController = null;
    var openReqs = null;
    var openOfType = [];
    var lastUpdate = 0;

    this.type = constantsObj.INVITE;

    var numCoinsAccepted = 0;
    var numLivesAccepted = 0;
    var numMovesAccepted = 0;
    var numSpinsAccepted = 0;

    var NUM_REQ_TYPES = 11;

    // This set of constants is also declared in constants.js.
    // Remember to add there as well.
    this.INVITE = 0;
    this.ASK_COINS = 1;
    this.SEND_COINS = 2;
    this.ASK_LIVES = 3;
    this.SEND_LIVES = 4;
    this.ASK_MOVES = 5;
    this.SEND_MOVES = 6;
    this.ASK_SPINS = 7;
    this.SEND_SPINS = 8;
    this.CREATURE_COINS = 9;
    this.CREATURE_SPINS = 10;

    var p = "",
        c = "",
        o = "",
        s = "";

    this.init = function () {
        // reqController = gameObj.getGameController();
        if (utilObj.isNative()) {
            reqController = new Controller(constantsObj.GAME_SERVER, constantsObj.GAME_SERVER_REST);
        } else {
            reqController = new Controller(constantsObj.GAME_SERVER_SECURE);
        }
        load();
    };

    function loadFbReqsObjLocal(fbReqData) {
        var fbReqInfo = JSON.parse(fbReqData);
        if (fbReqInfo.lastUpdate == undefined) {
            return;
        }
        lastUpdate = fbReqInfo.lastUpdate;
        openOfType = fbReqInfo.openOfType;
        numCoinsAccepted = fbReqInfo.numCoinsAccepted;
        numLivesAccepted = fbReqInfo.numLivesAccepted;
        numMovesAccepted = fbReqInfo.numMovesAccepted;
        if(fbReqInfo.numSpinsAccepted != undefined) {
            numSpinsAccepted = fbReqInfo.numSpinsAccepted;
        }
    }

    function load() {
        var fbReqData = ls.getItem(constantsObj.FB_REQUESTS_FILE);
        if (fbReqData == null || fbReqData == "") {
            // loggerObj.debugLog("No requests data found.");
            return false;
        }
        loadFbReqsObjLocal(fbReqData);

        fbRequestObj.processOpenInvites();
        return true;
    }

    this.resetDailyGifts = function () {
        numCoinsAccepted = 0;
        numSpinsAccepted = 0;
        numLivesAccepted = 0;
        numMovesAccepted = 0;
        fbRequestObj.save();
    };

    function getFbReqsObj() {
        var fbReqInfo = {};
        fbReqInfo.lastUpdate = lastUpdate;
        fbReqInfo.openOfType = openOfType;
        fbReqInfo.numCoinsAccepted = numCoinsAccepted;
        fbReqInfo.numSpinsAccepted = numSpinsAccepted;
        fbReqInfo.numLivesAccepted = numLivesAccepted;
        fbReqInfo.numMovesAccepted = numMovesAccepted;

        return fbReqInfo;
    }

    this.reset = function () {
        openReqs = null;
        openOfType = [];
        lastUpdate = 0;
        numCoinsAccepted = 0;
        numSpinsAccepted = 0;
        numLivesAccepted = 0;
        numMovesAccepted = 0;

        fbRequestObj.save();
    };

    this.save = function () {
        if (lastUpdate === undefined) {
            // loggerObj.debugLog("No need to save friends info");
            return false;
        }

        var fbReqInfo = getFbReqsObj();
        ls.setItem(constantsObj.FB_REQUESTS_FILE, utilObj.getStringifiedJson(fbReqInfo));

        // loggerObj.debugLog("Saving fb requests info:", fbReqInfo);
        return true;
    };

    this.determineReqType = function (reqInfo) {
        var msg = reqInfo.message.toLowerCase();
        var reqType = fbRequestObj.INVITE;

        // loggerObj.debugLog("Searching coins in :" + msg);
        if (msg.search("lives") != constantsObj.NOT_FOUND) {
            if (msg.search("send") != constantsObj.NOT_FOUND) {
                // example: Hey, I’m out of coins on WordUp - can you send me some?
                reqType = fbRequestObj.ASK_LIVES;
            } else {
                // example: Hey, here are some WordUp coins for you!
                reqType = fbRequestObj.SEND_LIVES;
            }
        } else if (msg.search("coins") != constantsObj.NOT_FOUND) {
            if (msg.search("send") != constantsObj.NOT_FOUND) {
                reqType = fbRequestObj.ASK_COINS;
            } else {
                reqType = fbRequestObj.SEND_COINS;
            }
        } else if (msg.search("spins") != constantsObj.NOT_FOUND) {
            if (msg.search("send") != constantsObj.NOT_FOUND) {
                reqType = fbRequestObj.ASK_SPINS;
            } else {
                reqType = fbRequestObj.SEND_SPINS;
            }
        } else if (msg.search("moves") != constantsObj.NOT_FOUND) {
            if (msg.search("send") != constantsObj.NOT_FOUND) {
                reqType = fbRequestObj.ASK_MOVES;
            } else {
                reqType = fbRequestObj.SEND_MOVES;
            }
        }
        return reqType;
    };

    this.setOpenReqs = function (reqData) {
        openReqs = reqData;
    };

    this.populateReqTypes = function () {
        if (openReqs == null) {
            return;
        }
        var friendMap = {};

        for (var i = 0; i < NUM_REQ_TYPES; i++) {
            openOfType[i] = [];
            friendMap[i] = {};
            friendMap[i]["count"] = 0;
        }

        for (var i = 0; i < openReqs.length; i++) {
            var reqType = fbRequestObj.determineReqType(openReqs[i]);
            openOfType[reqType].push(openReqs[i]);
            var friendId = 0;
            if (friendId.from) {
                friendId = openReqs[i].from.id;
            }
            if (friendMap[reqType][friendId] == undefined) {
                friendMap[reqType][friendId] = 1;
                friendMap[reqType]["count"] = friendMap[reqType]["count"] + 1
            }
        }
        var req_info = "";
        for (var i = 0; i < NUM_REQ_TYPES; i++) {
            req_info += i + ":" + friendMap[i]["count"] + ";";
        }

        trackObj.trackCounter(psConstantsObj.TRACK_DEBUG, psConstantsObj.TRACK_REQUEST, req_info, "", "", "", "", "", "");
        // loggerObj.debugLog("Freshly populated req types:", openOfType, openReqs);
    };

    this.storeReqId = function (reqId) {
        var map = userObj.getUserObjForSync();
        map.reqId = reqId;
        // loggerObj.debugLog("Attempting to store req id:", reqId);
        reqController.sendHttpRequest("fbRequest", "store", map);
    };

    this.grantReferralBonus = function (reqId) {
        var map = userObj.getUserObjForSync();
        map.reqId = reqId;
        reqController.sendHttpRequest("fbRequest", "grantReferralBonus", map);
    };

    this.checkOpenRequests = function () {
        fbWrapperObj.getAllRequests();
    };

    this.clearRequests = function (reqType) {
        if (openOfType[reqType] == undefined) {
            return;
        }
        for (var i = 0; i < openOfType[reqType].length; i++) {
            fbWrapperObj.deleteRequest(openOfType[reqType][i].id);
        }
        openOfType[reqType] = [];
        fbRequestObj.save();
    };

    this.handleOpenRequests = function (data) {
        fbRequestObj.setOpenReqs(data);
        fbRequestObj.populateReqTypes();

        fbRequestObj.processOpenInvites();
        lastUpdate = utilObj.getCurrentTimestamp();
        fbRequestObj.save();
    };

    this.processOpenInvites = function () {
        if (openOfType[fbRequestObj.INVITE] == undefined || openOfType[fbRequestObj.INVITE] == null || openOfType[fbRequestObj.INVITE].length == 0) {
            // loggerObj.debugLog("No open invites");
            return;
        }
        if (gameObj.getReferrer() == "") {
            // Grant bonus to last person who invited.
            var lastIndex = openOfType[fbRequestObj.INVITE].length - 1;
            var reqId = fbRequestObj.getReqIdFromReqObjId(openOfType[fbRequestObj.INVITE][lastIndex].id);
            // loggerObj.debugLog("Referrer is empty. Attempting ref bonus grant for reqId:", reqId);
            // loggerObj.debugLog("All open invites:", utilObj.getStringifiedJson(openOfType[fbRequestObj.INVITE]));
            fbRequestObj.grantReferralBonus(reqId);
        } else {
            if (!utilObj.isAndroid()) {
                // loggerObj.debugLog("Referrer already set. Clearing invites.");
                fbRequestObj.clearOpenInvites();
            }
        }
    };

    this.respHandler = function (action, data, success) {
        // loggerObj.debugLog("Action:", action, utilObj.getStringifiedJson(data), success);
    };

    this.getReqIdFromReqObjId = function (reqObjId) {
        var res = reqObjId.split("_");
        return res[0];
    };

    this.clearOpenInvites = function () {
        fbRequestObj.clearRequests(fbRequestObj.INVITE);
    };

    function wasSendSuccessful(code, msg) {
        var sentTo = msg.to;
        if (sentTo == undefined) {
            return false;
        }
        return true;
    }

    function sendRequests(title, cb, to, msg, c1, o1, s1, reqType) {
        c = c1;
        o = o1;
        s = s1;
        if (to == "" || to == null || to == undefined) {
            fbRequestObj.inviteFriends(title, to, c, o, s);
        } else {
            p = psConstantsObj.TRACK_APP;
            fbWrapperObj.sendRequests(title, msg, cb, to, p, c, o, s, reqType);
        }
    }

    this.reqSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        friendsObj.sendSuccessful(msg);
    };

    this.sendCoins = function (to, msg, c1, o1, s1) {
        var title = stringsObj.send_coins;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.SEND_COINS;
        friendsObj.setReqType(constantsObj.SEND_COINS);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };

    this.askCoins = function (to, msg, c1, o1, s1) {
        var title = stringsObj.ask_coins;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.ASK_COINS;
        friendsObj.setReqType(constantsObj.ASK_COINS);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };

    this.askSpins = function (to, msg, c1, o1, s1) {
        var title = stringsObj.ask_spins;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.ASK_SPINS;
        friendsObj.setReqType(constantsObj.ASK_SPINS);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };

    this.sendSpins = function (to, msg, c1, o1, s1) {
        var title = stringsObj.send_spins;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.SEND_SPINS;
        friendsObj.setReqType(constantsObj.SEND_SPINS);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };

    this.askLives = function (to, msg, c1, o1, s1) {
        var title = stringsObj.ask_lives;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.ASK_LIVES;
        friendsObj.setReqType(constantsObj.ASK_LIVES);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };

    this.sendLives = function (to, msg, c1, o1, s1) {
        var title = stringsObj.send_lives;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.SEND_LIVES;
        friendsObj.setReqType(constantsObj.SEND_LIVES);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };

    this.askMoves = function (to, msg, c1, o1, s1) {
        var title = stringsObj.ask_moves;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.ASK_MOVES;
        friendsObj.setReqType(constantsObj.ASK_MOVES);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };

    this.sendMoves = function (to, msg, c1, o1, s1) {
        var title = stringsObj.send_moves;
        var cb = fbRequestObj.reqSuccess;
        fbRequestObj.type = constantsObj.SEND_MOVES;
        friendsObj.setReqType(constantsObj.SEND_MOVES);
        sendRequests(title, cb, to, msg, c1, o1, s1, fbRequestObj.type);
    };


    this.inviteFriends = function (title, to, c1, o1, s1) {
        fbRequestObj.type = constantsObj.INVITE;
        friendsObj.setReqType(constantsObj.INVITE);
        p = psConstantsObj.TRACK_NON_APP;
        if (c1 != null) {
            c = c1;
        }
        if (o1 != null) {
            o = o1;
        }
        if (s1 != null && s1 != undefined) {
            s = s1;
        }

        if (!fbWrapperObj.isLoggedIn()) {
            var success = fbWrapperObj.checkAndShowFbConnect(constantsObj.FB_ACTION_INVITE);

            if (success == false) {
                fbWrapperObj.setFbAction(constantsObj.FB_ACTION_INVITE);
                fbWrapperObj.login();
            }
            return;
        }
        if (to == "" || to == null || to == undefined) {
            to = friendsObj.getNextInvitableBatch();
        }
        if (title == "" || title == undefined || title == null) {
            title = stringsObj.invite_friends;
        }
        fbWrapperObj.sendRequests(title, stringsObj.req_invite_desc, fbRequestObj.inviteSuccess, to, p, c, o);
    };

    this.inviteSuccess = function (code, msg, parentScreen, type) {
        if (!wasSendSuccessful(code, msg)) {
            if (curScreen != constantsObj.SCREEN_MFS
                && curScreen != psConstantsObj.SCREEN_LEVEL_INTRO
                && curScreen != psConstantsObj.SCREEN_LEVEL_OUTRO) {
                fbRequestObj.showMFS();
            }
            return;
        }
        friendsObj.sendSuccessful(msg, parentScreen);
    };

    this.getOpenReqCount = function () {
        load();
        var curTimestamp = utilObj.getCurrentTimestamp();
        if (curTimestamp - lastUpdate > constantsObj.FB_REQ_REFRESH_INTERVAL) {
            fbRequestObj.checkOpenRequests();
        }
        var reqCount = 0;
        // Not counting invites in open requests.
        for (var i = 1; i < NUM_REQ_TYPES; i++) {
            if (openOfType[i] != undefined) {
                var friendMap = {};
                var friendCount = 0;
                for (var j = 0; j < openOfType[i].length; j++) {
                    if (!openOfType[i][j].hasOwnProperty("from")) {
                        continue;
                    }

                    var friendId = openOfType[i][j].from.id;
                    if (friendMap[friendId] == undefined) {
                        friendMap[friendId] = 1;
                        friendCount++;
                    }
                }
                reqCount += friendCount;
            }
        }

        return reqCount;
    };

    function getReqInfo(reqType) {
        // loggerObj.debugLog("fetch requests = ", reqType);
        var info = null;
        if (openOfType[reqType] == undefined || openOfType[reqType].length == 0 || openOfType[reqType][0].from == undefined
            || openOfType[reqType][0].from.id == undefined) {
            return info;
        }
        var friendPic = friendsObj.getPlayingFriendPic(openOfType[reqType][0].from.id);
        var firstName = utilObj.getFirstName(openOfType[reqType][0].from.name);
        firstName = utilObj.formatName(firstName);
        var friendCount = 0;
        var friendMap = {};
        for (var i = 0; i < openOfType[reqType].length; i++) {
            var obj = openOfType[reqType][i];
            if (obj == undefined || obj.from == undefined || obj.from.id == undefined) {
                continue;
            }

            var friendId = openOfType[reqType][i].from.id;
            if (friendMap[friendId] == undefined) {
                friendMap[friendId] = 1;
                friendCount++;
            }
        }

        info = {
            'name': firstName,
            'pic': friendPic,
            'reqCount': openOfType[reqType].length,
            'count': friendCount,
            'reqType': reqType
        };
        return info;
    }

    this.getRcvdCoinsInfo = function () {
        return getReqInfo(fbRequestObj.SEND_COINS);
    };

    this.getAskedCoinsInfo = function () {
        return getReqInfo(fbRequestObj.ASK_COINS);
    };

    this.getRcvdSpinsInfo = function () {
        return getReqInfo(fbRequestObj.SEND_SPINS);
    };

    this.getAskedSpinsInfo = function () {
        return getReqInfo(fbRequestObj.ASK_SPINS);
    };

    this.getRcvdCreatureSpinsInfo = function () {
        return getReqInfo(fbRequestObj.CREATURE_SPINS);
    };

    this.getRcvdCreatureCoinsInfo = function () {
        return getReqInfo(fbRequestObj.CREATURE_COINS);
    };

    this.getAskedSpinsInfo = function () {
        return getReqInfo(fbRequestObj.ASK_SPINS);
    };

    this.getRcvdLivesInfo = function () {
        return getReqInfo(fbRequestObj.SEND_LIVES);
    };

    this.getAskedLivesInfo = function () {
        return getReqInfo(fbRequestObj.ASK_LIVES);
    };

    this.getRcvdMovesInfo = function () {
        return getReqInfo(fbRequestObj.SEND_MOVES);
    };

    this.getAskedMovesInfo = function () {
        return getReqInfo(fbRequestObj.ASK_MOVES);
    };

    this.sendTo = function (type, c1) {
        if (type != fbRequestObj.ASK_LIVES && type != fbRequestObj.ASK_COINS && type != fbRequestObj.ASK_MOVES && type != fbRequestObj.ASK_SPINS) {
            return;
        }

        fbRequestObj.type = type;
        if (openOfType[type] == undefined || openOfType[type].length == 0) {
            return;
        }

        var to = "";
        var friendMap = {};
        for (var i = 0; i < openOfType[type].length; i++) {
            var friendId = openOfType[type][i].from.id;
            if (friendMap[friendId] == undefined) {
                friendMap[friendId] = 1;
                if (friendsObj.canSendReqTo(friendId)) {
                    to += friendId + ",";
                    friendsObj.addPlayingFriend(friendId);
                }
            }
        }
        to = to.replace(/,$/, "");

        c = c1;
        o = "msg_response";
        var title = null,
            desc = null,
            cb = null;
        switch (type) {
            case fbRequestObj.ASK_LIVES:
                title = stringsObj.send_lives;
                desc = stringsObj.req_send_lives_desc;
                cb = fbRequestObj.sendLivesToSuccess;
                break;
            case fbRequestObj.ASK_COINS:
                title = stringsObj.send_coins;
                desc = stringsObj.req_send_coins_desc;
                cb = fbRequestObj.sendCoinsToSuccess;
                fbRequestObj.type = fbRequestObj.SEND_COINS;
                break;
            case fbRequestObj.ASK_SPINS:
                title = stringsObj.send_spins;
                desc = stringsObj.req_send_spins_desc;
                cb = fbRequestObj.sendSpinsToSuccess;
                fbRequestObj.type = fbRequestObj.SEND_SPINS;
                break;
            case fbRequestObj.ASK_MOVES:
                title = stringsObj.send_moves;
                desc = stringsObj.req_send_moves_desc;
                cb = fbRequestObj.sendMovesToSuccess;
                break;
        }
        sendRequests(title, cb, to, desc, c, o, "");
    };

    this.sendLivesToSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        fbRequestObj.clearRequests(fbRequestObj.ASK_LIVES);
        friendsObj.sendSuccessful(msg);


        showSentMsg(utilObj.ucFirst("lives"));
    };

    this.sendCoinsToSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        fbRequestObj.clearRequests(fbRequestObj.ASK_COINS);
        friendsObj.sendSuccessful(msg);


        showSentMsg(utilObj.ucFirst("coins"));
    };

    this.sendSpinsToSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        fbRequestObj.clearRequests(fbRequestObj.ASK_SPINS);
        friendsObj.sendSuccessful(msg);


        showSentMsg(utilObj.ucFirst("spins"));
    };

    this.sendMovesToSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        fbRequestObj.clearRequests(fbRequestObj.ASK_MOVES);
        friendsObj.sendSuccessful(msg);


        showSentMsg(utilObj.ucFirst("moves"));
    };


    this.acceptAndSendCoins = function () {
        fbRequestObj.acceptAndSend(fbRequestObj.SEND_COINS, psConstantsObj.TRACK_GIFT_COINS, "");

    };

    this.acceptAndSendLives = function () {
        fbRequestObj.acceptAndSend(fbRequestObj.SEND_LIVES, psConstantsObj.TRACK_GIFT_SPINS, "");
    };

    this.acceptAndSend = function (type, c1, s1) {
        if (type != fbRequestObj.SEND_LIVES && type != fbRequestObj.SEND_COINS && type != fbRequestObj.SEND_MOVES && type != constantsObj.SEND_SPINS && type != fbRequestObj.CREATURE_COINS && type != fbRequestObj.CREATURE_SPINS) {
            return;
        }
        if (type == fbRequestObj.SEND_MOVES) {
            var puCount = economyObj.getNumPowerUp(psConstantsObj.PU_EXTRA_MOVES_2);
            if (puCount >= psConstantsObj.EXTRA_MOVES_2_LIMIT) {
                utilObj.showMessage(stringsObj.extra_moves_limit);
                return;
            }
        }

        fbRequestObj.type = type;
        if (openOfType[type] == undefined || openOfType[type].length == 0) {
            return;
        }
        var to = "";
        var friendMap = {};
        friendsObj.setAttemptSendTo(new Array());

        for (var i = 0; i < openOfType[type].length; i++) {
            var friendId = openOfType[type][i].from.id;
            if (friendMap[friendId] == undefined) {
                friendMap[friendId] = 1;
                if (friendsObj.canSendReqTo(friendId)) {
                    to += friendId + ",";
                    friendsObj.addPlayingFriend(friendId);
                }
            }
        }
        to = to.replace(/,$/, "");
        c = c1;
        o = "msg_response";
        s = s1;

        if(type == constantsObj.SEND_COINS || type == constantsObj.CREATURE_COINS) {
            if(fbRequestObj.getNumCoinsAccepted() >= constantsObj.FB_REQ_COINS_LIMIT) {
                if (numCoinsAccepted >= constantsObj.FB_REQ_COINS_LIMIT) {
                    showAcceptLimitReached(constantsObj.FB_REQ_COINS_LIMIT + " coins");
                    return;
                }
            }
        }

        if(type == constantsObj.SEND_SPINS || type == constantsObj.CREATURE_SPINS) {
            if(fbRequestObj.getNumSpinsAccepted() >= constantsObj.FB_REQ_SPINS_LIMIT) {
                if (numSpinsAccepted >= constantsObj.FB_REQ_SPINS_LIMIT) {
                    showAcceptLimitReached(constantsObj.FB_REQ_SPINS_LIMIT + " spins");
                    return;
                }
            }
        }

        var title = null,
            desc = null,
            cb = null;
        switch (type) {
            case fbRequestObj.SEND_LIVES:
                title = stringsObj.send_lives;
                desc = stringsObj.req_send_lives_desc;
                cb = fbRequestObj.acceptAndSendLivesSuccess;
                break;
            case fbRequestObj.SEND_COINS:
                title = stringsObj.send_coins;
                desc = stringsObj.req_send_coins_desc;
                cb = fbRequestObj.acceptAndSendCoinsSuccess;
                break;
            case fbRequestObj.SEND_MOVES:
                title = stringsObj.send_moves;
                desc = stringsObj.req_send_moves_desc;
                cb = fbRequestObj.acceptAndSendMovesSuccess;
                break;
            case fbRequestObj.SEND_SPINS:
                title = stringsObj.send_spins;
                desc = stringsObj.req_send_spins_desc;
                cb = fbRequestObj.acceptAndSendSpinsSuccess;
                break;
            case fbRequestObj.CREATURE_COINS:
                title = stringsObj.send_coins;
                desc = stringsObj.req_send_coins_desc;
                cb = fbRequestObj.acceptAndSendCoinsSuccess;
                break;
            case fbRequestObj.CREATURE_SPINS:
                title = stringsObj.send_spins;
                desc = stringsObj.req_send_spins_desc;
                cb = fbRequestObj.acceptAndSendSpinsSuccess;
                break;

        }

        sendRequests(title, cb, to, desc, c, o, s);
        if (to == "") {
            if (type == fbRequestObj.SEND_LIVES) {
                fbRequestObj.grantAcceptedLives();
            } else if (type == fbRequestObj.SEND_COINS) {
                fbRequestObj.grantAcceptedCoins();
            } else if (type == fbRequestObj.SEND_MOVES) {
                fbRequestObj.grantAcceptedMoves();
            }

            fbRequestObj.clearRequests(type);
        }
    };

    this.acceptAndSendLivesSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        // loggerObj.debugLog("Accept and send Lives success:" + utilObj.getStringifiedJson(msg));
        fbRequestObj.grantAcceptedLives();
        fbRequestObj.clearRequests(fbRequestObj.SEND_LIVES);
        friendsObj.sendSuccessful(msg);
    };

    this.acceptAndSendCoinsSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        if(economyObj.getCash() >= constantsObj.MAX_CASH_ALLOWED){
            utilObj.showMessage("Please use the available cash first");
            return;
        }
        if (numCoinsAccepted >= constantsObj.FB_REQ_COINS_LIMIT) {
            showAcceptLimitReached(constantsObj.FB_REQ_COINS_LIMIT + " coins");
            return;
        }
        // loggerObj.debugLog("Accept and send Coins success:" + utilObj.getStringifiedJson(msg));
        fbRequestObj.grantAcceptedCoins();
        fbRequestObj.clearRequests(fbRequestObj.SEND_COINS);
        friendsObj.sendSuccessful(msg);
    };

    this.acceptAndSendSpinsSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        if(economyObj.getSpinCount() >= constantsObj.MAX_SPIN_ALLOWED){
            utilObj.showMessage("Please use the available spins first");
            return;
        }
        if (numSpinsAccepted >= constantsObj.FB_REQ_SPINS_LIMIT) {
            showAcceptLimitReached(constantsObj.FB_REQ_SPINS_LIMIT + " spins");
            return;
        }
        // loggerObj.debugLog("Accept and send spins success:" + utilObj.getStringifiedJson(msg));
        fbRequestObj.grantAcceptedSpins();
        fbRequestObj.clearRequests(fbRequestObj.SEND_SPINS);
        friendsObj.sendSuccessful(msg);
    };

    this.acceptAndSendMovesSuccess = function (code, msg) {
        if (!wasSendSuccessful(code, msg)) {
            return;
        }
        // loggerObj.debugLog("Accept and send Moves success:" + utilObj.getStringifiedJson(msg));
        fbRequestObj.grantAcceptedMoves();
        fbRequestObj.clearRequests(fbRequestObj.SEND_MOVES);
        friendsObj.sendSuccessful(msg);
    };

    function showAcceptedMsg(stuff) {
        var msg = stringsObj.accepted_gifts;
        msg = msg.replace("%STUFF%", stuff);
        utilObj.showMessage(msg);
    };

    function showSentMsg(stuff) {
        var msg = stringsObj.sent_gifts;
        msg = msg.replace("%STUFF%", stuff);
        utilObj.showMessage(msg);
    };

    function showAcceptLimitReached(stuff) {
        var msg = stringsObj.accept_limit;
        msg = msg.replace("%STUFF%", stuff);
        utilObj.showMessage(msg);
    }

    this.showAcceptLimitReachedMsg = function (stuff) {
        showAcceptLimitReached(stuff);
    };

    this.grantAcceptedLives = function () {
        if (numLivesAccepted >= psConstantsObj.FB_REQ_LIVES_LIMIT) {
            showAcceptLimitReached(psConstantsObj.FB_REQ_LIVES_LIMIT + " lives");
            return;
        }
        var reqType = fbRequestObj.SEND_LIVES;
        var friendCount = 0;
        var friendMap = {};
        for (var i = 0; i < openOfType[reqType].length; i++) {
            var friendId = openOfType[reqType][i].from.id;
            if (friendMap[friendId] == undefined) {
                friendMap[friendId] = 1;
                friendCount++;
            }
        }
        var success = economyObj.addLives(friendCount * psConstantsObj.LIVES_PER_REQUEST, psConstantsObj.TRACK_EARN, psConstantsObj.TRACK_GIFT, "", true);
        if (success) {
            numLivesAccepted += friendCount;
            showAcceptedMsg(utilObj.ucFirst("lives"));
        }
    };

    this.grantAcceptedCoins = function () {
        if (numCoinsAccepted >= constantsObj.FB_REQ_COINS_LIMIT) {
            showAcceptLimitReached(constantsObj.FB_REQ_COINS_LIMIT + " coins");
            return;
        }
        var reqType = fbRequestObj.SEND_COINS;
        var friendCount = 0;
        var friendMap = {};
        for (var i = 0; i < openOfType[reqType].length; i++) {
            var friendId = openOfType[reqType][i].from.id;
            if (friendMap[friendId] == undefined) {
                friendMap[friendId] = 1;
                friendCount++;
            }
        }

        var coinsPerRequest = economyObj.getCoinsPerRequest();
        var success = economyObj.addCash(friendCount * coinsPerRequest, constantsObj.TRACK_EARN, constantsObj.TRACK_GIFT, "");
        if (success) {
            numCoinsAccepted += (friendCount * coinsPerRequest);
            showAcceptedMsg(utilObj.ucFirst("coins"));
        }
    };

    this.grantAcceptedSpins = function () {
        if (numSpinsAccepted >= constantsObj.FB_REQ_SPINS_LIMIT) {
            showAcceptLimitReached(constantsObj.FB_REQ_SPINS_LIMIT + " spins");
            return;
        }
        var reqType = fbRequestObj.SEND_SPINS;
        var friendCount = 0;
        var friendMap = {};
        for (var i = 0; i < openOfType[reqType].length; i++) {
            var friendId = openOfType[reqType][i].from.id;
            if (friendMap[friendId] == undefined) {
                friendMap[friendId] = 1;
                friendCount++;
            }
        }
        var spinsToBeAdded = friendCount;
        var success = economyObj.addSpins(spinsToBeAdded, psConstantsObj.TRACK_EARN, psConstantsObj.TRACK_GET_SPINS, psConstantsObj.TRACK_GRANT);
        if (success) {
            numSpinsAccepted += friendCount;
            showAcceptedMsg(utilObj.ucFirst("spins"));
        }
    };

    this.grantAcceptedMoves = function () {
        if (numMovesAccepted >= psConstantsObj.FB_REQ_MOVES_LIMIT) {
            showAcceptLimitReached(psConstantsObj.FB_REQ_MOVES_LIMIT + " moves");
            return;
        }
        var reqType = fbRequestObj.SEND_MOVES;
        var friendCount = 0;
        var friendMap = {};
        for (var i = 0; i < openOfType[reqType].length; i++) {
            var friendId = openOfType[reqType][i].from.id;
            if (friendMap[friendId] == undefined) {
                friendMap[friendId] = 1;
                friendCount++;
            }
        }
        var success = economyObj.addPowerUp(psConstantsObj.PU_EXTRA_MOVES_2, friendCount * psConstantsObj.MOVES_PER_REQUEST, psConstantsObj.TRACK_GIFT);
        if (success) {
            numMovesAccepted += friendCount;
            showAcceptedMsg(utilObj.ucFirst("moves"));
        }
    };

    this.showMFS = function () {
        var screen = curScreen;
        /*if (curScreen == constantsObj.SCREEN_LEVEL_INTRO) {
         screen = constantsObj.SCREEN_LEVEL_INTRO;
         levelIntroLayer.redrawFlag = true;
         }
         if (curScreen == constantsObj.SCREEN_LEVEL_OUTRO) {
         screen = constantsObj.SCREEN_LEVEL_OUTRO;
         levelOutroLayer.redrawFlag = true;
         }*/
        if (curScreen == constantsObj.SCREEN_MFS) {
            // loggerObj.debugLog("lelouch , mfslayer.redraw called inside from fbrequest.showmfs");
            mfsLayer.redraw();
        } else if (curScreen == constantsObj.SCREEN_MESSAGE_CENTER) {
            /*popupLayer.setDontGoToLastScreen(true);
             popupLayer.closeWithDefault();
             mfsLayer.checkAndShow(fbRequestObj.type, curScreen, msgCenterPopup);*/
            // loggerObj.debugLog("lelouch , msgcenterpopup.openmfs called inside from fbrequest.showmfs");
            msgCenterPopup.openMfs(fbRequestObj.type);
        } else {
            // loggerObj.debugLog("lelouch , mfslayer.checkandshow called inside from fbrequest.showmfs");
            mfsLayer.checkAndShow(fbRequestObj.type, screen);
        }
    };

    this.getReqSendLimit = function () {
        return constantsObj.REQ_SEND_LIMIT_WEB;
    };

    this.getNumCoinsAccepted = function () {
        return numCoinsAccepted;
    };

    this.incrementNumCoinsAccepted = function (val) {
        numCoinsAccepted += val;
    };

    this.getNumSpinsAccepted = function () {
        return numSpinsAccepted;
    };

    this.incrementNumSpinsAccepted = function (val) {
        numSpinsAccepted += val;
    };
};