Track = function () {
    var track = null;
    var pipeline = null;

    var trackingData = new Array();
    var lastSync = 0;
    var localLastSync = 0;
    var lastSyncStart = 0;
    var trackingIndex = 0;
    var trackClient = constantsObj.CLIENT_UNKNOWN;
    var versionCode = 0;
    var origin = 0;
    var lastNTracking = new Array();
    var lastNTrackingCount = 0;


    var REFID_STR = "rid";
    var BUILD_STR = "b";
    var TRACK_ID_STR = "t";

    var trackingLastSec = 0;
    var MAX_TRACKING_ONE_SHOT = 500;
    var dayStartTracking = false;

    var trackingMap = {}; // Keep a map of all tracking entries to be able to drop dups
    var dupTrackingMap = {};
    var dupTrackingCount = 0;
    var syncMethodFailedCalls = 0;

    function loadTrackingData(trackData) {
        var tracking = JSON.parse(trackData);
        var tmpData = JSON.parse(tracking.data);
        var numEntries = tmpData.length;
        for (var j = 0; j < numEntries; j++) {
            trackingData.push({
                "data": tmpData[j],
                "trackId": getNextTrackingIndex()
            });
        }
        lastSync = tracking.lastSync;
        localLastSync = tracking.localLastSync;
        if (tracking.origin != undefined) {
            origin = tracking.origin;
        } else {
            origin = utilObj.getRandomIntInRange(1, psConstantsObj.TRACK_ORIGIN_MAX, true);
            // loggerObj.debugLog("Creating new origin:", origin);
        }
        if (tracking.trackingIndex != undefined) {
            trackingIndex = tracking.trackingIndex;
        }
        if (tracking.lastNTracking != undefined){
            lastNTracking = tracking.lastNTracking;
        }
        if (tracking.lastNTrackingCount != undefined){
            lastNTrackingCount = tracking.lastNTrackingCount;
        }
    }

    function getNextTrackingIndex() {
        trackingIndex += 1;
        trackingIndex = trackingIndex % psConstantsObj.TRACK_MOD_VAL;
        return trackingIndex;
    }

    this.init = function () {
        this.load();
    };

    this.load = function () {
        track = new Controller(psConstantsObj.TRACK_SERVER_REST);
        pipeline = new Controller(psConstantsObj.TRACK_SERVER_PIPELINE)
        
        var trackData = ls.getItem(constantsObj.TRACK_DATA_FILE);
        trackClient = utilObj.getTrackingClient();
        versionCode = utilObj.getVersionCode();

        if (trackData == null || trackData == "") {
            // loggerObj.debugLog("No tracking data yet");
            origin = utilObj.getRandomIntInRange(1, psConstantsObj.TRACK_ORIGIN_MAX, true);
            // loggerObj.debugLog("Creating new origin:", origin);
            return false;
        }
        loadTrackingData(trackData);

        return true;
    };

    function getTrackingObj() {
        var tracking = {};
        var tmpArr = [];
        var numEntries = trackingData.length;
        for (var j = 0; j < numEntries; j++) {
            tmpArr.push(trackingData[j].data);
        }
        tracking.data = utilObj.getStringifiedJson(tmpArr);
        tracking.lastSync = lastSync;
        tracking.localLastSync = localLastSync;
        tracking.origin = origin;
        tracking.trackingIndex = trackingIndex;
        tracking.lastNTracking = lastNTracking;
        tracking.lastNTrackingCount = lastNTrackingCount;
        return tracking;
    }

    this.save = function () {
        var tracking = getTrackingObj();
        ls.setItem(constantsObj.TRACK_DATA_FILE, utilObj.getStringifiedJson(tracking));
        return true;
    };

    var trackFieldSeparator = ":";
    var trackingSeparator = "^";
    this.syncPendingTracking = function (fromServerAck, force) {
        var refId = userObj.getTrackingRefId();
        var isOnline = utilObj.isPingWorking();
        if (refId == "" || !isOnline) {
            var trackRefId = "blank";
            var trackOnline = "online";
            if (refId != "") {
                trackRefId = "present";
            }
            if (!isOnline) {
                trackOnline = "offline";
            }
            analyticsObj.syncFabricEvent("track_sync_fail", trackRefId + "_" + trackOnline);
            syncMethodFailedCalls++;
            return;
        }
        var curTimestamp = utilObj.getCurrentTimestamp();
        var readyToSync = (curTimestamp - lastSync > constantsObj.TRACK_SYNC_INTERVAL
            || curTimestamp - localLastSync > constantsObj.TRACK_SYNC_INTERVAL
            || localLastSync > curTimestamp) && (curTimestamp - lastSyncStart > constantsObj.TRACK_SYNC_INTERVAL);

        if (trackingData.length > 0 && (readyToSync || fromServerAck || force || syncMethodFailedCalls >= constantsObj.TRACK_SYNC_INTERVAL / 10)) {
            syncMethodFailedCalls = 0;
            lastSyncStart = curTimestamp;
            var numEntries = trackingData.length;
            var data = "";
            for (var j = 0; j < MAX_TRACKING_ONE_SHOT && j < numEntries; j++) {
                var trackData = trackingData[j].data;
                var rid = trackData.rid;
                if (rid == "") {
                    rid = userObj.getTrackingRefId();
                }
                data += rid + trackFieldSeparator + trackData.k + trackFieldSeparator + trackData.p + trackFieldSeparator
                    + trackData.c + trackFieldSeparator + trackData.o + trackFieldSeparator + trackData.f + trackFieldSeparator
                    + trackData.g + trackFieldSeparator
                    + trackData.s + trackFieldSeparator
                    + trackData.b + trackFieldSeparator
                    + trackData.cc + trackFieldSeparator + trackData.oid + trackFieldSeparator + trackData.co + trackFieldSeparator
                    + trackingSeparator;
                // loggerObj.debugLog("Data:", trackData.s.replace(";", "*"));
            }
            data = trackObj.sanitizeTrackingData(data);

            var map = {};
            var refId = null;

            if (trackingData[0].data.rid != "") {
                refId = trackingData[0].data.rid;
            } else {
                refId = userObj.getTrackingRefId();
            }
            data = trackObj.sanitizeTrackingData(data);

            map["rid"] = refId + "";
            map["gps"] = userObj.getPsId() + "";
            map["gid"] = constantsObj.GAME_ID + "";
            map["cli"] = trackClient + "";
            map["or"] = "0";
            map["b"] = trackingData[0].data.b + "";

            map["tr"] = data;
            // loggerObj.debugLog("Sending data:", data);
            map["tid"] = trackingData[0].trackId + "";
            track.sendHttpRequest("track", "sync", map);
            analyticsObj.syncFabricEvent("track_sync", "start");

            if (trackObj.getDayStartTracking()) {
                //Request
                analyticsObj.syncFabricEvent(psConstantsObj.TRACK_DAY_START_TRACKING, psConstantsObj.TRACK_REQUEST);
            }

        } else {
            syncMethodFailedCalls++;
        }
    };

    this.syncToServer = function () {
        trackObj.syncPendingTracking(false);
    };

    this.respHandler = function (action, data, success) {
        // loggerObj.debugLog("Tracking Resp Handler:", action, data, success);
        analyticsObj.syncFabricEvent("track_sync", "end");
        if (action === "sync") {
            localLastSync = utilObj.getCurrentTimestamp();
            lastSync = parseInt(data.lastSync);
            var savedCount = parseInt(data.savedCount);
            //var numEntries = trackingData.length;
            var lastSavedIndex = parseInt(data.trackId);
            // loggerObj.debugLog("Tracking obj length:", savedCount, trackingData.length);
            if (success === false) {
                savedCount = 1; // Skip the last entry which might have caused some issue.
            }
            if (trackingData.length > 0 && lastSavedIndex == trackingData[0].trackId) {
                trackingData.splice(0, savedCount)
            }
            // loggerObj.debugLog("Tracking obj length:", savedCount, trackingData.length);
            trackObj.save();
            if (trackObj.getDayStartTracking()) {
                trackObj.setDayStartTracking(false);
                //success
                analyticsObj.syncFabricEvent(psConstantsObj.TRACK_DAY_START_TRACKING, psConstantsObj.TRACK_SUCCESS);
            }
            // trackObj.syncPendingTracking(true);
        } else {
            if (trackObj.getDayStartTracking()) {
                //failure
                analyticsObj.syncFabricEvent(psConstantsObj.TRACK_DAY_START_TRACKING, psConstantsObj.TRACK_FAIL);
            }

            utilObj.log("Unknown action received from server. Action:" + action + "; Data:" + data);
        }
    };

    this.sanitizeTrackingField = function (f) {
        f = f + "";
        f = f.replace(/[^a-zA-Z0-9_.]/g, "-");
        return f;
    };

    this.sanitizeTrackingData = function (f) {
        f = f + "";
        f = f.replace(/[^a-zA-Z0-9_:^.~]/g, "-");
        return f;
    };

    this.getLastNTrackingStr = function(){
        var startIndex = 0;
        if (lastNTracking.length >= MAX_TRACKING_ONE_SHOT) {
            startIndex = (lastNTrackingCount % MAX_TRACKING_ONE_SHOT);
        }
        var lastIndex = lastNTracking.length;
        var data = "";
        for( var i = startIndex; i < lastNTracking.length; i++){
            data += trackObj.refactorTrackingStorage(lastNTracking[i]) + "";
        }
        for(var i = 0; i < startIndex; i++){
            data += trackObj.refactorTrackingStorage(lastNTracking[i]) + "";
        }
        data = trackObj.sanitizeTrackingData(data);
        return data;
    };

    this.refactorTrackingStorage = function (val){
        var trackData = val;
        var rid = trackData.rid;
        if (rid == "") {
            rid = userObj.getTrackingRefId();
        }
        var data = "";
        data += rid + trackFieldSeparator + trackData.k + trackFieldSeparator + trackData.p + trackFieldSeparator
            + trackData.c + trackFieldSeparator + trackData.o + trackFieldSeparator + trackData.f + trackFieldSeparator
            + trackData.g + trackFieldSeparator
            + trackData.s + trackFieldSeparator
            + trackData.b + trackFieldSeparator
            + trackData.cc + trackFieldSeparator + trackData.oid + trackFieldSeparator + trackData.co + trackFieldSeparator
            + trackingSeparator;
        return data;

    };

    this.trackCounter = function (k, p, c, o, f, g, s, co, oppId) {
        return;
        if (k == undefined || p == undefined || c == undefined || o == undefined || f == undefined || g == undefined || s == undefined || co == undefined || oppId == undefined) {
            return;
        }

        var nextTrackId = getNextTrackingIndex();
        if (versionCode == 0) {
            versionCode = utilObj.getVersionCode();
        }
        var tracking = {};
        tracking["k"] = trackObj.sanitizeTrackingField(k);
        tracking["p"] = trackObj.sanitizeTrackingField(p);
        tracking["c"] = trackObj.sanitizeTrackingField(c);
        tracking["o"] = trackObj.sanitizeTrackingField(o);
        tracking["f"] = trackObj.sanitizeTrackingField(f);
        tracking["g"] = trackObj.sanitizeTrackingField(g);
        tracking["s"] = trackObj.sanitizeTrackingField(s);
        tracking["co"] = trackObj.sanitizeTrackingField(co);
        tracking["oid"] = trackObj.sanitizeTrackingField(oppId);
        tracking["cc"] = utilObj.getCurrentTimestamp();
        tracking[REFID_STR] = userObj.getTrackingRefId();
        tracking[BUILD_STR] = versionCode + "";
        tracking[TRACK_ID_STR] = nextTrackId;

        var trackMeta = {
            "data": tracking,
            "trackId": nextTrackId
        };
        var stringifiedTracking = utilObj.getStringifiedJson(tracking);
        var trackStr = tracking[REFID_STR] + ":" + tracking["k"] + ":" + tracking["p"] + ":" + tracking["c"] + ":" + tracking["o"] + ":" + tracking["f"] + ":" + tracking["g"]
            + ":" + tracking["s"] + ":" + tracking["co"] + ":" + tracking["oid"] + ":" + tracking["cc"];
        try {
            if (trackingMap[trackStr]) {
                // loggerObj.debugLog("Dropping duplicate tracking:", trackStr);
                if (dupTrackingMap[tracking["k"]]) {
                    dupTrackingMap[tracking["k"]] += 1;
                } else {
                    dupTrackingMap[tracking["k"]] = 1;
                }
                dupTrackingCount++;
                return;
            }
        } catch (err) {
            // loggerObj.debugLog("The recursion error ", err);
            return;
        }
        trackingMap[trackStr] = true;

        // loggerObj.debugLog("Adding tracking:" + utilObj.getStringifiedJson(tracking));
        if (stringifiedTracking.length < constantsObj.MAX_TRACKING_LEN) {
            trackingData.push(trackMeta);
        } else {
            // loggerObj.debugLog("Dropping tracking coz its too long:" + tracking);
        }

        if (k == psConstantsObj.TRACK_SESSION) {
            // loggerObj.debugLog("Session tracking:", utilObj.getStringifiedJson(tracking));
        }
        lastNTracking[lastNTrackingCount%MAX_TRACKING_ONE_SHOT] = tracking;
        lastNTrackingCount++;
        trackingLastSec++;
        if (utilObj.isIos()) {
            psUtilObj.sendReport(trackStr);
        }
    };

    this.resetTrackingLastSec = function () {
        trackingLastSec = 0;
    };

    this.trackCounterSampled = function (k, p, c, o, f, g, s, co, oppId) {
        var rand = utilObj.getRandomIntInRange(0, constantsObj.SAMPLING_LIMIT);
        if (rand == 1) {
            // 1% records kept
            trackObj.trackCounter(k, p, c, o, f, g, s, co, oppId);
        }
    };

    this.trackDialog = function (o, f, g, s) {
        var speciesTracking = gameSpecificObj.getTrackingPuzzleInfo();
        if (s != undefined && s != null) {
            speciesTracking = s;
        }
        // trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getTrackingBucketId(), gameObj.getTrackingPuzId(), o, f, g, speciesTracking, "1", "");
    };

    this.trackPuzzle = function (o, f, g) {
        trackObj.trackCounter(psConstantsObj.TRACK_PUZZLE, gameSpecificObj.getTrackingBucketId(), gameSpecificObj.getTrackingPuzId(), o, f, g, gameSpecificObj.getTrackingPuzzleInfo(), "1", "");
    };

    this.processNativeTracking = function (msg) {
        return;
        var jObj = null;
        if (utilObj.isNative()) {
            var trackingFields = msg.split(trackFieldSeparator);
            jObj = {
                k: trackingFields[0],
                p: trackingFields[1],
                c: trackingFields[2],
                o: trackingFields[3],
                f: trackingFields[4],
                g: trackingFields[5],
                s: trackingFields[6],
                co: trackingFields[7],
                oppId: trackingFields[8],
            };
        } else {
            try {
                jObj = JSON.parse(msg)
            } catch (err) {
                loggerObj.infoLog("error = ", err, msg);
            }
        }
        trackObj.trackCounter(jObj.k, jObj.p, jObj.c, jObj.o, jObj.f, jObj.g, jObj.s, jObj.co, jObj.oppId);
        trackObj.save();
    };


    this.getDayStartTracking = function () {
        return dayStartTracking;
    };

    this.setDayStartTracking = function (val) {
        dayStartTracking = val;
    };

    this.resetTrackingMap = function () {
        trackingMap = {};
    };

    this.trackDupEvent = function () {
        if (dupTrackingCount > 0) {
            // Find out the kingdom which has the most dups and trigger it.
            var maxCount = 0;
            var kMax;
            for (var k in dupTrackingMap) {
                if (dupTrackingMap[k] > maxCount) {
                    kMax = k;
                    maxCount = dupTrackingMap[k];
                }
            }
            dupTrackingCount = 0;
            dupTrackingMap = {};
        }
    };

    // TODO: Have to call this every session
    this.sendUpdateToDauMapping = function () {
        return;
        var refId = userObj.getPsId();
        if (refId == "" || refId == "undefined") {
            return;
        }
        var cli = utilObj.getTrackingClient();
        
        var data = {}
        data.gid = constantsObj.GAME_ID + "";
        data.cli = cli + "";
        data.ps_id = refId;
        data.tracking_id = userObj.getTrackingRefId() + "";
        data.gpsid = userObj.getGpsid() + "";
        data.sync_id = userObj.getSyncId() + "";
        data.xpromo_id = userObj.getXpromoId();
        data.adid = userObj.getAdjustId();
        if (cli == Constants.CLIENT_ANDROID) {
            data.adv_id = userObj.getAdvertiserId();
        } else if (cli == Constants.CLIENT_IOS) {
            data.idfa = userObj.getAdvertiserId();
        }
        data.device_id = userObj.getDeviceId();
        data.fbbid = userObj.getFbBid();
        data.appleid = userObj.getAppleUserId();
        
        pipeline.sendHttpRequest("dau_mapping", "sync", data);
    };

};