LeaderBoard = function () {
    var scoresList = null;
    var syncTimes = null;
    var leaderBoardController = null;

    this.init = function () {
        scoresList = [];
        syncTimes = [];
        leaderBoardController = gameObj.getGameController();
        for (var i = 0; i < constantsObj.TOTAL_LEVELS; i++) {
            scoresList[i] = [];
            syncTimes[i] = 0;
        }
        load();
    };

    function load () {
        var leaderBoardData = ls.getItem(constantsObj.LB_DATA_FILE);
        if (leaderBoardData == null || leaderBoardData == "") {
            return false;
        }
        loadLeaderBoardObjLocal(leaderBoardData);
        return true;
    }

    this.reset = function () {
        scoresList = [];
        syncTimes = [];
        for (var i = 0; i < constantsObj.TOTAL_LEVELS; i++) {
            scoresList[i] = [];
            syncTimes[i] = 0;
        }
    };

    function loadLeaderBoardObjLocal (leaderBoardData) {
        var leaderBoardInfo = JSON.parse(leaderBoardData);
        if (leaderBoardInfo.scoresList != undefined) {
            scoresList = leaderBoardInfo.scoresList;
        }
        if (leaderBoardInfo.syncTimes != undefined) {
            syncTimes = leaderBoardInfo.syncTimes;
        }
    }

    function getLeaderBoardsObj() {
        var leaderBoardInfo = {};
        leaderBoardInfo.scoresList = scoresList;
        leaderBoardInfo.syncTimes = syncTimes;

        return leaderBoardInfo;
    }

    this.save = function () {
        if (utilObj.isNonStopNoAnimOn()) {
            return;
        }
        var leaderBoardInfo = getLeaderBoardsObj();

        ls.setItem(constantsObj.LB_DATA_FILE, utilObj.getStringifiedJson(leaderBoardInfo));
        return true;
    };

    this.syncFriendLevelScoreLeaderBoard = function (levelNum) {
        if (!fbWrapperObj.isLoggedIn()) {
            return;
        }
        var lastPsUpdate = friendsObj.getLastPsUpdate();
        if (lastPsUpdate == 0) {
            // loggerObj.debugLog("Ps friends not yet fetched. Can't get leaderboard");
            return;
        }

        var curTimeStamp = utilObj.getCurrentTimestamp();
        for (var i = levelNum; i < constantsObj.TOTAL_LEVELS && i < levelNum + 3; i++) {
            if (curTimeStamp - syncTimes[i] > constantsObj.FRIENDS_LB_REFRESH_INTERVAL) {
                // loggerObj.debugLog("loading leaderboard data for level ", i);
                leaderBoardObj.fetchFriendLevelScoreLeaderBoard(i);
            }
        }
    };

    this.fetchFriendLevelScoreLeaderBoard = function (levelNum) {
        var lastPsUpdate = friendsObj.getLastPsUpdate();
        if (lastPsUpdate == 0) {
            // loggerObj.debugLog("Ps friends not yet fetched. Can't get leaderboard");
            return;
        }
        var refIds = friendsObj.getPsFriendRefIds();
        var map = userObj.getUserObj();
        if (map.fbAccessToken == "" || map.refId == "") {
            return;
        }
        map.lbKeyInfo = refIds;
        map.levelNum = String(levelNum);
        // loggerObj.debugLog("Friend leaderboard attempted fetch:", utilObj.getStringifiedJson(map));
        leaderBoardController.sendHttpRequest("lb", "getFriendLevelScoreBoard", map);
    };

    this.respHandler = function (action, data, success) {
        if (action == "getFriendLevelScoreBoard") {
            if (success == false) {
                return;
            }

            var scoreListStr = data["getFriendLevelScoreBoard"];
            var level = parseInt(data["levelNum"]);
            syncTimes[level] = utilObj.getCurrentTimestamp();
            // loggerObj.debugLog("leaderboard data received for level ", level, scoreListStr);

            var scoreList = JSON.parse(scoreListStr);
            scoresList[level] = scoreList;

            leaderBoardObj.save();
        }
    };

    this.getFriendLeaderBoardForLevel = function (levelNum) {
        leaderBoardObj.syncFriendLevelScoreLeaderBoard(levelNum);
        return scoresList[levelNum];
    };

    this.updateSelfScoreForLevel = function (level, score) {
        var refId = userObj.getRefId();
        var i;
        if (scoresList == null) {
            // Fixing JS Error
            scoresList = [];
            syncTimes = [];
        }
        for (i = 0; i < scoresList[level].length; i++) {
            if (scoresList[level][i]["Member"] == refId) {
                break;
            }
        }
        if (i < scoresList[level].length) {
            scoresList[level][i]["Score"] = score;
        } else {
            scoresList[level].push({"Score":score,"Member":refId});
        }

        var userScore = scoresList[level][i];
        i--;

        for (i; i >= 0; i--) {
            if (parseInt(scoresList[level][i]["Score"]) < parseInt(userScore["Score"])) {
                scoresList[level][i+1] = scoresList[level][i];
            } else {
                break;
            }
        }
        scoresList[level][i+1] = userScore;
    };

    function getFriendInfo(friendScoreInfo) {
        var friendInfo = null;
        var friend = friendsObj.getPsFriendForRefId(friendScoreInfo.Member);
        if (friend != null) {
            friendInfo = {};
            friendInfo.refId = friendScoreInfo.Member;
            friendInfo.score = friendScoreInfo.Score;
            friendInfo.name = friend.Name;
            friendInfo.firstName = utilObj.getFirstName(friend.Name);
        }
        return friendInfo;
    }

    this.getFriendWithHigherScore = function (userScore, levelNum) {
        var friendBoard = leaderBoardObj.getFriendLeaderBoardForLevel(levelNum);
        var friendWithHigherScore = null;
        for (var i = 0; i < friendBoard.length; i++) {
            var friendInfo = friendBoard[i];
            if (friendInfo.Score > userScore) {
                var friend = getFriendInfo(friendInfo);
                if (friend != null) {
                    friendWithHigherScore = friend;
                    break;
                }
            }
        }
        return friendWithHigherScore;
    };

    this.getFriendWithLowerScore = function (userScore, levelNum) {
        var friendBoard = leaderBoardObj.getFriendLeaderBoardForLevel(levelNum);
        var friendWithLowerScore = null;
        for (var i = friendBoard.length - 1; i >= 0; i--) {
            var friendInfo = friendBoard[i];
            if (friendInfo.Score < userScore) {
                var friend = getFriendInfo(friendInfo);
                if (friend != null) {
                    friendWithLowerScore = friend;
                    break;
                }
            }
        }
        return friendWithLowerScore;
    };

};