Magic = function () {
    var formatMagicPopup = false;
    var expScrollView = null;
    var inputTextField = null;

    // getters and setters here
    /**
     * format magic Popup decides if the magic popup needs to be formatted
     * @returns {boolean}
     */
    this.shouldFormatMagicPopup = function () {
        return formatMagicPopup;
    };

    this.setFormatMagicPopup = function (val) {
        formatMagicPopup = val;
    };

    this.getExpScrollView = function () {
        return expScrollView;
    };

    this.setExpScrollView = function (val) {
        expScrollView = val;
    };

    // utility functions here
    this.getMagicButton = function (screenName) {
        var btnHeight = constantsObj.HEADER_HEIGHT / 1.5;
        var btnWidth = constantsObj.DESIGN_WIDTH / 6;
        var magicButton = utilObj.getButton(btnWidth, btnHeight, 0.5, 1, centerXPos, (2 * centerYPos), constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
            obj: 'magicObj',
            fn: 'checkAndShowNativeMagicPopup',
            screen: screenName,
            params: {"sound": constantsObj.tile_white, "sound_on": true},
            track: {
                k: constantsObj.TRACK_GAME,
                p: constantsObj.TRACK_SETTINGS,
                c: gameObj.getCurPuzzleTracking(),
                o: "",
                f: "",
                g: "",
                s: "",
                co: 1,
                oppId: ""
            },
        }, false, ccui.Widget.PLIST_TEXTURE, constantsObj.SCALE_ONE);
        // magicButton.setColor(constantsObj.COLOR_ORANGE);
        return magicButton;
    };

    /**
     * this function checks and shows formatted/unformatted Magic Popup
     */
    this.checkAndShowNativeMagicPopup = function () {
        if (magicObj.shouldFormatMagicPopup()) {
            magicObj.showNativeMagicPopupFormatted();
            return;
        }

        magicObj.showNativeMagicPopup();
    };

    /**
     * this function shows unformatted magic popup Data
     */
    this.showNativeMagicPopup = function () {
        //loggerObj.debugLog("Magic NativeMagicPopup showing");
        var msgStr = "RefId: " + userObj.getRefId() + "\n" + "CurrentScreenName: " + utilObj.getCurrentScreenName() + "\n" + utilObj.getStringifiedJson(gameObj.getGameObj()) + "\n" + utilObj.getStringifiedJson(userObj.getUserObj());
        msgStr = msgStr + "\n" + ls.getItem(constantsObj.RUNTIME_CONFIG_FILE) + "\n" + ls.getItem(constantsObj.EXPERIMENTS_FILE);
        msgStr = msgStr + "\n" + ls.getItem(constantsObj.FB_REQUESTS_FILE);
        //loggerObj.debugLog("jsb call made to showNativeMagicPopup -> ", msgStr);
        if (utilObj.isAndroid()) {
            jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_ANDROID + "Util", "showNativeMagicPopup", "(Ljava/lang/String;)V", msgStr);
        } else if (utilObj.isIos()) {
            jsb.reflection.callStaticMethod("Util", "showNativeMagicPopup:", msgStr);
        }
    };

    /**
     * this function shows formatted output in Magic Popup
     */
    this.showNativeMagicPopupFormatted = function () {
        // loggerObj.debugLog("Magic NativeMagicPopup showing");
        var msgStr = "RefId: " + userObj.getRefId() + "\n" + "CurrentScreenName: " + utilObj.getCurrentScreenName() + "\n" + "\n";
        // all files arr contains all different types of files with the respective jsonObjects and titles for the files
        var allFilesArr = [
            {
                "jsonObject": gameObj.getGameObj(),
                "title": stringsObj.title_game
            },
            {
                "jsonObject": userObj.getUserObj(),
                "title": stringsObj.title_user
            },
            {
                "jsonObject": JSON.parse(ls.getItem(constantsObj.RUNTIME_CONFIG_FILE)),
                "title": stringsObj.title_runtime
            },
            {
                "jsonObject": JSON.parse(ls.getItem(constantsObj.EXPERIMENTS_FILE)),
                "title": stringsObj.title_experiments
            },
            {
                "jsonObject": JSON.parse(ls.getItem(constantsObj.FB_REQUESTS_FILE)),
                "title": stringsObj.title_fb
            }
        ];

        // sorting allFilesArr here based on the alphabetical order of the titles
        allFilesArr.sort(magicObj.compareTitles);

        var numFiles = allFilesArr.length;
        for (var i = 0; i < numFiles; i++) {
            // appending result of each fileType to msgStr
            if (allFilesArr[i]["title"] != stringsObj.title_experiments) {
                msgStr = msgStr + magicObj.getFormattedStringForMagicPopup(allFilesArr[i]) + "\n";
            } else {
                msgStr = msgStr + magicObj.getFormattedStringForExperimentsFile(allFilesArr[i]);
            }
        }
        if (utilObj.isAndroid()) {
            jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_ANDROID + "Util", "showNativeMagicPopup", "(Ljava/lang/String;)V", msgStr);
        } else if (utilObj.isIos()) {
            jsb.reflection.callStaticMethod("Util", "showNativeMagicPopup:", msgStr);
        }
    };

    /**
     * this function is for sorting file titles in alphabetical order
     * @param obj1
     * @param obj2
     * @returns {number}
     */
    this.compareTitles = function (obj1, obj2) {
        if (obj1["title"].toLowerCase() < obj2["title"].toLowerCase()) {
            return -1;
        }
        return 1;
    };

    /**
     * this function is for sorting keys based on alphabetical order
     * @param w1
     * @param w2
     * @returns {number}
     */
    this.compareStringForMagicPopup = function (w1, w2) {
        if (w1.toLowerCase() < w2.toLowerCase()) {
            return -1;
        }
        return 1;
    };

    /**
     * this function is for sorting experiment keys alphabetically
     * @param obj1
     * @param obj2
     * @returns {number}
     */
    this.compareStringsForExperiments = function (obj1, obj2) {
        if (obj1["expName"].toLowerCase() < obj2["expName"].toLowerCase()) {
            return -1;
        }
        return 1;
    };

    /**
     * this function is for alphabetically sorting any string array
     * @param w1
     * @param w2
     * @returns {number}
     */
    this.compareStringsGeneric = function (w1, w2) {
        if (w1.toLowerCase() < w2.toLowerCase()) {
            return -1;
        }
        return 1;
    };

    /**
     * this function takes input as parameters of a file type and returns the final string for the fileType
     * @param fileObj
     * @returns {*}
     */
    this.getFormattedStringForMagicPopup = function (fileObj) {
        var str = fileObj["title"];
        str += "\n" + "\n";
        var keysArr = [];
        var obj = fileObj["jsonObject"];
        for (var key in obj) {
            if (obj.hasOwnProperty(key)) {
                keysArr.push(key);
            }
        }
        keysArr.sort(magicObj.compareStringForMagicPopup);

        var numKeys = keysArr.length;
        for (var j = 0; j < numKeys; j++) {
            var val = obj[keysArr[j]];
            if (typeof val === 'object' && typeof val !== null) {
                val = utilObj.getStringifiedJson(val);
            }
            if (val == "") {
                continue;
            }
            str += keysArr[j] + ":\t" + val + "\n";
        }
        return str;
    };

    /**
     * this function returns the formatted string for magic popup for experiments data
     * @param fileObj
     * @returns {*}
     */
    this.getFormattedStringForExperimentsFile = function (fileObj) {
        var str = fileObj["title"];
        str += "\n" + "\n";
        var obj = fileObj["jsonObject"]["arr"];
        var objLength = obj.length;
        obj.sort(magicObj.compareStringsForExperiments);
        for (var i = 0; i < objLength; i++) {
            var curExpObj = obj[i];
            for (var key in curExpObj) {
                if (curExpObj.hasOwnProperty(key)) {
                    var val = curExpObj[key];
                    if (typeof val === 'object' && typeof val !== null) {
                        val = utilObj.getStringifiedJson(val);
                    }
                    if (val == "") {
                        continue;
                    }
                    str += key + ":\t" + val + "\n";
                }
            }
            str += "\n";
        }
        return str;
    };

    /**
     * this function gets the items required for each experiment section
     * @param expName
     * @param expTitleSectionHeight
     * @param variantSectionHeight
     * @param marginHeight
     * @param innerMarginHeight
     * @param scrollViewWidth
     * @param startYPos
     * @returns {{itemsArr: Array, yPos: *}}
     */
    this.getItemsForExperimentsSection = function (i, expName, expTitleSectionHeight, variantSectionHeight, marginHeight, innerMarginHeight, scrollViewWidth, totalScrollViewWidth, startYPos, zOrder) {
        var itemsArr = [];
        var expTitleXPos = 100;
        var variantTitleXPos = 200;
        var tickIconXPos = 100;
        var yPos = startYPos - expTitleSectionHeight / 2;
        var tickIconYPos = 0;
        var expIndex = i;

        // loggerObj.debugLog("debugFunctionalityLog: start of logging for items of experiment section");

        var chosenVariantForExperiment = epmObj.getChosenVariant(expName);

        // experiment title Section here
        // experiment title Bg here
        var expTitleBgSection = {
            type: constantsObj.TYPE_IMAGE,
            sprite: constantsObj.tile_white,
            scaleType: constantsObj.SCALE_9_SPRITE,
            x: totalScrollViewWidth / 2,
            y: yPos,
            anchorX: 0.5,
            anchorY: 0.5,
            width: scrollViewWidth,
            height: expTitleSectionHeight,
            zOrder: zOrder,
            arrType: constantsObj.SPRITE_ARRAY,
            spriteSheet: constantsObj.SPRITE_MISC,
            color: constantsObj.COLOR_BUTTON_GREEN,
            opacity: constantsObj.NOT_FOUND
        };
        itemsArr.push(expTitleBgSection);

        // experiment title label here
        var expTitleLblDetails = {
            type: constantsObj.TYPE_LABEL,
            lblText: expName.toUpperCase() + "",
            fontType: constantsObj.FONT_BM,
            fontSize: constantsObj.FONT_SFCOMPACT_REGULAR_60,
            x: totalScrollViewWidth / 2,
            y: yPos,
            anchorX: 0.5,
            anchorY: 0.5,
            zOrder: zOrder,
            arrType: constantsObj.LABEL_ARRAY,
            color: constantsObj.COLOR_WHITE,
            opacity: constantsObj.NOT_FOUND
        };
        itemsArr.push(expTitleLblDetails);

        yPos -= (expTitleSectionHeight / 2 + innerMarginHeight);
        // variant sections here
        var variantNamesForExperiment = epmObj.getVariantNamesForExperiment(expName);
        var numVariants = variantNamesForExperiment.length;
        for (var k = 0; k < numVariants; k++) {
            yPos -= variantSectionHeight / 2;
            var variantName = variantNamesForExperiment[k];
            // individual variant title Bgs here
            var variantTarget = {
                obj: "magicObj",
                fn: "assignVariantToExperiment",
                screen: curLayer.screen,
                params: {expName: expName, variantName: variantName, variantSectionYPos: yPos, expIndex: expIndex}
            };

            var variantTitleBgSection = {
                type: constantsObj.TYPE_BUTTON,
                sprite: constantsObj.tile_white,
                x: totalScrollViewWidth / 2,
                y: yPos,
                anchorX: 0.5,
                anchorY: 0.5,
                width: scrollViewWidth - 100,
                height: variantSectionHeight,
                zOrder: zOrder,
                color: constantsObj.COLOR_BUTTON_LIGHT_BLUE,
                opacity: constantsObj.NOT_FOUND,
                target: variantTarget,
                textureType: ccui.Widget.PLIST_TEXTURE,
                scale: constantsObj.SCALE_ONE
            };
            itemsArr.push(variantTitleBgSection);

            // individual variant title labels here
            var variantTitleLblDetails = {
                type: constantsObj.TYPE_LABEL,
                lblText: "Variant " + variantName + "",
                fontType: constantsObj.FONT_BM,
                fontSize: constantsObj.FONT_SFCOMPACT_REGULAR_60,
                x: variantTitleXPos,
                y: yPos,
                anchorX: 0.0,
                anchorY: 0.5,
                zOrder: zOrder,
                arrType: constantsObj.LABEL_ARRAY,
                color: constantsObj.COLOR_WHITE,
                opacity: constantsObj.NOT_FOUND
            };
            itemsArr.push(variantTitleLblDetails);


            if (variantName == chosenVariantForExperiment) {
                tickIconYPos = yPos;
            }

            yPos -= (variantSectionHeight / 2 + innerMarginHeight);
        }
        yPos += innerMarginHeight;

        // tick element here
        if (tickIconYPos != 0) {
            var tickIconSprite = {
                type: constantsObj.TYPE_IMAGE,
                sprite: constantsObj.tick,
                scaleType: constantsObj.SCALE_SPRITE,
                x: tickIconXPos,
                y: tickIconYPos,
                anchorX: 0.5,
                anchorY: 0.5,
                scale: constantsObj.SCALE_SPRITE_9,
                zOrder: zOrder,
                arrType: constantsObj.SPRITE_ARRAY,
                spriteSheet: constantsObj.SPRITE_MISC,
                color: constantsObj.COLOR_GREEN,
                opacity: constantsObj.NOT_FOUND,
                tag: (expIndex + 1)
            };
            itemsArr.push(tickIconSprite);
        }

        // loggerObj.debugLog("debugFunctionalityLog: end of logging for items of experiment section");
        return {itemsArr: itemsArr, yPos: yPos};
    };

    /**
     * this function populates Scroll View with the list of experiments and variants based on the values of array and the type variables
     * @param arrElement
     * @param type
     */
    this.populateScrollViewForExperimentVariantSelector = function () {
        // loop through the elements of the array element and based on the value of the type set the value of the button Action
        // for each element , need to put a white_tile button, a label for the showing the content of the arrElement
        var expArray = epmObj.getAllExperimentKeys();
        expArray.sort(magicObj.compareStringsGeneric);

        var numElements = expArray.length;

        var expTitleSectionHeight = 300;
        var variantSectionHeight = 150;
        var marginHeight = 40;
        var innerMarginHeight = 20;

        var scrollViewInnerHeight = 0;
        var spriteArr = [];
        var labelArr = [];
        var zOrder = constantsObj.FTUE_Z_ORDER;

        for (var i = 0; i < numElements; i++) {
            var expName = expArray[i];
            // loggerObj.debugLog("debugFunctionalityLog: expName -> ", expName);
            var numVariants = epmObj.getNumVariantsForExperiment(expName);
            scrollViewInnerHeight += (numVariants * (variantSectionHeight + innerMarginHeight) + expTitleSectionHeight + marginHeight);
        }
        scrollViewInnerHeight -= marginHeight;


        var scrollViewWidth = canvasWidth;
        var scrollViewHeight = constantsObj.DESIGN_HEIGHT;
        var scrollView = new ccui.ScrollView();
        scrollView.setDirection(ccui.ScrollView.DIR_VERTICAL);
        scrollView.setTouchEnabled(true);
        scrollView.setBounceEnabled(true);
        scrollView.setScrollBarEnabled(false);
        scrollView.setInertiaScrollEnabled(true);
        scrollView.setContentSize(cc.size(scrollViewWidth, scrollViewHeight));
        scrollView.setInnerContainerSize(cc.size(scrollViewWidth, scrollViewInnerHeight));
        scrollView.setAnchorPoint(cc.p(0.5, 1.0));
        utilObj.setPosition(scrollView, centerXPos, constantsObj.DESIGN_HEIGHT);

        var yPos = scrollViewInnerHeight;
        var parent = scrollView;

        // adding scrollView Bg here
        var scrollViewBg = utilObj.getScale9Sprite(constantsObj.tile_white, scrollViewWidth / 2, yPos, 0.5, 1.0, scrollViewWidth, scrollViewInnerHeight, spriteArr,
            parent, zOrder - 1, constantsObj.SPRITE_MISC);
        utilObj.setColor(scrollViewBg, constantsObj.COLOR_BLACK);
        //

        // adding crossIcon and closeBtn here
        var crossIcon = utilObj.getSprite(constantsObj.close_icon2, 100, 100, 0.5, 0.5, constantsObj.SCALE_SPRITE_1, null, null,
            zOrder + 2, constantsObj.SPRITE_MISC);

        var closeBtn = utilObj.getButton(200, 200, 0.5, 1.0,
            scrollViewWidth - 100, yPos - 200, constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'magicObj',
                fn: 'removeExperimentsScrollView',
                screen: curLayer.screen,
                params: {"sound": constantsObj.tile_white, "sound_on": true},
            }, false, ccui.Widget.PLIST_TEXTURE, constantsObj.SCALE_ONE);
        utilObj.addChild(scrollView, closeBtn, zOrder + 2);
        utilObj.addChild(closeBtn, crossIcon, zOrder + 2);
        // loggerObj.debugLog("debugFunctionalityLog: yPos, scrollViewInnerHeight -> ", yPos, scrollViewInnerHeight);
        //

        // moving close button along with experiments scrollView here
        scrollView.addEventListener(function (scope, eventType) {
            var innerContainerYPos = Math.abs(scrollView.getInnerContainerPosition().y);
            utilObj.setPositionY(closeBtn, innerContainerYPos + constantsObj.DESIGN_HEIGHT - 200);
        });
        //

        for (var k = 0; k < numElements; k++) {
            var expName = expArray[k];
            // loggerObj.debugLog("debugFunctionalityLog: expName -> ", expName);
            var experimentSectionDetails = magicObj.getItemsForExperimentsSection(k, expName, expTitleSectionHeight, variantSectionHeight, marginHeight, innerMarginHeight, scrollViewWidth - 200, scrollViewWidth, yPos, zOrder);
            var items = experimentSectionDetails.itemsArr;

            var itemsCount = items.length;

            for (var u = 0; u < itemsCount; u++) {
                var curItem = items[u];
                if (curItem == undefined || curItem == null) {
                    continue;
                }

                if (curItem.type == undefined || curItem.type == null) {
                    // loggerObj.debugLog("debugFunctionalityLog: curItem type not set or null");
                    continue;
                }

                var sprite;

                if (curItem.type == constantsObj.TYPE_LABEL) {
                    if (curItem.fontType == constantsObj.FONT_BM) {
                        var color = curItem.color;
                        if (color == constantsObj.NOT_FOUND) {
                            color = constantsObj.COLOR_WHITE;
                        }
                        sprite = utilObj.getBMFontLabel(curItem.lblText, curItem.fontSize, color,
                            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), curItem.x, curItem.y, curItem.anchorX, curItem.anchorY, labelArr, parent, curItem.zOrder);
                        if (curItem.opacity != constantsObj.NOT_FOUND) {
                            utilObj.setOpacity(sprite, curItem.opacity);
                        }
                    }
                } else if (curItem.type == constantsObj.TYPE_IMAGE) {
                    if (curItem.scaleType == constantsObj.SCALE_9_SPRITE) {
                        var color = curItem.color;
                        if (color == constantsObj.NOT_FOUND) {
                            color = constantsObj.COLOR_WHITE;
                        }
                        sprite = utilObj.getScale9Sprite(curItem.sprite, curItem.x, curItem.y, curItem.anchorX, curItem.anchorY, curItem.width, curItem.height,
                            spriteArr, parent, curItem.zOrder, curItem.spriteSheet);
                        utilObj.setColor(sprite, color);
                        if (curItem.opacity != constantsObj.NOT_FOUND) {
                            utilObj.setOpacity(sprite, curItem.opacity);
                        }
                    } else if (curItem.scaleType == constantsObj.SCALE_SPRITE) {
                        var color = curItem.color;
                        if (color == constantsObj.NOT_FOUND) {
                            color = constantsObj.COLOR_WHITE;
                        }
                        if (curItem.tag != undefined) {
                            sprite = utilObj.getSprite(curItem.sprite, curItem.x, curItem.y, curItem.anchorX, curItem.anchorY, curItem.scale, spriteArr, parent, curItem.zOrder, curItem.spriteSheet, curItem.tag);
                        } else {
                            sprite = utilObj.getSprite(curItem.sprite, curItem.x, curItem.y, curItem.anchorX, curItem.anchorY, curItem.scale, spriteArr, parent, curItem.zOrder, curItem.spriteSheet);
                        }
                        utilObj.setColor(sprite, color);
                        if (curItem.opacity != constantsObj.NOT_FOUND) {
                            utilObj.setOpacity(sprite, curItem.opacity);
                        }
                    }
                } else if (curItem.type == constantsObj.TYPE_BUTTON) {
                    var color = curItem.color;
                    if (color == constantsObj.NOT_FOUND) {
                        color = constantsObj.COLOR_WHITE;
                    }
                    sprite = utilObj.getButton(curItem.width, curItem.height, curItem.anchorX, curItem.anchorY, curItem.x, curItem.y, curItem.sprite, curItem.sprite, curItem.sprite, curItem.target, false, curItem.textureType, curItem.scale);
                    utilObj.setColor(sprite, color);
                    if (curItem.opacity != constantsObj.NOT_FOUND) {
                        utilObj.setOpacity(sprite, curItem.opacity);
                    }
                    utilObj.addChild(scrollView, sprite, zOrder);
                }
            }
            yPos = parseInt(experimentSectionDetails.yPos) - marginHeight;
        }

        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);
        utilObj.addChildWithTag(curLayer, scrollView, zOrder + 1, constantsObj.TAG_SCROLLVIEW_FOR_EXPERIMENTS);

        magicObj.setExpScrollView(scrollView);
        // loggerObj.debugLog("debugFunctionalityLog: scrollView successfully added");
    };

    /**
     * this function updates assigned variant of selected experiment based on params
     * @param sender
     * @param params
     */
    this.assignVariantToExperiment = function (params) {
        // loggerObj.debugLog("debugFunctionalityLog: params -> ", utilObj.getStringifiedJson(params));
        if (params == undefined || params == null || params.expName == undefined || params.expName == null || params.variantName == undefined || params.variantName == null) {
            // loggerObj.debugLog("experimentLog: params value is undefined or null while assigning variant to the selected experiment");
            return;
        }

        var variantName = params.variantName;
        var selectedExp = params.expName;
        var variantSectionYPos = params.variantSectionYPos;
        var experimentIndex = params.expIndex;
        epmObj.assignVariantForExperiment(selectedExp, variantName);

        // update position of tick element for the selected experiment value
        var expScrollView = magicObj.getExpScrollView();
        var tickElement = expScrollView.getChildByTag(parseInt(experimentIndex) + 1);
        utilObj.setPositionY(tickElement, variantSectionYPos);
        //
    };

    /**
     * switches on the flag for formatting magic popup data
     */
    this.formatMagicPopupData = function () {
        magicObj.setFormatMagicPopup(true);
    };

    /**
     * force completes puzzle
     */
    this.forceCompletePuzzle = function () {
        gameLayer.completeLevel();
    };

    this.becomePayer = function () {
        gameObj.setIsPayer(true);
    };

    this.magicPopup = function () {
        magicObj.checkAndShowNativeMagicPopup();
    };

    this.getWizardToolFunction = function (tool) {
        if (tool == undefined || tool == null) {
            // loggerObj.debugLog("debugFunctionalityLog: tool value is undefined or null while getting wizard tool function");
            return null;
        }
        if (tool == constantsObj.WIZARD_COMPLETE_PUZZLE) {
            return "forceCompletePuzzle";
        }
        if (tool == constantsObj.WIZARD_FORMAT_MAGIC_POPUP) {
            return "formatMagicPopupData";
        }
        if (tool == constantsObj.WIZARD_CHANGE_VARIANTS) {
            return "populateScrollViewForExperimentVariantSelector";
        }
        if (tool == constantsObj.WIZARD_CHANGE_BECOME_PAYER) {
            return "becomePayer";
        }
        if (tool == constantsObj.WIZARD_CHANGE_MAGIC_POPUP) {
            return "magicPopup";
        }
        if (tool == constantsObj.WIZARD_SKIP_COUNTRY) {
            return "createSkipCountryUi";
        }
        if (tool = constantsObj.WIZARD_SKIP_N_NODES) {
            return "createSkipNodesUi";
        }
    };

    this.getWizardToolId = function (tool) {
        if (tool == undefined || tool == null) {
            // loggerObj.debugLog("debugFunctionalityLog: tool value is undefined or null while getting wizard tool Id");
            return null;
        }

        if (tool == constantsObj.WIZARD_COMPLETE_PUZZLE) {
            return "CP";
        }
        if (tool == constantsObj.WIZARD_FORMAT_MAGIC_POPUP) {
            return "FM";
        }
        if (tool == constantsObj.WIZARD_CHANGE_VARIANTS) {
            return "CV";
        }
        if (tool == constantsObj.WIZARD_CHANGE_BECOME_PAYER) {
            return "PA";
        }
        if (tool == constantsObj.WIZARD_CHANGE_MAGIC_POPUP) {
            return "MP";
        }
        if (tool == constantsObj.WIZARD_SKIP_COUNTRY) {
            return "SC";
        }
        if (tool == constantsObj.WIZARD_SKIP_N_NODES) {
            return "SN";
        }
    };

    this.addButtonAtPos = function (tool, position) {
        if (tool == undefined || tool == null) {
            // loggerObj.debugLog("debugFunctionalityLog: tool value undefined or null while adding magic buttons at the positions -> ", tool);
            return false;
        }

        if (position == undefined || position == null) {
            // loggerObj.debugLog("debugFunctionalityLog: position value undefined or null while adding magic buttons at the positions -> ", position);
            return false;
        }

        var func = magicObj.getWizardToolFunction(tool);
        var btnPos = position;

        var btnXPos = btnPos.x;
        var btnYPos = btnPos.y;

        var toolBtn = utilObj.getButton(168, 168, 0.5, 0.5,
            btnXPos, btnYPos, constantsObj.rounded_white_scale9, constantsObj.rounded_white_scale9, constantsObj.rounded_white_scale9, {
                obj: "magicObj",
                fn: func,
                screen: curLayer.screen,
                feedBackType: constantsObj.ZOOMIN_FEEDBACK
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.setColor(toolBtn, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(toolBtn, constantsObj.OPACITY_80);
        utilObj.addChildWithTag(curLayer, toolBtn, constantsObj.UI_Z_ORDER + 1, tool);

        var toolIdTxt = magicObj.getWizardToolId(tool);
        var toolLbl = utilObj.getBMFontLabel(toolIdTxt, constantsObj.FONT_SFCOMPACT_MEDIUM_60, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), 84, 84, 0.5, 0.5, null, null, constantsObj.FTUE_Z_ORDER + 2);
        utilObj.addChild(toolBtn, toolLbl, constantsObj.UI_Z_ORDER + 2);
    };

    this.addWizardToolsAndCloseButton = function (sender, params) {
        // loggerObj.debugLog("debugFunctionalityLog: params for adding wizard Tools are -> ", utilObj.getStringifiedJson(params));

        var originValue = params.wizardButtonPosition;
        var xOrigin = originValue.x;
        var yOrigin = originValue.y;

        // adding wizard tool buttons here
        magicObj.addWizardTools(xOrigin, yOrigin);

        // adding close button for wizard tools here
        magicObj.addCloseButtonForWizardTools(xOrigin, yOrigin);
    };

    /**
     * this function adds wizard tools on the curLayer based
     * @param xOrigin
     * @param yOrigin
     */
    this.addWizardTools = function (xOrigin, yOrigin) {
        var wizardsRadius = 600;
        var wizardsTools = constantsObj.WIZARD_TOOLS;
        var numWizardTools = wizardsTools.length;
        var thetaValue = 360 / numWizardTools;

        for (var i = 0; i < numWizardTools; i++) {
            var angleValue = utilObj.toRadians(thetaValue * (i + 1));
            var xVal = wizardsRadius * Math.cos(angleValue) + xOrigin;
            var yVal = wizardsRadius * Math.sin(angleValue) + yOrigin;
            magicObj.addButtonAtPos(wizardsTools[i], {x: xVal, y: yVal});
        }
    };

    /**
     * this function adds close button for Wizard Tools
     * @param xOrigin
     * @param yOrigin
     */
    this.addCloseButtonForWizardTools = function (xOrigin, yOrigin) {
        var crossIcon = utilObj.getSprite(constantsObj.close_icon2, 84, 84, 0.5, 0.5, constantsObj.SCALE_SPRITE_43, null, null,
            constantsObj.UI_Z_ORDER + 2, constantsObj.SPRITE_MISC);

        var crossBtn = utilObj.getButton(168, 168, 0.5, 0.5,
            xOrigin, yOrigin, constantsObj.rounded_white_scale9, constantsObj.rounded_white_scale9, constantsObj.rounded_white_scale9, {
                obj: "magicObj",
                fn: "removeWizardToolsAndCloseButton",
                screen: curLayer.screen,
                feedBackType: constantsObj.ZOOMIN_FEEDBACK
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChildWithTag(curLayer, crossBtn, constantsObj.UI_Z_ORDER + 2, constantsObj.TAG_CLOSE);
        utilObj.setColor(crossBtn, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(crossBtn, constantsObj.OPACITY_80);
        utilObj.addChild(crossBtn, crossIcon, constantsObj.UI_Z_ORDER + 2);
    };

    /**
     * this function adds the wizard button the screen
     */
    this.addWizardButton = function () {
        var wizButtonCenterX = canvasWidth / 2;
        var wizButtonCenterY = constantsObj.DESIGN_HEIGHT / 2;
        var wizButtonPos = {x: wizButtonCenterX, y: wizButtonCenterY};

        var wizardButton = utilObj.getButton(168, 168, 0.5, 0.5,
            wizButtonCenterX, wizButtonCenterY, constantsObj.rounded_white_scale9, constantsObj.rounded_white_scale9, constantsObj.rounded_white_scale9, {
                obj: "magicObj",
                fn: "addWizardToolsAndCloseButton",
                screen: curLayer.screen,
                params: {wizardButtonPosition: wizButtonPos},
                feedBackType: constantsObj.ZOOMIN_FEEDBACK
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.setColor(wizardButton, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(wizardButton, constantsObj.OPACITY_80);
        utilObj.addChild(curLayer, wizardButton, constantsObj.UI_Z_ORDER);

        var magicLbl = utilObj.getBMFontLabel("M", constantsObj.FONT_SFCOMPACT_MEDIUM_60, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), wizButtonCenterX, wizButtonCenterY, 0.5, 0.5, null, null, constantsObj.UI_Z_ORDER + 2);
        utilObj.addChild(curLayer, magicLbl, constantsObj.UI_Z_ORDER + 1);
    };

    /**
     * this function removes the scrollView for experiments from curLayer
     */
    this.removeExperimentsScrollView = function () {
        utilObj.removeChildByTag(curLayer, constantsObj.TAG_SCROLLVIEW_FOR_EXPERIMENTS);
    };

    /**
     * this function removes wizard Tools and cross Icon from the game layer
     */
    this.removeWizardToolsAndCloseButton = function () {
        var wizardTools = constantsObj.WIZARD_TOOLS;

        // removing wizard Tools here
        for (var i = 0; i < wizardTools.length; i++) {
            utilObj.removeChildByTag(curLayer, wizardTools[i]);
        }

        // removing close button here
        utilObj.removeChildByTag(curLayer, constantsObj.TAG_CLOSE);
        utilObj.removeChildByTag(curLayer, constantsObj.TAG_EDIT_BOX);
        utilObj.removeChildByTag(curLayer, constantsObj.TAG_SUBMIT_BUTTON);
        utilObj.removeChildByTag(curLayer, constantsObj.TAG_SUBMIT_BUTTON_LBL);
    };

    /**
     * create a edit box with a submit button - can be used to take custom input for debugging
     */
    this.createEditBox = function (placeholderText, submitButtonText, submitButtonObj, submitButtonfn) {
        inputTextField = new cc.EditBox(cc.size(900 * canvasWidth / 1440, 150), new ccui.Scale9Sprite(constantsObj.white_rounded_3));
        inputTextField.setAnchorPoint(cc.p(0, 0.5));
        utilObj.setPositionX(inputTextField, 376 * canvasWidth / 1440);
        utilObj.setPositionY(inputTextField, centerYPos);
        inputTextField.setColor(constantsObj.COLOR_WHITE);
        inputTextField.setFontSize(70);
        inputTextField.setFontName(constantsObj.FONT_HELVETICA_MEDIUM_65);
        inputTextField.setFontColor(constantsObj.COLOR_BLACK);
        inputTextField.setPlaceHolder(placeholderText);
        inputTextField.setPlaceholderFontName(constantsObj.FONT_HELVETICA_MEDIUM_65);
        inputTextField.setPlaceholderFontSize(70);
        inputTextField.setPlaceholderFontColor(constantsObj.COLOR_GREY_06);
        utilObj.addChildWithTag(curLayer, inputTextField, constantsObj.UI_Z_ORDER + 2, constantsObj.TAG_EDIT_BOX);
        inputTextField.setInputMode(cc.EDITBOX_INPUT_MODE_SINGLELINE);
        inputTextField.setReturnType(cc.KEYBOARD_RETURNTYPE_SEND);
        utilObj.setOpacity(inputTextField, constantsObj.OPACITY_100);
        var submitNameBtn = utilObj.getButton(400, 150, 0.5, 0.5, centerXPos, centerYPos - 400,
            constantsObj.white_rounded_3, constantsObj.white_rounded_3, constantsObj.white_rounded_3, {
            obj: submitButtonObj,
            fn: submitButtonfn,
            screen: curLayer.screen,
            params: { "sound": constantsObj.button_tap_01, "sound_on": true, "editBox": inputTextField },
            feedBackType: constantsObj.ZOOMOUT_FEEDBACK
        }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.setColor(submitNameBtn, constantsObj.COLOR_BLACK);
        utilObj.addChildWithTag(curLayer, submitNameBtn, constantsObj.UI_Z_ORDER + 2, constantsObj.TAG_SUBMIT_BUTTON);
        utilObj.setOpacity(submitNameBtn, constantsObj.OPACITY_80);
        var toolLbl = utilObj.getBMFontLabel(submitButtonText, constantsObj.FONT_SFCOMPACT_MEDIUM_60, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, centerYPos - 400, 0.5, 0.5, null, null, constantsObj.FTUE_Z_ORDER + 2);
        utilObj.addChildWithTag(curLayer, toolLbl, constantsObj.UI_Z_ORDER + 2, constantsObj.TAG_SUBMIT_BUTTON_LBL);

    };

    /**
    * creates an edit box and submit button to take input the number of nodes to skip
    * @returns 
    */
    this.createSkipNodesUi = function () {
        magicObj.removeWizardToolsAndCloseButton();
        magicObj.createEditBox("Enter Nodes to skip", "Skip Nodes", 'magicObj', 'skipNodes');
    };

    /**
     * skips n nodes
     * @returns 
     */
    this.skipNodes = function () {
        var nodesToSkip = utilObj.convertToInt(inputTextField.getString(), 0);
        linearLevelObj.skipNodes(nodesToSkip);
        magicObj.removeWizardToolsAndCloseButton();
    };


    /**
     * creates an edit box and submit button to take input the number of nodes to skip
     *  @returns 
     */
    this.createSkipCountryUi = function () {
        magicObj.removeWizardToolsAndCloseButton();
        magicObj.createEditBox("Enter Countries to skip", "Skip Country", 'magicObj', 'skipCountry');
    };

    /**
     * Below function is used to skip countries
     * @returns
     */
    this.skipCountry = function () {
        var countriesToSkip = utilObj.convertToInt(inputTextField.getString(), 0);
        for (var i = 0; i < countriesToSkip; i++) {
            linearLevelObj.skipCountry();
        }
        magicObj.removeWizardToolsAndCloseButton();
    };
};
