Scores = function () {
    var scoresController = null;

    var scores = {};
    var scoresChanged = {};
    var lastUpdate = 0; // This is the value at the server.
    var lastSync = 0;
    var lastSyncStart = 0;
    var tempScoresData = {};

    this.init = function () {
        scoresController = gameObj.getGameController();
        load();
    };

    function load() {
        var scoresJson = ls.getItem(constantsObj.SCORES_FILE);
        if (scoresJson == null || scoresJson == "") {
            //loggerObj.debugLog("No scores data found.");
            return false;
        }
        scoresJson = JSON.parse(scoresJson);
        scores = scoresJson.scores;
        lastSync = scoresJson.lastSync;
        lastUpdate = scoresJson.lastUpdate;
        scoresChanged = scoresJson.scoresChanged;
        if(lastUpdate == undefined){
            lastUpdate = 0;
        }
        return true;
    }

    function save() {
        var scoresJson = {};
        scoresJson.scores = scores;
        scoresJson.lastSync = lastSync;
        scoresJson.lastUpdate = lastUpdate;
        scoresJson.scoresChanged = scoresChanged;
        ls.setItem(constantsObj.SCORES_FILE, utilObj.getStringifiedJson(scoresJson));
        return true;
    }

    this.reset = function () {
        scores = {};
        scoresChanged = {};
        lastSync = 0;
        lastUpdate = 0;
        lastSyncStart = 0;
        save();
    };

    function setTempScoreData(val) {
        for (var i in val) {
            tempScoresData[i] = val[i];
        }
    }

    this.syncToServer = function (forceSync) {
        if (!gameObj.getScoreSyncCheck()) {
            return;
        }
        if(psPrivacyObj.shouldAccountLockedBeShown()) {
            return;
        }
        var passesBasicChecks = utilObj.passesBasicSyncChecks(lastSyncStart, constantsObj.SCORES_SYNC_INTERVAL);
        if (forceSync) {
            var scoresData = userObj.getUserObj();
            if (scoresData.syncId == "") {
                return;
            }
            scoresController.sendHttpRequest("scores", "fetch", scoresData);
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_SCORES_SYNC, "fetch", "sync", "", utilObj.getCurrentTimestamp() + "", "", 1, "");
        } else if (passesBasicChecks) {
            var curTimestamp = utilObj.getCurrentTimestamp();
            lastSyncStart = curTimestamp;
            var scoresData = userObj.getUserObj();
            if (scoresData.syncId == "") {
                return;
            }
            scoresData.lastUpdate = lastUpdate + "";
            setTempScoreData(scoresChanged);
            scoresData.scoresChanged = utilObj.getStringifiedJson(scoresChanged);
            scoresController.sendHttpRequest("scores", "update", scoresData);
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_SCORES_SYNC, "update", "sync", "", utilObj.getCurrentTimestamp() + "", "", 1, "");
        }
    };

    this.respHandler = function (action, data, success) {
        //loggerObj.debugLog("Scores resp:", action, utilObj.getStringifiedJson(data))
        if (success == false) {
            return;
        }
        lastSync = utilObj.getCurrentTimestamp();
        if (action === "update") {
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_SCORES_SYNC, "update", "success", "", utilObj.getCurrentTimestamp() + "", "", 1, "");
            var scoresStr = data.s;
            //var scoresStr1 = data;
            //var score1 = JSON.parse(scoresStr1);
            var scorestmp = JSON.parse(scoresStr);
            if (utilObj.countProperties(scorestmp) == 0) {
                return;
            }
            scores = scorestmp;
            //gameObj.updateMasteryFromSync(scores);
            gameObj.initializeAchievementsDataFromScoreObj(scores);
            var temp = {};
            // Just in case some other scores got updated before the response came.
            for (var i in scoresChanged) {
                var scoreChangedClient = scoresChanged[i] % constantsObj.OffsetForScoresData;
                var scoreFromServer = scores[i] % constantsObj.OffsetForScoresData;
                if (scoreFromServer < scoreChangedClient) {
                    temp[i] = scoresChanged[i];
                    scores[i] = scoresChanged[i];
                }
            }
            scoresChanged = temp;
            lastUpdate = data.lastUpdate;
            save();
        } else if (action == "no_update") {
            // Checking if any score data is updated meanwhile response came.
            var currScoreChangedData = scoresChanged;
            scoresChanged = {};
            for (var i in currScoreChangedData) {
                if (tempScoresData[i] == undefined) {
                    scoresChanged[i] = currScoreChangedData[i];
                }
            }
            tempScoresData = {};
            lastUpdate = data.lastUpdate;
            save();
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_SCORES_SYNC, "no_update", "success", "", utilObj.getCurrentTimestamp() + "", "", 1, "");
        } else {
            loggerObj.errorLog("Unknown action received from scores server. Action:" + action + ";Data:" + data);
        }

    };

    this.getHighScore = function (puzzle) {
        var highScore = 0;
        if (scores[puzzle] != undefined) {
            if (scores[puzzle] >= constantsObj.OffsetForScoresData) {
                highScore =  parseInt(scores[puzzle]) % constantsObj.OffsetForScoresData;
            } else {
                highScore = parseInt(scores[puzzle]);
            }

        }
        return highScore;
    };

    this.setHighScore = function (puzzle) {
        if (scores[puzzle] != undefined) {
            scores[puzzle] = 0;
        }
    };

    this.updateHighScore = function (puzzle, score) {
        var prevScore = scores[puzzle];
        if (prevScore != undefined && parseInt(prevScore) >= constantsObj.OffsetForScoresData) {
            prevScore = parseInt(prevScore) % constantsObj.OffsetForScoresData;
        }

        var masteryNew = computeMasteryNew(puzzle, score);
        // For time, a lower value is better
        if (prevScore == undefined || prevScore > score || prevScore == 0) {
            // loggerObj.debugLog("calculating mastery2", masteryNew, score);
            scores[puzzle] = parseInt(score) + masteryNew * constantsObj.OffsetForScoresData;
            scoresChanged[puzzle] = parseInt(score) + masteryNew * constantsObj.OffsetForScoresData;
            save();
        }
    };

    function computeMastery(puzzle, score) {
        //loggerObj.debugLog("computing mastery for mastery mini123");
        if (gameObj.checkForMiniOrMidi(puzzle) == true) {
            return computeMasteryForMinisAndMidis(puzzle, score);
        }
        var forGold = 10;
        var forSilver = 20;

        if (score <= forGold * constantsObj.SECONDS_IN_MIN) {
            return constantsObj.MASTERY_GOLD;
        } else if (score <= forSilver * constantsObj.SECONDS_IN_MIN) {
            return constantsObj.MASTERY_SILVER;
        }

        if (score != undefined && score != null) {
            return constantsObj.MASTERY_BRONZE;
        }
        return constantsObj.MASTERY_NONE;
    };

    function computeMasteryForMinisAndMidis(puzzle, score) {
        //loggerObj.debugLog("computing mastery for mastery mini");
        var crossWordInfo = gameObj.getCrosswordForDate(puzzle);
        if (crossWordInfo == undefined) {
            return constantsObj.MASTERY_NONE;
        }
        var gridHeight = crossWordInfo.h;
        var packId = parseInt(puzzle.slice(4, 6));
        var isBritMini = gameObj.checkForBritMini(puzzle);

        if (gameObj.checkForDailyMini(puzzle)) {
            if (score <= 2 * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_GOLD;
            } else if (score <= 4 * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_SILVER;
            }
            if (score != undefined && score != null) {
                return constantsObj.MASTERY_BRONZE;
            }
            return constantsObj.MASTERY_NONE;
        } else if (isBritMini) {
            // keeping different time limits for mastery checks
            if (score <= 3 * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_GOLD;
            } else if (score <= 4 * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_SILVER;
            }
            if (score) {
                return constantsObj.MASTERY_BRONZE;
            }
            return constantsObj.MASTERY_NONE;
        } else {
            // For Mini and Midis Calc basis of gridheight
            var timeForGold = 5;
            var timeForSilver = 7;
            if (gridHeight == 5) {
                timeForGold = 1;
                timeForSilver = 2;
            } else if (gridHeight == 7){
                timeForGold = 2;
                timeForSilver = 5;
            }

            if (score <= timeForGold * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_GOLD;
            } else if (score <= timeForSilver * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_SILVER;
            }
            if (score != undefined && score != null) {
                return constantsObj.MASTERY_BRONZE;
            }
            return constantsObj.MASTERY_NONE;
        }

    };

    function computeMasteryNew(puzzle, score) {
        // loggerObj.debugLog("computing mastery for mastery mini123");
        if (gameObj.checkForMiniOrMidi(puzzle) == true) {
            return computeMasteryForMinisAndMidisNew(puzzle, score);
        }

        var timeCheckPoints = runtimeObj.getMainMasteryTime();
        var timeArr = timeCheckPoints.split(":");
        var forGold = parseFloat(timeArr[0]);
        var forSilver = parseFloat(timeArr[1]);

        if (score <= forGold * constantsObj.SECONDS_IN_MIN) {
            return constantsObj.MASTERY_GOLD;
        } else if (score <= forSilver * constantsObj.SECONDS_IN_MIN) {
            return constantsObj.MASTERY_SILVER;
        }

        if (score != undefined && score != null) {
            return constantsObj.MASTERY_BRONZE;
        }
        return constantsObj.MASTERY_NONE;
    };

    function computeMasteryForMinisAndMidisNew(puzzle, score) {
        // loggerObj.debugLog("computing mastery for mastery mini");
        var crossWordInfo = gameObj.getCrosswordForDate(puzzle);
        if (crossWordInfo == undefined) {
            return constantsObj.MASTERY_NONE;
        }
        var gridHeight = crossWordInfo.h;
        var packId = parseInt(puzzle.slice(4, 6));
        var isBritMini = gameObj.checkForBritMini(puzzle);

        if (gameObj.checkForDailyMini(puzzle)) {

            // Daily Mini mastery calc.
            var timeCheckPoints = runtimeObj.getMiniMasteryTime();
            var timeArr = timeCheckPoints.split(":");
            if (score <= parseFloat(timeArr[0]) * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_GOLD;
            } else if (score <= parseFloat(timeArr[1]) * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_SILVER;
            }
            if (score != undefined && score != null) {
                return constantsObj.MASTERY_BRONZE;
            }
            return constantsObj.MASTERY_NONE;
        } else if (isBritMini) {
            // keeping different time limits for mastery checks
            var timeForGold = 3;
            var timeForSilver = 4;
            if (score <= timeForGold * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_GOLD;
            } else if (score <= timeForSilver * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_SILVER;
            }
            if (score) {
                return constantsObj.MASTERY_BRONZE;
            }
            return constantsObj.MASTERY_NONE;
        } else {

            // For Mini and Midis Calc basis of gridheight
            var timeForGold = 5;
            var timeForSilver = 7;
            if (gridHeight == 5) {
                timeForGold = 1;
                timeForSilver = 2;
            } else if (gridHeight == 7){
                timeForGold = 2;
                timeForSilver = 5;
            }

            if (score <= timeForGold * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_GOLD;
            } else if (score <= timeForSilver * constantsObj.SECONDS_IN_MIN) {
                return constantsObj.MASTERY_SILVER;
            }
            if (score != undefined && score != null) {
                return constantsObj.MASTERY_BRONZE;
            }
            return constantsObj.MASTERY_NONE;
        }
    };

    this.getMastery = function (puzzle) {
        var score = scores[puzzle];
        if (score != undefined && score != null && score >= constantsObj.OffsetForScoresData) {
            return Math.floor(parseInt(score)/constantsObj.OffsetForScoresData);
        }
        return computeMastery(puzzle, score);
    };

    this.getMasteryForScore = function (puzzle, score) {
        return computeMastery(puzzle, score);
    };

    this.checkIfFirstSolved = function () {
        for (var key in scores) {
            if (scores.hasOwnProperty(key)) {
                if (parseInt(scores[key]) > 0) {
                    return false;
                }
            }
        }
        return true;
    };

    this.logout = function () {
        lastSyncStart = 0;
        scoresObj.syncToServer();
    };

    this.getScores = function () {
        return scores;
    };

    this.getChangedScores = function () {
        return scoresChanged;
    };
};