

/**
 * this will control all the backend features for the daily bonus
 * @constructor
 */

DailyBonus = function () {

    // state for the grant of daily bonus
    var NOT_GIVEN_FOR_THE_DAY = 0;
    var GIVEN_FOR_THE_DAY = 1;

    var lastDailyBonusGivenTimeStamp = 0;
    var stateOfDailyBonusForTheDay = NOT_GIVEN_FOR_THE_DAY;
    var fromNotifClick = false;
    var puzzleIdToOpenAfterDailyBonus = "20170301";

    // rev status map
    var revstatusMap = {};
    var bucketForUser = 10;

    // status for the w2e video started
    var isDailyBonusRewardActive = false;

    // storing the last daily bonus close time
    var lastDailyBonusClosedTimestamp = 0;

    // init for the dailyBonus
    this.init = function () {
        //
    };

    /*
    checks for the eligibility for the daily bonus to fall on screen on app launch
    @return bool value if daily bonus to be given
     */
    this.isEligibleForDailyBonus = function () {

      var toGiveDailyBonus = false;

      // loggerObj.debugLog("dailyBonus - coming to check eligibility",epmObj.getChosenVariant(constantsObj.EXP_DAILY_BONUS_NEW),  lastDailyBonusGivenTimeStamp, stateOfDailyBonusForTheDay, fromNotifClick, puzzleIdToOpenAfterDailyBonus,gameObj.getDForUser(), runtimeObj.getDailyBonusSurfacingDay());

      // experiment check for the daily bonus feature
        var dbSurfacingDay = runtimeObj.getDailyBonusSurfacingDay();
        if (utilObj.isIos()) {
            dbSurfacingDay = runtimeObj.getDailyBonusSurfacingDay_ios();
        }
      if (epmObj.getChosenVariant(constantsObj.EXP_DAILY_BONUS_NEW) != constantsObj.EXP_VAR_1 || gameObj.getDForUser() < dbSurfacingDay) {
          return toGiveDailyBonus;
      }

      // adding check for the home ftue shown when trying to show daily bonus on app launch
      if (!homeObj.getisHomeFtueShown()) {
          return toGiveDailyBonus;
      }

      // loggerObj.debugLog("dailyBonus - coming to check eligibility2", lastDailyBonusGivenTimeStamp, stateOfDailyBonusForTheDay);

      // date check for the daily bonus grant cases
      var currentTimeStamp = utilObj.getCurrentTimestamp();
      var currentDate = new Date(currentTimeStamp * 1000);

      var lastGrantDate = new Date(lastDailyBonusGivenTimeStamp * 1000);

      // loggerObj.debugLog("dailyBonus - condition for day change", utilObj.getDaysDiff(currentDate, lastGrantDate), currentDate, lastGrantDate);
      if (stateOfDailyBonusForTheDay == NOT_GIVEN_FOR_THE_DAY && utilObj.getDaysDiff(currentDate, lastGrantDate) > 0) {
          toGiveDailyBonus = true;
      }

      // loggerObj.debugLog("dailyBonus - show daily bonus", toGiveDailyBonus);
      return toGiveDailyBonus;
    };

    /**
     * sets for the grant to be given for the daily bonus
     * @param flag - to set the flags for the daily bonus grant
     * flag set - not given for the day
     *
     */
    this.toGiveBonusForTheDayFlag = function (flag) {
        if (flag) {
            stateOfDailyBonusForTheDay = NOT_GIVEN_FOR_THE_DAY;
        } else {
            stateOfDailyBonusForTheDay = GIVEN_FOR_THE_DAY;
        }
        // loggerObj.debugLog("dailyBonus - set not given", stateOfDailyBonusForTheDay);
    };

    /**
     * this function will open homelayer when daily bonus is eligible and save the progress to open puzzle layer
     * @param id - puzzle id
     */
    this.openHomeLayerFromNotifClick = function (id) {
        homeLayer.setVisible(true);
        homeLayer.show();

        dailyBonusObj.setFromNotifClick(true);
        dailyBonusObj.setPuzzleIdToOpenAfterDailyBonus(id);

        // loggerObj.debugLog("dailyBonus - from notif click", id);
    };

    // update values for rev status map value for corresponding on action
    this.updateRevStatusMap = function () {
        var todaysPuzzleDate = gameObj.getTodaysPuzzleDate();
        var id = utilObj.getIdForDate(todaysPuzzleDate);
        if (!revstatusMap[id]) {
            revstatusMap[id] = 0;
        }
        revstatusMap[id]++;
        // loggerObj.debugLog("dailyBonus reward - rev map increment", JSON.stringify(revstatusMap));
    };

    // updates the maps for rev status
    // deletes the days for which the value is not to be considered in the map
    this.updateMapsForDailyBonusTimePeriod = function () {
        var todaysPuzzleDate = gameObj.getTodaysPuzzleDate();
        var milliSecInDay = 86400000;

        var curTime = todaysPuzzleDate.getTime();
        var possibleDates = {};
        var dailyBonusTimerPeriod = runtimeObj.getDbTimePeriod();
        if (utilObj.isIos()) {
            dailyBonusTimerPeriod = runtimeObj.getDbTimePeriod_ios();
        }
        for (var i = 0; i < dailyBonusTimerPeriod; i++) {
            curTime -= milliSecInDay;
            var newDate = new Date(curTime);
            var newId = utilObj.getIdForDate(newDate);
            if (newId < 20170301) {
                break;
            }
            possibleDates[newId] = true;
        }

        for (var key in revstatusMap) {
            if (!possibleDates[key]) {
                if (revstatusMap[key]) {
                    delete revstatusMap[key];
                }
            }
        }

        // loggerObj.debugLog("dailyBonus reward - rev map after update", JSON.stringify(revstatusMap));
    };

    /**
     * updates the bucket of the user on new day start
     */
    this.updateBucketForUser = function () {

        var runtimeBucket = runtimeObj.getDailyBonusBucketR();
        if (utilObj.isIos()) {
            runtimeBucket = runtimeObj.getDailyBonusBucketR_ios();
        }
        var splitStr = runtimeBucket.split("_");

        var x = parseInt(splitStr[2]);
        var z = parseInt(splitStr[3]);
        var n = parseInt(splitStr[0]);
        var m = parseInt(splitStr[1]);

        var currentWallet = economyObj.getCash();
        var revStatusVal = dailyBonusObj.getSumOfRevStatus();

        if (userObj.getIsPayer() || subscriptionsDTCObj.getIsSubscriptionActive()){
            bucketForUser = 10;
        } else if (currentWallet <= n) {
            if (revStatusVal < x) {
                bucketForUser = 1;
            } else if (x <= revStatusVal && revStatusVal <= z) {
                bucketForUser = 2;
            } else if (revStatusVal > z) {
                bucketForUser = 3;
            }
        } else if (n < currentWallet && currentWallet <= m) {
            if (revStatusVal < x) {
                bucketForUser = 4;
            } else if (x <= revStatusVal && revStatusVal <= z) {
                bucketForUser = 5;
            } else if (revStatusVal > z) {
                bucketForUser = 6;
            }
        } else if (currentWallet > m) {
            if (revStatusVal < x) {
                bucketForUser = 7;
            } else if (x <= revStatusVal && revStatusVal <= z) {
                bucketForUser = 8;
            } else if (revStatusVal > z) {
                bucketForUser = 9;
            }
        }

        // loggerObj.debugLog("dailyBonus reward - buckett update", JSON.stringify(revstatusMap), bucketForUser, revStatusVal, currentWallet, n < currentWallet <= m);
        // loggerObj.debugLog("dailyBonus reward - buckett update runtime", x, z, n ,m);
    };

    // returns the reward to the user
    /**
     *
     * @param flag : true -> 2nd box, false -> 1st box
     * @return {number}
     */
    this.getRewardToGiveToUser = function (flag) {
        // loggerObj.debugLog("dailyBonus reward is bucket",bucketForUser);
        var reward_prob = runtimeObj.getDbRewards1();
        if (utilObj.isIos()) {
            reward_prob = runtimeObj.getDbRewards1_ios();
        }
        if (flag) {
            reward_prob = runtimeObj.getDbRewards2();
            if (utilObj.isIos()) {
                reward_prob = runtimeObj.getDbRewards2_ios();
            }
        }
        var splitStrArr = reward_prob.split("_");
        var startIndex = bucketForUser * 6 - 6;
        var prob_1, prob_2, prob_3, reward_r1, reward_r2, reward_r3 = 0;

        // as the runtime field will be edited now so, having a extra check with try
        try {
            prob_1 = parseInt(splitStrArr[startIndex]);
            prob_2 = parseInt(splitStrArr[startIndex + 1]);
            prob_3 = parseInt(splitStrArr[startIndex + 2]);
            reward_r1 = parseInt(splitStrArr[startIndex + 3]);
            reward_r2 = parseInt(splitStrArr[startIndex + 4]);
            reward_r3 = parseInt(splitStrArr[startIndex + 5]);
        } catch(e) {
            prob_1 = 80;
            prob_2 = 19;
            prob_3 = 1;
            reward_r1 = 2;
            reward_r2 = 5;
            reward_r3 = 8;
        }

        var arrForProb = [prob_1, prob_2, prob_3];
        var arrForReward = [reward_r1, reward_r2, reward_r3];

        var sum = 0;
        for (var i in arrForProb) {
            sum += arrForProb[i];
        }
        var rand_num = utilObj.getRandomNumberinRange(1, sum);
        var reward = 1;
        sum = 0;

        for (var i in arrForProb) {
            sum += arrForProb[i];
            if (rand_num <= sum) {
                reward = arrForReward[i];
                break;
            }
        }

        if (reward > 1) {
            var rand_num2 = utilObj.getRandomNumberinRange(1, 3);
            if (rand_num2 == 1) {
                reward -= 1;
            } else if(rand_num2 == 3) {
                reward += 1;
            }
        }
        // loggerObj.debugLog("dailyBonus reward is", bucketForUser, "reward", reward, "randNum", rand_num, "rewardArr", arrForReward, arrForProb, "rand2", rand_num2);
        return reward;
    };


    // sum of the values of the rev
    this.getSumOfRevStatus = function () {
        var sum = 0;
        for (var key in revstatusMap) {
            sum += revstatusMap[key];
        }
        return sum;
    };

    /**
     * checks if video is available
     * @return {boolean} availability value
     */
    this.ifVideoIsAvailable = function () {

       return utilObj.isOnline() && adsObj.checkRewardedVideoAvailable();
    };

    /**
     * triggers video for the same
     */
    this.triggerVideo = function () {
       dailyBonusObj.setIsDailyBonusRewardActive(true);
        if (!utilObj.isOnline()) {
            return;
        }
        if (adsObj.checkRewardedVideoAvailable()) {
            adsObj.showVideoAd(false, "spinner");
            adsObj.setNotRewardedVideo(false);
        }
    };

    /**
     * when the video complete callback this fn is called
     */
    this.callbackRewardDailyBonusVideo = function () {
        dailyBonusObj.setIsDailyBonusRewardActive(false);
        dailyBonusLayer.videoCompleteCallBack();
        // carry animation forward from here
    };

    // to unit test
    this.grantDailyBonus = function () {

        dailyBonusObj.toGiveBonusForTheDayFlag();
        dailyBonusObj.setLastDailyBonusGivenTimeStamp(utilObj.getCurrentTimestamp());
        toShowDailyBonus = false;
        // dailyBonusObj.setFromNotifClick(false);
        // dailyBonusObj.setPuzzleIdToOpenAfterDailyBonus("20170301");

        // loggerObj.debugLog("dailyBonus - granted", stateOfDailyBonusForTheDay, lastDailyBonusGivenTimeStamp, toShowDailyBonus, fromNotifClick, puzzleIdToOpenAfterDailyBonus);

    };

    /**
     * getter and setter for gameData variables
     */
    this.getLastDailyBonusGivenTimeStamp = function () {
        return lastDailyBonusGivenTimeStamp;
    };

    this.setLastDailyBonusGivenTimeStamp = function (val) {
        lastDailyBonusGivenTimeStamp = val;
    };

    this.getStateOfDailyBonusForTheDay = function () {
        return stateOfDailyBonusForTheDay;
    };

    this.setStateOfDailyBonusForTheDay = function (val) {
        stateOfDailyBonusForTheDay = val;
    };

    this.getFromNotifClick = function () {
        return fromNotifClick;
    };

    this.setFromNotifClick = function (val) {
        fromNotifClick = val;
    };

    this.getIsDailyBonusRewardActive = function () {
        return isDailyBonusRewardActive;
    };

    this.setIsDailyBonusRewardActive = function (val) {
        isDailyBonusRewardActive = val;
    };

    this.getLastDailyBonusClosedTimestamp = function () {
        return lastDailyBonusClosedTimestamp;
    };

    this.setLastDailyBonusClosedTimestamp = function (val) {
        lastDailyBonusClosedTimestamp = val;
    };

    this.getPuzzleIdToOpenAfterDailyBonus = function () {
        return puzzleIdToOpenAfterDailyBonus;
    };

    this.setPuzzleIdToOpenAfterDailyBonus = function (val) {
        puzzleIdToOpenAfterDailyBonus = val;
    };

    // for storing the variables in the gameData
    this.getRevStatusMap = function () {
        return revstatusMap;
    };

    this.setRevStatusMap = function (val) {
        revstatusMap = val;
    };

    // for storing the variables in the gameData
    this.getBucketForUser = function () {
        return bucketForUser;
    };

    this.setBucketForUser = function (val) {
        // loggerObj.debugLog("dailyBonus reward bucket setting", val);
        bucketForUser = val;
    };

    // -----------------

};