DownloadManager = function () {
    var DOWNLOADS_FILE = "downloads_";

    var BASE_MISC_SERVER = 1;
    var FILES_DOWNLOAD_LIMIT = 20;
    var downloadData = null;
    var downloadsMap = {};
    var successCallBackFunctionMap = {};
    var filesToDownload = {};

    this.init = function () {
        // loggerObj.debugLog("downloadManager Log: in init"); 
        DOWNLOADS_FILE += constantsObj.GAME_ID + ".json";
        // loggerObj.debugLog("downloadManager Log: download file",DOWNLOADS_FILE);
        downloadManagerObj.load();
        downloadManagerObj.loadData("downloadsV1.json", downloadManagerObj.markDataLoaded, res.download_json);
    };

    this.markDataLoaded = function (error, data) {
        // loggerObj.debugLog("downloadManager Log: in Mark Data Downloaded");
        if (error == null && data != null) {
            // loggerObj.debugLog("downloadManager Log: in Mark Data Downloaded NOT NULL");
            downloadData = data;
            downloadManagerObj.checkAndDownloadFiles(true);
            // loggerObj.debugLog("downloadManager Log: files to download", utilObj.getStringifiedJson(filesToDownload));
        }
    };

    this.load = function () {
        // loggerObj.debugLog("downloadManager Log: in load called");
        var downloadFilesJson = ls.getItem(DOWNLOADS_FILE);
        if (downloadFilesJson == null || downloadFilesJson == "") {
            // loggerObj.debugLog("downloadManager Log: in load is null");
            return false;
        }
        // loggerObj.debugLog("downloadManager Log: in load");
        downloadFilesJson = JSON.parse(downloadFilesJson);
        downloadsMap = downloadFilesJson.downloads;
        for (var downloadedFile in downloadsMap) {
            if (!jsb.fileUtils.isFileExist(downloadManagerObj.getAbsoluteFilePath(downloadedFile))) {
                downloadManagerObj.setDownloadsMapUndefinedForFile(downloadedFile);
            }
        }
        // loggerObj.debugLog("downloadManager Log: downloads map is ->", utilObj.getStringifiedJson(downloadsMap));
        return true;
    };

    this.save = function () {
        var downloadFilesJson = {};
        downloadFilesJson.downloads = downloadsMap;
        ls.setItem(DOWNLOADS_FILE, utilObj.getStringifiedJson(downloadFilesJson));
        // loggerObj.debugLog("downloadManager Log: downloads map is ->", utilObj.getStringifiedJson(downloadFilesJson));
        return true;
    };

    this.checkAndDownloadFiles = function (noDownload) {
        // loggerObj.debugLog("downloadManager Log: IN Check And DownLoad Files");
        if (utilObj.isWeb()) {
            return;
        }
        if (downloadData == null) {
            // loggerObj.debugLog("downloadManager Log: check and download files returning as downloaddata is null");
            return;
        }

        var filesFoundForDownload = 0;
        for (var index = 0; index < downloadData.length; index++) {
            var featureInfo = downloadData[index];
            var featuresFiles = featureInfo.fileNames;
            var fnName = featureInfo.downloadCheck;
            var scaling = featureInfo.scaling;
            var dimensionPath = "";
            var fileType = featureInfo.fileType;
            var experiment = featureInfo.experiment;


            var runtime = "";
            var version = 0;
            if (featureInfo.runtime != "") {
                runtime = runtimeObj.getFromRawRuntime(featureInfo.runtime);
                if (runtime == undefined) {
                    if (!noDownload) {
                        continue;
                    }
                } else {
                    version = parseInt(runtime);
                    runtime = "v" + runtime + "/";
                }
            }
            if (scaling) {
                var deviceWidth = utilObj.getDeviceWidth();
                if (deviceWidth < 720) {
                    dimensionPath = "hdpi/";
                } else if (deviceWidth >= 720 && deviceWidth < 1080) {
                    dimensionPath = "xhdpi/";
                } else if (deviceWidth >= 1080 && deviceWidth < 1440) {
                    dimensionPath = "xxhdpi/";
                } else if (deviceWidth >= 1440) {
                    dimensionPath = "xxxhdpi/";
                }

                if (utilObj.isIos()) {
                    dimensionPath = "xxxhdpi/";
                }
            }


            for (var j = 0; j < featuresFiles.length; j++) {
                var fileName = featuresFiles[j];
                var suffixName = "";
                if (experiment != "") {
                    if (!epmObj.doesExpExist(experiment)) {
                        continue;
                    }
                    suffixName = epmObj.getChosenVariant(experiment) + "." + fileType;
                }
                fileName += suffixName;
                filesToDownload[fileName] = true;
                if (noDownload) {
                    continue;
                }

                try {
                    if (eval(fnName + "(\"" + fileName + "\"," + version + ")")) {
                        var baseUrl = downloadManagerObj.getBaseUrl(featureInfo.baseServer);
                        var basePath = featureInfo.basePath;
                        var filePath = baseUrl + basePath + runtime + dimensionPath + fileName;
                        // loggerObj.debugLog("downloadManager Log: download file name, index, downloadsMap", fileName, index, filePath, utilObj.getStringifiedJson(downloadsMap));
                        // loggerObj.debugLog("downloadManager Log: download file name, index, url and saveAs", fileName, index, filePath, fileName);
                        if (featureInfo.successCallBack != "") {
                            // loggerObj.debugLog("downloadManager Log: In succesCallBack" , featureInfo.successCallBack);
                            successCallBackFunctionMap[fileName] = featureInfo.successCallBack;
                        }
                        downloadManagerObj.downloadFile(filePath, fileName, version);
                        // trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_DOWNLOAD, version, fileName, constantsObj.TRACK_START, "", "", "", "");
                        filesFoundForDownload++;
                    }
                } catch (error) {
                    loggerObj.debugLog("downloadManager Log: function is invalid", error);
                }
                if (filesFoundForDownload > FILES_DOWNLOAD_LIMIT) {
                    break;
                }
            }

            if (filesFoundForDownload > FILES_DOWNLOAD_LIMIT) {
                break;
            }
        }
    };

    this.getBaseUrl = function (baseServer) {
        if (baseServer == BASE_MISC_SERVER) {
            return constantsObj.MISC_SERVER;
        }
        return "";
    };

    this.downloadFile = function (url, saveAs, version) {
        return;
        
        var params = {
            f: saveAs,
            v: version
        };
        params = utilObj.getStringifiedJson(params);
        // loggerObj.debugLog("downloadManager Log: Download File",saveAs , url , version) ;
        if (utilObj.isAndroid()) {
            // loggerObj.debugLog("downloadManager Log: Download File ANDROID") ;
            jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_ANDROID + "Util", "downloadFile", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V",
                saveAs, url, "", "downloadManagerObj.downloadSuccess", params);

        } else if (utilObj.isIos()) {
            // updating saveAs here as downloadImageV1 requires the full path rather than just the image name
            saveAs = utilObj.getFilesDir() + saveAs + "";
            jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_IOS + "Util", "downloadImageV1:fileName:callback:callbackMsg:", url, saveAs, "downloadManagerObj.downloadSuccess", params);
        }
    };

    this.downloadSuccess = function (params) {
        params = JSON.parse(params);
        loggerObj.debugLog("downloadManager Log: Successfully downloaded:", utilObj.getStringifiedJson(params));

        var value = {
            t: utilObj.getCurrentTimestamp(),
            v: params.v
        };
        downloadsMap[params.f] = value;

        // loggerObj.debugLog("downloadManager Log: SuccesCallBackFunction OUT",successCallBackFunctionMap);
        // Added From WTrips DownloadManager
        if (utilObj.isValid(successCallBackFunctionMap[params.f])) {
            // loggerObj.debugLog("downloadManager Log: SuccesCallBackFunction" , successCallBackFunctionMap[params.f]);
            eval(successCallBackFunctionMap[params.f] + "()");
        }
        

        downloadManagerObj.save();
        trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_DOWNLOAD, params.v, params.f, constantsObj.TRACK_SUCCESS, "", "", "", "");
    };

    this.isFileInDownloadList = function (fileName) {
        if (filesToDownload[fileName]) {
            return true;
        }
        return false;
    };

    this.isFileDownloaded = function (fileName) {
        if (downloadsMap[fileName] != undefined) {
            return true;
        }
        return false;
    };

    this.getAbsoluteFilePath = function (fileName) {
        return utilObj.getAbsFilePath(fileName);
    };

    this.versionCheck = function (fileName, version) {
        var isDownloaded = downloadManagerObj.isFileDownloaded(fileName);
        loggerObj.debugLog("downloadManager Log: isFile downloaded and fileName", isDownloaded, fileName);
        if (isDownloaded) {
            var info = downloadsMap[fileName];
            var downloadedVersion = parseInt(info.v);
            if (downloadedVersion >= version) {
                return false;
            }
        }
        // loggerObj.debugLog("downloadManager Log: version check return true");
        return true;
    };

    this.loadData = function (fileName, callBack, localUrl) {
        var urlToUse = localUrl;
        if (fileName != "dtc2_countries.json") {
            if (downloadManagerObj.isFileDownloaded(fileName)) {
                loggerObj.debugLog("downloadManager Log: file Downloaded", fileName);
                urlToUse = downloadManagerObj.getAbsoluteFilePath(fileName);
                if (!jsb.fileUtils.isFileExist(urlToUse)) {
                    urlToUse = localUrl;
                    downloadManagerObj.setDownloadsMapUndefinedForFile(fileName);
                }
            }
        }
        loggerObj.debugLog("DOWNLOAD MANAGER : load Data url to use", urlToUse);
        if (urlToUse == undefined) {
            return;
        }
        cc.loader.loadJson(urlToUse, function (error, data) {
            if (utilObj.isFunction(callBack)) {
                callBack(error, data);
            }
        });
    };

    this.getFileNameWithoutExtension = function (fileName) {
        var filePrefix = fileName.substring(0, fileName.indexOf('.'));
        return filePrefix;
    };

    this.setDownloadsMapUndefinedForFile = function (fileName) {
        delete downloadsMap[fileName];
        downloadManagerObj.save();
        trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_DOWNLOAD, "", fileName, "file_missing", "", "", "", "");
    };
    //TODO: Remove files from app storage which are marked as downloaded, but not present in downloads json.
    //this.cleanupFiles
};
