/**
 * Model file for scrabble tile keyboard
 */
ScrabbleKeyboard = function () {

    // Scrable Keyboard contants
    var GRID_HEIGHT_16_9 = 1250;
    var GRID_HEIGHT_18_9 = 1520;
    this.AREA_TOP_BAR = 1;
    this.AREA_CLUE_BAR = 2;
    this.AREA_HINT_BAR = 3;
    this.BOTTOM_BLANK_AREA = 4;
    this.SHADE_WIDTH_VAR3 = 160;
    this.SHADE_WIDTH_VAR4 = 215;

    // Scrable Keyboard contants
    
    //LETTER COMBINATION ARRAY ::: DONOT CHANGE THIS
    this.LETTER_DISTRIBUTION_ARR = [[1],[2],[3],[4],[3,2],[3,3],[4,3],[4,4],[4,3,2],[4,3,3],[4,4,3],[5,4,3],[5,4,4],[5,5,4],[6,5,4],[6,5,5]];

    //Max value till which drag for user will be tracked
    this.MAX_DRAG_COUNT = 5;
    this.DRAG_OFFSET = 220;
    
    var seeWrongState = 0;
    var seeWrongStateDp = 0;
    var dragTrackCount = 0;
    var isRepeatedLtrToaterShown = false;
    var wrongTapCnt = 0;
    var wrongTapMesgShown = false;

    // see wrong state for sounds
    this.SW_NOT_PURCHASED = 0;
    this.SW_ON = 1;
    this.SW_OFF = 2; 

    //Drag types
    this.KB_DRAG = 1;
    this.KB_GRID_DRAG = 2;

    //PUZZLE TO SHOW REPEATED LETTER TOSTER
    this.PUZZLE_TO_SHOW_REPEAT_TOASTER = 2;

    //NUM OF WRONG TAPS AFTER WHICH TO SHOW TOASTER
    this.WRONG_TAP_THRESHOLD = 2;

    this.SCRABBLE_BG_HEIGHT = 600;
    this.init = function(){
        //init 
    };

     /**
     * loads object variables from a file
     * @param gameData
     */
    this.loadLocal = function (gameData) {
        if (gameData.seeWrongState != undefined) {
            seeWrongState = gameData.seeWrongState;
        }
        if (gameData.seeWrongStateDp != undefined) {
            seeWrongStateDp = gameData.seeWrongStateDp;
        }
        if (gameData.dragTrackCount != undefined) {
            dragTrackCount = gameData.dragTrackCount;
        }
        if (gameData.isRepeatedLtrToaterShown != undefined) {
            isRepeatedLtrToaterShown = gameData.isRepeatedLtrToaterShown;
        }
        if (gameData.wrongTapCnt != undefined) {
            wrongTapCnt = gameData.wrongTapCnt;
        }
        if (gameData.wrongTapMesgShown != undefined) {
            wrongTapMesgShown = gameData.wrongTapMesgShown;
        }
    };

    /**
     * gets object variables which need to be saved to a file
     * @param gameData
     * @returns {*}
     */
    this.getObjToSave = function (gameData) {
        gameData.seeWrongState = seeWrongState;
        gameData.seeWrongStateDp = seeWrongStateDp;
        gameData.dragTrackCount = dragTrackCount;
        gameData.isRepeatedLtrToaterShown = isRepeatedLtrToaterShown;
        gameData.wrongTapCnt = wrongTapCnt;
        gameData.wrongTapMesgShown = wrongTapMesgShown;
        return gameData;
    };

    this.getClueBarHeight = function() {
        return 230;
    };

    this.getClueBarHeightForDevice = function () {
        return scrabbleKeyboardObj.getClueBarHeight() + scrabbleKeyboardObj.extraSpaceDistribution(scrabbleKeyboardObj.AREA_CLUE_BAR);
    };

    this.getBannerAreaHeight = function() {
        return 220 + constantsObj.BEZEL_SAFE_AREA;
    };

    this.topBarHeight = function (){
        return 100;
    };

    this.getTopBarHeightForDevice = function () {
        return scrabbleKeyboardObj.topBarHeight() + scrabbleKeyboardObj.extraSpaceDistribution(scrabbleKeyboardObj.AREA_TOP_BAR);
    };

    this.getYPosForPuzzleScreenTopBarElements = function() {
        return utilObj.getDesignHeight() - scrabbleKeyboardObj.getBannerAreaHeight() - scrabbleKeyboardObj.getTopBarHeightForDevice();
    };

    this.getGridPixelHeight = function() {
        var aspectRatio = utilObj.getAspectRatio();
        if (aspectRatio >= 2) {
            return GRID_HEIGHT_18_9;
        }

        return GRID_HEIGHT_16_9;
    };

    //Getter /setter for drag track counter
    this.getDragTrackCount = function () {
        return dragTrackCount;
    };

    this.setDragTrackCount = function () {
        dragTrackCount += 1;
    };

     /**
     * function to return the hint bar height
     * @returns {number}
     */
    this.getHintsHeight = function () {
        var hintHeight = 170;
        return hintHeight;
    };

    /**
     * function to check for the extra space on puzzle screen
     * @returns {number}
     */
    this.extraSpaceOnPuzzleScreen = function () {
        var extraSpace = utilObj.getDesignHeight() - scrabbleKeyboardObj.getBannerAreaHeight() - scrabbleKeyboardObj.topBarHeight() - scrabbleKeyboardObj.getGridPixelHeight()
            - scrabbleKeyboardObj.getClueBarHeight() - scrabbleKeyboardObj.SCRABBLE_BG_HEIGHT - scrabbleKeyboardObj.getHintsHeight() - scrabbleKeyboardObj.getBottomBlankAreaHeight(); // - scrabbleKeyboardObj.getBottomBannerAreaHeight();
        return extraSpace;
    };

     /**
     * function to distribute extra space on puzzle screen between top bar, clue bar and hint bar
     * @param area
     * @returns {number}
     */
    this.extraSpaceDistribution = function (area) {
        var extraSpace = scrabbleKeyboardObj.extraSpaceOnPuzzleScreen();
        var res = 0;
        if (extraSpace <= 0) {
            return res;
        }
        var areaTopPercent = 13.18;
        var areaCluePercent = 45;
        var areaHintPercent = 21.82;
        var areaBottonBlank = 20;

        if (utilObj.isIos() && utilObj.getIsLikeIphoneX()) {
            areaTopPercent = 3.18;
            areaCluePercent = 45;
            areaHintPercent = 11.82;
            areaBottonBlank = 25;
        }

        switch (area) {
            case scrabbleKeyboardObj.AREA_TOP_BAR:
                res = extraSpace * areaTopPercent / 100;
                break;
            case scrabbleKeyboardObj.AREA_CLUE_BAR:
                res = extraSpace * areaCluePercent / 100;
                break;
            case scrabbleKeyboardObj.AREA_HINT_BAR:
                res = extraSpace * areaHintPercent / 100;
                break;
            case scrabbleKeyboardObj.BOTTOM_BLANK_AREA:
                res = extraSpace * areaBottonBlank /100;
            default:
                break;
        }
        return res;
    };

    this.getBottomBlankAreaHeight = function () {
        var bottomBlankAreaHeight = 0;
        if (utilObj.getIsLikeIphoneX()) {
            bottomBlankAreaHeight = 50;
        }
        return bottomBlankAreaHeight;
    };

    this.getHintsYPosShift = function () {
        var yPosShift = 0;
        var hintExtraSpace = scrabbleKeyboardObj.extraSpaceDistribution(scrabbleKeyboardObj.AREA_HINT_BAR);
        if (hintExtraSpace - 20 >= 0) {
            yPosShift += (hintExtraSpace - 20) / 2;
        }
        return yPosShift;
    };

    this.getTopBarHeight = function () {
        var topBarAreaHeight = 140;
        return topBarAreaHeight;
    };

    this.getScrabbleBgHeight = function () {
        var elementHeight = scrabbleKeyboardObj.getBannerBgHeight()  + scrabbleKeyboardObj.getGridPixelHeight() + scrabbleKeyboardObj.getClueBarHeight();
        return utilObj.getDesignHeight()  - elementHeight;
    };

    /**
     * returns the grid bg sprite
     * @returns
     */
    this.getGridBg = function () {
        return constantsObj.trans_button;
    };

    /**
     * returns header UI data
     * @returns 
     */
    this.getHeaderUIData = function () {
        return {
            btnBgColor: constantsObj.COLOR_BLACK,
            btnBgOpacity: constantsObj.OPACITY_30
        };
    };

    /**
     * returns banner BG UI data
     * @returns 
     */
    this.getBannerBgUIData = function () {
        return {
            bannerBgColor: utilObj.getColorForHexCode(realisticBgObj.getRealisticBgInfoMap()["tc2"]),
            bannerBgOpacity: constantsObj.OPACITY_100
        };
    };

    /**
     * returns clue bar UI data
     * @returns 
     */
    this.getClueBarUIData = function (isClassicThemeOpted) {
        var isClassicTheme = false;
        if (isClassicThemeOpted != undefined) {
            isClassicTheme = isClassicThemeOpted;
        }
        return {
            clueBgSprite: ((isClassicTheme ? constantsObj.tile_white : constantsObj.trans_button)),
            clueBgImg: true,
            shadeSprite: constantsObj.white_shade,
            clueBgColor: utilObj.getColorForHexCode(realisticBgObj.getRealisticBgInfoMap()["tc2"]),
            shadeColor: utilObj.getColorForHexCode(realisticBgObj.getRealisticBgInfoMap()["cc"]),
            arrowImg: constantsObj.clue_arrow,
            arrowScaling: constantsObj.SCALE_SPRITE_14,
            shadeWidth: scrabbleKeyboardObj.SHADE_WIDTH_VAR4,
            qwertyV2: true
        };
    };

    /** 
     * returns the icon holder for button clear amd shuffle on SK GL
     */
    this.getIconHolderSprite = function () {
        return (constantsObj.trans_button);
    }; 

    /**
     * returns the blur bg img and gradient to add in the current puzzle screen
     */
    this.getBlurBgAndGradient= function () {
        return { 
            "b" : realisticBgObj.getRealisticBgInfoMap()["b"], 
            "gr" : realisticBgObj.getRealisticBgInfoMap()["gr"]
        }
    };

    /**
     * returns qwerty kb bg color
     */
    this.getQwertyBgColorUIData = function (isClassicThemeOpted) {
        var isClassicTheme = false;
        if (isClassicThemeOpted != undefined) {
            isClassicTheme = isClassicThemeOpted;
        }
        return {
            bgColor: utilObj.getColorForHexCode(realisticBgObj.getRealisticBgInfoMap()["tc2"]),
            bgOpacity: constantsObj.OPACITY_100
        }
    };
    /**
     * returns active key sprite for the theme 
     */
    this.getActiveKeySprite = function () {
        return (realisticBgObj.getRealisticBgInfoMap()["ak2"]);
    };

    /**
     * returns tap color to be shown (keyPopup Bg Color)
     * @returns 
     */
    this.getTileTapColor = function () {
        return utilObj.getColorForHexCode(realisticBgObj.getRealisticBgInfoMap()["kc2"]);
    };

    /**
     * returns In game notif ui data 
     * @returns 
     */
    // NOTE: currently this image is not present in spritesheet as there is no ingame notif in arcade
    this.getInGameNotifBgUIData = function () {
        return {
            bgTile: realisticBgObj.getRealisticBgInfoMap()["nbg"],
            bgHeight: 300,
            bgOpacity: constantsObj.OPACITY_90
        };
    };

    /**
     * returns the keypopup bg sprite 
     * @param {*} isClassicThemeOpted 
     * @param {*} isSideButtonTap 
     * @returns 
     */
    this.getKeyPopupSprite = function (isClassicThemeOpted, isSideButtonTap) {
        var isClassicTheme = false;
        if (isClassicThemeOpted != undefined) {
            isClassicTheme = isClassicThemeOpted;
        }
        return (isSideButtonTap ? constantsObj.side_button_tap_qt : constantsObj.button_tap_qt);
    };

    /**
     * returns the keypopup bg sprite for tab
     * @param {*} isClassicThemeOpted 
     * @param {*} isSideButtonTap 
     * @returns 
     */
    this.getKeyPopupSpriteForTab = function (isClassicThemeOpted, isSideButtonTap) {
        var isClassicTheme = false;
        if (isClassicThemeOpted != undefined) {
            isClassicTheme = isClassicThemeOpted;
        }
        return (isSideButtonTap ? constantsObj.side_button_tap_tab : constantsObj.button_tap_qt_tab);
    };

    /**
     * checks whether to show repeated letter toaster 
     * @returns 
     */
    this.toShowRepeatingLetterToaster = function () {
        if (isRepeatedLtrToaterShown) {
            return false;
        }
        return (linearLevelObj.getDtc2PuzzleCompleted() + 1 >= scrabbleKeyboardObj.PUZZLE_TO_SHOW_REPEAT_TOASTER);
    };

    /**
     *  Setter/Gettet to track repeated letter toaster shown or not
     */
    this.setRepeatedLtrToasterShown = function (val) {
        isRepeatedLtrToaterShown = val;
    };

    this.getRepeatedLtrToasterShown = function () {
        return isRepeatedLtrToaterShown;
    };

    /**
     *  Getter/Setter for wrong tap count and toaster shown 
     * @returns 
     */
    this.getWrongTapCnt = function () {
        return wrongTapCnt;
    };

    this.setWrongTapCnt = function () {
        wrongTapCnt += 1;
    };

    this.setWrongTapMesgShown = function (val) {
        wrongTapMesgShown = val;
    };

    this.getWrongTapMesgShown = function () {
        return wrongTapMesgShown;
    };
};
