TimedHints = function(){
    var DATA_CALCULATION_DAYS = 2;
    var DEFAULT_INVENTORY = 1;
    var FIRST_SESSION = 3;
    var REFRESH = 4;
    var DEFAULT_REFRESH_TIME = 120;
    var REFILL_NOTIF_CAP = 2;

    var hintUsageMap = {};
    var sessionDataMap = {};
    var appResumeTime = 0;
    var hintInventory = {};
    var hintRefreshTime = {};
    var lastHintUseTime = {};
    var estimatedSessionEndTime = 0;
    var refillNotifTimestamp = 0;
    var refillNotifCount = 0;

    this.getHintUsageMap = function(){
        return hintUsageMap;
    };

    this.getSessionDataMap = function(){
        return sessionDataMap;
    };

    this.getAppResumeTime = function(){
        return appResumeTime;
    };

    this.getHintInventoryMap = function(){
        return hintInventory;
    };

    this.getHintRefreshTime = function(){
        return hintRefreshTime;
    };
    
    this.getLastHintUseTimeMap = function(){
        return lastHintUseTime;
    };

    this.getEstimatedSessionEndTime = function(){
        return estimatedSessionEndTime;
    };

    this.getRefillNotifTimestamp = function(){
        return refillNotifTimestamp;
    };

    this.setRefillNotifTimestamp = function(val){
        refillNotifTimestamp = val;
    };

    this.resetRefillNotifTimestamp = function(){
        refillNotifTimestamp = 0;
    };

    this.getRefillNotifCount = function(){
        return refillNotifCount;
    };

    this.setRefillNotifCount = function(val){
        refillNotifCount = val;
    };

    this.resetRefillNotifCount = function(){
        refillNotifCount = 0;
    };

    this.incRefillNotifCount = function(){
        refillNotifCount += 1;
    };

    /**
     * initialiser for timed Hints
     */
    this.init = function(){
        // initialisation code
    };

    /**
     * loads variable from game data file
     * @param {*} gameData 
     */
    this.loadLocal = function(gameData){
        if(gameData.hintUsageMap != undefined){
            hintUsageMap = gameData.hintUsageMap;
        }
        if(gameData.sessionDataMap != undefined){
            sessionDataMap = gameData.sessionDataMap;
        }
        if(gameData.hintInventory != undefined){
            hintInventory = gameData.hintInventory;
        }
        if(gameData.hintRefreshTime != undefined){
            hintRefreshTime = gameData.hintRefreshTime;
        }
        if(gameData.lastHintUseTime != undefined){
            lastHintUseTime = gameData.lastHintUseTime;
        }
        if(gameData.refillNotifTimestamp != undefined){
            refillNotifTimestamp = gameData.refillNotifTimestamp;
        }
        if(gameData.refillNotifCount != undefined){
            refillNotifCount = gameData.refillNotifCount;
        }
    };

    /**
     * saves variable to the game data file
     * @param {*} gameData 
     */
    this.getObjToSave = function (gameData) {
        gameData.hintUsageMap = hintUsageMap;
        gameData.sessionDataMap = sessionDataMap;
        gameData.hintInventory = hintInventory;
        gameData.hintRefreshTime = hintRefreshTime;
        gameData.lastHintUseTime = lastHintUseTime;
        gameData.refillNotifTimestamp = refillNotifTimestamp;
        gameData.refillNotifCount = refillNotifCount;
        return gameData;
    };

    /**
     * checks for valid hint type
     * @param {*} hintType 
     * @returns 
     */
    this.isValidHintType = function (hintType) {
        var hintTypesArr = timedHintsObj.getHintTypes();
        return (hintTypesArr.indexOf(hintType) > -1);
    };

    /**
     * updates hintUsage map for today for the passed hintType value
     * @param {*} hintType 
     * @returns 
     */
    this.updateHintUsage = function (hintType) {
        if (!timedHintsObj.isValidHintType(hintType)) {
            return false;
        }

        var todaysId = utilObj.getIdForDate(gameObj.getTodaysPuzzleDate());
        if (!utilObj.isOwnKeyOfObject(hintUsageMap, todaysId)) {
            hintUsageMap[todaysId] = {};
        }
        if (!utilObj.isOwnKeyOfObject(hintUsageMap[todaysId], hintType)) {
            hintUsageMap[todaysId][hintType] = 0;
        }

        hintUsageMap[todaysId][hintType] += 1;
        return true;
    };

    /**
     * handler for hint use
     * @param {*} hintType 
     * @returns 
     */
    this.onHintUse = function(hintType){
        if (!timedHintsObj.isValidHintType(hintType)) {
            return false;
        }

        timedHintsObj.updateHintUsage(hintType);
        timedHintsObj.useHintInventory(hintType);
        timedHintsObj.setLastHintUseTime(hintType);
        timedHintsObj.checkAndRefreshHint(hintType, utilObj.getCurrentTimestamp());
        timedHintsObj.updateGameLayerSprites();
    };

    /**
     * returns average daily hint usage for hint type
     * excludes data for today for average calculation
     * @param {*} hintType 
     * @returns 
     */
    this.getAverageDailyHintUsage = function (hintType) {
        var avg = 0;
        if (!timedHintsObj.isValidHintType(hintType)) {
            return avg;
        }

        var obj = hintUsageMap;
        var numKeys = 0;
        var usageCount = 0;
        var hintData = {};
        var todaysId = utilObj.getIdForDate(gameObj.getTodaysPuzzleDate());
        // loop through the hint Usage Map and calculate average for the given hint type
        for (var key in obj) {
            if (Object.prototype.hasOwnProperty.call(obj, key)) {
                // skipping data for today
                if (key == todaysId) {
                    continue;
                }
                numKeys++;
                hintData = obj[key];
                if (Object.prototype.hasOwnProperty.call(hintData, hintType)) {
                    usageCount += hintData[hintType];
                }
            }
        }

        // safety check for 0 keys
        if (numKeys == 0) {
            return avg;
        }

        avg = usageCount / numKeys;
        return avg;
    };

    /**
     * initialises hint usage map for the day
     */
    this.initHintUsage = function () {
        var todaysId = utilObj.getIdForDate(gameObj.getTodaysPuzzleDate());
        if (!utilObj.isOwnKeyOfObject(hintUsageMap, todaysId)) {
            hintUsageMap[todaysId] = {};
        }
        var hintTypesArr = timedHintsObj.getHintTypes();
        var numHintTypes = hintTypesArr.length;
        for (var i = 0; i < numHintTypes; i++) {
            var hintType = hintTypesArr[i];
            if (!utilObj.isOwnKeyOfObject(hintUsageMap[todaysId], hintType)) {
                hintUsageMap[todaysId][hintType] = 0;
            }
        }
    };

    /**
     * removes old data which is more then "data_calculation_days" days old
     * @param {*} obj 
     * @returns 
     */
    this.removeOldData = function(obj){
        if(!utilObj.isObject(obj)){
            return;
        }

        var date;
        var todaysPuzzleDate = gameObj.getTodaysPuzzleDate();
        for (var key in obj) {
            if (Object.prototype.hasOwnProperty.call(obj, key)) {
                date = utilObj.getDateForId(key);
                if (utilObj.getDaysDiff(todaysPuzzleDate, date) > DATA_CALCULATION_DAYS) {
                    delete obj[key];
                }
            }
        }
    };

    /**
     * removes all the old data from hint usage and session data maops
     */
    this.removeAllOldData = function(){
        var objArr = [hintUsageMap, sessionDataMap];
        var numObjs = objArr.length;
        for(var i = 0; i < numObjs; i++){
            timedHintsObj.removeOldData(objArr[i]);
        }
    };

    /**
     * initialises session data map for the day
     */
    this.initSessionData = function () {
        var todaysId = utilObj.getIdForDate(gameObj.getTodaysPuzzleDate());
        if (!utilObj.isOwnKeyOfObject(sessionDataMap, todaysId)) {
            sessionDataMap[todaysId] = [];
        }
    };

    /**
     * adds current session data in session data map
     */
    this.updateSessionData = function (curTimestamp) {
        var todaysId = utilObj.getIdForDate(gameObj.getTodaysPuzzleDate());
        if (!utilObj.isOwnKeyOfObject(sessionDataMap, todaysId)) {
            sessionDataMap[todaysId] = [];
        }
        var timeStamp = curTimestamp;
        if(isNaN(timeStamp) || timeStamp == null){
            timeStamp = utilObj.getCurrentTimestamp();
        }
        var sessionTime = timeStamp - appResumeTime;
        if (sessionTime > 0) {
            sessionDataMap[todaysId].push(sessionTime);
        }
    };

    /**
     * returns the average of all the session times
     * excludes todays data from calculation
     */
    this.getAverageSessionTime = function () {
        var avg = 0;
        var obj = sessionDataMap;
        var numSessions = 0;
        var totalSessionTime = 0;
        var todaysId = utilObj.getIdForDate(gameObj.getTodaysPuzzleDate());
        for (var key in obj) {
            if (Object.prototype.hasOwnProperty.call(obj, key)) {
                if (key == todaysId) {
                    continue;
                }
                var sessionsArr = obj[key];
                if (utilObj.isArray(sessionsArr)) {
                    numSessions += sessionsArr.length;

                    var reducer = (accumulator, curr) => accumulator + curr;
                    totalSessionTime += sessionsArr.reduce(reducer);
                }
            }
        }

        if (numSessions == 0) {
            return avg;
        }

        avg = totalSessionTime / numSessions;
        return avg;
    };

    /**
     * returns inventory for hintType
     * @param {*} hintType 
     * @returns 
     */
    this.getHintInventory = function (hintType) {
        if (!timedHintsObj.isValidHintType(hintType)) {
            return DEFAULT_INVENTORY;
        }
        if (!utilObj.isOwnKeyOfObject(hintInventory, hintType)) {
            hintInventory[hintType] = DEFAULT_INVENTORY;
        }
        return hintInventory[hintType];
    };

    /**
     * returns last hint use time for hint type
     * @param {*} hintType 
     * @returns 
     */
    this.getLastHintUseTime = function(hintType) {
        if (!timedHintsObj.isValidHintType(hintType)) {
            return 0;
        }
        if (!utilObj.isOwnKeyOfObject(lastHintUseTime, hintType)) {
            lastHintUseTime[hintType] = 0;
        }
        return lastHintUseTime[hintType];
    };

    /**
     * sets last hint use time for hintType
     * @param {*} hintType 
     * @returns 
     */
    this.setLastHintUseTime = function(hintType){
        if (!timedHintsObj.isValidHintType(hintType)) {
            return false;
        }
        lastHintUseTime[hintType] = utilObj.getCurrentTimestamp();
        return true;
    };

    /**
     * uses hint invntory for hintType
     * @param {*} hintType 
     * @returns 
     */
    this.useHintInventory = function (hintType) {
        if (!timedHintsObj.isValidHintType(hintType)) {
            return false;
        }

        if (!utilObj.isOwnKeyOfObject(hintInventory, hintType)) {
            hintInventory[hintType] = DEFAULT_INVENTORY;
        }

        hintInventory[hintType] -= 1;
        return true;
    };

    /**
     * sets estimated session end time
     */
    this.setEstimatedSessionEndTime = function () {
        estimatedSessionEndTime = (appResumeTime + Math.max(DEFAULT_REFRESH_TIME, Math.floor(timedHintsObj.getAverageSessionTime())));
    };

    /**
     * handles daily updates for timed hints
     * @param {*} isFirstSession 
     */
    this.updateDailyInfo = function (isFirstSession) {
        if (isFirstSession) {
            timedHintsObj.removeAllOldData();
            timedHintsObj.initHintUsage();
            timedHintsObj.initSessionData();
            timedHintsObj.initHintInventory();
            timedHintsObj.resetHintRefreshTime();
            timedHintsObj.resetRefillNotifCount();
        }
        timedHintsObj.appResume();
    };

    /**
     * timed hints app resume
     */
    this.appResume = function () {
        var curTimestamp = utilObj.getCurrentTimestamp();
        timedHintsObj.updateAppResumeTime(curTimestamp);
        timedHintsObj.setEstimatedSessionEndTime();
        timedHintsObj.refillNotifAppResume(curTimestamp);
        timedHintsObj.updateGameLayerSprites();
        timedHintsObj.checkAndShowHintRefillToaster();
    };

    /**
     * shows hint refill toaster if the refill variable is set
     * resets the variable after showing toaster message
     */
    this.checkAndShowHintRefillToaster = function () {
        if (curScreen == constantsObj.SCREEN_GAME) {
            var refillNotifClicked = utilObj.getRefill();
            if (refillNotifClicked && refillNotifClicked == "1") {
                utilObj.showMessage(stringsObj.refill_toaster_message);
                utilObj.resetRefill();
            }
        }
    };

    /**
     * app resume logic for refill notification
     * @param {*} curTimestamp 
     */
    this.refillNotifAppResume = function (curTimestamp) {
        var timeStamp = curTimestamp;
        if (isNaN(timeStamp) || timeStamp == null) {
            timeStamp = utilObj.getCurrentTimestamp();
        }
        
        if (refillNotifTimestamp && timeStamp > refillNotifTimestamp) {
            timedHintsObj.incRefillNotifCount();
        }

        timedHintsObj.resetRefillNotifTimestamp();

        // updates refill notif variables
        timedHintsObj.updateRefillNotifVariables(curTimestamp);
    };

    /**
     * timed hints app start or resume
     */
    this.updateAppResumeTime = function (curTimestamp) {
        var timeStamp = curTimestamp;
        if(isNaN(timeStamp) || timeStamp == null){
            timeStamp = utilObj.getCurrentTimestamp();
        }

        appResumeTime = timeStamp;
    };

    /**
     * updates variables for refill notification
     * @param {*} curTimestamp 
     * @returns 
     */
    this.updateRefillNotifVariables = function(curTimestamp){
        var timeStamp = curTimestamp;
        if(isNaN(timeStamp) || timeStamp == null){
            timeStamp = utilObj.getCurrentTimestamp();
        }

        // resetting refill notif timestamp value
        timedHintsObj.resetRefillNotifTimestamp();

        // check for maxRefreshTime value
        var hintTypesArr = timedHintsObj.getHintTypes();
        var numHints = hintTypesArr.length;
        var maxRefreshTime = 0;
        for (var i = 0; i < numHints; i++) {
            var hintType = hintTypesArr[i];
            maxRefreshTime = Math.max(maxRefreshTime, timedHintsObj.getRefreshTimeForHint(hintType));
        }
        if (maxRefreshTime < timeStamp) {
            return;
        }

        // notif hours condition
        var date = new Date(maxRefreshTime * 1000);
        var hour = date.getHours();
        if (hour >= 23 || hour <= 8) {
            return;
        }

        // notif cap condition
        if (refillNotifCount >= REFILL_NOTIF_CAP) {
            return;
        }

        // setting the values for refillNotif time stamp
        timedHintsObj.setRefillNotifTimestamp(maxRefreshTime);

        // game save call
        gameObj.save();
    };

    /**
     * timed hints app pause
     */
    this.appPause = function () {
        var curTimestamp = utilObj.getCurrentTimestamp();
        timedHintsObj.updateSessionData(curTimestamp);
        utilObj.resetRefill();
    };

    /**
     * returns hints to add in hint Inventory
     * handles first session and refresh scenarios
     * @param {*} source 
     * @param {*} hintType 
     * @returns 
     */
    this.getHintsToAdd = function (source, hintType) {
        var factor = 0.50;
        if (source == FIRST_SESSION) {
            factor = 0.50;
        }
        return Math.max(DEFAULT_INVENTORY, Math.floor(timedHintsObj.getAverageDailyHintUsage(hintType) * factor));
    };

    /**
     * initialises hint inventory
     */
    this.initHintInventory = function () {
        var hintTypesArr = timedHintsObj.getHintTypes();
        var numHintTypes = hintTypesArr.length;
        for (var i = 0; i < numHintTypes; i++) {
            var hintType = hintTypesArr[i];
            hintInventory[hintType] = timedHintsObj.getHintsToAdd(FIRST_SESSION, hintType);
        }
    };

    /**
     * resets hint refresh time in first session of the day
     */
    this.resetHintRefreshTime = function(){
        var hintTypesArr = timedHintsObj.getHintTypes();
        var numHintTypes = hintTypesArr.length;
        for (var i = 0; i < numHintTypes; i++) {
            var hintType = hintTypesArr[i];
            hintRefreshTime[hintType] = 0;
        }
    };

    /**
     * returns hint types
     * @returns 
     */
    this.getHintTypes = function () {
        return [constantsObj.REVEAL_LETTER_HINT, constantsObj.SHOW_WORD_HINT];
    };

    /**
     * returns hint refresh time for hint
     * @param {*} hintType 
     * @returns 
     */
    this.getRefreshTimeForHint = function(hintType){
        if(!timedHintsObj.isValidHintType(hintType)){
            return 0;
        }

        if(!utilObj.isOwnKeyOfObject(hintRefreshTime, hintType)){
            hintRefreshTime[hintType] = 0;
        }
        return hintRefreshTime[hintType];
    };

   /**
    * returns time left for hint refresh
    * @param {*} hintType 
    * @returns 
    */
    this.getTimeLeftForHintRefresh = function(hintType, curTimestamp){
        if(!timedHintsObj.isValidHintType(hintType)){
            return 0;
        }
        var timeStamp = curTimestamp;
        if(isNaN(timeStamp) || timeStamp == null){
            timeStamp = utilObj.getCurrentTimestamp();
        }
        return (timedHintsObj.getRefreshTimeForHint(hintType) - timeStamp);
    };

    /**
     * returns formatted time left string for hint refresh for hintType
     * @param {*} hintType 
     * @param {*} curTimestamp 
     * @returns 
     */
    this.getFormattedTimeForHintRefresh = function(hintType, curTimestamp){
        return utilObj.formatTime3(timedHintsObj.getTimeLeftForHintRefresh(hintType, curTimestamp));
    };

    /**
     * returns formatted time for hint refresh toaster message
     * @param {*} hintType 
     * @param {*} curTimestamp 
     * @returns 
     */
    this.getFormattedTimeForHintRefreshToasterMessage = function(hintType, curTimestamp){
        return utilObj.formatTime2(timedHintsObj.getTimeLeftForHintRefresh(hintType, curTimestamp));
    };

    /**
     * returns refill label for hintType
     * @param {*} hintType 
     */
    this.getRefillLbl = function (hintType) {
        return (hintType == constantsObj.REVEAL_LETTER_HINT ? gameLayer.revealLettersRefillLbl : gameLayer.showWordRefillLbl);
    };

    /**
     * checks and refreshes given hintType
     * @param {*} hintType 
     * @param {*} curTimestamp 
     */
    this.checkAndRefreshHint = function (hintType, curTimestamp) {
        if (!timedHintsObj.isValidHintType(hintType) || isNaN(curTimestamp) || curTimestamp == null) {
            return;
        }

        var inventory = timedHintsObj.getHintInventory(hintType);
        if (inventory > 0) {
            return;
        }
        var refreshTimeUpdated = false;
        var lastHintUse = timedHintsObj.getLastHintUseTime(hintType);
        if (!hintRefreshTime[hintType]) {
            hintRefreshTime[hintType] = Math.max(lastHintUse + DEFAULT_REFRESH_TIME, estimatedSessionEndTime);
            refreshTimeUpdated = true;
        }
        if (curTimestamp > hintRefreshTime[hintType]) {
            hintInventory[hintType] += timedHintsObj.getHintsToAdd(REFRESH, hintType);
            hintRefreshTime[hintType] = 0;
            refreshTimeUpdated = true;
            timedHintsObj.updateGameLayerSprites();
        } else {
            // updates hint refill timer label
            utilObj.setString(timedHintsObj.getRefillLbl(hintType), timedHintsObj.getFormattedTimeForHintRefresh(hintType, curTimestamp));
        }

        // updates refill notif variables in case of updates in hintRefreshTime map
        if (refreshTimeUpdated) {
            timedHintsObj.updateRefillNotifVariables(curTimestamp);
        }
    };

    /**
     * refresh logic for all hintTypes
     * @param {*} curTimestamp 
     */
    this.checkAndRefreshAllHints = function (curTimestamp) {
        var timeStamp = curTimestamp;
        if(isNaN(timeStamp) || timeStamp == null){
            timeStamp = utilObj.getCurrentTimestamp();
        }

        var hintTypesArr = timedHintsObj.getHintTypes();
        var numHints = hintTypesArr.length;
        for (var i = 0; i < numHints; i++) {
            var hintType = hintTypesArr[i];
            timedHintsObj.checkAndRefreshHint(hintType, timeStamp);
        }
    };

    /**
     * updates game layer sprites used for timed hints
     */
    this.updateGameLayerSprites = function () {
        var hintTypesArr = timedHintsObj.getHintTypes();
        var numHintTypes = hintTypesArr.length;

        for (var i = 0; i < numHintTypes; i++) {
            var hintType = hintTypesArr[i];
            var inventory = timedHintsObj.getHintInventory(hintType);

            var hintLbl = gameLayer.showWordLbl;
            var timedHintSprites = gameLayer.showWordTimedHintsSprites;
            if (hintType == constantsObj.REVEAL_LETTER_HINT) {
                hintLbl = gameLayer.revealLettersLbl;
                timedHintSprites = gameLayer.revealLettersTimedHintsSprites;
            }

            utilObj.hideAllNodes(timedHintSprites);
            utilObj.setVisible(hintLbl, true);
            if (inventory <= 0) {
                utilObj.setVisible(hintLbl, false);
                utilObj.showAllNodes(timedHintSprites);
            }
        }
    };

    /**
     * shows the values of timed hints variables in native ios popup
     * @returns 
     */
    this.showTimedHintsLogs = function(){
        var msgStr = "Timed Hints Logs\n";
        msgStr += "hintUsageMap \n " + utilObj.stringifiedObject(timedHintsObj.getHintUsageMap()) + "\n";
        msgStr += "SessionDataMap \n " + utilObj.stringifiedObject(timedHintsObj.getSessionDataMap()) + "\n";
        msgStr += "HintInventoryMap \n " + utilObj.stringifiedObject(timedHintsObj.getHintInventoryMap()) + "\n";
        msgStr += "HintRefreshTime \n " + utilObj.stringifiedObject(timedHintsObj.getHintRefreshTime()) + "\n";
        msgStr += "LastHintUseTimeMap \n " + utilObj.stringifiedObject(timedHintsObj.getLastHintUseTimeMap()) + "\n";
        msgStr += "AppResumeTime \n " + timedHintsObj.getAppResumeTime() + "\n";
        msgStr += "EstimatedSessionEndTime \n " + timedHintsObj.getEstimatedSessionEndTime() + "\n";
        msgStr += "averageHintUsageValue(show word) \n " + timedHintsObj.getAverageDailyHintUsage(constantsObj.SHOW_WORD_HINT) + "\n";
        msgStr += "averageHintUsageValue(reveal letters) \n " + timedHintsObj.getAverageDailyHintUsage(constantsObj.REVEAL_LETTER_HINT) + "\n";
        jsb.reflection.callStaticMethod("Util", "showNativeMagicPopup:", msgStr);
        return;
    };
};