FixedMinis = function () {
    // extra minis to be used for d0 fixed minis and bonus minis
    var extraMinis = {};
    var extraMinisLoaded = false;

    // fixed and Bonus minis will be populated from the extra Minis
    var fixedMinis = {};
    var bonusMinis = {};

    // map for storing crossword date and corresponding fixed mini Id
    var fixedMinisMap = {};
    var bonusMinisMap = {};

    // replacement minis which will replace the existing puzzles as per the assigned date IDs
    var replacementMinis = {};
    var replacementMinisLoaded = false;

    // date for D0 fixed mini puzzle
    var d0FixedMiniDateId = "";

    /**
     * resets all variables
     */
    this.reset = function () {
        extraMinis = {};
        extraMinisLoaded = false;
        fixedMinis = {};
        bonusMinis = {};
        fixedMinisMap = {};
        bonusMinisMap = {};
        replacementMinis = {};
        replacementMinisLoaded = false;
        d0FixedMiniDateId = "";
        loggerObj.debugLog("fixedMinisLog: fbFlow: resetting all fixed mini variables done");
    };

    /**
     * checks if the user is eligible for the d0 fixed minis functionality by checking the variants of "d0_fixed_minis" experiment
     * @returns {boolean}
     */
    this.isEligibleForFixedMinis = function () {
        if (fixedMinisVariant != constantsObj.EXP_VAR_1 && fixedMinisVariant != constantsObj.EXP_VAR_2) {
            return false;
        }
        return true;
    };

    /**
     * loads the extra minis and ifd0FixedMiniDateId is set, updates fixed, bonus and replacement minis
     * extra minis will be used for showing d0 fixed minis and the unused ones will be used for bonus minis
     */
    this.loadExtraMinisAndUpdateMinis = function () {
        var minisUrl = "res/crosswords/extra_minis_enc.json";
        try {
            cc.loader.loadJson(minisUrl, function (error, data) {
                extraMinis = data;
                extraMinisLoaded = true;
                loggerObj.debugLog("fixedMinisLog: loadLog: extraMinis loading done, d0FixedMiniDateId -> ", d0FixedMiniDateId);
                if (d0FixedMiniDateId) {
                    var dateValue = utilObj.getDateForId(d0FixedMiniDateId);
                    loggerObj.debugLog("fixedMinisLog: dateValue -> ", dateValue);
                    fixedMinisObj.splitAndSetMinisInGame(dateValue);
                }
            });
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_LOAD_FIXED_MINIS, minisUrl, constantsObj.TRACK_SUCCESS, "", "", "", "1", "");
        } catch (e) {
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_LOAD_FIXED_MINIS, minisUrl, constantsObj.TRACK_FAIL, "", "", "", "1", "");
        }
    };

    /**
     * loads the replacement minis here
     * replacement minis are to be used instead of the daily minis for any of the date IDS present in the replacement JSON,
     */
    function loadReplacementMinis() {
        var minisUrl = "res/crosswords/replacement_daily_minis_enc.json";
        try {
            cc.loader.loadJson(minisUrl, function (error, data) {
                replacementMinis = data;
                replacementMinisLoaded = true;
                loggerObj.debugLog("fixedMinisLog: loadLog: replacementMinis loading done");
            });
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_LOAD_REPLACEMENT_MINIS, minisUrl, constantsObj.TRACK_SUCCESS, "", "", "", "1", "");
        } catch (e) {
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_LOAD_REPLACEMENT_MINIS, minisUrl, constantsObj.TRACK_FAIL, "", "", "", "1", "");
        }
    }

    /**
     * performs initialisation tasks
     * loads the required JSONs here
     */
    this.init = function () {
        // loading extra minis and replacement minis only if the user is eligible for fixed minis
        loggerObj.debugLog("fixedMinisLog: in Init function, isEligibleForFixedMinis -> ", this.isEligibleForFixedMinis());
        if (this.isEligibleForFixedMinis()) {
            // load the JSON containing fixed and replacement minis and split between fixed and bonus minis
            this.loadExtraMinisAndUpdateMinis();
        }
    };

    /**
     * returns fixedMini obj variables which need to be saved in file
     * @param gameData
     * @returns {*}
     */
    this.getFixedMinisGameData = function (gameData) {
        gameData.d0FixedMiniDateId = d0FixedMiniDateId;
        return gameData;
    };

    /**
     * loads fixedMini obj variables from file
     * @param gameData
     */
    this.loadLocal = function (gameData) {
        if (gameData.d0FixedMiniDateId != undefined) {
            d0FixedMiniDateId = gameData.d0FixedMiniDateId;
        }
    };

    // getters and setters here
    this.getFixedMinisMap = function () {
        return fixedMinisMap;
    };

    this.resetFixedMinisMap = function () {
        fixedMinisMap = {};
    };

    this.getBonusMinisMap = function () {
        return bonusMinisMap;
    };

    this.resetBonusMinisMap = function () {
        bonusMinisMap = {};
    };

    this.getD0FixedMiniDateId = function () {
        return d0FixedMiniDateId;
    };

    this.setD0FixedMiniDateId = function (val) {
        d0FixedMiniDateId = val;
        loggerObj.debugLog("fixedMinisLog: fbFlow: d0FixedMiniDateId set to -> ", val);
    };
    //

    /**
     * splits extra minis between fixed and bonus minis
     * @param fixedMinisCount
     */
    function distributeExtraMinisBetweenFixedAndBonusMinis(fixedMinisCount) {
        loggerObj.debugLog("fixedMinisLog: call to distributeExtraMinisBetweenFixedAndBonusMinis with fixedMinisCount -> ", fixedMinisCount);
        var extraMinisKeysArr = utilObj.getKeysArrayFromMap(extraMinis);
        extraMinisKeysArr.sort(function (a, b) {
            return a - b;
        });

        // setting fixedMinisCount to the minimum of fixedMinisCount and the length of extraMiniKeysArr here
        fixedMinisCount = Math.min(fixedMinisCount, extraMinisKeysArr.length);

        var fixedMinisKeysArr = extraMinisKeysArr.slice(0, fixedMinisCount);
        var bonusMinisKeysArr = extraMinisKeysArr.slice(fixedMinisCount);

        var numFixedMinisKeys = fixedMinisKeysArr.length;
        for (var i = 0; i < numFixedMinisKeys; i++) {
            var keyVal = fixedMinisKeysArr[i];
            fixedMinis[keyVal] = extraMinis[keyVal];
        }

        var numBonusMinisKeys = bonusMinisKeysArr.length;
        for (var i = 0; i < numBonusMinisKeys; i++) {
            var keyVal = bonusMinisKeysArr[i];
            bonusMinis[keyVal] = extraMinis[keyVal];
        }
        loggerObj.debugLog("fixedMinisLog: distributed fixedMinis and bonus minis");
    }

    /**
     * calls to distribute extra minis between fixedMinis and bonusMinis
     */
    function splitExtraMinis() {
        loggerObj.debugLog("fixedMinisLog: call to splitExtraMinis");
        var fixedMinisCount = getNumFixedMinis();
        distributeExtraMinisBetweenFixedAndBonusMinis(fixedMinisCount);
    }

    /**
     * sets fixed mini puzzles to be used in the game from D0 to D(N-1)
     * sets values in fixedMiniMap to map the dailyMiniId to the new mini Id
     */
    function setFixedMiniPuzzlesInGame(d0MiniDate) {
        if (!d0MiniDate) {
            loggerObj.debugLog("fixedMinisLog: Invalid d0MiniDate -> ", d0MiniDate);
            return;
        }
        loggerObj.debugLog("fixedMinisLog: call to setFixedMiniPuzzlesInGame with d0MiniDate -> ", d0MiniDate);
        var currentCrosswordDate = d0MiniDate;
        for (var key in fixedMinis) {
            var dailyMiniId = gameObj.getDailyMiniIdForDate(currentCrosswordDate);
            if (fixedMinis.hasOwnProperty(key)) {
                fixedMinisMap[dailyMiniId] = key;
            }
            currentCrosswordDate = utilObj.getIncrementedDate(currentCrosswordDate, 1);
        }

        loggerObj.debugLog("fixedMinisLog: fixedMinisMap set to -> ", utilObj.getStringifiedJson(fixedMinisMap));
    }

    /**
     * sets bonus mini puzzles to be used in game from D(-1) to D(-(1 + N))
     * sets values in bonusMinisMap to map the dailyMiniId to the new mini Id
     */
    function setFixedBonusMiniPuzzlesInGame(d0MiniDate) {
        if (!d0MiniDate) {
            loggerObj.debugLog("fixedMinisLog: Invalid d0MiniDate -> ", d0MiniDate);
            return;
        }
        loggerObj.debugLog("fixedMinisLog: call to setFixedMiniPuzzlesInGame with d0MiniDate -> ", d0MiniDate);
        var currentCrosswordDate = d0MiniDate;
        for (var key in bonusMinis) {
            currentCrosswordDate = utilObj.getIncrementedDate(currentCrosswordDate, -1);
            var dailyMiniId = gameObj.getDailyMiniIdForDate(currentCrosswordDate);
            if (bonusMinis.hasOwnProperty(key)) {
                bonusMinisMap[dailyMiniId] = key;
            }
        }
        loggerObj.debugLog("fixedMinisLog: bonusMinisMap set to -> ", utilObj.getStringifiedJson(bonusMinisMap));
    }

    /**
     * sets fixed minis and bonus minis in game
     * @param date
     */
    function setFixedAndBonusMinisInGame(date) {
        loggerObj.debugLog("fixedMinisLog: call to setFixedMinisInGame with date -> ", date);
        setFixedMiniPuzzlesInGame(date);
        setFixedBonusMiniPuzzlesInGame(date);
    }

    /**
     * splits extra minis in fixed Minis
     * and calls to set fixed and bonus minis in game
     * and calls to load replacement minis as well
     * thereby setting fixed, bonus and replacement minis
     * @param date
     */
    this.splitAndSetMinisInGame = function (date) {
        loggerObj.debugLog("fixedMinisLog: call to splitAndSetMinisInGame with date -> ", date);
        splitExtraMinis();
        setFixedAndBonusMinisInGame(date);
        loadReplacementMinis();
    };

    /**
     * sets D0 fixedMiniDate and splits extra minis to set fixed and bonus mini in game
     * and then populates fixedMinisMap and bonusMinisMap using fixedMinis and bonusMinis
     * @param date
     */
    this.setD0FixedMiniDateIdAndSplitAndSetMinisInGame = function (date) {
        loggerObj.debugLog("fixedMinisLog: call to setD0FixedMiniDateIdAndSplitAndSetMinisInGame with date ->  ", date);
        var dateId = utilObj.getIdForDate(date);
        this.setD0FixedMiniDateId(dateId);
        this.splitAndSetMinisInGame(date);
    };

    /**
     * returns the number of fixed mini puzzles to be set according to the variant of "d0FixedMinis" experiment
     * @returns {number}
     */
    function getNumFixedMinis() {
        if (fixedMinisVariant == constantsObj.EXP_VAR_2) {
            return 7;
        }
        return 3;
    }

    /**
     * returns fixed mini puzzle by getting the key from "fixedMinisMap" and looking for it in "fixedMinis" map
     * @param date
     * @returns {*}
     */
    function getFixedMiniPuzzleForDate(date) {
        loggerObj.debugLog("fixedMinisLog: In getFixedMiniPuzzleForDate: date -> ", date);
        var fixedMiniPuzzleId = fixedMinisMap[date];
        loggerObj.debugLog("fixedMinisLog: In getFixedMiniPuzzleForDate: fixedMiniPuzzleId -> ", fixedMiniPuzzleId);
        var fixedMiniPuzzle = fixedMinis[fixedMiniPuzzleId];
        // setting d of fixedMiniPuzzle to date value, as this date value gets used while getting the crossword
        fixedMiniPuzzle["d"] = date;
        //
        return fixedMiniPuzzle;
    }

    /**
     * returns fixed mini puzzle id to show for any given date for tracking purposes
     * @param date
     */
    function getFixedMiniIdForTracking(date) {
        return fixedMinisMap[date];
    }

    /**
     * checks if any fixedMini exists for the given date by looking for key value in the "fixedMinisMap"
     * @param date
     * @returns {boolean}
     */
    function checkIfFixedMiniExistsForDate(date) {
        if (fixedMinisMap.hasOwnProperty(date)) {
            return true;
        }
        return false;
    }

    /**
     * returns bonus mini puzzle by getting the key from "bonusMinisMap" and looking for it in "bonusMinis" map
     * @param date
     * @returns {*}
     */
    function getBonusMiniPuzzleForDate(date) {
        loggerObj.debugLog("fixedMinisLog: In getBonusMiniPuzzleForDate: date -> ", date);
        var bonusMiniPuzzleId = bonusMinisMap[date];
        loggerObj.debugLog("fixedMinisLog: In getBonusMiniPuzzleForDate: bonusMiniPuzzleId -> ", bonusMiniPuzzleId);
        var bonusMiniPuzzle = bonusMinis[bonusMiniPuzzleId];
        // setting d of fixedMiniPuzzle to date value, as this date value gets used while getting the crossword
        bonusMiniPuzzle["d"] = date;
        //
        return bonusMiniPuzzle;
    }

    /**
     * returns bonus mini puzzle id to show for any given date for tracking purposes
     * @param date
     */
    function getBonusMiniIdForTracking(date) {
        return bonusMinisMap[date];
    }

    /**
     * checks if any bonusMini exists for the given date by looking for key value in the "bonusMinisMap"
     * @param date
     * @returns {boolean}
     */
    function checkIfBonusMiniExistsForDate(date) {
        if (bonusMinisMap.hasOwnProperty(date)) {
            return true;
        }
        return false;
    }

    /**
     * returns replacement mini puzzle from "replacementMinis" map
     * @param date
     * @returns {*}
     */
    function getReplacementMiniPuzzleForDate(date) {
        return replacementMinis[date];
    }

    /**
     * returns replacement mini id for tracking
     * @param date
     * @returns {string}
     */
    function getReplacementMiniIdForTracking(date) {
        var dateId = date + "_" + stringsObj.replacement_mini_suffix;
        return dateId;
    }

    /**
     * checks if any replacementMini exists for the given by date by looking for key value in "replacementMinis" map
     * @param date
     * @returns {boolean}
     */
    function checkIfReplacementMiniExistsForDate(date) {
        if (replacementMinis.hasOwnProperty(date)) {
            return true;
        }
        return false;
    }

    /**
     * this function checks if fixed or bonus mini exists for date
     * @param date
     * @returns {boolean}
     */
    this.checkIfFixedOrBonusMiniExistsForDate = function (date) {
        return (checkIfFixedMiniExistsForDate(date) || checkIfBonusMiniExistsForDate(date));
    };

    /**
     * checks and returns fixed or replacement mini puzzle for date
     * @param date
     * @returns {*}
     */
    this.checkAndGetMiniPuzzleForDate = function (date) {
        if (checkIfFixedMiniExistsForDate(date)) {
            loggerObj.debugLog("fixedMinisLog: fetching fixedMini for date -> ", date);
            return getFixedMiniPuzzleForDate(date);
        }

        if (checkIfBonusMiniExistsForDate(date)) {
            loggerObj.debugLog("fixedMinisLog: fetching bonusMini for date -> ", date);
            return getBonusMiniPuzzleForDate(date);
        }

        if (checkIfReplacementMiniExistsForDate(date)) {
            loggerObj.debugLog("fixedMinisLog: fetching replacementMini for date -> ", date);
            return getReplacementMiniPuzzleForDate(date);
        }

        loggerObj.debugLog("fixedMinisLog: no fixedMini exists for date -> ", date);
        return null;
    };

    /**
     * checks and returns fixed mini puzzle ids for tracking
     * @param date
     * @returns {string}
     */
    this.checkAndGetMiniPuzzleIdForTracking = function (date) {
        if (checkIfFixedMiniExistsForDate(date)) {
            return getFixedMiniIdForTracking(date);
        }

        if (checkIfBonusMiniExistsForDate(date)) {
            return getBonusMiniIdForTracking(date);
        }

        if(checkIfReplacementMiniExistsForDate(date)){
            return getReplacementMiniIdForTracking(date);
        }

        return "";
    };
};