MachineLearningData = function () {

    // Same for all Games
    var machineLearningController = null;
    var lastUpdatedIndex = -1;
    var lastUpdatedDate = null;
    var sessionTime = [];
    var coinBalance = [];
    var purchasesMade = [];
    var dollarsSpent = [];
    var inviteClicks = [];
    var daysSinceInstall = 0;
    var userOpenedHelpCenter = [];
    var inGameQuestsClicks = [];
    var inGameQuestsEarns = [];
    var crossGameQuestsClicks = [];
    var crossGameQuestsEarns = [];
    var offerwallClicks = [];
    var offerwallEarns = [];
    var hourOfDay = [];
    var availableMem = [];
    var totalMem = 0;
    var videosWatched = [];
    var videoEarns = [];
    var videoNotAvailable = [];
    var videoTriesCooldown = [];
    var notifClick = [];
    var notifSents = [];
    var interstitialsShown = [];
    var videoClicks = [];
    var interstitialClicks = [];
    var bannerClicks = [];
    var numSessions = [];
    var lastPauseTimeStamp = 0;
    var numAutoPopupsShown = [];
    var lowestFps = [];
    var lastSync = 0;
    var syncInProgress = false;
    var largestLaunchTime = [];

    // Based on Games
    var mainPuzzlesCompleted = [];
    var mainPuzzlesViewed = [];
    var mainPuzzlesReviewed = [];
    var miniPuzzlesCompleted = [];
    var miniPuzzlesViewed = [];
    var miniPuzzlesReviewed = [];
    var packPuzzlesCompleted = [];
    var packPuzzlesViewed = [];
    var packPuzzlesReviewed = [];
    var dailyPuzzleSolveTime = [];
    var dailyMiniSolveTime = [];
    var seeWrongHint = [];
    var showWordHint = [];
    var revealLetters = [];
    var userOpenedAchievements = [];



    // Init with data and controller
    this.init = function () {
        load();
        if (utilObj.isNative()) {
            machineLearningController = new Controller(constantsObj.LOGIN_SERVER, constantsObj.LOGIN_SERVER_REST);
        } else {
            machineLearningController = new Controller(constantsObj.LOGIN_SERVER_SECURE, constantsObj.LOGIN_SERVER_REST);
        }
    };

    // Updating the values of the ML days and incrementing indexes of the diff days
    this.updateIndexes = function () {
        if (lastUpdatedIndex == -1 || lastUpdatedDate == null) {
            machineLearningDataObj.setArrayIndexes(0);
            lastUpdatedDate = (new Date()).toDateString();
        } else {
            var todaysDate = new Date();
            var timeDiff = Math.abs(todaysDate.getTime() - new Date(lastUpdatedDate).getTime());
            var diffDays = Math.floor(timeDiff / (1000 * 3600 * 24));
            if (diffDays > 0) {
                if (diffDays > constantsObj.ML_DAYS) {
                    diffDays = constantsObj.ML_DAYS;
                }
                var lastIndex = lastUpdatedIndex;
                var roundCheck = false;
                if ((lastIndex + diffDays) % constantsObj.ML_DAYS <= lastIndex % constantsObj.ML_DAYS && lastIndex != constantsObj.ML_DAYS - 1) {
                    roundCheck = true;
                }
                for (var i = (lastUpdatedIndex + 1) % constantsObj.ML_DAYS; ; i = (i + 1) % constantsObj.ML_DAYS) {
                    machineLearningDataObj.setArrayIndexes(i);
                    if (!roundCheck) {
                        if (i >= (lastIndex + diffDays) % constantsObj.ML_DAYS) {
                            break;
                        }
                    }
                    if (i == constantsObj.ML_DAYS - 1) {
                        roundCheck = false;
                    }
                }
            }
            lastUpdatedDate = todaysDate.toDateString();
        }
    };

    // setting Array indexes
    this.setArrayIndexes = function (index) {
        // Same for all Games
        lastUpdatedIndex = index;
        sessionTime[index] = 0;
        coinBalance[index] = 0;
        purchasesMade[index] = 0;
        dollarsSpent[index] = 0;
        inviteClicks[index] = 0;
        userOpenedHelpCenter[index] = 0;
        inGameQuestsClicks[index] = 0;
        inGameQuestsEarns[index] = 0;
        crossGameQuestsClicks[index] = 0;
        crossGameQuestsEarns[index] = 0;
        offerwallClicks[index] = 0;
        offerwallEarns[index] = 0;
        hourOfDay[index] = 0;
        availableMem[index] = 0;
        videosWatched[index] = 0;
        videoEarns[index] = 0;
        videoNotAvailable[index] = 0;
        videoTriesCooldown[index] = 0;
        notifClick[index] = 0;
        notifSents[index] = 0;
        interstitialsShown[index] = 0;
        videoClicks[index] = 0;
        interstitialClicks[index] = 0;
        bannerClicks[index] = 0;
        numSessions[index] = 0;
        numAutoPopupsShown[index] = 0;
        lowestFps[index] = 0;
        largestLaunchTime[index] = 0;

        // Based on Games
        mainPuzzlesCompleted[index] = 0;
        mainPuzzlesViewed[index] = 0;
        mainPuzzlesReviewed[index] = 0;
        miniPuzzlesCompleted[index] = 0;
        miniPuzzlesViewed[index] = 0;
        miniPuzzlesReviewed[index] = 0
        packPuzzlesCompleted[index] = 0;
        packPuzzlesViewed[index] = 0;
        packPuzzlesReviewed[index] = 0;
        dailyPuzzleSolveTime[index] = 0;
        dailyMiniSolveTime[index] = 0;
        seeWrongHint[index] = 0;
        showWordHint[index] = 0;
        revealLetters[index] = 0;
        userOpenedAchievements[index] = 0;

    };

    // loading data from file
    function load() {
        var machineLearningJson = ls.getItem(constantsObj.MACHINE_LEARNING_DATA_FILE);
        if (machineLearningJson == null || machineLearningJson == "") {
            return false;
        }
        machineLearningJson = JSON.parse(machineLearningJson);

        // Same for all Games
        lastUpdatedIndex = machineLearningJson.lastUpdatedIndex;
        lastUpdatedDate = machineLearningJson.lastUpdatedDate;
        sessionTime = machineLearningJson.sessionTime;
        coinBalance = machineLearningJson.coinBalance;
        purchasesMade = machineLearningJson.purchasesMade;
        dollarsSpent = machineLearningJson.dollarsSpent;
        inviteClicks = machineLearningJson.inviteClicks;
        daysSinceInstall = machineLearningJson.daysSinceInstall;
        inGameQuestsClicks = machineLearningJson.inGameQuestsClicks;
        inGameQuestsEarns = machineLearningJson.inGameQuestsEarns;
        crossGameQuestsClicks = machineLearningJson.crossGameQuestsClicks;
        crossGameQuestsEarns = machineLearningJson.crossGameQuestsEarns;
        offerwallClicks = machineLearningJson.offerwallClicks;
        offerwallEarns = machineLearningJson.offerwallEarns;
        hourOfDay = machineLearningJson.hourOfDay;
        availableMem = machineLearningJson.availableMem;
        videosWatched = machineLearningJson.videosWatched;
        videoEarns = machineLearningJson.videoEarns;
        videoNotAvailable = machineLearningJson.videoNotAvailable;
        videoTriesCooldown = machineLearningJson.videoTriesCooldown;
        notifClick = machineLearningJson.notifClick;
        notifSents = machineLearningJson.notifSents;
        interstitialsShown = machineLearningJson.interstitialsShown;
        videoClicks = machineLearningJson.videoClicks;
        interstitialClicks = machineLearningJson.interstitialClicks;
        bannerClicks = machineLearningJson.bannerClicks;
        numSessions = machineLearningJson.numSessions;
        numAutoPopupsShown = machineLearningJson.numAutoPopupsShown;
        lowestFps = machineLearningJson.lowestFps;
        largestLaunchTime = machineLearningJson.largestLaunchTime;
        lastPauseTimeStamp = machineLearningJson.lastPauseTimeStamp;
        totalMem = machineLearningJson.totalMem;
        syncInProgress = machineLearningJson.syncInProgress;
        lastSync = machineLearningJson.lastSync;

        // Based on Games
        mainPuzzlesCompleted = machineLearningJson.mainPuzzlesCompleted;
        mainPuzzlesViewed = machineLearningJson.mainPuzzlesViewed;
        mainPuzzlesReviewed = machineLearningJson.mainPuzzlesReviewed;
        miniPuzzlesCompleted = machineLearningJson.miniPuzzlesCompleted;
        miniPuzzlesViewed = machineLearningJson.miniPuzzlesViewed;
        miniPuzzlesReviewed = machineLearningJson.miniPuzzlesReviewed;
        packPuzzlesCompleted = machineLearningJson.packPuzzlesCompleted;
        packPuzzlesViewed = machineLearningJson.packPuzzlesViewed;
        packPuzzlesReviewed = machineLearningJson.packPuzzlesReviewed;
        dailyPuzzleSolveTime = machineLearningJson.dailyPuzzleSolveTime;
        dailyMiniSolveTime = machineLearningJson.dailyMiniSolveTime;
        seeWrongHint = machineLearningJson.seeWrongHint;
        showWordHint = machineLearningJson.showWordHint;
        revealLetters = machineLearningJson.revealLetters;
        userOpenedHelpCenter = machineLearningJson.userOpenedHelpCenter;
        userOpenedAchievements = machineLearningJson.userOpenedAchievements;

        return true;
    }

    // saving in the file
    this.save = function () {
        var machineLearningJson = {};

        // Same for all Games
        machineLearningJson.lastUpdatedIndex = lastUpdatedIndex;
        machineLearningJson.lastUpdatedDate = lastUpdatedDate;
        machineLearningJson.sessionTime = sessionTime;
        machineLearningJson.coinBalance = coinBalance;
        machineLearningJson.purchasesMade = purchasesMade;
        machineLearningJson.dollarsSpent = dollarsSpent;
        machineLearningJson.daysSinceInstall = daysSinceInstall;
        machineLearningJson.inviteClicks = inviteClicks;
        machineLearningJson.userOpenedHelpCenter = userOpenedHelpCenter;
        machineLearningJson.inGameQuestsClicks = inGameQuestsClicks;
        machineLearningJson.inGameQuestsEarns = inGameQuestsEarns;
        machineLearningJson.crossGameQuestsClicks = crossGameQuestsClicks;
        machineLearningJson.crossGameQuestsEarns = crossGameQuestsEarns;
        machineLearningJson.offerwallClicks = offerwallClicks;
        machineLearningJson.offerwallEarns = offerwallEarns;
        machineLearningJson.hourOfDay = hourOfDay;
        machineLearningJson.availableMem = availableMem;
        machineLearningJson.videosWatched = videosWatched;
        machineLearningJson.videoEarns = videoEarns;
        machineLearningJson.videoNotAvailable = videoNotAvailable;
        machineLearningJson.videoTriesCooldown = videoTriesCooldown;
        machineLearningJson.notifClick = notifClick;
        machineLearningJson.notifSents = notifSents;
        machineLearningJson.interstitialsShown = interstitialsShown;
        machineLearningJson.videoClicks = videoClicks;
        machineLearningJson.interstitialClicks = interstitialClicks;
        machineLearningJson.bannerClicks = bannerClicks;
        machineLearningJson.numSessions = numSessions;
        machineLearningJson.numAutoPopupsShown = numAutoPopupsShown;
        machineLearningJson.lowestFps = lowestFps;
        machineLearningJson.largestLaunchTime = largestLaunchTime;
        machineLearningJson.lastPauseTimeStamp = lastPauseTimeStamp;
        machineLearningJson.totalMem = totalMem;
        machineLearningJson.syncInProgress = syncInProgress;
        machineLearningJson.lastSync = lastSync;

        // Based on Games
        machineLearningJson.mainPuzzlesCompleted = mainPuzzlesCompleted;
        machineLearningJson.mainPuzzlesViewed = mainPuzzlesViewed;
        machineLearningJson.mainPuzzlesReviewed = mainPuzzlesReviewed;
        machineLearningJson.miniPuzzlesCompleted = miniPuzzlesCompleted;
        machineLearningJson.miniPuzzlesViewed = miniPuzzlesViewed;
        machineLearningJson.miniPuzzlesReviewed = miniPuzzlesReviewed;
        machineLearningJson.packPuzzlesCompleted = packPuzzlesCompleted;
        machineLearningJson.packPuzzlesViewed = packPuzzlesViewed;
        machineLearningJson.packPuzzlesReviewed = packPuzzlesReviewed;
        machineLearningJson.dailyPuzzleSolveTime = dailyPuzzleSolveTime;
        machineLearningJson.dailyMiniSolveTime = dailyMiniSolveTime;
        machineLearningJson.seeWrongHint = seeWrongHint;
        machineLearningJson.showWordHint = showWordHint;
        machineLearningJson.revealLetters = revealLetters;
        machineLearningJson.userOpenedAchievements = userOpenedAchievements;

        ls.setItem(constantsObj.MACHINE_LEARNING_DATA_FILE, utilObj.getStringifiedJson(machineLearningJson));
        return true;
    };

    // syncing data to server once in a day
    this.syncToServer = function () {
        return; // Disabling this as it's not been used for a long time.
        if (lastUpdatedIndex == -1) {
            return;
        }
        var mlJson = {};
        mlJson.gameId = constantsObj.GAME_ID + "";
        mlJson.env = constantsObj.ENV + "";
        mlJson.data = machineLearningDataObj.getMLData();
        syncInProgress = true;
        lastSync = utilObj.getCurrentTimestamp();
        machineLearningController.sendHttpRequest("mlData", "save", mlJson);
    };

    this.respHandler = function (action, data, success) {
        if (action === "save") {
            if (success == true) {
                syncInProgress = false;
            }
        }
    };

    var separator = "^";

    // appending the mlData into single string with separator "^" to make it generic for all Games.
    this.getMLData = function () {
        var mlData = "";

        // Same for all Games
        mlData += userObj.getRefId() + separator;
        mlData += constantsObj.ML_VERSION + separator;
        mlData += constantsObj.GAME_ID + separator;
        mlData += getOrderedArrayOldest(sessionTime);
        mlData += getOrderedArrayOldest(coinBalance);
        mlData += getOrderedArrayOldest(purchasesMade);
        mlData += getOrderedArrayOldest(dollarsSpent);
        mlData += getOrderedArrayOldest(inviteClicks);
        mlData += daysSinceInstall + separator;
        mlData += totalMem + separator;
        mlData += getOrderedArrayOldest(userOpenedHelpCenter);
        mlData += getOrderedArrayOldest(inGameQuestsClicks);
        mlData += getOrderedArrayOldest(inGameQuestsEarns);
        mlData += getOrderedArrayOldest(crossGameQuestsClicks);
        mlData += getOrderedArrayOldest(crossGameQuestsEarns);
        mlData += getOrderedArrayOldest(offerwallClicks);
        mlData += getOrderedArrayOldest(offerwallEarns);
        mlData += getOrderedArrayOldest(hourOfDay);
        mlData += getOrderedArrayOldest(availableMem);
        mlData += getOrderedArrayOldest(videosWatched);
        mlData += getOrderedArrayOldest(videoEarns);
        mlData += getOrderedArrayOldest(videoNotAvailable);
        mlData += getOrderedArrayOldest(videoTriesCooldown);
        mlData += getOrderedArrayOldest(notifClick);
        mlData += getOrderedArrayOldest(notifSents);
        mlData += getOrderedArrayOldest(interstitialsShown);
        mlData += getOrderedArrayOldest(videoClicks);
        mlData += getOrderedArrayOldest(interstitialClicks);
        mlData += getOrderedArrayOldest(bannerClicks);
        mlData += getOrderedArrayOldest(numSessions);
        mlData += getOrderedArrayOldest(numAutoPopupsShown);
        mlData += getOrderedArrayOldest(lowestFps);
        mlData += getOrderedArrayOldest(largestLaunchTime);
        mlData += userObj.getCountryCode() + separator;
        mlData += userObj.getSubDivision() + separator;
        mlData += userObj.getCity() + separator;
        mlData += userObj.getInstallnetwork() + separator;
        mlData += userObj.getInstallCampaign() + separator;
        mlData += userObj.getInstallAdGroup() + separator;
        mlData += userObj.getInstallCreative() + separator;
        mlData += utilObj.getTrackingClient() + separator;
        mlData += utilObj.getDeviceModel() + separator;

        // Based on Games
        mlData += getOrderedArrayOldest(mainPuzzlesCompleted);
        mlData += getOrderedArrayOldest(mainPuzzlesViewed);
        mlData += getOrderedArrayOldest(mainPuzzlesReviewed);
        mlData += getOrderedArrayOldest(miniPuzzlesCompleted);
        mlData += getOrderedArrayOldest(miniPuzzlesViewed);
        mlData += getOrderedArrayOldest(miniPuzzlesReviewed);
        mlData += getOrderedArrayOldest(packPuzzlesCompleted);
        mlData += getOrderedArrayOldest(packPuzzlesViewed);
        mlData += getOrderedArrayOldest(packPuzzlesReviewed);
        mlData += getOrderedArrayOldest(dailyPuzzleSolveTime);
        mlData += getOrderedArrayOldest(dailyMiniSolveTime);
        mlData += getOrderedArrayOldest(seeWrongHint);
        mlData += getOrderedArrayOldest(showWordHint);
        mlData += getOrderedArrayOldest(revealLetters);
        mlData += getOrderedArrayOldest(userOpenedAchievements);

        mlData += syncReqRespCheckObj.getLastReqStartTimeStamp() + "";
        return mlData;
    };

    function getOrderedArrayOldest(arr) {
        var list = (arr.slice((lastUpdatedIndex + 1), arr.length)).length > 0 && (arr.slice(0, lastUpdatedIndex + 1)).length > 0 ? arr.slice((lastUpdatedIndex + 1), arr.length).join(':') + ":" + arr.slice(0, lastUpdatedIndex + 1).join(':') : arr.slice((lastUpdatedIndex + 1), arr.length).join(':') + arr.slice(0, lastUpdatedIndex + 1).join(':');
        return list + separator;
    }


    // Same for all Games
    this.getSyncInProgress = function () {
        return syncInProgress;
    };

    this.getLastSync = function () {
        return lastSync;
    };

    this.incSessionTime = function (val) {
        if (sessionTime[lastUpdatedIndex]) {
            sessionTime[lastUpdatedIndex] += val;
        } else {
            sessionTime[lastUpdatedIndex] = val;
        }
    };

    this.incCoinBalance = function (val) {
        coinBalance[lastUpdatedIndex] = val;
    };

    this.incPurchasesMade = function () {
        if (purchasesMade[lastUpdatedIndex]) {
            purchasesMade[lastUpdatedIndex] += 1;
        } else {
            purchasesMade[lastUpdatedIndex] = 1;
        }
    };

    this.incDollarsSpent = function (val) {
        var num = Math.round(parseFloat(val) * 100)/100;
        if (dollarsSpent[lastUpdatedIndex]) {
            dollarsSpent[lastUpdatedIndex] += num;
        } else {
            dollarsSpent[lastUpdatedIndex] = num;
        }
    };

    this.incInviteClicks = function () {
        if (inviteClicks[lastUpdatedIndex]) {
            inviteClicks[lastUpdatedIndex] += 1;
        } else {
            inviteClicks[lastUpdatedIndex] = 1;
        }
    };

    this.incDaysSinceInstall = function () {
        daysSinceInstall += 1;
    };

    this.setTotalMem = function (val) {
        if (utilObj.isAndroid()) {
            totalMem = parseInt(val);
        } else if (utilObj.isIos()) {
            var jsonData = JSON.parse(val);
            totalMem = parseInt(jsonData.mem);
        }
    };

    this.getTotalMem = function () {
        return totalMem;
    };

    this.incUserOpenedHelpCenter = function () {
        if (userOpenedHelpCenter[lastUpdatedIndex]) {
            userOpenedHelpCenter[lastUpdatedIndex] += 1;
        } else {
            userOpenedHelpCenter[lastUpdatedIndex] = 1;
        }
    };

    this.incInGameQuestsClicks = function () {
        if (inGameQuestsClicks[lastUpdatedIndex]) {
            inGameQuestsClicks[lastUpdatedIndex] += 1;
        } else {
            inGameQuestsClicks[lastUpdatedIndex] = 1;
        }
    };

    this.incInGameQuestsEarns = function () {
        if (inGameQuestsEarns[lastUpdatedIndex]) {
            inGameQuestsEarns[lastUpdatedIndex] += 1;
        } else {
            inGameQuestsEarns[lastUpdatedIndex] = 1;
        }
    };

    this.incCrossGameQuestsClicks = function () {
        if (crossGameQuestsClicks[lastUpdatedIndex]) {
            crossGameQuestsClicks[lastUpdatedIndex] += 1;
        } else {
            crossGameQuestsClicks[lastUpdatedIndex] = 1;
        }

    };

    this.incCrossGameQuestsEarns = function () {
        if (crossGameQuestsEarns[lastUpdatedIndex]) {
            crossGameQuestsEarns[lastUpdatedIndex] += 1;
        } else {
            crossGameQuestsEarns[lastUpdatedIndex] = 1;
        }

    };

    this.incOfferwallClicks = function () {
        if (offerwallClicks[lastUpdatedIndex]) {
            offerwallClicks[lastUpdatedIndex] += 1;
        } else {
            offerwallClicks[lastUpdatedIndex] = 1;
        }

    };

    this.setVideosWatched = function () {
        if (videosWatched[lastUpdatedIndex]) {
            videosWatched[lastUpdatedIndex] += 1;
        } else {
            videosWatched[lastUpdatedIndex] = 1;
        }
    };

    this.setVideoEarns = function () {
        if (videoEarns[lastUpdatedIndex]) {
            videoEarns[lastUpdatedIndex] += 1;
        } else {
            videoEarns[lastUpdatedIndex] = 1;
        }

    };

    this.setVideoNotAvailable = function () {
        if (videoNotAvailable[lastUpdatedIndex]) {
            videoNotAvailable[lastUpdatedIndex] += 1;
        } else {
            videoNotAvailable[lastUpdatedIndex] = 1;
        }
    };

    this.setVideoTriesCooldown = function () {
        if (videoTriesCooldown[lastUpdatedIndex]) {
            videoTriesCooldown[lastUpdatedIndex] += 1;
        } else {
            videoTriesCooldown[lastUpdatedIndex] = 1;
        }
    };

    this.setNotifClick = function () {
        if (notifClick[lastUpdatedIndex]) {
            notifClick[lastUpdatedIndex] += 1;
        } else {
            notifClick[lastUpdatedIndex] = 1;
        }
    };

    this.setNotifSents = function () {
        if (notifSents[lastUpdatedIndex]) {
            notifSents[lastUpdatedIndex] += 1;
        } else {
            notifSents[lastUpdatedIndex] = 1;
        }

    };

    this.setInterstitialsShown = function () {
        if (interstitialsShown[lastUpdatedIndex]) {
            interstitialsShown[lastUpdatedIndex] += 1;
        } else {
            interstitialsShown[lastUpdatedIndex] = 1;
        }

    };

    this.setVideoClicks = function () {
        if (videoClicks[lastUpdatedIndex]) {
            videoClicks[lastUpdatedIndex] += 1;
        } else {
            videoClicks[lastUpdatedIndex] = 1;
        }

    };

    this.setInterstitialClicks = function () {
        if (interstitialClicks[lastUpdatedIndex]) {
            interstitialClicks[lastUpdatedIndex] += 1;
        } else {
            interstitialClicks[lastUpdatedIndex] = 1;
        }

    };

    this.setBannerClicks = function () {
        if (bannerClicks[lastUpdatedIndex]) {
            bannerClicks[lastUpdatedIndex] += 1;
        } else {
            bannerClicks[lastUpdatedIndex] = 1;
        }
    };

    this.setLastAppPauseTimeStamp = function (val) {
        lastPauseTimeStamp = val;
    };

    /*
        for usage of the adjust new session event
     */
    this.getLastAppPauseTimeStamp = function () {
        return lastPauseTimeStamp;
    };

    this.incNumSessions = function () {
        if (utilObj.getCurrentTimestamp() - lastPauseTimeStamp >= 10 * constantsObj.SECONDS_IN_MIN) {
            if (numSessions[lastUpdatedIndex]) {
                numSessions[lastUpdatedIndex] += 1;
            } else {
                numSessions[lastUpdatedIndex] = 1;
            }

        }
    };

    this.incNumAutoPopupsShown = function () {
        if (numAutoPopupsShown[lastUpdatedIndex]) {
            numAutoPopupsShown[lastUpdatedIndex] += 1;
        } else {
            numAutoPopupsShown[lastUpdatedIndex] = 1;
        }

    };

    this.setLowestFps = function (val) {
        if (!lowestFps[lastUpdatedIndex] || lowestFps[lastUpdatedIndex] == 0 || lowestFps[lastUpdatedIndex] > val) {
            lowestFps[lastUpdatedIndex] = val;
        }
    };

    this.setLargestLaunchTime = function (val) {
        if (!largestLaunchTime[lastUpdatedIndex] || largestLaunchTime[lastUpdatedIndex] == 0 || largestLaunchTime[lastUpdatedIndex] < val) {
            largestLaunchTime[lastUpdatedIndex] = val;
        }
    };

    this.incOfferwallEarns = function (val) {
        if (offerwallEarns[lastUpdatedIndex]) {
            offerwallEarns[lastUpdatedIndex] += val;
        } else {
            offerwallEarns[lastUpdatedIndex] = val;
        }
    };

    this.setHourOfDay = function (val) {
        hourOfDay[lastUpdatedIndex] = val;
    };

    this.setAvailableMem = function (val) {
        if (utilObj.isAndroid()) {
            if (!availableMem[lastUpdatedIndex] || availableMem[lastUpdatedIndex] > parseInt(val) || availableMem[lastUpdatedIndex] == 0) {
                availableMem[lastUpdatedIndex] = parseInt(val);
            }
        } else if (utilObj.isIos()) {
            var jsonData = JSON.parse(val);
            if (!availableMem[lastUpdatedIndex] || availableMem[lastUpdatedIndex] > parseInt(jsonData.mem) || availableMem[lastUpdatedIndex] == 0) {
                availableMem[lastUpdatedIndex] = parseInt(jsonData.mem);
            }
        }
    };

    // ----------------------------


    // Based on Games
    this.incMainPuzzlesCompleted = function () {
        if (mainPuzzlesCompleted[lastUpdatedIndex]) {
            mainPuzzlesCompleted[lastUpdatedIndex] += 1;
        } else {
            mainPuzzlesCompleted[lastUpdatedIndex] = 1;
        }
    };

    this.incMainPuzzlesViewed = function () {
        if (mainPuzzlesViewed[lastUpdatedIndex]) {
            mainPuzzlesViewed[lastUpdatedIndex] += 1;
        } else {
            mainPuzzlesViewed[lastUpdatedIndex] = 1;
        }

    };

    this.incMainPuzzlesReviewed = function () {
        if (mainPuzzlesReviewed[lastUpdatedIndex]) {
            mainPuzzlesReviewed[lastUpdatedIndex] += 1;
        } else {
            mainPuzzlesReviewed[lastUpdatedIndex] = 1;
        }
        if (mainPuzzlesViewed[lastUpdatedIndex]) {
            mainPuzzlesViewed[lastUpdatedIndex] -= 1;
        }
    };

    // updating miniPuzzles Solved
    this.incMiniPuzzlesCompleted = function () {
        if (miniPuzzlesCompleted[lastUpdatedIndex]) {
            miniPuzzlesCompleted[lastUpdatedIndex] += 1;
        } else {
            miniPuzzlesCompleted[lastUpdatedIndex] = 1;
        }

    };

    this.incMiniPuzzlesViewed = function () {
        if (miniPuzzlesViewed[lastUpdatedIndex]) {
            miniPuzzlesViewed[lastUpdatedIndex] += 1;
        } else {
            miniPuzzlesViewed[lastUpdatedIndex] = 1;
        }

    };

    this.incMiniPuzzlesReviewed = function () {
        if (miniPuzzlesReviewed[lastUpdatedIndex]) {
            miniPuzzlesReviewed[lastUpdatedIndex] += 1;
        } else {
            miniPuzzlesReviewed[lastUpdatedIndex] = 1;
        }
        if (miniPuzzlesViewed[lastUpdatedIndex]) {
            miniPuzzlesViewed[lastUpdatedIndex] -= 1;
        }
    };

    this.incPackPuzzlesCompleted = function () {
        if (packPuzzlesCompleted[lastUpdatedIndex]) {
            packPuzzlesCompleted[lastUpdatedIndex] += 1;
        } else {
            packPuzzlesCompleted[lastUpdatedIndex] = 1;
        }

    };

    this.incPackPuzzlesViewed = function () {
        if (packPuzzlesViewed[lastUpdatedIndex]) {
            packPuzzlesViewed[lastUpdatedIndex] += 1;
        } else {
            packPuzzlesViewed[lastUpdatedIndex] = 1;
        }

    };

    this.incPackPuzzlesReviewed = function () {
        if (packPuzzlesReviewed[lastUpdatedIndex]) {
            packPuzzlesReviewed[lastUpdatedIndex] += 1;
        } else {
            packPuzzlesReviewed[lastUpdatedIndex] = 1;
        }
        if (packPuzzlesViewed[lastUpdatedIndex]) {
            packPuzzlesViewed[lastUpdatedIndex] -= 1;
        }
    };

    this.incDailyPuzzleSolveTime = function (val) {
        dailyPuzzleSolveTime[lastUpdatedIndex] = val;
    };

    this.incDailyMiniSolveTime = function (val) {
        dailyMiniSolveTime[lastUpdatedIndex] = val;
    };

    this.incSeeWrongHint = function () {
        if (seeWrongHint[lastUpdatedIndex]) {
            seeWrongHint[lastUpdatedIndex] += 1;
        } else {
            seeWrongHint[lastUpdatedIndex] = 1;
        }
    };

    this.incShowWordHint = function () {
        if (showWordHint[lastUpdatedIndex]) {
            showWordHint[lastUpdatedIndex] += 1;
        } else {
            showWordHint[lastUpdatedIndex] = 1;
        }
    };

    this.incRevealLetters = function () {
        if (revealLetters[lastUpdatedIndex]) {
            revealLetters[lastUpdatedIndex] += 1;
        } else {
            revealLetters[lastUpdatedIndex] = 1;
        }
    };


    this.incUserOpenedAchievements = function () {
        if (userOpenedAchievements[lastUpdatedIndex]) {
            userOpenedAchievements[lastUpdatedIndex] += 1;
        } else {
            userOpenedAchievements[lastUpdatedIndex] = 1;
        }
    };

    // ---------------------------


};