/*
 controls the quest map layer for wq and tq.
 */

QuestMap = function () {

    /*
     stores the count of puzzles solved in a week
     to be reset on the start of a new quest
     */
    var BIG_PUZZLE = "big";
    var SMALL_PUZZLE = "small";
    var puzzlesPlayedLastWeek = {};
    var collectibles = [];

    // init with the default values
    puzzlesPlayedLastWeek[SMALL_PUZZLE] = 0;
    puzzlesPlayedLastWeek[BIG_PUZZLE] = 0;

    // variable to control auto on logic of questMap layer
    var questMapAutoOn = false;
    var extraDelayForSurfacingTodaysChallengeSection = 0.1;
    //

    // returns the val for the puzzles played last week
    this.getPuzzlesPlayedLastWeek = function () {
        return puzzlesPlayedLastWeek;
    };

    /* sets the val for the puzzles played last week
     1. from gama data load
     2. resetting it on new quest
     */
    this.setPuzzlesPlayedLastWeek = function (data) {
        puzzlesPlayedLastWeek[BIG_PUZZLE] = data[BIG_PUZZLE];
        puzzlesPlayedLastWeek[SMALL_PUZZLE] = data[SMALL_PUZZLE];
    };

    /*
     increments the value of the puzzles completed after completion
     */
    this.incrementPuzzlesPlayedLastWeek = function (type) {
        if (type == BIG_PUZZLE) {
            puzzlesPlayedLastWeek[BIG_PUZZLE] += 1;
        } else {
            puzzlesPlayedLastWeek[SMALL_PUZZLE] += 1;
        }
    };

    // -----------------

    // this is called whenever a new quest is started to do all the settings
    this.setProgressOnStartOfNewQuest = function () {
        // resets the bucket of the user
        questMapObj.calculateBucketForTheUser();

        // size distribution for the quest
        questMapObj.setsSizeDistribution();

        // clear data to store for the next quest
        questMapObj.setPuzzlesPlayedLastWeek({"big": 0, "small": 0});
    };
    // ------------------

    // bucket logic for the user
    var BUCKET_1 = 1, BUCKET_2 = 2, BUCKET_3 = 3, BUCKET_4 = 4;
    var bucketAllocated = BUCKET_1;

    // returns the current bucket value for the user
    this.getBucketAllocated = function () {
        return bucketAllocated;
    };

    /* sets the value of the bucket
     1. game data load and save
     2. on new quest start to recalc the bucket of the user.
     */
    this.setBucketAllocated = function (val) {
        bucketAllocated = val;
    };

    /* calculate the bucket of the user on new quest start
     1. called from the quest start point so that it sets the val of the bucket for the user at the start
     */
    this.calculateBucketForTheUser = function () {
        var bigPuzzlesSolved = puzzlesPlayedLastWeek[BIG_PUZZLE];
        var smallPuzzlesSolved = puzzlesPlayedLastWeek[SMALL_PUZZLE];

        // runtime value for the bucket split
        var runtimeValue = runtimeObj.getEnggBucket();
        var splitStr = runtimeValue.split("_");
        var bigLimit = parseInt(splitStr[0]);
        var smallLimit = parseInt(splitStr[1]);

        // assign bucket based on the data
        var bucketToAllot = BUCKET_1;
        if (bigPuzzlesSolved <= bigLimit && smallPuzzlesSolved > smallLimit) {
            bucketToAllot = BUCKET_2;
        } else if (bigPuzzlesSolved > bigLimit && smallPuzzlesSolved <= smallLimit) {
            bucketToAllot = BUCKET_3;
        } else if (bigPuzzlesSolved > bigLimit && smallPuzzlesSolved > smallLimit) {
            bucketToAllot = BUCKET_4;
        }

        // setting the bucket for the user
        questMapObj.setBucketAllocated(bucketToAllot);

        loggerObj.debugLog("quest info bucket is", bucketToAllot);

    };

    // -----------------

    /*
     decides the puzzle split based on the bucket and sets the curve
     */

    var totalPuzzles = 30;
    var sizeDistributionCurve = [];
    var milestoneInfo = {};

    var MILESTONE_1 = 1;
    var MILESTONE_2 = 2;
    var MILESTONE_3 = 3;
    var SIZE = "size";
    var DIFFICULTY = "diff";

    var flag = false;

    // initialize the curve with alternate puzzles
    for (var i in totalPuzzles) {
        if (flag) {
            sizeDistributionCurve.push(BIG_PUZZLE);
            flag = false;
        } else {
            sizeDistributionCurve.push(SMALL_PUZZLE);
            flag = true;
        }
    }

    // initialize milestone info curve 
    milestoneInfo[MILESTONE_1] = {};
    milestoneInfo[MILESTONE_1][SIZE] = {"big": 0, "small": 2};
    milestoneInfo[MILESTONE_1][DIFFICULTY] = {"easy": 2, "medium": 0, "hard": 0};

    milestoneInfo[MILESTONE_2] = {};
    milestoneInfo[MILESTONE_2][SIZE] = {"big": 3, "small": 6};
    milestoneInfo[MILESTONE_2][DIFFICULTY] = {"easy": 6, "medium": 3, "hard": 0};

    milestoneInfo[MILESTONE_3] = {};
    milestoneInfo[MILESTONE_3][SIZE] = {"big": 6, "small": 13};
    milestoneInfo[MILESTONE_3][DIFFICULTY] = {"easy": 12, "medium": 5, "hard": 2};

    // getters and setters for the size distribution
    this.getSizeDistributionArr = function () {
        return sizeDistributionCurve;
    };

    this.setsSizeDistributionArr = function (val) {
        sizeDistributionCurve = val;
    };

    // getters and setters for the milestone info map
    this.getMilestoneInfo = function () {
        return milestoneInfo;
    };

    this.setMilestoneInfo = function (val) {
        milestoneInfo = val;
    };


    // getters and setters for totalpuzzles info
    this.getTotalPuzzles = function () {
        return totalPuzzles;
    };

    this.setTotalPuzzles = function (val) {
        totalPuzzles = val;
    };


    // initialize with the size distribution for the quest
    this.setsSizeDistribution = function () {
        var puzzle_split = runtimeObj.getPuzzleSplit();
        var splitStrArr = puzzle_split.split("_");

        var splitVal;
        if (bucketAllocated == BUCKET_1) {
            splitVal = 0;
        } else if (bucketAllocated == BUCKET_2) {
            splitVal = 15;
        } else if (bucketAllocated == BUCKET_3) {
            splitVal = 30;
        } else {
            splitVal = 45;
        }

        // splitting arr for the current bucket
        var arrForSplit = splitStrArr.slice(splitVal, splitVal + 15);

        milestoneInfo[MILESTONE_1] = {};
        milestoneInfo[MILESTONE_1][SIZE] = {"big": arrForSplit[0], "small": arrForSplit[1]};
        milestoneInfo[MILESTONE_1][DIFFICULTY] = {
            "easy": arrForSplit[2],
            "medium": arrForSplit[3],
            "hard": arrForSplit[4]
        };

        milestoneInfo[MILESTONE_2] = {};
        milestoneInfo[MILESTONE_2][SIZE] = {"big": arrForSplit[5], "small": arrForSplit[6]};
        milestoneInfo[MILESTONE_2][DIFFICULTY] = {
            "easy": arrForSplit[7],
            "medium": arrForSplit[8],
            "hard": arrForSplit[9]
        };

        milestoneInfo[MILESTONE_3] = {};
        milestoneInfo[MILESTONE_3][SIZE] = {"big": arrForSplit[10], "small": arrForSplit[11]};
        milestoneInfo[MILESTONE_3][DIFFICULTY] = {
            "easy": arrForSplit[12],
            "medium": arrForSplit[13],
            "hard": arrForSplit[14]
        };

        // calculate total number of puzzles for quest
        totalPuzzles = 0;
        for (var i = MILESTONE_1; i <= MILESTONE_3; i++) {
            totalPuzzles += parseInt(milestoneInfo[i][SIZE]["big"]);
            totalPuzzles += parseInt(milestoneInfo[i][SIZE]["small"]);
        }

        // size distribution assignment
        sizeDistributionCurve = [];
        questMapObj.calcDistributionBasedOnInputSizes(milestoneInfo[MILESTONE_1][SIZE]["big"], milestoneInfo[MILESTONE_1][SIZE]["small"]);
        questMapObj.calcDistributionBasedOnInputSizes(milestoneInfo[MILESTONE_2][SIZE]["big"], milestoneInfo[MILESTONE_2][SIZE]["small"]);
        questMapObj.calcDistributionBasedOnInputSizes(milestoneInfo[MILESTONE_3][SIZE]["big"], milestoneInfo[MILESTONE_3][SIZE]["small"]);

        loggerObj.debugLog("quest info is1", JSON.stringify(milestoneInfo));

    };

    /**
     * selecting size based on logic for curve setting
     * @param x number of big puzzles
     * @param y number of small puzzles
     */
    this.calcDistributionBasedOnInputSizes = function (x, y) {
        var bigPuzzles = parseInt(x);
        var smallPuzzles = parseInt(y);

        // calc floor val for distribution
        var disFloorVal;
        var countLarger;
        if (bigPuzzles > smallPuzzles) {
            disFloorVal = Math.floor(bigPuzzles / smallPuzzles);
            countLarger = BIG_PUZZLE;
        } else {
            disFloorVal = Math.floor(smallPuzzles / bigPuzzles);
            countLarger = SMALL_PUZZLE;
        }

        /* distributes puzzles based on size distribution
         iterating through all puzzles checking for repeat interval and distribution the puzzles
         */
        var totalItr = bigPuzzles + smallPuzzles;
        var repeatInterval = disFloorVal + 1;
        for (var i = 1; i <= totalItr; i++) {
            if (countLarger == BIG_PUZZLE) {
                if (i % repeatInterval == 0 && smallPuzzles > 0) {
                    sizeDistributionCurve.push(SMALL_PUZZLE);
                    smallPuzzles--;
                } else {
                    sizeDistributionCurve.push(BIG_PUZZLE);
                    bigPuzzles--;
                }
            } else {
                if (i % repeatInterval == 0 && bigPuzzles > 0) {
                    sizeDistributionCurve.push(BIG_PUZZLE);
                    bigPuzzles--;
                } else {
                    sizeDistributionCurve.push(SMALL_PUZZLE);
                    smallPuzzles--;
                }
            }
            // loggerObj.debugLog("quest info is3", sizeDistributionCurve);
        }

        // loggerObj.debugLog("quest info is2", sizeDistributionCurve, totalItr);

    };

    // -----------------

    var puzzleWaterFall = {
        "hardbig": [{diff: "hard", size: "big"}, {diff: "medium", size: "big"}, {
            diff: "hard",
            size: "small"
        }, {diff: "medium", size: "small"}, {diff: "easy", size: "small"}],
        "easybig": [{diff: "medium", size: "big"},{diff: "hard", size: "big"}, {
            diff: "hard",
            size: "small"
        }, {diff: "medium", size: "small"}, {diff: "easy", size: "small"}],
        "mediumbig": [{diff: "medium", size: "big"}, {diff: "hard", size: "small"}, {
            diff: "medium",
            size: "small"
        }, {diff: "hard", size: "big"}, {diff: "easy", size: "small"}],
        "hardsmall": [{diff: "hard", size: "small"}, {diff: "medium", size: "big"}, {
            diff: "medium",
            size: "small"
        }, {diff: "easy", size: "small"}, {diff: "hard", size: "big"}],
        "mediumsmall": [{diff: "medium", size: "small"}, {diff: "easy", size: "small"}, {
            diff: "hard",
            size: "small"
        }, {diff: "medium", size: "big"}, {diff: "hard", size: "big"}],
        "easysmall": [{diff: "easy", size: "small"}, {diff: "medium", size: "small"}, {
            diff: "hard",
            size: "small"
        }, {diff: "medium", size: "big"}, {diff: "hard", size: "big"}]
    };

    /**
     * selects the puzzle for the quest as it demands
     * @param index for which the puzzle is required
     */
    this.getPuzzleForIndex = function (index) {

        // edge case handling for the size dist. of puzzles
        if (utilObj.getStringifiedJson(sizeDistributionCurve) == utilObj.getStringifiedJson([])) {
            questMapObj.setProgressOnStartOfNewQuest();
        }

        // gets the puzzle size i should have
        var puzzleSize = sizeDistributionCurve[index - 1];

        // gets the difficulty of the puzzle
        var arrForDifficulty = [];
        for (var i = MILESTONE_1; i <= MILESTONE_3; i++) {
            arrForDifficulty.push(parseInt(milestoneInfo[i][DIFFICULTY]["easy"]));
            arrForDifficulty.push(parseInt(milestoneInfo[i][DIFFICULTY]["medium"]));
            arrForDifficulty.push(parseInt(milestoneInfo[i][DIFFICULTY]["hard"]));
        }

        var index1 = 0;
        var totalPuz = 0;
        for (var i in arrForDifficulty) {
            totalPuz += arrForDifficulty[i];
            if (index - 1 < totalPuz) {
                index1 = i;
                break;
            }
        }

        // decides difficulty
        var puzzDifficulty = "easy";
        if (index1 % 3 == 1) {
            puzzDifficulty = "medium";
        } else if (index1 % 3 == 2) {
            puzzDifficulty = "hard";
        }

        var waterFallString = puzzDifficulty + puzzleSize;
        var arrForFetchingPuzzleID = puzzleWaterFall[waterFallString];

        // fetching puzzle id
        var puzzleId = null;

        // loops around for the puzzle id type
        for (var i in arrForFetchingPuzzleID) {
            var size = arrForFetchingPuzzleID[i].size;
            var diff = arrForFetchingPuzzleID[i].diff;
            puzzleId = questMapObj.getPuzzleForSizeAndDifficulty(size, diff);
            if (puzzleId != null) {
                break;
            }
        }

        // taking puzzle from inProgress
        var isInProgress = false;
        var isCompleted = false;
        if (puzzleId == null) {
            // utilObj.showMessage("trying puzzle from inprogress! alert panwar " + puzzleSize + " diff " + puzzDifficulty);
            for (var i in arrForFetchingPuzzleID) {
                var size = arrForFetchingPuzzleID[i].size;
                var diff = arrForFetchingPuzzleID[i].diff;
                puzzleId = questMapObj.getPuzzleForSizeAndDifficultyInProgress(size, diff);
                if (puzzleId != null) {
                    isInProgress = true;
                    break;
                }
            }

            // taking puzzle from completed array
            if (puzzleId == null) {
                // utilObj.showMessage("trying puzzle from completed! alert panwar instantly " + puzzleSize + " diff " + puzzDifficulty);
                for (var i in arrForFetchingPuzzleID) {
                    var size = arrForFetchingPuzzleID[i].size;
                    var diff = arrForFetchingPuzzleID[i].diff;
                    puzzleId = questMapObj.getPuzzleForSizeAndDifficultyCompleted(size, diff);
                    if (puzzleId != null) {
                        isCompleted = true;
                        break;
                    }
                }
            }

            if (puzzleId == null) {
                // utilObj.showMessage("random puzzle taken! don't alert panwar instantly " + puzzleSize + " diff " + puzzDifficulty);
                return {id: "00000001", type: FREE_PACKS, price: 0}
            }
        }

        if (isInProgress) {
            return {id: puzzleId, type: FREE_SUBS_PUZZLE, price: 0};
        } else if (isCompleted) {
            var isMiniOrMidi = gameObj.checkForMiniOrMidi(puzzleId);
            if (isMiniOrMidi) {
                return {id: puzzleId, type: FREE_SUBS_PUZZLE, price: 0};
            }
            return {id: puzzleId, type: PAID, price: economyObj.getReplayCrosswordCost()};
        }

        // fetching the price and type of the puzzle
        var priceAndType = questMapObj.getPriceForPuzzle(puzzleId, puzzleSize);

        // questMapLayer.lblForDebug.setString("pzid_" + puzzleId  +"_puzzleSize_"  + puzzleSize + "_puzDiff_" + puzzDifficulty + "_bucket_" + bucketAllocated );

        return {id: puzzleId, type: priceAndType.type, price: priceAndType.price};
    };

    // bonus archive variables to control the puzzle surfaced
    var todaysQuestBonusArchivePuzzle = null;
    var numberOfBonusArchiveWatched = 0;

    this.getTodaysQuestBonusArchivePuzzle = function () {
        return todaysQuestBonusArchivePuzzle;
    };

    this.setTodaysQuestBonusArchivePuzzle = function (val) {
        todaysQuestBonusArchivePuzzle = val;
    };

    this.getNumberOfBonusArchiveWatched = function () {
        return numberOfBonusArchiveWatched;
    };

    this.setNumberOfBonusArchiveWatched = function (val) {
        numberOfBonusArchiveWatched = val;
    };

    // bonus mini variables to control the puzzle surfaced
    var todaysQuestBonusMiniPuzzle = null;
    var numberOfBonusMiniWatched = 0;

    this.getTodaysQuestBonusMiniPuzzle = function () {
        return todaysQuestBonusMiniPuzzle;
    };

    this.setTodaysQuestBonusMiniPuzzle = function (val) {
        todaysQuestBonusMiniPuzzle = val;
    };

    this.getNumberOfBonusMiniWatched = function () {
        return numberOfBonusMiniWatched;
    };

    this.setNumberOfBonusMiniWatched = function (val) {
        numberOfBonusMiniWatched = val;
    };

    var FREE_SUBS_PUZZLE = "subsFree";
    var BONUS_ARCHIVE = "bonusArchive";
    var PAID = "paid";
    var FREE_PACKS = "packsFree";
    var BONUS_MINI = "bonusMini";

    /**
     * gets the price and type of the puzzle
     * @param id puzzle id
     * @param size size of puzzle i.e big, small
     * @return {*} map of price and type of puzzle
     */
    this.getPriceForPuzzle = function (id, size) {
        if (size == BIG_PUZZLE) {

            // subs check
            var subFree = subscriptionsDTCObj.checkSubscriptionStatus() == constantsObj.SUBSCRIPTION_ACTIVE && gameObj.getFreeArchivePuzzleSub() > 0;
            if (subFree) {
                return {type: FREE_SUBS_PUZZLE, price: 0};
            }

            // bonus archive check
            var bonusArchiveEligible = (((gameObj.getNumberOfBonusArchiveWatched() + questMapObj.getNumberOfBonusArchiveWatched()) < runtimeObj.getNumberOfBonusArchive()) || id == questMapObj.getTodaysQuestBonusArchivePuzzle()) && adsObj.checkRewardedVideoAvailable() && utilObj.isOnline();
            if (bonusArchiveEligible) {
                if (questMapObj.getTodaysQuestBonusArchivePuzzle() == null) {
                    questMapObj.setNumberOfBonusArchiveWatched(questMapObj.getNumberOfBonusArchiveWatched() + 1);
                }
                questMapObj.setTodaysQuestBonusArchivePuzzle(id);
                return {type: BONUS_ARCHIVE, price: 0};
            }

            // paid archive
            return {type: PAID, price: parseInt(runtimeObj.getQuestArchivePrice())};

        } else {

            // free pack
            if (!gameObj.checkForDailyMini(id)) {
                return {type: FREE_PACKS, price: 0};
            }

            // sub free
            var subFree = subscriptionsDTCObj.checkSubscriptionStatus() == constantsObj.SUBSCRIPTION_ACTIVE;
            if (subFree) {
                return {type: FREE_SUBS_PUZZLE, price: 0};
            }

            var coolDownForDailyMiniArchive = runtimeObj.getCoolDownBonusDailyMini();
            if (subscriptionsDTCObj.getIsSubscriptionInActive()) {
                coolDownForDailyMiniArchive = runtimeObj.getCoolDownBonusDailyMiniSubs();
            }
            var bonusMiniCond = (gameObj.getMastery1() >= runtimeObj.getMasteryMiniArchiveCondition() && ((gameObj.getNumberOfBonusDailyMini() + questMapObj.getNumberOfBonusMiniWatched()) < coolDownForDailyMiniArchive)) || id == questMapObj.getTodaysQuestBonusMiniPuzzle();
            if (bonusMiniCond) {
                if (questMapObj.getTodaysQuestBonusMiniPuzzle() == null) {
                    questMapObj.setNumberOfBonusMiniWatched(questMapObj.getNumberOfBonusMiniWatched() + 1);
                }
                questMapObj.setTodaysQuestBonusMiniPuzzle(id);
                return {type: BONUS_MINI, price: 0};
            }

            // paid minis
            return {type: PAID, price: parseInt(runtimeObj.getQuestMiniPrice())};
        }
    };

    /**
     * gets the puzzle for the size and difficulty
     * @param size puzzle size
     * @param diff sifficulty for the puzzle
     * @return the puzzle id selected
     */
    this.getPuzzleForSizeAndDifficulty = function (size, diff) {
        var puzzleData = questPuzzleData[size];
        var puzzleSelected = null;
        var inProgress = gameObj.getInProgress();

        for (var i in puzzleData) {
            var id = puzzleData[i][0];
            var cr = parseFloat(puzzleData[i][1]);
            if (!questMapObj.isPuzzleEligibleForQuestMap(id)) {
                continue;
            }

            if (diff == "easy" && gameObj.checkForMiniOrMidi(id)) {
                var crToCheck = 75;
                if (gameObj.checkForDailyMini(id)) {
                    crToCheck = 70;
                }
                if (cr >= crToCheck) {
                    if (scoresObj.getMastery(id) == 0 && !inProgress.hasOwnProperty(id)) {
                        puzzleSelected = id;
                        break;
                    }
                }
            } else if (diff == "medium") {
                if (!gameObj.checkForMiniOrMidi(id)) {
                    if (cr >= 60) {
                        if (scoresObj.getMastery(id) == 0 && !inProgress.hasOwnProperty(id)) {
                            puzzleSelected = id;
                            break;
                        }
                    }
                } else if (gameObj.checkForDailyMini(id)) {
                    if (cr < 70) {
                        if (scoresObj.getMastery(id) == 0 && !inProgress.hasOwnProperty(id)) {
                            puzzleSelected = id;
                            break;
                        }
                    }
                } else {
                    if (60 <= cr && cr < 75) {
                        if (scoresObj.getMastery(id) == 0 && !inProgress.hasOwnProperty(id)) {
                            puzzleSelected = id;
                            break;
                        }
                    }
                }
            } else if (diff == "hard" && !gameObj.checkForDailyMini(id)) {
                if (cr < 60) {
                    if (scoresObj.getMastery(id) == 0 && !inProgress.hasOwnProperty(id)) {
                        puzzleSelected = id;
                        break;
                    }
                }
            }
        }

        return puzzleSelected;
    };

    var IN_PROGRESS = "inProgress";
    var COMPLETED = "compeleted";

    /**
     * returns the puzzle based on cr, diff, inprogress or completed
     * @param id puzzleId
     * @param cr clear rates
     * @param diff difficulty of the puzzle
     * @param option if puzzle is completed or inprogress
     * @return {*} pzid
     */
    this.getPuzzleForCr = function (id, cr, diff, option) {
        var puzzleSelected = null;
        var inProgress = gameObj.getInProgress();

        if (diff == "easy" && gameObj.checkForMiniOrMidi(id)) {
            var crToCheck = 75;
            if (gameObj.checkForDailyMini(id)) {
                crToCheck = 70;
            }
            if (cr >= crToCheck) {
                    switch (option) {
                        case IN_PROGRESS:
                            if (scoresObj.getMastery(id) == 0) {
                                puzzleSelected = id;
                            }
                        case COMPLETED:
                            if (!inProgress.hasOwnProperty(id)) {
                                puzzleSelected = id;
                            }
                    }
            }
        } else if (diff == "medium") {
            if (!gameObj.checkForMiniOrMidi(id)) {
                if (cr >= 60) {
                    switch (option) {
                        case IN_PROGRESS:
                            if (scoresObj.getMastery(id) == 0) {
                                puzzleSelected = id;
                            }
                        case COMPLETED:
                            if (!inProgress.hasOwnProperty(id)) {
                                puzzleSelected = id;
                            }
                    }
                }
            } else if (gameObj.checkForDailyMini(id)) {
                if (cr < 70) {
                    switch (option) {
                        case IN_PROGRESS:
                            if (scoresObj.getMastery(id) == 0) {
                                puzzleSelected = id;
                            }
                        case COMPLETED:
                            if (!inProgress.hasOwnProperty(id)) {
                                puzzleSelected = id;
                            }
                    }
                }
            } else {
                if (60 <= cr && cr < 75) {
                    switch (option) {
                        case IN_PROGRESS:
                            if (scoresObj.getMastery(id) == 0) {
                                puzzleSelected = id;
                            }
                        case COMPLETED:
                            if (!inProgress.hasOwnProperty(id)) {
                                puzzleSelected = id;
                            }
                    }
                }
            }
        } else if (diff == "hard" && !gameObj.checkForDailyMini(id)) {
            if (cr < 60) {
                switch (option) {
                    case IN_PROGRESS:
                        if (scoresObj.getMastery(id) == 0) {
                            puzzleSelected = id;
                        }
                    case COMPLETED:
                        if (!inProgress.hasOwnProperty(id)) {
                            puzzleSelected = id;
                        }
                }
            }
        }

        return puzzleSelected;
    };

    // -------------------

    // returns cr of pzid if available
    this.getCrForPuzzleIdFromJson = function (pzid, size) {
        var puzzleData = questPuzzleData[size];

        for (i in puzzleData) {
            var id = puzzleData[i][0];
            if (id == pzid) {
                return puzzleData[i][1];
            }
        }

        return null;
    };


    /**
     * gets the puzzle for the size and difficulty in Progress
     * @param size puzzle size
     * @param diff sifficulty for the puzzle
     * @return the puzzle id selected
     */
    this.getPuzzleForSizeAndDifficultyInProgress = function (size, diff) {
        var puzzleSelected = null;
        var inProgress = gameObj.getInProgress();

        // loggerObj.debugLog("inprogress puzzles are", JSON.stringify(inProgress));

        for( var i in inProgress) {
            var id = inProgress[i];
            var isSmall = gameObj.checkForMiniOrMidi(i);
            if (isSmall && size != SMALL_PUZZLE) {
                continue;
            }

            var cr = questMapObj.getCrForPuzzleIdFromJson(i, size);
            if (cr == null) {
                continue;
            }

            puzzleSelected = questMapObj.getPuzzleForCr(i, parseFloat(cr), diff, IN_PROGRESS);
            if (puzzleSelected != null) {
                if (!questMapObj.isPuzzleEligibleForQuestMap(puzzleSelected)) {
                    puzzleSelected = null;
                    continue;
                }
                break;
            }
        }

        return puzzleSelected;
    };

    /**
     * checks if D0 puzzle is selected
     * @param crossWordIndex puzzle id
     * @return {boolean}
     * @constructor
     */
    this.IfD0PuzzleSelected = function (crossWordIndex) {
        var installDay = gameObj.getInstallDay();
        var d0PuzzleId = gameObj.getD0Id();
        var createdDate = "";
        var createdDateId = "";

        if (userObj.getCreated() != "") {
            createdDate = new Date(userObj.getCreated() * 1000);
            createdDateId = utilObj.getIdForDate(createdDate);
        }

        var isD0 = false;
        if (installDay == crossWordIndex && (!fbWrapperObj.isLoggedIn() || gameObj.getOldFbUser() == false || (gameObj.getOldFbUser() == true && (gameObj.getInstallDay() == "20170301" || createdDateId == crossWordIndex)))) {
            if (d0PuzzleId != null && d0PuzzleId != undefined && d0PuzzleId != "0") {
                isD0 = true;
            }
        }

        var isPuzzleIDD0 = false;

        if (crossWordIndex.substr(0, 6) == "201700") {
            isPuzzleIDD0 = true;
        }

        return isD0 || isPuzzleIDD0;
    };

    /**
     * gets the puzzle for the size and difficulty complete
     * @param size puzzle size
     * @param diff sifficulty for the puzzle
     * @return the puzzle id selected
     */
    this.getPuzzleForSizeAndDifficultyCompleted = function (size, diff) {
        var puzzleSelected = null;
        var scores = scoresObj.getScores();

        // loggerObj.debugLog("inprogress puzzles are", JSON.stringify(inProgress));
        var arrToTraverse = [];
        for (var i in scores) {
            var pzid = i;
            arrToTraverse.push(pzid);
        }

        arrToTraverse = utilObj.shuffleArray(arrToTraverse);

        for( var index in arrToTraverse) {
            var i = arrToTraverse[index];

            // adding check for not showing daily mini in replay section
            var isDailyMini = gameObj.checkForDailyMini(i);
            if (isDailyMini) {
                continue;
            }

            var isSmall = gameObj.checkForMiniOrMidi(i);
            if (isSmall && size != SMALL_PUZZLE) {
                continue;
            }
            var cr = questMapObj.getCrForPuzzleIdFromJson(i, size);
            if (cr == null) {
                continue;
            }
            puzzleSelected = questMapObj.getPuzzleForCr(i, parseFloat(cr), diff, COMPLETED);
            if (puzzleSelected != null) {
                if (!questMapObj.isPuzzleEligibleForQuestMap(puzzleSelected)) {
                    puzzleSelected = null;
                    continue;
                }
                break;
            }

        }

        return puzzleSelected;
    };


    // -------------------


    // gets the milestone info
    this.getMilestoneInfoForQuest = function () {
        var milestone1Puzzles = parseInt(milestoneInfo[MILESTONE_1][SIZE]["big"]) + parseInt(milestoneInfo[MILESTONE_1][SIZE]["small"]);
        var milestone2PuzzlesCum = milestone1Puzzles + parseInt(milestoneInfo[MILESTONE_2][SIZE]["big"]) + parseInt(milestoneInfo[MILESTONE_2][SIZE]["small"]);
        var milestone2Puzzles = milestone2PuzzlesCum - milestone1Puzzles;
        var milestone3PuzzlesCum = milestone2PuzzlesCum + parseInt(milestoneInfo[MILESTONE_3][SIZE]["big"]) + parseInt(milestoneInfo[MILESTONE_3][SIZE]["small"]);
        var milestone3Puzzles = milestone3PuzzlesCum - milestone2PuzzlesCum;

        var info = {
            "0": {
                index: milestone1Puzzles,
                totalPuzzles: milestone1Puzzles,
                id: 1,
            },
            "1": {
                index: milestone2PuzzlesCum,
                totalPuzzles: milestone2Puzzles,
                id: 2,
            },
            "2": {
                index: milestone3PuzzlesCum,
                totalPuzzles: milestone3Puzzles,
                id: 3,
            }
        };

        return info;
    };

    // -------------------

    /**
     * for fetching if the puzzles are there available for next quest to start
     * @returns number of puzzles for the next quest if it starts right now
     */
    this.getNumberOfPuzzlesInNextQuest = function () {
        var bigPuzzlesSolved = puzzlesPlayedLastWeek[BIG_PUZZLE];
        var smallPuzzlesSolved = puzzlesPlayedLastWeek[SMALL_PUZZLE];

        // runtime value for the bucket split
        var runtimeValue = runtimeObj.getEnggBucket();
        var splitStr = runtimeValue.split("_");
        var bigLimit = parseInt(splitStr[0]);
        var smallLimit = parseInt(splitStr[1]);

        // assign bucket based on the data
        var bucketForUser = BUCKET_1;
        if (bigPuzzlesSolved <= bigLimit && smallPuzzlesSolved > smallLimit) {
            bucketForUser = BUCKET_2;
        } else if (bigPuzzlesSolved > bigLimit && smallPuzzlesSolved <= smallLimit) {
            bucketForUser = BUCKET_3;
        } else if (bigPuzzlesSolved > bigLimit && smallPuzzlesSolved > smallLimit) {
            bucketForUser = BUCKET_4;
        }


        var puzzle_split = runtimeObj.getPuzzleSplit();
        var splitStrArr = puzzle_split.split("_");

        var splitVal;
        if (bucketForUser == BUCKET_1) {
            splitVal = 0;
        } else if (bucketForUser == BUCKET_2) {
            splitVal = 15;
        } else if (bucketForUser == BUCKET_3) {
            splitVal = 30;
        } else {
            splitVal = 45;
        }

        // splitting arr for the current bucket
        var arrForSplit = splitStrArr.slice(splitVal, splitVal + 15);

        var milesInfoForNextQuest = {};

        milesInfoForNextQuest[MILESTONE_1] = {};
        milesInfoForNextQuest[MILESTONE_1][SIZE] = {"big": arrForSplit[0], "small": arrForSplit[1]};
        milesInfoForNextQuest[MILESTONE_1][DIFFICULTY] = {
            "easy": arrForSplit[2],
            "medium": arrForSplit[3],
            "hard": arrForSplit[4]
        };

        milesInfoForNextQuest[MILESTONE_2] = {};
        milesInfoForNextQuest[MILESTONE_2][SIZE] = {"big": arrForSplit[5], "small": arrForSplit[6]};
        milesInfoForNextQuest[MILESTONE_2][DIFFICULTY] = {
            "easy": arrForSplit[7],
            "medium": arrForSplit[8],
            "hard": arrForSplit[9]
        };

        milesInfoForNextQuest[MILESTONE_3] = {};
        milesInfoForNextQuest[MILESTONE_3][SIZE] = {"big": arrForSplit[10], "small": arrForSplit[11]};
        milesInfoForNextQuest[MILESTONE_3][DIFFICULTY] = {
            "easy": arrForSplit[12],
            "medium": arrForSplit[13],
            "hard": arrForSplit[14]
        };

        // calculate total number of puzzles for quest
        var totalPuzzlesForNextQuest = 0;
        for (var i = MILESTONE_1; i <= MILESTONE_3; i++) {
            totalPuzzlesForNextQuest += parseInt(milesInfoForNextQuest[i][SIZE]["big"]);
            totalPuzzlesForNextQuest += parseInt(milesInfoForNextQuest[i][SIZE]["small"]);
        }

        return totalPuzzlesForNextQuest;

    };


    /**reward logic for the weekly quest
     * @level milestone level
     * @return the reward value
      */
    this.getRewardNumber = function (milestoneLevel) {
        var questReward = runtimeObj.getWQRewardProb();

        var questStr = questReward.split(":");
        var nonPayerStr = questStr[0];
        var payerStr = questStr[1];
        var strToUse;

        // checks for the payer probs
        var isPayer = gameObj.getIsPayer();
        if (isPayer) {
            strToUse = payerStr;
        } else {
            strToUse = nonPayerStr;
        }

        var splitStr = strToUse.split("_");
        var reward1, reward2, reward3, prob1, prob2, prob3;

        // getting data from the runtimes for the milsetone
        reward1 = splitStr[0 + (6 * (milestoneLevel - 1))];
        reward2 = splitStr[2 + (6 * (milestoneLevel - 1))];
        reward3 = splitStr[4 + (6 * (milestoneLevel - 1))];
        prob1 = splitStr[1 + (6 * (milestoneLevel - 1))];
        prob2 = splitStr[3 + (6 * (milestoneLevel - 1))];
        prob3 = splitStr[5 + (6 * (milestoneLevel - 1))];

        var probVals = [parseInt(prob1), parseInt(prob2), parseInt(prob3)];
        var rewardVals = [parseInt(reward1), parseInt(reward2), parseInt(reward3)];

        var totalVal = 0;
        for( var i in probVals) {
            totalVal += probVals[i];
        }

        var randomNum = utilObj.getRandomIntInRange(0, totalVal); // random value for the reward probs

        totalVal = 0;
        var index = 0;
        for( var i in probVals) {
            totalVal += probVals[i];
            if (randomNum <= totalVal) {
                index = i;
                break;
            }
        }

        var rewardtoGive = rewardVals[index];
        return rewardtoGive;
    };

    // loads puzzle data from the json
    var questPuzzleData = null;
    this.loadQuestPuzzleJsonData = function () {
        var puzzleDataUrl = constantsObj.QUEST_PUZZLE_DATA;
        try {
            cc.loader.loadJson(puzzleDataUrl, function (error, data) {
                questPuzzleData = data;
                // questMapObj.getPuzzleForIndex(10);
                // loggerObj.debugLog("quest info is 5", questPuzzleData);
            });
        } catch (e) {
            loggerObj.errorLog("Exception while loading quest puzzle json:", e);
        }
    };

    var wqData = {};
    var tqData = {};
    this.init = function() {
        // load puzzle json data
        questMapObj.loadQuestPuzzleJsonData();

        questMapObj.loadWqData();
        questMapObj.loadTqData();

    };

    this.loadWqData = function () {
        var wqDataUrl = "res/jsons/questImages.json";
        try {
            cc.loader.loadJson(wqDataUrl, function (error, data) {
                wqData = data;
            });
        } catch (e) {
            //loggerObj.errorLog("Exception while loading json:", e);
        }
    };

    this.loadTqData = function () {
        var tqDataUrl = "res/jsons/themedQuestImages.json";
        try {
            cc.loader.loadJson(tqDataUrl, function (error, data) {
                tqData = data;
            });
        } catch (e) {
            //loggerObj.errorLog("Exception while loading json:", e);
        }
    };

    this.getWqData = function () {
        return wqData;
    };

    this.getTqData = function () {
        return tqData;
    };

    this.getPuzState = function (id) {
        var state = "";
        var inProgress = gameObj.getInProgress();
        var score = scoresObj.getHighScore(id);
        if (inProgress[id] == undefined && score > 0) {
            state = constantsObj.STATE_PUZ_COMPLETED;
        } else if (inProgress[id] != undefined) {
            state = constantsObj.STATE_PUZ_OPEN;
        } else {
            state = constantsObj.STATE_PUZ_CLOSED;
        }
        return state;
    };

    this.getPuzStateById = function (id) {
        var state = "";
        var inProgress = gameObj.getInProgress();
        var score = scoresObj.getHighScore(id);
        if (inProgress[id] == undefined && score > 0) {
            state = constantsObj.STATE_PUZ_COMPLETED;
        } else if (inProgress[id] != undefined) {
            state = constantsObj.STATE_PUZ_OPEN;
        } else {
            state = constantsObj.STATE_PUZ_CLOSED;
        }
        return state;
    };



    this.getBackGroundPath = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                return tqData["tq" + "_" + imageType]["theme_bg"];
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                return wqData[year + "_" + weekNum + "_" + imageType]["theme_bg"];
            }
        } catch (err) {
           return constantsObj.forest_bg;
        }
    };

    this.getWeeklyThemeText = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                return tqData["tq" + "_" + imageType]["theme"];
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                return wqData[year + "_" + weekNum + "_" + imageType]["theme"];
            }
        } catch (err) {
            return "Weekly Quest";
        }
    };

    this.getWeeklyThemeHomeText = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                return tqData["tq" + "_" + imageType]["quest_home_desc"];
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                return wqData[year + "_" + weekNum + "_" + imageType]["quest_home_desc"];
            }
        } catch (err) {
            return "Solve puzzles & win cool rewards!";
        }
    };

    this.getHomeTextColor = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                var color = tqData["tq" + "_" + imageType]["hs_textColor"];
                return homeObj.getColorFromStringCode(color);
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                var color = wqData[year + "_" + weekNum + "_" + imageType]["hs_textColor"];
                return homeObj.getColorFromStringCode(color);
            }
        } catch (err) {
            return cc.color(0, 0 ,0);
        }
    };

    this.getColorForVectorPath = function(year , weekNum){
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                var color = tqData["tq" + "_" + "vector"]["path_color"];
                return homeObj.getColorFromStringCode(color);
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                var color = wqData[year + "_" + weekNum + "_" + "vector"]["path_color"];
                return homeObj.getColorFromStringCode(color);
            }
        } catch (err) {
            return cc.color(0, 0 ,0);
        }
    };

    this.getColorForRealisticPath = function(year , weekNum){
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                var color = tqData["tq" + "_" + "realistic"]["path_color"];
                return homeObj.getColorFromStringCode(color);
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                var color = wqData[year + "_" + weekNum + "_" + "realistic"]["path_color"];
                return homeObj.getColorFromStringCode(color);
            }
        } catch (err) {
            return cc.color(255, 255 ,255);
        }
    };

    this.getPlayBgColor = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                var color = tqData["tq" + "_" + imageType]["play_button_color"];
                return homeObj.getColorFromStringCode(color);
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                var color = wqData[year + "_" + weekNum + "_" + imageType]["play_button_color"];
                return homeObj.getColorFromStringCode(color);
            }
        } catch (err) {
            return cc.color(65, 183, 81);
        }
    };
    
    this.getBarColorCodeForQuest = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsWeeklyQuestLocked()) {
                var colorCode = wqData[year + "_" + weekNum + "_" + imageType]["pb_bar_fill_code"];
                return homeObj.getColorFromStringCode(colorCode);
            }

        } catch (err) {
            return cc.color(36, 167, 224);
        }
    };


    this.getWeeklyThemeImage = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                return tqData["tq" + "_" + imageType]["theme_img"];
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                return wqData[year + "_" + weekNum + "_" + imageType]["theme_img"];
            }
        } catch (err) {
            return constantsObj.forest_img;
        }
    };

    this.getMilestoneImage = function (year, weekNum, experiment, milestoneNum) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        loggerObj.debugLog("QuestPuzzleUnlockPopup - year weekNum, exp, mile ", year, weekNum, experiment, milestoneNum);
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                if (milestoneNum == 0) {
                    return tqData["tq" + "_" + imageType]["milestone_1"];
                } else if (milestoneNum == 1) {
                    return tqData["tq" + "_" + imageType]["milestone_2"];
                } else if (milestoneNum == 2) {
                    return tqData["tq" + "_" + imageType]["milestone_3"];
                }
            }
            if (!gameObj.getIsWeeklyQuestLocked()) {
                if (milestoneNum == 0) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["milestone_1"];
                } else if (milestoneNum == 1) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["milestone_2"];
                } else if (milestoneNum == 2) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["milestone_3"];
                }
            }

        } catch (err) {
            if (milestoneNum == 0) {
                return constantsObj.tq_gift_open1;
            } else if (milestoneNum == 1) {
                return constantsObj.tq_gift_open4;
            } else if (milestoneNum == 2) {
                return constantsObj.tq_gift_open3;
            }
            return constantsObj.tq_gift_open1;
        }
    };

    this.getMilestoneImageById = function (id, milestoneNum) {
        var imageType = "vector";
        var experiment = gameObj.getExpNewWeeklyQuestVariant();
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                if (milestoneNum == 0) {
                    return tqData[id]["milestone_1"];
                } else if (milestoneNum == 1) {
                    return tqData[id]["milestone_2"];
                } else if (milestoneNum == 2) {
                    return tqData[id]["milestone_3"];
                }
            }

            if (!gameObj.getIsWeeklyQuestLocked()) {
                if (milestoneNum == 0) {
                    return wqData[id]["milestone_1"];
                } else if (milestoneNum == 1) {
                    return wqData[id]["milestone_2"];
                } else if (milestoneNum == 2) {
                    return wqData[id]["milestone_3"];
                }
            }

        } catch (err) {
            return constantsObj.forest_bg;
        }
    };

    this.getMilestoneDescription = function (year, weekNum, experiment, milestoneNum) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                if (milestoneNum == 0) {
                    return tqData["tq" + "_" + imageType]["description_1"];
                } else if (milestoneNum == 1) {
                    return tqData["tq" + "_" + imageType]["description_2"];
                } else if (milestoneNum == 2) {
                    return tqData["tq" + "_" + imageType]["description_3"];
                }
            }

            if (!gameObj.getIsWeeklyQuestLocked()) {
                if (milestoneNum == 0) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["description_1"];
                } else if (milestoneNum == 1) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["description_2"];
                } else if (milestoneNum == 2) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["description_3"];
                }
            }

        } catch (err) {
            return "Well done! You've earned yourself some extra coins!";
        }
    };

    this.getToPushCollectible = function (year, weekNum, experiment, milestoneNum) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                if (milestoneNum == 0) {
                    return tqData["tq" + "_" + imageType]["description_1"];
                } else if (milestoneNum == 1) {
                    return tqData["tq" + "_" + imageType]["description_2"];
                } else if (milestoneNum == 2) {
                    return tqData["tq" + "_" + imageType]["description_3"];
                }
            }

            if (!gameObj.getIsWeeklyQuestLocked()) {
                if (milestoneNum == 0) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["description_1"];
                } else if (milestoneNum == 1) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["description_2"];
                } else if (milestoneNum == 2) {
                    return wqData[year + "_" + weekNum + "_" + imageType]["description_3"];
                }
            }

        } catch (err) {
            if(milestoneNum == 2){
                return "You have completed all the milestones this week! Start a new Quest on Sunday!";
            }
            return "You've earned yourself some extra coins! Keep going to earn more!";
        }
    };

    this.getMilestoneTheme = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                return tqData["tq" + "_" + imageType]["theme"];
            }

            if (!gameObj.getIsWeeklyQuestLocked()) {
                return wqData[year + "_" + weekNum + "_" + imageType]["theme"];
            }
        } catch (err) {
            return "";
        }
    };

    this.getMilestoneCollectible = function (year, weekNum, experiment, milestoneNum) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                return tqData["tq" + "_" + imageType]["milestone_" + (milestoneNum + 1) + "_collectible"];
            }

            if (!gameObj.getIsWeeklyQuestLocked()) {
                return wqData[year + "_" + weekNum + "_" + imageType]["milestone_" + (milestoneNum + 1) + "_collectible"];
            }
        } catch (err) {
            if(milestoneNum != undefined) {
                return constantsObj.DEFAULT_QUEST["default_week"]["milestone_" + (milestoneNum + 1) + "_collectible"];
            }

            return "";

        }
    };

    this.getMilestonePopupText = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsWeeklyQuestLocked()) {
                return wqData[year + "_" + weekNum + "_" + imageType]["milestone_text"];
            }
            if (!gameObj.getIsThemedQuestLocked()) {
                return tqData["tq" + "_" + imageType]["milestone_text"];
            }
            return "Solve all the previous puzzles to unlock this Milestone and earn rewards!";
        } catch (err) {
            return "Solve all the previous puzzles to unlock this Milestone and earn rewards!";
        }
    };

    this.getHomeGradientColorStart = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                var color = tqData["tq" + "_" + imageType]["quest_home_gradient_start"];
                return homeObj.getColorFromStringCode(color);
            }

            if (!gameObj.getIsWeeklyQuestLocked()) {
                var color = wqData[year + "_" + weekNum + "_" + imageType]["quest_home_gradient_start"];
                return homeObj.getColorFromStringCode(color);
            }
        } catch (err) {
            return homeObj.getColorFromStringCode("252, 193, 10");
        }
    };

    this.getHomeGradientColorEnd = function (year, weekNum, experiment) {
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        try {
            if (!gameObj.getIsThemedQuestLocked()) {
                var color = tqData["tq" + "_" + imageType]["quest_home_gradient_end"];
                return homeObj.getColorFromStringCode(color);
            }

            if (!gameObj.getIsWeeklyQuestLocked()) {
                var color = wqData[year + "_" + weekNum + "_" + imageType]["quest_home_gradient_end"];
                return homeObj.getColorFromStringCode(color);
            }
        } catch (err) {
            return homeObj.getColorFromStringCode("232, 156, 21");
        }
    };

    this.getWeekNumber = function (date) {
        var prefixes = ['First', 'Second', 'Third', 'Fourth', 'Fifth'];
        return prefixes[Math.floor(date.getDate() / 7)];

    };

    this.getWeek = function (year1, month, day) {
        var when = new Date(year1, month, day);
        var year = questMapObj.y2k(year1);
        var newYear = new Date(year1, 0, 1);
        var offset = 7 + 1 - newYear.getDay();
        if (offset == 8) {
            offset = 1;
        }
        var daynum = ((Date.UTC(year, when.getMonth(), when.getDate(), 0, 0, 0) - Date.UTC(year, 0, 1, 0, 0, 0)) / 1000 / 60 / 60 / 24) + 1;
        var weeknum = Math.floor((daynum - offset + 7) / 7);
        if (utilObj.isAndroid()) {
            if (weeknum == 0) {
                year1--;
                var prevNewYear = new Date(year1, 0, 1);
                var prevOffset = 7 + 1 - prevNewYear.getDay();
                if (prevOffset == 2 || prevOffset == 8) weeknum = 53; else weeknum = 52;
            }
        }
        // if (weeknum == 0) {
        //     year1--;
        //     var prevNewYear = new Date(year1, 0, 1);
        //     var prevOffset = 7 + 1 - prevNewYear.getDay();
        //     if (prevOffset == 2 || prevOffset == 8) weeknum = 53; else weeknum = 52;
        // }
        return weeknum;
    };

    this.y2k = function (number) {
        return (number < 1000) ? number + 1900 : number;
    };

    this.getCurrentWeeklyQuestMilestone = function () {
        var milestonesCompleted = gameObj.getWeeklyQuestMilestonesCompleted();
        var currentMilestone = 0;
        if (milestonesCompleted == undefined || milestonesCompleted == null) {
            currentMilestone = 1;
            return currentMilestone;
        }
        if (milestonesCompleted[2] == 1) {
            currentMilestone = 3;
        } else if (milestonesCompleted[1] == 1) {
            currentMilestone = 3;
        } else if (milestonesCompleted[0] == 1) {
            currentMilestone = 2;
        } else if (milestonesCompleted [0] == 0) {
            currentMilestone = 1;
        }
        return currentMilestone;
    };

    this.getLastWeeklyMileStoneCompleted = function () {
        var milestonesCompleted = gameObj.getWeeklyQuestMilestonesCompleted();
        var lastMilestoneCompleted = 0;
        if (milestonesCompleted == undefined || milestonesCompleted == null) {
            lastMilestoneCompleted = 0;
            return lastMilestoneCompleted;
        }
        if (milestonesCompleted[2] == 1) {
            lastMilestoneCompleted = 3;
        } else if (milestonesCompleted[1] == 1) {
            lastMilestoneCompleted = 2;
        } else if (milestonesCompleted[0] == 1) {
            lastMilestoneCompleted = 1;
        } else if (milestonesCompleted [0] == 0) {
            lastMilestoneCompleted = 0;
        }
        return lastMilestoneCompleted;
    };

    this.getCurrentThemedQuestMilestone = function () {
        var milestonesCompleted = gameObj.getThemedQuestMilestonesCompleted();
        var currentMilestone = 0;
        if (milestonesCompleted == undefined || milestonesCompleted == null) {
            currentMilestone = 1;
            return currentMilestone;
        }
        if (milestonesCompleted[2] == 1) {
            currentMilestone = 3;
        } else if (milestonesCompleted[1] == 1) {
            currentMilestone = 3;
        } else if (milestonesCompleted[0] == 1) {
            currentMilestone = 2;
        } else if (milestonesCompleted [0] == 0) {
            currentMilestone = 1;
        }
        return currentMilestone;
    };

    this.getLastThemedMileStoneCompleted = function () {
        var milestonesCompleted = gameObj.getThemedQuestMilestonesCompleted();
        var lastMilestoneCompleted = 0;
        if (milestonesCompleted == undefined || milestonesCompleted == null) {
            lastMilestoneCompleted = 0;
            return lastMilestoneCompleted;
        }
        if (milestonesCompleted[2] == 1) {
            lastMilestoneCompleted = 3;
        } else if (milestonesCompleted[1] == 1) {
            lastMilestoneCompleted = 2;
        } else if (milestonesCompleted[0] == 1) {
            lastMilestoneCompleted = 1;
        } else if (milestonesCompleted [0] == 0) {
            lastMilestoneCompleted = 0;
        }
        return lastMilestoneCompleted;
    };

    // this.getIndexForId = function(id){
    //      for(var i=0 ; i< questMapLayer.idIndexArr.length ; ++i){
    //          if(questMapLayer.idIndexArr[i] == id){
    //              return (i+1);
    //          }
    //      }
    // };

    this.getWeekIdOfYear = function () {
        var experiment = gameObj.getExpNewWeeklyQuestVariant();

        var now = new Date();
        var year = questMapObj.y2k(now.getYear());
        var month = now.getMonth();
        var date = now.getDate();

        var weekNum = questMapObj.getWeek(year, month, date);
        var imageType = "vector";
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        var id = year + "_" + weekNum + "_" + imageType;
        return id;
    };

    this.getTypeAndPriceOfPuzzle = function (index) {
        var info1 = [];
        var info = {};
        if(utilObj.getStringifiedJson(gameObj.getCurPuzzleInfo()) == utilObj.getStringifiedJson([])) {
            info = questMapObj.getPuzzleForIndex(index);
            info1.push(info);
            gameObj.setCurPuzzleInfo(info1);
        }else{
            info1 = gameObj.getCurPuzzleInfo();
            info = info1[0];
            if(info.id != gameObj.getClickedPuzzleId()){
                info = questMapObj.getPuzzleForIndex(index);
                info1 = [];
                info1.push(info);
                gameObj.setCurPuzzleInfo(info1);
            }
        }
        var type = "";
        if (info.type == FREE_PACKS || info.type == FREE_SUBS_PUZZLE || info.type == BONUS_MINI) {
            type = "free";
        } else if (info.type == BONUS_ARCHIVE) {
            type = "w2e";
        } else if (info.type == PAID) {
            var price = info.price;
            type = price + "";
        }

        return type;
    };

    this.updatePuzzleCompletionVariables = function (id) {
        var puzzleIndexOnMaplayer = gameObj.getClickedPuzzleNum();
        questMapLayer.lastLevel = puzzleIndexOnMaplayer;
        if(questMapLayer.lastLevel == 1){
            gameObj.setWqTapHereMessageSeen(true);
        }

        gameObj.setMapLastLevel(questMapLayer.lastLevel);

        //animShown need to be true when animation is shown on layer show for this puzzle.
        var info = {
            id: id,
            puzzleIndex: puzzleIndexOnMaplayer,
            completed: true,
            animShown: false,
        };
        questMapLayer.puzzlesCompleted = gameObj.getPuzzlesCompletedArray();


        questMapLayer.puzzlesCompleted.push(info);
        var array = [];
        gameObj.setCurPuzzleInfo(array);
        gameObj.setPuzzlesCompletedArray(questMapLayer.puzzlesCompleted);
    };

    this.updateMilestonesCompletedArray = function(){
        var imageType = "vector";
        var experiment = gameObj.getExpNewWeeklyQuestVariant();
        if (experiment == constantsObj.EXP_VAR_2) {
            imageType = "realistic";
        }
        var lastLevel = gameObj.getMapLastLevel();
        var idOfYear = questMapObj.getWeekIdOfYear();

        var now = new Date();
        var year = questMapObj.y2k(now.getYear());
        var month = now.getMonth();
        var date = now.getDate();
        var weekNum = questMapObj.getWeek(year, month, date);

        // this will contain the information for the animation trigger
        var infoForAnim = {};

        if (!gameObj.getIsThemedQuestLocked()) {
            var milestonesCompleted = gameObj.getThemedQuestMilestonesCompleted();
            if (milestonesCompleted == null) {
                milestonesCompleted = [0, 0, 0];
                gameObj.setThemedQuestMilestonesCompleted(milestonesCompleted);
                gameObj.setThemedQuestCompleted(false);
            }
            if (questMapLayer.allMilestonesInfo[0]["index"] == lastLevel) {
                if (milestonesCompleted[0] == 0) {
                    milestonesCompleted[0] = 1;
                    var stringDesc = questMapObj.getMilestoneDescription(year, weekNum, experiment, 0);
                    if(stringDesc != "") {
                        var info = {
                            idOfYear: idOfYear,
                            collectible: 0,
                        };
                        collectibles.push(info);
                    }
                    gameObj.setThemedQuestMilestonesCompleted(milestonesCompleted);

                    infoForAnim["mileStoneIndex"] = 1;
                    infoForAnim["lastLevel"] = lastLevel;
                }
            } else if (questMapLayer.allMilestonesInfo[1]["index"] == lastLevel) {
                if (milestonesCompleted[1] == 0) {
                    milestonesCompleted[1] = 1;
                    var stringDesc = questMapObj.getMilestoneDescription(year, weekNum, experiment, 1);
                    if(stringDesc != "") {
                        var info = {
                            idOfYear: idOfYear,
                            collectible: 1,
                        };
                        collectibles.push(info);
                    }
                    gameObj.setThemedQuestMilestonesCompleted(milestonesCompleted);

                    infoForAnim["mileStoneIndex"] = 2;
                    infoForAnim["lastLevel"] = lastLevel;
                }
            } else if (questMapLayer.allMilestonesInfo[2]["index"] == lastLevel) {
                if (milestonesCompleted[2] == 0) {
                    milestonesCompleted[2] = 1;
                    var stringDesc = questMapObj.getMilestoneDescription(year, weekNum, experiment, 2);
                    if(stringDesc != "") {
                        var info = {
                            idOfYear: idOfYear,
                            collectible: 2,
                        };
                        collectibles.push(info);
                    }
                    gameObj.setThemedQuestMilestonesCompleted(milestonesCompleted);

                    infoForAnim["mileStoneIndex"] = 3;
                    infoForAnim["lastLevel"] = lastLevel;

                    gameObj.setIsQuestEligileToShow(false);
                }
            }
        }
        if (gameObj.getIsWeeklyQuestLocked() == false) {
            var milestonesCompleted = gameObj.getWeeklyQuestMilestonesCompleted();
            if (milestonesCompleted == null) {
                milestonesCompleted = [0, 0, 0];
                gameObj.setWeeklyQuestMilestonesCompleted(milestonesCompleted);
                gameObj.setWeeklyQuestCompleted(false);
            }
            if (questMapLayer.allMilestonesInfo[0]["index"] == lastLevel) {
                if (milestonesCompleted[0] == 0) {
                    milestonesCompleted[0] = 1;
                    var stringDesc = questMapObj.getMilestoneDescription(year, weekNum, experiment, 0);
                    if(stringDesc != "") {
                        var info = {
                            idOfYear: idOfYear,
                            collectible: 0,
                        };
                        collectibles.push(info);
                    }
                    gameObj.setWeeklyQuestMilestonesCompleted(milestonesCompleted);

                    infoForAnim["mileStoneIndex"] = 1;
                    infoForAnim["lastLevel"] = lastLevel;
                }
            } else if (questMapLayer.allMilestonesInfo[1]["index"] == lastLevel) {
                if (milestonesCompleted[1] == 0) {
                    milestonesCompleted[1] = 1;
                    var stringDesc = questMapObj.getMilestoneDescription(year, weekNum, experiment, 1);
                    if(stringDesc != "") {
                        var info = {
                            idOfYear: idOfYear,
                            collectible: 1,
                        };
                        collectibles.push(info);
                    }
                    gameObj.setWeeklyQuestMilestonesCompleted(milestonesCompleted);

                    infoForAnim["mileStoneIndex"] = 2;
                    infoForAnim["lastLevel"] = lastLevel;
                }
            } else if (questMapLayer.allMilestonesInfo[2]["index"] == lastLevel) {
                if (milestonesCompleted[2] == 0) {
                    milestonesCompleted[2] = 1;
                    var stringDesc = questMapObj.getMilestoneDescription(year, weekNum, experiment, 2);
                    if(stringDesc != "") {
                        var info = {
                            idOfYear: idOfYear,
                            collectible: 2,
                        };
                        collectibles.push(info);
                    }
                    gameObj.setWeeklyQuestMilestonesCompleted(milestonesCompleted);

                    infoForAnim["mileStoneIndex"] = 3;
                    infoForAnim["lastLevel"] = lastLevel;

                    gameObj.setIsQuestEligileToShow(false);
                }
            }
        }
        return infoForAnim;
    };

    /**
     * returns the update if milestone will be updated on next view of quest map screen
     */
    this.getIfMileStoneUpdated = function() {
        var lastLevel = gameObj.getMapLastLevel();

        // this will contain the information for the animation trigger
        var milestonesCompleted2 = gameObj.getThemedQuestMilestonesCompleted();
        var milestonesCompleted1 = null;
        if (milestonesCompleted2 != null) {
            milestonesCompleted1 = [];
            for(var i in milestonesCompleted2) {
                milestonesCompleted1.push(milestonesCompleted2[i]);
            }
        }

        if (!gameObj.getIsThemedQuestLocked()) {
            if (milestonesCompleted1 == null) {
                milestonesCompleted1 = [0, 0, 0];
                gameObj.setThemedQuestMilestonesCompleted(milestonesCompleted1);
                gameObj.setThemedQuestCompleted(false);
            }
            if (questMapLayer.allMilestonesInfo == null) {
                return milestonesCompleted1;
            }

            if (questMapLayer.allMilestonesInfo[0]["index"] == lastLevel) {
                if (milestonesCompleted1[0] == 0) {
                    milestonesCompleted1[0] = 1;
                }
            } else if (questMapLayer.allMilestonesInfo[1]["index"] == lastLevel) {
                if (milestonesCompleted1[1] == 0) {
                    milestonesCompleted1[1] = 1;
                }
            } else if (questMapLayer.allMilestonesInfo[2]["index"] == lastLevel) {
                if (milestonesCompleted1[2] == 0) {
                    milestonesCompleted1[2] = 1;
                }
            }
        }

        loggerObj.debugLog("tq milestone completed1", milestonesCompleted1, milestonesCompleted2);
        return milestonesCompleted1;
    };

    this.updateRewardOfWeeklyQuest = function(milestoneNum){
        //grant coins and show animation here
        if(utilObj.isStaging()) {
            var coins = 100;
            economyObj.addCash(coins, constantsObj.TRACK_EARN, "", "");
        }
    };

    this.updateRewardOfThemedQuest = function(mileStoneNum){
        //grant coins and show animation here
        if(utilObj.isStaging()) {
            var coins = 200;
            economyObj.addCash(coins, constantsObj.TRACK_EARN, "", "");
        }
    };

    this.getIsMilestoneDiscovered = function (mileStoneNum) {
        if (!gameObj.getIsThemedQuestLocked()) {
            var milestonesCompleted = gameObj.getThemedQuestMilestonesCompleted();
            if (milestonesCompleted == null) {
                milestonesCompleted = [0, 0, 0];
                gameObj.setThemedQuestMilestonesCompleted(milestonesCompleted);
                gameObj.setThemedQuestCompleted(false);
            }
            if (milestonesCompleted[mileStoneNum] == 1) {
                return true;
            } else {
                return false;
            }
        }

        if (!gameObj.getIsWeeklyQuestLocked()) {
            var milestonesCompleted = gameObj.getWeeklyQuestMilestonesCompleted();
            if (milestonesCompleted == null) {
                milestonesCompleted = [0, 0, 0];
                gameObj.setWeeklyQuestMilestonesCompleted(milestonesCompleted);
                gameObj.setWeeklyQuestCompleted(false);
            }
            if (milestonesCompleted[mileStoneNum] == 1) {
                return true;
            } else {
                return false;
            }
        }
    };

    this.resetLayerVar = function () {
        questMapLayer.path = [],
            questMapLayer.milestoneNodes = [],
            questMapLayer.levelNodes = {},
            questMapLayer.milestoneNodesMap = {},
            questMapLayer.milestoneNodesAddedPos = {},
            questMapLayer.curMilestoneNode = null,
            questMapLayer.curPuzzleNo = null,
            questMapLayer.scrollViewPos = null,
            questMapLayer.lastScrollViewPos = null,
            questMapLayer.curMilestoneInfo = {},
            questMapLayer.allMilestonesInfo = null,
            questMapLayer.nextBtnXPosIndex = 0,
            questMapLayer.nextBtnYPosIndex = 0,
            questMapLayer.totalPuzzles = 0,
            questMapLayer.lastLevel = 0,
            questMapLayer.currentLevelHeight = null,
            questMapLayer.jumpToPercent = null,
            questMapLayer.levelNodePos = {},
            questMapLayer.curLevelNum = 0,
            questMapLayer.isLevelNodeAdded = {},
            questMapLayer.totalNodesNumBelow = null,
            questMapLayer.totalNodesNumAbove = 0,
            questMapLayer.indexArr = [],
            questMapLayer.milestoneLastNodeIndex = [],
            questMapLayer.curLevelHeight = null,
            questMapLayer.puzzlesCompleted = [],
            questMapLayer.clickedPuzzleId = 0,
            questMapLayer.clickedPuzzleIndex = 0,
            gameObj.setClickedPuzzleNum(questMapLayer.clickedPuzzleIndex);
            questMapLayer.shouldAddButton = 1
    };

    this.resetVariablesForNewQuest = function () {
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, "", "", constantsObj.TRACK_WQ_MAP, constantsObj.TRACK_RESET, "weekly_quest_reset" , "", "", "");
        var puzzlesCompleted = [];
        gameObj.setPuzzlesCompletedArray(puzzlesCompleted);
        gameObj.setMapLastLevel(0);
        questMapObj.resetLayerVar();
        gameObj.setWeeklyQuestCompleted(false);
        gameObj.setThemedQuestCompleted(false);
        gameObj.setThemedQuestMilestonesCompleted([0,0,0]);
        gameObj.setWeeklyQuestMilestonesCompleted([0,0,0]);

        var array = [];
        gameObj.setCurPuzzleInfo(array);
    };

    this.getCollectibles = function () {
        return collectibles;
    };

    this.setCollectibles = function (val) {
        collectibles = val;
    };

    /**
     * creates a string from collectibles array to pass to server
     * @returns {string}
     */
    this.getStringifiedCollectibles = function () {
        var numCollectibles = collectibles.length;
        var collectiblesStr = "";
        for(var i = 0; i < numCollectibles; i++){
            collectiblesStr += collectibles[i].idOfYear + "-" + collectibles[i].collectible;
            if(i < (numCollectibles - 1)){
                collectiblesStr += ":";
            }
        }
        return collectiblesStr;
    };

    /**
     * updates collectibles data as received from server
     * @param inp
     */
    this.updateCollectiblesDataFromServer = function (inp) {
        if (inp == undefined || inp == null || inp == "") {
            return;
        }
        var collectiblesStrArr = inp.split(":");
        var numCollectibles = collectiblesStrArr.length;
        collectibles = [];
        for (var j = 0; j < numCollectibles; j++) {
            var infoArr = collectiblesStrArr[j].split("-");
            collectibles.push({
                idOfYear: infoArr[0],
                collectible: parseInt(infoArr[1])
            });
        }
    };

    this.isQuestMapAutoOn = function () {
        return questMapAutoOn;
    };

    this.setQuestMapAutoOn = function (val) {
        questMapAutoOn = val;
    };

    this.getExtraDelayForSurfacingTodaysChallengeSection = function () {
        return extraDelayForSurfacingTodaysChallengeSection;
    };

    this.setExtraDelayForSurfacingTodaysChallengeSection = function (val) {
        extraDelayForSurfacingTodaysChallengeSection = val;
    };

    /**
     * this function checks if the notifBubble and shine animation have to be shown on daily challenge section of questMapLayer
     * @returns {boolean}
     */
    this.shouldShowNotifBubbleAnimation = function () {
        if(gameObj.getDcAnimationQuestMapViewsToday() >= runtimeObj.getDCQuestMapMaxViewPerDay()){
            return false;
        }

        if(utilObj.getCurrentTimestamp() - gameObj.getDcAnimationLastSeenTimestamp() <= runtimeObj.getDCQuestMapCooldownMinutes() * constantsObj.SECONDS_IN_MIN){
            return false;
        }
        return true;
    };

    /**
     * checks if puzzle is eligible for showing up on QuestMapLayer
     * returns false, if it is a d0 puzzle or fixed/bonus mini puzzle
     */
    this.isPuzzleEligibleForQuestMap = function (puzzleId) {
        if(questMapObj.IfD0PuzzleSelected(puzzleId)){
            return false;
        }

        if(fixedMinisObj.isEligibleForFixedMinis()) {
            if (fixedMinisObj.checkIfFixedOrBonusMiniExistsForDate(puzzleId)) {
                return false;
            }
        }
        return true;
    };
};
