Spinner = function () {
    var revstatusMap = {};
    var engagementStatusMap = {};
    var mapFrenzyWordPuzzle = {};
    var mapDiscountHintPuzzle = {};

    // update values for rev status map value for corresponding on action
    this.updateRevStatusMap = function () {
        var todaysPuzzleDate = gameObj.getTodaysPuzzleDate();
        var id = utilObj.getIdForDate(todaysPuzzleDate);
        if (!revstatusMap[id]) {
            revstatusMap[id] = 0;
        }
        revstatusMap[id]++;
        loggerObj.debugLog("map for spinner rev is", JSON.stringify(revstatusMap));
    };

    // update engagement status map value for corresponding on action
    this.updateEngagementStatusMap = function () {
        var todaysPuzzleDate = gameObj.getTodaysPuzzleDate();
        var id = utilObj.getIdForDate(todaysPuzzleDate);
        if (!engagementStatusMap[id]) {
            engagementStatusMap[id] = 0;
        }
        engagementStatusMap[id]++;
        loggerObj.debugLog("map for spinner engage is", JSON.stringify(engagementStatusMap));
    };

    // for storing the variables in the gameData
    this.getRevStatusMap = function () {
        return revstatusMap;
    };

    this.setRevStatusMap = function (val) {
        revstatusMap = val;
    };

    this.getEngagementMap = function () {
        return engagementStatusMap;
    };

    this.setEngagementMap = function (val) {
        engagementStatusMap = val;
    };

    this.getMapFrenzyWordPuzzle = function () {
        return mapFrenzyWordPuzzle;
    };

    this.setMapFrenzyWordPuzzle = function (val) {
        mapFrenzyWordPuzzle = val;
    };

    this.getMapDiscountHintPuzzle = function () {
        return mapDiscountHintPuzzle;
    };

    this.setMapDiscountHintPuzzle = function (val) {
        mapDiscountHintPuzzle = val;
    };

    // updates the maps for rev status and engagement status
    // deletes the days for which the value is not to be considered in the map
    this.updateMapsForSpinnerTimePeriod = function () {
        var todaysPuzzleDate = gameObj.getTodaysPuzzleDate();
        var milliSecInDay = 86400000;

        var curTime = todaysPuzzleDate.getTime();
        var possibleDates = {};
        var spinnerTimerPeriod = runtimeObj.getSpinnerTimePeriod();
        for (var i = 0; i < spinnerTimerPeriod; i++) {
            curTime -= milliSecInDay;
            var newDate = new Date(curTime);
            var newId = utilObj.getIdForDate(newDate);
            if (newId < 20170301) {
                break;
            }
            possibleDates[newId] = true;
        }

        for (var key in revstatusMap) {
            if (!possibleDates[key]) {
                if (revstatusMap[key]) {
                    delete revstatusMap[key];
                }
            }
        }

        for (var key in engagementStatusMap) {
            if (!possibleDates[key]) {
                if (engagementStatusMap[key]) {
                    delete engagementStatusMap[key];
                }
            }
        }
        loggerObj.debugLog("map for spinner is", JSON.stringify(revstatusMap), JSON.stringify(engagementStatusMap));
    };

    // sum of the values of the rev
    this.getSumOfRevStatus = function () {
        var sum = 0;
        for (var key in revstatusMap) {
            sum += revstatusMap[key];
        }
        return sum;
    };

    // sum of the values of the engage
    this.getSumOfEngagementStatus = function () {
        var sum = 0;
        for (var key in engagementStatusMap) {
            sum += engagementStatusMap[key];
        }
        return sum;
    };

    // return bucket in which the user falls
    this.getBucketNumberForUser = function () {
        var days_eligibility = runtimeObj.getSpinnerFirstDays();
        var spinner_rev = runtimeObj.getSpinnerRevEngagement();
        var spinner_engage = runtimeObj.getSpinnerPuzzleEngagement();

        if (gameObj.getIsPayer() || subscriptionsDTCObj.getIsSubscriptionActive()) {
            return 10;
        }

        if (utilObj.getCurrentTimestamp() - userObj.getCreated() <= days_eligibility * constantsObj.SECONDS_IN_DAY) {
            return 9;
        }

        var revValue = spinnerObj.getSumOfRevStatus();
        var engagevalue = spinnerObj.getSumOfEngagementStatus();

        if (revValue >= spinner_rev) {
            // active
            if (engagevalue >= spinner_engage) {
                return 1;
            } else if (engagevalue > 0 && engagevalue < spinner_engage) {
                return 4;
            } else {
                return 7;
            }
        } else if (revValue > 0 && revValue < spinner_rev) {
            // dormant
            if (engagevalue >= spinner_engage) {
                return 2;
            } else if (engagevalue > 0 && engagevalue < spinner_engage) {
                return 5;
            } else {
                return 8;
            }
        } else {
            // Never
            if (engagevalue >= spinner_engage) {
                return 3;
            } else if (engagevalue > 0 && engagevalue < spinner_engage) {
                return 6;
            } else {
                return 9;
            }
        }
    };

    // returns the reward to the user
    this.getRewardToGiveToUser = function () {
        var bucketForUser = spinnerObj.getBucketNumberForUser();
        var reward_prob = runtimeObj.getSpinnerRewardProb();
        var splitStrArr = reward_prob.split("_");
        var startIndex = bucketForUser * 6 - 6;
        var reward_r1, reward_r2, reward_r3, reward_r4, reward_r5, reward_r6 = 0;

        // as the runtime field will be edited now so, having a extra check with try
        try {
            reward_r1 = parseInt(splitStrArr[startIndex]);
            reward_r2 = parseInt(splitStrArr[startIndex + 1]);
            reward_r3 = parseInt(splitStrArr[startIndex + 2]);
            reward_r4 = parseInt(splitStrArr[startIndex + 3]);
            reward_r5 = parseInt(splitStrArr[startIndex + 4]);
            reward_r6 = parseInt(splitStrArr[startIndex + 5]);
        } catch(e) {
            reward_r1 = 50;
            reward_r2 = 3;
            reward_r3 = 1;
            reward_r4 = 35;
            reward_r5 = 10;
            reward_r6 = 1;
        }

        var arrForReward = [reward_r1, reward_r2, reward_r3, reward_r4, reward_r5, reward_r6];

        // frenzy edge cases
        var checkIfFrenzyNotAvailable = spinnerObj.checkIfAllFrenzyCompleted();
        var date1 = gameObj.getTodaysPuzzleDate();
        var id = utilObj.getIdForDate(date1);
        var isTodaysPuzzle = (gameLayer.crosswordDate == gameObj.getIdForTodaysPuzzle() || (gameLayer.crosswordDate == gameObj.getD0Id() && id == gameObj.getInstallDay())) || frenzyObj.checkAndUpdateFrenzyActive(); // added condition for the frenzy purchase check for frenzy in archive
        if (checkIfFrenzyNotAvailable || !isTodaysPuzzle || mapFrenzyWordPuzzle[gameLayer.crosswordDate] || !spinnerObj.checkfrenzyCondition()) {
            arrForReward[3] = 0
        }

        // disc. hint edge cases
        if (mapDiscountHintPuzzle[gameLayer.crosswordDate]) {
            arrForReward[4] = 0
        }
        var sum = 0;
        for (var i in arrForReward) {
            sum += arrForReward[i];
        }
        var rand_num = utilObj.getRandomNumberinRange(1, sum);
        var reward = 1;
        sum = 0;

        for (var i in arrForReward) {
            sum += arrForReward[i];
            if (rand_num <= sum) {
                reward = parseInt(i) + 1;
                break;
            }
        }
        loggerObj.debugLog("reward is", bucketForUser, "cqwwcwc", reward, "wefew", rand_num, "scsc", arrForReward);
        return reward;
    };

    this.checkfrenzyCondition = function () {
        if(!frenzyObj.checkAndUpdateFrenzyActive()){
            return false;
        }
        return true;
    };

    this.spinnerSurfacingLogic = function () {
        utilObj.setVisible(gameLayer.toggleGridIcon , false);
        utilObj.setVisible(gameLayer.bgCircle , true);
        utilObj.setVisible(gameLayer.bgCircle1 , true);
        utilObj.setVisible(gameLayer.toggleGridIcon1, true);
        utilObj.setVisible(gameLayer.arrowSprite1 , false);
        var spinState = spinnerObj.getSpinState();
        var isMiniOrMidi = gameObj.checkForMiniOrMidi(gameLayer.crosswordDate);
        var isTimerRunning = gameObj.getIsTimerRunning();
        var spinnerTimeElapsed = gameObj.getSpinnerTimeElapsed();
        var params = {
            icon : gameLayer.circle,
            label : gameLayer.circleText,
            animCircle : gameLayer.circleAnim ,
        };
        if (spinState == constantsObj.FREE_SPINS_AVAILABLE) {
            utilObj.setVisible(gameLayer.circle, true);
            utilObj.setVisible(gameLayer.circleText, true);
            utilObj.setVisible(gameLayer.freeVideoBg , false);
            utilObj.setVisible(gameLayer.freeVideoLabel , false);
            if( !gameLayer.spinnerAnimShown  &&  (curScreen == constantsObj.SCREEN_GAME && !isMiniOrMidi)  && spinnerTimeElapsed > 5 ) {
                spinnerObj.animateSpinnerNotif(gameLayer, params);
            }
            if (utilObj.isNodeValid(gameLayer.circleText)) {
                gameLayer.circleText.setString(economyObj.getSpinCount() + "");
            }
            //utilObj.setVisible(gameLayer.w2eIcon, false);
        } else if (spinState == constantsObj.SPIN_BY_VIDEO) {
            utilObj.setVisible(gameLayer.circle, false);
            utilObj.setVisible(gameLayer.circleText, false);
            utilObj.setVisible(gameLayer.circleAnim , false);
            utilObj.setVisible(gameLayer.freeVideoBg , true);
            utilObj.setVisible(gameLayer.freeVideoLabel , true);

            // if( !gameLayer.spinnerAnimShown  && (curScreen == constantsObj.SCREEN_GAME && !isMiniOrMidi)  && spinnerTimeElapsed > 5 ) {
            //     spinnerObj.animateSpinnerNotif(gameLayer, params);
            // }
            // if (utilObj.isNodeValid(gameLayer.circleText)) {
            //     gameLayer.circleText.setString("");
            // }
            //utilObj.setVisible(gameLayer.w2eIcon, true);

        } else if (spinState == constantsObj.SPIN_COOLDOWN) {
            utilObj.setVisible(gameLayer.circle, false);
            utilObj.setVisible(gameLayer.circleText, false);
            utilObj.setVisible(gameLayer.circleAnim , false);
            utilObj.setVisible(gameLayer.freeVideoBg , false);
            utilObj.setVisible(gameLayer.freeVideoLabel , false);
            // if (utilObj.isNodeValid(gameLayer.circleText)) {
            //     gameLayer.circleText.setString(0 + "");
            // }
            // if (utilObj.isNodeValid(gameLayer.circleText)) {
            //     gameLayer.circleText.setString("+");
            // }
           // utilObj.setVisible(gameLayer.w2eIcon, false);
        }

    };

    // animate the nudge for the spinner jewel notif
    this.animateSpinnerNotif = function (sender, params) {
        var icon = params.icon;
        var label = params.label;
        var animCircle = params.animCircle;
        gameLayer.spinnerAnimShown = true;
        utilObj.setVisible(animCircle, true);

        if (!utilObj.isNodeValid(icon) || !utilObj.isNodeValid(label) || !utilObj.isNodeValid(animCircle)) {
            return;
        }

        var scalex = icon.getScaleX();
        var scaleY = icon.getScaleY();

        var scaleLabelX = label.getScaleX();
        var scaleLabelY = label.getScaleY();

        var scaleUp = utilObj.getScaleTo(0.4, scalex * 1.2 , scaleY * 1.2);
        var scaleDown = utilObj.getScaleTo(0.4, scalex , scaleY);

        var scaleUp1 = animObj.getScaleDifferentTo(0.4, scaleLabelX * 1.2, scaleLabelY * 1.2);
        var scaleDown1 = animObj.getScaleDifferentTo(0.4, scaleLabelX, scaleLabelY);

        var scaleCircle = animObj.getScaleDifferentTo(0.6, scalex * 1.6, scaleY * 1.6);
        var fadeOut = utilObj.getFadeOut(0.6);
        var delay = utilObj.getDelayTime(0.78);
        var spawn = cc.Spawn([scaleCircle, fadeOut]);
        var scaleBack = animObj.getScaleDifferentTo(0.01, scalex, scaleY);
        var fadeIn = utilObj.getFadeIn(0.01);
        var seq = [scaleBack,fadeIn];

        var delay1 = utilObj.getDelayTime(1.2);
        var delay2 = utilObj.getDelayTime(0.6);

        var spawn1 = cc.Spawn([delay1, utilObj.getSequence(seq) ]);

        var animSeq = [scaleUp, scaleDown, delay1];
        var animSeq1 = [scaleUp1, scaleDown1, delay1];
        var animSeq2 = [delay,scaleBack,fadeIn, spawn, delay2];

        animObj.runAction(icon, utilObj.getSequence(animSeq));
        animObj.runAction(label, utilObj.getSequence(animSeq1));
        animObj.runAction(animCircle, utilObj.getSequence(animSeq2));
    };



    this.getSpinState = function () {
        var spinsAvailable = economyObj.getSpinCount();
        if (spinsAvailable > 0) {
            return constantsObj.FREE_SPINS_AVAILABLE;
        } else if (spinsAvailable <= 0) {
             if (adsObj.checkRewardedVideoAvailable() && newSpinnerPopup.getIsVideoAvailable()) {
                return constantsObj.SPIN_BY_VIDEO;
            } else {
                return constantsObj.SPIN_COOLDOWN;
            }
        }
    };

    this.getSpinnerReward = function () {
        newSpinnerPopup.show();
        trackObj.trackCounter(constantsObj.TRACK_PUZZLE, gameObj.getCurPuzzleTracking(), gameObj.getTrackingPuzzleTheme(), constantsObj.TRACK_SPINNER, constantsObj.TRACK_CLICK, 1, "", "", "");
    };


    this.checkIfAllFrenzyCompleted = function () {
        var puzzleMap = gameLayer.puzzleMap;
        var MapForFrenzyWordTile = gameObj.getMapForFrenzyWordTile();
        var wordsArr = [];
        loggerObj.debugLog("frenzy v2 - check for the clue number here3 ",frenzyV2Obj.conditionForFrenzyV2Type2());
        if (frenzyV2Obj.conditionForFrenzyV2Type2()) {
            var maxReveal = frenzyObj.getMaxRevealCount();
            loggerObj.debugLog("frenzy v2 - checkIfAllFrenzyCompleted condition to give frenzt by spinner ",frenzyV2Obj.checkFrenzyAvailableForSpinner(), maxReveal, gameLayer.frenzyRevealCount);
            if (gameLayer.frenzyRevealCount >= maxReveal) {
                return true;
            }
            if (frenzyV2Obj.checkFrenzyAvailableForSpinner()) {
                return false;
            }
        } else {
            if (frenzyObj.getFrenzyMasteryAvg() < 2) {
                wordsArr = gameObj.getFrenzyEasyWordList();
            } else {
                wordsArr = gameObj.getFrenzyHardWordList();
            }

            if (frenzyV2Obj.getExperimentAllocationFrenzyV2() == constantsObj.EXP_VAR_1) {

                var FRENZY_HARD = "FrenzyHard";
                var FRENZY_MEDIUM = "FrenzyMedium";
                var FRENZY_EASY = "FrenzyEasy";

                var frenzyBucket = frenzyV2Obj.getPuzzleLevelBasedBuckets(gameLayer.crosswordDate);
                if (frenzyBucket == FRENZY_EASY) {
                    wordsArr = gameObj.getFrenzyV2EasyList();
                } else if (frenzyBucket == FRENZY_MEDIUM) {
                    wordsArr = gameObj.getFrenzyV2MediumList();
                } else {
                    wordsArr = gameObj.getFrenzyV2HardList();
                }
            }

            if (wordsArr == undefined || wordsArr == null) {
                wordsArr = [];
            }
            var answerGrid = gameObj.getAnswerGrid();
            var attemptedGrid = gameObj.getAttemptedGrid();

            for (var w = 0; w < wordsArr.length; w++) {
                if (puzzleMap[wordsArr[w]]) {
                    // loggerObj.debugLog("Map for the word is", wordsArr[w], JSON.stringify(MapForFrenzyWordTile[wordsArr[w]]));
                    if (MapForFrenzyWordTile[wordsArr[w]]) {
                        for (var i = MapForFrenzyWordTile[wordsArr[w]].startX; i <= MapForFrenzyWordTile[wordsArr[w]].endX; i++) {
                            for (var j = MapForFrenzyWordTile[wordsArr[w]].startY; j <= MapForFrenzyWordTile[wordsArr[w]].endY; j++) {
                                // loggerObj.debugLog("Map for the word is 123123");
                                if (gameLayer.gridTiles != null && gameLayer.gridTiles[j][i]) {
                                    if (attemptedGrid[j] && attemptedGrid[j][i] && !(attemptedGrid[j][i] >= "A" && attemptedGrid[j][i] <= "Z")) {
                                        return false;
                                    }
                                    // loggerObj.debugLog("Map for the word is123", gameObj.getAttemptedTile(j ,i));
                                }
                            }
                        }
                    }
                }
            }
        }

        return true;
    };


    // called when frenzy reward is given
    this.selectAndRevealFrenzyWord = function () {
        var puzzleMap = gameLayer.puzzleMap;
        var MapForFrenzyWordTile = gameObj.getMapForFrenzyWordTile();
        var wordsArr = [];

        if (frenzyV2Obj.conditionForFrenzyV2Type2()) {
            wordsArr = frenzyV2Obj.getFrenzyWordForSpinner();
        } else {
            if (frenzyObj.getFrenzyMasteryAvg() < 2) {
                wordsArr = gameObj.getFrenzyEasyWordList();
            } else {
                wordsArr = gameObj.getFrenzyHardWordList();
            }

            if (frenzyV2Obj.getExperimentAllocationFrenzyV2() == constantsObj.EXP_VAR_1) {

                var FRENZY_HARD = "FrenzyHard";
                var FRENZY_MEDIUM = "FrenzyMedium";
                var FRENZY_EASY = "FrenzyEasy";

                var frenzyBucket = frenzyV2Obj.getPuzzleLevelBasedBuckets(gameLayer.crosswordDate);
                if (frenzyBucket == FRENZY_EASY) {
                    wordsArr = gameObj.getFrenzyV2EasyList();
                } else if (frenzyBucket == FRENZY_MEDIUM) {
                    wordsArr = gameObj.getFrenzyV2MediumList();
                } else {
                    wordsArr = gameObj.getFrenzyV2HardList();
                }
            }
        }
        loggerObj.debugLog("frenzy v2 - check for the clue number value3 ", utilObj.getStringifiedJson(wordsArr));

        if (wordsArr == undefined || wordsArr == null) {
            wordsArr = [];
        }
        var wordsArrToSelect = [];
        var answerGrid = gameObj.getAnswerGrid();
        var attemptedGrid = gameObj.getAttemptedGrid();

        var flag = false;
        for (var w = 0; w < wordsArr.length; w++) {
            if (puzzleMap[wordsArr[w]]) {
                // loggerObj.debugLog("Map for the word is", wordsArr[w], JSON.stringify(MapForFrenzyWordTile[wordsArr[w]]));
                if (MapForFrenzyWordTile[wordsArr[w]]) {
                    flag = false;
                    for (var i = MapForFrenzyWordTile[wordsArr[w]].startX; i <= MapForFrenzyWordTile[wordsArr[w]].endX; i++) {
                        for (var j = MapForFrenzyWordTile[wordsArr[w]].startY; j <= MapForFrenzyWordTile[wordsArr[w]].endY; j++) {
                            // loggerObj.debugLog("Map for the word is 123123");
                            if (gameLayer.gridTiles != null && gameLayer.gridTiles[j] != undefined && gameLayer.gridTiles[j][i] != undefined && gameLayer.gridTiles[j][i]) {
                                if (attemptedGrid[j] && attemptedGrid[j][i] && !(attemptedGrid[j][i] >= "A" && attemptedGrid[j][i] <= "Z")) {
                                    flag = true;
                                }
                                // loggerObj.debugLog("Map for the word is123", gameObj.getAttemptedTile(j ,i));
                            }
                        }
                    }
                    if (flag)
                        wordsArrToSelect.push(wordsArr[w]);
                }
            }
        }

        wordsArrToSelect = utilObj.shuffleArray(wordsArrToSelect);
        mapFrenzyWordPuzzle[gameLayer.crosswordDate] = wordsArrToSelect[0];

        // returns the first tile where the reward is given
        // position will be respect to the gridBg
        if (MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]]) {
            var mapForWord = MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]];
            for (var i = mapForWord.startX; i <= mapForWord.endX; i++) {
                for (var j = mapForWord.startY; j <= mapForWord.endY; j++) {
                    if (gameLayer.gridTiles != null && gameLayer.gridTiles[j] != undefined && gameLayer.gridTiles[j][i] != undefined && gameLayer.gridTiles[j][i]) {
                        if (frenzyV2Obj.conditionForFrenzyV2Type2()) {
                            frenzyV2Obj.setFrenzyWordsCountForPuzzles(gameLayer.crosswordDate);
                        }
                        trackObj.trackCounter(constantsObj.TRACK_PUZZLE, gameObj.getCurPuzzleTracking(), gameObj.getTrackingPuzzleTheme(), "frenzy_spinner", mapFrenzyWordPuzzle[gameLayer.crosswordDate], "highlight", "", 1, "");
                        return gameLayer.gridTiles[j][i];
                    }
                }
            }
        }
        return null;
    };

    // color frenzy tile with anim on giving reward
    this.colorFrenzyTileWithAnim = function () {
        var MapForFrenzyWordTile = gameObj.getMapForFrenzyWordTile();
        var colorTo = new cc.TintTo(0.1, 255, 184, 212);
        var totalDelay = 0;
        var delayMul = 0.1;
        var counter = 0;
        if (MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]]) {
            var mapForWord = MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]];
            for (var i = mapForWord.startX; i <= mapForWord.endX; i++) {
                for (var j = mapForWord.startY; j <= mapForWord.endY; j++) {
                    if (gameLayer.gridTiles != null && gameLayer.gridTiles[j] != undefined && gameLayer.gridTiles[j][i] != undefined && gameLayer.gridTiles[j][i]) {
                        var animArr = [];
                        totalDelay += ((delayMul * counter));
                        var delay = utilObj.getDelayTime(delayMul * counter);
                        counter++;
                        animArr.push(delay);
                        //gameLayer.gridTiles[j][i].setColor(constantsObj.COLOR_FRENZY_SPINNER1);
                        animArr.push(colorTo);
                        animObj.runAction(gameLayer.gridTiles[j][i], utilObj.getSequence(animArr));
                    }
                }
            }
            var fadeOut = utilObj.getFadeOut(totalDelay);
            animObj.runAction(transBgLayer.spinResultIconGlow, fadeOut);
            utilObj.delayedCallback(transBgLayer, transBgLayer.hide, totalDelay, null, false);
            utilObj.delayedCallback(gameLayer, gameLayer.setAnimRunningValue, totalDelay, null, false);
        }
    };


    // called from game layer on show and hide
    // doesn't get called on giving reward
    this.checkAndColorFrenzyTile = function () {
        var MapForFrenzyWordTile = gameObj.getMapForFrenzyWordTile();
        if (MapForFrenzyWordTile == null) {
            gameLayer.getPuzzleMap();
            MapForFrenzyWordTile = gameObj.getMapForFrenzyWordTile();
        }

        var answerGrid = gameObj.getAnswerGrid();
        var attemptedGrid = gameObj.getAttemptedGrid();
        loggerObj.debugLog("Map for the word ", JSON.stringify(mapFrenzyWordPuzzle));

        if (MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]]) {
            var mapForWord = MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]];
            var answerWord = "";
            var attemptedWord = "";
            for (var i = mapForWord.startX; i <= mapForWord.endX; i++) {
                for (var j = mapForWord.startY; j <= mapForWord.endY; j++) {
                    if (answerGrid[j] && answerGrid[j][i]) {
                        answerWord += answerGrid[j][i];
                    }
                    if (attemptedGrid[j] && attemptedGrid[j][i]) {
                        attemptedWord += attemptedGrid[j][i];
                    }
                }
            }
            if (answerWord != attemptedWord) {
                for (var i = mapForWord.startX; i <= mapForWord.endX; i++) {
                    for (var j = mapForWord.startY; j <= mapForWord.endY; j++) {
                        if (gameLayer.gridTiles != null && gameLayer.gridTiles[j] != undefined && gameLayer.gridTiles[j][i] != undefined && gameLayer.gridTiles[j][i]) {
                            gameLayer.gridTiles[j][i].setColor(constantsObj.COLOR_FRENZY_SPINNER1);
                        }
                    }
                }
            }
        }
    };

    this.checkIfCurrentWordToColorFrenzy = function (x, y, isAcross, byPassCompleteCheck) {
        var answerWord = "";
        var l = x;
        var m = y;
        var maX = gameLayer.gridWidth;
        var maY = gameLayer.gridHeight;

        var answerGrid = gameObj.getAnswerGrid();
        var attemptedGrid = gameObj.getAttemptedGrid();

        var MapForFrenzyWordTile = gameObj.getMapForFrenzyWordTile();
        if (MapForFrenzyWordTile == null) {
            gameLayer.getPuzzleMap();
            MapForFrenzyWordTile = gameObj.getMapForFrenzyWordTile();
        }


        // across
        while (l >= 0 && l < maX && answerGrid[l] && answerGrid[l][m] && answerGrid[l][m] >= "A" && answerGrid[l][m] <= "Z") {
            answerWord = answerGrid[l][m] + answerWord;
            l--;
        }
        l = x + 1;
        m = y;
        while (l >= 0 && l < maX && answerGrid[l] && answerGrid[l][m] && answerGrid[l][m] >= "A" && answerGrid[l][m] <= "Z") {
            answerWord = answerWord + answerGrid[l][m];
            l++;
        }

        if (MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]]) {
            var mapForWord = MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]];
            var answerWord1 = "";
            var attemptedWord = "";
            for (var i = mapForWord.startX; i <= mapForWord.endX; i++) {
                for (var j = mapForWord.startY; j <= mapForWord.endY; j++) {
                    if (answerGrid[j] && answerGrid[j][i]) {
                        answerWord1 += answerGrid[j][i];
                    } else {
                        return false;
                    }
                    if (attemptedGrid[j] && attemptedGrid[j][i]) {
                        attemptedWord += attemptedGrid[j][i];
                    } else {
                        return false;
                    }
                }
            }
            if (byPassCompleteCheck) {
                if (answerWord == answerWord1) {
                    return true;
                }
            } else if (answerWord1 != attemptedWord && answerWord == answerWord1) {
                if (isAcross) {
                    return true;
                }
            }
        }


        // downwards
        answerWord = "";
        l = x;
        m = y;

        while (m >= 0 && m < maY && answerGrid[l] && answerGrid[l][m] && answerGrid[l][m] >= "A" && answerGrid[l][m] <= "Z") {
            answerWord = answerGrid[l][m] + answerWord;
            m--;
        }
        l = x;
        m = y + 1;
        while (m >= 0 && m < maY && answerGrid[l] && answerGrid[l][m] && answerGrid[l][m] >= "A" && answerGrid[l][m] <= "Z") {
            answerWord = answerWord + answerGrid[l][m];
            m++;
        }

        if (MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]]) {
            var mapForWord = MapForFrenzyWordTile[mapFrenzyWordPuzzle[gameLayer.crosswordDate]];
            var answerWord1 = "";
            var attemptedWord = "";
            for (var i = mapForWord.startX; i <= mapForWord.endX; i++) {
                for (var j = mapForWord.startY; j <= mapForWord.endY; j++) {
                    if (answerGrid[j] && answerGrid[j][i]) {
                        answerWord1 += answerGrid[j][i];
                    } else {
                        return false;
                    }
                    if (attemptedGrid[j] && attemptedGrid[j][i]) {
                        attemptedWord += attemptedGrid[j][i];
                    } else {
                        return false;
                    }
                }
            }
            if (byPassCompleteCheck) {
                if (answerWord == answerWord1) {
                    return true;
                }
            } else if (answerWord1 != attemptedWord && answerWord == answerWord1) {
                if (!isAcross) {
                    return true;
                }
            }
        }
        return false;
    };

    this.freeHintSelection = function () {
        // if(userObj.getIsPayer() || subscriptionsDTCObj.getIsSubscriptionActive()){
        //     return constantsObj.SHOW_WORD_SPINNER1;
        // }
        var randVal = utilObj.getRandomIntInRange(0, 2);
        if (randVal > 0) {
            return constantsObj.SHOW_WORD_SPINNER1;
        } else {
            return constantsObj.SHOW_10_LETTERS_SPINNER1;
        }
    };

    this.spinnerHintDisc = function () {
        if(userObj.getIsPayer() || subscriptionsDTCObj.getIsSubscriptionActive()){
            return constantsObj.DISCOUNT_ON_SHOW_WORD;
        }
        var randVal = utilObj.getRandomIntInRange(0, 2);
        if (randVal > 0) {
            return constantsObj.DISCOUNT_ON_SHOW_LETTERS;
        } else {
            return constantsObj.DISCOUNT_ON_SHOW_WORD;
        }
    };

    this.isSpinByVideoAvailable = function () {
        var videoCap = runtimeObj.getSpinnerVideoCap();
        if(userObj.getIsPayer() || subscriptionsDTCObj.getIsSubscriptionActive()){
            videoCap = runtimeObj.getSpinnerVideoCapForPayer();
        }
        var watchVideoAvailable = gameObj.getSpinnerVideoWatchCount();
        var curTime = utilObj.getCurrentTimestamp();
        var lastVideoTime = gameObj.getSpinnerLastVideoTime();
        var videoRefillTime = runtimeObj.getSpinnerVideoRefillTime();
        var diffTime = curTime - lastVideoTime;
        var noOfVideoGrants = Math.floor(diffTime / videoRefillTime);
        loggerObj.debugLog("video values", videoCap, watchVideoAvailable, diffTime, noOfVideoGrants, watchVideoAvailable + noOfVideoGrants <= videoCap);
        if (watchVideoAvailable + noOfVideoGrants <= videoCap) {
            gameObj.setSpinnerVideoWatchCount(watchVideoAvailable + noOfVideoGrants);
        } else {
            gameObj.setSpinnerVideoWatchCount(videoCap);
        }
        if (gameObj.getSpinnerVideoWatchCount() > 0) {
            return true;
        }
        return false;
    };

    this.isExtraVideoOptionAvailable = function () {
        var secondLastVideoTime = gameObj.getSpinnerSecondLastVideoTime();
        var lastVideoTime = gameObj.getSpinnerLastVideoTime();
        var diff = lastVideoTime - secondLastVideoTime;
        if (diff < 3600 && diff != 0 && gameObj.getIsPossibleToReturn()) {
            return true;
        } else {
            return false;
        }
    };


    // give discounts on +10 letters to user for the discount hint reward
    this.giveDiscountHintReward10Letters = function () {
        mapDiscountHintPuzzle[gameLayer.crosswordDate] = 1;
        spinnerObj.setStringForReward10Letters();
    };

    // sets the string value for the reveal letters price on game screen in hint section
    this.setStringForReward10Letters = function () {
        var revealLettersCount = spinnerObj.getRevealLetterCost();
        var revealLettersCountStr = "" + revealLettersCount;
        if (utilObj.isNodeValid(gameLayer.numDiscCoinsRL)) {
            gameLayer.numDiscCoinsRL.setString(revealLettersCountStr);
            if(economyObj.getFreeRevealLetters() != 0){
                utilObj.setVisible(gameLayer.numDiscCoinsRL , false);
                utilObj.setVisible(gameLayer.redCutLineRL , false);
            }else{
                utilObj.setVisible(gameLayer.numDiscCoinsRL , true);
                utilObj.setVisible(gameLayer.redCutLineRL , true);
            }

            // loggerObj.debugLog("spinner label for hint1", gameLayer.numCoinsRL_1.getString());
        }
    };

    // give discounts on show word to user for the discount hint reward
    this.giveDiscountHintRewardRevealWord = function () {
        mapDiscountHintPuzzle[gameLayer.crosswordDate] = 2;
        spinnerObj.setStringForRewardWord();
    };

    // sets the string value for the reveal word price on game screen in hint section
    this.setStringForRewardWord = function () {
        var revealWordCost = spinnerObj.getRevealWordCost();
        var revealWordCostStr = revealWordCost + "";
        if (utilObj.isNodeValid(gameLayer.numDiscCoinsRW)) {
            gameLayer.numDiscCoinsRW.setString(revealWordCostStr);
            if(economyObj.getFreeRevealWord() != 0 ){
                utilObj.setVisible(gameLayer.numDiscCoinsRW , false);
                utilObj.setVisible(gameLayer.redCutLineRW , false);
            }else{
                utilObj.setVisible(gameLayer.numDiscCoinsRW , true);
                utilObj.setVisible(gameLayer.redCutLineRW , true);
            }

            // loggerObj.debugLog("spinner label for hint2", gameLayer.numCoinsRW.getString());
        }
    };

    // called after the hint options is rendered on the screen
    this.changeLabelsForHint = function () {
        loggerObj.debugLog("spinner hint map", JSON.stringify(mapDiscountHintPuzzle));
        if (mapDiscountHintPuzzle[gameLayer.crosswordDate]) {
            if (mapDiscountHintPuzzle[gameLayer.crosswordDate] == 1) {
                spinnerObj.setStringForReward10Letters();
            } else if (mapDiscountHintPuzzle[gameLayer.crosswordDate] == 2) {
                spinnerObj.setStringForRewardWord();
            }
        }
    };

    // gets the discounted price for the reveal word hint
    this.getRevealWordCost = function () {
        var discount = runtimeObj.getSpinnerHintDisc();
        var revealWordCost = economyObj.getRevealWordCost();
        if (mapDiscountHintPuzzle[gameLayer.crosswordDate]) {
            if (mapDiscountHintPuzzle[gameLayer.crosswordDate] == 2) {
                revealWordCost = revealWordCost * ((100 - discount) / 100);
            }
        }
        return revealWordCost;
    };


    // gets the discounted price for the reveal letters hint
    this.getRevealLetterCost = function () {
        var discount = runtimeObj.getSpinnerHintDisc();
        var revealLettersCount = economyObj.getRevealLetterCost();
        if (mapDiscountHintPuzzle[gameLayer.crosswordDate]) {
            if (mapDiscountHintPuzzle[gameLayer.crosswordDate] == 1) {
                revealLettersCount = revealLettersCount * ((100 - discount) / 100);
            }
        }
        return revealLettersCount;
    };


    // checks if eligibility for the discounted hint is satisfies for reveal word
    // reveal word should be granted with the discounted price for the puzzle
    this.checkRevealWordDiscountSpinner = function () {
        if (mapDiscountHintPuzzle[gameLayer.crosswordDate]) {
            if (mapDiscountHintPuzzle[gameLayer.crosswordDate] == 2) {
                return true;
            }
        }
        return false;
    };

    // checks if eligibility for the discounted hint satisfies for reveal word
    // reveal word should be granted with the discounted price for the puzzle
    this.checkReveallettersDiscountSpinner = function () {
        if (mapDiscountHintPuzzle[gameLayer.crosswordDate]) {
            if (mapDiscountHintPuzzle[gameLayer.crosswordDate] == 1) {
                return true;
            }
        }
        return false;
    };

    // trigger free reveal 10 letter reward
    this.giveFreeRevealLettersReward = function () {
        gameObj.activateRevealLetters(true, 10);
        trackObj.trackCounter(constantsObj.TRACK_PUZZLE, gameObj.getCurPuzzleTracking(), gameObj.getTrackingPuzzleTheme(), "use_hint", "reveal_10", "spin_free", gameObj.getPuzzleSpeciesTracking(), "", "");
        trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_EARN, "spin_reveal_10", "", economyObj.getCash(), 10, gameObj.getPuzzleSpeciesTracking(), "", "");
        trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_SPEND, "spin_reveal_10", "", economyObj.getCash(), -10, gameObj.getPuzzleSpeciesTracking(), "", "");
    };

    // trigger free reveal word reward
    this.giveFreeRevealWordReward = function () {
        gameObj.activateRevealWord(true);
        trackObj.trackCounter(constantsObj.TRACK_PUZZLE, gameObj.getCurPuzzleTracking(), gameObj.getTrackingPuzzleTheme(), "use_hint", "reveal_word", "spin_free", gameObj.getPuzzleSpeciesTracking(), "", "");
        trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_EARN, "spin_reveal_word", "", economyObj.getCash(), 10, gameObj.getPuzzleSpeciesTracking(), "", "");
        trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_SPEND, "spin_reveal_word", "", economyObj.getCash(), -10, gameObj.getPuzzleSpeciesTracking(), "", "");
    };

    this.callShineAnimation = function () {
        return;
    };


    this.animateShine = function (params) {
        var node = params.node;
        var xPos = params.xPos;
        utilObj.setVisible(node, true);
        if(!utilObj.isNodeValid(node)){
            return ;
        }
        //loggerObj.debugLog("shine Anim 3",xPos);
        utilObj.setPositionX(node , xPos);
        var width = params.width;
        var height = params.height;
        var move1 = new cc.MoveTo(1.15, node.getPositionX() + width + 200, node.getPositionY());

        animObj.runAction(node, move1);
    };

};