Subscriptions = function () {
    var NOT_SUBSCRIBED = 0;
    var SUBSCRIPTION_RECOVERED = 1; // A subscription was recovered from account hold.
    var SUBSCRIPTION_RENEWED = 2; // An active subscription was renewed.
    var SUBSCRIPTION_CANCELED = 3; // A subscription was either voluntarily or involuntarily cancelled. For voluntary cancellation, sent when the user cancels.
    var SUBSCRIPTION_PURCHASED = 4; // A new subscription was purchased.
    var SUBSCRIPTION_ON_HOLD = 5; // A subscription has entered account hold (if enabled).
    var SUBSCRIPTION_IN_GRACE_PERIOD = 6; // A subscription has entered grace period (if enabled).
    var SUBSCRIPTION_RESTARTED = 7; // User has reactivated their subscription from Play > Account > Subscriptions (requires opt-in for subscription restoration)
    var SUBSCRIPTION_CHANGE_RENEWAL_PREF = 8;

    var PKG_TYPE_SUB = 10;
    var curSubStatus = -1;

    var iosSubscriptions = ["in.crossy.daily_crossword.subscription.coindrip1", "in.crossy.daily_crossword.subscription.coindrip2", "in.crossy.daily_crossword.subscription.freearchive3", "in.crossy.daily_crossword.subscription.freearchive2"];
    var gplaySubscriptions = ["in.crossy.daily_crossword.subscription.coindrip1", "in.crossy.daily_crossword.subscription.coindrip2", "in.crossy.daily_crossword.subscription.freearchive3", "in.crossy.daily_crossword.subscription.freearchive2"];
    var MANAGE_URL_IOS = "https://buy.itunes.apple.com/WebObjects/MZFinance.woa/wa/manageSubscriptions";
    var MANAGE_URL_GPLAY = "https://play.google.com/store/account/subscriptions";
    // For more direct access in GPLAY
    // https://play.google.com/store/account/subscriptions?sku=your-sub-product-id&package=your-app-package

    var SUBSCRIPTIONS_FILE_JS = "wc_subscriptions_js.json";
    var SUBSCRIPTIONS_FILE = "wc_subscriptions.json";
    var SUBSCRIPTION_PREFIX = "in.crossy.daily_crossword.subscription.";

    var subscriptionsController = null;
    var SUBSCRIPTIONS_SYNC_INTERVAL = 1800;
    var subscriptionPkgs = null;
    var subscriptions = null;
    var lastSubscriptionLoad = 0;
    var loadDelay = 10;
    var lastSyncClient = 0;
    var activeOnClient = null; // This is populated as true or false after receiving the response from playstore/appstore regarding the status of the subscription - after each check for any pending payments.
    var subscriptionId = "";
    var activatedOnServer = false;
    var orderId = null;
    var isOffline = false;
    ////////////////////////////////////
    // Variables synced from server side
    var pkgId = "";
    var gid;
    var client;
    var active;
    var lastStatus;
    var lastUpdate;
    var lastSync;
    var clientStatus;
    var renewalCount;
    var totalRenewalCount;
    var joiningBonusGiven = false;
    ////////////////////////////////////

    this.init = function () {
        subscriptionsController = userObj.getLoginRestController();
        if (utilObj.isStaging()) {
            SUBSCRIPTIONS_SYNC_INTERVAL = 30;
        }
        load();
    };

    function load() {
        window.setTimeout(subscriptionsObj.loadFromFile, 5000);
        var subData = ls.getItem(SUBSCRIPTIONS_FILE_JS);
        if (subData == null || subData == "") {
            return;
        }
        var sub = JSON.parse(subData);
        if (sub.subscriptionId == undefined) {
            return;
        }
        subscriptionId = sub.subscriptionId;
        activatedOnServer = sub.activatedOnServer;
        activeOnClient = sub.activeOnClient;
        orderId = sub.orderId;
        if (sub.pkgId == undefined) {
            return;
        }
        pkgId = sub.pkgId;
        gid = sub.gid;
        client = sub.client;
        active = sub.active;
        lastStatus = sub.lastStatus;
        lastUpdate = sub.lastUpdate;
        lastSync = sub.lastSync;
        clientStatus = sub.clientStatus;
        renewalCount = sub.renewalCount;
        totalRenewalCount = sub.totalRenewalCount;
        joiningBonusGiven = sub.joiningBonusGiven;
        loggerObj.debugLog("Subscription loading data joining bonus, activeOnClient, server active status", joiningBonusGiven, activeOnClient, active);
        loggerObj.debugLog("Loaded Subscriptions data:", subData);
    }

    this.loadFromFile = function () {
        var curTimestamp = utilObj.getCurrentTimestamp();
        /*if (subscriptions != null) {
            if (curTimestamp - lastSubscriptionLoad < loadDelay) {
                loggerObj.debugLog("Subscriptions - Not fetching:", curTimestamp, lastSubscriptionLoad);
                return;
            }
        }*/
        var filePath = utilObj.getAbsFilePathV1(SUBSCRIPTIONS_FILE);
        loggerObj.debugLog("Subscriptions - Checking for file:", filePath);
        if (utilObj.fileExistsV1(filePath)) {
            lastSubscriptionLoad = curTimestamp;
            cc.loader.loadJson(filePath, function (error, data) {
                if (error == null) {
                    subscriptions = data;
                    loggerObj.debugLog("Subscriptions - data loaded as:", utilObj.getStringifiedJson(subscriptions));
                    if (utilObj.isIos()) {
                        var otid = subscriptions.InApp[subscriptions.InApp.length - 1].OriginalTransactionIdentifier;
                        var orderId = subscriptions.InApp[subscriptions.InApp.length - 1].TransactionIdentifier;
                        var pkgId = subscriptions.InApp[subscriptions.InApp.length - 1].ProductIdentifier;
                        loggerObj.debugLog("Subscriptions - Original transaction identifier:", otid);
                        if (pkgId.startsWith(SUBSCRIPTION_PREFIX)) {
                            subscriptionsObj.setSubscriptionId(otid);
                            subscriptionsObj.fireSubscriptionIdTracking(otid, pkgId);
                            subscriptionsObj.setOrderId(orderId, pkgId);
                            subscriptionsObj.checkAndUpdateIosSubscriptionStatus();
                        }
                    }
                } else {
                    loggerObj.debugLog("Subscriptions - Error when loading subscriptions:", error, data);
                }
            });
        }
    };

    this.save = function () {
        if (subscriptionId === "" || subscriptionId === undefined) {
            //loggerObj.debugLog("No need to save blank subscription info");
            return false;
        }
        var sub = {};
        sub.subscriptionId = subscriptionId;
        sub.activatedOnServer = activatedOnServer;
        sub.activeOnClient = activeOnClient;
        sub.orderId = orderId;
        if (pkgId != "") {
            sub.pkgId = pkgId;
            sub.gid = gid;
            sub.client = client;
            sub.active = active;
            sub.lastStatus = lastStatus;
            sub.lastUpdate = lastUpdate;
            sub.lastSync = lastSync;
            sub.clientStatus = clientStatus;
            sub.renewalCount = renewalCount;
            sub.totalRenewalCount = totalRenewalCount;
            sub.joiningBonusGiven = joiningBonusGiven;
        }
        loggerObj.debugLog("Subscription saving ", active, joiningBonusGiven);
        ls.setItem(SUBSCRIPTIONS_FILE_JS, utilObj.getStringifiedJson(sub));
    };

    this.setOrderId = function (oId, pkgId) {
        if (orderId != oId) {
            // order id changed
            orderId = oId;
            var pkgIdArr = pkgId.split(".");
            var newPkgIdStr = pkgIdArr[pkgIdArr.length - 2] + "." + pkgIdArr[pkgIdArr.length - 1];
            trackObj.trackCounter(constantsObj.TRACK_STORE, constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_GRANT, orderId + "", newPkgIdStr + "", "", gameObj.getPuzzleSpeciesTracking(), "1", "");
        }
    };

    this.setSubscriptionId = function (subId) {
        if (subscriptionId != subId) {
            activatedOnServer = false;
        }
        subscriptionId = subId;
        loggerObj.debugLog("Subscriptions - id set to:", subscriptionId);
    };

    this.setActiveOnClient = function (active) {
        activeOnClient = active;
        if (activatedOnServer) {
            // This is mainly to handle the case of re-subscription.
            activatedOnServer = active;
        }
        loggerObj.debugLog("Setting active status on client:" + active);
    };

    this.resetActivatedOnServer = function () {
        activatedOnServer = false;
    };

    this.syncToServer = function () {
        loggerObj.debugLog("Subscriptions - sync call start");
        if (userObj.getRefId() == "") {
            return;
        }
        if ((subscriptionId == "" || activeOnClient == null)) {
            if (!fbWrapperObj.isLoggedIn()) {
                return;
            }
            if (gameObj.getIsPayer()) {
                if (!gameObj.getIsJoiningBonusGiven() && !gameObj.getSubscriptionFreeTrialUsed()) {
                    return;
                }
            } else {
                if (!gameObj.getSubscriptionFreeTrialUsed()) {
                    return;
                }
            }
        }
        loggerObj.debugLog("Subscriptions - sync call");
        var curTimestamp = utilObj.getCurrentTimestamp();
        if (curTimestamp - lastSyncClient >= SUBSCRIPTIONS_SYNC_INTERVAL) {
            var subscriptionsData = userObj.getUserObj();
            subscriptionsData.subId = subscriptionId;
            subscriptionsData.activeOnClient = activeOnClient + "";
            subscriptionsData.renewalCount = renewalCount + "";
            subscriptionsData.joiningBonusGiven = joiningBonusGiven + "";
            subscriptionsController.sendHttpRequest("subs", "sync", subscriptionsData);
            lastSyncClient = curTimestamp;
        }
    };

    this.checkAndActivateSubscription = function () {

        if (subscriptionId == "" || userObj.getRefId() == "") {
            return;
        }
        if (activatedOnServer) {
            return;
        }
        loggerObj.debugLog("Subscriptions - activate Sub:", subscriptionId);
        var subscriptionsData = userObj.getUserObj();
        subscriptionsData.subId = subscriptionId;
        subscriptionsController.sendHttpRequest("subs", "activate", subscriptionsData);
    };

    this.respHandler = function (action, data, success) {
        loggerObj.debugLog("Subscriptions resp:", action, utilObj.getStringifiedJson(data));
        if (success == false && action != "activate") {
            subscriptionsObj.checkAndActivateSubscription();
            return;
        }
        if (action == "activate") {
            activatedOnServer = true;
        } else if (action == "sync") {
            try {
                var subsData = JSON.parse(data.sub);
                var statusBeforeSync = lastStatus;
                subscriptionId = subsData.s;
                pkgId = subsData.p;
                gid = subsData.g;
                client = subsData.cl;
                active = subsData.a;
                lastStatus = subsData.ls;
                lastUpdate = subsData.lu;
                lastSync = subsData.l;
                clientStatus = subsData.cs;
                loggerObj.debugLog("Subscription Renewal Count", renewalCount, subsData.c, active);
                // if (renewalCount < subsData.c) {
                //     // hiding subscribe popup, showing congrats popup and granting benefits other than joining bonus
                //     gameObj.grantOtherBonuses();
                // }
                var lastRenewalCount = renewalCount;
                renewalCount = subsData.c;
                totalRenewalCount = subsData.tc;
                subscriptionsObj.sendSubStatusTracking(lastRenewalCount, renewalCount, statusBeforeSync, lastStatus, pkgId);

                if (!joiningBonusGiven) {
                    loggerObj.debugLog("Subscription checking ->1", subsData.j);
                    if (subsData.j == true) {
                        // Just received info from server that joining bonus for this SubscriptionId has already been granted.
                        loggerObj.debugLog("Subscription getting info from server that joining bonus is already given");
                        joiningBonusGiven = true;
                    } else {
                        joiningBonusGiven = subscriptionsObj.checkAndGrantJoiningBonus(statusBeforeSync, lastStatus);
                        loggerObj.debugLog("Subscription checking if status changes and should joining bonus be given", joiningBonusGiven, statusBeforeSync, lastStatus);
                        if (joiningBonusGiven) {
                            if (renewalCount > 0) {
                                totalRenewalCount -= renewalCount;
                                if (totalRenewalCount < 0) {
                                    totalRenewalCount = 0;
                                }
                                renewalCount = 0;
                            }
                        }
                        lastSyncClient = 0; // Make sure the next sync happens quickly to update all these values to server.
                    }
                }
            } catch (e) {

            }
        } else {
            loggerObj.errorLog("Unknown action received from subscriptions server. Action:" + action + ";Data:" + data);
        }
        subscriptionsObj.save();
    };

    this.sendSubStatusTracking = function (lastCount, currentCount, lastStatus, currentStatus, pkgId) {

        if (currentStatus == SUBSCRIPTION_CANCELED) {
            // gameObj.setShouldGiveBurstHintDropOnStatusChange(true);
        }

        if (lastCount != currentCount || currentStatus != lastStatus) {
            var dollarPrice = subscriptionsObj.getPkgsPriceNumeric(pkgId);
            if (currentStatus == SUBSCRIPTION_CANCELED) {
                if (joiningBonusGiven) {
                    trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_PAID_END, "", economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                } else {
                    trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_FREE_TRIAL_END, "", economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                }
            } else if (lastCount != currentCount && currentStatus == SUBSCRIPTION_RENEWED) {
                if (joiningBonusGiven) {
                    trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_PURCHASE, "renewal", dollarPrice, economyObj.getCash(), "", "", "1", "");
                    trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_PAID_RENEWAL, dollarPrice, economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                } else {
                    trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_FREE_TRIAL_END, "", economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                    trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_PAID_START, dollarPrice, economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                }
            } else if (currentStatus == SUBSCRIPTION_PURCHASED && lastStatus == SUBSCRIPTION_CANCELED) {
                var sId = subscriptionId;
                if (utilObj.isPlayStore()) {
                    sId = sId.substr(0, 24);
                }
                // Triggering adjust event for subscription purchase
                // var adjustEventToFire = subscriptionsObj.getAdjustIdToFireForSubscription(pkgId);
                // analyticsObj.triggerAdjustEvent(adjustEventToFire);

                analyticsObj.triggerEvents(constantsObj.EVENT_TYPE_SUBSCRIPTION, pkgId);
                
                trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_PURCHASE, "re-subscription", dollarPrice,economyObj.getCash(), "", "", "1", "");
                trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PAID_RESTART, sId, dollarPrice, "", "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_PAID_RESTART, dollarPrice, economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
            }
        }
    };

    this.getJoiningBonusStatus = function () {
        return joiningBonusGiven;
    };

    this.checkAndGrantJoiningBonus = function (curStatus, updatedStatus) {
        if (joiningBonusGiven) {
            return joiningBonusGiven;
        }
        loggerObj.debugLog("Subscription grant joining bonus, status changes 1-> ", curStatus, updatedStatus);
        var shouldGiveBonus = false;

        if (updatedStatus == SUBSCRIPTION_RENEWED) {
            shouldGiveBonus = true;
        } else if (curStatus == updatedStatus) {
            shouldGiveBonus = false;
        } else if (curStatus == SUBSCRIPTION_CANCELED && updatedStatus == SUBSCRIPTION_PURCHASED) {
            shouldGiveBonus = true;
        }

        if (shouldGiveBonus) {
            loggerObj.debugLog("Subscription grant joining bonus, status changes 2-> ", curStatus, updatedStatus);
            subscriptionsObj.grantJoiningBonus();
        }
        return joiningBonusGiven;
    };

    this.grantJoiningBonus = function () {
        subscriptionsObj.setJoiningBonusGiven();
        subscriptionsDTCObj.grantJoiningBonus();
        // TODO: Grant all items for joining bonus - call a function in gameObj for the same.
        // This also marks a variable in user's game data saying joining bonus granted for this refId - which is synced to server
    };

    this.getAvailableSubscriptions = function () {
        if (subscriptionPkgs != null) {
            return subscriptionPkgs;
        }
        if (utilObj.isIos()) {
            subscriptionPkgs = iosSubscriptions;
        } else if (utilObj.isPlayStore()) {
            subscriptionPkgs = gplaySubscriptions;
        } else if (utilObj.isAmazonAppStore()) {

        }
        return subscriptionPkgs;
    };

    this.startPurchase = function (productIdentifier) {
        if (utilObj.isIos()) {
            jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_IOS + "Payments", "startPurchase:", productIdentifier);
        } else if (utilObj.isPlayStore()) {
            jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_ANDROID + "Payments", "startPurchase", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V",
                productIdentifier, constantsObj.TRACK_SUBSCRIPTION + "", "");
        }
        // else if (utilObj.isAmazonAppStore()) {
        //
        // }
    };

    this.checkAndUpdateIosSubscriptionStatus = function () {
        var inApp = subscriptions.InApp;
        var curTimestampMs = utilObj.getCurrentTimestampMs();
        var foundActiveSubscription = false;
        for (var purchase in inApp) {
            // var productIdentifier = availableSubs[product];
            var purchaseInfo = inApp[purchase];
            // “To check whether a purchase has been canceled by Apple Customer Support, look for the Cancellation Date field in the receipt.
            // If the field contains a date, regardless of the subscription’s expiration date, the purchase has been canceled. With respect to providing content or service, treat a canceled transaction the same as if no purchase had ever been made.”
            if (purchaseInfo.CancelDate != "") {
                continue;
            }
            var expDate = new Date(purchaseInfo.SubExpDate);
            if (expDate.getTime() - curTimestampMs > 0) {
                loggerObj.debugLog("Purchase date - ", purchaseInfo.SubExpDate, expDate, curTimestampMs, expDate.getTime(), expDate.getTime() - curTimestampMs);
                foundActiveSubscription = true;
                break;
            } else {
                loggerObj.debugLog("Purchase date inactive - ", purchaseInfo.SubExpDate, expDate, curTimestampMs, expDate.getTime(), expDate.getTime() - curTimestampMs);
            }
        }
        subscriptionsObj.setActiveOnClient(foundActiveSubscription);
    };

    // This is called from the native side in case of Android when subscription is inactive
    this.clientSubscriptionInactive = function () {
        loggerObj.debugLog("Subscription is inactive on client");
        // reset everything on client
        subscriptionsObj.setActiveOnClient(false);
    };

    // This gets called from native side when subscription is active
    this.checkAndGrantItems = function (purchaseInfo) {
        // This gets called multiple times, and not just the first purchase.

        if (utilObj.isPlayStore()) {
            var androidPurchaseObj = JSON.parse(purchaseInfo);
            var subId = androidPurchaseObj.purchaseToken;
            subId = subId.substr(0, 24);
            subscriptionsObj.fireSubscriptionIdTracking(subId, androidPurchaseObj.productId);
        }

        loggerObj.debugLog("Subscriptions - granting items for subscription:", purchaseInfo);
        if (utilObj.isIos()) {
            subscriptionsObj.loadFromFile();
            if (activeOnClient) {
                subscriptionsDTCObj.grantSubscriptionBenefits();
                subscriptionsObj.checkAndActivateSubscription();
            }
        } else if (utilObj.isPlayStore()) {
            var purchaseObj = JSON.parse(purchaseInfo);
            subscriptionsObj.setSubscriptionId(purchaseObj.purchaseToken);
            subscriptionsObj.setOrderId(purchaseObj.orderId, purchaseObj.productId);
            subscriptionsObj.setActiveOnClient(true);
            subscriptionsDTCObj.grantSubscriptionBenefits();
            loggerObj.debugLog("Subscription is active on client", subscriptionsObj.isSubscriptionActive());
            subscriptionsObj.checkAndActivateSubscription();
            // Gets called repeatedly with this info
            // {"productId":"in.playsimple.wordtrip.subscription.bursthint5","orderId":"GPA.3300-8695-2650-49341","purchaseToken":"pbnkbagogcoflandgbndbfjf.AO-J1Ozm_TuIjGFtEyICgPqvExq2TZBdg2D8L15K3XbQmU_5Mf_MwSKFcOMA_XiFQSjWbZIKIASrEmi2yQe1mf4CHe6e8-ubPI1tTv9mOJ_wwvafOBlb1F07hqtGIm02t1748ZlCQBhlSnxdI90Q3OJYum14gT1DIzSGDoe1rjFtxaBnnT0DTxY"}
        } else if (utilObj.isAmazonAppStore()) {

        }
        subscriptionsObj.save();
    };

    this.setJoiningBonusGiven = function () {
        joiningBonusGiven = true;
    };

    this.manageSubscriptions = function () {
        if (utilObj.isIos()) {
            utilObj.openUrl(MANAGE_URL_IOS);
        } else if (utilObj.isPlayStore()) {
            utilObj.openUrl(MANAGE_URL_GPLAY);
        } else if (utilObj.isAmazonAppStore()) {

        } else {

        }
    };

    this.isSubscriptionActive = function () {

        // this switch is for turning on subscription for users who are facing issues in production.
        if (gameObj.getSubscriptionSwitch()) {
            return true;
        }

        // Also use the lastUpdate and lastSync values stored to handle offline cases.
        // If the lastUpdate was more than 7+3=10 days back, then subscription benefits should disappear.
        var curTimestamp = utilObj.getCurrentTimestamp();
        if (lastUpdate != undefined && lastUpdate != 0) {

            // handling offline case, based on pkg's billing period
            var offlinePeriod = 368 * constantsObj.SECONDS_IN_DAY;
            var subPkgId = subscriptionsObj.getAvailableSubscriptions();
            if (pkgId == subPkgId[0]) {
                offlinePeriod = 10 * constantsObj.SECONDS_IN_DAY;
            }
            if (pkgId == subPkgId[1] || pkgId == subPkgId[2]) {
                offlinePeriod = 33 * constantsObj.SECONDS_IN_DAY;
            }
            // loggerObj.debugLog("SubscriptionOffline case",subPkgId[3],curTimestamp - lastUpdate > offlinePeriod, curTimestamp, lastUpdate, curTimestamp - lastUpdate, offlinePeriod, pkgId, utilObj.isOnline());
            if (curTimestamp - lastUpdate > offlinePeriod) {
                if (activeOnClient || (fbWrapperObj.isLoggedIn() && active)) {
                    isOffline = !utilObj.isOnline();
                }
                trackObj.trackCounter(constantsObj.TRACK_DEBUG, constantsObj.TRACK_SUBSCRIPTION, "offline_condition", isOffline, "", "", "", "1", "");
                return false;
            }
        }
        if (activeOnClient || (fbWrapperObj.isLoggedIn() && active)) {
            // Subscription active according to server or client.
            return true;
        }
        return false;
    };

    this.getActiveSubscriptionPkgId = function () {
        return pkgId;
    };

    this.isSubscriptionOffline = function () {
        return isOffline;
    };

    this.setSubscriptionFeatureSurfacingTime = function (createdTime) {
        var currentTime = utilObj.getCurrentTimestamp();
        var userActiveDay = (currentTime - createdTime) / constantsObj.SECONDS_IN_DAY;

        var startTime;
        if (userActiveDay >= 7) {
            startTime = currentTime;
        } else {
            startTime = currentTime + (7 - userActiveDay) * constantsObj.SECONDS_IN_DAY;
            var date = new Date(startTime * 1000);
            startTime = startTime - date.getHours() * 60 * 60 - date.getMinutes() * 60 - date.getSeconds();
        }
        gameObj.setSubscriptionsFeatureStart(startTime);
    };

    this.fireSubscriptionIdTracking = function (subId, pkgId) {
        if (!gameObj.getSubscriptionFreeTrialUsed()) {
            gameObj.setSubscriptionFreeTrialUsed(true);
            var dollarPrice = subscriptionsObj.getPkgsPriceNumeric(pkgId);
            if (runtimeObj.isSubscriptionFreeTrialActive()) {
                trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_PURCHASE, "freetrial", dollarPrice, economyObj.getCash(), "", "", "1", "");
                trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_FREE_TRIAL_START, "", economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_FREE_TRIAL_START, subId, dollarPrice, "", "", gameObj.getPuzzleSpeciesTracking(), "1", "");
            } else {
                trackObj.trackCounter(constantsObj.TRACK_CASH, constantsObj.TRACK_PURCHASE, "subscription", dollarPrice, economyObj.getCash(), "", "", "1", "");
                trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PURCHASE, constantsObj.TRACK_PAID_START, dollarPrice, economyObj.getCash(), "", gameObj.getPuzzleSpeciesTracking(), "1", "");
                trackObj.trackCounter(constantsObj.TRACK_SUBSCRIPTION, constantsObj.TRACK_PAID_START, subId, dollarPrice, "", "", gameObj.getPuzzleSpeciesTracking(), "1", "");
            }
            // Triggering adjust event for subscription purchase
            // var adjustEventToFire = subscriptionsObj.getAdjustIdToFireForSubscription(pkgId);
            // analyticsObj.triggerAdjustEvent(adjustEventToFire);

            analyticsObj.triggerEvents(constantsObj.EVENT_TYPE_SUBSCRIPTION, pkgId);
        }
    };

    this.getSubscriptionPkgPriceWeekly = function (cut) {
        var subscriptionsList = subscriptionsObj.getAvailableSubscriptions();
        var subscriptionPkgId = subscriptionsList[0];

        if (subscriptionExp == constantsObj.EXP_VAR_2 || subscriptionExp == constantsObj.EXP_VAR_3) {
            subscriptionPkgId = subscriptionsList[2];
        }
        loggerObj.debugLog("sub packages",subscriptionExp, subscriptionPkgId, subscriptionsList);
        var subscriptionPackages = paymentsObj.getSubscriptionPackages();
        var pkgIndex = 0;
        for (var i = 0; i < subscriptionPackages.length; i++) {
            if (subscriptionPkgId == subscriptionPackages[i]["name"]) {
                pkgIndex = i;
            }
        }
        if (cut) {
            return utilObj.getSubsPkgPrices(subscriptionPackages[pkgIndex].price, cut);
        }
        return subscriptionPackages[pkgIndex].price;
    };

    this.getSubscriptionPkgPriceMonthly = function (cut) {
        var subscriptionsList = subscriptionsObj.getAvailableSubscriptions();
        var subscriptionPkgId = subscriptionsList[1];
        loggerObj.debugLog("");
        if (subscriptionExp == constantsObj.EXP_VAR_2 || subscriptionExp == constantsObj.EXP_VAR_3) {
            subscriptionPkgId = subscriptionsList[3];
        }
        var subscriptionPackages = paymentsObj.getSubscriptionPackages();
        var pkgIndex = 0;
        for (var i = 0; i < subscriptionPackages.length; i++) {
            if (subscriptionPkgId == subscriptionPackages[i]["name"]) {
                pkgIndex = i;
            }
        }
        return utilObj.getSubsPkgPrices(subscriptionPackages[pkgIndex].price, cut);
    };

    this.subscribe = function (offset) {
        if (!utilObj.isOnline()) {
            utilObj.showMessage(stringsObj.not_online);
            return;
        }
        var subscriptionsList = subscriptionsObj.getAvailableSubscriptions();
        var subscriptionPkgId = subscriptionsList[0 + offset];
        if (subscriptionExp == constantsObj.EXP_VAR_2 || subscriptionExp == constantsObj.EXP_VAR_3) {
            subscriptionPkgId = subscriptionsList[2 + offset];
        }
        subscriptionsObj.startPurchase(subscriptionPkgId);
    };

    this.getPkgsPriceNumeric = function (pkgId) {
        if (pkgId.indexOf("coindrip1") != -1) {
            return constantsObj.SUB_PKG1_PRICE;
        } else if (pkgId.indexOf("coindrip2") != -1) {
            return constantsObj.SUB_PKG2_PRICE;
        } else if (pkgId.indexOf("freearchive3") != -1) {
            return constantsObj.SUB_PKG3_PRICE;
        }
        return constantsObj.SUB_PKG4_PRICE;
    };

    this.getAdjustIdToFireForSubscription = function (pkgId) {
        if(pkgId == undefined || pkgId == null || pkgId == ""){
            return constantsObj.ADJUST_SUBSCRIPTION_4;
        }
        if (pkgId.indexOf("coindrip1") != -1) {
            return constantsObj.ADJUST_SUBSCRIPTION_1;
        } else if (pkgId.indexOf("coindrip2") != -1) {
            return constantsObj.ADJUST_SUBSCRIPTION_2;
        } else if (pkgId.indexOf("freearchive3") != -1) {
            return constantsObj.ADJUST_SUBSCRIPTION_3;
        }
        return constantsObj.ADJUST_SUBSCRIPTION_4;
    };

    this.getgPlaySubscriptionPackages = function () {
        return gplaySubscriptions;
    };

    this.getSubPkgId = function () {
        return pkgId;
    };

    //Example json for android:
    // For active subscription
    // {"productId":"in.playsimple.wordtrip.subscription.bursthint5","orderId":"GPA.3389-9079-3678-20730","purchaseToken":"clfajaloknfbppfpbcilppem.AO-J1OxyZULZVqxS7GqOcWbRVIIVks7bUkiHj8QJV0tZNSAFqgBhwLbg97T2LnKXD6PLPPmjfeldbs_UcIQKFn8yA6GLiPbQ87rHus1Fcmf-sSoMmkaJ3Jbf6nwjaV0CzL8mFTrdPVDen0cmWpaMtU3ttN_U-y5bhBHiS2S0ixT03wpW6ryqYf0"}
    // Disappears from list of owned items once its expired. Also, data doesn't change on renewals.

    // Example Json for ios
    /*
    {
	"OrigVer": "1.0",
	"Version": "143",
	"BundleIdentifier": "com.littleengine.wordtrek-crossword",
	"ReceiptCreationDate": "2018-07-16T13:48:42Z",
	"InApp": [{
		"CancelDate": "",
		"PurchaseDate": "2018-07-11T08:54:10Z",
		"SubExpDate": "2018-07-11T08:57:10Z",
		"WebItemId": 18446744072224674000,
		"OriginalTransactionIdentifier": "1000000416407304",
		"OriginalPurchaseDate": "2018-07-13T13:41:25Z",
		"Quantity": 1,
		"TransactionIdentifier": "1000000417616704",
		"ProductIdentifier": "in.playsimple.wordtrip.subscription.bursthint5"
	}, {
		"CancelDate": "",
		"PurchaseDate": "2018-07-11T08:57:10Z",
		"SubExpDate": "2018-07-11T09:00:10Z",
		"WebItemId": 18446744072224674000,
		"OriginalTransactionIdentifier": "1000000416407304",
		"OriginalPurchaseDate": "2018-07-11T09:33:23Z",
		"Quantity": 1,
		"TransactionIdentifier": "1000000416434551",
		"ProductIdentifier": "in.playsimple.wordtrip.subscription.bursthint5"
	}, {
		"CancelDate": "",
		"PurchaseDate": "2018-07-11T09:00:10Z",
		"SubExpDate": "2018-07-11T09:03:10Z",
		"WebItemId": 18446744072224674000,
		"OriginalTransactionIdentifier": "1000000416407304",
		"OriginalPurchaseDate": "2018-07-13T13:41:25Z",
		"Quantity": 1,
		"TransactionIdentifier": "1000000417616700",
		"ProductIdentifier": "in.playsimple.wordtrip.subscription.bursthint5"
	}, {
		"CancelDate": "",
		"PurchaseDate": "2018-07-11T09:03:10Z",
		"SubExpDate": "2018-07-11T09:06:10Z",
		"WebItemId": 18446744072224674000,
		"OriginalTransactionIdentifier": "1000000416407304",
		"OriginalPurchaseDate": "2018-07-13T13:41:25Z",
		"Quantity": 1,
		"TransactionIdentifier": "1000000417616699",
		"ProductIdentifier": "in.playsimple.wordtrip.subscription.bursthint5"
	}, {
		"CancelDate": "",
		"PurchaseDate": "2018-07-11T09:06:10Z",
		"SubExpDate": "2018-07-11T09:09:10Z",
		"WebItemId": 18446744072224674000,
		"OriginalTransactionIdentifier": "1000000416407304",
		"OriginalPurchaseDate": "2018-07-13T13:41:25Z",
		"Quantity": 1,
		"TransactionIdentifier": "1000000417616702",
		"ProductIdentifier": "in.playsimple.wordtrip.subscription.bursthint5"
	}, {
		"CancelDate": "",
		"PurchaseDate": "2018-07-11T09:09:10Z",
		"SubExpDate": "2018-07-11T09:12:10Z",
		"WebItemId": 18446744072224674000,
		"OriginalTransactionIdentifier": "1000000416407304",
		"OriginalPurchaseDate": "2018-07-13T13:41:25Z",
		"Quantity": 1,
		"TransactionIdentifier": "1000000417616703",
		"ProductIdentifier": "in.playsimple.wordtrip.subscription.bursthint5"
	}]
}
     */
};