SubscriptionsDTC = function () {

    // function for checking subscription status, first check is for active status, followed by experiments check,
    // followed by if subscription feature surfacing started or not.
    // Based on these conditions, we have following status for subscription:
    // 1. subscription is active
    // 2. subscription is not allocated
    // 3. subscription is inactive
    // 4. subscription is allocated but not started
    // Need to satisfy condition for the subscription UI
    this.checkSubscriptionStatus = function () {
        var subscriptionStatus = subscriptionsObj.isSubscriptionActive();

        if (subscriptionStatus && isSubscriptionUiActive) {
            return constantsObj.SUBSCRIPTION_ACTIVE;
        } else {
            if (utilObj.isAndroid()) {
                if (subscriptionExp == constantsObj.EXP_VAR_CONTROL) {
                    return constantsObj.SUBSCRIPTION_NOT_ALLOCATED;
                } else {
                    if (gameObj.getMastery1() > 0 && isSubscriptionUiActive) {
                        return constantsObj.SUBSCRIPTION_INACTIVE;
                    }
                    return constantsObj.SUBSCRIPTION_ALLOCATED_BUT_NOT_STARTED;
                    // if (gameObj.checkVersionCodesForIos()) {
                    //     return constantsObj.SUBSCRIPTION_INACTIVE;
                    // }
                }
            } else {
                if (subscriptionsDTCObj.checkVersionCodesForIos() && isSubscriptionUiActive) {
                    return constantsObj.SUBSCRIPTION_INACTIVE;
                } else {
                    if (subscriptionExp == constantsObj.EXP_VAR_CONTROL) {
                        return constantsObj.SUBSCRIPTION_NOT_ALLOCATED;
                    } else {
                        if (/*gameObj.getIsNewUserEligibleForSub() &&*/ gameObj.getMastery1() > 0 && isSubscriptionUiActive) {
                            return constantsObj.SUBSCRIPTION_INACTIVE;
                        }
                        return constantsObj.SUBSCRIPTION_ALLOCATED_BUT_NOT_STARTED;
                        // if (gameObj.checkVersionCodesForIos()) {
                        //     return constantsObj.SUBSCRIPTION_INACTIVE;
                        // }
                    }
                }
            }
        }
    };

    //******* Subscription Functions *******//
    // function for checking if runtime and build version codes are same, then show subscriptions on store screen and make subscription status as inactive.
    // Put a check for this function in gameLayer too, for not auto triggering subscription popup in game screen.
    this.checkVersionCodesForIos = function () {
        var reviewVersionCode = runtimeObj.getReviewVersionCode();
        var buildVersionCode = utilObj.getVersionCode();
        return (utilObj.isIos() && reviewVersionCode == buildVersionCode);
    };

    // To check for the UI for the calendar Layer on App start
    this.checkSubscriptionStatusOnAppStart = function () {
        var subscriptionStatus = subscriptionsObj.isSubscriptionActive();

        if (subscriptionStatus) {
            return constantsObj.SUBSCRIPTION_ACTIVE;
        } else {
            if (utilObj.isAndroid()) {
                if (subscriptionExp == constantsObj.EXP_VAR_CONTROL) {
                    return constantsObj.SUBSCRIPTION_NOT_ALLOCATED;
                } else {
                    if (gameObj.getMastery1() > 0) {
                        return constantsObj.SUBSCRIPTION_INACTIVE;
                    }
                    return constantsObj.SUBSCRIPTION_ALLOCATED_BUT_NOT_STARTED;
                    // if (gameObj.checkVersionCodesForIos()) {
                    //     return constantsObj.SUBSCRIPTION_INACTIVE;
                    // }
                }
            } else {
                if (subscriptionsDTCObj.checkVersionCodesForIos()) {
                    return constantsObj.SUBSCRIPTION_INACTIVE;
                } else {
                    if (subscriptionExp == constantsObj.EXP_VAR_CONTROL) {
                        return constantsObj.SUBSCRIPTION_NOT_ALLOCATED;
                    } else {
                        if (/*gameObj.getIsNewUserEligibleForSub() &&*/ gameObj.getMastery1() > 0) {
                            return constantsObj.SUBSCRIPTION_INACTIVE;
                        }
                        return constantsObj.SUBSCRIPTION_ALLOCATED_BUT_NOT_STARTED;
                        // if (gameObj.checkVersionCodesForIos()) {
                        //     return constantsObj.SUBSCRIPTION_INACTIVE;
                        // }
                    }
                }
            }
        }
    };


    // function for getting the UI type for subscriptions, first check is for joining bonus status, followed by if free trial UI has to be shown
    // Based on these conditions, we have following UI types for subscription:
    // 1. UI renew
    // 2. UI free trial
    // 3. UI no free trial
    this.checkUITypeForSubscription = function () {
        var subRuntime = runtimeObj.isSubscriptionFreeTrialActive();
        var subFreeStatus = gameObj.getSubscriptionFreeTrialUsed();
        var joiningBonusStatus = subscriptionsObj.getJoiningBonusStatus();
        if (joiningBonusStatus || gameObj.getIsJoiningBonusGiven()) {
            return constantsObj.SUBSCRIPTION_UI_RENEW;
        } else {
            if (subRuntime && !subFreeStatus) {
                return constantsObj.SUBSCRIPTION_UI_FREE_TRIAL;
            } else {
                return constantsObj.SUBSCRIPTION_UI_NO_FREE_TRIAL;
            }
        }
    };

    this.getIsSubscriptionInActive = function () {
        return subscriptionsDTCObj.checkSubscriptionStatus() == constantsObj.SUBSCRIPTION_INACTIVE;
    };

    this.getIsSubscriptionActive = function () {
        return subscriptionsDTCObj.checkSubscriptionStatus() == constantsObj.SUBSCRIPTION_ACTIVE;
    };

    // this function is called every time from checkAndGrantItems when subscription is active.
    // It actually grants all subscription benefits immediately after subscription purchase.
    // Following cases are handled:
    // 1. Free trial user and showing congratulations popup
    // 2. No free trial user and showing congratulations popup and granting joining bonus immediately(if not given).
    // 3. Renew user and showing congratulations popup
    // 4. Auto trigger of congratulations popup if he has not seen it, after purchase
    this.grantSubscriptionBenefits = function () {
        loggerObj.debugLog("Subscription grant benefits immediately", curScreen, gameObj.getIsJoiningBonusGiven(), subscriptionsDTCObj.checkUITypeForSubscription());
        if ( gameObj.getTrackCongratsPopupType() != "" && curScreen == constantsObj.SCREEN_SUBSCRIPTIONS && (!gameObj.getIsJoiningBonusGiven() || subscriptionsDTCObj.checkUITypeForSubscription() == constantsObj.SUBSCRIPTION_UI_RENEW)) {
            // subscriptionsLaunchPopup.setCongratulationsPopupNodes();
            // subscriptionsLaunchPopup.playConfettiEffect();

            subscriptionsLaunchLayer.addCongratulationsNode(true);
            gameObj.setLastDateDailyDripGiven(gameObj.getTodaysPuzzleDate().getTime() - (constantsObj.SECONDS_IN_DAY * 1000));
            var trackType = gameObj.getTrackCongratsPopupType();
            gameObj.setTrackCongratsPopupType("");
            if (trackType == constantsObj.TRACK_SUBSCRITPION_JOINING_BONUS_POPUP) {
                //logger.debugLog("Subscription setting joining bonus given true");
                gameObj.setIsJoiningBonusCongratsPopupShown(true);
                subscriptionsObj.setJoiningBonusGiven();
            }
            // trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getTrackingBucketId(), gameObj.getTrackingPuzId(), trackType, constantsObj.TRACK_VIEW, "", "", "1", "");
        }
        else if (!gameObj.getIsJoiningBonusGiven() || subscriptionsDTCObj.checkUITypeForSubscription() == constantsObj.SUBSCRIPTION_UI_RENEW) {
            if (gameObj.getTrackCongratsPopupType() != "") {
                gameObj.setLastDateDailyDripGiven(gameObj.getTodaysPuzzleDate().getTime() - (constantsObj.SECONDS_IN_DAY * 1000));
                gameObj.setCongratsPopupTypeForAutoTrigger(gameObj.getTrackCongratsPopupType());
                subscriptionsDTCObj.autoTriggerCongratulations();
                gameObj.setTrackCongratsPopupType("");
            }
        }
    };


    // Cases to AutoTrigger
    this.autoTriggerCongratulations = function () {
        var screenArr = [constantsObj.SCREEN_HOME, constantsObj.SCREEN_CALENDAR];
        var shouldShowPopup = false;
        for (var i = 0; i < screenArr.length; i++) {
            if (curScreen == screenArr[i]) {
                shouldShowPopup = true;
                break;
            }
        }
        if (shouldShowPopup) {
            gameObj.setScreenTypeForSubsLayer(constantsObj.SUBSCRIPTION_CONGRATULATIONS);
            subscriptionsLaunchLayer.show(true, true);
        } else {
            if (curScreen == constantsObj.SCREEN_STORE_LAYER) {
                utilObj.showMessage("Your Subscription is active now!")
            }
            gameObj.setShouldShowJoiningBonusGrantPopup(true);
        }
    };

    // this function is called when user's free trial is over and his subscription is renewed for the first time.
    // It grants joining bonus in case of free trial auto renewal
    this.grantJoiningBonus = function () {
        loggerObj.debugLog("Subscription granting joining bonus on renew call", curScreen, gameObj.getIsJoiningBonusGiven());
        if (!gameObj.getIsJoiningBonusGiven()) {
            gameObj.setCongratsPopupTypeForAutoTrigger(constantsObj.TRACK_SUBSCRITPION_JOINING_BONUS_POPUP);
            subscriptionsDTCObj.autoTriggerCongratulations();
        }
    };

    // Joining Bonus
    this.getJoiningBonus = function () {
        if (subscriptionExp == constantsObj.EXP_VAR_2 || subscriptionExp == constantsObj.EXP_VAR_3) {
            return constantsObj.SUBS_JOININGBONUS2;
        } else {
            return constantsObj.SUBS_JOININGBONUS1;
        }
    };

    //get Daily Drip Coins
    this.getDailyDripAccumulatedCoins = function () {
        var time = gameObj.getLastDateDailyDripGiven();
        var date = new Date(time);
        var todaysDate = gameObj.getTodaysPuzzleDate();
        if (date == null) {
            return 15;
        }
        try {
            var diff = Math.abs(utilObj.getDaysDiff(todaysDate, date));
            if (diff == 0 || date == 1) {
                return 15;
            } else {
                var value =  diff * 15;
                return value > 105? 105: value;
            }
        } catch (e) {
            return 15;
        }
    };

    this.checkIfEligibleForDailyDrip = function () {
        if (gameObj.getLastDateDailyDripGiven() == null || utilObj.getDaysDiff(gameObj.getTodaysPuzzleDate(), new Date(gameObj.getLastDateDailyDripGiven())) > 0) {
            return true;
        } else {
            return false;
        }
    };

    this.checkSubscriptionOffline = function () {
        return (subscriptionsObj.isSubscriptionOffline() && !gameObj.getSubscriptionOfflineShown() && !utilObj.isOnline());
    };

    this.shouldShowCalendarV1 = function () {
        return (subscriptionsDTCObj.getIsSubscriptionInActive() || subscriptionsDTCObj.getIsSubscriptionActive());
    };
};