package in.playsimple.pspn;

import android.content.Context;
import android.util.Log;

import org.json.JSONObject;

import in.playsimple.PSPNGameSpecific;

public class Quests {
    private static Context context = null;
    private static Quests quests = null;

    private boolean shouldShowDailyQuestNotif = false;
    private boolean isDailyQuestCompleted = false;
    private long dailyQuestUnlockTime = 0;

    private Quests() {

    }

    public static Quests get() throws Exception {
        if (Quests.context == null) {
            throw new Exception(
                    "Quest context must be initialized before asking for game object.");
        }
        if (Quests.quests == null) {
            quests = new Quests();
        }
        quests.load();
        return quests;
    }

    public static void setContext(Context context) {
        Quests.context = context;
    }

    public static boolean isContextSet() {
        return Quests.context != null;
    }

    public static void saveNotifData(int dailyQuestUnlockTime, boolean shouldShowDailyQuestNotif, boolean isDailyQuestCompleted) {
        try {
            JSONObject notifInfo = new JSONObject();
            notifInfo.put("shouldShowDailyQuestNotif", shouldShowDailyQuestNotif);
            notifInfo.put("isDailyQuestCompleted", isDailyQuestCompleted);
            notifInfo.put("dailyQuestUnlockTime", dailyQuestUnlockTime);
            PSPNGameSpecific.saveToLocalStorage(PSPNGameSpecific.QUESTS_NOTIF_DATA,
                    notifInfo.toString());
        } catch (Exception e) {
            PSPNGameSpecific.logException(e);
        }
    }

    private boolean load() {
        try {


            // 1. First lets try and read the data from Flutter maintained QUESTS_NOTIF_DATA
            String questData = PSPNGameSpecific.loadFromLocalStorage(PSPNGameSpecific.QUESTS_NOTIF_DATA);

            // 2. Incase QUESTS_NOTIF_DATA is blank, lets query QUESTS_DATA (Format maintained by cocos
            if(questData == null || questData.equals("")) {
                questData = PSPNGameSpecific.loadFromLocalStorage(PSPNGameSpecific.QUESTS_DATA);
                if (questData == null) {
                    return false;
                }
            }
            JSONObject questsObj = new JSONObject(questData);
            if (!questsObj.isNull("shouldShowDailyQuestNotif")) {
                this.shouldShowDailyQuestNotif = questsObj.getBoolean("shouldShowDailyQuestNotif");
            }
            if (!questsObj.isNull("isDailyQuestCompleted")) {
                this.isDailyQuestCompleted = questsObj.getBoolean("isDailyQuestCompleted");
            }
            if (!questsObj.isNull("dailyQuestUnlockTime")) {
                this.dailyQuestUnlockTime = questsObj.getLong("dailyQuestUnlockTime");
            }
        } catch (Exception e) {
            PSPNGameSpecific.logException(e);
            Log.i(PSPNGameSpecific.TAG, "No data for quest yet.");
            return false;
        }
        return true;
    }

    public boolean getShouldShowDailyQuestNotif() {
        return shouldShowDailyQuestNotif;
    }

    public boolean getIsDailyQuestCompleted() {
        return isDailyQuestCompleted;
    }

    public long getDailyQuestUnlockTime() {
        return dailyQuestUnlockTime;
    }
}
