XPromoBubblePop = function () {
    var EXP_NAME = "PSPNbubblePop";

    var bubblePop = null; // feature progress data - used for syncing to the server as well
    var rt = null; // local variable to hold the currently evaluated runtime values
    var runtimeFetched = false; // control variable to avoid fetch runtimes over and over again
    var triggerLock = false; // control variable for the flow - makes it easy for async flows later

    var bubblePopConfigData = undefined; // local variable to hold the configuration data loaded from JSON
    var bubblePopConfigLoaded = false; // flag to ensure having bubble pop config is loaded in locally

    var CHECK_INTERVAL = 60;
    var MIN_DELAY_TO_TRIGGER = 5;
    var MAX_DELAY_TO_TRIGGER = 15;
    var MAX_DAYS_LIMIT = 3000; // Rather than using Number.MAX_VALUE, lets use an improbably high number

    var isBubbleOnScreen = false;
    var bubbleTypeIndex = -1;
    var bubbleTypeName = "";
    var bubbleShownAt = 0;
    var coinsToGrant = 0;
    var isPopVideoPlaying = false;
    var popVideoType = constantsObj.NOT_FOUND;
    var popXpromoType = constantsObj.NOT_FOUND;
    var specialRewardGrant = {};

    var trackingMap = {};

    // Bubble types
    var NUM_BUBBLE_TYPES = 5;

    var BUBBLE_COIN_GRANT = 0;
    var BUBBLE_VIDEO_LOW = 1;
    var BUBBLE_VIDEO_HIGH = 2;
    var BUBBLE_XPROMO_LOW = 3;
    var BUBBLE_XPROMO_HIGH = 4;

    // Config JSON Strings
    var CONFIG_ACTIONS = "actions";
    var CONFIG_ACTIONS_TYPE = "type";

    var CONFIG_TRIGGERS = "triggers";
    var CONFIG_ELIGIBILITY = "eligibility";
    var CONFIG_SURFACING = "surfacing";
    var CONFIG_REWARDS = "rewards";

    var CONFIG_GLOBAL_SURFACING = "globalSurfacing";

    var CONFIG_NAME = "name";
    var CONFIG_PROGRESS_FETCH = "getProgress";
    var CONFIG_RUNTIMECHECK = "runtimeCheck";
    var CONFIG_TARGETVALUE = "targetValue";
    var CONFIG_OPERATOR = "operator";
    var CONFIG_PASS_TYPE = "passType";

    var CONFIG_OPERATOR_EQUAL = "equals";
    var CONFIG_OPERATOR_LESS_THAN = "lessThan";
    var CONFIG_OPERATOR_LESS_THAN_EQUAL = "lessThanEqual";
    var CONFIG_OPERATOR_MORE_THAN = "moreThan";
    var CONFIG_OPERATOR_MORE_THAN_EQUAL = "moreThanEqual";

    var bubbleTypeIndexMap = {
        "xpromo-high": BUBBLE_XPROMO_HIGH,
        "xpromo-low": BUBBLE_XPROMO_LOW,
        "video-high": BUBBLE_VIDEO_HIGH,
        "video-low": BUBBLE_VIDEO_LOW,
        "coin": BUBBLE_COIN_GRANT
    };

    var BUBBLE_POP_CONTENT_DOWNLOAD_LINK = constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/bubblePop/creatives/";

    this.init = function () {

        //Lets load the JSON configurations
        cc.loader.loadJson(res.bubblepop_config_json, function (error, data) {
            if (error == null && data != null && data.hasOwnProperty(constantsObj.GAME_ID)) {
                bubblePopConfigData = data[constantsObj.GAME_ID];
                bubblePopConfigLoaded = true;
                bubblePopObj.sortConfigByPriority();
            }
        });

        var emptyList = [0, 0, 0, 0, 0]; // Default values for all types of bubbles.
        rt = {
            bubbleFloatTime: 15, // Total time for which the bubble would be on screen before auto pop happens
            bubbleUnstableTime: 5, // Time after which the animation for creating urgency is to be started
            wallet: 100, // Wallet thresholds applicable
            minBubblesInDay: [0, 1, 1, 1, 1], // Minimum number of bubbles which need to be shown today before special bubble types becomes eligible
            maxBubblesInDay: 3, // Max number of bubbles which can be shown in a day
            minTimeBetweenTwoBubbles: 30, // Min interval between bubbles
            cashGrant: [0, 10, 20, 100, 200], // Cash grant for different bubble types.
//            cashGrantHigher: [0, 20, 200], // Higher reward cash grant for different bubble types.
            cooldownLower: [30, 2880, 2880, 7200, 7200], // Min Cooldown applicable for different bubble types - in days, except for coins in which case it's minutes.
            cooldownUpper: [30, 2880, 2880, 7200, 7200], // Max Cooldown applicable for different bubble types - in days, except for coins in which case it's minutes.
            minLevel: 60, // Number of levels that need to completed before showing the bubbles
            payerCheck: 2, // Flag to control this feature for payer / non-payer. 0 - Only Non Payer, 1 - Only Payer, 2 - Both
            ////////////////////////////////////////
            // Values related to xPromo quests start
            highRewardDn: 30, // After which Dn should the high reward xPromo quests get triggered.
            installedPSGame: 30, // Minimum days since user has installed another PS game
            playedPSGame: 15, // Minimum days since user has played another PS game
            installedPSGameHigh: 30, // Higher bucket - Minimum days since user has installed another PS game
            playedPSGameHigh: 15, // Higher bucket - Minimum days since user has played another PS game
            // Values related to xPromo quests end
            ////////////////////////////////////////
            ////////////////////////////////////////
            // Values related to video quests start
            daysSinceVideoWatchedLow: 3, // Number of days since user has watched any video for triggering low video reward
            daysSinceVideoWatchedHigh: 7, // Number of days since user has watched any video for triggering high video reward
            // Values related to video quests end
            ////////////////////////////////////////
            ////////////////////////////////////////
            // Values related to coins start
            coinRewardValues: [1, 2, 3], // Coins to be granted on popping any type of bubble
            coinRewardProb: [40, 40, 20], // Probabilities for the corresponding coin grant value - should add up to 100.
            // Values related to coins end
            ////////////////////////////////////////
            ////////////////////////////////////////
            // Cooldown for xpromo start
            xpromoViewCooldown: 3, // Number of views after which the xpromo-ed game goes to cooldown
            xpromoClickCooldown: 3, // Number of clicks after which the xpromo-ed game goes to cooldown
            xpromoGamesLimit: 5, // Number of top priority games to be xpromo-ed (including self)
            // Cooldown for xpromo end
            ////////////////////////////////////////

            //Runtimes added in the V2 - Do not remove any existing runtimes as they are being used on the older builds
            minLevelNew: [60, 60, 60, 60, 60], // Number of levels that need to completed before showing a specific bubble type
            daysSinceInstall: [7, 7, 7, 7, 7], // Days after install date that a bubble is visible
            mainSwitch: [1, 1, 1, 1, 1], // Main switch if a particular bubble type is to be shown or not
            // Runtimes for v2 end
        };
        bubblePop = {
            lastCheck: 0,
            lastShown: utilObj.createClone(emptyList),
            bubblesShownInDay: 0,
            bubbleSkippedTimes: utilObj.createClone(emptyList),
            daysSinceVideoWatched: 0,
            lastUpdateTime: utilObj.getCurrentTimestamp(),
            xpromoClicks: {},
            xpromoCooldowns: {},
            w2ePopupDisabledForBubblePop: false,
        };
    };

    this.load = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }

        if (quests.bubblePop != undefined) {
            bubblePop = quests.bubblePop;
        }

        if(bubblePop.lastShown == undefined || bubblePop.lastShown.length != NUM_BUBBLE_TYPES) {
            bubblePop.lastShown = [0, 0, 0, 0, 0];
        }

        if(bubblePop.bubbleSkippedTimes == undefined) {
            bubblePop.bubbleSkippedTimes = [0, 0, 0, 0, 0];
        }

        if(bubblePop.xpromoClicks == undefined) {
            bubblePop.xpromoClicks = {};
        }

        if (bubblePop.xpromoCooldowns == undefined) {
            bubblePop.xpromoCooldowns = {};
        }

        if(bubblePop.lastUpdateTime == undefined) {
            bubblePop.lastUpdateTime = utilObj.getCurrentTimestamp();
        }

        if (bubblePop.w2ePopupDisabledForBubblePop == undefined) {
            bubblePop.w2ePopupDisabledForBubblePop = false;
        }
    };

    this.save = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }
        quests.bubblePop = bubblePop;
        return quests;
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Basic Bubble Pop methods /////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Method to return the current variant of the bubble pop master experiment
     * @returns {number} - number corresponding to the variant, 0 for control / off.
     */
    this.getBubblePopExperimentVariant = function() {
        return epmObj.getChosenVariant(EXP_NAME);
    };

    /**
     * Method to check if the bubble pop is feature is currently active or in-active for a player
     * @returns {boolean} - True / False
     */
    this.isBubblePopActive = function() {
        var result = false;

        if(pspnFlutterObj.isFlutterEnabled() && !pspnFlutterObj.isBubblePopDataLoaded()) {
            return result;
        }

        if(xPromoGameSpecificObj.isBubblePopEnabled() && bubblePopObj.getBubblePopExperimentVariant() != xpromoConstantsObj.EXP_VAR_CONTROL) {
            var runtimeValues = bubblePopObj.getRuntimes();
            var isPayer = xPromoGameSpecificObj.getIsPayerUser();
            result = ((runtimeValues.payerCheck == 0 && isPayer == false) ||
                (runtimeValues.payerCheck == 1 && isPayer == true) ||
                (runtimeValues.payerCheck == 2));
        }
        return result;
    };

    /**
     * Method to fetch runtime value based on the experiment variant. Also caches the value for the rest of the session.
     * @returns {object}  - Returns a JS object with runtime values.
     */
    this.getRuntimes = function () {
        // Example Runtime values:
        // Value: 9$bp1,cg-0-10-20-100-200:cl-1-1-1-1-1:cu-0-1-0-1-0:cr-1-2-3:crp-40-40-20:bft-20:but-4:w-270:mib-0-1-1-1-1:mab-20:mint-1:hdn-1:ipg-1:ppg-1:ipgh-1:ppgh-1:vl-1:vh-1:ml-2:xvc-2:xcc-1:xgl-6:pc-2
        // cashGrant:cashGrant:cooldownLower:cooldownUpper:coinRewardValues:coinRewardProb:bubbleFloatTime:bubbleUnstableTime:wallet:minBubblesInDay:maxBubblesInDay:minTimeBetweenTwoBubbles:highRewardDn:installedPSGame:playedPSGame:installedPSGameHigh:playedPSGameHigh:daysSinceVideoWatchedLow:daysSinceVideoWatchedHigh:mimLevel:xpromoViewCooldown:xpromoClickCooldown:xpromoGamesLimit:payerCheck
        //
        // After Parsing:
        //     "bp1": {
        //     "cg":[0,10,20,100,200],
        //     "cl":[1,1,1,1,1],
        //     "cu":[0,1,0,1,0],
        //     "cr":[1,2,3],
        //     "crp":[40,40,20],
        //     "bft":["20"],
        //     "but":["4"],
        //     "w":["270"],
        //     "mib":[0,1,1,1,1],
        //     "mab":["20"],
        //     "mint":["1"],
        //     "hdn":["1"],
        //     "ipg":["1"],
        //     "ppg":["1"],
        //     "ipgh":["1"],
        //     "ppgh":["1"],
        //     "vl":["1"],
        //     "vh":["1"],
        //     "ml":["2"],
        //     "xvc":["2"],
        //     "xcc":["1"],
        //     "xgl":["6"],
        //     "pc":["2"]
        // }
        if(runtimeFetched) {
            return rt;
        }

        var updatedRuntime = null;
        var variant = bubblePopObj.getBubblePopExperimentVariant();
        // Different configuration is expected for different variants.
        if (variant == xpromoConstantsObj.EXP_VAR_1) {
            updatedRuntime = xpromoQuestRuntimesObj.getRuntime("bp1", null);
        } else if (variant == xpromoConstantsObj.EXP_VAR_2) {
            updatedRuntime = xpromoQuestRuntimesObj.getRuntime("bp2", null);
        } else {
            // No runtime info needed for control.
        }

        if (updatedRuntime != null) {
            for (var i = 0; i < updatedRuntime.cg.length; i++) {
                updatedRuntime.cg[i] = parseInt(updatedRuntime.cg[i]);
                updatedRuntime.cl[i] = parseInt(updatedRuntime.cl[i]);
                updatedRuntime.cu[i] = parseInt(updatedRuntime.cu[i]);
                updatedRuntime.mib[i] = parseInt(updatedRuntime.mib[i]);
                if(updatedRuntime.mln != undefined && updatedRuntime.dsi != undefined && updatedRuntime.mst != undefined) {
                    updatedRuntime.mln[i] = parseInt(updatedRuntime.mln[i]);
                    updatedRuntime.dsi[i] = parseInt(updatedRuntime.dsi[i]);
                    updatedRuntime.mst[i] = parseInt(updatedRuntime.mst[i]);
                }
            }
            for (var i = 0; i < updatedRuntime.cr.length; i++) {
                updatedRuntime.cr[i] = parseInt(updatedRuntime.cr[i]);
                updatedRuntime.crp[i] = parseInt(updatedRuntime.crp[i]);
            }
            rt.cashGrant = updatedRuntime.cg;
            rt.cooldownLower = updatedRuntime.cl;
            rt.cooldownUpper = updatedRuntime.cu;
            rt.coinRewardValues = updatedRuntime.cr;
            rt.coinRewardProb = updatedRuntime.crp;
            rt.bubbleFloatTime = parseInt(updatedRuntime.bft[0]);
            rt.bubbleUnstableTime = parseInt(updatedRuntime.but[0]);
            rt.wallet = parseInt(updatedRuntime.w[0]);
            rt.minBubblesInDay = updatedRuntime.mib;
            rt.maxBubblesInDay = parseInt(updatedRuntime.mab[0]);
            rt.minTimeBetweenTwoBubbles = parseInt(updatedRuntime.mint[0]);
            rt.highRewardDn = parseInt(updatedRuntime.hdn[0]);
            rt.installedPSGame = parseInt(updatedRuntime.ipg[0]);
            rt.playedPSGame = parseInt(updatedRuntime.ppg[0]);
            rt.installedPSGameHigh = parseInt(updatedRuntime.ipgh[0]);
            rt.playedPSGameHigh = parseInt(updatedRuntime.ppgh[0]);
            rt.daysSinceVideoWatchedLow = parseInt(updatedRuntime.vl[0]);
            rt.daysSinceVideoWatchedHigh = parseInt(updatedRuntime.vh[0]);
            rt.minLevel = parseInt(updatedRuntime.ml[0]);
            rt.payerCheck = parseInt(updatedRuntime.pc[0]);
            rt.xpromoClickCooldown = parseInt(updatedRuntime.xcc[0]);
            rt.xpromoViewCooldown = parseInt(updatedRuntime.xvc[0]);
            rt.xpromoGamesLimit = parseInt(updatedRuntime.xgl[0]);
            // Below runtimes are added as a v2
            if(updatedRuntime.mln != undefined && updatedRuntime.dsi != undefined && updatedRuntime.mst != undefined) {
                rt.minLevelNew =  updatedRuntime.mln;
                rt.daysSinceInstall = updatedRuntime.dsi;
                rt.mainSwitch = updatedRuntime.mst;
            }
            runtimeFetched = true;
        }

        return rt;
    };

    /**
     * Method to sort the config by priority
     * This is done to ensure that the while checking, the first bubble to pass the criteria is of the highest possible priority
     */
    this.sortConfigByPriority = function() {
        if(bubblePopConfigLoaded && utilObj.isArray(bubblePopConfigData[CONFIG_ACTIONS])) {

            // local function for sorting objects by their priority
            var prioritySorter = function(a, b) {
                return a.priority - b.priority;
            };

            // First lets sort the configurations by priority
            bubblePopConfigData[CONFIG_ACTIONS].sort(prioritySorter);

            // Now lets sort each of the reward configs in an action by priority
            for (var index = 0; index < bubblePopConfigData[CONFIG_ACTIONS].length; index++) {
                var action = bubblePopConfigData[CONFIG_ACTIONS][index];
                if(utilObj.isArray(action[CONFIG_REWARDS])) {
                    action[CONFIG_REWARDS].sort(prioritySorter);
                }
            }
        }
    };

    /**
     * Get the array index corresponding to the bubble type
     * @param actionType - String representing the type of the bubble
     * @returns {number}
     */
    this.getIndexByBubbleType = function(actionType) {
        if(actionType != undefined && bubbleTypeIndexMap.hasOwnProperty(actionType)) {
            return bubbleTypeIndexMap[actionType];
        }
        return constantsObj.NOT_FOUND;
    };

    /**
     * Wrapper method to process a condition based on runtimeCheck or targetValue
     * @param config - JS Object for the configuration of the condition
     * @param actionType - String representing the type of action this is ("xpromo", "video",... etc)
     * @returns {boolean} - Returns true / false depending if the condition is pass / fail respectively.
     *                      Returns false of unsupported condition type
     */
    this.processCondition = function(config, actionType) {
        var result = false;

        if(config.hasOwnProperty(CONFIG_RUNTIMECHECK) && config[CONFIG_RUNTIMECHECK] != null
            && bubblePopObj.evaluateCondition(config, actionType, true)) {
            result = true;

        } else if (config.hasOwnProperty(CONFIG_TARGETVALUE) && config[CONFIG_TARGETVALUE] != null
            && bubblePopObj.evaluateCondition(config, actionType, false)){
            result = true;
        }

        return result;
    };

    /**
     * Method to run the condition check string using the eval. Also figures out if the condition requires an index lookup
     * @param config - JS Object for the configuration of the condition
     * @param actionType - String representing the type of action this is ("xpromo", "video",... etc)
     * @param runtimeCheck - true / false depending if it needs a runtime check or targetValue comparision
     * @returns {boolean} Returns true / false depending if the condition is pass / fail respectively.
     */
    this.evaluateCondition = function(config, actionType, runtimeCheck) {
        var result = false;
        if(config.hasOwnProperty(CONFIG_PROGRESS_FETCH) && config[CONFIG_PROGRESS_FETCH] != null) {

            var params = [];
            var queryByBubbleType = false;
            if(config.hasOwnProperty(CONFIG_PASS_TYPE) && config[CONFIG_PASS_TYPE] === true) {
                params.push(actionType);
                queryByBubbleType = true;
            }

            var progress = eval(config[CONFIG_PROGRESS_FETCH]).apply(null, params);
            var runtimeValues = bubblePopObj.getRuntimes();

            var requiredProgress = (runtimeCheck) ? runtimeValues[config[CONFIG_RUNTIMECHECK]] : config[CONFIG_TARGETVALUE];
            if(queryByBubbleType && utilObj.isArray(requiredProgress)) {
                var index = bubblePopObj.getIndexByBubbleType(actionType);
                if(index != constantsObj.NOT_FOUND) {
                    requiredProgress = requiredProgress[index];
                } else {
                    requiredProgress = constantsObj.NOT_FOUND;
                }
            }

            result = (bubblePopObj.evaluateOperator(progress, requiredProgress, config[CONFIG_OPERATOR]));
        }
        return result;
    };

    /**
     * Wrapper method to run an evaluation operation depending on the operator, current and required progress
     * @param curProgress - value representing the current value
     * @param reqProgress - value representing the required value
     * @param operator - operator applied between the current and required value
     * @returns {boolean} - Returns the evaluation of "curProgress <operator> reqProgress"
     */
    this.evaluateOperator = function(curProgress, reqProgress, operator) {
        var result = false;
        if(curProgress != undefined && reqProgress != undefined && operator != undefined) {
            switch (operator) {
                case CONFIG_OPERATOR_EQUAL:
                    result = (curProgress == reqProgress);
                    break;
                case CONFIG_OPERATOR_LESS_THAN:
                    result = (curProgress < reqProgress);
                    break;
                case CONFIG_OPERATOR_LESS_THAN_EQUAL:
                    result = (curProgress <= reqProgress);
                    break;
                case CONFIG_OPERATOR_MORE_THAN:
                    result = (curProgress > reqProgress);
                    break;
                case CONFIG_OPERATOR_MORE_THAN_EQUAL:
                    result = (curProgress >= reqProgress);
                    break;
                default:
                    break;
            }
        }
        return result;
    };


    /**
     * Entry point method which initiates the trigger check for Bubble Pop
     * This method needs to be called from the different trigger points in the game.
     */
    this.initiateTriggerCheck = function () {
        if(bubblePopObj.isBubblePopActive()) {
            if(triggerLock) {
                return;
            }

            var curTimestamp = utilObj.getCurrentTimestamp();
            if (curTimestamp - bubblePop.lastCheck < CHECK_INTERVAL) {
                return;
            }

            bubblePop.lastCheck = curTimestamp;
            triggerLock = true;

            var bubbleToShow = false;

            if(bubblePopConfigLoaded && utilObj.isArray(bubblePopConfigData[CONFIG_ACTIONS])) {

                // Lets try and load the runtimes if not yet done
                var runtimeValues = bubblePopObj.getRuntimes();

                // Need to reset Cross Promo cooldowns for games if all are reached
                bubblePopObj.checkResetXpromoCooldowns();

                // Lets iterate over all possible actions and see if any of the triggers pass
                for (var index = 0; index < bubblePopConfigData[CONFIG_ACTIONS].length; index++) {
                    // Lets first check if the main switch is on for a bubble type before initiating checks
                    var action = bubblePopConfigData[CONFIG_ACTIONS][index];
                    var switchIndex = bubblePopObj.getIndexByBubbleType(action[CONFIG_ACTIONS_TYPE]);
                    if(utilObj.isArray(runtimeValues.mainSwitch) && runtimeValues.mainSwitch[switchIndex] == 1) {
                        var passedTriggers = bubblePopObj.checkTrigger(action);

                        // If no triggers have passed, no point evaluating further.
                        if(passedTriggers.length > 0) {

                            bubblePopObj.initiateBubbleTracking();
                            bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_PHYLUM, passedTriggers[0]); // Pushing the passed trigger

                            if(bubblePopObj.checkEligibility(action) && bubblePopObj.checkSurfacing(action, bubblePopConfigData[CONFIG_GLOBAL_SURFACING])) {
                                // All checks are passed, lets calculate the reward grants and keep it in place.
                                bubblePopObj.evaluateSpecialRewards(action, passedTriggers);

                                // Lets set the necessary flags and add a delayed call with a random trigger duration for showing the bubble
                                bubbleToShow = true;
                                bubbleTypeName = action[CONFIG_ACTIONS_TYPE];
                                bubbleTypeIndex = bubblePopObj.getIndexByBubbleType(action[CONFIG_ACTIONS_TYPE]);
                                utilObj.delayedCallback(gameLayer, bubblePopObj.showBubble, utilObj.getRandomIntInRange(MIN_DELAY_TO_TRIGGER, MAX_DELAY_TO_TRIGGER), null, false);
                                loggerObj.debugLog("bubblepop: Trying to show bubble of type=" + bubbleTypeName);

                                bubblePopObj.applyTracking();
                                bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, action[CONFIG_ACTIONS_TYPE]);
                                break;
                            }
                            bubblePopObj.applyTracking();
                        }
                    }
                }
            }
            if(!bubbleToShow) {
                triggerLock = false;
            }
        } else { // If Bubble Pop V1 is in-active, check if V2 is active
            bubblePopNewObj.initiateTriggerCheck();
        }
    };

    /**
     * Method that checks if any of the triggers have passed
     * @param action - JS Object for the configuration of the action
     * @returns {Array} - List of strings representing the passed triggers
     */
    this.checkTrigger = function(action) {
        var result = [];
        if(action != undefined && utilObj.isArray(action[CONFIG_TRIGGERS])) {
            for (var index = 0; index < action[CONFIG_TRIGGERS].length; index++) {
                var trigger = action[CONFIG_TRIGGERS][index];

                if(bubblePopObj.processCondition(trigger, action[CONFIG_ACTIONS_TYPE])) {
                    result.push(trigger[CONFIG_NAME]);
                }
            }
        }
        return result;
    };

    /**
     * Method that checks all the eligibility criteria for a bubble type
     * @param action - JS Object for the configuration of the action
     * @returns {boolean} - True if all eligibilities have passed, false otherwise
     */
    this.checkEligibility = function(action) {
        var result = true;
        bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, "success");

        if(action != undefined && utilObj.isArray(action[CONFIG_ELIGIBILITY])) {
            for (var index = 0; (index < action[CONFIG_ELIGIBILITY].length && result); index++) {
                var eligibility = action[CONFIG_ELIGIBILITY][index];

                if(bubblePopObj.processCondition(eligibility, action[CONFIG_ACTIONS_TYPE])) {
                    result = result && true;
                } else {
                    result = false;
                    bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, eligibility[CONFIG_NAME]);
                }
            }
        } else {
            result = false;
        }
        return result;
    };

    /**
     * Method that checks all the surfacing criteria for a bubble type
     * @param action - JS Object for the configuration of the action
     * @param globalSurfacing - JS Object for the configuration of global surfacing
     * @returns {boolean} - True if all surfacings have passed, false otherwise
     */
    this.checkSurfacing = function(action, globalSurfacing) {
        var result = true;
        bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, "success");

        if(action != undefined && action.hasOwnProperty(CONFIG_SURFACING)) {
            var surfacings;
            if(action[CONFIG_SURFACING].hasOwnProperty("useGlobal") && action[CONFIG_SURFACING]["useGlobal"] === true
                && utilObj.isArray(globalSurfacing)) {
                surfacings = globalSurfacing;
            } else if (action[CONFIG_SURFACING].hasOwnProperty("rules") && utilObj.isArray(action[CONFIG_SURFACING]["rules"])) {
                surfacings = action[CONFIG_SURFACING]["rules"];
            } else {
                result = false;
                bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, "json-data-error");
            }

            if(result) {
                for (var index = 0; (index < surfacings.length && result); index++) {
                    var surf = surfacings[index];

                    if(bubblePopObj.processCondition(surf, action[CONFIG_ACTIONS_TYPE])) {
                        result = result && true;
                    } else {
                        result = false;
                        bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, surf[CONFIG_NAME]);
                    }
                }
            }
        }
        return result;
    };

    /**
     * Method to evaluate additional / special rewards that need to be granted based on the triggers that have passed
     * and stores it locally for the session
     * @param action - JS Object for the configuration of the action
     * @param passedTriggers - List of all the triggers that have passed for this action type.
     */
    this.evaluateSpecialRewards = function(action, passedTriggers) {
        if(action != undefined && utilObj.isArray(action[CONFIG_REWARDS])) {

            specialRewardGrant = {}; // Setting special rewards to null - to remove any existing reward

            for (var index = 0; index < action[CONFIG_REWARDS].length; index++) {
                var reward = action[CONFIG_REWARDS][index];

                if(utilObj.isArray(reward["passedTriggers"]) && utilObj.isArray(passedTriggers)) {
                    //Find the intersection, and check if there is any
                    var intersection = reward["passedTriggers"].filter(function(n) {
                        return passedTriggers.indexOf(n) !== -1;
                    });

                    if(intersection.length > 0) {
                        // First lets try to see if the grant value is defined by the runtime - currently the support is only for Coins
                        if(reward.hasOwnProperty(CONFIG_RUNTIMECHECK) && reward[CONFIG_RUNTIMECHECK] != null) {
                            var runtimeValues = bubblePopObj.getRuntimes();
                            var grants = runtimeValues[reward[CONFIG_RUNTIMECHECK]];

                            if(utilObj.isArray(grants) && reward.hasOwnProperty(CONFIG_PASS_TYPE) && reward[CONFIG_PASS_TYPE] === true) {
                                var actionType = action[CONFIG_ACTIONS_TYPE];
                                var i = bubblePopObj.getIndexByBubbleType(actionType);
                                if(i != constantsObj.NOT_FOUND) {
                                    grants = grants[i];
                                }
                            }
                            specialRewardGrant["coins"] = grants;
                            break;

                            // Else we will use reward and quantity value defined in the JSON config
                        } else if (reward.hasOwnProperty("reward") && reward["reward"] != null
                            && reward.hasOwnProperty("quantity") && reward["quantity"] != null){
                            specialRewardGrant[reward["reward"]] = reward["quantity"];
                            break;

                        }
                    }
                }
            }
        }
    };

    /**
     * Wrapper method to update all data that need to change on a fresh date (for the first session only)
     * (This is currently being invoked on the first sesion of the day from updateDailyInfo in xpromoUtil)
     */
    this.updateBubblesDailyData = function () {
        if (bubblePop == null) {
            return;
        }
        var curTime = utilObj.getCurrentTimestamp();
        var daysPassed = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(curTime, bubblePop.lastUpdateTime));

        bubblePop.bubblesShownInDay = 0;
        bubblePop.daysSinceVideoWatched += daysPassed;
        bubblePop.bubbleSkippedTimes = [0, 0, 0, 0, 0];
        bubblePop.lastUpdateTime = curTime;
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Basic Bubble Pop methods ///////////////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for specific reward calc / grant methods /////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Method to calculate the basic coin reward that needs to be granted based on runtime values and probability
     * @returns {number} - Number of coins that needs to granted
     */
    this.calcCoinReward = function () {
        var rew = 0;

        var rand = utilObj.getRandomIntInRange(0, 100);
        var sum = 0;
        for (var i = 0; i < rt.coinRewardProb.length; i++) {
            sum += rt.coinRewardProb[i];
            if (sum > rand) {
                rew = rt.coinRewardValues[i];
                break;
            }
        }
        return rew;
    };

    /**
     * Method to grant basic coin reward and try initiating any custom flows based on the bubble type
     * @param params - JS Object holding the cash amount to grant and also the layer where the action is triggered
     */
    this.grantPopRewards = function (params) {
        if(params.cashToGrant > 0) {
            if((xPromoGameSpecificObj.getCash() + params.cashToGrant) <= constantsObj.MAX_CASH_ALLOWED) {
                utilObj.delayedCallback(params.layerForBubblePop, xPromoGameSpecificObj.grantCoinsAfterDelay, 2.0, {cashToGrant: params.cashToGrant, phylum: constantsObj.TRACK_EARN, class: "bubblepop-basic-coins", order: ""}, false);
                xPromoGameSpecificObj.showCoinGrantMessage(params.cashToGrant);
            } else {
                utilObj.showMessage(stringsObj.coin_limit_reached);
            }
        }
        switch (bubbleTypeIndex) {
            case BUBBLE_VIDEO_HIGH:
            case BUBBLE_VIDEO_LOW:
                popVideoType = bubbleTypeName; // This flow would later go and reset the bubbleTypeName, hence keeping it till video complete
                utilObj.delayedCallback(params.layerForBubblePop, bubblePopObj.playVideoFromPop, 0.2, null, false);
                break;
            case BUBBLE_XPROMO_HIGH:
            case BUBBLE_XPROMO_LOW:
                popXpromoType = bubbleTypeName; // This flow would later go and reset the bubbleTypeName, hence keeping it for xpromo popup
                utilObj.delayedCallback(params.layerForBubblePop, bubblePopObj.showXpromoQuest, 0.2, null, false);
                break;
            case BUBBLE_COIN_GRANT:
            default:
                break;
        }
        // // In all cases we need to do the coin grant.
        // bubblePopUiObj.playRewardAnim(bubbleTypeIndex);
    };

    /**
     * Method to grant the special reward (Currently being called from customized flows for special bubbles)
     * @param sender - layer where the grant needs to happen
     * @param params - JS object containing the "reward" to be granted and also the "source"
     */
    this.grantSpecialReward = function(sender, params) {
        if (sender == undefined) {
            sender = gameLayer;
        }
        var reward = params.rewards, source = params.source, showMessage = params.showMessage;
        if(reward != undefined) {
            for(var type in reward) {
                var quantity = reward[type];

                if(quantity > 0) {
                    switch(type) {
                        case "coins":
                            if((xPromoGameSpecificObj.getCash() + quantity) <= constantsObj.MAX_CASH_ALLOWED) {
                                if(showMessage == undefined || showMessage == true) {
                                    xPromoGameSpecificObj.canW2eCoinGrant(0, 0, quantity, false, sender);
                                    utilObj.delayedCallback(sender, xPromoGameSpecificObj.grantCoinsAfterDelay, 2.0, {cashToGrant: quantity, phylum: constantsObj.TRACK_EARN, class: (source != undefined) ? source : "", order: ""}, false);
                                    xPromoGameSpecificObj.showCoinGrantMessage(quantity);
                                } else {
                                    utilObj.delayedCallback(sender, xPromoGameSpecificObj.grantCoinsAfterDelay, 2.0, {cashToGrant: quantity, phylum: constantsObj.TRACK_EARN, class: (source != undefined) ? source : "", order: ""}, false);
                                }

                                // Tracking for granting from bubble source
                                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, (params.phylum != undefined) ? params.phylum : "", (params.bubbleName != undefined) ? params.bubbleName : "",
                                    xpromoConstantsObj.TRACK_BUBBLE_POP, "final_grant", "", quantity, "", "");
                            } else {
                                utilObj.showMessage(stringsObj.coin_limit_reached);
                            }
                            break;
                        // Add more reward types here when a new reward is defined in the JSON config.
                        default:
                            break;
                    }
                }
            }
            specialRewardGrant = {};
        }
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for specific reward calc / grant methods ///////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for UI utility methods ///////////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Helper method to auto pop a bubble from other game flows (Eg: Puzzle Completion)
     */
    this.tryAutoPopBubble = function () {
        if(bubblePopObj.isBubblePopActive()) {
            var layerForBubblePop = utilObj.getLayerForScreen(curScreen);
            bubblePopUiObj.autoPopBubble(layerForBubblePop);
        } else { // If Bubble Pop V1 is in-active, check if V2 is active
            bubblePopNewObj.tryAutoPopBubble();
        }
    };

    /**
     * Method to check and initiate the bubble show and call the frontend to do the same.
     */
    this.showBubble = function () {
        if (!xPromoGameSpecificObj.canShowBubblePopAnimationOnLayer() || popupLayer.popupOpen || ftueObj.isFtueVisible() || (xPromoGameSpecificObj.isSettingsLayerEnabled() && curLayer == settingsLayer)) {
            triggerLock = false;
            bubbleTypeIndex = constantsObj.NOT_FOUND;
            bubbleTypeName = "";
            return;
        }

        isBubbleOnScreen = true;
        bubbleShownAt = utilObj.getCurrentTimestamp();
        bubblePop.lastShown[bubbleTypeIndex] = bubbleShownAt;
        bubblePop.bubblesShownInDay++;

        for(var i = 0; i < NUM_BUBBLE_TYPES; i++) {
            if(bubbleTypeIndex != i) {
                bubblePop.bubbleSkippedTimes[i]++;
            } else {
                bubblePop.bubbleSkippedTimes[i] = 0;
            }
        }

        coinsToGrant = bubblePopObj.calcCoinReward();
        bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, constantsObj.TRACK_VIEW);
        bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_GENUS, xPromoGameSpecificObj.getDailyPuzzleClears());
        bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_SPECIES, coinsToGrant + "");
        bubblePopObj.applyTracking();

        bubblePopUiObj.showBubble(bubbleTypeIndex, coinsToGrant);
    };

    /**
     * Method to handle pop of the bubble from the UI
     * @param params - JS Object holding the cash amount to grant and also the layer where the action is triggered
     */
    this.popBubble = function (params) {
        bubblePopObj.grantPopRewards(params);

        bubblePopObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, constantsObj.TRACK_CLICK);
        bubblePopObj.applyTracking();

        triggerLock = false;
        isBubbleOnScreen = false;
        bubbleTypeIndex = constantsObj.NOT_FOUND;
        bubbleTypeName = "";
    };

    /**
     * Method to handle auto-pop of the bubble from the UI
     */
    this.autoPopBubble = function() {
        triggerLock = false;
        isBubbleOnScreen = false;
        bubbleTypeIndex = constantsObj.NOT_FOUND;
        bubbleTypeName = "";
    };

    /**
     * Method to get the amount of time the bubble needs to float around on the screen
     * @returns {number} - Number in seconds
     */
    this.getBubbleFloatTime = function () {
        var runtime = bubblePopObj.getRuntimes();
        return runtime.bubbleFloatTime;
    };

    /**
     * Method to get the time at the tail of the float time when the bubble starts to flicker
     * @returns {number} - Number in seconds
     */
    this.getBubbleUnstableTime = function () {
        var runtime = bubblePopObj.getRuntimes();
        return runtime.bubbleUnstableTime;
    };

    /**
     * Method to check if the bubble is currently visible on the screen or not
     * @returns {boolean} - True / False
     */
    this.isBubbleVisibleOnScreen = function () {
        return isBubbleOnScreen;
    };

    /**
     * Method to return the special reward grant as an Object
     * @returns {object} - JS Object with the reward name as key and value as quantity Eg: {"coins" : 100}
     */
    this.getSpecialRewardToGrant = function () {
        return specialRewardGrant;
    };

    this.getW2ePopupDisabledForBubblePop = function () {
        return bubblePop.w2ePopupDisabledForBubblePop;
    };

    this.setW2ePopupDisabledForBubblePop = function (val, onPlayClick) {
        bubblePop.w2ePopupDisabledForBubblePop = val;

        if(onPlayClick) {
            isPopVideoPlaying = true;
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, "popup", popVideoType,
                xpromoConstantsObj.TRACK_BUBBLE_POP, val ? "click_check" : "click_not_check", "", "", "", "");
        }

    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for UI utility methods /////////////////////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Video bubble specific conditions /////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Method to initiate the the showing of Ad
     */
    this.playVideoFromPop = function () {
        if ((curLayer == gameLayer || lastLayer == gameLayer) && softLaunchObj.getIsPuzzleCompleted() == true) {
            return;
        }

        if (!xPromoGameSpecificObj.checkRewardedVideo(false)) {
            utilObj.showMessage(stringsObj.video_not_available);
            return;
        }

        if (bubblePopObj.getW2ePopupDisabledForBubblePop()) {
            utilObj.playSound(constantsObj.button_tap_01, false);
            isPopVideoPlaying = true;
            xPromoGameSpecificObj.showVideoAd();
        } else {
            // This does not work for DTC, as there is no w2eInfoPopup, the flow should never reach here
            w2eInfoPopup.show(true, bubblePopObj.getSpecialRewardToGrant());

            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, "popup", popVideoType,
                xpromoConstantsObj.TRACK_BUBBLE_POP, constantsObj.TRACK_VIEW, "", "", "", "");
        }
    };

    /**
     * Method to reset days since the last video was watched
     */
    this.resetDaysSinceVideoWatched = function () {
        if (bubblePop == null) {
            return;
        }
        bubblePop.daysSinceVideoWatched = 0;
    };

    /**
     * Wrapper method to check if the video was being played from bubble pop and the reward that needs to be granted.
     * @returns {boolean} - Whether the reward was successfully granted or not.
     */
    this.checkAndGrantVideoReward = function () {
        if (isPopVideoPlaying && bubblePopObj.isBubblePopActive()) {
            isPopVideoPlaying = false;

            var showMessage = true;
            var xpromoAfterW2eGrantExp = epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_XPROMO_W2E);

            if(xPromoGameSpecificObj.isScreenForW2eXpromo(updatingToScreen) || xPromoGameSpecificObj.checkScreenIsW2E(updatingToScreen)) {
                var layerForAnimation = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(updatingToScreen);
                if (xpromoAfterW2eGrantExp != constantsObj.EXP_VAR_CONTROL) {

                    xpromoW2eObj.setRewardValueOverride(specialRewardGrant["coins"]);
                    if(xpromoW2eObj.showW2EPopup()) {
                        showMessage = false;
                    } else {
                        xpromoW2eObj.setRewardValueOverride(constantsObj.NOT_FOUND);
                    }
                }
            }
            // Pass the current screen layer to help with the reward animation
            var layerForAnimationOnCurScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen);
            var paramsMap = {"rewards": specialRewardGrant, "source": "bubblepop-video", "bubbleName": popVideoType, "showMessage": showMessage, "phylum": "popup"};
            bubblePopObj.grantSpecialReward(layerForAnimationOnCurScreen, paramsMap);

            return true;
        } else { // If Bubble Pop V1 is in-active, check if V2 is active
            return bubblePopNewObj.checkAndGrantVideoReward();
        }
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Video bubble specific conditions ///////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Xpromo bubble specific conditions ////////
    ////////////////////////////////////////////////////////////////////////


    /**
     * Wrapper method to surface the xpromo quest and UI given the required set of conditions pass
     * @returns {boolean} - Whether the xpromo popup was shown or not.
     */
    this.showXpromoQuest = function () {
        if ((curLayer == gameLayer || lastLayer == gameLayer) && softLaunchObj.getIsPuzzleCompleted() == true) {
            return false;
        }
        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopObj.getRuntimes();
        var questPrioritiesArr = Object.keys(xpromoQuests);
        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];

            if(!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) || !bubblePopObj.isBubblePopCreativeAvailable(questId, false) || bubblePopObj.isViewOrClickCooldownReached(gameId)) {
                continue;
            }

            if (bubblePop.xpromoCooldowns[gameId] == undefined) {
                bubblePop.xpromoCooldowns[gameId] = {
                    "viewCount": 0,
                    "clickCount": 0,
                };
            }

            bubblePop.xpromoCooldowns[gameId].viewCount++;

            bubblePopUiObj.show(true, {gameId: gameId, bubbleName: popXpromoType});
            return true;
        }
        return false;
    };

    /**
     * Method to check if all xpromo-able games have reach their cooldown and reset them if required.
     */
    this.checkResetXpromoCooldowns = function () {
        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;
        var questPrioritiesArr = Object.keys(xpromoQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });


        var reset = true;

        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit && reset); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];

            if(xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) && bubblePopObj.isBubblePopCreativeAvailable(questId, false)) {
                if(bubblePop.xpromoCooldowns[gameId] != undefined) {
                    if(bubblePop.xpromoCooldowns[gameId].viewCount < xpromoViewCoolDown && bubblePop.xpromoCooldowns[gameId].clickCount < xpromoClickCoolDown) {
                        reset = false;
                    }
                } else {
                    reset = false;
                }
            }
        }

        if(reset) {
            bubblePop.xpromoCooldowns = {};
        }
    };

    /**
     * Method to check if the creative / bubble pop image for a xpromo-able game is available or not.
     * @param questId - quest ID for the adwall quest corresponding to the xpromo game
     * @param downloadOnFail - Boolean to initiate download if the requested file does not exist
     * @returns {boolean} - Whether the creative is downloaded and available.
     */
    this.isBubblePopCreativeAvailable = function (questId, downloadOnFail = false) {
        var result = false;
        var curQuests;

        if(pspnFlutterObj.isFlutterEnabled()) {
            curQuests = pspnFlutterObj.getAdwallQuestMap();
        } else {
            curQuests = questsObj.getCurQuests();
        }

        if(curQuests != undefined && curQuests[questId] != undefined) {
            var questData = curQuests[questId];
            if(questData.creatives) {
                var questCreativeMap = xpromoUtilObj.getKeyValuePairForCreatives(questData.creatives);
                if(questCreativeMap.bubblePop_creative != undefined) {

                    var creativeName = questCreativeMap.bubblePop_creative;
                    result = utilObj.fileExists(creativeName);

                    if(!result && downloadOnFail == true) {
                        var contentLink = BUBBLE_POP_CONTENT_DOWNLOAD_LINK + questCreativeMap.bubblePop_creative;
                        xPromoGameSpecificObj.downloadFileV1(contentLink, creativeName);
                    }
                }
            }
        }
        return result;
    };

    /**
     * Method to check whether a particular game has reached its surfacing cooldown in the xpromo
     * @param gameId - game ID for the game
     * @returns {boolean} - Whether the cooldown is reached or not
     */
    this.isViewOrClickCooldownReached = function(gameId) {
        var result = false;

        var runtimeValues = bubblePopObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;

        if (bubblePop.xpromoCooldowns[gameId] != undefined && (bubblePop.xpromoCooldowns[gameId].viewCount >= xpromoViewCoolDown || bubblePop.xpromoCooldowns[gameId].clickCount >= xpromoClickCoolDown)) {
            result = true;
        }

        return result;
    };

    /**
     * Method to start the xpromo quest and save the details. Used again when the xpromo-ed game is installed and the player needs to be rewarded
     * @param gameId - game ID for the game
     * @param questId - adwall quest id from the game
     * @param bubbleName - Name of the bubble to be marked
     */
    this.markXpromoInteraction = function (gameId, questId, bubbleName) {
        if(!bubblePopObj.isBubblePopActive()) {
            return;
        }

        if(bubblePop == null || Object.keys(specialRewardGrant).length === 0 || bubblePop.xpromoCooldowns[gameId] == undefined) {
            return;
        }

        questsObj.makeRelatedQuestsStatusStarted(questId);
        xpromoUtilObj.markXpromoStartedFromOtherSource(gameId, xpromoConstantsObj.TRACK_BUBBLE_POP);

        bubblePop.xpromoClicks[gameId] = {
            "ts": utilObj.getCurrentTimestamp(),
            "rewards": utilObj.createClone(specialRewardGrant),
            "questId": questId,
            "bubbleName": bubbleName
        };

        bubblePop.xpromoCooldowns[gameId].clickCount++;

        // Incase W2E was earlier seen for this game, lets update the grant value to match whats currently being shown.
        if(xpromoW2eObj.getXpromoGameId() == gameId && (bubblePop.xpromoClicks[gameId]["rewards"]).hasOwnProperty("coins")) {
            xpromoW2eObj.setMultipliedLastCoinsAdded(bubblePop.xpromoClicks[gameId]["rewards"]["coins"]);
        }


    };

    /**
     * Method to check the completion of a xpromo initiated from the Bubble Pop flow and reward the player about the same.
     */
    this.checkAndGrantXpromoReward = function () {
        if(!bubblePopObj.isBubblePopActive()) { // If Bubble Pop V1 is in-active, check if V2 is active
            bubblePopNewObj.checkAndGrantXpromoReward();
            return;
        }

        if(bubblePop == null || bubblePop.xpromoClicks == undefined) {
            return;
        }

        var attributionPeriod = xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_SURFACING_ATTRIBUTION_WINDOW) * 60 * 60;
        var curTime = utilObj.getCurrentTimestamp();
        utilObj.resetIsAppInstalledMap();
        for(var gameId in bubblePop.xpromoClicks) {
            var delEntry = false;
            var clickData = bubblePop.xpromoClicks[gameId];
            var clickTime = clickData["ts"];
            var questId = clickData["questId"];
            var bubbleName = clickData["bubbleName"];

            var incentivizedElsewhere = xpromoUtilObj.isGameAlreadyMarkedForRewarding(gameId, xpromoConstantsObj.TRACK_BUBBLE_POP);

            if(!incentivizedElsewhere && clickTime > 0 && ((clickTime + attributionPeriod) >= curTime)
                && xpromoUtilObj.isAppInstalledByGameId(gameId)) {

                // This is to account for the scenario when W2E for the same game is seen after BP,
                // In this scenario, only the W2E reward should be granted and not BP. Hence the check for gameId below

                var questCondition = false;
                if(pspnFlutterObj.isFlutterEnabled()) {
                    questCondition = pspnFlutterObj.isInstallQuestEligibleForGame(parseInt(gameId));
                } else {
                    questCondition = questsObj.getCrossGameQuestStatusForId(questId) != xpromoConstantsObj.QUEST_CLAIMED;
                }

                if(xpromoW2eObj.getXpromoGameId() != gameId && questCondition && adWallObj.getXpromoInteractionTime(gameId) < clickTime) {

                    // Lets get the current and last screen layers
                    var layerForAnimationOnLastScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(lastScreen);
                    var layerForAnimationOnCurScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen);
                    var paramsMap = {"rewards": clickData["rewards"], "source": "bubblepop-xpromo", "bubbleName": bubbleName, "phylum": questId};

                    // If the popup is open and we are on the xpromo screen, lets close and do a delayed reward grant.
                    if (popupLayer.popupOpen && (xPromoGameSpecificObj.isScreenForW2eXpromo(lastScreen))) {
                        popupLayer.closeWithDefault();
                        utilObj.delayedCallback(layerForAnimationOnLastScreen, bubblePopObj.grantSpecialReward, 2, paramsMap, false);
                    } else { // Else grant the reward immediately
                        bubblePopObj.grantSpecialReward(layerForAnimationOnCurScreen, paramsMap);
                    }
                    questsObj.makeRelatedQuestsForW2eClaimed(questId);
                }
                delEntry = true;
            }

            if (((clickTime + attributionPeriod) < curTime) || incentivizedElsewhere) {
                delEntry = true;
            }

            if(delEntry) { // Lets delete the rewards for the gameId as they are either granted to expired
                delete bubblePop.xpromoClicks[gameId];
            }
        }
    };

    /**
     * Getter method to retrieve the xpromo click time for a given game id
     * @param gameId
     * @returns number value of timestamp last interacted, zero if not applicable.
     */
    this.getXpromoInteractionTime = function (gameId) {
        var result = 1;
        if(!bubblePopObj.isBubblePopActive()) { // If Bubble Pop V1 is in-active, check if V2 is active
            return bubblePopNewObj.getXpromoInteractionTime(gameId);
        }

        if(bubblePop == null || bubblePop.xpromoClicks == undefined ||
            bubblePop.xpromoClicks[gameId] == undefined || bubblePop.xpromoClicks[gameId]["ts"] == undefined) {
            return result;
        }

        return bubblePop.xpromoClicks[gameId]["ts"];
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Xpromo bubble specific conditions //////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for JSON called methods //////////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Helper method to get the number of bubbles shown in the day
     * @returns {number} - bubbles shown today
     */
    this.getBubblesShownInTheDay = function() {
        if(bubblePop == null || !bubblePop.hasOwnProperty("bubblesShownInDay")) {
            return constantsObj.NOT_FOUND;
        }
        return bubblePop.bubblesShownInDay;
    };

    /**
     * Helper method to get the number of days since any other PS game is installed
     * @returns {number} - days since other PS game installed
     */
    this.daysSinceInstalledOtherGame = function() {
        var days = xpromoUtilObj.daysSinceInstalledOtherGame();
        if(days == constantsObj.NOT_FOUND) {
            days = MAX_DAYS_LIMIT;
        }
        return days;
    };

    /**
     * Helper method to get the number of days since any other PS game played
     * @returns {number} - days since other PS game played
     */
    this.daysSincePlayedOtherGame = function() {
        var days = xpromoUtilObj.daysSincePlayedOtherGame();
        if(days == constantsObj.NOT_FOUND) {
            days = MAX_DAYS_LIMIT;
        }
        return days;
    };

    /**
     * Helper method to get the number of times for which a particular bubble is not shown
     * @param actionType - Bubble Type
     * @returns {number} - Number of times a bubble has been skipped consecutively
     */
    this.getOtherBubblesSeenConsecutivelyToday = function(actionType) {
        var index = bubblePopObj.getIndexByBubbleType(actionType);

        if(index != constantsObj.NOT_FOUND) {
            return bubblePop.bubbleSkippedTimes[index];
        }
        return constantsObj.NOT_FOUND;
    };

    /**
     * Number of days install if no PS game is installed on the device
     * @returns {number}
     */
    this.daysSinceLoneInstall = function() {
        if(!xpromoUtilObj.isAnyPsGameInstalled()) {
            return xpromoUtilObj.getUserDaysSinceInstall();
        }
        return constantsObj.NOT_FOUND;
    };

    /**
     * Helper method to get the minutes since a particular bubble was last shown
     * @param actionType - Bubble type
     * @returns {number} - Number corresponding to minutes
     */
    this.getMinutesSinceBubbleShown = function(actionType) {
        var index = bubblePopObj.getIndexByBubbleType(actionType);

        if(index != constantsObj.NOT_FOUND) {
            return xpromoUtilObj.getMinutesDiffFromSeconds(utilObj.getCurrentTimestamp(), bubblePop.lastShown[index]);
        }
        return constantsObj.NOT_FOUND;
    };


    /**
     * Helper method to get the minutes since the last bubble was shown
     * @returns {number} - Number corresponding to minutes
     */
    this.getMinutesSinceAnyBubbleShown = function ()  {
        var result = MAX_DAYS_LIMIT * 24 * 60;
        for (var type in bubbleTypeIndexMap) {
            var index = bubbleTypeIndexMap[type];
            if(index != constantsObj.NOT_FOUND) {
                var mins = xpromoUtilObj.getMinutesDiffFromSeconds(utilObj.getCurrentTimestamp(), bubblePop.lastShown[index]);
                result = Math.min(mins, result);
            }
        }
        return result;
    };

    /**
     * Helper method to get the number of days since the player has watched a video
     * @returns {number} - Number corresponding to days
     */
    this.getDaysSinceVideoWatched = function() {
        if(bubblePop == null || !bubblePop.hasOwnProperty("daysSinceVideoWatched")) {
            return constantsObj.NOT_FOUND;
        }
        return bubblePop.daysSinceVideoWatched;
    };

    /**
     * Helper method to check if any game is eligible for the xpromo
     * IMPORTANT NOTE: This method also initiates the download of creatives,
     * hence might return false for the first ever time since the creatives wont be downloaded
     * 
     * @returns {boolean} - True if a game is found, false other wise
     */
    this.anyEligibleGames = function () {
        var result = false;
        var adwallQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopObj.getRuntimes();
        var questPrioritiesArr = Object.keys(adwallQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        if (questPrioritiesArr.length > 0) {
            for (var i = 0; (i < questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
                var game = questPrioritiesArr[i];
                var adwallQuest = adwallQuests[game];
                var gameId = adwallQuest.gh,
                    questId = adwallQuest.id;

                if (!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId)) {
                    continue;
                }

                if(bubblePopObj.isViewOrClickCooldownReached(gameId)) {
                    continue;
                }

                if(bubblePopObj.isBubblePopCreativeAvailable(questId, true)) {
                    result = result || true;
                }
            }
        }
        return result;
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for JSON called methods ////////////////////////
    ////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Tracking Utility methods  ////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Utility method to clear the tracking map
     */
    this.initiateBubbleTracking = function() {
        trackingMap = {};
    };

    /**
     * Utility method to store a particular field - value pair in the tracking map
     * @param field - string for the tracking field
     * @param value - string value that needs to be stored
     */
    this.setTrackingDetails = function(field, value) {
        trackingMap[field] = value;
    };

    /**
     * Method to run the track call with the given set of parameters set.
     * This design was added to handle flows where different parameters were set in different methods of the flow and
     * passing the data between them was cumbersome.
     */
    this.applyTracking = function() {
        var keys = Object.keys(trackingMap);
        if(keys.length > 0) {
            var phylum = (trackingMap[xpromoConstantsObj.TRACK_PHYLUM] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_PHYLUM] : "";
            var cls = (trackingMap[xpromoConstantsObj.TRACK_CLASS] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_CLASS] : "";
            var family = (trackingMap[xpromoConstantsObj.TRACK_FAMILY] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_FAMILY] : "";
            var genus = (trackingMap[xpromoConstantsObj.TRACK_GENUS] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_GENUS] : "";
            var species = (trackingMap[xpromoConstantsObj.TRACK_SPECIES] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_SPECIES] : "";

            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, phylum, cls,
                xpromoConstantsObj.TRACK_BUBBLE_POP, family, genus, species, "", "");
        }
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Tracking Utility methods  //////////////////
    ////////////////////////////////////////////////////////////////////////


};
