XPromoBubblePopNew = function () {
    var EXP_NAME_OLD = "PSPNbubblePop";
    var EXP_NAME = "PSPNbubblePopNew";
    var EXP_ADDITIONAL_XPROMO = "PSPNadditionalRewardXpromo";

    var bubblePopV2 = null; // feature progress data - used for syncing to the server as well
    var rt = null; // local variable to hold the currently evaluated runtime values
    var runtimeFetched = false; // control variable to avoid fetch runtimes over and over again
    var triggerLock = false; // control variable for the flow - makes it easy for async flows later
    var lastRuntimesSyncTime = 0; // last time runtimes were synced

    var bubblePopConfigData = undefined; // local variable to hold the configuration data loaded from JSON
    var bubblePopConfigLoaded = false; // flag to ensure having bubble pop config is loaded in locally

    var CHECK_INTERVAL = 60;
    var MIN_DELAY_TO_TRIGGER = 5;
    var MAX_DELAY_TO_TRIGGER = 15;
    var MAX_DAYS_LIMIT = 3000; // Rather than using Number.MAX_VALUE, lets use an improbably high number

    var isBubbleOnScreen = false;
    var bubbleTypeIndex = -1;
    var bubbleTypeName = "";
    var bubbleShownAt = 0;
    var coinsToGrant = 0;
    var isPopVideoPlaying = false;
    var popVideoType = constantsObj.NOT_FOUND;
    var popXpromoType = constantsObj.NOT_FOUND;
    var popIapType = constantsObj.NOT_FOUND;
    var specialRewardGrant = {};
    var additionalRewardVariant = -1;

    var trackingMap = {};
    var trackingForPhylum = "";

    // Bubble types
    var NUM_BUBBLE_TYPES = 6;

    var BUBBLE_COIN_GRANT = 0;
    var BUBBLE_VIDEO = 1;
    var BUBBLE_XPROMO = 2;
    var BUBBLE_IAP = 3;
    var BUBBLE_WORD = 4;
    var BUBBLE_XPROMO_WORD = 5;

    // HISTORY SIZE
    var HISTORY_SIZE = 11;

    // Config JSON Strings
    var CONFIG_ACTIONS = "actions";
    var CONFIG_ACTIONS_TYPE = "type";

    var CONFIG_TRIGGERS = "triggers";
    var CONFIG_ELIGIBILITY = "eligibility";
    var CONFIG_SURFACING = "surfacing";
    var CONFIG_REWARDS = "rewards";

    var CONFIG_GLOBAL_SURFACING = "globalSurfacing";

    var CONFIG_NAME = "name";
    var CONFIG_PROGRESS_FETCH = "getProgress";
    var CONFIG_RUNTIMECHECK = "runtimeCheck";
    var CONFIG_TARGETVALUE = "targetValue";
    var CONFIG_OPERATOR = "operator";
    var CONFIG_PASS_TYPE = "passType";
    var CONFIG_MULTIPLIERS = "multipliers";
    var CONFIG_OR_CONDITION = "orCondition";
    var CONFIG_RULES = "rules";

    var CONFIG_OPERATOR_EQUAL = "equals";
    var CONFIG_OPERATOR_LESS_THAN = "lessThan";
    var CONFIG_OPERATOR_LESS_THAN_EQUAL = "lessThanEqual";
    var CONFIG_OPERATOR_MORE_THAN = "moreThan";
    var CONFIG_OPERATOR_MORE_THAN_EQUAL = "moreThanEqual";

    var bubbleTypeIndexMap = {
        "iap": BUBBLE_IAP,
        "xpromo": BUBBLE_XPROMO,
        "video": BUBBLE_VIDEO,
        "coin": BUBBLE_COIN_GRANT,
        "xpromo-word": BUBBLE_XPROMO_WORD,
        "word": BUBBLE_WORD
    };

    var BUBBLE_POP_CONTENT_DOWNLOAD_LINK = constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/bubblePop/creatives/";

    this.init = function () {

        //Lets load the JSON configurations
        cc.loader.loadJson(res.bubblepopNew_config_json, function (error, data) {
            if (error == null && data != null && data.hasOwnProperty(constantsObj.GAME_ID)) {
                bubblePopConfigData = data[constantsObj.GAME_ID];
                bubblePopConfigLoaded = true;
                bubblePopNewObj.sortConfigByPriority();
            }
        });

        var emptyList = [0, 0, 0, 0, 0, 0]; // Default values for all types of bubbles.
        rt = {
            // Runtimes for trigger conditions start
            puzzlePercentage: [0.5, 0.8, 1.1, 1.5, 0, 0], // Percentage multiplier on the median puzzles played in last X days
            timeSpent: [100, 300, 500, 900, 0, 0], // Time spent in the game on the given day
            cooldownUpper: [1, 1, 1, 1, 1, 0], // Max Cooldown applicable for different bubble types - in days
            // Runtimes for trigger conditions end

            // Runtimes for eligibility conditions start
            daysSinceInstall: [7, 7, 7, 7, 0, 0], // Days after install date that a bubble is visible
            minBubblesInDay: [0, 1, 1, 1, 0, 0], // Minimum number of bubbles which need to be shown today before special bubble types becomes eligible
            cooldownLower: [1, 1, 1, 1, 0 ,0], // Min Cooldown applicable for different bubble types - in days
            payerCheck: [2, 2, 2, 2, 2, 2], // Flag to control this feature for payer / non-payer. 0 - Only Non Payer, 1 - Only Payer, 2 - Both
            maxBubblesInDayNew: [1, 1, 1, 1, 5, 5], // Max number of bubbles which can be shown in a day for every bubble type
            minLevelNew: [2, 2, 2, 2, 0, 0], // Number of levels that need to completed before showing a specific bubble type
            // Runtimes for eligibility conditions end

            // Runtimes for surfacing conditions start
            trackingSwitch : [0, 0, 0, 0, 0, 0], //Tracking switch if set for bubble throws tracking of bubble even if mainSwitch is off
            mainSwitch: [0, 0, 0, 0, 1, 0], // Main switch if a particular bubble type is to be shown or not
            maxBubblesInDay: 10, // Max number of bubbles which can be shown in a day (overall)
            minTimeBetweenTwoBubbles: 0, // Min interval between bubbles
            // Runtimes for surfacing conditions end

            // Values related to coins start
            coinRewardValues: [1, 2, 3], // Coins to be granted on popping any type of bubble
            coinRewardProb: [40, 40, 20], // Probabilities for the corresponding coin grant value - should add up to 100.
            // Values related to coins end

            // Bubble Display Property start
            sound: 1, // Whether the sound is enabled on the bubble
            bubbleFloatTime: 15, // Total time for which the bubble would be on screen before auto pop happens
            bubbleUnstableTime: 5, // Time after which the animation for creating urgency is to be started
            // Bubble Display Property start

            // Base reward values start
            cashGrant: [0, 10, 100, 200, 0, 0], // Cash grant for different bubble types.
            grantMultiplier: [1, 2, 2, 1, 0, 0], // Cash grant multiplier when special reward conditions pass
            // Base reward values end

            // Engagement based multiplier start
            engagementDays: [1, 1, 1, 1, 0, 0], // Engagement days below which addtional engagement multiplier trigger
            engagementMultiplier: [2, 2, 2, 2, 0, 0], // Engagement multiplier
            // Engagement based multiplier end

            // Values related to Reward Multiplier for Video start
            videoLow: 3, // X Dau Days in which video is not watched
            videoHigh: 7, // X Calendar Days in which video is not watched
            // Values related to Reward Multiplier for Video end


            // Values related to Reward Multiplier for Xpromo start
            installedPSGame: 30, // Minimum days since user has installed another PS game
            // Values related to Reward Multiplier for Xpromo end

            // Cooldown for xpromo start
            xpromoViewCooldown: 3, // Number of views after which the xpromo-ed game goes to cooldown
            xpromoClickCooldown: 3, // Number of clicks after which the xpromo-ed game goes to cooldown
            xpromoGamesLimit: 5, // Number of top priority games to be xpromo-ed (including self)
            // Cooldown for xpromo end

            // Values related to Reward for IAP start
            daysSincePurchase: 3, // Number of days since purchase for IAP Bubble to trigger
            wallet: 100, // Wallet thresholds applicable
            iapMaxCoins: 500, // Max number of coins that can be rewarded from IAP
            iapUpsells: 2, // Max number of times a package purchased through Bubble pop is upselled
            // Values related to Reward for IAP end

            //Values related to xpromo word and word bubble start
            minWalletBalance:[100,100], // Minimum wallet balance requirement
            cohortArr:[1,5,7], //Array that denotes the different cohorts of the user
            cohortValues:[0,0,0], //Array that denotes min Dau days of the users in the respective cohorts
            hasUserEverXpromoted :  [0, 2], // Flag to control this feature for Xpromoted / Hasn't Xpromoted. 0 - Only hasnt Xpromoted, 1 - Only has Xpromoted, 2 - Both
            willCoinsBeGranted : [0, 0], //Flag to tell whether coins will be granted on word bubble Pop
            runtimeLeafReward : [1,1], //Leaf reward for bubble Tap
            runtimePriority : [1 ,2 ,3 ,4 ,5 ,6], //Runtime based priority, sorts config from smallest(Highest priority) to largest
            runtimeLeafRewardXpromo:25, //Leaf reward for Xpromo
            gamesForAdditionalXpromo : [8, 9, 10, 11, 14, 15, 16], //Array of Games that would host xpromo Quest
            levelsForAdditionalXpromo : [2, 10, 1, 10, 10, 10, 1], //Array of levels in the respective games
            //Values related to xpromo word and word bubble end
        };
        bubblePopV2 = {
            lastCheck: 0,
            lastShown: utilObj.createClone(emptyList),
            lastShownInSession: utilObj.createClone(emptyList),
            bubblesShownInDay: 0,
            bubbleSkippedTimes: utilObj.createClone(emptyList),
            daysSinceVideoWatched: 0,
            lastUpdateTime: utilObj.getCurrentTimestamp(),
            xpromoClicks: {},
            xpromoCooldowns: {},
            xpromoWordClicks : {},
            xpromoWordCooldowns :{},
            w2ePopupDisabledForBubblePop: false,

            initTime: utilObj.getCurrentTimestamp(),
            dauDays: [1],
            puzzlesHistory: [0],
            videoHistory: [0],
            bubblesShownInDayIndividual: utilObj.createClone(emptyList),

            iapPackageToShow: "",
            iapPackageShownTimes: 0,
            iapQuantity: constantsObj.NOT_FOUND,

        };
    };

    this.load = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }

        if (quests.bubblePopV2 != undefined) {
            bubblePopV2 = quests.bubblePopV2;
        }

        if(bubblePopV2.lastShown == undefined ) {
            bubblePopV2.lastShown = [0, 0, 0, 0, 0, 0];
        } else if (bubblePopV2.lastShown.length != NUM_BUBBLE_TYPES) {
            if (bubblePopV2.lastShown.length < NUM_BUBBLE_TYPES){
                bubblePopV2.lastShown = bubblePopV2.lastShown.concat(Array(NUM_BUBBLE_TYPES - bubblePopV2.lastShown.length).fill(0));
            } else {
                bubblePopV2.lastShown = bubblePopV2.lastShown.slice(0, NUM_BUBBLE_TYPES);
            }
        }

        if(bubblePopV2.lastShownInSession == undefined ) {
            bubblePopV2.lastShownInSession = [0, 0, 0, 0, 0, 0];
        } else if(bubblePopV2.lastShownInSession.length != NUM_BUBBLE_TYPES) {
            if(bubblePopV2.lastShownInSession.length < NUM_BUBBLE_TYPES){
                bubblePopV2.lastShownInSession = bubblePopV2.lastShownInSession.concat(Array(NUM_BUBBLE_TYPES - bubblePopV2.lastShownInSession.length).fill(0));
            } else {
                bubblePopV2.lastShownInSession = bubblePopV2.lastShownInSession.slice(0,NUM_BUBBLE_TYPES);
            }
        }

        if(bubblePopV2.bubblesShownInDay == undefined) {
            bubblePopV2.bubblesShownInDay = 0;
        }

        if(bubblePopV2.bubbleSkippedTimes == undefined) {
            bubblePopV2.bubbleSkippedTimes = [0, 0, 0, 0, 0, 0];
        } else if (bubblePopV2.bubbleSkippedTimes.length != NUM_BUBBLE_TYPES) {
            if (bubblePopV2.bubbleSkippedTimes.length < NUM_BUBBLE_TYPES){
                bubblePopV2.bubbleSkippedTimes = bubblePopV2.bubbleSkippedTimes.concat(Array(NUM_BUBBLE_TYPES - bubblePopV2.bubbleSkippedTimes.length).fill(0));
            } else {
                bubblePopV2.bubbleSkippedTimes = bubblePopV2.bubbleSkippedTimes.slice(0, NUM_BUBBLE_TYPES);
            }
        }

        if(bubblePopV2.xpromoClicks == undefined) {
            bubblePopV2.xpromoClicks = {};
        }

        if(bubblePopV2.xpromoWordClicks == undefined){
            bubblePopV2.xpromoWordClicks =  {};
        }

        if (bubblePopV2.xpromoCooldowns == undefined) {
            bubblePopV2.xpromoCooldowns = {};
        }

        if (bubblePopV2.xpromoWordCooldowns == undefined){
            bubblePopV2.xpromoWordCooldowns = {};
        }

        if(bubblePopV2.lastUpdateTime == undefined) {
            bubblePopV2.lastUpdateTime = utilObj.getCurrentTimestamp();
        }

        if (bubblePopV2.w2ePopupDisabledForBubblePop == undefined) {
            bubblePopV2.w2ePopupDisabledForBubblePop = false;
        }

        if(bubblePopV2.initTime == undefined) {
            bubblePopV2.initTime = utilObj.getCurrentTimestamp();
        }

        if(bubblePopV2.dauDays == undefined) {
            bubblePopV2.dauDays = [1];
        }

        if(bubblePopV2.puzzlesHistory == undefined) {
            bubblePopV2.puzzlesHistory = [0];
        }

        if(bubblePopV2.videoHistory == undefined) {
            bubblePopV2.videoHistory = [0];
        }

        if(bubblePopV2.bubblesShownInDayIndividual == undefined) {
            bubblePopV2.bubblesShownInDayIndividual = [0, 0, 0, 0, 0, 0];
        } else if (bubblePopV2.bubblesShownInDayIndividual.length != NUM_BUBBLE_TYPES) {
            if (bubblePopV2.bubblesShownInDayIndividual.length < NUM_BUBBLE_TYPES){
                bubblePopV2.bubblesShownInDayIndividual = bubblePopV2.bubblesShownInDayIndividual.concat(Array(NUM_BUBBLE_TYPES - bubblePopV2.bubblesShownInDayIndividual.length).fill(0));
            } else {
                bubblePopV2.bubblesShownInDayIndividual = bubblePopV2.bubblesShownInDayIndividual.slice(0, NUM_BUBBLE_TYPES);
            }
        }

        // This block is added as a correction logic for the data arrays
        // The data migration bug had left some of the indexes having undefined / NaN values
        for(var index = 0; index < NUM_BUBBLE_TYPES; index++) {
            if(bubblePopV2.lastShown[index] == undefined || isNaN(bubblePopV2.lastShown[index]) || isNaN(parseInt(bubblePopV2.lastShown[index]))) {
                bubblePopV2.lastShown[index] = 0;
            }
            if(bubblePopV2.lastShownInSession[index] == undefined || isNaN(bubblePopV2.lastShownInSession[index]) || isNaN(parseInt(bubblePopV2.lastShownInSession[index]))) {
                bubblePopV2.lastShownInSession[index] = 0;
            }
            if(bubblePopV2.bubbleSkippedTimes[index] == undefined || isNaN(bubblePopV2.bubbleSkippedTimes[index]) || isNaN(parseInt(bubblePopV2.bubbleSkippedTimes[index]))) {
                bubblePopV2.bubbleSkippedTimes[index] = 0;
            }
            if(bubblePopV2.bubblesShownInDayIndividual[index] == undefined || isNaN(bubblePopV2.bubblesShownInDayIndividual[index]) || isNaN(parseInt(bubblePopV2.bubblesShownInDayIndividual[index]))) {
                bubblePopV2.bubblesShownInDayIndividual[index] = 0;
            }
        }

        if(bubblePopV2.iapPackageToShow == undefined) {
            bubblePopV2.iapPackageToShow = "";
        }

        if(bubblePopV2.iapPackageShownTimes == undefined) {
            bubblePopV2.iapPackageShownTimes = 0;
        }

        if(bubblePopV2.iapQuantity == undefined) {
            bubblePopV2.iapQuantity = constantsObj.NOT_FOUND;
        }
    };

    this.save = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }

        // Setting the daily data before saving the bubblePopV2 Object
        if(bubblePopV2.puzzlesHistory != null && bubblePopV2.puzzlesHistory.length > 0) {
            bubblePopV2.puzzlesHistory[bubblePopV2.puzzlesHistory.length - 1] = xPromoGameSpecificObj.getDailyPuzzleClears()
        } else {
            bubblePopV2.puzzlesHistory = [xPromoGameSpecificObj.getDailyPuzzleClears()];
        }

        if(bubblePopV2.videoHistory != null && bubblePopV2.videoHistory.length > 0) {
            bubblePopV2.videoHistory[bubblePopV2.videoHistory.length - 1] = xPromoGameSpecificObj.getDailyWatchVideosCount();
        } else {
            bubblePopV2.videoHistory = [xPromoGameSpecificObj.getDailyWatchVideosCount()];
        }

        if(bubblePopV2.dauDays != null && bubblePopV2.dauDays.length > 0) {
            bubblePopV2.dauDays[bubblePopV2.dauDays.length - 1] = 1
        } else {
            bubblePopV2.dauDays = [1];
        }

        quests.bubblePopV2 = bubblePopV2;
        // loggerObj.debugLog("Saving bubblePopV2 Data=" + utilObj.getStringifiedJson(bubblePopV2));
        return quests;
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Basic Bubble Pop methods /////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Method to return the current variant of the bubble pop master experiment
     * @returns {number} - number corresponding to the variant, 0 for control / off.
     */
    this.getBubblePopExperimentVariant = function() {
        // Bubble Pop New Experiment is enabled only if the older experiment is in control variant
        if(epmObj.getChosenVariant(EXP_NAME_OLD) == xpromoConstantsObj.EXP_VAR_CONTROL) {
            return epmObj.getChosenVariant(EXP_NAME);
        }
        return xpromoConstantsObj.EXP_VAR_CONTROL;
    };

    this.getAdditionalRewardForXpromoVariant = function () {
        if(additionalRewardVariant == -1){
            if(xPromoGameSpecificObj.isBubblePopAdditionalRewardEnabled()){
                additionalRewardVariant = epmObj.getChosenVariant(EXP_ADDITIONAL_XPROMO);
            } else {
                additionalRewardVariant = constantsObj.EXP_VAR_CONTROL;
            }
        }
        return additionalRewardVariant;
    };

    /**
     * Method to check if the bubble pop is feature is currently active or in-active for a player
     * @returns {boolean} - True / False
     */
    this.isBubblePopActive = function() {
        return xPromoGameSpecificObj.isBubblePopV2Enabled() && (bubblePopNewObj.getBubblePopExperimentVariant() != xpromoConstantsObj.EXP_VAR_CONTROL);
    };

    /**
     * Method to fetch runtime value based on the experiment variant. Also caches the value for the rest of the session.
     * @returns {object}  - Returns a JS object with runtime values.
     */
    this.getRuntimes = function (tryForceFetch) {
        if(tryForceFetch == undefined || tryForceFetch != true || !utilObj.isIos()) {
            tryForceFetch = false;
        }

        if(runtimeFetched && (!tryForceFetch || (utilObj.getCurrentTimestamp() - lastRuntimesSyncTime) <= constantsObj.SECONDS_IN_HOUR)) {
            return rt;
        }

        additionalRewardVariant = -1; // Resetting additionalRewardVariant to -1 so that the latest experiment value is synced
        var updatedRuntime = null;
        var variant = bubblePopNewObj.getBubblePopExperimentVariant();
        var defaultKey = "bpn1";
        var runtimeKey = defaultKey;
        if(variant > constantsObj.EXP_VAR_CONTROL) {
            runtimeKey = "bpn" + (parseInt(variant)).toString();
        }
        // Different configuration is expected for different variants.
        updatedRuntime = xpromoQuestRuntimesObj.getRuntime(runtimeKey, null);
        if(updatedRuntime == null || updatedRuntime == undefined ) {
            updatedRuntime = xpromoQuestRuntimesObj.getRuntime(defaultKey, null);
        }
        if (updatedRuntime != null) {
            for (var i = 0; i < updatedRuntime.pp.length; i++) {
                updatedRuntime.pp[i] = parseFloat(updatedRuntime.pp[i]);
                updatedRuntime.ts[i] = parseInt(updatedRuntime.ts[i]);
                updatedRuntime.cl[i] = parseInt(updatedRuntime.cl[i]);

                updatedRuntime.dsi[i] = parseInt(updatedRuntime.dsi[i]);
                updatedRuntime.mib[i] = parseInt(updatedRuntime.mib[i]);
                updatedRuntime.cu[i] = parseInt(updatedRuntime.cu[i]);
                updatedRuntime.pc[i] = parseInt(updatedRuntime.pc[i]);
                updatedRuntime.mabn[i] = parseInt(updatedRuntime.mabn[i]);
                updatedRuntime.mln[i] = parseInt(updatedRuntime.mln[i]);

                updatedRuntime.mst[i] = parseInt(updatedRuntime.mst[i]);

                updatedRuntime.cg[i] = parseInt(updatedRuntime.cg[i]);
                updatedRuntime.gm[i] = parseFloat(updatedRuntime.gm[i]);

                updatedRuntime.ed[i] = parseInt(updatedRuntime.ed[i]);
                updatedRuntime.em[i] = parseFloat(updatedRuntime.em[i]);
            }
            for (var i = 0; i < updatedRuntime.cr.length; i++) {
                updatedRuntime.cr[i] = parseInt(updatedRuntime.cr[i]);
                updatedRuntime.crp[i] = parseInt(updatedRuntime.crp[i]);
            }


            rt.puzzlePercentage = updatedRuntime.pp;
            rt.timeSpent = updatedRuntime.ts;
            rt.cooldownUpper = updatedRuntime.cu;

            rt.daysSinceInstall = updatedRuntime.dsi;
            rt.minBubblesInDay = updatedRuntime.mib;
            rt.cooldownLower = updatedRuntime.cl;
            rt.payerCheck = updatedRuntime.pc;
            rt.maxBubblesInDayNew = updatedRuntime.mabn;
            rt.minLevelNew =  updatedRuntime.mln;

            rt.mainSwitch =  utilObj.createClone(updatedRuntime.mst); // Keeping a clone of the value as we are overwritting this parameter manually during flow
            rt.maxBubblesInDay = parseInt(updatedRuntime.mab[0]);
            rt.minTimeBetweenTwoBubbles = parseInt(updatedRuntime.mint[0]);

            rt.coinRewardValues = updatedRuntime.cr;
            rt.coinRewardProb = updatedRuntime.crp;

            rt.sound = parseInt(updatedRuntime.s[0]);
            rt.bubbleFloatTime = parseInt(updatedRuntime.bft[0]);
            rt.bubbleUnstableTime = parseInt(updatedRuntime.but[0]);

            rt.cashGrant = updatedRuntime.cg;
            rt.grantMultiplier = updatedRuntime.gm;

            rt.engagementDays = updatedRuntime.ed;
            rt.engagementMultiplier = updatedRuntime.em;

            rt.videoLow = parseInt(updatedRuntime.vl[0]);
            rt.videoHigh = parseInt(updatedRuntime.vh[0]);

            rt.installedPSGame = parseInt(updatedRuntime.ipg[0]);

            rt.xpromoClickCooldown = parseInt(updatedRuntime.xcc[0]);
            rt.xpromoViewCooldown = parseInt(updatedRuntime.xvc[0]);
            rt.xpromoGamesLimit = parseInt(updatedRuntime.xgl[0]);

            rt.daysSincePurchase = parseInt(updatedRuntime.dsp[0]);
            rt.wallet = parseInt(updatedRuntime.w[0]);
            rt.iapMaxCoins = parseInt(updatedRuntime.iapm[0]);
            rt.iapUpsells = parseInt(updatedRuntime.iapu[0]);

            if(bubblePopNewObj.checkXpromoAdditionalGrantRuntimes(updatedRuntime) == true){
                for(var i = 0 ;  i < updatedRuntime.gfax.length ; i++){
                    updatedRuntime.gfax[i] = parseInt(updatedRuntime.gfax[i]);
                    updatedRuntime.lfax[i] = parseInt(updatedRuntime.lfax[i]);
                }
                for(var i = 0 ; i < updatedRuntime.cha.length ; i++){
                    updatedRuntime.cha[i] = parseInt(updatedRuntime.cha[i]);
                    updatedRuntime.chv[i] = parseInt(updatedRuntime.chv[i]);
                }
                for(var i = 0 ; i < updatedRuntime.rtp.length ; i++){
                    updatedRuntime.rtp[i] = parseInt(updatedRuntime.rtp[i]);
                    updatedRuntime.trst[i] = parseInt(updatedRuntime.trst[i]);
                }
                for(var i = 0 ; i < updatedRuntime.hux.length ; i++){
                    updatedRuntime.hux[i] = parseInt(updatedRuntime.hux[i]);
                    updatedRuntime.wbcg[i] = parseInt(updatedRuntime.wbcg[i]);
                    updatedRuntime.mawb[i] = parseInt(updatedRuntime.mawb[i]);
                    updatedRuntime.wswb[i] = parseInt(updatedRuntime.wswb[i]);
                }
                rt.willCoinsBeGranted = updatedRuntime.wbcg;
                rt.hasUserEverXpromoted = updatedRuntime.hux;
                rt.minWalletBalance = updatedRuntime.mawb;
                rt.runtimeLeafReward = updatedRuntime.wswb;
                rt.cohortArr = updatedRuntime.cha;
                rt.cohortValues = updatedRuntime.chv;
                rt.runtimePriority = updatedRuntime.rtp;
                rt.runtimeLeafRewardXpromo = parseInt(updatedRuntime.wswxib);
                rt.gamesForAdditionalXpromo = updatedRuntime.gfax;
                rt.levelsForAdditionalXpromo = updatedRuntime.lfax;
                rt.trackingSwitch = updatedRuntime.trst;
                bubblePopNewObj.sortConfigByPriority(rt.runtimePriority);
                bubblePopNewObj.resetMainSwitchForAdditionalXpromo(rt.mainSwitch);
            }
            runtimeFetched = true;
            lastRuntimesSyncTime = utilObj.getCurrentTimestamp();

        }
        return rt;
    };

    /**
     * Method to check whether runtimes of xpromoAdditional reward exist or not
     * @return {bool}*/
    this.checkXpromoAdditionalGrantRuntimes = function(updatedRuntime) {
        var newRuntimes = [
            "mawb",
            "cha",
            "chv",
            "hux",
            "rtp",
            "wswb",
            "wswxib",
            "wbcg",
            "gfax",
            "lfax",
            "trst",
        ];
        for(var i = 0 ; i < newRuntimes.length ; i++){
            if(!updatedRuntime.hasOwnProperty(newRuntimes[i])){
                return false;
            }
        }

        return true;
    };

    /**
     * Method to sort the config by priority
     * This is done to ensure that the while checking, the first bubble to pass the criteria is of the highest possible priority
     */
    this.sortConfigByPriority = function(runtimePriority) {
        if(bubblePopConfigLoaded && utilObj.isArray(bubblePopConfigData[CONFIG_ACTIONS])) {
            // local function for sorting objects by their priority
            var prioritySorter = function(a, b) {
                return a.priority - b.priority;
            };
            // First lets sort the configurations by priority
            if(bubblePopNewObj.getAdditionalRewardForXpromoVariant() != xpromoConstantsObj.EXP_VAR_CONTROL && runtimePriority != undefined && runtimePriority!= null && runtimePriority.length == NUM_BUBBLE_TYPES){

                var runtimePrioritySorter = function(a,b){
                    return runtimePriority[bubblePopNewObj.getIndexByBubbleType(a["type"])] - runtimePriority[bubblePopNewObj.getIndexByBubbleType(b["type"])];
                };
                bubblePopConfigData[CONFIG_ACTIONS].sort(runtimePrioritySorter);
            } else{
                bubblePopConfigData[CONFIG_ACTIONS].sort(prioritySorter);
            }
            // Now lets sort each of the reward configs in an action by priority
            for (var index = 0; index < bubblePopConfigData[CONFIG_ACTIONS].length; index++) {
                var action = bubblePopConfigData[CONFIG_ACTIONS][index];
                if(utilObj.isArray(action[CONFIG_REWARDS])) {
                    action[CONFIG_REWARDS].sort(prioritySorter);
                }
            }
        }
    };

    /**
     * Get the array index corresponding to the bubble type
     * @param actionType - String representing the type of the bubble
     * @returns {number}
     */
    this.getIndexByBubbleType = function(actionType) {
        if(actionType != undefined && bubbleTypeIndexMap.hasOwnProperty(actionType)) {
            return bubbleTypeIndexMap[actionType];
        }
        return constantsObj.NOT_FOUND;
    };

    /**
     * Wrapper method to process a condition based on runtimeCheck or targetValue
     * @param config - JS Object for the configuration of the condition
     * @param actionType - String representing the type of action this is ("xpromo", "video",... etc)
     * @returns {boolean} - Returns true / false depending if the condition is pass / fail respectively.
     *                      Returns false of unsupported condition type
     */
    this.processCondition = function(config, actionType) {
        var result = false;

        if(config.hasOwnProperty(CONFIG_RUNTIMECHECK) && config[CONFIG_RUNTIMECHECK] != null
            && bubblePopNewObj.evaluateCondition(config, actionType, true)) {
            result = true;

        } else if (config.hasOwnProperty(CONFIG_TARGETVALUE) && config[CONFIG_TARGETVALUE] != null
            && bubblePopNewObj.evaluateCondition(config, actionType, false)){
            result = true;
        }

        return result;
    };

    /**
     * Method to run the condition check string using the eval. Also figures out if the condition requires an index lookup
     * @param config - JS Object for the configuration of the condition
     * @param actionType - String representing the type of action this is ("xpromo", "video",... etc)
     * @param runtimeCheck - true / false depending if it needs a runtime check or targetValue comparision
     * @returns {boolean} Returns true / false depending if the condition is pass / fail respectively.
     */
    this.evaluateCondition = function(config, actionType, runtimeCheck) {
        var result = false;
        if(config.hasOwnProperty(CONFIG_PROGRESS_FETCH) && config[CONFIG_PROGRESS_FETCH] != null) {

            var params = [];
            var queryByBubbleType = false;
            if(config.hasOwnProperty(CONFIG_PASS_TYPE) && config[CONFIG_PASS_TYPE] === true) {
                params.push(actionType);
                queryByBubbleType = true;
            }

            var progress = eval(config[CONFIG_PROGRESS_FETCH]).apply(null, params);
            var runtimeValues = bubblePopNewObj.getRuntimes();

            var requiredProgress = (runtimeCheck) ? runtimeValues[config[CONFIG_RUNTIMECHECK]] : config[CONFIG_TARGETVALUE];
            if(queryByBubbleType && utilObj.isArray(requiredProgress)) {
                var index = bubblePopNewObj.getIndexByBubbleType(actionType);
                if(index != constantsObj.NOT_FOUND) {
                    requiredProgress = requiredProgress[index];
                } else {
                    requiredProgress = constantsObj.NOT_FOUND;
                }
            }
            result = (bubblePopNewObj.evaluateOperator(progress, requiredProgress, config[CONFIG_OPERATOR]));
        }
        return result;
    };

    /**
     * Wrapper method to run an evaluation operation depending on the operator, current and required progress
     * @param curProgress - value representing the current value
     * @param reqProgress - value representing the required value
     * @param operator - operator applied between the current and required value
     * @returns {boolean} - Returns the evaluation of "curProgress <operator> reqProgress"
     */
    this.evaluateOperator = function(curProgress, reqProgress, operator) {
        var result = false;
        if(curProgress != undefined && reqProgress != undefined && operator != undefined) {
            switch (operator) {
                case CONFIG_OPERATOR_EQUAL:
                    result = (curProgress == reqProgress);
                    break;
                case CONFIG_OPERATOR_LESS_THAN:
                    result = (curProgress < reqProgress);
                    break;
                case CONFIG_OPERATOR_LESS_THAN_EQUAL:
                    result = (curProgress <= reqProgress);
                    break;
                case CONFIG_OPERATOR_MORE_THAN:
                    result = (curProgress > reqProgress);
                    break;
                case CONFIG_OPERATOR_MORE_THAN_EQUAL:
                    result = (curProgress >= reqProgress);
                    break;
                default:
                    break;
            }
        }
        return result;
    };

    this.resetMainSwitchForAdditionalXpromo = function(mainSwitch){
        var index_word = bubblePopNewObj.getIndexByBubbleType("word");
        var index_xpromo_word = bubblePopNewObj.getIndexByBubbleType("xpromo-word");
        if(mainSwitch.length == NUM_BUBBLE_TYPES){
            if(bubblePopNewObj.getAdditionalRewardForXpromoVariant() == xpromoConstantsObj.EXP_VAR_CONTROL){
                var index_word = bubblePopNewObj.getIndexByBubbleType("word");
                var index_xpromo_word = bubblePopNewObj.getIndexByBubbleType("xpromo-word");
                mainSwitch[index_word] = 0;
                mainSwitch[index_xpromo_word] = 0;

            } else{

            }
        }
    };

    /**
     * Entry point method which initiates the trigger check for Bubble Pop
     * This method needs to be called from the different trigger points in the game.
     */
    this.initiateTriggerCheck = function () {
        if(!triggerLock && bubblePopNewObj.isBubblePopActive()) {
            var curTimestamp = utilObj.getCurrentTimestamp();
            if (curTimestamp - bubblePopV2.lastCheck < CHECK_INTERVAL) {
                return;
            }
            bubblePopV2.lastCheck = curTimestamp;
            triggerLock = true;
            var bubbleToShow = false;
            if(bubblePopConfigLoaded && utilObj.isArray(bubblePopConfigData[CONFIG_ACTIONS])) {
                // Lets try and load the runtimes if not yet done
                var runtimeValues = bubblePopNewObj.getRuntimes(true);

                // Need to reset Cross Promo cooldowns for games if all are reached
                bubblePopNewObj.checkResetXpromoCooldowns();
                bubblePopNewObj.checkResetXpromoWordCooldowns();

                // Lets iterate over all possible actions and see if any of the triggers pass
                for (var index = 0; index < bubblePopConfigData[CONFIG_ACTIONS].length; index++) {
                    // Lets first check if the main switch is on for a bubble type before initiating checks
                    var action = bubblePopConfigData[CONFIG_ACTIONS][index];
                    var switchIndex = bubblePopNewObj.getIndexByBubbleType(action[CONFIG_ACTIONS_TYPE]);
                    if((utilObj.isArray(runtimeValues.mainSwitch) && (switchIndex <= runtimeValues.mainSwitch.length -1 )&& runtimeValues.mainSwitch[switchIndex] == 1) || ( utilObj.isArray(runtimeValues.trackingSwitch) && runtimeValues.trackingSwitch[switchIndex] == 1) ){
                        var passedTriggers = bubblePopNewObj.checkTrigger(action);

                        // If no triggers have passed, no point evaluating further.
                        if(passedTriggers.length > 0) {
                            bubblePopNewObj.initiateBubbleTracking();
                            if(switchIndex == BUBBLE_WORD || switchIndex == BUBBLE_XPROMO_WORD){
                                bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_ORDER, "bubble_pop_v3");
                            }
                            bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_PHYLUM, passedTriggers[0]); // Pushing the passed trigger
                            bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_GENUS, action[CONFIG_ACTIONS_TYPE]);
                            if(runtimeValues.trackingSwitch[switchIndex] == 1 && runtimeValues.mainSwitch[switchIndex] == 0){
                                var xpromoAd = bubblePopNewObj.getAdditionalRewardForXpromoVariant();
                                var bubblePopOld = bubblePopObj.getBubblePopExperimentVariant();
                                var bubblePopNewVar = bubblePopNewObj.getBubblePopExperimentVariant();
                                var runtimeUsed = (bubblePopNewVar == xpromoConstantsObj.EXP_VAR_CONTROL) ? 1 : bubblePopNewVar;
                                bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_SPECIES, "var"+xpromoAd.toString() + "_var"+bubblePopNewVar.toString()+ "_var"+bubblePopOld.toString() +"_bpn"+ runtimeUsed.toString());
                            }
                            if(bubblePopNewObj.checkEligibility(action) && bubblePopNewObj.checkSurfacing(action, bubblePopConfigData[CONFIG_GLOBAL_SURFACING])) {
                                // All checks are passed, lets calculate the reward grants and keep it in place.
                                bubblePopNewObj.evaluateSpecialRewards(action, passedTriggers);
                                // Lets set the necessary flags and add a delayed call with a random trigger duration for showing the bubble
                                bubbleTypeName = action[CONFIG_ACTIONS_TYPE];
                                bubbleTypeIndex = bubblePopNewObj.getIndexByBubbleType(action[CONFIG_ACTIONS_TYPE]);
                                bubblePopNewObj.setTrackingForPhylum(passedTriggers[0]);
                                if(utilObj.isArray(runtimeValues.mainSwitch) && (switchIndex <= runtimeValues.mainSwitch.length -1 )&& runtimeValues.mainSwitch[switchIndex] == 1) {
                                    bubbleToShow = true;
                                    utilObj.delayedCallback(gameLayer, bubblePopNewObj.showBubble, utilObj.getRandomIntInRange(MIN_DELAY_TO_TRIGGER, MAX_DELAY_TO_TRIGGER), null, false);
                                    loggerObj.debugLog("Trying to show bubble of index=" + bubbleTypeIndex.toString() + " type=" + bubbleTypeName);
                                    bubblePopNewObj.applyTracking();
                                    bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_SPECIES , "");
                                    bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, action[CONFIG_ACTIONS_TYPE]);
                                    break;
                                }
                            }
                            bubblePopNewObj.applyTracking();
                            bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_SPECIES , "");
                        }
                    }
                }
            }
            if(!bubbleToShow) {
                triggerLock = false;
            }
        }
    };

    /**
     * Method that checks if any of the triggers have passed
     * @param action - JS Object for the configuration of the action
     * @returns {Array} - List of strings representing the passed triggers
     */
    this.checkTrigger = function(action) {
        var result = [];
        if(action != undefined && utilObj.isArray(action[CONFIG_TRIGGERS])) {
            for (var index = 0; index < action[CONFIG_TRIGGERS].length; index++) {
                var trigger = action[CONFIG_TRIGGERS][index];

                if(bubblePopNewObj.processCondition(trigger, action[CONFIG_ACTIONS_TYPE])) {
                    result.push(trigger[CONFIG_NAME]);
                }
            }
        }
        return result;
    };

    /**
     * Method that checks all the eligibility criteria for a bubble type
     * @param action - JS Object for the configuration of the action
     * @returns {boolean} - True if all eligibilities have passed, false otherwise
     */
    this.checkEligibility = function(action) {
        var result = true;
        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, "success");

        if(action != undefined && utilObj.isArray(action[CONFIG_ELIGIBILITY])) {
            for (var index = 0; (index < action[CONFIG_ELIGIBILITY].length && result); index++) {
                var eligibility = action[CONFIG_ELIGIBILITY][index];

                if(bubblePopNewObj.processCondition(eligibility, action[CONFIG_ACTIONS_TYPE])) {
                    result = result && true;
                } else {
                    result = false;
                    bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, eligibility[CONFIG_NAME]);
                }
            }
        } else {
            result = false;
        }
        return result;
    };

    /**
     * Method that checks all the surfacing criteria for a bubble type
     * @param action - JS Object for the configuration of the action
     * @param globalSurfacing - JS Object for the configuration of global surfacing
     * @returns {boolean} - True if all surfacings have passed, false otherwise
     */
    this.checkSurfacing = function(action, globalSurfacing) {
        var result = true;
        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, "success");

        if(action != undefined && action.hasOwnProperty(CONFIG_SURFACING)) {
            var surfacings;
            if(action[CONFIG_SURFACING].hasOwnProperty("useGlobal") && action[CONFIG_SURFACING]["useGlobal"] === true
                && utilObj.isArray(globalSurfacing)) {
                surfacings = globalSurfacing;
            } else if (action[CONFIG_SURFACING].hasOwnProperty(CONFIG_RULES) && utilObj.isArray(action[CONFIG_SURFACING][CONFIG_RULES])) {
                surfacings = action[CONFIG_SURFACING][CONFIG_RULES];
            } else {
                result = false;
                bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, "json-data-error");
            }

            if(result) {
                for (var index = 0; (index < surfacings.length && result); index++) {
                    var surf = surfacings[index];

                    if(bubblePopNewObj.processCondition(surf, action[CONFIG_ACTIONS_TYPE])) {
                        result = result && true;
                    } else {
                        result = false;
                        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, surf[CONFIG_NAME]);
                    }
                }
            }
        }
        return result;
    };

    /**
     * Method to evaluate additional / special rewards that need to be granted based on the triggers that have passed
     * and stores it locally for the session
     * @param action - JS Object for the configuration of the action
     * @param passedTriggers - List of all the triggers that have passed for this action type.
     */
    this.evaluateSpecialRewards = function(action, passedTriggers) {
        if(action != undefined && utilObj.isArray(action[CONFIG_REWARDS])) {

            specialRewardGrant = {}; // Setting special rewards to null - to remove any existing reward

            for (var index = 0; index < action[CONFIG_REWARDS].length; index++) {
                var reward = action[CONFIG_REWARDS][index];

                if(utilObj.isArray(reward["passedTriggers"]) && utilObj.isArray(passedTriggers)) {
                    //Find the intersection, and check if there is any
                    var intersection = reward["passedTriggers"].filter(function(n) {
                        return passedTriggers.indexOf(n) !== -1;
                    });

                    if(intersection.length > 0) {
                        // First lets try to see if the grant value is defined by the runtime - currently the support is only for Coins
                        if(reward.hasOwnProperty(CONFIG_RUNTIMECHECK) && reward[CONFIG_RUNTIMECHECK] != null) {
                            var runtimeValues = bubblePopNewObj.getRuntimes();
                            var grants = runtimeValues[reward[CONFIG_RUNTIMECHECK]];

                            if(utilObj.isArray(grants) && reward.hasOwnProperty(CONFIG_PASS_TYPE) && reward[CONFIG_PASS_TYPE] === true) {
                                var actionType = action[CONFIG_ACTIONS_TYPE];
                                var i = bubblePopNewObj.getIndexByBubbleType(actionType);
                                if(i != constantsObj.NOT_FOUND) {
                                    grants = grants[i];
                                }
                            }

                            // Applying multipliers
                            var totalMultipliers = 1;
                            if(reward.hasOwnProperty(CONFIG_MULTIPLIERS) && utilObj.isArray(reward[CONFIG_MULTIPLIERS])) {
                                for(var multiplierIndex = 0; multiplierIndex < reward[CONFIG_MULTIPLIERS].length; multiplierIndex++) {
                                    var multiplierInfo = reward[CONFIG_MULTIPLIERS][multiplierIndex];
                                    var isOrCondition = multiplierInfo.hasOwnProperty(CONFIG_OR_CONDITION) && multiplierInfo[CONFIG_OR_CONDITION] == true;

                                    // Go over the rules of mutliplierInfo
                                    if(multiplierInfo.hasOwnProperty(CONFIG_RULES) && utilObj.isArray(multiplierInfo[CONFIG_RULES])) {
                                        var passedMultiplier = true;
                                        for (var rulesIndex = 0; rulesIndex < multiplierInfo[CONFIG_RULES].length; rulesIndex++) {
                                            var rule = multiplierInfo[CONFIG_RULES][rulesIndex];

                                            if(bubblePopNewObj.processCondition(rule, action[CONFIG_ACTIONS_TYPE])) {
                                                passedMultiplier = passedMultiplier && true;
                                                if(isOrCondition) {
                                                    passedMultiplier = true;
                                                    break;
                                                }
                                            } else {
                                                passedMultiplier = false;
                                                if(!isOrCondition) {
                                                    break;
                                                }
                                                // bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, rule[CONFIG_NAME]);
                                            }
                                        }

                                        if(passedMultiplier) {
                                            if(multiplierInfo.hasOwnProperty(CONFIG_RUNTIMECHECK) && multiplierInfo[CONFIG_RUNTIMECHECK] != null) {
                                                var multiplier = runtimeValues[multiplierInfo[CONFIG_RUNTIMECHECK]];
                                                if(utilObj.isArray(multiplier) && multiplierInfo.hasOwnProperty(CONFIG_PASS_TYPE) && multiplierInfo[CONFIG_PASS_TYPE] === true) {
                                                    var actionType = action[CONFIG_ACTIONS_TYPE];
                                                    var i = bubblePopNewObj.getIndexByBubbleType(actionType);
                                                    if(i != constantsObj.NOT_FOUND) {
                                                        multiplier = multiplier[i];
                                                    }
                                                }
                                            }
                                            grants *= multiplier;
                                            totalMultipliers *= multiplier;
                                        }
                                    }
                                }
                            }

                            specialRewardGrant["coins"] = Math.ceil(grants/5) * 5; // Rounding off to the nearest
                            specialRewardGrant["totalMultipliers"] = totalMultipliers;
                            break;

                            // Else we will use reward and quantity value defined in the JSON config
                        } else if (reward.hasOwnProperty("reward") && reward["reward"] != null
                            && reward.hasOwnProperty("quantity") && reward["quantity"] != null){
                            specialRewardGrant[reward["reward"]] = reward["quantity"];
                            break;

                        }
                    }
                }
            }
        }
    };

    /**
     * Wrapper method to update all data that need to change on a fresh date (for the first session only)
     * (This is currently being invoked on the first sesion of the day from updateDailyInfo in xpromoUtil)
     */
    this.updateBubblesDailyData = function () {
        if (bubblePopV2 == null) {
            return;
        }
        var curTime = utilObj.getCurrentTimestamp();
        var daysPassed = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(curTime, bubblePopV2.lastUpdateTime));

        bubblePopV2.bubblesShownInDay = 0;
        bubblePopV2.daysSinceVideoWatched += daysPassed;
        bubblePopV2.bubblesShownInDayIndividual = [0, 0, 0, 0, 0, 0];
        bubblePopV2.lastShownInSession = [0, 0, 0, 0, 0, 0];

        // If the number of days passed are higher than the history, we restrict the addition to the history length to save computations
        if(daysPassed > HISTORY_SIZE) {
            daysPassed = HISTORY_SIZE;
        }

        for(var index = 0; index < daysPassed; index++) {
            bubblePopV2.dauDays.push(0);
            bubblePopV2.puzzlesHistory.push(0);
            bubblePopV2.videoHistory.push(0);
        }
        // Setting the current day as an active Dau Day
        bubblePopV2.dauDays[bubblePopV2.dauDays.length - 1] = 1;

        if(bubblePopV2.dauDays.length > HISTORY_SIZE) {
            var excess = bubblePopV2.dauDays.length - HISTORY_SIZE;
            bubblePopV2.dauDays.splice(0, excess);
            bubblePopV2.puzzlesHistory.splice(0, excess);
            bubblePopV2.videoHistory.splice(0, excess);
        }

        bubblePopV2.lastUpdateTime = curTime;
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Basic Bubble Pop methods ///////////////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for specific reward calc / grant methods /////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Method to calculate the basic coin reward that needs to be granted based on runtime values and probability
     * @returns {number} - Number of coins that needs to granted
     */
    this.calcCoinReward = function () {
        var rew = 0;

        var rand = utilObj.getRandomIntInRange(0, 100);
        var sum = 0;
        for (var i = 0; i < rt.coinRewardProb.length; i++) {
            sum += rt.coinRewardProb[i];
            if (sum > rand) {
                rew = rt.coinRewardValues[i];
                break;
            }
        }
        return rew;
    };

    /**
     * Method to grant basic coin reward and try initiating any custom flows based on the bubble type
     * @param params - JS Object holding the cash amount to grant and also the layer where the action is triggered
     */
    this.grantPopRewards = function (params) {
        if(params.cashToGrant > 0) {
            if((xPromoGameSpecificObj.getCash() + params.cashToGrant) <= constantsObj.MAX_CASH_ALLOWED) {
                utilObj.delayedCallback(params.layerForBubblePop, xPromoGameSpecificObj.grantCoinsAfterDelay, 2.0, {cashToGrant: params.cashToGrant, phylum: constantsObj.TRACK_EARN, class: xpromoConstantsObj.TRACK_BUBBLE_POP_V2, order: bubbleTypeName + "_tap"}, false);

                // Tracking for granting from bubble source
                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, "bubble_tap", bubbleTypeName,
                    xpromoConstantsObj.TRACK_BUBBLE_POP_V2, "tap_grant", "bubblepop-basic-coins", params.cashToGrant, "", "");

                xPromoGameSpecificObj.showCoinGrantMessage(params.cashToGrant);
            } else {
                utilObj.showMessage(stringsObj.coin_limit_reached);
            }
        }
        switch (bubbleTypeIndex) {
            case BUBBLE_IAP:
                popIapType = bubbleTypeName;
                utilObj.delayedCallback(params.layerForBubblePop, bubblePopNewObj.handleIAPBubbleTap, 0.2, null, false);
                break;
            case BUBBLE_VIDEO:
                popVideoType = bubbleTypeName; // This flow would later go and reset the bubbleTypeName, hence keeping it till video complete
                utilObj.delayedCallback(params.layerForBubblePop, bubblePopNewObj.playVideoFromPop, 0.2, null, false);
                break;
            case BUBBLE_XPROMO_WORD:
                utilObj.delayedCallback(params.layerForBubblePop, bubblePopNewObj.showXpromoWordQuest, 0.2, null, false);
                break;
            case BUBBLE_WORD:
                utilObj.delayedCallback(params.layerForBubblePop, bubblePopNewObj.showWordQuest, 0.2, null, false);
                break;
            case BUBBLE_XPROMO:
                popXpromoType = bubbleTypeName; // This flow would later go and reset the bubbleTypeName, hence keeping it for xpromo popup
                utilObj.delayedCallback(params.layerForBubblePop, bubblePopNewObj.showXpromoQuest, 0.2, null, false);
                break;
            case BUBBLE_COIN_GRANT:
            default:
                break;
        }
        // // In all cases we need to do the coin grant.
        // bubblePopUiObj.playRewardAnim(bubbleTypeIndex);
    };

    /**
     * Method to grant the special reward (Currently being called from customized flows for special bubbles)
     * @param sender - layer where the grant needs to happen
     * @param params - JS object containing the "reward" to be granted and also the "source"
     */
    this.grantSpecialReward = function(sender, params) {
        if (sender == undefined) {
            sender = gameLayer;
        }
        var reward = params.rewards, source = params.source, showMessage = params.showMessage;
        if(reward != undefined) {
            for(var type in reward) {
                var quantity = reward[type];
                if(quantity > 0) {
                    switch(type) {
                        case "coins":
                            if((xPromoGameSpecificObj.getCash() + quantity) <= constantsObj.MAX_CASH_ALLOWED) {
                                if(showMessage == undefined || showMessage == true) {
                                    xPromoGameSpecificObj.canW2eCoinGrant(0, 0, quantity, false, sender);
                                    utilObj.delayedCallback(sender, xPromoGameSpecificObj.grantCoinsAfterDelay, 2.0, {cashToGrant: quantity, phylum: constantsObj.TRACK_EARN, class: xpromoConstantsObj.TRACK_BUBBLE_POP_V2, order: (params.bubbleName != undefined) ? params.bubbleName : ""}, false);
                                    xPromoGameSpecificObj.showCoinGrantMessage(quantity);
                                } else {
                                    utilObj.delayedCallback(sender, xPromoGameSpecificObj.grantCoinsAfterDelay, 2.0, {cashToGrant: quantity, phylum: constantsObj.TRACK_EARN, class: xpromoConstantsObj.TRACK_BUBBLE_POP_V2, order: (params.bubbleName != undefined) ? params.bubbleName : ""}, false);
                                }

                                // Tracking for granting from bubble source
                                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, (params.phylum != undefined) ? params.phylum : "", (params.bubbleName != undefined) ? params.bubbleName : "",
                                    xpromoConstantsObj.TRACK_BUBBLE_POP_V2, "final_grant", (source != undefined) ? source : "", quantity, "", "");
                            } else {
                                utilObj.showMessage(stringsObj.coin_limit_reached);
                            }
                            break;
                        case "wisdom-leaves":
                            wordOfDayXPromoCongratsLayer.show({
                                icon : params["icon"],
                                gName : params["gameName"],
                                wisdomScore : quantity,
                            });
                            break;
                        // Add more reward types here when a new reward is defined in the JSON config.
                        default:
                            break;
                    }
                }
            }
            specialRewardGrant = {};
        }
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for specific reward calc / grant methods ///////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for UI utility methods ///////////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Helper method to auto pop a bubble from other game flows (Eg: Puzzle Completion)
     */
    this.tryAutoPopBubble = function () {
        if(bubblePopNewObj.isBubblePopActive()) {
            var layerForBubblePop = utilObj.getLayerForScreen(curScreen);
            pspnGameAnimLayer.autoPopBubble(layerForBubblePop);
        }
    };

    /**
     * Method to check and initiate the bubble show and call the frontend to do the same.
     */
    this.showBubble = function () {
        if (!xPromoGameSpecificObj.canShowBubblePopAnimationOnLayer() || popupLayer.popupOpen || ftueObj.isFtueVisible() || (xPromoGameSpecificObj.isSettingsLayerEnabled() && curLayer == settingsLayer)) {
            triggerLock = false;
            bubbleTypeIndex = constantsObj.NOT_FOUND;
            bubbleTypeName = "";
            return;
        }
        isBubbleOnScreen = true;
        bubbleShownAt = utilObj.getCurrentTimestamp();
        bubblePopV2.lastShownInSession[bubbleTypeIndex] = xPromoGameSpecificObj.getTimeSpentInSession();
        bubblePopV2.lastShown[bubbleTypeIndex] = bubbleShownAt;
        bubblePopV2.bubblesShownInDayIndividual[bubbleTypeIndex]++;
        bubblePopV2.bubblesShownInDay++;
        for(var i = 0; i < NUM_BUBBLE_TYPES; i++) {
            if(bubbleTypeIndex != i) {
                bubblePopV2.bubbleSkippedTimes[i]++;
            } else {
                bubblePopV2.bubbleSkippedTimes[i] = 0;
            }
        }
        var runtimes = bubblePopNewObj.getRuntimes();
        coinsToGrant = bubblePopNewObj.calcCoinReward();
        var iapDiscount = 0;
        // Check if engagement based multipler is active for coin type of bubble
        if(bubbleTypeIndex == BUBBLE_COIN_GRANT) {
            var runtime = bubblePopNewObj.getRuntimes();
            if(runtime.hasOwnProperty("engagementDays") && runtime.engagementDays.length > bubbleTypeIndex &&
                runtime.hasOwnProperty("engagementMultiplier") && runtime.engagementMultiplier.length > bubbleTypeIndex) {
                if (bubblePopNewObj.getDauDaysFromHistory(bubbleTypeName) < runtime.engagementDays[bubbleTypeIndex]) {
                    coinsToGrant *= runtime.engagementMultiplier[bubbleTypeIndex];
                }
            }
        } else if (bubbleTypeIndex == BUBBLE_IAP) {
            var result = bubblePopNewObj.getIAPPackageInfoToShow();
            if(result == null) {
                return;
            }
            iapDiscount = result["percentage"];
        } else if (bubbleTypeIndex == BUBBLE_XPROMO_WORD || bubbleTypeIndex == BUBBLE_WORD){
            pspnGameObj.setWordOfTheDay();
            var index = bubbleTypeIndex - BUBBLE_WORD;
            if(runtimes.willCoinsBeGranted[index] == 0){
                coinsToGrant = 0;
            }
        }


        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, constantsObj.TRACK_VIEW);
        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_GENUS, xPromoGameSpecificObj.getDailyPuzzleClears());
        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_SPECIES, coinsToGrant + "");
        bubblePopNewObj.applyTracking();

        var finalRewardMultiplier = 1;
        var finalRewardGrant = bubblePopNewObj.getSpecialRewardToGrant();
        if(finalRewardGrant != undefined && finalRewardGrant.hasOwnProperty("totalMultipliers")) {
            finalRewardMultiplier = finalRewardGrant.totalMultipliers;
        } else {
            if(bubbleTypeIndex == BUBBLE_XPROMO_WORD ){
                var rewardXpromo = parseInt(bubblePopNewObj.getWisdomLeafRewardForXpromo());
                var rewardNormal = parseInt(runtimes.runtimeLeafReward[1]);
                finalRewardMultiplier = parseInt(rewardXpromo/rewardNormal);
            }

        }
        pspnGameAnimLayer.showBubble(bubbleTypeIndex, coinsToGrant, finalRewardMultiplier, iapDiscount);
    };

    /**
     * Method to handle pop of the bubble from the UI
     * @param params - JS Object holding the cash amount to grant and also the layer where the action is triggered
     */
    this.popBubble = function (params) {
        bubblePopNewObj.grantPopRewards(params);

        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, constantsObj.TRACK_CLICK);
        bubblePopNewObj.applyTracking();

        triggerLock = false;
        isBubbleOnScreen = false;
        bubbleTypeIndex = constantsObj.NOT_FOUND;
        bubbleTypeName = "";
    };

    /**
     * Method to handle auto-pop of the bubble from the UI
     */
    this.autoPopBubble = function() {
        triggerLock = false;
        isBubbleOnScreen = false;
        bubbleTypeIndex = constantsObj.NOT_FOUND;
        bubbleTypeName = "";
    };

    /**
     * Method to get the amount of time the bubble needs to float around on the screen
     * @returns {number} - Number in seconds
     */
    this.getBubbleFloatTime = function () {
        var runtime = bubblePopNewObj.getRuntimes();
        return runtime.bubbleFloatTime;
    };

    /**
     * Method to get the time at the tail of the float time when the bubble starts to flicker
     * @returns {number} - Number in seconds
     */
    this.getBubbleUnstableTime = function () {
        var runtime = bubblePopNewObj.getRuntimes();
        return runtime.bubbleUnstableTime;
    };

    /**
     * Method to return if sound for the bubble is enabled
     * @returns {boolean}
     */
    this.getBubbleSoundToPlay = function () {
        var runtime = bubblePopNewObj.getRuntimes();
        return (runtime.hasOwnProperty("sound") && runtime.sound == 1);
    };
    /**
     * Method to check if the bubble is currently visible on the screen or not
     * @returns {boolean} - True / False
     */
    this.isBubbleVisibleOnScreen = function () {
        return isBubbleOnScreen;
    };

    /**
     * Method to return the special reward grant as an Object
     * @returns {object} - JS Object with the reward name as key and value as quantity Eg: {"coins" : 100}
     */
    this.getSpecialRewardToGrant = function () {
        return specialRewardGrant;
    };

    this.getW2ePopupDisabledForBubblePop = function () {
        return bubblePopV2.w2ePopupDisabledForBubblePop;
    };

    this.setW2ePopupDisabledForBubblePop = function (val, onPlayClick) {
        bubblePopV2.w2ePopupDisabledForBubblePop = val;

        if(onPlayClick) {
            isPopVideoPlaying = true;
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, "popup", popVideoType,
                xpromoConstantsObj.TRACK_BUBBLE_POP_V2, val ? "click_check" : "click_not_check", "", "", "", "");
        }

    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for UI utility methods /////////////////////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Video bubble specific conditions /////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Method to initiate the the showing of Ad
     */
    this.playVideoFromPop = function () {
        if ((curLayer == gameLayer || lastLayer == gameLayer) && softLaunchObj.getIsPuzzleCompleted() == true) {
            return;
        }

        if (!xPromoGameSpecificObj.checkRewardedVideo(false)) {
            utilObj.showMessage(stringsObj.video_not_available);
            return;
        }

        if (bubblePopNewObj.getW2ePopupDisabledForBubblePop()) {
            utilObj.playSound(constantsObj.button_tap_01, false);
            isPopVideoPlaying = true;
            xPromoGameSpecificObj.showVideoAd();
        } else {
            // This does not work for DTC, as there is no w2eInfoPopup, the flow should never reach here
            var params = utilObj.createClone(bubblePopNewObj.getSpecialRewardToGrant());
            params["isBubblePopV2"] = true;
            w2eInfoPopup.show(true, params);

            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, "popup", popVideoType,
                xpromoConstantsObj.TRACK_BUBBLE_POP_V2, constantsObj.TRACK_VIEW, "", "", "", "");
        }
    };

    /**
     * Method to reset days since the last video was watched
     */
    this.resetDaysSinceVideoWatched = function () {
        if (bubblePopV2 == null) {
            return;
        }
        bubblePopV2.daysSinceVideoWatched = 0;
    };

    /**
     * Wrapper method to check if the video was being played from bubble pop and the reward that needs to be granted.
     * @returns {boolean} - Whether the reward was successfully granted or not.
     */
    this.checkAndGrantVideoReward = function () {
        if (isPopVideoPlaying && bubblePopNewObj.isBubblePopActive()) {
            isPopVideoPlaying = false;

            var showMessage = true;
            var xpromoAfterW2eGrantExp = epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_XPROMO_W2E);

            if(xPromoGameSpecificObj.isScreenForW2eXpromo(updatingToScreen) || xPromoGameSpecificObj.checkScreenIsW2E(updatingToScreen)) {
                var layerForAnimation = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(updatingToScreen);
                if (xpromoAfterW2eGrantExp != constantsObj.EXP_VAR_CONTROL) {

                    xpromoW2eObj.setRewardValueOverride(specialRewardGrant["coins"]);
                    if(xpromoW2eObj.showW2EPopup()) {
                        showMessage = false;
                    } else {
                        xpromoW2eObj.setRewardValueOverride(constantsObj.NOT_FOUND);
                    }
                }
            }
            // Pass the current screen layer to help with the reward animation
            var layerForAnimationOnCurScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen);
            var paramsMap = {"rewards": specialRewardGrant, "source": "bubblepop-video", "bubbleName": popVideoType, "showMessage": showMessage, "phylum": "popup"};
            bubblePopNewObj.grantSpecialReward(layerForAnimationOnCurScreen, paramsMap);

            return true;
        }
        return false;
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Video bubble specific conditions ///////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Xpromo bubble specific conditions ////////
    ////////////////////////////////////////////////////////////////////////


    /**
     * Wrapper method to surface the xpromo quest and UI given the required set of conditions pass
     * @returns {boolean} - Whether the xpromo popup was shown or not.
     */
    this.showXpromoQuest = function () {
        if ((curLayer == gameLayer || lastLayer == gameLayer) && softLaunchObj.getIsPuzzleCompleted() == true) {
            return false;
        }
        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopNewObj.getRuntimes();
        var questPrioritiesArr = Object.keys(xpromoQuests);
        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];

            if(!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) || !bubblePopNewObj.isBubblePopCreativeAvailable(questId, false) || bubblePopNewObj.isViewOrClickCooldownReached(gameId)) {
                continue;
            }

            if (bubblePopV2.xpromoCooldowns[gameId] == undefined) {
                bubblePopV2.xpromoCooldowns[gameId] = {
                    "viewCount": 0,
                    "clickCount": 0,
                };
            }

            bubblePopV2.xpromoCooldowns[gameId].viewCount++;

            bubblePopUiObj.show(true, {gameId: gameId, bubbleName: popXpromoType, isBubblePopV2: true});
            return true;
        }
        return false;
    };

    /**
     * Method to check if all xpromo-able games have reach their cooldown and reset them if required.
     */
    this.checkResetXpromoCooldowns = function () {
        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopNewObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;
        var questPrioritiesArr = Object.keys(xpromoQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });


        var reset = true;

        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit && reset); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];

            if(xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) && bubblePopNewObj.isBubblePopCreativeAvailable(questId, false)) {
                if(bubblePopV2.xpromoCooldowns[gameId] != undefined) {
                    if(bubblePopV2.xpromoCooldowns[gameId].viewCount < xpromoViewCoolDown && bubblePopV2.xpromoCooldowns[gameId].clickCount < xpromoClickCoolDown) {
                        reset = false;
                    }
                } else {
                    reset = false;
                }
            }
        }

        if(reset) {
            bubblePopV2.xpromoCooldowns = {};
        }
    };

    /**
     * Method to check if the creative / bubble pop image for a xpromo-able game is available or not.
     * @param questId - quest ID for the adwall quest corresponding to the xpromo game
     * @param downloadOnFail - Boolean to initiate download if the requested file does not exist
     * @returns {boolean} - Whether the creative is downloaded and available.
     */
    this.isBubblePopCreativeAvailable = function (questId, downloadOnFail = false) {
        var result = false;
        var curQuests = questsObj.getCurQuests();

        if(curQuests[questId] != undefined) {
            var questData = curQuests[questId];
            if(questData.creatives) {
                var questCreativeMap = xpromoUtilObj.getKeyValuePairForCreatives(questData.creatives);
                if(questCreativeMap.bubblePop_creative != undefined) {

                    var creativeName = questCreativeMap.bubblePop_creative;
                    result = utilObj.fileExists(creativeName);

                    if(!result && downloadOnFail == true) {
                        var contentLink = BUBBLE_POP_CONTENT_DOWNLOAD_LINK + questCreativeMap.bubblePop_creative;
                        xPromoGameSpecificObj.downloadFileV1(contentLink, creativeName);
                    }
                }
            }
        }
        return result;
    };

    /**
     * Method to check whether a particular game has reached its surfacing cooldown in the xpromo
     * @param gameId - game ID for the game
     * @returns {boolean} - Whether the cooldown is reached or not
     */
    this.isViewOrClickCooldownReached = function(gameId) {
        var result = false;

        var runtimeValues = bubblePopNewObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;

        if (bubblePopV2.xpromoCooldowns[gameId] != undefined && (bubblePopV2.xpromoCooldowns[gameId].viewCount >= xpromoViewCoolDown || bubblePopV2.xpromoCooldowns[gameId].clickCount >= xpromoClickCoolDown)) {
            result = true;
        }

        return result;
    };

    /**
     * Method to start the xpromo quest and save the details. Used again when the xpromo-ed game is installed and the player needs to be rewarded
     * @param gameId - game ID for the game
     * @param questId - adwall quest id from the game
     * @param bubbleName - Name of the bubble to be marked
     */
    this.markXpromoInteraction = function (gameId, questId, bubbleName) {
        if(!bubblePopNewObj.isBubblePopActive()) {
            return;
        }

        if(bubblePopV2 == null || Object.keys(specialRewardGrant).length === 0 || bubblePopV2.xpromoCooldowns[gameId] == undefined) {
            return;
        }

        questsObj.makeRelatedQuestsStatusStarted(questId);
        xpromoUtilObj.markXpromoStartedFromOtherSource(gameId, xpromoConstantsObj.TRACK_BUBBLE_POP_V2);

        bubblePopV2.xpromoClicks[gameId] = {
            "ts": utilObj.getCurrentTimestamp(),
            "rewards": utilObj.createClone(specialRewardGrant),
            "questId": questId,
            "bubbleName": bubbleName
        };

        bubblePopV2.xpromoCooldowns[gameId].clickCount++;

        // Incase W2E was earlier seen for this game, lets update the grant value to match whats currently being shown.
        if(xpromoW2eObj.getXpromoGameId() == gameId && (bubblePopV2.xpromoClicks[gameId]["rewards"]).hasOwnProperty("coins")) {
            xpromoW2eObj.setMultipliedLastCoinsAdded(bubblePopV2.xpromoClicks[gameId]["rewards"]["coins"]);
        }


    };

    /**
     * Method to check the completion of a xpromo initiated from the Bubble Pop flow and reward the player about the same.
     */
    this.checkAndGrantXpromoReward = function () {
        if(!bubblePopNewObj.isBubblePopActive()) {
            return;
        }

        if(bubblePopV2 == null || bubblePopV2.xpromoClicks == undefined) {
            bubblePopNewObj.checkAndGrantXpromoWordReward();
            return;
        }
        var attributionPeriod = xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_SURFACING_ATTRIBUTION_WINDOW) * 60 * 60;
        var curTime = utilObj.getCurrentTimestamp();
        utilObj.resetIsAppInstalledMap();
        for(var gameId in bubblePopV2.xpromoClicks) {
            var delEntry = false;
            var clickData = bubblePopV2.xpromoClicks[gameId];
            var clickTime = clickData["ts"];
            var questId = clickData["questId"];
            var bubbleName = clickData["bubbleName"];

            var incentivizedElsewhere = xpromoUtilObj.isGameAlreadyMarkedForRewarding(gameId, xpromoConstantsObj.TRACK_BUBBLE_POP_V2);

            if(!incentivizedElsewhere && clickTime > 0 && ((clickTime + attributionPeriod) >= curTime)
                && xpromoUtilObj.isAppInstalledByGameId(gameId)) {

                // This is to account for the scenario when W2E for the same game is seen after BP,
                // In this scenario, only the W2E reward should be granted and not BP. Hence the check for gameId below
                if(xpromoW2eObj.getXpromoGameId() != gameId && questsObj.getCrossGameQuestStatusForId(questId) != xpromoConstantsObj.QUEST_CLAIMED && adWallObj.getXpromoInteractionTime(gameId) < clickTime) {
                    // Lets get the current and last screen layers
                    var layerForAnimationOnLastScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(lastScreen);
                    var layerForAnimationOnCurScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen);
                    var paramsMap = {"rewards": clickData["rewards"], "source": "bubblepop-xpromo", "bubbleName": bubbleName, "phylum": questId};

                    // If the popup is open and we are on the xpromo screen, lets close and do a delayed reward grant.
                    if (popupLayer.popupOpen && (xPromoGameSpecificObj.isScreenForW2eXpromo(lastScreen))) {
                        popupLayer.closeWithDefault();
                        utilObj.delayedCallback(layerForAnimationOnLastScreen, bubblePopNewObj.grantSpecialReward, 2, paramsMap, false);
                    } else { // Else grant the reward immediately
                        bubblePopNewObj.grantSpecialReward(layerForAnimationOnCurScreen, paramsMap);
                    }
                    questsObj.makeRelatedQuestsForW2eClaimed(questId);
                }
                delEntry = true;
            }

            if (((clickTime + attributionPeriod) < curTime) || incentivizedElsewhere) {
                delEntry = true;
            }
            if(delEntry) { // Lets delete the rewards for the gameId as they are either granted to expired
                delete bubblePopV2.xpromoClicks[gameId];
            }
        }
        bubblePopNewObj.checkAndGrantXpromoWordReward();
    };

    /**
     * Getter method to retrieve the xpromo click time for a given game id
     * @param gameId
     * @returns number value of timestamp last interacted, zero if not applicable.
     */
    this.getXpromoInteractionTime = function (gameId) {
        var result = 1;
        if(!bubblePopNewObj.isBubblePopActive()) {
            return result;
        }

        if(bubblePopV2 == null || bubblePopV2.xpromoClicks == undefined ||
            bubblePopV2.xpromoClicks[gameId] == undefined || bubblePopV2.xpromoClicks[gameId]["ts"] == undefined) {
            return result;
        }

        return bubblePopV2.xpromoClicks[gameId]["ts"];
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Xpromo bubble specific conditions //////////
    ////////////////////////////////////////////////////////////////////////


    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for IAP bubble specific conditions ///////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Wrapper method to get valid pack contents and other info from the game for IAP bubble
     * @returns {null} / allInfo Object
     */
    this.getValidIAPPacksInfo = function() {
        var allInfo = xPromoGameSpecificObj.getPackagesInfoForBubblePopIAP();

        if(allInfo == null || allInfo["packages"] == undefined || allInfo["discounts"] == undefined || allInfo["popularIndex"] == undefined) {
            return null;
        }

        if(!utilObj.isArray(allInfo["packages"]) || !utilObj.isArray(allInfo["discounts"]) || allInfo["packages"].length == 0 || allInfo["packages"].length != allInfo["discounts"].length) {
            return null;
        }

        return allInfo;
    };

    /**
     * Method to retrieve the package information on the package to be show by the IAP Type bubble
     * @param fromTapAction - boolean if this was triggered on the tap of the bubble
     * @returns {*} - Object holding packageInfo, percentage discount and if this is a custom package, or a null value
     */
    this.getIAPPackageInfoToShow = function(fromTapAction) {

        // Now lets calculate what needs to be shown and how much
        var allInfo = bubblePopNewObj.getValidIAPPacksInfo();
        if(allInfo == null) {
            return null;
        }

        var packagesInfo = allInfo["packages"];
        if(!utilObj.isArray(packagesInfo) || packagesInfo.length < (allInfo["popularIndex"] + 1)) {
            return null;
        }

        var percentMore = allInfo["discounts"];
        var popularPackage = packagesInfo[allInfo["popularIndex"]];

        var packageNameList = [];
        for(var index = 0; index < packagesInfo.length; index++) {
            packageNameList.push(xPromoGameSpecificObj.getIAPPackageName(packagesInfo[index]["name"]));
        }

        // Ensuring we do not get out of bound errors
        while (packagesInfo.length > percentMore.length) {
            percentMore.push(0);
        }

        var finalPackage = popularPackage;
        var customDeal = false;
        var percentage = percentMore[allInfo["popularIndex"]];

        var runtime = bubblePopNewObj.getRuntimes();
        var payerString = "np";
        var saleString = "flat";

        if(xPromoGameSpecificObj.getIsPayerUser()) {

            var purchaseHistory = paymentsObj.getPurchaseHistoryInfo();
            // 1. Check if the payer has purchased through the bubble first up and has not hit cooldown
            // 2. Check if there is sufficient purchase, if so show the highest purchase from that.
            if(bubblePopV2 != null && bubblePopV2.iapPackageToShow != undefined && bubblePopV2.iapPackageToShow != "" && bubblePopV2.iapPackageShownTimes < runtime.iapUpsells) {
                var selectedIndex = packageNameList.indexOf(bubblePopV2.iapPackageToShow);
                if(selectedIndex > -1) {
                    finalPackage = packagesInfo[selectedIndex];
                    percentage = percentMore[selectedIndex];
                    if(fromTapAction) {
                        bubblePopV2.iapPackageShownTimes++;
                    }
                    saleString = "upsell";
                }
            } else if (utilObj.isArray(purchaseHistory) && purchaseHistory.length > 0) {
                // Pick the highest value purchase in the last two purchases (if there is only one purchase, pick from that)
                var selectedIndex = constantsObj.NOT_FOUND;
                for(var index = purchaseHistory.length - 1; index >= purchaseHistory.length - 2 && index >= 0; index--) {
                    var pkgName = xPromoGameSpecificObj.getIAPPackageName(purchaseHistory[index]["pkgName"]);
                    var pkgIndex = packageNameList.indexOf(pkgName);
                    if((selectedIndex == constantsObj.NOT_FOUND || pkgIndex > selectedIndex) && pkgIndex != constantsObj.NOT_FOUND) {
                        selectedIndex = pkgIndex;
                        finalPackage = packagesInfo[selectedIndex];
                        percentage = percentMore[selectedIndex];
                    }
                }
            }

            payerString = "p";
        } else {
            var daysSinceInstall = Math.round(xpromoUtilObj.getUserDaysSinceInstall());
            if ((daysSinceInstall > 20 && daysSinceInstall <= 30) || !runtime.hasOwnProperty("iapMaxCoins") || !(parseInt(runtime.iapMaxCoins) > 0)) {
                finalPackage = packagesInfo[0];
                percentage = percentMore[0];
            } else if (daysSinceInstall > 30) {
                var bubblePopPackages = xPromoGameSpecificObj.getBubblePopIAPPackage();
                if(!utilObj.isArray(bubblePopPackages) || bubblePopPackages.length == 0) {
                    return null;
                }

                var bubblePopPackage = bubblePopPackages[0];
                bubblePopPackage["qty"] = Math.min(parseInt(xPromoGameSpecificObj.getBaseCoinsForBubblePopIAP()) + Math.ceil((daysSinceInstall - 30) / 10) * 50, parseInt(runtime.iapMaxCoins));
                finalPackage = bubblePopPackage;
                percentage = Math.ceil((1 - (parseInt(packagesInfo[0]["qty"]) / bubblePopPackage["qty"])) * 100);
                customDeal = true;
                saleString = "upsell";
            }

        }

        if(!fromTapAction) {
            bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, constantsObj.TRACK_VIEW);
        } else {
            bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, constantsObj.TRACK_CLICK);
        }
        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_GENUS, saleString + "_" + payerString);
        bubblePopNewObj.setTrackingDetails(xpromoConstantsObj.TRACK_SPECIES, finalPackage["priceNumeric"] + "_" + finalPackage["qty"]);
        bubblePopNewObj.applyTracking();

        var result = {
            "pkgInfo": finalPackage,
            "isCustomDeal": customDeal,
            "percentage": percentage,
            "saleType": saleString + "_" + payerString,
            "saleInfo": finalPackage["priceNumeric"] + "_" + finalPackage["qty"],
        };

        return result;
    };

    /**
     * Method to set purchase through bubble pop and hence set information for the up sell flows
     * @param pkgName - name of the package purchasesd
     */
    this.setIAPPurchaseThroughBubble = function(pkgName) {
        if (bubblePopV2 == null || !bubblePopNewObj.isBubblePopActive()) {
            return;
        }

        pkgName = xPromoGameSpecificObj.getIAPPackageName(pkgName);

        // Do not set this for custom made bubble pop packages
        var bubbleIapPackage = xPromoGameSpecificObj.getBubblePopIAPPackageNames();
        if(bubbleIapPackage == null || bubbleIapPackage == pkgName) {
            return;
        }

        var allInfo = bubblePopNewObj.getValidIAPPacksInfo();
        if(allInfo == null) {
            return;
        }

        var packagesInfo = allInfo["packages"];
        for(var index = 0; index < packagesInfo.length; index++) {
            var name = xPromoGameSpecificObj.getIAPPackageName(packagesInfo[index]["name"]);
            if(pkgName == name) {
                if(packagesInfo.length > (index+1)) {
                    bubblePopV2.iapPackageToShow = xPromoGameSpecificObj.getIAPPackageName(packagesInfo[index+1]["name"]);
                } else {
                    bubblePopV2.iapPackageToShow = name;
                }
                bubblePopV2.iapPackageShownTimes = 0;
                break;
            }
        }
    };

    /**
     * Handle tap on the IAP Bubble with custom error handling
     */
    this.handleIAPBubbleTap = function() {
        var params = bubblePopNewObj.getIAPPackageInfoToShow(true);
        if(params != null) {
            bubblePopIapPopup.show(params);
        } else {
            utilObj.showMessage(stringsObj.error_generic);
        }
    };

    /**
     * Setting the coin value that would be granted on purchase
     */
    this.setIAPQuantityToGrant = function(qty) {
        if(bubblePopV2 != null) {
            bubblePopV2.iapQuantity = qty;
        }
    };

    /**
     * Getting the coin value that would be granted on purchase
     */
    this.getIAPQuantityToGrant = function() {
        if(bubblePopV2 != null) {
            return bubblePopV2.iapQuantity;
        }
        return
    };



    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for IAP bubble specific conditions /////////////
    ////////////////////////////////////////////////////////////////////////

    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for JSON called methods //////////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Helper method to get the number of bubbles shown in the day
     * @returns {number} - bubbles shown today
     */
    this.getBubblesShownInTheDay = function() {
        if(bubblePopV2 == null || !bubblePopV2.hasOwnProperty("bubblesShownInDay")) {
            return constantsObj.NOT_FOUND;
        }
        return bubblePopV2.bubblesShownInDay;
    };

    this.getBubblesShownInTheDayEvery = function(actionType) {
        var index = bubblePopNewObj.getIndexByBubbleType(actionType);
        if(bubblePopV2 == null || !bubblePopV2.hasOwnProperty("bubblesShownInDayIndividual") || bubblePopV2.bubblesShownInDayIndividual.length <= index) {
            return constantsObj.NOT_FOUND;
        }

        return bubblePopV2.bubblesShownInDayIndividual[index];
    };

    /**
     * Number of days install if no PS game is installed on the device
     * @returns {number}
     */
    this.daysSinceLoneInstall = function() {
        if(!xpromoUtilObj.isAnyPsGameInstalled()) {
            return xpromoUtilObj.getUserDaysSinceInstall();
        }
        return constantsObj.NOT_FOUND;
    };

    /**
     * Helper method to get the number of times for which a particular bubble is not shown
     * @param actionType - Bubble Type
     * @returns {number} - Number of times a bubble has been skipped consecutively
     */
    this.getOtherBubblesSeenConsecutively = function(actionType) {
        var index = bubblePopNewObj.getIndexByBubbleType(actionType);

        if(index != constantsObj.NOT_FOUND) {
            return bubblePopV2.bubbleSkippedTimes[index];
        }
        return constantsObj.NOT_FOUND;
    };

    /**
     * Helper method to get the days since a particular bubble was last shown
     * @param actionType - Bubble Type
     * @returns {number} - Number corresponding to days
     */
    this.getDaysSinceBubbleShown = function(actionType) {
        var index = bubblePopNewObj.getIndexByBubbleType(actionType);

        if(index != constantsObj.NOT_FOUND && bubblePopV2 != null && bubblePopV2.hasOwnProperty("lastShown")) {
            return xpromoUtilObj.getDaysDiffBetweenTimestamps(utilObj.getCurrentTimestamp(), bubblePopV2.lastShown[index]);
        }
        return constantsObj.NOT_FOUND;
    };

    /**
     * Helper method to get the minutes since the last bubble was shown
     * @returns {number} - Number corresponding to minutes
     */
    this.getMinutesSinceAnyBubbleShown = function ()  {
        var result = MAX_DAYS_LIMIT * 24 * 60;
        for (var type in bubbleTypeIndexMap) {
            var index = bubbleTypeIndexMap[type];
            if(index != constantsObj.NOT_FOUND) {
                var mins = xpromoUtilObj.getMinutesDiffFromSeconds(utilObj.getCurrentTimestamp(), bubblePopV2.lastShown[index]);
                result = Math.min(mins, result);
            }
        }
        return result;
    };

    /**
     * Helper method to get the number of days since the player has watched a video
     * @returns {number} - Number corresponding to days
     */
    this.getDaysSinceVideoWatched = function() {
        if(bubblePopV2 == null || !bubblePopV2.hasOwnProperty("daysSinceVideoWatched")) {
            return constantsObj.NOT_FOUND;
        }
        return bubblePopV2.daysSinceVideoWatched;
    };

    /**
     * Helper method to check if any game is eligible for the xpromo
     * IMPORTANT NOTE: This method also initiates the download of creatives,
     * hence might return false for the first ever time since the creatives wont be downloaded
     *
     * @returns {boolean} - True if a game is found, false other wise
     */
    this.anyEligibleGames = function () {
        var result = false;
        var adwallQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopNewObj.getRuntimes();
        var questPrioritiesArr = Object.keys(adwallQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        if (questPrioritiesArr.length > 0) {
            for (var i = 0; (i < questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
                var game = questPrioritiesArr[i];
                var adwallQuest = adwallQuests[game];
                var gameId = adwallQuest.gh,
                    questId = adwallQuest.id;

                if (!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId)) {
                    continue;
                }

                if(bubblePopNewObj.isViewOrClickCooldownReached(gameId)) {
                    continue;
                }

                if(bubblePopNewObj.isBubblePopCreativeAvailable(questId, true)) {
                    result = result || true;
                }
            }
        }
        return result;
    };

    /**
     * Method to compute the required number of puzzles to be played before making a bubble type trigger eligible
     * @param actionType - Bubble Type
     * @returns {boolean} - Returns true if the condition meets, else false
     */
    this.meetsPuzzlePercentage = function(actionType) {
        var result = false;
        var index = bubblePopNewObj.getIndexByBubbleType(actionType);
        var runtimeValues = bubblePopNewObj.getRuntimes();

        // If percentage is zero, that means this condition is shut off
        if(index == constantsObj.NOT_FOUND || !runtimeValues.hasOwnProperty("puzzlePercentage") || runtimeValues.puzzlePercentage[index] == 0) {
            return result;
        }

        // If we haven't had 10 days of data since initialization, fail this condition
        if(bubblePopV2 == null || !bubblePopV2.hasOwnProperty("initTime") || (bubblePopV2.initTime + ((HISTORY_SIZE-1) * constantsObj.SECONDS_IN_DAY)) > utilObj.getCurrentTimestamp() || bubblePopV2.puzzlesHistory.length < HISTORY_SIZE) {
            return result;
        }

        var puzzlesHistory = bubblePopV2.puzzlesHistory.slice();

        // Finding median number of puzzles played
        puzzlesHistory.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        var midIndex = parseInt(HISTORY_SIZE / 2);
        var median = parseInt(puzzlesHistory[midIndex-1]);
        if(HISTORY_SIZE % 2 != 0) { // If this is an odd sized array
            median = Math.ceil((median + puzzlesHistory[midIndex]) / 2);
        }

        var puzzleEligible = Math.floor(median * runtimeValues.puzzlePercentage[index]);

        result = (xPromoGameSpecificObj.getDailyPuzzleClears() >= puzzleEligible);
        return result;
    };


    /**
     * Method to compute the required amount of time spent in session before a specific bubble type becomes eligible
     * @param actionType - Bubble Type
     * @returns {boolean} - Returns true if the condition meets, else false
     */
    this.meetsTimeBasedTrigger = function(actionType) {
        var index = bubblePopNewObj.getIndexByBubbleType(actionType);
        var runtimeValues = bubblePopNewObj.getRuntimes();

        // If percentage is zero, then only this condition needs to be evaluated
        if(index == constantsObj.NOT_FOUND || !runtimeValues.hasOwnProperty("puzzlePercentage") || runtimeValues.puzzlePercentage[index] != 0) {
            return false;
        }

        if(bubblePopV2 == null || !bubblePopV2.hasOwnProperty("lastShownInSession") || bubblePopV2.lastShownInSession.length <= index) {
            return false;
        }

        var timeSinceLastShown = xPromoGameSpecificObj.getTimeSpentInSession() - bubblePopV2.lastShownInSession[index];
        return runtimeValues.timeSpent[index] <= timeSinceLastShown;
    };

    /**
     * Method to compute the day gap between two bubbles of a specific type
     * @param actionType - Bubble Type
     * @returns {boolean} - Returns true if the condition meets, else false
     */
    this.meetsDaysSinceBubbleSeen = function(actionType) {
        var result = false;
        var index = bubblePopNewObj.getIndexByBubbleType(actionType);
        var runtimeValues = bubblePopNewObj.getRuntimes();
        if(index == constantsObj.NOT_FOUND || !runtimeValues.hasOwnProperty("cooldownUpper") || runtimeValues.cooldownUpper[index] == 0) {
            return result;
        }
        if(bubblePopV2 == null || !bubblePopV2.hasOwnProperty("lastShown") || bubblePopV2.lastShown.length <= index) {
            return result;
        }
        var daysPassed = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(utilObj.getCurrentTimestamp(), bubblePopV2.lastShown[index]));
        result = (daysPassed >= runtimeValues.cooldownUpper[index]);
        return result;
    };

    /**
     * Helper method to check if the payer flag validates with the payer status of the player
     * @param actionType - Bubble Type
     * @returns {boolean} - true / false
     */
    this.meetsPayerCheck = function(actionType) {
        var result = false;
        var index = bubblePopNewObj.getIndexByBubbleType(actionType);
        var runtimeValues = bubblePopNewObj.getRuntimes();

        if(index == constantsObj.NOT_FOUND || !runtimeValues.hasOwnProperty("payerCheck") || runtimeValues.payerCheck.length <= index) {
            return result;
        }

        result = runtimeValues.payerCheck[index] == 2 || (runtimeValues.payerCheck[index] == 1 && xPromoGameSpecificObj.getIsPayer()) || (runtimeValues.payerCheck[index] == 0 && !xPromoGameSpecificObj.getIsPayer());
        return result;
    };

    /**
     * Helper method to retrieve the number of DAU days in the last HISTORY_SIZE - 1 days prior to current day
     * @param actionType - Bubble Type
     * @returns {number} - Number of DAU Days
     */
    this.getDauDaysFromHistory = function(actionType) {

        // If we haven't had 10 days of data since initialization, fail this condition
        if(bubblePopV2 == null || bubblePopV2.initTime == null || (bubblePopV2.initTime + ((HISTORY_SIZE-1) * constantsObj.SECONDS_IN_DAY)) > utilObj.getCurrentTimestamp() || bubblePopV2.dauDays.length < HISTORY_SIZE) {
            return MAX_DAYS_LIMIT;
        }

        var dauDays = 0;
        for(var index = HISTORY_SIZE - 2; index >= 0; index--) {
            if(bubblePopV2.dauDays[index] == 1) {
                dauDays++;
            }
        }

        return dauDays;
    };

    /**
     * Helper method to get the number of DAU Days since any video watch
     * @returns {number} - Number of DAU Days since video watch
     */
    this.getDauDaysSinceVideoWatched = function() {

        // If we haven't had 10 days of data since initialization, fail this condition
        if(bubblePopV2 == null || bubblePopV2.initTime == null || (bubblePopV2.initTime + ((HISTORY_SIZE-1) * constantsObj.SECONDS_IN_DAY)) > utilObj.getCurrentTimestamp() || bubblePopV2.dauDays.length < HISTORY_SIZE) {
            return constantsObj.NOT_FOUND;
        }

        if(xPromoGameSpecificObj.getDailyWatchVideosCount() > 0) {
            return 0;
        }

        var dauDays = 0;
        for(var index = HISTORY_SIZE - 2; index >= 0; index--) {
            if(bubblePopV2.dauDays[index] == 1) {
                if(bubblePopV2.videoHistory[index] == 0) {
                    dauDays++;
                } else {
                    break;
                }
            }
        }

        return dauDays;
    };

    /**
     * Custom eligiblity condition for IAP Bubble related to Payer and Non-Payer
     * @returns {boolean} - true / false
     */
    this.isEligibleForIAPBubble = function() {
        return xPromoGameSpecificObj.getIsPayerUser() || (xpromoUtilObj.getUserDaysSinceInstall() > 10);
    };


    /**
     * Get number of days since the player made a purchase
     * @returns {number}
     */
    this.getDaysSinceLastPurchase = function() {
        return Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(xPromoGameSpecificObj.getLastPurchaseDate(), utilObj.getCurrentTimestamp()));
    };


    /**
     * Custom eligiblity condition for IAP Bubble related to Pack availability
     * @returns {boolean} - true / false
     */
    this.isIAPPackValid = function() {
        return (bubblePopNewObj.getValidIAPPacksInfo() != null);

    };

    /**
     * Eligiblity check if the current screen is game screen for word and xpromo-word bubbles
     * @returns {boolean} - true / false
     */
    this.isValidScreenForWordBubbles = function() {
        return curScreen == constantsObj.SCREEN_GAME;
    };

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for JSON called methods ////////////////////////
    ////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////
    /////////////Methods for Xpromo - word and Word bubble Eligibility////////
    /////////////////////////////////////////////////////////////////////////


    /**
     * Custom eligibility condition for word bubble and xpromo bubble which compares the runtime Value for user cohort
     * @returns {bool}
     * */
    this.isUserPastMinDauDaysForCohort = function () {
        var runtimeValues = bubblePopNewObj.getRuntimes();
        if(bubblePopV2 == null || !runtimeValues.hasOwnProperty("cohortArr") || !runtimeValues.hasOwnProperty("cohortValues") || runtimeValues.cohortValues.length != runtimeValues.cohortArr.length ) {
            return false;
        }

        if((bubblePopV2.initTime + ((HISTORY_SIZE-1) * constantsObj.SECONDS_IN_DAY)) > utilObj.getCurrentTimestamp() || bubblePopV2.dauDays.length < HISTORY_SIZE){
            return false;
        }
        var userCohortIndex = gameObj.getUserCohortIndex(runtimeValues.cohortArr);
        return runtimeValues.cohortValues[userCohortIndex] <= bubblePopNewObj.getDauDaysFromHistory("");
    };

    /**
     * Custion eligibility condition to see whether user is past min wallet balance required
     * @returns {bool}
     * */
    this.minWalletBalanceSatisfied = function(action) {
        var index = bubblePopNewObj.getIndexByBubbleType(action) - BUBBLE_WORD;
        var runtimes = bubblePopNewObj.getRuntimes();
        return (runtimes.minWalletBalance[index] <= xPromoGameSpecificObj.getCash());
    };

    /**
     * Custion eligibility condition to see whether user matches XpromoHistory requirement -> runtimes.hasUserEverXpromoted == 1 ->show only to those xpromoted, runtimes.hasUserEverXpromoted == 0 ->show only to those never xpromoted , runtimes.hasUserEverXpromoted == 2 show to all
     * @returns {bool}
     * */
    this.isUserMatchingXpromoRequirement = function(action){
        var runtimes = bubblePopNewObj.getRuntimes();
        var index = bubblePopNewObj.getIndexByBubbleType(action) - BUBBLE_WORD;
        if(bubblePopV2 == null || !runtimes.hasOwnProperty("hasUserEverXpromoted")){
            return false;
        }
        var hasUserXpromoted = bubblePopNewObj.hasXpromoted();
        return runtimes.hasUserEverXpromoted[index] == 2 || (runtimes.hasUserEverXpromoted[index] == 1 && hasUserXpromoted) || (runtimes.hasUserEverXpromoted[index] == 0 && !hasUserXpromoted);
    };
    /**
     * Checks whether user has ever cross promoted
     * @return {bool}*/
    this.hasXpromoted = function(){
        var allQuests = questsObj.getAllQuests();
        if(allQuests == null || allQuests.length == 0  ){
            return false;
        }
        for(var id in allQuests){
            if(allQuests[id] == undefined){
                continue;
            }
            var questId = id;
            var questType = allQuests[id].qt;
            var status = questsObj.getCrossGameQuestStatusForId(questId);
            if((status == xpromoConstantsObj.QUEST_COMPLETED || status == xpromoConstantsObj.QUEST_CLAIMED ) && questType == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME){
                return true;
            }
        }
        return false;
    };

    ///////////////////////////////////////////////
    //////////Xpromo Word Condtions///////////////
    /////////////////////////////////////////////

    /**
     * Checks to see whether any game is eligibile for xpromoWordPopup quest
     * @return {bool}*/

    this.isAnyGameEligibleForXpromoWord = function(){
        var result = false;
        var adwallQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopNewObj.getRuntimes();
        var questPrioritiesArr = Object.keys(adwallQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        if (questPrioritiesArr.length > 0) {
            for (var i = 0; (i < questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
                var game = questPrioritiesArr[i];
                var adwallQuest = adwallQuests[game];
                var gameId = adwallQuest.gh, questId = adwallQuest.id;
                if (!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId)) {
                    continue;
                }

                if(gameId != constantsObj.GAME_ID && runtimeValues.gamesForAdditionalXpromo.indexOf(parseInt(gameId)) == -1){
                    continue;
                }

                if(bubblePopNewObj.isXpromoWordViewOrClickCooldownReached(gameId)) {
                    continue;
                }
                if(bubblePopNewObj.isXpromoAditionalRewardCreativeAvailable(gameId,true)) {
                    result = result || true;
                }
            }
        }
        return result;
    };

    /**
     * Checks whether game icon available on device and downloads on fail if flag is set to true
     * @return {bool}*/
    this.isXpromoAditionalRewardCreativeAvailable = function(gameId, downloadOnFail = false){
        var result = false;
        var imageName = "icon_" + gameId + ".png";
        result = utilObj.fileExists(imageName);
        if (!result && downloadOnFail == true) {
            var psIconLink = xpromoUtilObj.getImageLinkFromName(imageName);
            xPromoGameSpecificObj.downloadFileV2(psIconLink, imageName);
        }
        return result;
    };

    /**
     * Checks whether click or view cool down reached for any game for xpromoWordClicks
     * @return {bool}*/

    this.isXpromoWordViewOrClickCooldownReached = function(gameId){
        var result = false;

        var runtimeValues = bubblePopNewObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;

        if (bubblePopV2.xpromoWordCooldowns[gameId] != undefined && (bubblePopV2.xpromoWordCooldowns[gameId].viewCount >= xpromoViewCoolDown || bubblePopV2.xpromoWordCooldowns[gameId].clickCount >= xpromoClickCoolDown)) {
            result = true;
        }
        return result;
    };

    /**
     * Checks whether click or view cool down reached for any game for xpromoWordClicks and resets it if all games have reached the same
     * */
    this.checkResetXpromoWordCooldowns = function () {
        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopNewObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;
        var questPrioritiesArr = Object.keys(xpromoQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        var reset = true;

        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit && reset); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];
            if(xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) && bubblePopNewObj.isXpromoAditionalRewardCreativeAvailable(gameId,false) && (runtimeValues.gamesForAdditionalXpromo.indexOf(parseInt(gameId)) != -1)) {
                if(bubblePopV2.xpromoWordCooldowns[gameId] != undefined ) {
                    if(bubblePopV2.xpromoWordCooldowns[gameId].viewCount < xpromoViewCoolDown && bubblePopV2.xpromoWordCooldowns[gameId].clickCount < xpromoClickCoolDown) {
                        reset = false;
                    }
                } else {
                    reset = false;
                }
            }
        }

        if(reset) {
            bubblePopV2.xpromoWordCooldowns = {};
        }
    };

    /////////////////////////////////////////////////////////
    /////////////////Word quest functions///////////////////
    ///////////////////////////////////////////////////////

    /**
     * Shows word of day popup for word type bubble
     **/
    this.showWordQuest = function(){
        var wordOfTheDay = pspnGameObj.getWordOfTheDay();
        var runtimes = bubblePopNewObj.getRuntimes();
        wordOfDayPopup.show({
            showXPromo: false,
            word: wordOfTheDay,
            icon: null,
            gName: "",
            numPuzzles: 0,
            wisdomScore: runtimes.runtimeLeafReward[0],
            bubbleName : "word",
        });
    };

    ///////////////////////////////////////////////////////
    //////////////Xpromo-word quest functions/////////////
    /////////////////////////////////////////////////////
    /**
     * Shows word of day popup for xpromo-word type bubble and stores interaction info regarding the game viewed in the popup
     **/
    this.showXpromoWordQuest = function () {
        if ((curLayer == gameLayer || lastLayer == gameLayer) && softLaunchObj.getIsPuzzleCompleted() == true) {
            return false;
        }
        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = bubblePopNewObj.getRuntimes();
        var questPrioritiesArr = Object.keys(xpromoQuests);
        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });


        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];

            if(!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) || !bubblePopNewObj.isXpromoAditionalRewardCreativeAvailable(gameId,false) || bubblePopNewObj.isXpromoWordViewOrClickCooldownReached(gameId) || runtimeValues.gamesForAdditionalXpromo.indexOf(parseInt(gameId)) == -1) {
                continue;
            }


            if (bubblePopV2.xpromoWordCooldowns[gameId] == undefined) {
                bubblePopV2.xpromoWordCooldowns[gameId] = {
                    "viewCount": 0,
                    "clickCount": 0,
                };
            }

            bubblePopV2.xpromoWordCooldowns[gameId].viewCount++;
            var imgName = "icon_" + gameId + ".png";
            var gameName = xpromoConstantsObj.PSGames[gameId][2];
            if(gameId == 8 || gameId == "8"){
                gameName = "DTC";
            }
            var wordOfTheDay = pspnGameObj.getWordOfTheDay();
            var runtimes = bubblePopNewObj.getRuntimes();
            var levelRqd = runtimes.levelsForAdditionalXpromo[runtimes.gamesForAdditionalXpromo.indexOf(parseInt(gameId))];
            wordOfDayPopup.show({
                showXPromo: true,
                word: wordOfTheDay,
                icon: imgName,
                gName: gameName,
                numPuzzles: levelRqd,
                wisdomScore: runtimes.runtimeLeafReward[1],
                xpromoWisdomScore:bubblePopNewObj.getWisdomLeafRewardForXpromo(),
                gameId : gameId,
                bubbleName : "xpromo-word",
                redirectionCallback: bubblePopNewObj.redirectionCallback,
            });
            return true;
        }
        return false;
    };

    /**
     * Redirection CallBack which gets called on popup continue that will redirect to playstore / game after relevant info is stored
     * */
    this.redirectionCallback = function (gameId,bubbleName,screenName,level) {
        var adwallQuests = questsObj.getAdWallQuests();
        var questData;
        var runtimes = bubblePopNewObj.getRuntimes();
        for (var quest in adwallQuests) {
            if (adwallQuests[quest].gh != constantsObj.GAME_ID && gameId == adwallQuests[quest].gh) {
                questData = adwallQuests[quest];
                break;
            }
        }

        var questId = questData.id;
        bubblePopNewObj.markXpromoWordInteraction(gameId, questId, bubbleName,level);
        var campaignName, redirectionParams, trackId;
        campaignName = xpromoConstantsObj.TRACK_BUBBLE_POP;
        var bubblePopExpVar = epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_BUBBLE_POP);
        trackId = campaignName + "_" + bubblePopExpVar;
        redirectionParams = xpromoUtilObj.generateRedirectionParams(questData);
        if (questsObj.getShouldIncrementOnClick() == 1) {
            questsObj.setShouldIncrementOnClick(0);
        }
        questsObj.redirectToStoreWithTracking(campaignName, screenName, redirectionParams, trackId);
    };

    /**
     * Used to store relevant info about the interaction via the xpromo word popup
     * */

    this.markXpromoWordInteraction = function(gameId, questId, bubbleName,level) {
        if(!bubblePopNewObj.isBubblePopActive()) {
            return;
        }

        if(bubblePopV2 == null || bubblePopV2.xpromoWordCooldowns[gameId] == undefined) {
            return;
        }

        questsObj.makeRelatedQuestsStatusStarted(questId);
        xpromoUtilObj.markXpromoStartedFromOtherSource(gameId, xpromoConstantsObj.TRACK_BUBBLE_POP_V2_WORD);

        bubblePopV2.xpromoWordClicks[gameId] = {
            "ts": utilObj.getCurrentTimestamp(),
            "rewards": {"wisdom-leaves": bubblePopNewObj.getWisdomLeafRewardForXpromo()},
            "questId": questId,
            "bubbleName": bubbleName,
            "levelToCompleteAtView":level
        };

        bubblePopV2.xpromoWordCooldowns[gameId].clickCount++;
    };

    /**
     * Executed on screen load of bubblePop
     * Checks whtether the game is installed and whether the level required has been cleared and grants reward in that case
     * */
    this.checkAndGrantXpromoWordReward = function(){
        if(!bubblePopNewObj.isBubblePopActive()) {
            return;
        }
        if(bubblePopV2 == null || bubblePopV2.xpromoWordClicks == undefined || bubblePopNewObj.getAdditionalRewardForXpromoVariant() == xpromoConstantsObj.EXP_VAR_CONTROL) {
            return;
        }

        // NOT (Either on valid bubble screen or (popup is open && lastscreen was game screen))
        if(!(bubblePopNewObj.isValidScreenForWordBubbles() || (popupLayer.popupOpen && lastScreen == constantsObj.SCREEN_GAME))) {
            return;
        }

        var attributionPeriod = xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_SURFACING_ATTRIBUTION_WINDOW) * 60 * 60;
        var curTime = utilObj.getCurrentTimestamp();
        utilObj.resetIsAppInstalledMap();
        for(var gameId in bubblePopV2.xpromoWordClicks) {
            var delEntry = false;
            var clickData = bubblePopV2.xpromoWordClicks[gameId];
            var clickTime = clickData["ts"];
            var levelCompletion = parseInt(clickData["levelToCompleteAtView"]);
            var questId = clickData["questId"];
            var bubbleName = clickData["bubbleName"];

            var incentivizedElsewhere = xpromoUtilObj.isGameAlreadyMarkedForRewarding(gameId, xpromoConstantsObj.TRACK_BUBBLE_POP_V2_WORD);

            if(!incentivizedElsewhere && clickTime > 0 && ((clickTime + attributionPeriod) >= curTime)
                && xpromoUtilObj.isAppInstalledByGameId(gameId) && xpromoUtilObj.getLevelForGame(gameId) >= levelCompletion) {
                // This is to account for the scenario when W2E for the same game is seen after BP,
                // In this scenario, only the W2E reward should be granted and not BP. Hence the check for gameId below
                if(xpromoW2eObj.getXpromoGameId() != gameId && questsObj.getCrossGameQuestStatusForId(questId) != xpromoConstantsObj.QUEST_CLAIMED && adWallObj.getXpromoInteractionTime(gameId) < clickTime && bubblePopObj.getXpromoInteractionTime(gameId) < clickTime) {
                    // Lets get the current and last screen layers
                    var layerForAnimationOnLastScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(lastScreen);
                    var layerForAnimationOnCurScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen);
                    var imgName = "icon_" + gameId + ".png";
                    var gameName = xpromoConstantsObj.PSGames[gameId][2];
                    if(gameId == 8 || gameId == "8"){
                        gameName = "DTC";
                    }
                    var paramsMap = {"rewards": clickData["rewards"], "source": "bubblepop-xpromo-word", "bubbleName": bubbleName, "phylum": questId, "gameName":gameName , "icon":imgName};
                    // If the popup is open and we are on the xpromo screen, lets close and do a delayed reward grant.
                    if (popupLayer.popupOpen && lastScreen == constantsObj.SCREEN_GAME) {
                        popupLayer.closeWithDefault();
                        utilObj.delayedCallback(layerForAnimationOnLastScreen, bubblePopNewObj.grantSpecialReward, 2, paramsMap, false);
                    } else { // Else grant the reward immediately
                        bubblePopNewObj.grantSpecialReward(layerForAnimationOnCurScreen, paramsMap);
                    }
                    questsObj.makeRelatedQuestsForW2eClaimed(questId);
                }
                delEntry = true;
            }

            if (((clickTime + attributionPeriod) < curTime) || incentivizedElsewhere) {
                delEntry = true;
            }
            if(delEntry) { // Lets delete the rewards for the gameId as they are either granted to expired
                delete bubblePopV2.xpromoWordClicks[gameId];
            }
        }
    };

    /**
     * Method to mark that xpromo has been started from other sources and hence this source is no longer valid to grant incentives
     * @param gameId
     */
    this.markXpromoStartedFromOtherSource = function (gameId) {
        if(!bubblePopNewObj.isBubblePopActive()) {
            return;
        }

        if(bubblePopV2 == null || bubblePopV2.xpromoWordClicks == undefined || bubblePopNewObj.getAdditionalRewardForXpromoVariant() == xpromoConstantsObj.EXP_VAR_CONTROL) {
            return;
        }

        if(bubblePopV2.xpromoWordClicks[gameId] != undefined) {
            delete bubblePopV2.xpromoWordClicks[gameId];
        }
    };

    /**
     * Method to check if a particular game is already marked for granting reward - this is to ensure we do not do double grants from other sources
     * @param gameId
     * @returns {boolean}
     */
    this.isGameAlreadyMarkedForRewarding = function(gameId) {
        if(!bubblePopNewObj.isBubblePopActive()) {
            return false;
        }

        if(bubblePopV2 == null || bubblePopV2.xpromoWordClicks == undefined || bubblePopNewObj.getAdditionalRewardForXpromoVariant() == xpromoConstantsObj.EXP_VAR_CONTROL) {
            return false;
        }

        if(bubblePopV2.xpromoWordClicks[gameId] != undefined) {
            var attributionPeriod = xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_SURFACING_ATTRIBUTION_WINDOW) * 60 * 60;
            var curTime = utilObj.getCurrentTimestamp();
            var clickData = bubblePopV2.xpromoWordClicks[gameId];
            var clickTime = clickData["ts"];
            return (clickTime > 0 && ((clickTime + attributionPeriod) >= curTime));
        }


        return false;
    };

    /**
     * Getter for Xpromo Wisdom leaf reward
     * */

    this.getWisdomLeafRewardForXpromo = function () {
      var runtimes = bubblePopNewObj.getRuntimes();
      return runtimes.runtimeLeafRewardXpromo;
    };




    ////////////////////////////////////////////////////////////////////////
    //////// Starting Section for Tracking Utility methods  ////////////////
    ////////////////////////////////////////////////////////////////////////

    /**
     * Utility method to clear the tracking map
     */
    this.initiateBubbleTracking = function() {
        trackingMap = {};
    };

    /**
     * Utility method to store a particular field - value pair in the tracking map
     * @param field - string for the tracking field
     * @param value - string value that needs to be stored
     */
    this.setTrackingDetails = function(field, value) {
        trackingMap[field] = value;
    };

    /**
     * Method to run the track call with the given set of parameters set.
     * This design was added to handle flows where different parameters were set in different methods of the flow and
     * passing the data between them was cumbersome.
     */
    this.applyTracking = function() {
        var keys = Object.keys(trackingMap);
        if(keys.length > 0) {
            var phylum = (trackingMap[xpromoConstantsObj.TRACK_PHYLUM] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_PHYLUM] : "";
            var cls = (trackingMap[xpromoConstantsObj.TRACK_CLASS] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_CLASS] : "";
            var family = (trackingMap[xpromoConstantsObj.TRACK_FAMILY] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_FAMILY] : "";
            var genus = (trackingMap[xpromoConstantsObj.TRACK_GENUS] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_GENUS] : "";
            var species = (trackingMap[xpromoConstantsObj.TRACK_SPECIES] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_SPECIES] : "";
            var order = trackingMap[xpromoConstantsObj.TRACK_ORDER] != undefined ? trackingMap[xpromoConstantsObj.TRACK_ORDER] : xpromoConstantsObj.TRACK_BUBBLE_POP_V2;
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, phylum, cls,
                order, family, genus, species, "", "");
        }
    };

    this.getTrackingForPhylum = function () {
        return trackingForPhylum;
    }

    this.setTrackingForPhylum = function (val) {
        trackingForPhylum = val;
    }

    ////////////////////////////////////////////////////////////////////////
    //////// Ending Section for Tracking Utility methods  //////////////////
    ////////////////////////////////////////////////////////////////////////


};
