XpromoDailyQuests = function () {
    var oldQuestData = null;
    var lastOldDataUpdate = null;
    var streakQuestTypesForToday = undefined; // Session variable - Used for caching current streak quests for today in the session
    var streakQuestCacheDate = undefined; // Session variable - Date representing the day the streak quests were cached - facilitates date rollover during session
    var stringifiedStreakQuests = ""; // Session variable - Streak Quests extracted out of the base cur_quest.json and stringified
    var questLevelsMap = {};

    // Persistent Data fields used for DQ Replacement
    var dailyQuestToBeReplaced = null;
    var installQuestToReplaceDailyQuest = null;
    var dailyQuestLastReplacedMidnight = null;

    var dqReplacementPos = -1; // Session variable used for dq replacement position

    /**
     * Loading the user data into memory
     * @param quests
     */
    this.load = function (quests) {
        if (quests.oldQuestData != undefined) {
            oldQuestData = quests.oldQuestData;
        }

        if(quests.lastOldDataUpdate != undefined) {
            lastOldDataUpdate = quests.lastOldDataUpdate;
        }

        if(quests.dailyQuestToBeReplaced != undefined){
            dailyQuestToBeReplaced = quests.dailyQuestToBeReplaced;
        }

        if(quests.installQuestToReplaceDailyQuest != undefined){
            installQuestToReplaceDailyQuest = quests.installQuestToReplaceDailyQuest;
        }

        if(quests.dailyQuestLastReplacedMidnight != undefined){
            dailyQuestLastReplacedMidnight = quests.dailyQuestLastReplacedMidnight;
        }

    };

    /**
     * Updating the user data
     * @param quests
     * @returns {*} updates quests object
     */
    this.save = function (quests) {
        quests.lastOldDataUpdate = lastOldDataUpdate;
        quests.oldQuestData = oldQuestData;
        quests.dailyQuestToBeReplaced = dailyQuestToBeReplaced;
        quests.installQuestToReplaceDailyQuest = installQuestToReplaceDailyQuest;
        quests.dailyQuestLastReplacedMidnight = dailyQuestLastReplacedMidnight;
        return quests;
    };

    /**
     * Utility method to check if the old quest data needs to be updated (if its not updated on the current date).
     * @returns {boolean} True / False
     */
    this.shouldUpdateOldData = function() {
        return (lastOldDataUpdate !== xpromoUtilObj.getCurrentDateInString());
    };

    /**
     * Wrapper to be used while trying to update the old quest data, ensures if the required conditions are met
     * @param data - quest data
     */
    this.tryUpdateOldQuestData = function() {
        if(dailyQuestsObj.shouldUpdateOldData() && stringifiedStreakQuests != "" && stringifiedStreakQuests != undefined && stringifiedStreakQuests != null) {
            oldQuestData = stringifiedStreakQuests;
            lastOldDataUpdate = xpromoUtilObj.getCurrentDateInString();

            questsObj.setExperimentsToChangeOnDailyBasis();

        }
    };

    /**
     * Method to check if the streak quests are currently active.
     * @returns {boolean} True / False
     */
    this.isStreakQuestsExpActive = function() {
        return xPromoGameSpecificObj.areStreakQuestsEnabled() && (questsObj.getExperimentsToChangeOnDailyBasis(xpromoConstantsObj.EXP_DAILY_QUEST_V2) == constantsObj.EXP_VAR_1);
    };

    /**
     * Merges older quests to the curQuest data for Daily Quests
     * @param curQuests - The curQuest object in which the data needs to be replaced
     */
    this.mergeOlderQuestsIfEligible = function(curQuests) {
        // The merge needs to run only when the experiment variant is set to 1.
        if(!dailyQuestsObj.isStreakQuestsExpActive()) {
            return;
        }

        if(oldQuestData != null) {
            var oldQuests = JSON.parse(oldQuestData);
            var quest;
            var questToRemove = [];

            // Lets delete all the streak quests data from the cur data list
            for (quest in curQuests) {
                if(curQuests[quest] == undefined) {
                    continue;
                }
                if(dailyQuestsObj.isStreakQuest(curQuests[quest].qt)) {
                    questToRemove.push(quest);
                }
            }

            for(var index in questToRemove) {
                delete curQuests[questToRemove[index]];
            }

            // Lets replace all the streak quests data from the old data list
            for(quest in oldQuests) {
                if(oldQuests[quest] == undefined) {
                    continue;
                }

                if(dailyQuestsObj.isStreakQuest(oldQuests[quest].qt)) {
                    curQuests[quest] = oldQuests[quest];
                }
            }
        }
    };

    /**
     * Creates a stringified object for the curQuests with the streak quests data only
     * @param curQuests - Complete Quest JSON object
     */
    this.stringifyStreakQuests = function(curQuests) {
        var result = {};
        for (var quest in curQuests) {
            if(curQuests[quest] == undefined) {
                continue;
            }

            if(dailyQuestsObj.isStreakQuest(curQuests[quest].qt)) {
                result[quest] = curQuests[quest];
            }
        }

        stringifiedStreakQuests = JSON.stringify(result);
    };

    /**
     * Getter method to fetch the current set of streak quests which are active for today
     * @returns Object - An object with questTypes as keys which needs to be surfaced today.
     */
    this.getStreakQuestTypesForToday = function () {
        if(streakQuestTypesForToday != undefined && streakQuestCacheDate != undefined && streakQuestCacheDate == xpromoUtilObj.getCurrentDateInString()) {
            return streakQuestTypesForToday;
        }

        // Its not yet set, hence need to calculate for the first time in the day
        var streakQuestTypes = {};
        var curQuests = questsObj.getCurQuests();
        for (var quest in curQuests) {
            var questData = curQuests[quest];
            if(questData != undefined) {
                var questType = questData.qt;
                if (questType.indexOf(xpromoConstantsObj.QUEST_TYPE_STREAK_QUESTS) == 0) {
                    if((questData.pf == 0 && questsObj.getPayerStatusForDailyQuests() == xpromoConstantsObj.BECAME_PAYER) || (questData.pf == 1 && questsObj.getPayerStatusForDailyQuests() < xpromoConstantsObj.BECAME_PAYER)) {
                        continue;
                    }

                    var bucket = curQuests[quest].b, questTypeFound = constantsObj.NOT_FOUND;
                    if (streakQuestTypes[bucket] == undefined) {
                        streakQuestTypes[bucket] = [];
                    }
                    for (var i=0; i<streakQuestTypes[bucket].length; i++) {
                        if (streakQuestTypes[bucket][i] == questType) {
                            questTypeFound = i;
                            break;
                        }
                    }
                    if (questTypeFound == constantsObj.NOT_FOUND) {
                        streakQuestTypes[bucket].push(questType);
                    }
                }
            } else {
                trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "questData-undefined", quest + "", questsObj.getcurQuestVersion() + "", "", "", "", "");
            }
        }

        var streakQuestTypesKeys = Object.keys(streakQuestTypes);

        if(streakQuestTypesKeys.length >= 3) {

            var streakQuestsLengthArr = [];
            for (var bucket in streakQuestTypes) {
                streakQuestsLengthArr.push(streakQuestTypes[bucket].length);
            }
            var streakPatternArr = dailyQuestsObj.getStreakQuestsPattern(streakQuestsLengthArr, streakQuestTypes);

            if(streakPatternArr.length > 0) {
                var today = new Date();
                var utc = Date.UTC(today.getFullYear(), today.getMonth(), today.getDate(), 0, 0, 0, 0) / 1000;
                var days = Math.floor(utc / constantsObj.SECONDS_IN_DAY);

                var indexInStreakPatternArr = days % streakPatternArr.length;
                var streakPatternToBeUsed = streakPatternArr[indexInStreakPatternArr];

                if(streakPatternToBeUsed != undefined) {

                    var questsTypesThatCanBeUsedToday = {}, iteratorForQuestTypes = 0;
                    for (var bucket in streakQuestTypes) {
                        questsTypesThatCanBeUsedToday[streakQuestTypes[bucket][streakPatternToBeUsed[iteratorForQuestTypes]]] = 1;
                        iteratorForQuestTypes++;
                    }
                    dailyQuestsObj.setStreakQuestTypesForToday(questsTypesThatCanBeUsedToday);
                    return questsTypesThatCanBeUsedToday;

                } else {
                    trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "streakPatternToBeUsed-fail", indexInStreakPatternArr + "", questsObj.getcurQuestVersion() + "", "", "", "", "");
                }
            } else {
                trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "streakPatternArr-fail", streakPatternArr.length + "", questsObj.getcurQuestVersion() + "", "", "", "", "");
            }
        } else {
            trackObj.trackCounter(constantsObj.TRACK_DEBUG, xpromoConstantsObj.TRACK_QUEST_CENTER_DAILY_QUESTS, "streakQuestTypesKeys-fail", streakQuestTypesKeys.length + "", questsObj.getcurQuestVersion() + "", "", "", "", "");
        }

        dailyQuestsObj.setStreakQuestTypesForToday({});
        return {};

    };

    /**
     * Wrapper for updating streakQQuestTypesForToday, it also updates the cached date
     * @param val - streaQuestTypesData in Object format.
     */
    this.setStreakQuestTypesForToday = function (val) {
        streakQuestTypesForToday = val;
        streakQuestCacheDate = xpromoUtilObj.getCurrentDateInString();
    };

    /**
     * Wrapper method to get the progress for a specific quest type - the progress is evaluated based on the mapping
     * provided for the task in dailyQuestsConfig.json
     * @param questType
     * @returns {number} - Number representing the current progress
     */
    this.getDailyQuestsStatus = function (questType) {
        var curLevel = 0;
        if (questConfigObj.isDailyQuestConfigInJson(questType)) {
            var functionName = questConfigObj.getDailyQuestConfigData(questType, xpromoConstantsObj.GET_SCORE);
            curLevel = eval(functionName).apply(this);
        }
        return curLevel;
    };

    this.showInGameDailyQuests = function (questData, lastDailyQuestsMap, candidatesInBucket, currentDailyQuestMap, dailyQuestsEligibleToBeShown) {
        var curTimestamp = utilObj.getCurrentTimestamp();
        var inGameProgress = questsObj.getInGameProgressMap();
        var questBucket = questData.b;
        var questId = questData.id;
        var questPriority = questData.p;
        if (questData.cr == undefined || questData.pf == undefined || !dailyQuestsObj.isDailyQuestsEligibleDays()) {
            return;
        }

        if ((questData.pf == 0 && questsObj.getPayerStatusForDailyQuests() == xpromoConstantsObj.BECAME_PAYER) || (questData.pf == 1 && questsObj.getPayerStatusForDailyQuests() < xpromoConstantsObj.BECAME_PAYER)) {
            questsObj.setDailyQuestLocked(0);
            return;
        }

        if (dailyQuestsObj.canQuestBeShown(questData) == false) {
            return;
        }

        var multiplier = questFlagsObj.getMultiplierForBucket(questData.b);
        var questStatus = xpromoConstantsObj.QUEST_STARTED;
        var curLevel = dailyQuestsObj.getDailyQuestsStatus(questData.qt);
        var remainingTime = constantsObj.SECONDS_IN_DAY - curTimestamp % constantsObj.SECONDS_IN_DAY;

        var requiredLevel = dailyQuestsObj.getProgressRequired(questData);
        if (requiredLevel == "na" || requiredLevel == 0 || requiredLevel == undefined || requiredLevel == "undefined") {
            questsObj.setDailyQuestLocked(0);
            return;
        }
        if (inGameProgress[questId] == undefined) {
            inGameProgress[questId] = {
                "id": questId,
                "status": xpromoConstantsObj.QUEST_STARTED,
                "params": [curLevel]
            };
        }
        requiredLevel = Math.ceil(requiredLevel * multiplier);
        questData.d = questsObj.makeStringAppropriate(questData.d, requiredLevel, questData.qt);
        if (curLevel >= requiredLevel) {
            inGameProgress[questId].params[0] = requiredLevel;
            curLevel = requiredLevel;
            questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_COMPLETED);
        }
        questStatus = inGameProgress[questId].status;

        var d = new Date();
        if (inGameProgress[questId] != undefined && inGameProgress[questId].status == xpromoConstantsObj.QUEST_COMPLETED) {
            questData.curProgress = requiredLevel;
        } else {
            questData.curProgress = curLevel;
        }
        questData.reqProgress = requiredLevel;
        questData.status = questStatus;
        questData.params = [curLevel, curTimestamp];
        if (inGameProgress[questId] != undefined) {
            inGameProgress[questId].params = questData.params;
        }
        var timeRemaining = remainingTime + d.getTimezoneOffset() * constantsObj.SECONDS_IN_MIN;
        if (timeRemaining < 0) {
            timeRemaining = constantsObj.SECONDS_IN_DAY + timeRemaining;
        }
        questData.timeLimit = timeRemaining;
        var lastDailyQuest = constantsObj.NOT_FOUND;
        if (lastDailyQuestsMap[questBucket] == undefined) {
            lastDailyQuestsMap[questBucket] = questData;
        } else {
            var lastDailyQuest = lastDailyQuestsMap[questBucket].p;
            if (questPriority > lastDailyQuest) {
                lastDailyQuestsMap[questBucket] = questData;
            }
        }
        if (inGameProgress[questId] == undefined || inGameProgress[questId].status != xpromoConstantsObj.QUEST_CLAIMED) {
            candidatesInBucket[questBucket].push(questData);
        }
        if (inGameProgress[questId] == undefined || inGameProgress[questId].status == xpromoConstantsObj.QUEST_STARTED) {
            candidatesInBucket[questBucket].push(questData);
            currentDailyQuestMap[questBucket] = questData;
        }

        if(questLevelsMap == undefined) {
            questLevelsMap = {};
        }
        if(questLevelsMap[questData.qt] == undefined) {
            questLevelsMap[questData.qt] = [];
        }
        questLevelsMap[questData.qt].push(questId);

        dailyQuestsEligibleToBeShown[questId] = 1;
    };

    /**
     * this function generates all the possible patterns for streak quests
     * @param streakQuestsLengthArr this is an array which contains number of different quest types inside a buckets
     * @returns {Array} return all the patterns for streak quest types.
     */
    this.getStreakQuestsPattern = function (streakQuestsLengthArr) {
        var failCount = 0;
        var firstBucketLength = streakQuestsLengthArr[0], secondBucketLength = streakQuestsLengthArr[1], thirdBucketLength = streakQuestsLengthArr[2];
        var bucketMap = [0, 0, 0];
        var streakQuestPatternArr = [], streakPatternMap = {}, differentStreakPatternsGenerated = 0, iterationNumber = 0;
        while (failCount <= firstBucketLength * secondBucketLength * thirdBucketLength) {
            if ((streakQuestPatternArr.length == 0 || dailyQuestsObj.checkIfPatternCanBeAdded(streakQuestPatternArr[streakQuestPatternArr.length - 1], bucketMap, streakPatternMap, streakQuestsLengthArr))) {
                streakQuestPatternArr.push([bucketMap[0], bucketMap[1], bucketMap[2]]);
                var pattern = bucketMap[0] + "-" + bucketMap[1] + "-" + bucketMap[2];
                streakPatternMap[pattern] = 1;
                failCount = 0;
            }
            failCount++;
            iterationNumber++;
            bucketMap[2] = iterationNumber % thirdBucketLength;
            bucketMap[1] = (Math.floor(iterationNumber / thirdBucketLength)) % secondBucketLength;
            bucketMap[0] = (Math.floor(iterationNumber / (thirdBucketLength * secondBucketLength))) % firstBucketLength;
        }
        return streakQuestPatternArr
    };

    /**
     * checks whether lastBucketMap can be inserted in streakPatternMap
     * @param lastBucketMap contains the array of quest types which were last inserted in streakPatternMap
     * @param currentBucketMap contains the array of quest types which are required to be checked, so as to insert in streakPatternMap
     * @param streakPatternMap contains all the combinations of quest types which can be used
     * @param streakQuestsLengthArr this is an array which contains number of different quest types inside a buckets
     * @returns {boolean} if currentBucketMap has atleast two different quest types than lastBucketMap
     */
    this.checkIfPatternCanBeAdded = function (lastBucketMap, currentBucketMap, streakPatternMap, streakQuestsLengthArr) {
        if (lastBucketMap == undefined) {
            return false;
        }
        var currentPattern = currentBucketMap[0] + "-" + currentBucketMap[1] + "-" + currentBucketMap[2];
        if (streakPatternMap[currentPattern] != undefined) {
            return false;
        }
        var numOfDifferences = 0;

        for(var i = 0 ; i < 3; i++) {
            if(lastBucketMap[i] != currentBucketMap[i]) {
                numOfDifferences++;
            }
        }

        var singularBucket = 0;
        for(var i = 0; i < 3; i++) {
            var bucketLen = streakQuestsLengthArr[i];
            if(bucketLen == 1) {
                singularBucket++;
            }
        }

        return (singularBucket == 3) || (singularBucket == 2 && numOfDifferences >= 1) || (numOfDifferences >= 2);
    };

    /**
     * Method to check if a given daily / streak quest be shown or not
     * @param questData - Quest Data for the corresponding quest
     * @returns {boolean}
     */
    this.canQuestBeShown = function (questData) {
        var result = false;
        var questType = questData.qt;
        if (!dailyQuestsObj.isStreakQuestsExpActive() || Object.keys(dailyQuestsObj.getStreakQuestTypesForToday()).length == 0) {
            result = (questType.indexOf(xpromoConstantsObj.QUEST_TYPE_DAILY_QUESTS) == 0);
        } else {
            var questsTypesThatCanBeUsedToday = dailyQuestsObj.getStreakQuestTypesForToday();
            result = (questsTypesThatCanBeUsedToday[questType] != undefined);
        }
        return result;
    };

    /**
     * Checks if a given questType is a streak quest
     * @param questType string representing quest type
     * @returns {boolean} True / False
     */
    this.isStreakQuest = function (questType) {
        return (questType.indexOf(xpromoConstantsObj.QUEST_TYPE_STREAK_QUESTS) == 0);
    };

    /**
     * Checks if a given questType is a daily quest
     * @param questType string representing quest type
     * @returns {boolean} True / False
     */
    this.isDailyQuest = function (questType) {
        return (questType.indexOf(xpromoConstantsObj.QUEST_TYPE_DAILY_QUESTS) == 0);
    };

    /**
     * Checks if a given questType is a daily or a streak quest
     * @param questType string representing quest type
     * @returns {boolean} True / False
     */
    this.isDailyQuestOrStreakQuest = function (questType) {
        return dailyQuestsObj.isStreakQuest(questType) || dailyQuestsObj.isDailyQuest(questType);
    };

    /**
     * Returns the quest type if the given quest config is present in the JSON
     * @param questType string representing quest type
     * @returns {*} (questType, if it is present in JSON) / false.
     */
    this.isDailyQuestOrStreakQuestEnabled = function (questType) {
        if (questConfigObj.isDailyQuestConfigInJson(questType)) {
            return questType;
        }
        return false;
    };

    /**
     * Returns if the daily quest type is unlocked in the game
     * @param questType string representing quest type
     */
    this.isQuestTypeUnlocked = function(questType) {
        var result = true;
        var unlockCheck = questConfigObj.getDailyQuestConfigData(questType, xpromoConstantsObj.QUEST_UNLOCK_CHECK);
        if (unlockCheck != "") {
            result = eval(unlockCheck).apply(this, [questType]);
        }
        return result;
    };

    this.resetQuestLevelMap = function() {
        questLevelsMap = {};
    };

    this.getDailyQuestLevel = function(questType, questId) {
        var levelToFetch = constantsObj.NOT_FOUND;
        if(questLevelsMap != undefined && questLevelsMap[questType] != undefined && utilObj.isArray(questLevelsMap[questType])) {
            levelToFetch = questLevelsMap[questType].indexOf(questId);
        }
        return levelToFetch;
    };

    this.getProgressRequired = function(questData) {
        // 1. Check if Daily Quest Personalization is Active
        if(xPromoGameSpecificObj.isDQPersonalizationEnabled()) {
            var levelToFetch = dailyQuestsObj.getDailyQuestLevel(questData.qt, questData.id);
            if(levelToFetch == constantsObj.NOT_FOUND) {
                // The quest is yet to be added to the map, lets use the length of the map for the particular quest type
                levelToFetch = 0;
                if(questLevelsMap != undefined && questLevelsMap[questData.qt] != undefined && utilObj.isArray(questLevelsMap[questData.qt])) {
                    levelToFetch = questLevelsMap[questData.qt].length;
                }
            }

            // 2. Call method to get personalized value with questData.qt and levelToFetch
            var reqProgress = xPromoGameSpecificObj.getRequiredDQProgress(questData.qt, levelToFetch);
            if(reqProgress != constantsObj.NOT_FOUND) {
                return reqProgress;
            }

            // 3. Incase the reqProgress == constantsObj.NOT_FOUND, we need to fallback to the non-personalized logic
        }

        return dailyQuestsObj.calculateProgressRequired(questData);
    };

    this.calculateProgressRequired = function(questData) {
        var allParams = questData.cr.split(';');
        var requiredLevel = 0;
        var daysFromInstall = dailyQuestsObj.getDQDaysFromInstall();

        // This logic is responsible for parsing the configurationa and figuring out the progress required
        for (var i = 0; i < allParams.length; i++) {
            if (allParams[i] != 0) {
                var paramsForCoherts = allParams[i].split(':');
                var cohertDays = paramsForCoherts[0].split('-');
                cohertDays = cohertDays[1];
                questsObj.setCohortDays(cohertDays);
                if (cohertDays == "i") {
                    requiredLevel = paramsForCoherts[1];
                } else if (daysFromInstall <= cohertDays) {
                    requiredLevel = paramsForCoherts[1];
                    break;
                }
            } else {
                break;
            }
        }

        return requiredLevel;
    };

    this.isDailyQuestsEligibleDays = function () {
        if (crossGameDailyQuestsObj.getShowDailyQuestLock() == false) {
            return true;
        }

        var daysFromInstall = dailyQuestsObj.getDQDaysFromInstall();
        var lockDays = dailyQuestsObj.getDailyQuestLockDays();

        if (lockDays <= 0) {
            return true;
        }
        return (daysFromInstall >= lockDays);
    };

    this.getDailyQuestUnlockTime = function () {
        var created = parseInt(userObj.getUserObj().createdAt);
        if(isNaN(created) || created == 0) {
            created = xPromoGameSpecificObj.getInstallTimeStamp();
        }

        var lockDays = dailyQuestsObj.getDailyQuestLockDays();

        var startDate = new Date((created + (lockDays * constantsObj.SECONDS_IN_DAY)) * 1000);
        startDate.setHours(0, 0, 0, 0);
        return startDate.getTime() / 1000;
    };

    this.getDailyQuestLockDays = function() {
        var lockDays = xpromoGameRuntimeObj.getDailyQuestsDaysNonPayer();
        if (xPromoGameSpecificObj.getIsPayerUser()) {
            lockDays = xpromoGameRuntimeObj.getDailyQuestsDaysPayer();
        }

        // config added
        if (utilObj.isStaging() && qaConfigObj.getQAConfigData("dailyQuestLocked") == 1) {
            lockDays = qaConfigObj.getQAConfigData("dailyQuestLockedTime");
        }

        return lockDays;
    };

    this.getDQDaysFromInstall = function() {
        var created = parseInt(userObj.getUserObj().createdAt);
        if(isNaN(created) || created == 0) {
            created = xPromoGameSpecificObj.getInstallTimeStamp();
        }

        return xpromoUtilObj.getDaysDiffBetweenTimestamps(created, utilObj.getCurrentTimestamp());
    };

    this.resetDailyQuestReplacementItems = function(){
        dailyQuestToBeReplaced = null;
        installQuestToReplaceDailyQuest = null;
    };

    this.setQuestToBeReplaced = function(val){
        if(dailyQuestToBeReplaced == null) {
            dailyQuestToBeReplaced = val;
        }
    };

    this.getQuestToBeReplaced = function(){
        return dailyQuestToBeReplaced;
    };

    this.getQuestToReplace = function(availableGameQuests, numOfAvailableGameQuests) {
        if(dailyQuestsObj.isEligibleForDQReplacement() && dailyQuestsObj.getQuestToBeReplaced()) {
            if(installQuestToReplaceDailyQuest == null) {
                for(var i = 0 ; i < numOfAvailableGameQuests ; i++){
                    if(availableGameQuests[i] != undefined) {
                        if(availableGameQuests[i].qt == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME && availableGameQuests[i].status != xpromoConstantsObj.QUEST_COMPLETED){
                            installQuestToReplaceDailyQuest = availableGameQuests[i].id;
                            break;
                        }
                    }
                }
            }
            if(installQuestToReplaceDailyQuest != null) {
                var curQuests = questsObj.getCurQuests();
                return curQuests[installQuestToReplaceDailyQuest];
            }
        }
        return null;
    };

    this.hasDailyQuestReplacementCompletedBeforeStart = function(){
        var curQuests = questsObj.getCurQuests();

        var gameIdHosted = constantsObj.NOT_FOUND;
        if(installQuestToReplaceDailyQuest != null || curQuests[installQuestToReplaceDailyQuest] != undefined){
            gameIdHosted = curQuests[installQuestToReplaceDailyQuest].gh;
        }

        return (installQuestToReplaceDailyQuest != null &&
            (questsObj.getCrossGameQuestStatusForId(installQuestToReplaceDailyQuest) == xpromoConstantsObj.QUEST_COMPLETED_BEFORE_START ||
                gameIdHosted == constantsObj.NOT_FOUND ||
                (xpromoUtilObj.isAppInstalledByGameId(gameIdHosted) && questsObj.getCrossGameQuestStatusForId(installQuestToReplaceDailyQuest) == xpromoConstantsObj.QUEST_NOT_STARTED)));
    };

    this.isDailyQuestReplacement = function(questId) {
        return dailyQuestsObj.isEligibleForDQReplacement() && installQuestToReplaceDailyQuest != null && installQuestToReplaceDailyQuest == questId && dailyQuestsObj.getQuestToBeReplaced() != null;
    };

    this.shouldQuestBeExcluded = function(questId, bucket) {
        var curQuests = questsObj.getCurQuests();
        if(installQuestToReplaceDailyQuest != null && curQuests[installQuestToReplaceDailyQuest] != undefined) {
            var currentQuest = curQuests[installQuestToReplaceDailyQuest];
            var bucketOfReplacement = currentQuest.b;
            var status = currentQuest.status;
            var relatedQuests = currentQuest["r"];
            if (relatedQuests != undefined && relatedQuests != "0") {
                relatedQuests = relatedQuests.split(":");
            }
            return dailyQuestsObj.isDailyQuestReplacement(questId) || (relatedQuests != undefined && relatedQuests.indexOf(questId) != -1) || (status != xpromoConstantsObj.QUEST_CLAIMED && bucket == bucketOfReplacement);
        }
        return false; // this is added for safe handling, ideally the flow should not reach here
    };

    this.setQuestReplacedTime = function(){
        var dailyQuestLastReplacedMidnightTS = new Date(utilObj.getCurrentTimestamp() * 1000);
        dailyQuestLastReplacedMidnightTS.setHours(0, 0, 0, 0);
        dailyQuestLastReplacedMidnight = dailyQuestLastReplacedMidnightTS.getTime() / 1000;
    };

    this.getDailyQuestReplacementVariant = function(){
        if(xPromoGameSpecificObj.isDailyQuestReplacementEnabled()) {
            return epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_DQ_REPLACEMENT);
        }
        return xpromoConstantsObj.EXP_VAR_CONTROL;
    };

    this.isEligibleForDQReplacement = function(){
        return (dailyQuestsObj.getDailyQuestReplacementVariant() != xpromoConstantsObj.EXP_VAR_CONTROL && dailyQuestsObj.isDQReplacementDayCountSatisfied() && !dailyQuestsObj.hasDailyQuestReplacementCompletedBeforeStart());
    };

    this.isDQReplacementDayCountSatisfied = function() {
        if(dailyQuestLastReplacedMidnight != null) {
            var daysDifference = xpromoUtilObj.getDaysDiffBetweenTimestamps(utilObj.getCurrentTimestamp(), dailyQuestLastReplacedMidnight);
            return (daysDifference == 0 || daysDifference >= parseInt(xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_DAILY_QUEST_REPLACEMENT_DAYS)));
        }
        return true;
    };

    this.getDQReplacementSocialQuestLabel = function (gameId) {
        var gameName = xpromoConstantsObj.PSGames[gameId][2];
        if(gameId == 8 || gameId == "8"){
            gameName = "DTC";
        }
        var socialFriend = socialQuestObj.getCurrentFriend();
        if(socialFriend != undefined && socialFriend["friend"] != undefined && socialFriend["friend"]["name"] != undefined) {
            return socialFriend["friend"]["name"] +" is playing " + gameName + ". Install Now!";
        }
        return "Install " + gameName;
    };

    this.setDQReplacementPosition = function(i) {
        if(dqReplacementPos == -1){
            dqReplacementPos = i+1;
        }
    };

    this.getDQReplacementPosition = function () {
        return dqReplacementPos;
    }
};