XPromoGiftBox = function () {

    var pspnGiftBox = null;

    var rt = null; // local variable to hold the currently evaluated runtime values
    var runtimeFetched = false; // control variable to avoid fetch runtimes over and over again
    var triggerLock = false; // control variable for the flow - makes it easy for async flows later
    var giftBoxConfigData = undefined; // local variable to hold the configuration data loaded from JSON
    var giftBoxConfigLoaded = false; // flag to ensure having bubble pop config is loaded in locally

    var CHECK_INTERVAL = 10;
    var MIN_DELAY_TO_TRIGGER = 5;
    var MAX_DELAY_TO_TRIGGER = 15;
    var MAX_DAYS_LIMIT = 3000; // Rather than using Number.MAX_VALUE, lets use an improbably high number

    var giftBoxIndex = -1;
    var giftBoxTypeName = "";
    var trackingForPhylum = "";
    var trackingMap = {};

    // GIFTBOX TYPES
    var NUM_GIFTBOX_TYPES = 3;
    var GIFTBOX_XPROMO = 2;
    var GIFTBOX_VIDEO = 1;
    var GIFTBOX_LETTER_GRANT = 0;


    // Config JSON Strings
    var CONFIG_ACTIONS = "actions";
    var CONFIG_ACTIONS_TYPE = "type";

    var CONFIG_TRIGGERS = "triggers";
    var CONFIG_ELIGIBILITY = "eligibility";
    var CONFIG_SURFACING = "surfacing";
    var CONFIG_REWARDS = "rewards";

    var CONFIG_GLOBAL_SURFACING = "globalSurfacing";

    var CONFIG_NAME = "name";
    var CONFIG_PROGRESS_FETCH = "getProgress";
    var CONFIG_RUNTIMECHECK = "runtimeCheck";
    var CONFIG_TARGETVALUE = "targetValue";
    var CONFIG_OPERATOR = "operator";
    var CONFIG_PASS_TYPE = "passType";

    var CONFIG_OPERATOR_EQUAL = "equals";
    var CONFIG_OPERATOR_LESS_THAN = "lessThan";
    var CONFIG_OPERATOR_LESS_THAN_EQUAL = "lessThanEqual";
    var CONFIG_OPERATOR_MORE_THAN = "moreThan";
    var CONFIG_OPERATOR_MORE_THAN_EQUAL = "moreThanEqual";
    var BUBBLE_POP_CONTENT_DOWNLOAD_LINK = constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/bubblePop/creatives/";
    var giftBoxIndexMap = {
        "letter": GIFTBOX_LETTER_GRANT,
        "video": GIFTBOX_VIDEO,
        "xpromo": GIFTBOX_XPROMO
    };

    this.init = function () {
        cc.loader.loadJson(res.giftBox_config_json, function (error, data) {
            if (error == null && data != null && data.hasOwnProperty(constantsObj.GAME_ID)) {
                giftBoxConfigData = data[constantsObj.GAME_ID];
                giftBoxConfigLoaded = true;
                giftBoxObj.sortConfigByPriority();
            }
        });
        pspnGiftBox = {
            lastCheck: 0,
            xpromoClicks: {},
            xpromoCooldowns: {},
        };
    };

    this.load = function(quests){
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }

        if(quests.pspnGiftBox != undefined){
            pspnGiftBox = quests.pspnGiftBox;

        }

        if(pspnGiftBox.lastCheck == undefined) {
            pspnGiftBox.lastCheck = 0;
        }

        if(pspnGiftBox.xpromoClicks == undefined){
            pspnGiftBox.xpromoClicks = {};
        }

        if(pspnGiftBox.xpromoCooldowns == undefined){
            pspnGiftBox.xpromoCooldowns = {};
        }
    };

    this.save = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }
        quests.pspnGiftBox = pspnGiftBox;

        return quests;
    };


    ///////////////////////////////////////////////////////////////////////////////
    //////////////////Starting section for Basic Giftbox methods//////////////////
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Method to check if the giftBox is feature is currently active or in-active for a player
     * @returns {boolean} - True / False
     */
    this.isGiftBoxActive = function() {
        var result = false;
        if(xPromoGameSpecificObj.isGiftBoxEnabled()) {
           return puzzleGiftBoxObj.areBubblePopVariantsSatisfy();
        }
        return result;
    };

    /**
     * Method to fetch runtime value based on the experiment variant. Also caches the value for the rest of the session.
     * @returns {object}  - Returns a JS object with runtime values.
     */
    this.getRuntimes = function (){
        if(runtimeFetched) {
            return rt;
        }

        rt = runtimeObj.getGiftBoxRuntimes();
        runtimeFetched = true;
        return rt;

    };

    /**
     * Method to sort the config by priority
     * This is done to ensure that the while checking, the first bubble(giftBox) to pass the criteria is of the highest possible priority
     */
    this.sortConfigByPriority = function() {
        if(giftBoxConfigLoaded&& utilObj.isArray(giftBoxConfigData[CONFIG_ACTIONS])) {

            // local function for sorting objects by their priority
            var prioritySorter = function(a, b) {
                return a.priority - b.priority;
            };

            // First lets sort the configurations by priority
            giftBoxConfigData[CONFIG_ACTIONS].sort(prioritySorter);

            // Now lets sort each of the reward configs in an action by priority
            for (var index = 0; index < giftBoxConfigData[CONFIG_ACTIONS].length; index++) {
                var action = giftBoxConfigData[CONFIG_ACTIONS][index];
                if(utilObj.isArray(action[CONFIG_REWARDS])) {
                    action[CONFIG_REWARDS].sort(prioritySorter);
                }
            }
        }
    };

    /**
     * Get the array index corresponding to the giftBox type
     * @param actionType - String representing the type of the giftBox
     * @returns {number}
     */
    this.getIndexByGiftBoxType = function(actionType) {
        if(actionType != undefined && giftBoxIndexMap.hasOwnProperty(actionType)) {
            return giftBoxIndexMap[actionType];
        }
        return constantsObj.NOT_FOUND;
    };


    /**
     * Wrapper method to process a condition based on runtimeCheck or targetValue
     * @param config - JS Object for the configuration of the condition
     * @param actionType - String representing the type of action this is ("xpromo", "video",... etc)
     * @returns {boolean} - Returns true / false depending if the condition is pass / fail respectively.
     *                      Returns false of unsupported condition type
     */
    this.processCondition = function(config, actionType) {
        var result = false;

        if(config.hasOwnProperty(CONFIG_RUNTIMECHECK) && config[CONFIG_RUNTIMECHECK] != null
            && giftBoxObj.evaluateCondition(config, actionType, true)) {
            result = true;

        } else if (config.hasOwnProperty(CONFIG_TARGETVALUE) && config[CONFIG_TARGETVALUE] != null
            && giftBoxObj.evaluateCondition(config, actionType, false)){
            result = true;
        }

        return result;
    };

    /**
     * Method to run the condition check string using the eval. Also figures out if the condition requires an index lookup
     * @param config - JS Object for the configuration of the condition
     * @param actionType - String representing the type of action this is ("xpromo", "video",... etc)
     * @param runtimeCheck - true / false depending if it needs a runtime check or targetValue comparision
     * @returns {boolean} Returns true / false depending if the condition is pass / fail respectively.
     */
    this.evaluateCondition = function(config, actionType, runtimeCheck) {
        var result = false;
        if(config.hasOwnProperty(CONFIG_PROGRESS_FETCH) && config[CONFIG_PROGRESS_FETCH] != null) {

            var params = [];
            var queryByBubbleType = false;
            if(config.hasOwnProperty(CONFIG_PASS_TYPE) && config[CONFIG_PASS_TYPE] === true) {
                params.push(actionType);
                queryByBubbleType = true;
            }
            var progress = eval(config[CONFIG_PROGRESS_FETCH]).apply(null, params);
            var runtimeValues = giftBoxObj.getRuntimes();
            var requiredProgress = (runtimeCheck) ? runtimeValues[config[CONFIG_RUNTIMECHECK]] : config[CONFIG_TARGETVALUE];
            if(queryByBubbleType && utilObj.isArray(requiredProgress)) {
                var index = giftBoxObj.getIndexByGiftBoxType(actionType);
                if(index != constantsObj.NOT_FOUND) {
                    requiredProgress = requiredProgress[index];
                } else {
                    requiredProgress = constantsObj.NOT_FOUND;
                }
            }
            result = (giftBoxObj.evaluateOperator(progress, requiredProgress, config[CONFIG_OPERATOR]));
        }
        return result;
    };

    /**
     * Wrapper method to run an evaluation operation depending on the operator, current and required progress
     * @param curProgress - value representing the current value
     * @param reqProgress - value representing the required value
     * @param operator - operator applied between the current and required value
     * @returns {boolean} - Returns the evaluation of "curProgress <operator> reqProgress"
     */
    this.evaluateOperator = function(curProgress, reqProgress, operator) {
        var result = false;
        if(curProgress != undefined && reqProgress != undefined && operator != undefined) {
            switch (operator) {
                case CONFIG_OPERATOR_EQUAL:
                    result = (curProgress == reqProgress);
                    break;
                case CONFIG_OPERATOR_LESS_THAN:
                    result = (curProgress < reqProgress);
                    break;
                case CONFIG_OPERATOR_LESS_THAN_EQUAL:
                    result = (curProgress <= reqProgress);
                    break;
                case CONFIG_OPERATOR_MORE_THAN:
                    result = (curProgress > reqProgress);
                    break;
                case CONFIG_OPERATOR_MORE_THAN_EQUAL:
                    result = (curProgress >= reqProgress);
                    break;
                default:
                    break;
            }
        }
        return result;
    };


    /**
     * Entry point method which initiates the trigger check for Bubble Pop(GiftBox) feature
     * This method needs to be called from the different trigger points in the game.
     */
    this.initiateTriggerCheck = function () {
        if(!triggerLock && giftBoxObj.isGiftBoxActive()) {
            var curTimestamp = utilObj.getCurrentTimestamp();
            if (curTimestamp - pspnGiftBox.lastCheck < CHECK_INTERVAL) {
                return;
            }
            pspnGiftBox.lastCheck = curTimestamp;
            triggerLock = true;

            var giftBoxToShow = false;
            if(giftBoxConfigLoaded && utilObj.isArray(giftBoxConfigData[CONFIG_ACTIONS])) {
                // Lets try and load the runtimes if not yet done
                var runtimeValues = giftBoxObj.getRuntimes();
                // Need to reset Cross Promo cooldowns for games if all are reached
                giftBoxObj.checkResetXpromoCooldowns();
                // Payer Status
                var isPayer = xPromoGameSpecificObj.getIsPayerUser();
                // Lets iterate over all possible actions and see if any of the triggers pass
                for (var index = 0; index < giftBoxConfigData[CONFIG_ACTIONS].length; index++) {
                    // Lets first check if the main switch is on for a bubble type before initiating checks
                    var action = giftBoxConfigData[CONFIG_ACTIONS][index];
                    var switchIndex = giftBoxObj.getIndexByGiftBoxType(action[CONFIG_ACTIONS_TYPE]);
                    if(utilObj.isArray(runtimeValues.giftBoxMainSwitch) && runtimeValues.giftBoxMainSwitch[switchIndex] == 1 &&
                        utilObj.isArray(runtimeValues.giftBoxPayerEnable) && (runtimeValues.giftBoxPayerEnable[switchIndex] == 2 || (runtimeValues.giftBoxPayerEnable[switchIndex] == 1 && isPayer) || (runtimeValues.giftBoxPayerEnable[switchIndex] == 0 && !isPayer))) {
                        var passedTriggers = giftBoxObj.checkTrigger(action);
                        // If no triggers have passed, no point evaluating further.
                        if (passedTriggers.length > 0) {
                            giftBoxObj.initiateBubbleTracking();
                            giftBoxObj.setTrackingDetails(xpromoConstantsObj.TRACK_PHYLUM, passedTriggers[0]); // Pushing the passed trigger
                            if (giftBoxObj.checkEligibility(action) && giftBoxObj.checkSurfacing(action, giftBoxConfigData[CONFIG_GLOBAL_SURFACING])) {
                                // All checks are passed, lets calculate the reward grants and keep it in place.
                                // Lets set the necessary flags and add a delayed call with a random trigger duration for showing the bubble

                                giftBoxToShow = true;
                                giftBoxTypeName = action[CONFIG_ACTIONS_TYPE];
                                giftBoxIndex = giftBoxObj.getIndexByGiftBoxType(action[CONFIG_ACTIONS_TYPE]);
                                giftBoxObj.setTrackingForPhylum(passedTriggers[0]);
                                animObj.bubblePopGiftAnim(runtimeValues.giftBoxDisplayTime, giftBoxTypeName, giftBoxIndex);
                                triggerLock = false;
                                giftBoxObj.applyTracking();
                                giftBoxObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, action[CONFIG_ACTIONS_TYPE]);
                                break;
                            }
                            giftBoxObj.applyTracking();
                        }
                    }
                }
            }
            if(!giftBoxToShow) {
                triggerLock = false;
            }
        }
    };


    /**
     * Method that checks if any of the triggers have passed
     * @param action - JS Object for the configuration of the action
     * @returns {Array} - List of strings representing the passed triggers
     */

    this.checkTrigger = function(action) {
        var result = [];
        if(action != undefined && utilObj.isArray(action[CONFIG_TRIGGERS])) {
            for (var index = 0; index < action[CONFIG_TRIGGERS].length; index++) {
                var trigger = action[CONFIG_TRIGGERS][index];

                if(giftBoxObj.processCondition(trigger, action[CONFIG_ACTIONS_TYPE])) {
                    result.push(trigger[CONFIG_NAME]);
                }
            }
        }
        return result;

    };

    /**
     * Method that checks all the eligibility criteria for a bubble(giftBox) type
     * @param action - JS Object for the configuration of the action
     * @returns {boolean} - True if all eligibilities have passed, false otherwise
     */
    this.checkEligibility = function(action) {
        var result = true;
        giftBoxObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, "success");

        if(action != undefined && utilObj.isArray(action[CONFIG_ELIGIBILITY])) {
            for (var index = 0; (index < action[CONFIG_ELIGIBILITY].length && result); index++) {
                var eligibility = action[CONFIG_ELIGIBILITY][index];

                if(giftBoxObj.processCondition(eligibility, action[CONFIG_ACTIONS_TYPE])) {
                    result = result && true;
                } else {
                    result = false;
                    giftBoxObj.setTrackingDetails(xpromoConstantsObj.TRACK_CLASS, eligibility[CONFIG_NAME]);
                }
            }
        } else {
            result = false;
        }
        return result;

    };

    /**
     * Method that checks all the surfacing criteria for a bubble(giftBox) type
     * @param action - JS Object for the configuration of the action
     * @param globalSurfacing - JS Object for the configuration of global surfacing
     * @returns {boolean} - True if all surfacings have passed, false otherwise
     */
    this.checkSurfacing = function(action, globalSurfacing) {
        var result = true;
        giftBoxObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, "success");

        if(action != undefined && action.hasOwnProperty(CONFIG_SURFACING)) {
            var surfacings;
            if(action[CONFIG_SURFACING].hasOwnProperty("useGlobal") && action[CONFIG_SURFACING]["useGlobal"] === true
                && utilObj.isArray(globalSurfacing)) {
                surfacings = globalSurfacing;
            } else if (action[CONFIG_SURFACING].hasOwnProperty("rules") && utilObj.isArray(action[CONFIG_SURFACING]["rules"])) {
                surfacings = action[CONFIG_SURFACING]["rules"];
            } else {
                result = false;
                giftBoxObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, "json-data-error");
            }

            if(result) {
                for (var index = 0; (index < surfacings.length && result); index++) {
                    var surf = surfacings[index];

                    if(giftBoxObj.processCondition(surf, action[CONFIG_ACTIONS_TYPE])) {
                        result = result && true;
                    } else {
                        result = false;
                        giftBoxObj.setTrackingDetails(xpromoConstantsObj.TRACK_FAMILY, surf[CONFIG_NAME]);
                    }
                }
            }
        }
        return result;
    };

    /////////////////////////////////////////////////
    /////////Xpromo Surfacing Conditions////////////
    ///////////////////////////////////////////////

    /**
     * Helper method to check if any game is eligible for the xpromo
     * IMPORTANT NOTE: This method also initiates the download of creatives,
     * hence might return false for the first ever time since the creatives wont be downloaded
     *
     * @returns {boolean} - True if a game is found, false other wise
     */
    this.anyEligibleGames = function () {
        var result = false;
        var adwallQuests = questsObj.getAdWallQuests();
        var runtimeValues = giftBoxObj.getRuntimes();
        var questPrioritiesArr = Object.keys(adwallQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });
        if (questPrioritiesArr.length > 0) {
            for (var i = 0; (i < questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
                var game = questPrioritiesArr[i];
                var adwallQuest = adwallQuests[game];
                var gameId = adwallQuest.gh,
                    questId = adwallQuest.id;
                if (!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId)) {
                    continue;
                }
                if(giftBoxObj.isViewOrClickCooldownReached(gameId)) {
                    continue;
                }
                if(giftBoxObj.isBubblePopCreativeAvailable(questId, true)) {
                    result = result || true;
                }

            }
        }

        return result;
    };

    /**
     * Method to check if the creative / giftBox pop image for a xpromo-able game is available or not.
     * @param questId - quest ID for the adwall quest corresponding to the xpromo game
     * @param downloadOnFail - Boolean to initiate download if the requested file does not exist
     * @returns {boolean} - Whether the creative is downloaded and available.
     */
    this.isBubblePopCreativeAvailable = function (questId, downloadOnFail) {
        var result = false;
        var curQuests = questsObj.getCurQuests();

        if(curQuests[questId] != undefined) {
            var questData = curQuests[questId];
            if(questData.creatives) {
                var questCreativeMap = xpromoUtilObj.getKeyValuePairForCreatives(questData.creatives);
                if(questCreativeMap.bubblePop_creative != undefined) {
                    var creativeName = questCreativeMap.bubblePop_creative;
                    result = utilObj.fileExists(creativeName);

                    if(!result && downloadOnFail == true) {
                        var contentLink = BUBBLE_POP_CONTENT_DOWNLOAD_LINK + questCreativeMap.bubblePop_creative;
                        xPromoGameSpecificObj.downloadFileV1(contentLink, creativeName);
                    }
                }
            }
        }

        return result;
    };


    /**
     * Method to check whether a particular game has reached its surfacing cooldown in the xpromo
     * @param gameId - game ID for the game
     * @returns {boolean} - Whether the cooldown is reached or not
     */
    this.isViewOrClickCooldownReached = function(gameId) {
        var result = false;
        var runtimeValues = giftBoxObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;
        if (pspnGiftBox.xpromoCooldowns[gameId] != undefined && (pspnGiftBox.xpromoCooldowns[gameId].viewCount >= xpromoViewCoolDown || pspnGiftBox.xpromoCooldowns[gameId].clickCount >= xpromoClickCoolDown)) {
            result = true;
        }

        return result;
    };

    /**
     * Method to check if the puzzle is being replayed by the player.
     * @returns {boolean} - Whether the puzzle is being replayed or not
     */
    this.isReplayingPuzzle = function() {
        var result = false;
        if(gameLayer != undefined && gameLayer.crosswordDate != null) {
            result = (scoresObj.getHighScore(gameLayer.crosswordDate) > 0);
        }
        return result;
    };

    ////////////////////////////////////////////////
    /////////Xpromo Interaction Methods////////////
    //////////////////////////////////////////////

    /**
    * Method to start the xpromo quest and save the details. Used again when the xpromo-ed game is installed and the player needs to be rewarded
    * @param gameId - game ID for the game
    * @param questId - adwall quest id from the game
    * @param bubbleName - Name of the bubble to be marked
    */
    this.markXpromoInteraction = function (gameId, questId, giftBoxName) {
        if(!giftBoxObj.isGiftBoxActive()) {
            return;
        }

        if(pspnGiftBox == null ||  pspnGiftBox.xpromoCooldowns[gameId] == undefined) {
            return;
        }

        questsObj.makeRelatedQuestsStatusStarted(questId);
        pspnGiftBox.xpromoClicks[gameId] = {
            "ts": utilObj.getCurrentTimestamp(),
            "rewards": {"coins": giftBoxObj.getXpromoCoinReward()},
            "questId": questId,
            "bubbleName": giftBoxName
        };
        pspnGiftBox.xpromoCooldowns[gameId].clickCount++;
    };

    /**
     * Getter method to retrieve the xpromo click time for a given game id
     * @param gameId
     * @returns number value of timestamp last interacted, zero if not applicable.
     */
    this.getXpromoInteractionTime = function (gameId) {
        var result = 1;
        if(!giftBoxObj.isGiftBoxActive()) {
            return result;
        }

        if(pspnGiftBox == null || pspnGiftBox.xpromoClicks == undefined ||
            pspnGiftBox.xpromoClicks[gameId] == undefined || pspnGiftBox.xpromoClicks[gameId]["ts"] == undefined) {
            return result;
        }

        return pspnGiftBox.xpromoClicks[gameId]["ts"];
    };

    /**
     * Wrapper method to surface the xpromo quest and UI given the required set of conditions pass
     * @returns {boolean} - Whether the xpromo popup was shown or not.
     */
    this.showXpromoQuest = function () {
        if(curLayer != gameLayer || gameObj.isCrosswordComplete()){
            //if curlayer not game layer or if the puzzle is already completed
            return false;
        }

        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = giftBoxObj.getRuntimes();
        var questPrioritiesArr = Object.keys(xpromoQuests);
        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });

        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];

            if(!xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) || !giftBoxObj.isBubblePopCreativeAvailable(questId, false) || giftBoxObj.isViewOrClickCooldownReached(gameId)) {
                continue;
            }

            if (pspnGiftBox.xpromoCooldowns[gameId] == undefined) {
                pspnGiftBox.xpromoCooldowns[gameId] = {
                    "viewCount": 0,
                    "clickCount": 0,
                };
            }

            pspnGiftBox.xpromoCooldowns[gameId].viewCount++;

            giftBoxUiObj.show(true, {gameId: gameId});
            return true;
        }
        return false;
    };

    /**
     * Method to check if all xpromo-able games have reach their cooldown and reset them if required.
     */

    this.getXpromoCoinReward = function(){
        var runtimes = giftBoxObj.getRuntimes();
        var isPayer = xPromoGameSpecificObj.getIsPayerUser();
        if(isPayer){
            return runtimes.xpromoPayerReward;
        }
        var walletBalace =xPromoGameSpecificObj.getCash();
        if(walletBalace >= runtimes.xpromoWB){
            return runtimes.xpromoCoinReward;
        }
        return runtimes.xpromoCoinRewardHigh;

    };

    this.checkResetXpromoCooldowns = function () {
        var xpromoQuests = questsObj.getAdWallQuests();
        var runtimeValues = giftBoxObj.getRuntimes();
        var xpromoViewCoolDown = runtimeValues.xpromoViewCooldown;
        var xpromoClickCoolDown = runtimeValues.xpromoClickCooldown;
        var questPrioritiesArr = Object.keys(xpromoQuests);

        questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
            return parseInt(a) - parseInt(b);
        });
        var reset = true;
        for (var i = 0; (i<questPrioritiesArr.length && i < runtimeValues.xpromoGamesLimit && reset); i++) {
            var game = questPrioritiesArr[i];
            var gameId = xpromoQuests[game]["gh"], questId = xpromoQuests[game]["id"];

            if(xpromoUtilObj.isGameEligibleForXpromo(gameId, questId) && bubblePopObj.isBubblePopCreativeAvailable(questId, false)) {
                if(pspnGiftBox.xpromoCooldowns[gameId] != undefined) {
                    if(pspnGiftBox.xpromoCooldowns[gameId].viewCount < xpromoViewCoolDown && pspnGiftBox.xpromoCooldowns[gameId].clickCount < xpromoClickCoolDown) {
                        reset = false;
                    }
                } else {
                    reset = false;
                }
            }
        }
        if(reset) {
            pspnGiftBox.xpromoCooldowns = {};
        }
    };

    /**
     * Method to check the completion of a xpromo initiated from the Bubble Pop flow and reward the player about the same.
     */
    this.checkAndGrantXpromoReward = function () {
        if(!giftBoxObj.isGiftBoxActive()) { // If Bubble Pop V1 is in-active, check if V2 is active
            return;
        }

        if(pspnGiftBox == null || pspnGiftBox.xpromoClicks == undefined) {
            return;
        }

        var attributionPeriod = xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_SURFACING_ATTRIBUTION_WINDOW) * 60 * 60;
        var curTime = utilObj.getCurrentTimestamp();
        utilObj.resetIsAppInstalledMap();
        for(var gameId in pspnGiftBox.xpromoClicks) {
            var delEntry = false;
            var clickData = pspnGiftBox.xpromoClicks[gameId];
            var clickTime = clickData["ts"];
            var questId = clickData["questId"];
            var bubbleName = clickData["bubbleName"];
            if(clickTime > 0 && ((clickTime + attributionPeriod) >= curTime)
                && xpromoUtilObj.isAppInstalledByGameId(gameId)) {
                // This is to account for the scenario when W2E for the same game is seen after BP,
                // In this scenario, only the W2E reward should be granted and not BP. Hence the check for gameId below
                if(xpromoW2eObj.getXpromoGameId() != gameId && questsObj.getCrossGameQuestStatusForId(questId) != xpromoConstantsObj.QUEST_CLAIMED && adWallObj.getXpromoInteractionTime(gameId) < clickTime) {
                    // Lets get the current and last screen layers
                    var layerForAnimationOnLastScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(lastScreen);
                    var layerForAnimationOnCurScreen = xPromoGameSpecificObj.getLayerForCoinGrantAnimation(curScreen);
                    var paramsMap = {"rewards": clickData["rewards"], "source": "giftBox-xpromo", "bubbleName": "xpromo", "phylum": questId};
                    // If the popup is open and we are on the xpromo screen, lets close and do a delayed reward grant.
                    if (popupLayer.popupOpen && (xPromoGameSpecificObj.isScreenForW2eXpromo(lastScreen))) {
                        popupLayer.closeWithDefault();
                        utilObj.delayedCallback(layerForAnimationOnLastScreen, giftBoxObj.grantSpecialReward, 2, paramsMap, false);
                    } else { // Else grant the reward immediately
                        giftBoxObj.grantSpecialReward(layerForAnimationOnCurScreen, paramsMap);
                    }
                    questsObj.makeRelatedQuestsForW2eClaimed(questId);
                }
                delEntry = true;
            }

            if ((clickTime + attributionPeriod) < curTime) {
                delEntry = true;
            }

            if(delEntry) { // Lets delete the rewards for the gameId as they are either granted to expired
                delete pspnGiftBox.xpromoClicks[gameId];
            }
        }
    };

    /**
     * Method to grant the special reward (Currently being called from customized flows for special bubbles)
     * @param sender - layer where the grant needs to happen
     * @param params - JS object containing the "reward" to be granted and also the "source"
     */
    this.grantSpecialReward = function(sender, params) {
        if (sender == undefined) {
            sender = gameLayer;
        }
        var reward = params.rewards, source = params.source, showMessage = params.showMessage;
        if(reward != undefined) {
            var quantity = reward.coins;
            if((xPromoGameSpecificObj.getCash() + quantity) <= constantsObj.MAX_CASH_ALLOWED) {
                if(showMessage == undefined || showMessage == true) {
                    xPromoGameSpecificObj.showCoinGrantMessage(quantity);
                }
                xPromoGameSpecificObj.addCash(quantity, constantsObj.TRACK_EARN, (source != undefined) ? source : "", "");  
                trackObj.trackCounter(constantsObj.TRACK_DIALOG, gameObj.getCurPuzzleTracking(), gameObj.getTrackingPuzzleTheme(),
                    "giftBox_xpromo_grant",constantsObj.TRACK_VIEW, reward.coins, gameObj.getPuzzleSpeciesTracking(), "", "");
                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, params.phylum, "xpromo",
                    xpromoConstantsObj.TRACK_GIFT_BOX,"final_grant",xPromoGameSpecificObj.getCash(), reward.coins, "", "");

            } else {
                utilObj.showMessage(stringsObj.coin_limit_reached);
            }
        }
    };

      ///////////////////////////////////////////////
     ////////Tracking Methods///////////////////////
    ///////////////////////////////////////////////
    /**
     * Utility method to clear the tracking map
     */
    this.initiateBubbleTracking = function() {
        trackingMap = {};
    };

    /**
     * Utility method to store a particular field - value pair in the tracking map
     * @param field - string for the tracking field
     * @param value - string value that needs to be stored
     */
    this.setTrackingDetails = function(field, value) {
        trackingMap[field] = value;
    };

    /**
     * Method to run the track call with the given set of parameters set.
     * This design was added to handle flows where different parameters were set in different methods of the flow and
     * passing the data between them was cumbersome.
     */
    this.applyTracking = function() {
        var keys = Object.keys(trackingMap);
        if(keys.length > 0) {
            var phylum = (trackingMap[xpromoConstantsObj.TRACK_PHYLUM] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_PHYLUM] : "";
            var cls = (trackingMap[xpromoConstantsObj.TRACK_CLASS] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_CLASS] : "";
            var family = (trackingMap[xpromoConstantsObj.TRACK_FAMILY] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_FAMILY] : "";
            var genus = (trackingMap[xpromoConstantsObj.TRACK_GENUS] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_GENUS] : "";
            var species = (trackingMap[xpromoConstantsObj.TRACK_SPECIES] != undefined) ? trackingMap[xpromoConstantsObj.TRACK_SPECIES] : "";

            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, phylum, cls,
                xpromoConstantsObj.TRACK_GIFT_BOX, family, genus, species, "", "");
        }
    };

    this.setTrackingForPhylum = function(trackingDetails){
        trackingForPhylum = trackingDetails;
    };

    this.getTrackingForPhylum = function(){
        return trackingForPhylum;
    };

    this.getUserBucketForXpromo = function(){
        var runtimes = giftBoxObj.getRuntimes();
        var isPayer = xPromoGameSpecificObj.getIsPayerUser();
        if(isPayer){
            return 2;
        }
        var walletBalace =xPromoGameSpecificObj.getCash();
        if(walletBalace >= runtimes.xpromoWB){
            return 1;
        }
        return 0;
    };



};
