PspnActionHistory = function () {

    var actionHistory = null;

    var HISTORY_SIZE = 31;

    this.init = function () {
        actionHistory = {
            dauDays: [1],
            puzzleClears: [0],
            w2eVideos: [0],
            lastUpdateTime: utilObj.getCurrentTimestamp(),
        }
    };

    this.load = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }

        // This loop ensures that the data picked from storage does not override any newly added fields
        if (quests.pspnActionHistory != undefined) {
            var keys = Object.keys(quests.pspnActionHistory);
            for(var i = 0; i < keys.length; i++) {
                var key = keys[i];
                if(actionHistory.hasOwnProperty(key)) {
                    actionHistory[key] = quests.pspnActionHistory[key]
                }
            }
        }
    };

    this.save = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }

        if(actionHistory.dauDays != null && actionHistory.dauDays.length > 0) {
            actionHistory.dauDays[actionHistory.dauDays.length - 1] = 1;
        }

        if(actionHistory.puzzleClears != null && actionHistory.puzzleClears.length > 0) {
            actionHistory.puzzleClears[actionHistory.puzzleClears.length - 1] = xPromoGameSpecificObj.getDailyPuzzleClears();
        }

        if(actionHistory.w2eVideos != null && actionHistory.w2eVideos.length > 0) {
            actionHistory.w2eVideos[actionHistory.w2eVideos.length - 1] = xPromoGameSpecificObj.getDailyWatchVideosCount();
        }

        quests.pspnActionHistory = actionHistory;
        return quests;
    };

    this.updatePspnActionHistory = function() {
        if(actionHistory == null) {
            return;
        }

        var curTime = utilObj.getCurrentTimestamp();
        var daysPassed = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(curTime, actionHistory.lastUpdateTime));

        for(var index = 0; index < daysPassed; index++) {
            actionHistory.dauDays.push(0);
            actionHistory.puzzleClears.push(0);
            actionHistory.w2eVideos.push(0);
        }

        var keys = Object.keys(actionHistory);
        for(var i = 0; i < keys.length; i++) {
            var key = keys[i];
            if(actionHistory.hasOwnProperty(key) && utilObj.isArray(actionHistory[key]) && actionHistory[key].length > HISTORY_SIZE) {
                var excess = actionHistory[key].length - HISTORY_SIZE;
                actionHistory[key].splice(0, excess);
            }
        }

        actionHistory.lastUpdateTime = utilObj.getCurrentTimestamp();
    };

    var getMedian = function(field, historyPeriod) {
        var result = constantsObj.NOT_FOUND;

        if(utilObj.isArray(field) && field.length > historyPeriod) {
            var sortedData = field.slice(field.length - historyPeriod - 1, field.length - 1);
            sortedData.sort(function(a, b) { // Sorting by the priority
                return parseInt(a) - parseInt(b);
            });

            var midIndex = parseInt(historyPeriod / 2);
            var median = parseInt(sortedData[midIndex]);

            if(historyPeriod % 2 == 0) { // If this is an odd sized array
                median = Math.ceil((median + sortedData[midIndex - 1]) / 2);
            }
            result = median;
        }

        return result;

    };

    var getDauDaysValues = function(field, dauMap) {
        var dauDays = dauMap.slice();
        var fieldCopy = field.slice();

        if(dauDays.length > fieldCopy.length) {
            var pad = Array(dauDays.length - fieldCopy.length).fill(0);
            fieldCopy = pad.concat(fieldCopy);
        } else if (dauDays.length < fieldCopy.length) {
            var pad = Array(fieldCopy.length - dauDays.length).fill(0);
            dauDays = pad.concat(dauDays);
        }

        var fieldDauValue = [];
        for(var i = dauDays.length - 1; i >= 0; i--) {
            if(dauDays[i] == 1) {
                fieldDauValue.unshift(fieldCopy[i]);
            }
        }

        return fieldDauValue;
    };

    this.getPuzzleClearsMedian = function(historyPeriod) {
        if(actionHistory == null) {
            return constantsObj.NOT_FOUND;
        }

        return getMedian(actionHistory.puzzleClears, historyPeriod);
    };

    this.getPuzzleClearsMedianByDau = function(historyPeriod) {
        if(actionHistory == null) {
            return constantsObj.NOT_FOUND;
        }

        if(utilObj.isArray(actionHistory.dauDays) && actionHistory.dauDays.length > historyPeriod &&
            utilObj.isArray(actionHistory.puzzleClears) && actionHistory.puzzleClears.length > historyPeriod) {
            return getMedian(getDauDaysValues(actionHistory.puzzleClears, actionHistory.dauDays), historyPeriod);
        }
        return constantsObj.NOT_FOUND;
    };

    this.getW2eVideosMedian = function(historyPeriod) {
        if(actionHistory == null) {
            return constantsObj.NOT_FOUND;
        }

        return getMedian(actionHistory.w2eVideos, historyPeriod);
    };

    this.getDauDaysInCalendarDays = function(historyPeriod) {
        var result = constantsObj.NOT_FOUND;

        if(actionHistory == null) {
            return result;
        }

        if(utilObj.isArray(actionHistory.dauDays) && actionHistory.dauDays.length > historyPeriod) {
            var historyInFocus = actionHistory.dauDays.slice(actionHistory.dauDays.length - historyPeriod - 1, actionHistory.dauDays.length - 1);
            var dauDays = 0;
            for(var index = 0; index < historyInFocus.length; index++) {
                if(historyInFocus[index] == 1) {
                    dauDays++;
                }
            }
            result = dauDays;
        }

        return result;
    }

};

