PspnFlutter = function () {

    var isInitialized = false;
    var flutterFileBackedUp = false;
    var questCompletedConfig = {};
    var screenBeforeFlutter = -1;
    var screenRedirectionFromFlutter = -1;
    var flutterExperimentsToSync = [
        "PSPNquestRotation",
        "wordwarsRequests",
        "requestsRedirection",
        "pspnXpromoW2e",
        "pspnXpromoW2eNew",
        "pspnXpromoW2eOpt",
        "PSPNcgdq",
        "PSPNcgdqOpt",
        "pspnXpromoRedirect",
        "adwall",
        "adwall_new",
        "redirectToAppStore",
        "shareLinks",
        "questInvite",
        "PSPNpseudoquestInvite",
        "dailyQuestV2",
        "PSPNbubblePop",
        "AdwallNewUI",
        "PSPNExitXpromo",
        "PSPNAdjTrackerFix",
        "PSPNQuestMinipopClick",
        "pspnAllQuestsClickable",
    ];
    var flutterExperiment = -1;
    var PSPN_FLUTTER_ENABLE_EXP = "PSPNFlutter";
    var PSPN_QC_AB_EXP = "qc_ab";
    var FRIENDS_DATA_FILE = "friends.json";

    var adwallQuests = null;
    var adwallQuestsMap = null;
    var runtimeData = null;
    var w2eFeatureData = null;
    var w2eFeatureDataLoaded = false;
    var bubblePopData = null;
    var bubblePopDataLoaded = false;
    var eligibleGames = null;

    this.init = function() {
        // Always need to ensure that experiment manager init is called before reaching here.
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.updateFlutterExperiments();
        }
    };

    ////////////////////////////////////////////
    /////////// File Related Methods ///////////
    ////////////////////////////////////////////

    /// Migrating Data from Cocos -> Flutter ///

    function migrationToFlutterComplete(status){
        // loggerObj.debugLog("Data Migration Logs - Migrate from Cocos to Flutter migrationToFlutterComplete status=" + status);
        if(status == true) {
            flutterFileBackedUp = true;
        }
    }

    function migrateToFlutter(status) {
        // loggerObj.debugLog("Data Migration Logs - Migrate from Cocos to Flutter migrateToFlutter status=" + status);
        if(status == true) {
            migrationToFlutterComplete(true);
        } else {
            var path = jsb.fileUtils.getWritablePath();
            path += "flutter-" + constantsObj.QUESTS_DATA;

            var fileStatus = true;
            var data = ls.getItem(constantsObj.QUESTS_DATA);
            if(data != null && data != "") {
                fileStatus = jsb.fileUtils.writeStringToFile(data, path);
            }
            migrationToFlutterComplete(fileStatus);
        }
    }

    this.flutterFileSetup = function() {
        // loggerObj.debugLog("Data Migration Logs - Trying migrate from Cocos to Flutter");
        pspnFlutterObj.checkIfFlutterFileExists(migrateToFlutter);
    };


    //// Common Method for files migration ////

    this.checkIfFlutterFileExists = function(callback) {
        return false;
        var path = jsb.fileUtils.getWritablePath();
        path += "flutter-" + constantsObj.QUESTS_DATA;
        var status = jsb.fileUtils.isFileExist(path);
        callback(status);
    };


    /// Migrating Data from Flutter -> Cocos ///

    function migrationFromFlutterComplete(status) {
        // loggerObj.debugLog("Data Migration Logs - Migrate from Flutter to Cocos migrationFromFlutterComplete status=" + status);
        questsObj.initContinued();
    }

    function migrateFields(jsonData, field) {
        // loggerObj.debugLog("Data Migration Logs - Migrate from Flutter to Cocos migrateFields field=" + field);
        if(jsonData != null && jsonData[field] != undefined) {
            var keys = Object.keys(jsonData[field]);
            for(var i = 0; i < keys.length; i++) {
                var key = keys[i];
                jsonData[key] = jsonData[field][key];
                // loggerObj.debugLog("Data Migration Logs - Migrate from Flutter to Cocos migrateFields key=" + key);
            }
        }
    }

    function flutterFileRemoved(status) {
        // loggerObj.debugLog("Data Migration Logs - Migrate from Flutter to Cocos flutterFileRemoved status=" + status);
    }

    function migrateFromFlutter(status) {
        // loggerObj.debugLog("Data Migration Logs - Migrate from Flutter to Cocos migrateFromFlutter status=" + status);
        if(status == false) {
            migrationFromFlutterComplete(true);
        } else {
            var path = jsb.fileUtils.getWritablePath();
            path += "flutter-" + constantsObj.QUESTS_DATA;

            var status = true;
            var data = jsb.fileUtils.getStringFromFile(path);
            if(data != null && data != "") {
                try {
                    var jsonData = JSON.parse(data);
                    if(jsonData != null) {
                        // Certain feature specific data field are moved to their individual classes in Flutter
                        // Hence while migrating the data, need to ensure that the invidual classes are flattened out
                        // like it is expected in Cocos Quest Center
                        migrateFields(jsonData, "questRuntimesData");
                        migrateFields(jsonData, "dailyQuestsData");
                        migrateFields(jsonData, "socialQuestsData");
                        migrateFields(jsonData, "questRotationData");
                        migrateFields(jsonData, "commonPromotionData");
                        migrateFields(jsonData, "crossGameDailyQuestsData");
                        migrateFields(jsonData, "w2eFeatureData");
                        ls.setItem(constantsObj.QUESTS_DATA, utilObj.getStringifiedJson(jsonData));
                    }
                } catch (e) {
                    status = false;
                    // loggerObj.debugLog("Data Migration Logs - error in migration e=" + e.toString());
                }
            }
            jsb.fileUtils.removeFile(path, flutterFileRemoved);
            ls.setItem("notif_" + constantsObj.QUESTS_DATA, "");
            migrationFromFlutterComplete(status);
        }
    }

    this.cocosFileSetup = function() {
        // loggerObj.debugLog("Data Migration Logs - Trying migrate from Flutter to Cocos");
        pspnFlutterObj.checkIfFlutterFileExists(migrateFromFlutter);
    };


    function friendDataSaveCallback(status) {
        // loggerObj.debugLog("PSPN Flutter: friendData save callback status=" + status);
    }

    this.saveFriendData = function(data) {
        if(data != null && data != "") {
            var path = jsb.fileUtils.getWritablePath();
            path += "flutter-" + FRIENDS_DATA_FILE;
            jsb.fileUtils.writeStringToFile(data, path, friendDataSaveCallback);
        } else {
            // loggerObj.debugLog("PSPN Flutter: saveFriendData called with empty data");
        }
    };

    this.getRuntime = function(runtimeName, runtimePos) {
        if(runtimeData != null && runtimeData[runtimeName] != undefined) {
            if(runtimePos == undefined) {
                return runtimeData[runtimeName];
            } else {
                return runtimeData[runtimeName][runtimePos];
            }
        }
        // loggerObj.debugLog("Abdul Runtime Error fetching runtimeName=" + runtimeName + " runtimePos=" + runtimePos);
        return null;
    };

    this.getAdwallQuests = function() {
        if(adwallQuests != null) {
            return adwallQuests;
        }

        // loggerObj.debugLog("Abdul Adwall Error getting all quests");
        return {};
    };

    this.getAdwallQuestById = function(questId) {
        if(adwallQuestsMap != null && adwallQuestsMap[questId] != undefined) {
            return adwallQuestsMap[questId]
        }

        // loggerObj.debugLog("Abdul Adwall Error getting questId=" + questId);
        return {};
    };

    this.getAdwallQuestMap = function() {
        if(adwallQuestsMap != null) {
            return adwallQuestsMap;
        }

        // loggerObj.debugLog("Abdul Adwall Error getting adwallQuestsMap");
        return {};
    };

    this.getW2eFeatureData = function(field) {
        if(w2eFeatureData != null && w2eFeatureData[field] != undefined) {
            return w2eFeatureData[field];
        }
        // loggerObj.debugLog("Abdul W2E Error getting for field=" + field);
        return null;
    };

    this.setW2eFeatureData = function(field, value) {
        if(w2eFeatureData != null) {
            w2eFeatureData[field] = value;
        } else {
            // loggerObj.debugLog("Abdul W2E Error setting for field=" + field);
        }
    };

    this.isInstallQuestEligibleForGame = function(gameId) {
        return (eligibleGames != null && eligibleGames.indexOf(gameId) >= 0);
    };

    ////////////////////////////////////////////
    ////// Game Side Intermediate Methods //////
    ////////////////////////////////////////////


    this.tryInitializeFlutterQuestCenter = function() {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.initializeFlutterQuestCenter();
        }
    };

    this.initializeFlutterQuestCenter = function() {
        if(isInitialized  || !runtimeObj.isLocalDataAvailable() || !epmObj.isLocalDataAvailable()) { // First time sync hasn't happened yet with the server. Lets wait.
            return;
        }

        if(!flutterFileBackedUp) {
            return;
        }

        isInitialized = true;
        var flutterData = getBaseFlutterParams();
        getInitParams(flutterData);
        getQuestProgressParams(flutterData);
        flutterInitializeAdwall(flutterData);
        sendDataToFlutterModule(flutterData, false, true);

        if (xPromoGameSpecificObj.isInButtonFillVariant()) {
            xPromoGameSpecificObj.showFilledQuestButton(false);
        }
    };

    this.isFlutterEnabled = function() {
        if(flutterExperiment != -1) {
            return flutterExperiment == constantsObj.EXP_VAR_1;
        }

        flutterExperiment = (!xPromoFlutterGameSpecificObj.isFlutterAlwaysEnabled()) ? epmObj.getChosenVariant(PSPN_FLUTTER_ENABLE_EXP) : constantsObj.EXP_VAR_1;
        // loggerObj.debugLog("Data Migration Logs - flutterExperiment=" + flutterExperiment);
        return flutterExperiment == constantsObj.EXP_VAR_1;
    };

    function sendDataToFlutterModule(flutterData, isActivity, isCritical) {
        if(isActivity == undefined) {
            isActivity = false;
        }
        if(isCritical == undefined) {
            isCritical = false;
        }

        if(utilObj.isAndroid()) {
            if(isCritical) {
                jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_FLUTTER + "FlutterBridge", "sendDataToFlutterModule", "(Ljava/lang/String;ZZ)V", JSON.stringify(flutterData), isActivity, true);
            } else {
                jsb.reflection.callStaticMethod(constantsObj.PACKAGE_PATH_FLUTTER + "FlutterBridge", "sendDataToFlutterModule", "(Ljava/lang/String;Z)V", JSON.stringify(flutterData), isActivity);
            }
        } else if (utilObj.isIos()) {
            if(isCritical) {
                jsb.reflection.callStaticMethod("FlutterBridge", "sendDataToFlutterModule:isActivity:isCritical:", JSON.stringify(flutterData), isActivity, true);
            } else {
                jsb.reflection.callStaticMethod("FlutterBridge", "sendDataToFlutterModule:isActivity:", JSON.stringify(flutterData), isActivity);
            }
        }
    };

    this.flutterAppPause = function() {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.shareDataBetweenGames();
        }
        runtimeObj.syncToServer();
        questsObj.save();
        gameObj.save();
        trackObj.save();
    };

    this.flutterAppResume = function() {
        if(pspnFlutterObj.isFlutterEnabled()) {
            pspnFlutterObj.gettingSharedData();
        }
    };

    this.setRedirectionFromFlutter = function(src){
        screenRedirectionFromFlutter = src;
    };

    this.getScreenRedirectionFromFlutter = function () {
        return screenRedirectionFromFlutter;
    };

    this.setFlutterLastScreen = function(src){
        screenBeforeFlutter = src;
    };

    this.getFlutterLastScreen = function(){
        if(screenBeforeFlutter != -1) {
            return screenBeforeFlutter;
        }
        return constantsObj.SCREEN_GAME;
    };


    function getBaseFlutterParams() {
        var flutterData = {};
        flutterData["route"] = "pspn";
        flutterData["controller"] = "quests";
        flutterData["action"] = "initialize";

        // TODO - Evaluate the need of passing this block with every quest related call, also evaluate some of the fields
        flutterData["gameState"] = {
            "userLevel": parseInt(xPromoGameSpecificObj.getLevelForQuests()),
            "consLogins": parseInt(xPromoGameSpecificObj.getConsecutiveLogins()) > 0 ? parseInt(xPromoGameSpecificObj.getConsecutiveLogins()) : 1,
            "appVersion": parseInt(utilObj.getVersionCode()),
            "ratedVersion": parseInt(xPromoGameSpecificObj.getLastRatedVersion()),
            "userCreated": parseInt(userObj.getCreated()),
            "fbConnected": xPromoGameSpecificObj.isLoggedIn(),
            "notifPerms": xPromoGameSpecificObj.getNotifPermissionForIos(),
            "isOnline": utilObj.isOnline(),
            "coins": parseInt(xPromoGameSpecificObj.getCash()),
            "trackingPhylum": xPromoGameSpecificObj.getPhylumForTracking().toString(),
            "trackingClass":  xPromoGameSpecificObj.getClassForTracking().toString(),
            "isPayer": xPromoGameSpecificObj.getIsPayerUser(),
            "fbConnectCoins": xPromoFlutterGameSpecificObj.getFbConnectCash(),
            "inviteCoins": xPromoFlutterGameSpecificObj.getReferralReward(),
            "dollarOverride": xPromoFlutterGameSpecificObj.getDollarValueOverride(),
        };

        return flutterData;
    };

    function getInitParams(flutterData) {
        var expString = "control";
        var expVariant = epmObj.getChosenVariant(PSPN_QC_AB_EXP);
        if(expVariant > 0) {
            expString = "var" + expVariant.toString();
        }

        flutterData["init"] = {
            "gameId": constantsObj.GAME_ID,
            "clientId": utilObj.getTrackingClient(),
            "refId": userObj.getRefId(),
            "adId": xpromoUtilObj.getAdvertiserId(),
            "isLat": utilObj.isLimitAdTracking(),
            "isLatWithoutAdjust": xPromoGameSpecificObj.isLimitTrackingWithoutAdjustId(),
            "country": utilObj.getCountry(),
            "build": parseInt(utilObj.getVersionCode()),
            "maxCoins": constantsObj.MAX_CASH_ALLOWED,
            "baseFile": constantsObj.QUESTS_DATA,
            "questVersion": runtimeObj.getQuestCenterVersion(),
            "questAllocation": expString,
            "callbacks": {
                "exitQuestCenter": {
                    "exit": true,
                    "callback": "pspnFlutterObj.flutterQuestCenterExit",
                    "params": [],
                },
                "addCash": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionAddCash",
                    "params": ["cashReward", "questId"],
                },
                "showQuestAnim": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionShowQuestAnimation",
                    "params": ["questId", "status", "questDescription"],
                },
                "updateNotifs": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionUpdateNotifSurfacing",
                    "params": ["completedQuests", "activeQuests", "claimedQuests"],
                },
                "updateNewTag": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionUpdateNewTag",
                    "params": [],
                },
                "redirectToStore": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionRedirectToStore",
                    "params": ["questParams"],
                },
                "fbConnect": {
                    "exit": true,
                    "callback": "pspnFlutterObj.flutterActionFbLogin",
                    "params": [],
                },
                "inviteFriends": {
                    "exit": true,
                    "callback": "pspnFlutterObj.flutterActionInviteFriends",
                    "params": [],
                },
                "inviteXFriends": {
                    "exit": true,
                    "callback": "pspnFlutterObj.flutterActionInviteFriends",
                    "params": [],
                },
                "rateUs": {
                    "exit": true,
                    "callback": "pspnFlutterObj.flutterActionShowAndAwardRateUs",
                    "params": [],
                },
                "inviteWhatsapp": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionShareOnWhatsapp",
                    "params": [],
                },
                "inviteTwitter": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionShareOnTwtr",
                    "params": [],
                },
                "inviteMessage": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionShareOnSms",
                    "params": [],
                },
                "inviteEmail": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionShareOnMail",
                    "params": [],
                },
                "notifPermission": {
                    "exit": true,
                    "callback": "pspnFlutterObj.flutterActionNotifPermission",
                    "params": [],
                },
                "inviteFbMessenger": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionShareOnMessenger",
                    "params": [],
                },
                "inviteGooglePlus": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionShareOnGooglePlus",
                    "params": [],
                },
                "resetDailyQuests": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterResetDailyQuests",
                    "params": [],
                },
                "showGameWithToaster":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionRedirectToGame",
                    "params":[]
                },
                "purchaseCoins":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionShowCoinStore",
                    "params":[]
                },
                "completeOffer":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionCompleteOffer",
                    "params":[]
                },
                "spinSpinner":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionShowSpinner",
                    "params":[]
                },
                "dailyQuestsSpins":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionShowSpinner",
                    "params":[]
                },
                "dailyQuestsWatchToEarn":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionShowVideo",
                    "params":[]
                },
                "watchVideos":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionShowVideo",
                    "params":[]
                },
                "watchVideosInTime":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionShowVideo",
                    "params":[]
                },
                "playDailyPuzzle":{
                    "exit":true,
                    "callback":"pspnFlutterObj.flutterActionShowDailyPuzzle",
                    "params":[]
                }
            },
        };
    }

    function getQuestProgressParams(flutterData) {
        flutterData["gameProgress"] = {
            "dailyBonusClaims": xPromoGameSpecificObj.getDailyBonusClaims(),
            "dailyVideoWatches": xPromoGameSpecificObj.getDailyWatchVideosCount(),
            "dailyPuzzleClears": xPromoGameSpecificObj.getDailyPuzzleClears(),
            "inviteXFriends": xPromoFlutterGameSpecificObj.getFriendsInvited(),
            "levelConsecutiveDays": xPromoFlutterGameSpecificObj.getPuzzleSolvedStreak(),
            "videosWatched": xPromoFlutterGameSpecificObj.getRewardedVideosWatched()
        };

        flutterData["questUnlockState"] = {
            "spinSpinner": xPromoFlutterGameSpecificObj.isSpinnerAvailable(),
            "useHintTarget": xPromoFlutterGameSpecificObj.isPickHintAvailable(),
            "useHintBurst": xPromoFlutterGameSpecificObj.isBurstHintAvailable(),
            "playDailyPuzzle": xPromoFlutterGameSpecificObj.isDailyPuzzleUnlocked(),
            "purchasePremiumAvatar": xPromoGameSpecificObj.isProfileEnabled(),
            "clickDictionaryAudio": xPromoGameSpecificObj.isAudioQuestEnabled(),
            "openSubscription": xPromoGameSpecificObj.shouldShowSubscriptionQuest(),
            "watchVideosInTime": xPromoFlutterGameSpecificObj.shouldShowVideoInTimeQuest(),
            "completeTutorial": xPromoFlutterGameSpecificObj.shouldShowTutorialQuest(),
            "inviteXFriends": xPromoFlutterGameSpecificObj.canInviteFriends(),
            "inviteFriends": xPromoFlutterGameSpecificObj.canInviteFriends(),
        }
    }

    function flutterInitializeAdwall(flutterData) {
        flutterData["initAdwall"] = {
            "gameName": xPromoGameSpecificObj.getGameName(),
            "gameId": constantsObj.GAME_ID,
            "clientId": utilObj.getTrackingClient(),
            "refId": userObj.getRefId(),
            "adId": xpromoUtilObj.getAdvertiserId(),
            "adwallBucket": constantsObj.QUEST_AD_WALL,
            "isTablet": utilObj.isTablet(),
            "isOnline": utilObj.isOnline(),
            "isPayer": xPromoGameSpecificObj.getIsPayerUser(),
            "installTime": xPromoGameSpecificObj.getInstallTimeStamp(),
            "enableAutoTrigger": true,
            "callbacks": {
                "redirectToStore": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterActionRedirectToStore",
                    "params": ["questParams"],
                },
                "updateCrossGameQuestStatus": {
                    "exit": false,
                    "callback": "pspnFlutterObj.flutterUpdateCrossGameQuestStatus",
                    "params": ["questId", "status"],
                },
            }
        };

        flutterData["adwallState"] = {
            "firstShown": adWallObj.getAdwallFirstShown(),
            "shownCount": adWallObj.getAdwallShownCount(),
            "lastShown": adWallObj.getLastShownAdWallTimeStamp(),
            "sessionCount": adWallObj.getAdwallSessionCountInDay(),
            "dayStartTs": utilObj.getCurrentTimestamp(),
        }
    };

    this.save = function() {
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "saveProgress";

        if(w2eFeatureData != null && pspnFlutterObj.isW2eFeatureDataLoaded()) {
            xpromoW2eObj.save(w2eFeatureData);
            flutterData["w2eFeatureData"] = w2eFeatureData;
        }

        if(bubblePopData != null && pspnFlutterObj.isBubblePopDataLoaded()) {
            var quests = {};
            quests.bubblePop = bubblePopData;
            bubblePopObj.save(quests);
            flutterData["bubblePopData"] = quests.bubblePop;
        }

        sendDataToFlutterModule(flutterData, false, false);
    };

    this.isBubblePopDataLoaded = function() {
        return bubblePopDataLoaded;
    };

    this.isW2eFeatureDataLoaded = function() {
        return w2eFeatureDataLoaded;
    };

    this.updateAllQuests = function(questType) {
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "updateAllQuests";
        getQuestProgressParams(flutterData);
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.updateQuestStatusCounts = function() {
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "updateQuestStatusCounts";
        getQuestProgressParams(flutterData);
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.shareDataBetweenGames = function(){
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "shareDataBetweenGames";
        sendDataToFlutterModule(flutterData,false,false);
    };

    this.gettingSharedData = function(){
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "gettingSharedData";
        sendDataToFlutterModule(flutterData,false,false);
    };

    this.incActionForQuestType = function(questType) {
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "incQuestProgress";
        flutterData["questType"] = questType;
        getQuestProgressParams(flutterData);
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.shouldResetDailyQuests = function() {
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "shouldResetDailyQuests";
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.updateDailyQuestsNotif = function() {
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "updateDailyQuestsNotif";
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.resetInGameQuestData = function() {
        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "resetQuestData";
        sendDataToFlutterModule(flutterData, false, true);

        // TOOD - Look at making this code more generic
        flutterData = getBaseFlutterParams();
        getInitParams(flutterData);
        getQuestProgressParams(flutterData);
        flutterInitializeAdwall(flutterData);
        sendDataToFlutterModule(flutterData, false, true);

        if (xPromoGameSpecificObj.isInButtonFillVariant()) {
            xPromoGameSpecificObj.showFilledQuestButton(false);
        }

    };

    this.openQuestCenter = function() {
        if(curScreen != undefined) {
            if(pspnFlutterObj.getScreenRedirectionFromFlutter() != curScreen){
                pspnFlutterObj.setFlutterLastScreen(curScreen);
            }

            var lastScreen = pspnFlutterObj.getFlutterLastScreen();
            if(lastScreen != curScreen){
                var lastLayer = utilObj.getLayerForScreen(lastScreen);
                if(lastLayer != null && utilObj.isFunction(lastLayer.redraw)) {
                    lastLayer.redraw();
                }
            }
        }

        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "showQuestCenter";
        getQuestProgressParams(flutterData);
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.openAdwall = function() {
        var flutterData = {};
        flutterData["route"] = "pspn";
        flutterData["controller"] = "adwall";
        flutterData["action"] = "showAdwall";
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.updateFlutterExperiments = function() {
        var flutterData = {};
        flutterData["route"] = "pspn";
        flutterData["controller"] = "experiments";
        flutterData["action"] = "update";

        var experimentsInfo = {};
        for(var key in flutterExperimentsToSync) {
            experimentsInfo[flutterExperimentsToSync[key]] = parseInt(epmObj.getChosenVariant(flutterExperimentsToSync[key]));
        }

        flutterData["experimentsInfo"] = experimentsInfo;
        sendDataToFlutterModule(flutterData, false, true);
    };

    this.updateRelatedQuestsStatus = function(questId, questStatus) {
        var quest = pspnFlutterObj.getAdwallQuestById(questId);
        if(quest != undefined && quest["r"] != undefined) {
            var relatedQuests = quest.r.split(":");

            var flutterData = getBaseFlutterParams();
            flutterData["action"] = "updateRelatedQuestsStatus";
            flutterData["relatedQuests"] = relatedQuests;
            flutterData["questStatus"] = questStatus;

            getQuestProgressParams(flutterData);
            sendDataToFlutterModule(flutterData, false, false);

            // In case we are setting the quest status apart from not_started and started for a given game id, lets remove it from the eligible list
            if(questStatus != xpromoConstantsObj.QUEST_STARTED && questStatus != xpromoConstantsObj.QUEST_NOT_STARTED && quest["gh"] != undefined && eligibleGames != null) {
                var index = eligibleGames.indexOf(quest["gh"]);
                if(index >= 0) {
                    eligibleGames.splice(index, 1);
                }
            }

        }

    };

    this.setQuestCompletedConfig = function (questId,val) {
        questCompletedConfig[questId] = val;
    };

    this.getQuestCompletedConfig = function(questId) {
        return questCompletedConfig[questId];
    };
    this.deleteQuestCompletedId = function(questId){
      delete questCompletedConfig[questId];
    };

    ////////////////////////////////////////////
    ///////// Flutter Handling Methods /////////
    ////////////////////////////////////////////


    function decodeFlutterMessage(msg) {
        var params = {};
        try {
            msg = decodeURIComponent(msg);
            params = JSON.parse(msg);
        } catch (e) {
        }
        return params;
    };

    this.flutterQuestCenterExit = function() {
        var fromBackButton = true;
        xPromoFlutterGameSpecificObj.flutterCustomGameExitHandle(fromBackButton);
    };

    this.flutterActionAddCash = function(msg) {
        var params = decodeFlutterMessage(msg);
        if(params["cashReward"] != undefined && params["questId"] != undefined && parseInt(params["cashReward"]) > 0) {
            var cashReward = parseInt(params["cashReward"]);
            var success = xPromoGameSpecificObj.addCash(cashReward, constantsObj.TRACK_EARN, xpromoConstantsObj.TRACK_QUEST_CENTER, params["questId"] + "");
            xPromoGameSpecificObj.fireEarnTracking(false, constantsObj.TRACK_QUEST_CENTER, constantsObj.TRACK_COINS, xPromoGameSpecificObj.getCashEquivalent() + "", cashReward + "", params["questId"] + "", success);
        }
        questsObj.setQuestAnimation(false);
    };

    this.flutterActionFbLogin = function() {
        xPromoFlutterGameSpecificObj.flutterActionFbLogin();
    };

    this.flutterActionInviteFriends = function() {
        xPromoFlutterGameSpecificObj.flutterActionShowInvitePopup();
    };

    this.flutterActionShowAndAwardRateUs = function() {
        xPromoGameSpecificObj.showAndAwardRateUs();
        if (utilObj.isOnline()) {
            questsObj.incActionForQuestType(xpromoConstantsObj.QUEST_TYPE_RATE_US);
        }
    };

    this.flutterActionShareOnWhatsapp = function () {
        utilObj.shareOnWhatsapp();
    };

    this.flutterActionShareOnTwtr = function () {
        utilObj.shareOnTwtr();
    };

    this.flutterActionShareOnSms = function () {
        utilObj.shareOnSms();
    };

    this.flutterActionShareOnMail = function () {
        utilObj.shareOnMail();
    };

    this.flutterActionShareOnMessenger = function () {
        utilObj.shareOnMessenger();
    };

    this.flutterActionShareOnGooglePlus = function () {
        utilObj.shareOnGooglePlus();
    };

    this.flutterActionShowQuestAnimation = function(msg) {
        var params = decodeFlutterMessage(msg);
        if(params["questConfig"] != undefined) {
            questsObj.setQuestAnimation(true);
            questsObj.setQuestDescription(params["questConfig"]["d"]);
            questsObj.setQuestCompleteId(params["questConfig"]["id"]);
            pspnFlutterObj.setQuestCompletedConfig(params["questConfig"]["id"],params["questConfig"]);
            questsObj.getQuestToBeAnimated().push(params["questConfig"]["id"]);

            var questType = params["questConfig"]["qt"];
            var gameHosted = params["questConfig"]["gh"];
            var questIconPath = "icon_" + gameHosted + ".png";
            var questIcon = xpromoUtilObj.getAbsFilePath(questIconPath);
            if (!utilObj.isWeb() && !utilObj.fileExists(questIconPath)) {
                questIcon = questConfigObj.getConfigData("icon_game");
                xPromoGameSpecificObj.downloadFileV1(constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/icons/" + questIconPath, questIconPath);
            }
            if (xpromoConstantsObj.QUEST_ICON_GAME_MAP[constantsObj.GAME_ID] != undefined && xpromoConstantsObj.QUEST_ICON_GAME_MAP[constantsObj.GAME_ID][questType] != undefined) {
                questIcon = xpromoConstantsObj.QUEST_ICON_GAME_MAP[constantsObj.GAME_ID][questType];
            } else if (xpromoConstantsObj.QUEST_ICON_MAP[questType] != undefined) {
                questIcon = xpromoConstantsObj.QUEST_ICON_MAP[questType];
            }

            var coinReward = 0;
            if(params["coinReward"] != undefined) {
                coinReward = parseInt(params["coinReward"]);
            }
            xPromoFlutterGameSpecificObj.flutterCustomGameAnimSurfacing(questsObj.getQuestCompleteId(), questsObj.getQuestDescription(), questIcon, coinReward);
        }

    };

    this.flutterActionUpdateNotifSurfacing = function(msg) {
        var params = decodeFlutterMessage(msg);

        if(params["completedQuests"] != undefined && params["activeQuests"] != undefined && params["claimedQuests"] != undefined) {
            questsObj.setCompletedQuests(params["completedQuests"]);
            questsObj.setActiveQuests(params["activeQuests"]);
            questsObj.setClaimedQuests(params["claimedQuests"]);

            var completedQuestIds = [];
            var activeQuestIds = [];
            var claimedQuestIds = [];
            if(utilObj.isArray(params["completedQuestIds"])) {
                completedQuestIds = params["completedQuestIds"];
            }
            if(utilObj.isArray(params["activeQuestIds"])) {
                activeQuestIds = params["activeQuestIds"];
            }
            if(utilObj.isArray(params["claimedQuestIds"])) {
                claimedQuestIds = params["claimedQuestIds"];
            }

            var shouldShowNewTag = !questsObj.getQuestCenterSeen();
            xPromoFlutterGameSpecificObj.flutterCustomGameNotifSurfacing(shouldShowNewTag, completedQuestIds, activeQuestIds, claimedQuestIds);
        }
    };

    this.flutterActionUpdateNewTag = function() {
        // Both of these variables are maintained as part of gameData, hence this flow works
        questsObj.setQuestCenterSeen(false);
        questsObj.setQuestCenterSeenTimestamp(null);
    };

    this.flutterActionRedirectToStore = function(msg) {
        var params = decodeFlutterMessage(msg);
        questsObj.redirectToStoreWithTracking(params["campaignName"], "adwall", params, null, null);
    };

    this.flutterActionShowCoinStore = function() {
        xPromoFlutterGameSpecificObj.flutterActionCoinStore();
    };

    this.flutterActionCompleteOffer = function () {
        xPromoFlutterGameSpecificObj.flutterActionCompleteOffer();
    };

    this.flutterActionShowDailyPuzzle = function() {
        xPromoFlutterGameSpecificObj.flutterActionShowDailyPuzzle();
    };

    this.flutterActionRedirectToGame = function() {
        xPromoFlutterGameSpecificObj.flutterActionRedirectToGame();
    };

    this.flutterActionShowSpinner = function() {
        xPromoFlutterGameSpecificObj.flutterActionShowSpinner();
    };

    this.flutterActionShowVideo = function() {
        xPromoFlutterGameSpecificObj.flutterActionWatchVideos();
    };

    this.flutterUpdateCrossGameQuestStatus = function(msg) {
        var params = decodeFlutterMessage(msg);
        var status = parseInt(params["status"]);
        questsObj.updateCrossGameQuestStatus(params["questId"], status);
    };

    this.flutterResetDailyQuests = function(msg) {
        xPromoGameSpecificObj.setDailyWatchVideosCount(0);
        xPromoGameSpecificObj.setDailyBonusClaims(0);
        xPromoGameSpecificObj.setDailyPuzzleClears(0);

        var flutterData = getBaseFlutterParams();
        flutterData["action"] = "resetDailyQuestsComplete";
        getQuestProgressParams(flutterData);
        sendDataToFlutterModule(flutterData, false, false);
    };

    this.flutterInitializeAdwallQuests = function(msg) {
        var params = decodeFlutterMessage(msg);
        adwallQuests = JSON.parse(params["adwallDataString"]);

        if(adwallQuests != undefined) {
            for(var index in adwallQuests) {
                if(adwallQuestsMap == null) {
                    adwallQuestsMap = {};
                }

                var adwallData = adwallQuests[index];
                adwallQuestsMap[adwallData["id"]] = adwallData;

                var imageName = "icon_" + adwallData["gh"] + ".png";
                var psIconLink = xpromoUtilObj.getImageLinkFromName(imageName);
                if (!utilObj.fileExists(imageName)) {
                    xPromoGameSpecificObj.downloadFileV2(psIconLink, imageName);
                }

            }
            moreGamesLayer.downloadReqdImages();
        }

        // loggerObj.debugLog("Abdul Adwall Data=" + utilObj.getStringifiedJson(adwallQuests));
    };

    this.flutterInitializeW2EData = function(msg) {
        var params = decodeFlutterMessage(msg);
        w2eFeatureData = JSON.parse(params["featureDataString"]);
        xpromoW2eObj.load(w2eFeatureData);
        w2eFeatureDataLoaded = true;

        // loggerObj.debugLog("Abdul W2E Feature Data=" + utilObj.getStringifiedJson(w2eFeatureData));
    };

    this.flutterInitializeBubblePopData = function(msg) {
        var params = decodeFlutterMessage(msg);
        bubblePopData = JSON.parse(params["featureDataString"]);

        var quests = {};
        quests.bubblePop = bubblePopData;
        bubblePopObj.load(quests);
        bubblePopDataLoaded = true;

        // loggerObj.debugLog("Abdul BubblePop Feature Data=" + utilObj.getStringifiedJson(bubblePopData));
    };

    this.flutterInitializeQuestRuntimes = function(msg) {
        var params = decodeFlutterMessage(msg);
        runtimeData = JSON.parse(params["runtimesString"]);

        xpromoUtilObj.updateXpromoConstantsFromRuntime();

        // var keys = "";
        // for(var i in runtimeData) {
        //     keys += i + ",";
        // }
        // loggerObj.debugLog("Abdul Runtime Keys=" + keys);
    };

    this.flutterUpdateEligibleGames = function(msg) {
        var params = decodeFlutterMessage(msg);
        eligibleGames = JSON.parse(params["eligibleGames"]);

        // loggerObj.debugLog("Abdul Eligible games data=" + utilObj.getStringifiedJson(eligibleGames));
    };

};