PspnRedirectFtue = function () {

    var pspnRedirectFtue = null; // feature progress data - used for syncing to the server as well
    var rt = null; // local variable to hold the currently evaluated runtime values
    var runtimeFetched = false; // control variable to avoid fetch runtimes over and over again
    var lastRuntimesSyncTime = 0; // last time runtimes were synced
    var grantedReward = 0;
    var gameInstalled = -1;

    var RUNTIME_DISABLED = 1000;
    var EXTRA_Z_ORDER = 10;

    var TRIGGER_PUZZLE_MEDIAN = "trig-puzzle-median";
    var TRIGGER_TIME_IN_DAY = "trig-game-time";
    var ELIGIBILITY_DAYS_INSTALL = "elig-days-install";
    var ELIGIBILITY_PLAY_FREQUENCY = "elig-play-frequency";
    var ELIGIBILITY_RECEIVER_THRESHOLD = "elig-receiver-threshold";
    var ELIGIBILITY_COOLDOWN = "elig-cooldown";
    var ELIGIBILITY_PAYER = "elig-pnp";
    var ELIGIBILITY_MIN_LEVEL = "elig-min-level";
    var ELIGIBILITY_WALLET_THRESHOLD = "elig-wallet-limit";

    var popupSurfacingMap = {
        "showPopup": false,
        "trigger": "none",
        "eligibility": "none",
        "gameToSurface": -1,
        "levelsToComplete": -1,
        "reward": -1,
        "gameName": "",
    };

    this.init = function () {
        rt = {
            // Triggers
            // T1
            puzzleDays: 7, // Days over which the player puzzle progress should be considered
            puzzleFactor: [1, 1], // Factor applied on top of player puzzle progress ( Index 0 -> leaderboard off days, Index 1 -> leaderboard days)
            // T2
            minutesPlayed: [15, 15], // Time the game is being played on a given day ( Index 0 -> leaderboard off days, Index 1 -> leaderboard days)

            // Eligibilities
            // E1
            daysSinceInstall: 10, // Days after install that this feature is visible
            // E2
            playFrequency: [3, 10], // Player's play frequency across days (Index 0 -> Dau days required, Index 1 -> Calendar days history to look at)
            // E3
            receiverThreshold: 5, // Days since the reciever game was last played
            // E4
            cooldown: 1440, // Minutes threshold for surfacing a popup
            // E5
            payerCheck: 2, // Flag to control this feature for payer / non-payer. 0 - Only Non Payer, 1 - Only Payer, 2 - Both
            // E6
            minLevel: 10, // Minimum number of levels to be played in the receiver game before feature becomes active
            // E7
            walletThreshold: 500, // Maximum wallet balance upto which the feature is available

            // Surfacing
            // S2
            exceedsDailyPuzzle: 0, // Flag corresponding to the this having higher priority than daily puzzle surfacing

            // Rewards
            coinReward: 100, // Number of coins to grant on playing the receiver game
            gamesAllowed: [9], // Games that should be allowed for xPlay
            gamesTarget: [10], // Amount of game progress required for the corresponding game
            attributionWindow: 2880, // Time limit for attribution from last click

            trackingFlag: 0, // Flag to enable tracking for control variant
        };

        pspnRedirectFtue = {
            lastShown: 0,
            xplayClicks: {},
        };

        pspnRedirectFtueObj.resetPopupSurfacingMap();
    };

    this.load = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }

        if (quests.pspnRedirectFtue != undefined) {
            pspnRedirectFtue = quests.pspnRedirectFtue;
        }

        if(pspnRedirectFtue.lastShown == undefined) {
            pspnRedirectFtue.lastShown = 0;
        }

        if(pspnRedirectFtue.xplayClicks == undefined) {
            pspnRedirectFtue.xplayClicks = {};
        }
    };

    this.save = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }
        quests.pspnRedirectFtue = pspnRedirectFtue;
        return quests;
    };

    /**
     * Method to return the current variant of the redirect Ftue experiment
     * @returns {number} - number corresponding to the variant, 0 for control / off.
     */
    this.getExperimentVariant = function() {
        return epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_REDIRECT_FTUE);
    };

    /**
     * Method to check if the redirect ftue feature is currently active or in-active for a player
     * @returns {boolean} - True / False
     */
    this.isRedirectFtueActive = function() {
        return (xPromoGameSpecificObj.isRedirectFtueEnabled() && pspnRedirectFtueObj.getExperimentVariant() != xpromoConstantsObj.EXP_VAR_CONTROL);
    };

    /**
     * Method to fetch runtime value based on the experiment variant. Also caches the value for the rest of the session.
     * @returns {object}  - Returns a JS object with runtime values.
     */
    this.getRuntimes = function (tryForceFetch) {
        if(tryForceFetch == undefined || tryForceFetch != true) {
            tryForceFetch = false;
        }

        if(runtimeFetched && (!tryForceFetch || (utilObj.getCurrentTimestamp() - lastRuntimesSyncTime) <= constantsObj.SECONDS_IN_HOUR)) {
            return rt;
        }

        var variant = pspnRedirectFtueObj.getExperimentVariant();
        var defaultKey = "rf1";
        var runtimeKey = defaultKey;
        if(variant > constantsObj.EXP_VAR_CONTROL) {
            runtimeKey = "rf" + (parseInt(variant)).toString();
        }
        // Different configuration is expected for different variants.
        var updatedRuntime = xpromoQuestRuntimesObj.getRuntime(runtimeKey, null);
        if(updatedRuntime == null || updatedRuntime == undefined ) {
            updatedRuntime = xpromoQuestRuntimesObj.getRuntime(defaultKey, null);
        }

        if (updatedRuntime != null) {
            for (var i = 0; i < updatedRuntime.pf.length; i++) {
                updatedRuntime.pf[i] = parseFloat(updatedRuntime.pf[i]);
                updatedRuntime.mp[i] = parseInt(updatedRuntime.mp[i]);
            }

            for (var i = 0; i < updatedRuntime.fq.length; i++) {
                updatedRuntime.fq[i] = parseInt(updatedRuntime.fq[i]);
            }

            if(updatedRuntime.ga.length == updatedRuntime.gt.length) {
                for (var i = 0; i < updatedRuntime.ga.length; i++) {
                    updatedRuntime.ga[i] = parseInt(updatedRuntime.ga[i]);
                    updatedRuntime.gt[i] = parseInt(updatedRuntime.gt[i]);
                }
            } else {
                updatedRuntime.ga = [];
                updatedRuntime.gt = [];
            }

            rt.puzzleDays = parseInt(updatedRuntime.pd[0]);
            rt.puzzleFactor = updatedRuntime.pf;
            rt.minutesPlayed = updatedRuntime.mp;
            rt.daysSinceInstall = parseInt(updatedRuntime.dsi[0]);
            rt.playFrequency = updatedRuntime.fq;
            rt.receiverThreshold = parseInt(updatedRuntime.rt[0]);
            rt.cooldown = parseInt(updatedRuntime.cl[0]);
            rt.payerCheck = parseInt(updatedRuntime.pc[0]);
            rt.minLevel = parseInt(updatedRuntime.ml[0]);
            rt.walletThreshold = parseInt(updatedRuntime.wt[0]);
            rt.exceedsDailyPuzzle = parseInt(updatedRuntime.p[0]);
            rt.coinReward = parseInt(updatedRuntime.cr[0]);
            rt.gamesAllowed = updatedRuntime.ga;
            rt.gamesTarget = updatedRuntime.gt;
            rt.attributionWindow = parseInt(updatedRuntime.aw[0]);
            rt.trackingFlag = parseInt(updatedRuntime.tf[0]);

            // loggerObj.debugLog("REDIRECT FTUE: runtimes", utilObj.getStringifiedJson(rt));
            lastRuntimesSyncTime = utilObj.getCurrentTimestamp();
            runtimeFetched = true;
        }

        return rt;
    };


    /**
     * Helper method to reset the popup surfacing map
     */
    this.resetPopupSurfacingMap = function() {
        popupSurfacingMap = {
            "showPopup": false,
            "trigger": "none",
            "eligibility": "none",
            "gameToSurface": -1,
            "levelsToComplete": -1,
            "reward": -1,
            "gameName": "",
        };
    };

    /**
     * Entry level method which determines and persists (for the session) if redirect ftue can be surfaced
     * @param src
     * @returns {boolean}
     */
    this.checkXplayPopupCondition = function(src) {
        var success = false;

        if(xPromoGameSpecificObj.isRedirectFtueEnabled()) {
            // Lets try and load the runtimes if not yet done
            var runtimeValues = pspnRedirectFtueObj.getRuntimes(true);

            if(pspnRedirectFtueObj.isRedirectFtueActive() || runtimeValues.trackingFlag == 1) {
                var passedTriggers = pspnRedirectFtueObj.checkTrigger();

                // If no triggers have passed, no point evaluating further.
                if(passedTriggers.length > 0) {

                    pspnRedirectFtueObj.resetPopupSurfacingMap();
                    popupSurfacingMap.trigger = passedTriggers[0];

                    if(pspnRedirectFtueObj.checkEligibility() && pspnRedirectFtueObj.isRedirectFtueActive()) {
                        var gameToSurface = pspnRedirectFtueObj.getEligibleGame();
                        if(gameToSurface != constantsObj.NOT_FOUND && runtimeValues.coinReward > 0) {
                            var levelsToComplete = pspnRedirectFtueObj.getLevelsToCompleteForXplay(gameToSurface);
                            if(levelsToComplete != constantsObj.NOT_FOUND) {

                                // Since the eligiblities have passed, we can store the popup details for it to be shown in the future
                                // loggerObj.debugLog("REDIRECT FTUE: eligible to show popup!", gameToSurface, levelsToComplete, runtimeValues.coinReward);
                                var gameName = (xpromoConstantsObj.PSGames[gameToSurface] != undefined) ? xpromoConstantsObj.PSGames[gameToSurface][2] : "";
                                if(gameToSurface == 8) {
                                    gameName = "DTC";
                                }
                                popupSurfacingMap.gameToSurface = gameToSurface;
                                popupSurfacingMap.levelsToComplete = levelsToComplete;
                                popupSurfacingMap.reward = runtimeValues.coinReward;
                                popupSurfacingMap.gameName = gameName;
                                popupSurfacingMap.showPopup = true;

                                // For going to LLP screen, success is set to true only if this is the immediate priority item to be shown, else it remains false even if eligibilities have passed
                                // For going to non llp screen, success can be true
                                success = (src == "llp") ? (runtimeValues.exceedsDailyPuzzle == 1) : true;
                            }
                        }
                    }

                    trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO,
                        popupSurfacingMap.trigger,
                        popupSurfacingMap.eligibility,
                        xpromoConstantsObj.TRACK_PSPN_REDIRECT_FTUE,
                        "", xpromoConstantsObj.TRACK_XPLAY, "var" + pspnRedirectFtueObj.getExperimentVariant(), "", "");
                }
            }
        }
        return success;
    };


    /**
     * Method to surface the redirect FTUE popup given it was previously persisted in the session
     * @param dailyPuzzleStatus
     * @returns {boolean}
     */
    this.tryShowXplayPopup = function(dailyPuzzleStatus) {
        // loggerObj.debugLog("REDIRECT FTUE: trying to show popup!");
        var success = false;
        if(pspnRedirectFtueObj.isRedirectFtueActive()) {
            if(popupSurfacingMap != undefined && Object.keys(popupSurfacingMap).length > 0) {
                if(popupSurfacingMap["showPopup"] != undefined && popupSurfacingMap["showPopup"] == true) {

                    trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO,
                        popupSurfacingMap.trigger,
                        popupSurfacingMap.eligibility,
                        xpromoConstantsObj.TRACK_PSPN_REDIRECT_FTUE,
                        "success", xpromoConstantsObj.TRACK_XPLAY, dailyPuzzleStatus, "", "");

                    pspnRedirectFtue.lastShown = utilObj.getCurrentTimestamp();
                    pspsRedirectFtueUiObj.show(xPromoGameSpecificObj.popupShowWithAnim(), {
                        gameId: popupSurfacingMap.gameToSurface,
                        target: popupSurfacingMap.levelsToComplete,
                        reward: popupSurfacingMap.reward,
                        gameName: popupSurfacingMap.gameName,
                        trackingSpecies: dailyPuzzleStatus,
                    });
                    pspnRedirectFtueObj.resetPopupSurfacingMap();
                    success = true;
                }
            }
        }
        return success;
    };


    /**
     * Method to log scenarios when a valid surfacing got deprioritized due to other game conditions
     * @param surfacingFailure
     * @param dailyPuzzleStatus
     */
    this.tryLogSurfacingFailure = function(surfacingFailure, dailyPuzzleStatus) {
        if(pspnRedirectFtueObj.isRedirectFtueActive()) {
            if(popupSurfacingMap != undefined && Object.keys(popupSurfacingMap).length > 0) {
                if (popupSurfacingMap["showPopup"] != undefined && popupSurfacingMap["showPopup"] == true) {
                    trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO,
                        popupSurfacingMap.trigger,
                        popupSurfacingMap.eligibility,
                        xpromoConstantsObj.TRACK_PSPN_REDIRECT_FTUE,
                        surfacingFailure, xpromoConstantsObj.TRACK_XPLAY, dailyPuzzleStatus, "", "");
                }
            }
        }
    };

    /**
     * Helper method to check if a value is marked with the special RUNTIME_DISABLED value
     * @param value
     * @returns {boolean}
     */
    function isDisabled(value) {
        return (parseInt(value) == RUNTIME_DISABLED);
    }

    /**
     * Method that checks if any of the triggers have passed
     * @returns {Array} - List of strings representing the passed triggers
     */
    this.checkTrigger = function() {
        var result = [];

        var runtime = pspnRedirectFtueObj.getRuntimes();

        // Checking for trigger related to Puzzle played - T1
        if(!isDisabled(runtime.puzzleDays)) {
            var puzzleFactorIndex = (xPromoGameSpecificObj.isEventActive()) ? 1 : 0;
            if(utilObj.isArray(runtime.puzzleFactor) && runtime.puzzleFactor[puzzleFactorIndex] != undefined
                && !isDisabled(runtime.puzzleFactor[puzzleFactorIndex])) {

                var puzzlesCompleted = xPromoGameSpecificObj.getDailyPuzzleClears();
                var medianPuzzleCompletes = pspnActionHistory.getPuzzleClearsMedianByDau(runtime.puzzleDays);
                // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: TRIGGER_PUZZLE_MEDIAN medianPuzzles=", medianPuzzleCompletes , "current=", puzzlesCompleted, "factor=", runtime.puzzleFactor[puzzleFactorIndex], "target=", Math.ceil(Math.max(1, medianPuzzleCompletes) * runtime.puzzleFactor[puzzleFactorIndex]));
                if(medianPuzzleCompletes != constantsObj.NOT_FOUND && puzzlesCompleted >= Math.ceil(Math.max(1, medianPuzzleCompletes) * runtime.puzzleFactor[puzzleFactorIndex])) {
                    result.push(TRIGGER_PUZZLE_MEDIAN);
                }
            }
        }

        // Checking for trigger related to time in game - T2
        if(result.length == 0) {
            var minutesPlayedIndex = (xPromoGameSpecificObj.isEventActive()) ? 1 : 0;
            if(utilObj.isArray(runtime.minutesPlayed) && runtime.minutesPlayed[minutesPlayedIndex] != undefined
                && !isDisabled(runtime.minutesPlayed[minutesPlayedIndex])) {

                // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: TRIGGER_TIME_IN_DAY actual=", (xPromoGameSpecificObj.getTimeSpentInSession() / constantsObj.SECONDS_IN_MIN), "required=" + runtime.minutesPlayed[minutesPlayedIndex]);
                if(xPromoGameSpecificObj.getTimeSpentInSession() >= runtime.minutesPlayed[minutesPlayedIndex] * constantsObj.SECONDS_IN_MIN) {
                    result.push(TRIGGER_TIME_IN_DAY);
                }
            }
        }

        return result;
    };

    /**
     * Method that checks all the eligibility criteria are passing
     * @returns {boolean} - True if all eligibilities have passed, false otherwise
     */
    this.checkEligibility = function() {
        var runtime = pspnRedirectFtueObj.getRuntimes();

        // Checking for eligibility related to days since install - E1
        popupSurfacingMap.eligibility = ELIGIBILITY_DAYS_INSTALL;
        // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_DAYS_INSTALL actual=", xpromoUtilObj.getGameDaysSinceInstall() , "required=", runtime.daysSinceInstall);
        if(!isDisabled(runtime.daysSinceInstall) && xpromoUtilObj.getGameDaysSinceInstall() < runtime.daysSinceInstall) {
            return false;
        }

        // Checking for eligibility related to play frequency - E2
        popupSurfacingMap.eligibility = ELIGIBILITY_PLAY_FREQUENCY;
        if(utilObj.isArray(runtime.playFrequency) && runtime.playFrequency.length >= 2) {
            var requiredDauDays = runtime.playFrequency[0];
            var calendarDays = runtime.playFrequency[1];
            // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_PLAY_FREQUENCY actual=", pspnActionHistory.getDauDaysInCalendarDays(calendarDays) , "required=", requiredDauDays);
            if(!isDisabled(requiredDauDays) && !isDisabled(calendarDays)) {
                var actualDauDays = pspnActionHistory.getDauDaysInCalendarDays(calendarDays);
                if(actualDauDays == constantsObj.NOT_FOUND || actualDauDays < requiredDauDays) {
                    return false;
                }
            }
        } else {
            return false;
        }

        // Checking for eligibility related to days since any game last played - E3
        popupSurfacingMap.eligibility = ELIGIBILITY_RECEIVER_THRESHOLD;
        if(utilObj.isArray(runtime.gamesAllowed) && runtime.gamesAllowed.length > 0) {
            // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_RECEIVER_THRESHOLD eligible game=", pspnRedirectFtueObj.getEligibleGame() , "actual=", xpromoUtilObj.daysSincePlayedOtherGame() , "required=", runtime.receiverThreshold);
            if(!isDisabled(runtime.receiverThreshold)) {
                if(pspnRedirectFtueObj.getEligibleGame() == constantsObj.NOT_FOUND || xpromoUtilObj.daysSincePlayedOtherGame() < runtime.receiverThreshold) {
                    return false;
                }
            }
        } else {
            return false;
        }

        // Checking for eligibility related to last popup impression - E4
        popupSurfacingMap.eligibility = ELIGIBILITY_COOLDOWN;
        if(pspnRedirectFtue.lastShown != undefined) {
            // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_COOLDOWN actual=", ((utilObj.getCurrentTimestamp() - pspnRedirectFtue.lastShown) / constantsObj.SECONDS_IN_MIN) , "required=", runtime.cooldown);
            if(!isDisabled(runtime.cooldown) && ((utilObj.getCurrentTimestamp() - pspnRedirectFtue.lastShown) < (runtime.cooldown * constantsObj.SECONDS_IN_MIN))) {
                return false;
            }
        } else {
            return false;
        }

        // Checking for eligibility related to payer check - E5
        popupSurfacingMap.eligibility = ELIGIBILITY_PAYER;
        // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_PAYER isPayer=", xPromoGameSpecificObj.getIsPayerUser() , "runtime=", runtime.payerCheck);
        if(runtime.payerCheck != 2) {
            var isPayer = xPromoGameSpecificObj.getIsPayerUser();
            if((runtime.payerCheck == 0 && isPayer) || (runtime.payerCheck == 1 && !isPayer)) {
                return false;
            }
        }

        // Checking for eligibility related to minimum levels played - E6
        popupSurfacingMap.eligibility = ELIGIBILITY_MIN_LEVEL;
        // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_MIN_LEVEL actual=", xPromoGameSpecificObj.getPuzCompleted() , "required=", runtime.minLevel);
        if(!isDisabled(runtime.minLevel)) {
            if(parseInt(xPromoGameSpecificObj.getPuzCompleted()) < runtime.minLevel) {
                return false;
            }
        }

        // Checking for eligibility related to wallet threshold - E7
        popupSurfacingMap.eligibility = ELIGIBILITY_WALLET_THRESHOLD;
        // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_WALLET_THRESHOLD actual=", xPromoGameSpecificObj.getCash() , "required=", runtime.walletThreshold);
        if(!isDisabled(runtime.walletThreshold)) {
            if(xPromoGameSpecificObj.getCash() >= runtime.walletThreshold) {
                return false;
            }
        }

        // loggerObj.debugLog("REDIRECT FTUE: EVALUATION: ELIGIBILITY_ALL_SUCCESS");

        popupSurfacingMap.eligibility = "success";
        return true
    };

    /**
     * Helper method to get any eligible game for xpromo surfacing
     * @returns {Number} gameId if an eligible game is found, -1 otherwise
     */
    this.getEligibleGame = function() {
        var result = constantsObj.NOT_FOUND;

        var installedGamesStrArr = xpromoUtilObj.getInstalledPSGames();
        var installedGames = installedGamesStrArr.map(Number);
        var runtime = pspnRedirectFtueObj.getRuntimes();
        if(installedGames.length > 0 && utilObj.isArray(runtime.gamesAllowed) && runtime.gamesAllowed.length > 0) {

            //Find the intersection, and check if there is any
            var intersection = installedGames.filter(function(n) {
                return runtime.gamesAllowed.indexOf(n) !== -1;
            });

            if(intersection.length > 0) {
                var adwallQuests = questsObj.getAdWallQuests();
                var questPrioritiesArr = Object.keys(adwallQuests);
                questPrioritiesArr.sort(function(a, b) { // Sorting by the priority
                    return parseInt(a) - parseInt(b);
                });

                if (questPrioritiesArr.length > 0) {
                    for (var i = 0; i < questPrioritiesArr.length; i++) {
                        var game = questPrioritiesArr[i];
                        var adwallQuest = adwallQuests[game];
                        var gameId = adwallQuest.gh;

                        if(intersection.indexOf(gameId) != -1 && gameId != constantsObj.GAME_ID) {
                            result = gameId;
                            break;
                        }
                    }
                }
            }
        }

        return result;
    };

    /**
     * Method to get the number of levels required to be completed for a given target game
     * @param gameId
     * @returns {Number} level count in all valid scenarios, -1 otherwise
     */
    this.getLevelsToCompleteForXplay = function(gameId) {
        var result = constantsObj.NOT_FOUND;

        var runtime = pspnRedirectFtueObj.getRuntimes();
        if(utilObj.isArray(runtime.gamesAllowed) && runtime.gamesAllowed.length > 0) {
            var index = runtime.gamesAllowed.indexOf(gameId);
            if(index != -1 && runtime.gamesTarget.length > index && parseInt(runtime.gamesTarget[index]) > 0) {
                result = parseInt(runtime.gamesTarget[index]);
            }
        }
        return result;
    };


    /**
     * Method to handle the click on the play now button in the redirect FTUE popup
     * @param sender
     * @param params
     */
    this.playNow = function(sender, params) {
        // loggerObj.debugLog("REDIRECT FTUE: playNow", utilObj.getStringifiedJson(params));
        if(params != undefined && params.gameId != undefined) {
            var adwallQuests = questsObj.getAdWallQuests();
            for (var quest in adwallQuests) {
                if (adwallQuests[quest].gh == params.gameId) {
                    popupLayer.close({noAnim: xPromoGameSpecificObj.popupShowWithAnim()});
                    pspnRedirectFtueObj.markXplayInteraction(params.gameId);
                    var redirectionParams = xpromoUtilObj.generateRedirectionParams(adwallQuests[quest]);
                    questsObj.redirectToStoreWithTracking(xpromoConstantsObj.TRACK_PSPN_REDIRECT_FTUE, utilObj.getScreenName(params.screen), redirectionParams, xpromoConstantsObj.TRACK_PSPN_REDIRECT_FTUE);
                    break;
                }
            }
        }
    };

    /**
     * Method to persist click action related data and state changes
     * @param gameId
     */
    this.markXplayInteraction = function (gameId) {
        if(!pspnRedirectFtueObj.isRedirectFtueActive()) {
            return;
        }

        if(pspnRedirectFtue == null) {
            return;
        }

        if(pspnRedirectFtue.xplayClicks == undefined) {
            pspnRedirectFtue.xplayClicks = {};
        }

        var currentGameLevel = xpromoUtilObj.getLevelForGame(gameId);
        // loggerObj.debugLog("REDIRECT FTUE: currentGameLevel", currentGameLevel, "levels for xplay", pspnRedirectFtueObj.getLevelsToCompleteForXplay(gameId));
        if(currentGameLevel < 0) {
            currentGameLevel = 0;
        }

        var runtime = pspnRedirectFtueObj.getRuntimes();
        pspnRedirectFtue.xplayClicks[gameId] = {
            "ts": utilObj.getCurrentTimestamp(),
            "rewards": runtime.coinReward,
            "target": pspnRedirectFtueObj.getLevelsToCompleteForXplay(gameId) + currentGameLevel,
        };

        // loggerObj.debugLog("REDIRECT FTUE: markXplayInteraction", utilObj.getStringifiedJson(pspnRedirectFtue.xplayClicks));
    };


    /**
     * Method to check and grant any eligible xplay reward
     */
    this.checkAndGrantXplayReward = function () {
        if(!pspnRedirectFtueObj.isRedirectFtueActive()) {
            return;
        }

        if(pspnRedirectFtue == null || pspnRedirectFtue.xplayClicks == undefined) {
            return;
        }

        var allowedScreens = [constantsObj.SCREEN_GAME, constantsObj.SCREEN_LINEAR_PROGRESSION, constantsObj.SCREEN_HOME];
        if(allowedScreens.indexOf(curScreen) == -1 || xPromoGameSpecificObj.isFtueVisible() || (curScreen == constantsObj.SCREEN_GAME && gameLayer.ftueShown)) {
            return;
        }

        var runtime = pspnRedirectFtueObj.getRuntimes();

        var attributionPeriod = runtime.attributionWindow * constantsObj.SECONDS_IN_MIN;
        var curTime = utilObj.getCurrentTimestamp();

        for(var gameId in pspnRedirectFtue.xplayClicks) {
            var delEntry = false;
            var clickData = pspnRedirectFtue.xplayClicks[gameId];
            var clickTime = clickData["ts"];
            var reward = clickData["rewards"];
            var target = clickData["target"];

            // loggerObj.debugLog("REDIRECT FTUE: reward check conditions ", gameId, target, reward, (clickTime > 0 && ((clickTime + attributionPeriod) >= curTime)), xpromoUtilObj.getLevelForGame(gameId));

            if(clickTime > 0 && ((clickTime + attributionPeriod) >= curTime)
                && xpromoUtilObj.isAppInstalledByGameId(gameId) && xpromoUtilObj.getLevelForGame(gameId) >= target) {

                // loggerObj.debugLog("REDIRECT FTUE: eligible to reward!", gameId, target, reward);
                xPromoGameSpecificObj.addCash(reward, constantsObj.TRACK_EARN, xpromoConstantsObj.TRACK_QUEST_CENTER, xpromoConstantsObj.TRACK_PSPN_REDIRECT_FTUE);
                xPromoGameSpecificObj.fireEarnTracking(false, xpromoConstantsObj.TRACK_QUEST_CENTER, constantsObj.TRACK_COINS, xPromoGameSpecificObj.getCashEquivalent() + "", reward + "", xpromoConstantsObj.TRACK_PSPN_REDIRECT_FTUE, true);

                if(constantsObj.GAME_ID == 11) {
                    giftBoxAnimationObj.show(curLayer, false, reward, {
                        handlerObj: 'pspnRedirectFtueObj',
                        actionOnObj: 'clickHandler',
                        customLayout: pspnRedirectFtueObj.addRewardScreenMsg,
                        showBonusBanner: false,
                        boxTypeToUse: "red",
                        customZOrder: constantsObj.FTUE_Z_ORDER + EXTRA_Z_ORDER,
                    });
                } else {
                    xPromoGameSpecificObj.showCoinGrantMessage(reward);
                }

                grantedReward = reward;
                gameInstalled = gameId;
                delEntry = true;
            }

            if (((clickTime + attributionPeriod) < curTime)) {
                delEntry = true;
            }

            if(delEntry) { // Lets delete the rewards for the gameId as they are either granted to expired
                delete pspnRedirectFtue.xplayClicks[gameId];
            }
        }
    };


    /**
     * Click handler for reward screen claim button
     */
    this.clickHandler = function() {
        giftBoxAnimationObj.claimBtnFn();
    };

    /**
     * Custom UI utility method to handle population of elements in the reward animation UI
     * @param layer
     */
    this.addRewardScreenMsg = function(layer) {
        var labelArr = [];
        var spriteArr = [];

        var fadeInNode = function (node) {
            animationObj.runAction(node, animationObj.getFadeIn(0.3));
        };

        var congratsLbl = utilObj.getBMFontLabel(xpromoStringsObj.congratulations, constantsObj.FONT_SFCOMPACT_MEDIUM_140, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), centerXPos, centerYPos + 1150, 0.5, 0.5, labelArr, layer, constantsObj.FTUE_Z_ORDER + EXTRA_Z_ORDER);
        fadeInNode(congratsLbl);

        if(parseInt(grantedReward) > 0) {
            var youEarned = xpromoStringsObj.you_earned_compact.replace("%X%", grantedReward.toString());
            var text1 = utilObj.getBMFontLabel(youEarned, constantsObj.FONT_SFCOMPACT_MEDIUM_80, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), centerXPos, centerYPos + 950, 0.5, 0.5, labelArr, layer, constantsObj.FTUE_Z_ORDER + EXTRA_Z_ORDER);
            fadeInNode(text1);

            if (gameInstalled != -1) {
                var gameName = (xpromoConstantsObj.PSGames[gameInstalled] != undefined) ? xpromoConstantsObj.PSGames[gameInstalled][2] : "";
                if(gameInstalled == 8) {
                    gameName = "DTC";
                }
                var forGame = xpromoStringsObj.for_game.replace("%GAME%", gameName);

                var text2 = utilObj.getBMFontLabel(forGame, constantsObj.FONT_SFCOMPACT_MEDIUM_80, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), centerXPos, centerYPos + 850, 0.5, 0.5, labelArr, layer, constantsObj.FTUE_Z_ORDER + EXTRA_Z_ORDER);
                var gameIcon = "icon_" + gameInstalled + ".png";
                if (!utilObj.isWeb() && !utilObj.fileExists(gameIcon)) {
                    xPromoGameSpecificObj.downloadFileV1(constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/icons/" + gameIcon, gameIcon);
                } else {
                    var iconSprite = utilObj.getSprite(xpromoUtilObj.getAbsFilePath(gameIcon), centerXPos + text2.getContentSize().width / 2, centerYPos + 850, 0, 0.5, constantsObj.SCALE_SPRITE_8, spriteArr, layer, constantsObj.FTUE_Z_ORDER + EXTRA_Z_ORDER, xpromoConstantsObj.SPRITE_QUESTS);
                    fadeInNode(iconSprite);
                }
                fadeInNode(text2);
            }
        }

        giftBoxAnimationObj.addToAnimArray(labelArr, spriteArr);

        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);
    };
};
