QuestRotation = function () {

    var bucketInfo = {};
    var lastQuestClicked = {};
    // * this list will tell us whether to merge these quests for a session or not
    var nonMergingQuestList = {};

    this.init = function () {

    };

    this.getBucketLimits = function (bucketId) {
        var limits = xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_QUEST_ROTATION, bucketId);
        var params = null;
        if (limits != undefined && limits.length >= 4) {
            params = {
                views: parseInt(limits[0]),
                clicks: parseInt(limits[1]),
                days: parseInt(limits[2]),
                persistentDuration : parseInt(limits[3])
            };
        }
        return params;
    };

    this.load = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }
        if (quests.bucketInfo != undefined) {
            bucketInfo = quests.bucketInfo;
        }
        if (quests.lastQuestClicked != undefined) {
            lastQuestClicked = quests.lastQuestClicked;
        }
        if (quests.nonMergingQuestList != undefined) {
            nonMergingQuestList = quests.nonMergingQuestList;
        }
    };

    this.save = function (quests) {
        if (quests == null || quests == "" || quests == undefined) {
            return false;
        }
        quests.bucketInfo = bucketInfo;
        quests.lastQuestClicked = lastQuestClicked;
        quests.nonMergingQuestList = nonMergingQuestList;
        return quests;
    };

    this.getBucketInfo = function (bucketId) {
        if (bucketInfo[bucketId] != undefined) {
            return bucketInfo[bucketId];
        }
        return null;
    };

    this.getLastQuestClicked = function () {
        return lastQuestClicked;
    };

    this.incBucketViewCount = function (bucketId) {
        if (bucketInfo[bucketId] == undefined) {
            return;
        }
        loggerObj.debugLog("Showing bucketid, view incremented : bucketId - ", bucketId, "views count- ", bucketInfo[bucketId].views);
        bucketInfo[bucketId].views++;
    };

    this.incBucketClickCount = function (bucketId, questId) {
        if (bucketInfo[bucketId] == undefined) {
            return;
        }
        loggerObj.debugLog("Showing bucketid, click incremented : bucketId - ", bucketId, "clicks count ", bucketInfo[bucketId].clicks);
        bucketInfo[bucketId].clicks++;
        loggerObj.debugLog("this is quest rotation 1 ", questId);
        lastQuestClicked[questId] = 1;
    };

    this.getNonMergingQuestList = function () {
        return nonMergingQuestList;
    };

    this.shouldRotateQuestInBucket = function (bucketId, questStatus, quest) {
        var shouldRotate = false;
        if (epmObj.getChosenVariant(xpromoConstantsObj.EXP_PSPN_QUEST_ROTATION) == constantsObj.EXP_VAR_CONTROL || bucketInfo[bucketId] == undefined) {
            return shouldRotate;
        }
        if (questStatus == xpromoConstantsObj.QUEST_COMPLETED) {
            return shouldRotate;
        }
        var bucketLimits = questRotationObj.getBucketLimits(bucketId);
        if(bucketLimits == null || bucketLimits == undefined) {
            return shouldRotate;
        }
        var currTime = utilObj.getCurrentTimestamp();

        if (bucketLimits.views <= bucketInfo[bucketId].views ||
            bucketLimits.clicks <= bucketInfo[bucketId].clicks ||
            (bucketInfo[bucketId].resetTs != undefined && bucketInfo[bucketId].resetTs > 0 && Math.floor((currTime - bucketInfo[bucketId].resetTs) / constantsObj.SECONDS_IN_DAY) >= bucketLimits.days)) {
            if (questStatus == xpromoConstantsObj.QUEST_STARTED) {
                if (bucketInfo[bucketId].limitsReachedTime == 0) {
                    bucketInfo[bucketId].limitsReachedTime = currTime;
                } else if ((currTime - bucketInfo[bucketId].limitsReachedTime) >= (constantsObj.SECONDS_IN_HOUR * bucketLimits.persistentDuration)) {
                    shouldRotate = true;
                }
            } else {
                shouldRotate = true;
            }
        }
        return shouldRotate;
    };

    this.getQuestIndexForBucket = function (bucketId, questsInBucket, unseenBucket) {
        if (bucketInfo[bucketId] == undefined || bucketInfo[bucketId].numQuests != questsInBucket) {
            // bucket information is changed now,so resetting
            questRotationObj.resetParamsForBucket(bucketId, 0, questsInBucket);
        }
        if (unseenBucket[0].status == xpromoConstantsObj.QUEST_COMPLETED || unseenBucket[0].status == xpromoConstantsObj.QUEST_STARTED) {
            return 0;
        }
        var questIndex = bucketInfo[bucketId].questIndex;
        if (unseenBucket[questIndex] == undefined) {
            questRotationObj.resetParamsForBucket(bucketId, 0, questsInBucket);
            return 0;
        }
        return questIndex;
    };

    this.rotateToNextQuest = function (bucketId, questsInBucket) {
        var questIndex = bucketInfo[bucketId].questIndex + 1;
        if (questIndex >= questsInBucket) {
            questIndex = 0;
        }
        questRotationObj.resetParamsForBucket(bucketId, questIndex, questsInBucket);
    };

    this.resetParamsForBucket = function (bucketId, questIndex, questsInBucket) {
        bucketInfo[bucketId] = {
            views: 0,
            clicks: 0,
            limitsReachedTime: 0,
            questIndex: questIndex,
            numQuests: questsInBucket,
            resetTs: utilObj.getCurrentTimestamp()
        };
    };

    this.getCandidateForBucket = function (candidatesInBucket) {
        var candidateForBucket = {};
        var questsSeen = {};
        // var curQuests = questsObj.getCurQuests();
        for (var bucketId in candidatesInBucket) {
            var bucket = candidatesInBucket[bucketId];
            var updatedIndex = 0;
            if (bucket.length == 0) {
                continue;
            }
            var unseenBucket = [];
            for (var questId in bucket) {
                var quest = bucket[questId];
                var relatedQuests = questsObj.getRelatedQuestsForQuestId(quest.id);
                var relatedQuestCliked = false;
                if (relatedQuests != undefined && relatedQuests != null) {
                    for (var relQuest in relatedQuests) {
                        if (lastQuestClicked[relatedQuests[relQuest]] == 1) {
                            relatedQuestCliked = true;
                            break;
                        }
                    }
                }
                if (relatedQuestCliked == false && questsSeen[quest.id] == undefined) {
                    unseenBucket.push(quest);
                }
            }
            if (unseenBucket.length == 0) {
                continue;
            }
            unseenBucket.sort(function (a, b) {
                return a.p - b.p;
            });
            unseenBucket.sort(function (a, b) {
                return b.status - a.status;
            });

            var index = questRotationObj.getQuestIndexForBucket(bucketId, bucket.length, unseenBucket);
            if (unseenBucket.length != 1) {
                if (questRotationObj.shouldRotateQuestInBucket(bucketId, unseenBucket[index].status, unseenBucket[index])) {
                    questRotationObj.rotateToNextQuest(bucketId, bucket.length);
                    var questId = unseenBucket[index].id;
                    if (questsObj.isQuestInGame(questId)) {
                        questsObj.updateInGameQuestStatus(questId, xpromoConstantsObj.QUEST_NOT_STARTED);
                    } else {
                        questsObj.updateCrossGameQuestStatus(questId, xpromoConstantsObj.QUEST_NOT_STARTED);
                    }
                    questRotationObj.addQuestsInNonMergingQuestList(questId);
                    lastQuestClicked[questId] = 0;
                    /////////////////////////////////////////
                    // Misc quest specific checks
                    if (unseenBucket[index].isSocialQuest == 1) {
                        socialQuestObj.resetCurrentFriend();
                    }
                    /////////////////////////////////////////
                }
                updatedIndex = questRotationObj.getQuestIndexForBucket(bucketId, bucket.length, unseenBucket);
            }
            candidateForBucket[bucketId] = unseenBucket[updatedIndex];
            questsObj.makeRelatedQuestsSeen(unseenBucket[updatedIndex].id, questsSeen);
        }
        return candidateForBucket;
    };

    this.addQuestsInNonMergingQuestList = function (questId) {
        var crossGameRelatedQuests = questsObj.getRelatedQuests();
        if (crossGameRelatedQuests == undefined || crossGameRelatedQuests == null) {
            return;
        }
        var questInQueue = [questId];
        while (questInQueue.length > 0) {
            var curQuest = questInQueue.pop();
            nonMergingQuestList[curQuest] = 1;
            loggerObj.debugLog("this is quest rotation 11 ", questId, curQuest);
            if (crossGameRelatedQuests[curQuest] != undefined) {
                for (var j = 0; j < crossGameRelatedQuests[curQuest].length; j++) {
                    if (nonMergingQuestList[crossGameRelatedQuests[curQuest][j]] == undefined) {
                        questInQueue.push(crossGameRelatedQuests[curQuest][j]);
                    }
                }
            }
        }
    };

};