Requests = function () {
    var REQUESTS_CONTROLLER = "requests";
    var ACTION_CREATE_REQUESTS = "createRequests";
    var ACTION_GET_REQUESTS_DATA = "getRequestsData";
    var ACTION_UPDATE_REQUESTS_DATA = "updateRequests";
    var requestsController = null;
    var lastRequestsFetchTimeStamp = 0;
    var lastOldRequestListString = "";

    // this is map with key as senderRefid + receiverRefId and value as request. We need this so that we don't overwrite any request with the same key
    var existingRequestsMap = {};
    var isWordWarsFtueSeen = false;
    var REQUESTS_FETCH_INTERVAL = 600;

    // getter setters
    this.getLastOldRequestListString = function () {
        return lastOldRequestListString;
    };

    this.setLastOldRequestListString = function (val) {
        lastOldRequestListString = val;
    };

    // initializing requests
    // setting controller same as LOGIN SERVER as requests server side is controlled on login server only
    // for testing purposes staging fetch interval is set as 10 secs
    this.init = function () {
        requestsController = new Controller(constantsObj.LOGIN_SERVER_REST);
        load();
        // for staging keeping it as 10 secs
        if (constantsObj.ENV == constantsObj.STAGING) {
            REQUESTS_FETCH_INTERVAL = 10;
        }
    };

    function load() {
        var requestsData = ls.getItem(constantsObj.REQUESTS_DATA);
        if (requestsData == null || requestsData == "") {
            return false;
        }
        var requests = JSON.parse(requestsData);
        lastRequestsFetchTimeStamp = requests.lastRequestsFetchTimeStamp;
        isWordWarsFtueSeen = requests.isWordWarsFtueSeen;
        existingRequestsMap = requests.existingRequestsMap;
        lastOldRequestListString = requests.lastOldRequestListString;

        if (lastRequestsFetchTimeStamp == undefined) {
            lastRequestsFetchTimeStamp = 0;
        }
        if (isWordWarsFtueSeen == undefined) {
            isWordWarsFtueSeen = false;
        }
        if (existingRequestsMap == undefined) {
            existingRequestsMap = {};
        }
        if (lastOldRequestListString == undefined) {
            lastOldRequestListString = "";
        }
    };

    this.save = function () {
        var requests = {};
        requests.lastRequestsFetchTimeStamp = lastRequestsFetchTimeStamp;
        requests.isWordWarsFtueSeen = isWordWarsFtueSeen;
        requests.existingRequestsMap = existingRequestsMap;
        requests.lastOldRequestListString = lastOldRequestListString;
        ls.setItem(constantsObj.REQUESTS_DATA, utilObj.getStringifiedJson(requests));
    };

    this.syncToServer = function () {
        if (epmObj.getChosenVariant(xpromoConstantsObj.EXP_WORDWARS_REQUESTS) == xpromoConstantsObj.EXP_VAR_CONTROL) {
            return;
        }
        var curTimestamp = utilObj.getCurrentTimestamp();
        if (curTimestamp - lastRequestsFetchTimeStamp > REQUESTS_FETCH_INTERVAL) {
            requestsObj.getRequestsData();
        }
    };

    this.respHandler = function (action, data, success) {
        var curTimestamp = utilObj.getCurrentTimestamp();
        if (success == true) {
            if (action == ACTION_GET_REQUESTS_DATA) {
                lastRequestsFetchTimeStamp = curTimestamp;
                var requestsData = JSON.parse(data.r);
                var newRequestsList = requestsData.n;
                var newRequestsListLength = 0;

                // if old list length is 0, then we will first try to pick old list data from server (this can happen in case when user deletes app data)
                var oldListLength = Object.keys(existingRequestsMap).length;
                if (oldListLength == 0 && requestsData.o != "" && requestsData.o != undefined) {
                    requestsObj.storeDataInOldList(requestsData.o);
                }
                requestsObj.mergeRequestsData(requestsData.o);

                if (newRequestsList != "" && requestsData.n != undefined) {
                    newRequestsListLength = requestsObj.storeDataInOldList(requestsData.n);
                }
                requestsObj.updateRequestsData(newRequestsListLength);
            }
            requestsObj.save();
        }
    };

    this.getRequestsData = function () {
        var data = {};
        data.receiverAdId = xpromoUtilObj.getAdvertiserId();
        if (data.receiverAdId == "" || data.receiverAdId == null || data.receiverAdId == undefined) {
            return;
        }
        data.env = constantsObj.ENV;
        requestsController.sendHttpRequest(REQUESTS_CONTROLLER, ACTION_GET_REQUESTS_DATA, data);
    };

    this.updateRequestsData = function (newRequestsProcessed) {
        if (newRequestsProcessed == undefined) {
            newRequestsProcessed = 0;
        }
        var data = {};
        data.receiverAdId = xpromoUtilObj.getAdvertiserId();
        if (data.receiverAdId == "" || data.receiverAdId == null || data.receiverAdId == undefined) {
            return;
        }
        data.env = constantsObj.ENV;
        data.requestsProcessed = newRequestsProcessed.toString();
        data.oldRequestList = requestsObj.stringifyOldRequestsList();
        if (data.oldRequestList == "" || data.oldRequestList == requestsObj.getLastOldRequestListString()) {
            return;
        }
        requestsObj.setLastOldRequestListString(data.oldRequestList);
        requestsController.sendHttpRequest(REQUESTS_CONTROLLER, ACTION_UPDATE_REQUESTS_DATA, data);
    };

    this.storeDataInOldList = function (list) {
        if (list == undefined) {
            return;
        }
        var listDataNew = list, requestsProcessed = 0;
        listDataNew = listDataNew.split("_");
        var oldListLength = Object.keys(existingRequestsMap).length;
        for (var i = 0; i < listDataNew.length; i++) {
            var listData = listDataNew[i];
            if (listData == "") {
                continue;
            }
            listData = listData.split(":");
            if (existingRequestsMap[listData[6] + listData[9]] == undefined || existingRequestsMap[listData[6] + listData[9]] == {}) {
                var startTime = utilObj.getCurrentTimestamp();
                if (listData[14] != undefined && listData[14] != null && listData[14] != "") {
                    startTime = listData[14];
                }
                var timeLimit = xpromoGameRuntimeObj.getRequestTime() * constantsObj.SECONDS_IN_HOUR;
                var requestId = listData[6] + listData[9];
                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, listData[6], "1", xpromoConstantsObj.TRACK_WARS_QUEST, xpromoConstantsObj.TRACK_RECEIVED, "", listData[4], "", "");
                var req = {
                    // index field will be used in case we need to change some data at specific field within client, this field will not be stored on server
                    // in case we need a unique id for request we will use hash = senderRefId + receiverRefId
                    "arrInd": i + oldListLength, // array index
                    "id": requestId, // request Id
                    "qt": listData[0], // request type
                    "status": listData[1], // request status
                    "pushNotifFlag": listData[2],
                    "re": listData[3] / questsObj.getDollarValue(), // reward
                    "botType": listData[4], // bot type
                    "gh": listData[5], // sender game id
                    "senderRefId": listData[6],
                    "senderName": listData[7],
                    "senderScore": listData[8],
                    "receiverRefId": listData[9],
                    "gs": [listData[10]], // game shown, this is inserted in an array so as to keep functionality same as quest center
                    "gameBoard": listData[11],
                    "senderFbId": listData[12], // NA for bot
                    // extra params is added in case we need to pass more params otherwise it would be nil
                    "extraParams": listData[13],
                    // the time when the request was started
                    "startTime": startTime,
                    "timeLimit": timeLimit,
                    "params": [timeLimit, startTime],
                };
                existingRequestsMap[requestId] = req;
            }
            requestsProcessed += 1;
        }
        requestsObj.save();
        return requestsProcessed;
    };

    this.mergeRequestsData = function (list) {
        if (list == undefined) {
            return;
        }
        var listDataNew = list, requestsProcessed = 0;
        listDataNew = listDataNew.split("_");
        for (var i = 0; i < listDataNew.length; i++) {

            var listData = listDataNew[i];
            if (listData == "") {
                continue;
            }
            listData = listData.split(":");
            if (existingRequestsMap[listData[6] + listData[9]] != undefined && (existingRequestsMap[listData[6] + listData[9]].status == xpromoConstantsObj.REQUEST_STARTED || existingRequestsMap[listData[6] + listData[9]].status == xpromoConstantsObj.REQUEST_PLAYED)) {
                if (listData[1] == xpromoConstantsObj.REQUEST_COMPLETED) {
                    requestsObj.updateRequestStatus(existingRequestsMap[listData[6] + listData[9]], listData[1]);
                }
            } else {
                if (listData[1] == xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START) {
                    requestsObj.updateRequestStatus(existingRequestsMap[listData[6] + listData[9]], listData[1]);
                }
            }
        }
    };

    this.stringifyOldRequestsList = function () {
        var oldRequestsListDataArr = requestsObj.sortMapOnStartTime();
        var oldRequestsListString = "";
        for (var i = 0; i < oldRequestsListDataArr.length; i++) {
            var oldRequestsListData = oldRequestsListDataArr[i];
            // if the quest is already claimed then we dont store its data as it would be useless
            if (oldRequestsListData == undefined || oldRequestsListData.status == xpromoConstantsObj.REQUEST_CLAIMED || oldRequestsListData.status == xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START) {
                continue;
            }
            oldRequestsListString += oldRequestsListData.qt + ":" + oldRequestsListData.status + ":" + oldRequestsListData.pushNotifFlag + ":" +
                (oldRequestsListData.re * questsObj.getDollarValue()) + ":" + oldRequestsListData.botType + ":" + oldRequestsListData.gh + ":" +
                oldRequestsListData.senderRefId + ":" + oldRequestsListData.senderName + ":" + oldRequestsListData.senderScore + ":" +
                oldRequestsListData.receiverRefId + ":" + oldRequestsListData.gs[0] + ":" + oldRequestsListData.gameBoard + ":"
                + oldRequestsListData.senderFbId + ":" + oldRequestsListData.extraParams + ":" + oldRequestsListData.startTime + "_";
        }
        return oldRequestsListString;
    };

    // we will consider only one request type at a time otherwise it can create issues between different requests having same request type.
    // this.getRequestMap = function (requestType) {
    //     loggerObj.debugLog("this is one requests data all ", utilObj.getStringifiedJson(oldRequestsList));
    //     for (var i = 0; i<oldRequestsList.length; i++) {
    //         loggerObj.debugLog("this is quest type ", oldRequestsList[i].qt, requestType);
    //         if (requestType == oldRequestsList[i].qt){
    //             return oldRequestsList[i];
    //         }
    //     }
    // };

    // we will consider only one request type at a time otherwise it can create issues between different requests having same request type.
    this.updateRequestMap = function (requestData) {
        existingRequestsMap[requestData.senderRefId + requestData.receiverRefId] = requestData;
    };

    this.getIsWordWarsFtueSeen = function () {
        return isWordWarsFtueSeen;
    };

    this.setIsWordWarsFtueSeen = function (val) {
        isWordWarsFtueSeen = val;
    };

    this.redirectToStore = function (campaignName, screenName, params) {
        var link, installedAlready = 0;
        questsObj.resetRedirectDataForAdjust();
        var redirectExp = epmObj.getChosenVariant(xpromoConstantsObj.EXP_REDIRECT_TO_APP_STORE);
        if (utilObj.isAndroid()) {
            if (xpromoUtilObj.isAppInstalled(params["adrAppId"])) {
                installedAlready = 1;
                utilObj.launchAnotherApp(params["adrAppId"]);
            }
            // link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
            // utilObj.openStore(null, link, screenName);
            else if (redirectExp == constantsObj.EXP_VAR_CONTROL) {
                link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
                utilObj.openStore(null, link, screenName);
            } else {
                //xpromoUtilObj.hitUrl(xpromoConstantsObj.trackAndroidMap[params["gameId"]], campaignName);
                utilObj.openUrl("market://details?id=" + params["adrAppId"]);
            }
        } else if (utilObj.isIos()) {
            if (xpromoUtilObj.isAppInstalled(params["fbId"])) {
                installedAlready = 1;
                utilObj.launchAnotherApp(params["fbId"]);
            } else if (redirectExp == constantsObj.EXP_VAR_CONTROL) {
                link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
                utilObj.openStore(null, link, screenName);
            } else if (redirectExp == constantsObj.EXP_VAR_1) {
                //xpromoUtilObj.hitUrl(xpromoConstantsObj.trackIosMap[params["game_id"]], campaignName);
                utilObj.openUrl("itms-apps://itunes.apple.com/app/id" + params["iosAppId"]);
            } else if (redirectExp == constantsObj.EXP_VAR_2) {
                //xpromoUtilObj.hitUrl(xpromoConstantsObj.trackIosMap[params["game_id"]], campaignName);
                link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
                utilObj.openStore(params["iosAppId"], link, screenName);
            }

            // xpromoUtilObj.hitUrl(trackIosMap[params["gameId"]], campaignName);
            // link = questsObj.getGameShareLink(params["gameLink"], screenName, campaignName);
            // utilObj.openStore(params["appId"], link, screenName);

            // if (epmObj.getChosenVariant(xpromoConstantsObj.EXP_REDIRECT_TO_APP_STORE) == constantsObj.EXP_VAR_CONTROL) {
            //     xpromoUtilObj.hitUrl(moreGamesLayer.trackAndroidMap[game_id], campaignName);
            // }
        }
    };

    this.getCurrentActiveRequest = function () {
        var expVar = epmObj.getChosenVariant(xpromoConstantsObj.EXP_WORDWARS_REQUESTS);
        if (expVar == constantsObj.EXP_VAR_CONTROL) {
            return [];
        }
        if (xpromoUtilObj.isWebStaging()) {
            var request_data = qaConfigObj.getQAConfigData("wordwars_request");
            if (request_data != "" && request_data != -1) {
                return [request_data];
            }
        }
        xPromoGameSpecificObj.checkInstalledPlaysimpleGames();
        var currentReq = [];
        // loggerObj.debugLog("this is orl ", utilObj.getStringifiedJson(oldRequestsList));
        var sortedRequestArr = requestsObj.sortMapOnStartTime();
        for (var i = 0; i < sortedRequestArr.length; i++) {
            if (sortedRequestArr[i] == undefined) {
                continue;
            }
            if (utilObj.getCurrentTimestamp() - sortedRequestArr[i].params[1] >= sortedRequestArr[i].timeLimit) {
                requestsObj.updateRequestStatus(sortedRequestArr[i], xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START);
                continue;
            }
            if ((sortedRequestArr[i].status == xpromoConstantsObj.REQUEST_NOT_STARTED) && xPromoGameSpecificObj.checkIfAppInstalledWithGameId(sortedRequestArr[i].gh)) {
                requestsObj.updateRequestStatus(sortedRequestArr[i], xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START);
                continue;
            }
            if (sortedRequestArr[i].status == xpromoConstantsObj.QUEST_COMPLETED && sortedRequestArr[i].re == 0) {
                sortedRequestArr[i].status = xpromoConstantsObj.QUEST_CLAIMED;
                continue;
            }
            if (sortedRequestArr[i].status != xpromoConstantsObj.REQUEST_CLAIMED && sortedRequestArr[i].status != xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START) {
                var requestDesc = stringsObj.request_message_word_wars;
                requestDesc = requestDesc.replace("{name}", sortedRequestArr[i].senderName.split(" ")[0]);
                var request = sortedRequestArr[i];
                request.d = requestDesc;
                request.b = 1;
                request.qp = 0;
                request.p = 1;
                request.isRequest = 1;
                request.trackingId = xpromoConstantsObj.TRACK_WARS_QUEST + "_" + expVar;
                currentReq.push(request);
                break;
            }
        }
        return currentReq;
    };

    this.sortMapOnStartTime = function () {
        var sortedRequestArr = [];
        for (var request in existingRequestsMap) {
            sortedRequestArr.push(existingRequestsMap[request]);
        }

        sortedRequestArr.sort(function (a, b) {
            return a.startTime - b.startTime;
        });
        return sortedRequestArr;
    };

    this.updateRequestStatus = function (requestData, newState) {
        if (existingRequestsMap[requestData.senderRefId + requestData.receiverRefId] == undefined) {
            return;
        }
        var requestId = requestData.senderRefId + requestData.receiverRefId;
        var curRequestStatus = existingRequestsMap[requestId].status;

        if (newState == xpromoConstantsObj.QUEST_COMPLETED && requestData.re == 0) {
            newState = xpromoConstantsObj.QUEST_CLAIMED;
        }

        if (newState > curRequestStatus) {
            requestData.status = newState;
            existingRequestsMap[requestId].status = newState;
            if (newState == xpromoConstantsObj.REQUEST_COMPLETED) {
                questsObj.addQuestsToBeAnimated(requestId);
            }
            if (newState == xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START || newState == xpromoConstantsObj.REQUEST_CLAIMED) {
                existingRequestsMap[requestId] = undefined;
            }
        } else if (curRequestStatus == xpromoConstantsObj.REQUEST_PLAYED && (newState == xpromoConstantsObj.REQUEST_COMPLETED || newState == xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START || newState == xpromoConstantsObj.REQUEST_CLAIMED)) {
            requestData.status = newState;
            existingRequestsMap[requestId].status = newState;
            if (newState == xpromoConstantsObj.REQUEST_COMPLETED) {
                questsObj.addQuestsToBeAnimated(requestId);
            }
            if (newState == xpromoConstantsObj.REQUEST_COMPLETED_BEFORE_START || newState == xpromoConstantsObj.REQUEST_CLAIMED) {
                existingRequestsMap[requestId] = undefined;
            }
        }
        requestsObj.updateRequestsData(0);
    };

    this.claimRequestReward = function (requestId) {
        var requestData = existingRequestsMap[requestId];
        if (!utilObj.isOnline()) {
            utilObj.showMessage(stringsObj.not_online);
            return;
        }
        if (requestData == undefined || requestData == null) {
            return;
        }
        if (requestData.status != xpromoConstantsObj.QUEST_COMPLETED) {
            utilObj.showMessage(stringsObj.error_generic);
            return;
        }

        var cashReward = Math.round(requestData.re * questsObj.getDollarValue());
        trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, requestData.senderRefId, "1", xpromoConstantsObj.TRACK_WARS_QUEST, constantsObj.TRACK_CLAIM, "", requestData.botType, requestsObj.getTrackingIdForWordwarsRequest(), "");
        var isSuccessful = economyObj.addCash(cashReward, constantsObj.TRACK_EARN, xpromoConstantsObj.TRACK_QUEST_CENTER, requestId + "");
        xPromoGameSpecificObj.fireEarnTracking(false, constantsObj.TRACK_QUEST_CENTER, constantsObj.TRACK_COINS, xPromoGameSpecificObj.getCashEquivalent() + "", cashReward + "", requestId + "", isSuccessful);

        var requestRelatedQuest = requestData.r.split(":");
        requestsObj.updateRequestStatus(requestData, xpromoConstantsObj.REQUEST_CLAIMED);
        for (var relatedQuest in requestRelatedQuest) {
            var rq = requestRelatedQuest[relatedQuest];
            if (rq != "") {
                questsObj.updateCrossGameQuestStatus(rq, xpromoConstantsObj.QUEST_CLAIMED);
            }
        }
    };

    this.getTrackingIdForWordwarsRequest = function () {
        var expVar = epmObj.getChosenVariant(xpromoConstantsObj.EXP_WORDWARS_REQUESTS);
        return xpromoConstantsObj.TRACK_WARS_QUEST + "_" + expVar;
    };

    this.getImageLinkFromName = function (isBot, gameIconName, senderFbId) {
        if (!utilObj.isWeb()) {
            return utilObj.getAbsFilePath(gameIconName);
        }
        var gameIconPath = "";
        if (isBot == 1) {
            gameIconPath = constantsObj.MISC_SERVER_STATIC_BASE + "wordwars/Images/botProfiles/" + gameIconName;

        } else {
            gameIconPath = friendsObj.getFbPicUrl(senderFbId);
        }
        return gameIconPath;
    };
};