PspnUserTracking = function () {

    // User State Snapshot
    var fbConnectStatus = false;
    var payerStatus = false;
    var latStatus = false;
    var musicStatus = false;
    var daysSinceInstall = 0;
    var daysSinceLastPlayed = 0;
    var iosNotifPermission = false;
    var contactBookPermission = false;
    var daysSinceLastPurchase = 0;
    var daysSinceLastVideoWatch = 0;
    var daysSinceOfferwallView = 0;
    var daysSinceXpromoInstall = 0;
    var daysConsecutivePlay = 0;
    var subscriptionStatus = 0;
    var gameScore = 0;

    // Game State Snapshot
    var playerLevel = 0;
    var walletBalance = 0;
    var spinsAvailable = 0;
    var dailyContentStatus = -1;
    var videoAvailable = false;
    var hintsAvailable = false;
    var leaderboardPosition = -1;
    var leaderboardTimeSpent = 0;
    var daysPlayed = 0;
    var totalTimeSpent = 0;

    // Cross Game User State Snapshot
    var daysSinceOtherPlayed = 0;
    var psGamesInstalled = 0;
    var psGamesUninstalled = 0; //This is currently not being tracked and will always be set to 0
    var daysSinceOtherInstall = 0;

    // Cross Game Social State Snapshot
    var friendsInGame = 0;
    var friendsInPS = 0; //This is currently not being tracked and will always be set to 0
    var askFriends = 0;
    var inviteFriends = 0;

    // Cached tracking state for the session
    var trackingEnabled = -1;

    // Separator
    var separator = "_";
    var version = "v1";
    var end = "_e";

    var EXP_NAME = "PSPNIntTracking";

    var updateUserStateSnapshot = function() {
        fbConnectStatus = xPromoGameSpecificObj.isLoggedIn();
        payerStatus = xPromoGameSpecificObj.getIsPayer();
        latStatus = xPromoGameSpecificObj.isLimitTrackingWithoutAdjustId();
        musicStatus = xPromoGameSpecificObj.isMusicEnabled();
        daysSinceInstall = machineLearningDataObj.getDaysSinceInstall();

        if(xPromoGameSpecificObj.getPrevDay() > 0) {
            daysSinceLastPlayed = utilObj.getDayOfYear() - xPromoGameSpecificObj.getPrevDay();
            if(daysSinceLastPlayed < 1) {
                daysSinceLastPlayed += 365;
            }
        }

        iosNotifPermission = xPromoGameSpecificObj.getNotifPermissionForIos();
        contactBookPermission = xPromoGameSpecificObj.getContactBookPermission();

        var currentTimestamp = utilObj.getCurrentTimestamp();

        daysSinceLastPurchase = -1;
        var lastPurchaseDate = xPromoGameSpecificObj.getLastPurchaseDate();
        if(lastPurchaseDate != undefined && lastPurchaseDate != 0) {
            daysSinceLastPurchase = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(lastPurchaseDate, currentTimestamp));
        }

        daysSinceLastVideoWatch = -1;
        var lastVideoDate = machineLearningDataObj.getDayVideoViewed();
        if(lastVideoDate != -1) {
            daysSinceLastVideoWatch = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(lastVideoDate, currentTimestamp));
        }

        daysSinceOfferwallView = -1;
        var lastOfferwallDate = machineLearningDataObj.getDayOfferwallViewed();
        if(lastOfferwallDate != -1) {
            daysSinceOfferwallView = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(lastOfferwallDate, currentTimestamp));
        }

        daysSinceXpromoInstall = -1;
        var lastXpromoDate = machineLearningDataObj.getDayXpromoInstalled();
        if(lastXpromoDate != -1) {
            daysSinceXpromoInstall = Math.round(xpromoUtilObj.getDaysDiffBetweenTimestamps(lastXpromoDate, currentTimestamp));
        }

        daysConsecutivePlay = xPromoGameSpecificObj.getConsecutiveLogins();
        subscriptionStatus = xPromoGameSpecificObj.checkSubscriptionStatus();
        gameScore = xPromoGameSpecificObj.getGameScore();


    };

    var updateGameStateSnapshot = function() {
        playerLevel = xPromoGameSpecificObj.getPuzCompleted();
        walletBalance = xPromoGameSpecificObj.getCash();
        spinsAvailable = xPromoGameSpecificObj.getSpinsAvailable();
        dailyContentStatus = xPromoGameSpecificObj.getDailyContentStatus();
        videoAvailable = xpromoUtilObj.isVideoAvailable();
        hintsAvailable = xPromoGameSpecificObj.getNumFreeHints();
        leaderboardPosition = xPromoGameSpecificObj.getEventRank();
        leaderboardTimeSpent = machineLearningDataObj.getTotalLBSessionTime();
        daysPlayed = machineLearningDataObj.getDaysPlayed();
        totalTimeSpent = machineLearningDataObj.getTotalSessionTime();

        var sinceResume = analyticsObj.timeSinceResume();
        if(sinceResume > 0) {
            totalTimeSpent += sinceResume;
            if(leaderboardPosition != -1) {
                leaderboardTimeSpent += sinceResume;
            }
        }

        totalTimeSpent = Math.ceil(totalTimeSpent / constantsObj.SECONDS_IN_MIN);
        leaderboardTimeSpent = Math.ceil(leaderboardTimeSpent / constantsObj.SECONDS_IN_MIN);
    };

    var updateXGameUserStateSnapshot = function() {
        daysSinceOtherPlayed = Math.round(xpromoUtilObj.daysSincePlayedOtherGame());
        psGamesInstalled = xpromoUtilObj.getInstalledPSGames().length;
        psGamesUninstalled = 0;
        daysSinceOtherInstall = Math.round(xpromoUtilObj.daysSinceInstalledOtherGame());
    };

    var updateXGameSocialStateSnapshot = function() {
        friendsInGame = (friendsObj.getPlayingFriends() != undefined) ? friendsObj.getPlayingFriends().length : 0;
        friendsInPS = 0;
        askFriends = machineLearningDataObj.getTotalAsks();
        inviteFriends = machineLearningDataObj.getTotalInvites();
    };

    var processInput = function(data) {
        if(typeof data == "boolean") {
            return data ? "1" : "0";
        } else if (typeof data == "number") {
            if(isNaN(data)) {
                data = -1;
            }
            return data.toString();
        }

        if (typeof data != "string") {
            loggerObj.debugLog("Incorrect data type specified in processInput type=" + typeof data);
            return "0";
        }
        return data;
    };

    var getUserStateTrackingString = function() {
        updateUserStateSnapshot();

        var result = "";
        result += processInput(fbConnectStatus) + separator;
        result += processInput(payerStatus) + separator;
        result += processInput(latStatus) + separator;
        result += processInput(musicStatus) + separator;
        result += processInput(daysSinceInstall) + separator;
        result += processInput(daysSinceLastPlayed) + separator;
        result += processInput(iosNotifPermission) + separator;
        result += processInput(contactBookPermission) + separator;
        result += processInput(daysSinceLastPurchase) + separator;
        result += processInput(daysSinceLastVideoWatch) + separator;
        result += processInput(daysSinceOfferwallView) + separator;
        result += processInput(daysSinceXpromoInstall) + separator;
        result += processInput(daysConsecutivePlay) + separator;
        result += processInput(subscriptionStatus) + separator;
        result += processInput(gameScore);
        return result;
    };

    var getGameStateTrackingString = function() {
        updateGameStateSnapshot();

        var result = "";
        result += processInput(playerLevel) + separator;
        result += processInput(walletBalance) + separator;
        result += processInput(spinsAvailable) + separator;
        result += processInput(dailyContentStatus) + separator;
        result += processInput(videoAvailable) + separator;
        result += processInput(hintsAvailable) + separator;
        result += processInput(leaderboardPosition) + separator;
        result += processInput(leaderboardTimeSpent) + separator;
        result += processInput(daysPlayed) + separator;
        result += processInput(totalTimeSpent);
        return result;
    };

    var getXGameUserStateTrackingString = function() {
        updateXGameUserStateSnapshot();

        var result = "";
        result += processInput(daysSinceOtherPlayed) + separator;
        result += processInput(psGamesInstalled) + separator;
        result += processInput(psGamesUninstalled) + separator;
        result += processInput(daysSinceOtherInstall);
        return result;
    };

    var getXGameSocialStateTrackingString = function() {
        updateXGameSocialStateSnapshot();

        var result = "";
        result += processInput(friendsInGame) + separator;
        result += processInput(friendsInPS) + separator;
        result += processInput(askFriends) + separator;
        result += processInput(inviteFriends);
        return result;
    };



    this.isTrackingEnabled = function() {
        if(trackingEnabled == -1) {
            if(xPromoGameSpecificObj.isUserInteractionTrackingEnabled()) {
                trackingEnabled = epmObj.getChosenVariant(EXP_NAME);
            } else {
                trackingEnabled = constantsObj.EXP_VAR_CONTROL;
            }
        }
        return (trackingEnabled == constantsObj.EXP_VAR_1);
    };

    this.trackUserSnapshot = function(screen, event) {
        if(!pspnUserTracking.isTrackingEnabled()) {
            return false;
        }

        var src = utilObj.getScreenName(screen);

        var views = 0;
        switch (screen) {
            case constantsObj.SCREEN_XPROMO_W2E:
                views = machineLearningDataObj.getUserOpenedXpromoW2e();
                break;
            case constantsObj.SCREEN_QUEST_CENTER:
                views = machineLearningDataObj.getUserOpenedQuestCenter();
                break;
            case constantsObj.SCREEN_MORE_GAMES_LAYER:
                views = machineLearningDataObj.getUserOpenedAdwall();
                break;
            case constantsObj.SCREEN_WORD_WARS_REQUEST:
                views = machineLearningDataObj.getUserOpenedWarsRequest();
                break;
        }

        var userTracking = version + separator + getUserStateTrackingString() + end;
        var gameTracking = version + separator + getGameStateTrackingString() + end;
        var xgameStateTracking = version + separator + getXGameUserStateTrackingString() +
            separator + getXGameSocialStateTrackingString() + end;

        trackObj.trackCounter(
            xpromoConstantsObj.TRACK_XPROMO_INTERACTION, //k
            views, //p
            userTracking, //c
            src, //o
            gameTracking, //f
            xgameStateTracking, //g
            event, //s
            "", ""); //co, appId
    }

    this.trackBehaviourStatus = function() {
        if(!pspnUserTracking.isTrackingEnabled()) {
            return false;
        }

        var userBehaviour1Day = version + separator + machineLearningDataObj.getUserBehaviourTrackingString(separator, 1) + end; // Data for 1 days
        var xgameBehaviour1Day = version + separator + machineLearningDataObj.getXGameBehaviourTrackingString(separator, 1) + end; // Data for 1 days

        trackObj.trackCounter(
            xpromoConstantsObj.TRACK_XPROMO_INTERACTION, //k
            "1", //p
            userBehaviour1Day, //c
            "game_start", //o
            xgameBehaviour1Day, //f
            "", //g
            "start", //s
            "", ""); //co, appId


        var userBehaviour7Day = version + separator + machineLearningDataObj.getUserBehaviourTrackingString(separator, 7) + end; // Data for 7 days
        var xgameBehaviour7Day = version + separator + machineLearningDataObj.getXGameBehaviourTrackingString(separator, 7) + end; // Data for 7 days

        trackObj.trackCounter(
            xpromoConstantsObj.TRACK_XPROMO_INTERACTION, //k
            "7", //p
            userBehaviour7Day, //c
            "game_start", //o
            xgameBehaviour7Day, //f
            "", //g
            "start", //s
            "", ""); //co, appId

        utilObj.delayedCallback(gameLayer, adWallObj.trackAndStoreCompetitorGames, 0.5, null, false);
    }

};