XpromoGameRuntime = function () {
    var dailyQuestsRuntimes = new DailyQuestsRuntime();
    var adwallVariables = new AdwallVars();
    var softLaunchRuntime = 2;
    var socialQuestRuntime = 5;
    var socialQuestString = new SocialQuestString();
    var requestRuntimes = new RequestsRuntimes();
    var requestRedirectionTime = 2;
    var wordwarsRequestRedirectText = new WordwarsRequestRedirectText();

    this.init = function () {

    };

    this.loadRuntime = function (runtimeData) {
        if (runtimeData.dqr != undefined) {
            dailyQuestsRuntimes.load(runtimeData.dqr);
        }
        if (runtimeData.awr != undefined) {
            adwallVariables.load(runtimeData.awr);
        }
        if (runtimeData.sqs != undefined) {
            socialQuestString.load(runtimeData.sqs);
        }
        if (runtimeData.sqr != undefined) {
            socialQuestRuntime = runtimeData.sqr;
        }
        if (runtimeData.slr != undefined) {
            softLaunchRuntime = runtimeData.slr;
        }
        if (runtimeData.reqr != undefined) {
            runtimeData.reqr = runtimeData.reqr + "";
            requestRuntimes.load(runtimeData.reqr);
        }
        if (runtimeData.requestRedirectionTime != undefined) {
            requestRedirectionTime = runtimeData.requestRedirectionTime;
        }
        if (runtimeData.wrrt1 != undefined) {
            wordwarsRequestRedirectText.load(runtimeData.wrrt1);
        }
        runtimeData.wrrt1 = wordwarsRequestRedirectText.getWordwarsRedirectionTextString();

    };

    this.saveRuntime = function (runtimeData) {
        runtimeData.dqr = dailyQuestsRuntimes.getDailyQuestsRuntimeData();
        runtimeData.awr = adwallVariables.getAdwallData();
        runtimeData.sqs = socialQuestString.getSocialQuestStringSave();
        runtimeData.sqr = socialQuestRuntime;
        runtimeData.slr = softLaunchRuntime;
        runtimeData.reqr = requestRuntimes.getRequestTime() + ":" + requestRuntimes.getRequestNotifDays();
        runtimeData.requestRedirectionTime = requestRedirectionTime;

        return runtimeData;
    };
    
    this.respHandlerLoad = function (vals) {
        dailyQuestsRuntimes.load(vals['dqr']);
        adwallVariables.load(vals['awr']);
        socialQuestString.load(vals['sqs']);
        socialQuestRuntime = parseInt(vals['sqr']);
        softLaunchRuntime = parseInt(vals['slr']);
        requestRuntimes.load(vals['reqr']);
        requestRedirectionTime = parseInt(vals['rrt']);
        wordwarsRequestRedirectText.load(vals['wrrt1']);

    };

    this.getDailyQuestsDaysPayer = function () {
        return dailyQuestsRuntimes.getDailyQuestsPayerDays();
    };

    this.getDailyQuestsDaysNonPayer = function () {
        return dailyQuestsRuntimes.getDailyQuestsNonPayerDays();
    };

    this.getAdWallDays = function () {
        var adwallDays = adwallVariables.getAdWallDays();
        if (utilObj.isIos()) {
            adwallDays = adwallVariables.getAdWallDaysIos();
        }
        return adwallDays;
    };

    this.getAdwallLimit = function () {
        var adwallLimit = adwallVariables.getAdwallLimit();
        if (utilObj.isIos()) {
            adwallLimit = adwallVariables.getAdwallLimitIos();
        }
        return adwallLimit;
    };

    this.getAdwallCooldownTime = function () {
        var adwallCooldownTime = adwallVariables.getAdwallCooldownTime();
        if (utilObj.isIos()) {
            adwallCooldownTime = adwallVariables.getAdwallCooldownTimeIos();
        }
        return adwallCooldownTime;
    };

    this.getSocialQuestString = function () {
        return socialQuestString.getSocialQuestString();
    };

    this.getSocialQuestStringExp1 = function () {
        return socialQuestString.getSocialQuestStringExp1();
    };

    this.getSocialQuestStringExp2 = function () {
        return socialQuestString.getSocialQuestStringExp2();
    };

    this.getSocialQuestRuntime = function () {
        return socialQuestRuntime;
    };
    this.getSoftLaunchRuntime = function () {
        return softLaunchRuntime;
    };
    this.getRequestTime = function () {
        return requestRuntimes.getRequestTime();
    };
    this.getRequestNotifDays = function () {
        return requestRuntimes.getRequestNotifDays();
    };
    this.getRequestRedirectionTime = function () {
        return requestRedirectionTime;
    };
    this.getWordwarsRedirectionText = function () {
        return wordwarsRequestRedirectText.getWordwarsRedirectionText();
    };
};

DailyQuestsRuntime = function () {
    var dailyQuestsPayer = 2, dailyQuestsNonPayer = 2;
    this.load = function (data) {
        if (data == "" || data == undefined) {
            return;
        }
        var info = data.split(":");
        dailyQuestsPayer = parseInt(info[0]);
        dailyQuestsNonPayer = parseInt(info[1]);

    };

    this.getDailyQuestsRuntimeData = function () {
        var str = dailyQuestsPayer + ":" + dailyQuestsNonPayer + "";
        return str;
    };

    this.getDailyQuestsPayerDays = function () {
        return parseInt(dailyQuestsPayer);
    };

    this.getDailyQuestsNonPayerDays = function () {
        return parseInt(dailyQuestsNonPayer);
    };

};

AdwallVars = function () {
    var adwallCooldownTime = 30, adwallLimit = 5, adwallDays = 2, adwallCooldownTimeIos = 30, adwallLimitIos = 5,
        adwallDaysIos = 2;
    this.load = function (data) {
        if (data == "" || data == undefined) {
            return;
        }
        var info = data.split(":");
        adwallCooldownTime = parseFloat(info[0]);
        adwallLimit = parseFloat(info[1]);
        adwallDays = parseFloat(info[2]);
        adwallCooldownTimeIos = parseFloat(info[3]);
        adwallLimitIos = parseFloat(info[4]);
        adwallDaysIos = parseFloat(info[5]);
    };

    this.getAdwallData = function () {
        var str = adwallCooldownTime + ":" + adwallLimit + ":" + adwallDays + ":" + adwallCooldownTimeIos + ":" + adwallLimitIos + ":" + adwallDaysIos + "";
        return str;
    };

    this.getAdwallCooldownTime = function () {
        return adwallCooldownTime;
    };

    this.getAdwallLimit = function () {
        return adwallLimit;
    };

    this.getAdWallDays = function () {
        return adwallDays;
    };

    this.getAdwallCooldownTimeIos = function () {
        return adwallCooldownTimeIos;
    };

    this.getAdwallLimitIos = function () {
        return adwallLimitIos;
    };

    this.getAdWallDaysIos = function () {
        return adwallDaysIos;
    };
};
SocialQuestString = function () {
    var socialQuestStringExp1 = "%name% is playing %game%", socialQuestStringExp2 = "%game% is being played by %name%";
    this.load = function (data) {
        if (data == "" || data == undefined) {
            return;
        }
        var info = data.split(":");
        socialQuestStringExp1 = info[0];
        socialQuestStringExp2 = info[1];
    };

    this.getSocialQuestString = function () {
        var exp = epmObj.getChosenVariant(xpromoConstantsObj.EXP_QUEST_INVITE);
        if (exp == constantsObj.EXP_VAR_1) {
            return socialQuestStringExp1;

        } else {
            return socialQuestStringExp2;
        }
    };

    this.getSocialQuestStringExp1 = function () {
        return socialQuestStringExp1;
    };

    this.getSocialQuestStringExp2 = function () {
        return socialQuestStringExp2;
    };

    this.getSocialQuestStringSave = function () {
        return socialQuestStringExp1 + ":" + socialQuestStringExp2;
    };
};
RequestsRuntimes = function () {
    var requestTime = 24, requestNotifDays = 3;
    this.load = function (data) {
        if (data == "" || data == undefined) {
            return;
        }
        var info = data.split(":");
        requestTime = parseInt(info[0]);
        requestNotifDays = parseInt(info[1]);
    };
    this.getRequestTime = function () {
        return requestTime;
    };
    this.getRequestNotifDays = function () {
        return requestNotifDays;
    };
};
WordwarsRequestRedirectText = function () {
    var wordwarsRedirectionText1 = "Well done. Continue the game on Word Wars.",
        wordwarsRedirectionText2 = "Let's continue this game on Word wars.",
        wordwarsRedirectionText3 = "Let's continue this game on Word wars.";
    this.load = function (data) {
        if (data == "" || data == undefined) {
            return;
        }
        var info = data.split(":");
        wordwarsRedirectionText1 = info[0];
        wordwarsRedirectionText2 = info[1];
        wordwarsRedirectionText3 = info[2];
    };

    this.getWordwarsRedirectionText = function () {
        var expVar = epmObj.getChosenVariant(xpromoConstantsObj.EXP_REQUESTS_REDIRECTION);
        if (expVar == constantsObj.EXP_VAR_CONTROL) {
            return wordwarsRedirectionText1;
        } else if (expVar == constantsObj.EXP_VAR_1) {
            return wordwarsRedirectionText2;
        } else {
            return wordwarsRedirectionText3;
        }
    };

    this.getWordwarsRedirectionTextString = function () {
        return wordwarsRedirectionText1 + ":" + wordwarsRedirectionText2 + ":" + wordwarsRedirectionText3;
    };
};
