var MoreGamesLayer = cc.Layer.extend({
    images: [],
    imageDots: [],
    curImage: 0,
    lastImage: 0,
    touchStartX: null,
    touchEndX: null,
    gameMap: {},
    gameMapWithId: {},
    installQuestData: {},
    carouselMap: {},
    carouselSeqRepeat: null,
    prevLayer: null,
    trackAndroidMap: {},
    trackIosMap: {},
    animRunning: false,
    carouselShown: false,
    recommendationScrollView: null,
    recommendationTobBarConfig: null,
    incentiveAdWall: false,


    /**
     * Main wrapper method to enable more games layer to be shown
     * @param src - source of invocation as a string
     */
    show: function (src) {
        moreGamesLayer.images = [];
        moreGamesLayer.imageDots = [];
        moreGamesLayer.curImage = 0;
        moreGamesLayer.touchStartX = null;
        moreGamesLayer.touchEndX = null;
        moreGamesLayer.gameMap = {};
        moreGamesLayer.gameMapWithId = {};
        moreGamesLayer.installQuestData = {};
        moreGamesLayer.carouselMap = {};
        moreGamesLayer.carouselSeqRepeat = null;
        moreGamesLayer.carouselShown = false;
        var screen = constantsObj.SCREEN_MORE_GAMES_LAYER;
        xPromoGameSpecificObj.setCurScreen(screen);
        var screenName = utilObj.getScreenName(screen);
        utilObj.updateScreenBegin(screen);
        var spriteArr = [];
        var labelArr = [];
        moreGamesLayer.curImage = 0;
        moreGamesLayer.lastImage = 0;
        var zOrder = 1;

        var trackAndroidMap = {};
        var trackIosMap = {};

        moreGamesLayer.trackAndroidMap[4] = "cnglv7h";
        moreGamesLayer.trackAndroidMap[8] = "zc3zz6";
        moreGamesLayer.trackAndroidMap[10] = "jgrqiq";
        moreGamesLayer.trackAndroidMap[11] = "w0rvv7";
        moreGamesLayer.trackAndroidMap[12] = "ogum16";
        moreGamesLayer.trackAndroidMap[14] = "qfjq2r9";
        moreGamesLayer.trackAndroidMap[16] = "1smbcq4";

        moreGamesLayer.trackIosMap[4] = "lfzm5rr";
        moreGamesLayer.trackIosMap[8] = "zc3zz6";
        moreGamesLayer.trackIosMap[10] = "jgrqiq";
        moreGamesLayer.trackIosMap[11] = "w0rvv7";
        moreGamesLayer.trackIosMap[12] = "ogum16";
        moreGamesLayer.trackIosMap[14] = "qfjq2r9";
        moreGamesLayer.trackIosMap[16] = "1smbcq4";


        var bg = utilObj.getScale9Sprite(constantsObj.tile_white, centerXPos, 0, 0.5, 0, canvasWidth + xPromoGameSpecificObj.getWidthErr(), xPromoGameSpecificObj.getFullDesignHeight() + constantsObj.BEZEL_SAFE_AREA, spriteArr, moreGamesLayer, zOrder, constantsObj.SPRITE_MISC);
        var bgColor = cc.color(17, 29, 45);
        bg.setColor(bgColor);


        var adWallNewUI = adWallObj.getAdWallNewUIExperiment();
        var questData = questsObj.getAvailableCrossGameQuests();
        //loggerObj.debugLog("this is android quest data " + utilObj.getStringifiedJson(questData));
        for (var i = 0; i < questData.length; i++) {
            if (questData[i]["qt"] == xpromoConstantsObj.QUEST_TYPE_INSTALL_GAME) {
                var questDevice = [0, 0, 0, 0];
                for (var j = 0; j < questData[i]["cl"].length; j++) {
                    questDevice[parseInt(questData[i]["cl"][j])] = 1;
                }
                if (utilObj.getTrackingClient() == constantsObj.CLIENT_AMAZON && questDevice[2]) {
                    moreGamesLayer.installQuestData[questData[i]["gh"]] = questData[i];
                } else if (utilObj.getTrackingClient() != constantsObj.CLIENT_AMAZON && utilObj.isAndroid() && questDevice[0]) {
                    //loggerObj.debugLog("this is android quest " + utilObj.getStringifiedJson(questData[i]));
                    moreGamesLayer.installQuestData[questData[i]["gh"]] = questData[i];
                } else if (utilObj.getTrackingClient() != constantsObj.CLIENT_AMAZON && utilObj.isIos() && questDevice[1]) {
                    moreGamesLayer.installQuestData[questData[i]["gh"]] = questData[i];
                }
            }
        }
        xPromoGameSpecificObj.hideBannerAd(screen);

        moreGamesLayer.gameMap = questsObj.getAdWallQuests();
        //loggerObj.debugLog("this is gameMap", utilObj.getStringifiedJson(moreGamesLayer.gameMap));


        for (var game in moreGamesLayer.gameMap) {
            var gameData = moreGamesLayer.gameMap[game];
            var gameId = gameData["gh"];
            if (gameId == constantsObj.GAME_ID) {
                continue;
            }
            moreGamesLayer.gameMapWithId[gameData["gh"]] = gameData;
        }

        moreGamesLayer.incentiveAdWall = adWallObj.isEligibileForIncentivizedAdwall(src);
        if(moreGamesLayer.incentiveAdWall) {
            adWallObj.setIncentiveAdWallShown();
        }

        moreGamesLayer.populateBackToGame(spriteArr, labelArr, zOrder, screen);

        // Switch between icon view and scroll view for recommendations
        if(adWallNewUI == constantsObj.EXP_VAR_3) {
            moreGamesLayer.populateRecommendationsScroll(spriteArr, labelArr, zOrder, screen);
        } else {
            moreGamesLayer.populateRecommendations(spriteArr, labelArr, zOrder, screen);
        }

        utilObj.delayedCallback(moreGamesLayer, moreGamesLayer.showCarousel, 0.1, {centerXPos: centerXPos, zOrder: zOrder});

        utilObj.updateScreen(screen, moreGamesLayer, null, src);
        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);

        if(xPromoGameSpecificObj.isUserInteractionTrackingEnabled()) {
            machineLearningDataObj.incUserOpenedAdwall();
        }
        pspnUserTracking.trackUserSnapshot(constantsObj.SCREEN_MORE_GAMES_LAYER, constantsObj.TRACK_VIEW);
    },

    /**
     * This method is responsible for laying out the back to game button and adding it to the more games layer.
     * @param spriteArr - sprites to be added
     * @param labelArr - labels to be added
     * @param zOrder - base zOrder of the layer
     * @param screen - screen constant
     */
    populateBackToGame: function (spriteArr, labelArr, zOrder, screen) {

        var adWallNewUI = adWallObj.getAdWallNewUIExperiment();
        var isTopElement = true;
        if(adWallNewUI == constantsObj.EXP_VAR_4) {
            isTopElement = false;
        }

        var backToGameLayer = new cc.Node();
        var sectionHeight = moreGamesLayer.getBackToGameSectionHeight();
        var contentHeight = isTopElement ? sectionHeight + constantsObj.BEZEL_SAFE_AREA : sectionHeight;
        backToGameLayer.setContentSize(canvasWidth + xPromoGameSpecificObj.getWidthErr(), contentHeight);

        var header = utilObj.getScale9Sprite(constantsObj.tile_white, centerXPos, 0, 0.5, 0, canvasWidth + xPromoGameSpecificObj.getWidthErr(), contentHeight, spriteArr, backToGameLayer, zOrder, constantsObj.SPRITE_MISC);
        var darkBlue = cc.color(34, 63, 107);
        header.setColor(darkBlue);

        var childArr = [];
        childArr = xPromoGameSpecificObj.showAdWallHeader(spriteArr, labelArr, zOrder, backToGameLayer, sectionHeight);

        var transButtonWidth = canvasWidth;
        var transButtonHeight = contentHeight;
        var transButtonX = 0;
        var transButtonY = 0;
        var transButtonAnchorX = 0;
        var transButtonAnchorY = 0;
        if(adWallNewUI == constantsObj.EXP_VAR_2) {
            var close = xPromoGameSpecificObj.getCloseButton();
            var closeIconScale = constantsObj.SCALE_SPRITE_19;
            if(constantsObj.GAME_ID == 8 || constantsObj.GAME_ID == 17) {
                closeIconScale = constantsObj.SCALE_SPRITE_14;
            }
            var closeImg = utilObj.getSprite(close,  canvasWidth - 50, sectionHeight / 2, 1, 0.5, closeIconScale, spriteArr, backToGameLayer, zOrder, constantsObj.SPRITE_MISC);
            transButtonWidth = 100;
            transButtonHeight = 100;
            transButtonX = canvasWidth - 50;
            transButtonY = sectionHeight / 2;
            transButtonAnchorX = 1;
            transButtonAnchorY = 0.5;
            childArr.push(closeImg);
        } else {
            var gameIconName = "icon_" + constantsObj.GAME_ID + ".png";
            var gameLogo = utilObj.getSprite(xpromoUtilObj.getAbsFilePath(gameIconName), canvasWidth - 25, sectionHeight / 2, 1, 0.5, constantsObj.SCALE_SPRITE_14, spriteArr, backToGameLayer, zOrder, constantsObj.SPRITE_LOCAL);
            // game ICON
            gameLogo.setContentSize(270, 270);
        }

        var trans_btn = utilObj.getButton(transButtonWidth, transButtonHeight, transButtonAnchorX, transButtonAnchorY,
            transButtonX, transButtonY, constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'moreGamesLayer',
                fn: 'backToPuzzle',
                screen: screen,
                params: {"sound": constantsObj.button_tap_01, "sound_on": true},
                track: {
                    k: constantsObj.TRACK_DIALOG,
                    p: xPromoGameSpecificObj.getPhylumForTracking(),
                    c: xPromoGameSpecificObj.getClassForTracking(),
                    o: xpromoConstantsObj.TRACK_ADWALL,
                    f: constantsObj.TRACK_CLOSE,
                    g: constantsObj.TRACK_HOME,
                    s: "",
                    co: 1,
                    oppId: ""
                },
                child: childArr,
                feedBackType: constantsObj.ZOOMOUT_FEEDBACK
            }, false, ccui.Widget.PLIST_TEXTURE, constantsObj.SCALE_ONE);
        backToGameLayer.addChild(trans_btn, zOrder);

        var point = moreGamesLayer.getBackToGameSectionPositions();
        backToGameLayer.setPosition(point.x, point.y);
        backToGameLayer.setAnchorPoint(0.5, 0);
        utilObj.addChild(moreGamesLayer, backToGameLayer, zOrder+1);
    },

    /**
     * Method to add elements to the back to game section of the screen
     * @param spriteArr - sprites to be added
     * @param labelArr - labels to be added
     * @param zOrder - base zOrder of the layer
     * @param layerToAdd - layer to which the sprites and labels are to be added
     * @param sectionHeight - height of the layer to which this needs to be added
     * @returns {Array} - Array of nodes (sprites and labels) added
     */
    diffMoreGamesUi: function (spriteArr, labelArr, zOrder, layerToAdd, sectionHeight) {
        var childArr = [];
        var imageName = constantsObj.ps_logo;
        var psLogo = utilObj.getSprite(xpromoUtilObj.getAbsFilePath(imageName), 25, sectionHeight / 2, 0, 0.5, constantsObj.SCALE_SPRITE_14, spriteArr, layerToAdd, zOrder, constantsObj.SPRITE_LOCAL);
        var gameLabelYPos = sectionHeight / 2;
        var gameLabelFontSize = constantsObj.FONT_SFCOMPACT_MEDIUM_100;
        if(adWallObj.getAdWallNewUIExperiment() != constantsObj.EXP_VAR_2) {
            var backToGameBtn = utilObj.getScale9Sprite(constantsObj.button_normal, centerXPos, sectionHeight - 240, 0.5, 0.5, 600, 120, spriteArr, layerToAdd, zOrder, constantsObj.SPRITE_MISC);
            var backToGameLabel = utilObj.getBMFontLabel(stringsObj.back_to_game, constantsObj.FONT_SFCOMPACT_MEDIUM_60, constantsObj.COLOR_BLACK, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, sectionHeight - 240, 0.5, 0.5, labelArr, layerToAdd, zOrder);
            childArr = [backToGameBtn, backToGameLabel];
            gameLabelYPos = sectionHeight - 100;
            gameLabelFontSize = constantsObj.FONT_SFCOMPACT_MEDIUM_80;
        }

        // Specifically for DTC as the
        if(constantsObj.GAME_ID == 8 || constantsObj.GAME_ID == 17) {
            gameLabelFontSize = constantsObj.FONT_SFCOMPACT_REGULAR_70;
        }

        var gameLabel = utilObj.getBMFontLabel(xPromoGameSpecificObj.getGameName(), gameLabelFontSize, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, gameLabelYPos, 0.5, 0.5, labelArr, layerToAdd, zOrder);
        return childArr;
    },

    /**
     * Method to populate the recommended game icons for the player to click and install / play from.
     * @param spriteArr - prites to be added
     * @param labelArr - labels to be added
     * @param zOrder - base zOrder of the layer
     * @param screen - screen constant
     */
    populateRecommendations: function(spriteArr, labelArr, zOrder, screen) {

        var recommendationLayer = new cc.Node();
        var sectionHeight = moreGamesLayer.getRecommendedSectionHeight();
        recommendationLayer.setContentSize(canvasWidth + xPromoGameSpecificObj.getWidthErr(), sectionHeight);

        var adWallNewUI = adWallObj.getAdWallNewUIExperiment();
        var hideOtherGamesLabel = true;
        var removeGameLabels = false;
        var additionalYDisplacement = 0;

        // Need to hide the other games label and line-separator when not control and variant 4 of the new UI.
        if(adWallNewUI == constantsObj.EXP_VAR_CONTROL || adWallNewUI == constantsObj.EXP_VAR_4) {
            hideOtherGamesLabel = false;
        }

        // Incase of incentiveAdWall, lets not populate the "Recommended" label, rather add a ribbon and font incentivising installs.
        var amount = adWallObj.getAdwallIncentiveCoinAmount();
        if(moreGamesLayer.incentiveAdWall) {
            var banner = utilObj.getSprite(xpromoConstantsObj.bubble_popup_ribbon, canvasWidth / 2, sectionHeight - 150, 0.5, 0.5, constantsObj.SCALE_SPRITE_5, spriteArr, recommendationLayer, zOrder, xpromoConstantsObj.SPRITE_QUESTS);
            banner.setScaleX(constantsObj.SCALE_SPRITE_11);
            banner.setColor(xpromoConstantsObj.COLOR_PURPLE_BANNER);
            var desc = xpromoStringsObj.adwall_coins.replace('%VALUE%', amount.toString());
            var coinIconOffset = amount > 99 ? 575 : 560;
            var coinLabel = utilObj.getBMFontLabel(desc, constantsObj.FONT_SFCOMPACT_MEDIUM_70, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), canvasWidth / 2, sectionHeight - 150, 0.5, 0.5, labelArr, recommendationLayer, zOrder);
            var coinIcon = utilObj.getSprite(questConfigObj.getConfigData("icon_coins"), (canvasWidth / 2) - coinIconOffset , sectionHeight - 155, 0.5, 0.5, constantsObj.SCALE_SPRITE_10, spriteArr, recommendationLayer, zOrder, xpromoConstantsObj.SPRITE_QUESTS);
            removeGameLabels = true;
            additionalYDisplacement = 115;
        } else {
            var recomendLabel = utilObj.getBMFontLabel(stringsObj.recommended_for_you, constantsObj.FONT_SFCOMPACT_MEDIUM_70, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), 100, sectionHeight - 50, 0, 1, labelArr, recommendationLayer, zOrder);
        }

        var gameNumber = 0;
        var yDiff = 335 + additionalYDisplacement;
        var xDiff = 100;
        var trackAppType = xpromoConstantsObj.TRACK_RECOMMENDED;

        for (var game in moreGamesLayer.gameMap) {
            var gameData = moreGamesLayer.gameMap[game];
            var gameId = gameData["gh"];
            var questId = gameData["id"];
            if (gameId == constantsObj.GAME_ID) {
                continue;
            }

            // Not show games which are not eligible to be cross promoted in case of incentivised ad wall.
            if(moreGamesLayer.incentiveAdWall && !xpromoUtilObj.isGameEligibleForXpromo(gameId, questId)) {
                continue;
            }

            // hitting adjust view tracking
            var trackId = xpromoUtilObj.getAdjustTrackerForGameId(gameId); // moreGamesLayer.trackAndroidMap[gameId];
            var campaignName = "adwall_" + constantsObj.GAME_ID;
            xpromoUtilObj.hitAdjustViewTracking(trackId, campaignName);

            if (gameNumber / 3 == 1) {
                yDiff += hideOtherGamesLabel ? 520 : 670;
                yDiff -= additionalYDisplacement;
                xDiff = 100;
                if(!hideOtherGamesLabel) {
                    trackAppType = xpromoConstantsObj.TRACK_OTHER_APPS;
                }
            }
            var iconSprite = "#icon_" + gameId + ".png";
            var imageMap = adWallObj.getImagesForGameId(gameId);
            if(imageMap.icon != undefined) {
                iconSprite = xpromoUtilObj.getAbsFilePath(imageMap.icon);
            }

            var gameLogo = utilObj.getSprite(iconSprite, xDiff, sectionHeight - yDiff, 0, 0.5, constantsObj.SCALE_SPRITE_1, spriteArr, recommendationLayer, zOrder, constantsObj.SPRITE_LOCAL);
            gameLogo.setContentSize(206, 206);
            if(!removeGameLabels) {
                var gameLabel = utilObj.getBMFontLabel(gameData["d"], constantsObj.FONT_SFCOMPACT_REGULAR_60, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xDiff + 315 / 2, sectionHeight - yDiff - 220, 0.5, 0.5, labelArr, recommendationLayer, zOrder);
            }
            var trans_btn = utilObj.getButton(400, 400, 0, 0.5,
                xDiff, sectionHeight - yDiff, constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                    obj: 'moreGamesLayer',
                    fn: (moreGamesLayer.incentiveAdWall) ? 'actionOnIncentivisedBtn' : 'actionOnBtn',
                    screen: screen,
                    params: (moreGamesLayer.incentiveAdWall) ? {gameId: gameId, questId: questId, coins: amount} : gameId,
                    track: {
                        k: xpromoConstantsObj.TRACK_XPROMO,
                        p: xpromoConstantsObj.TRACK_ADWALL,
                        c: gameId,
                        o: xpromoConstantsObj.TRACK_ADWALL,
                        f: constantsObj.TRACK_CLICK,
                        g: trackAppType,
                        s: "",
                        co: 1,
                        oppId: ""
                    },
                }, false, ccui.Widget.PLIST_TEXTURE, constantsObj.SCALE_ONE);
            recommendationLayer.addChild(trans_btn, zOrder);
            gameNumber += 1;
            xDiff += centerXPos - 100 - 315 / 2;

            // Limiting the number of games to be populated to 6 as per UI space.
            if(gameNumber >= 6) {
                 break;
            }
        }

        if(!hideOtherGamesLabel) {
            var horizontalLine = utilObj.getScale9Sprite(constantsObj.tile_white, centerXPos, sectionHeight - 670, 0.5, 0.5, 1320, 4, spriteArr, recommendationLayer, zOrder, constantsObj.SPRITE_MISC);

            var otherGamesLabel = utilObj.getBMFontLabel(stringsObj.other_games, constantsObj.FONT_SFCOMPACT_MEDIUM_70, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), 100, sectionHeight - 50 - 670, 0, 1, labelArr, recommendationLayer, zOrder);
        }

        var point = moreGamesLayer.getRecommendedSectionPositions();
        recommendationLayer.setPosition(point.x, point.y);
        recommendationLayer.setAnchorPoint(0.5, 1);
        utilObj.addChild(moreGamesLayer, recommendationLayer, zOrder+1);
    },

    /**
     * Method to setup the configuration of the
     * @param sectionHeight
     * @param canvasWidth
     */
    setTabConfig: function(sectionHeight, canvasWidth) {
        moreGamesLayer.recommendationTobBarConfig = [{
            btn: xpromoConstantsObj.btn_toggle_01_off,
            btnSel: xpromoConstantsObj.btn_toggle_01_on,
            ele: null,
            eleSel: null,
            eleLabel: null,
            xPos: (canvasWidth / 2) - 367,
            yPos: sectionHeight - 225,
            label: xpromoStringsObj.adwall_tab_1,
            flip: false,
            tab: 1,
            runtime: xpromoConstantsObj.RUNTIME_ADWALL_NEW_RECOMMEND_1,
            trackName: "most_recent",
        }, {
            btn: xpromoConstantsObj.btn_toggle_02_off,
            btnSel: xpromoConstantsObj.btn_toggle_02_on,
            ele: null,
            eleSel: null,
            eleLabel: null,
            xPos: (canvasWidth / 2),
            yPos: sectionHeight - 225,
            label: xpromoStringsObj.adwall_tab_2,
            flip: false,
            tab: 2,
            runtime: xpromoConstantsObj.RUNTIME_ADWALL_NEW_RECOMMEND_2,
            trackName: "most_popular",
        }, {
            btn: xpromoConstantsObj.btn_toggle_01_off,
            btnSel: xpromoConstantsObj.btn_toggle_01_on,
            ele: null,
            eleSel: null,
            eleLabel: null,
            xPos: (canvasWidth / 2) + 367,
            yPos: sectionHeight - 225,
            label: xpromoStringsObj.adwall_tab_3,
            flip: true,
            tab: 3,
            runtime: xpromoConstantsObj.RUNTIME_ADWALL_NEW_RECOMMEND_3,
            trackName: "friend_fav",
        }]
    },

    /**
     * Method to populate the recommended game carousels for the player to click and install / play from.
     * @param spriteArr - sprites to be added
     * @param labelArr - labels to be added
     * @param zOrder - base zOrder of the layer
     * @param screen - screen constant
     */
    populateRecommendationsScroll: function(spriteArr, labelArr, zOrder, screen) {
        var recommendationLayer = new cc.Node();
        var sectionHeight = moreGamesLayer.getRecommendedSectionHeight();
        recommendationLayer.setContentSize(canvasWidth + xPromoGameSpecificObj.getWidthErr(), sectionHeight);

        var moreGamesLabel = utilObj.getBMFontLabel(xpromoStringsObj.more_games + ":", constantsObj.FONT_SFCOMPACT_MEDIUM_80, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), canvasWidth / 2, sectionHeight - 75, 0.5, 0.5, labelArr, recommendationLayer, zOrder);
        moreGamesLayer.setTabConfig(sectionHeight, canvasWidth);

        for(var index in moreGamesLayer.recommendationTobBarConfig) {
            var config = moreGamesLayer.recommendationTobBarConfig[index];
            config.ele = utilObj.getSprite(config.btn, config.xPos, config.yPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_6, spriteArr, recommendationLayer, zOrder, constantsObj.SPRITE_MISC);
            config.eleSel = utilObj.getSprite(config.btnSel, config.xPos, config.yPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_6, spriteArr, recommendationLayer, zOrder, constantsObj.SPRITE_MISC);
            config.eleLabel = utilObj.getBMFontLabel(config.label, constantsObj.FONT_SFCOMPACT_MEDIUM_40, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), config.xPos, config.yPos, 0.5, 0.5, labelArr, recommendationLayer, zOrder);
            utilObj.setVisible(config.eleSel, false);

            if(config.flip) {
                config.ele.setFlippedX(true);
                config.eleSel.setFlippedX(true);
            }

            var trans_btn = utilObj.getButton(350, 140, 0.5, 0.5,
                config.xPos, config.yPos, constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                    obj: 'moreGamesLayer',
                    fn: 'actionOnTabRecScroll',
                    screen: screen,
                    params: config.tab,
                }, false, ccui.Widget.PLIST_TEXTURE, constantsObj.SCALE_ONE);
            recommendationLayer.addChild(trans_btn, zOrder+1);
        }

        moreGamesLayer.recommendationScrollView = new ccui.ScrollView();
        var scrollWidth = canvasWidth;
        var scrollHeight = sectionHeight - 375;
        moreGamesLayer.recommendationScrollView.setDirection(ccui.ScrollView.DIR_HORIZONTAL);
        moreGamesLayer.recommendationScrollView.setTouchEnabled(true);
        moreGamesLayer.recommendationScrollView.setBounceEnabled(true);
        moreGamesLayer.recommendationScrollView.setInertiaScrollEnabled(true);
        utilObj.setPosition(moreGamesLayer.recommendationScrollView, 0, scrollHeight);
        moreGamesLayer.recommendationScrollView.setContentSize(cc.size(scrollWidth, scrollHeight));
        moreGamesLayer.recommendationScrollView.setSwallowTouches(false);
        moreGamesLayer.recommendationScrollView.setAnchorPoint(cc.p(0, 1));

        utilObj.addChild(recommendationLayer, moreGamesLayer.recommendationScrollView, zOrder+1);

        moreGamesLayer.actionOnTabRecScroll(2);

        var point = moreGamesLayer.getRecommendedSectionPositions();
        recommendationLayer.setPosition(point.x, point.y);
        recommendationLayer.setAnchorPoint(0.5, 1);
        utilObj.addChild(moreGamesLayer, recommendationLayer, zOrder+1);
    },

    /**
     * Carousel tab click handler.
     * @param tab - corresponds to the tab number being clicked
     */
    actionOnTabRecScroll: function(tab) {
        for(var index in moreGamesLayer.recommendationTobBarConfig) {
            var config = moreGamesLayer.recommendationTobBarConfig[index];
            if(moreGamesLayer.recommendationTobBarConfig[index].tab != tab) {
                utilObj.setVisible(config.eleSel, false);
                utilObj.setColor(config.eleLabel, constantsObj.COLOR_WHITE);
            } else {
                utilObj.setVisible(config.eleSel, true);
                utilObj.setColor(config.eleLabel, constantsObj.COLOR_BLACK);
                var gameRuntimes = xpromoQuestRuntimesObj.getRuntime(config.runtime);
                if(gameRuntimes != undefined && gameRuntimes != "") {
                    var gameIds = gameRuntimes.split("^");
                    if(gameIds.length > 0) {
                        utilObj.removeAllChildren(moreGamesLayer.recommendationScrollView);
                        moreGamesLayer.populateScrollViewRecommendation(gameIds, config.trackName);
                    }
                }
            }
        }
    },

    /**
     * Method to populate the contents of a recommended scroll view with the game thumbnails
     * @param gameIds - list of game ids which would be populated in the view
     */
    populateScrollViewRecommendation: function(gameIds, trackName) {
        var scrollHeight = moreGamesLayer.getRecommendedSectionHeight() - 375;
        var spriteArr = [];
        var labelArr = [];
        var zOrder = 1;
        var screen = constantsObj.SCREEN_MORE_GAMES_LAYER;

        var count = 0;
        var xPos = -600;
        var yPos = 0;

        for (var index in gameIds) {
            var gameId = parseInt(gameIds[index]);
            if (isNaN(gameId) || gameId == constantsObj.GAME_ID || moreGamesLayer.gameMapWithId[gameId] == undefined) {
                continue;
            }

            var gameData = moreGamesLayer.gameMapWithId[gameId];

            // hitting adjust view tracking
            var trackId = xpromoUtilObj.getAdjustTrackerForGameId(gameId); // moreGamesLayer.trackAndroidMap[gameId];
            var campaignName = "adwall_" + constantsObj.GAME_ID;
            xpromoUtilObj.hitAdjustViewTracking(trackId, campaignName);

            if(count % 2 == 0) {
                xPos = xPos + 650;
                yPos = scrollHeight;
            } else {
                yPos = scrollHeight - 500;
            }

            var gameIconName = "thumb_" + gameId + ".jpg";
            var imageMap = adWallObj.getImagesForGameId(gameId);
            if(imageMap.thumbnail != undefined) {
                gameIconName = imageMap.thumbnail;
            }

            var gameLogo = utilObj.getSprite(xpromoUtilObj.getAbsFilePath(gameIconName), xPos, yPos, 0, 1, constantsObj.SCALE_SPRITE_6, spriteArr, moreGamesLayer.recommendationScrollView, zOrder, constantsObj.SPRITE_LOCAL);
            var gameLabel = utilObj.getBMFontLabel(gameData["d"], constantsObj.FONT_SFCOMPACT_REGULAR_60, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos + 275, yPos - 425, 0.5, 0.5, labelArr, moreGamesLayer.recommendationScrollView, zOrder);

            var trans_btn = utilObj.getButton(560, 475, 0, 1,
                xPos, yPos, constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                    obj: 'moreGamesLayer',
                    fn: 'actionOnBtn',
                    screen: screen,
                    params: gameId,
                    track: {
                        k: xpromoConstantsObj.TRACK_XPROMO,
                        p: xpromoConstantsObj.TRACK_ADWALL,
                        c: gameId,
                        o: xpromoConstantsObj.TRACK_ADWALL,
                        f: constantsObj.TRACK_CLICK,
                        g: "more_games",
                        s: trackName,
                        co: 1,
                        oppId: ""
                    },
                }, false, ccui.Widget.PLIST_TEXTURE, constantsObj.SCALE_ONE);
            moreGamesLayer.recommendationScrollView.addChild(trans_btn, zOrder+1);

            count++;
        }

        moreGamesLayer.recommendationScrollView.setInnerContainerSize(cc.size(xPos + 650, scrollHeight));
        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);
    },

    /**
     * Method to populate the carousel section and add it to the more games layer
     * @param sender - initiator
     * @param params - params passed from the delayed caller
     */
    showCarousel: function (sender, params) {
        if(curScreen == constantsObj.SCREEN_MORE_GAMES_LAYER) {

            var isInSocialVariant = adWallObj.isInAdWallNewSocialVariant();

            var carouselLayer = new cc.Node();
            var sectionHeight = moreGamesLayer.getCarouseSectionHeight();
            carouselLayer.setContentSize(canvasWidth + xPromoGameSpecificObj.getWidthErr(), sectionHeight);

            var spriteArr = [];
            var labelArr = [];
            var centerXPos = params["centerXPos"];
            var zOrder = params["zOrder"];

            var xPos = centerXPos;
            var curImg = 0;

            var friendsDisplayData = null;
            if(isInSocialVariant) {
                friendsDisplayData = adWallObj.getFriendsToDisplay();
            }

            var amount = adWallObj.getAdwallIncentiveCoinAmount();

            var gamesInCarousel = adWallObj.getGamesVisibleInCarousel();
            var gameMapSorted = adWallObj.isInAdWallNewSocialVariant() ? adWallObj.getSortedGameMapKeysByInstalled() : Object.keys(moreGamesLayer.gameMap);
            for (var index in gameMapSorted) {
                var gameData = moreGamesLayer.gameMap[gameMapSorted[index]];
                var gameId = gameData["gh"];
                var questId = gameData["id"];
                if (gamesInCarousel.indexOf(gameId) == -1) {
                    continue;
                }

                if(moreGamesLayer.incentiveAdWall && !xpromoUtilObj.isGameEligibleForXpromo(gameId, questId)) {
                    continue;
                }

                var gameCarousel = new cc.Node();

                moreGamesLayer.carouselMap[curImg] = {gameId: gameId, questId: questId, coins: amount, xpromo: moreGamesLayer.incentiveAdWall, trackName: xpromoConstantsObj.TRACK_CAROUSEL} ;

                var imageMap = adWallObj.getImagesForGameId(gameId);
                var gameIconName = "carousel_" + gameId + ".jpg";
                var markersOffset = 0;
                if(imageMap.carousel != undefined) {
                    gameIconName = imageMap.carousel;
                }

                if(isInSocialVariant) {
                    gameIconName = "adwall_social_" + gameId + ".jpg";
                    if(imageMap.social_carousel != undefined) {
                        gameIconName = imageMap.social_carousel;
                    }
                    markersOffset = 50;
                }

                var tabImageFix = 0;
                var gameIconLink = constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/carousel/" + gameIconName;
                if ((utilObj.isTablet() && !isInSocialVariant) || (isInSocialVariant && canvasWidth > 1440)) {
                    tabImageFix = -5;
                    gameIconLink = constantsObj.MISC_SERVER_STATIC_BASE + "quests/images/carousel_tab/" + gameIconName;
                }
                if (!utilObj.fileExists(gameIconName)) {
                    xPromoGameSpecificObj.downloadFileV2(gameIconLink, gameIconName);
                }

                // Incase of the social variant we need to populate friend images, label texts etc within the carousel
                if(isInSocialVariant && friendsDisplayData[gameId] != undefined && friendsDisplayData[gameId].length == 4) {

                    var friendIndex = 0;
                    var friendsGameData = friendsDisplayData[gameId];

                    var clipperNode;
                    if(!utilObj.isWeb()) {
                        var stencil = utilObj.getSprite(xpromoConstantsObj.circle_player, centerXPos + 355, 613 + tabImageFix + markersOffset, 0.5, 0.5, 1.08, spriteArr);
                        stencil.setContentSize(194, 194);
                        clipperNode = cc.ClippingNode(stencil);
                        clipperNode.setAlphaThreshold(0);
                        utilObj.addChild(gameCarousel, clipperNode, zOrder+1);
                    } else {
                        clipperNode = gameCarousel
                    }

                    var pic = adWallObj.getFriendImageFromData(friendsGameData[friendIndex]);
                    friendIndex++;

                    var friendPic = utilObj.getSprite(pic, centerXPos + 355, 613 + tabImageFix + markersOffset, 0.5, 0.5, 1.1, spriteArr,
                        clipperNode, zOrder+1, constantsObj.SPRITE_LOCAL);
                    friendPic.setContentSize(206, 206);

                    var positionOffset = 184;
                    for(var others = 0; others < 3; others++) {
                        var smallImgsX = (centerXPos - 353) + (others * positionOffset);
                        if(!utilObj.isWeb()) {
                            var subStencil = utilObj.getSprite(xpromoConstantsObj.circle_player, smallImgsX, 116 + markersOffset, 0.5, 0.5, 0.79, spriteArr);
                            subStencil.setContentSize(194, 194);
                            var subClipper = cc.ClippingNode(subStencil);
                            subClipper.setAlphaThreshold(0);
                            utilObj.addChild(gameCarousel, subClipper, zOrder+1);
                        } else {
                            subClipper = gameCarousel;
                        }

                        pic = adWallObj.getFriendImageFromData(friendsGameData[friendIndex]);
                        friendIndex++;

                        var subFriendPic = utilObj.getSprite(pic, smallImgsX, 116 + markersOffset, 0.5, 0.5, 0.81, spriteArr,
                            subClipper, zOrder+1, constantsObj.SPRITE_LOCAL);
                        subFriendPic.setContentSize(206, 206);

                    }

                    var name = friendsGameData[0].name;
                    var splitName = name.split(" ");
                    var firstName = splitName[0];
                    var fontSize = constantsObj.FONT_SFCOMPACT_MEDIUM_50;
                    var nameToUse = name;
                    var fontWidth = 650;
                    var trackStr = (friendsGameData[0].bot) ? "bot" : "fb";
                    moreGamesLayer.carouselMap[curImg].trackName = xpromoConstantsObj.TRACK_CAROUSEL + "_" + trackStr + "_" + friendsGameData[0].refId;
                    if (name.length > 10) {
                        nameToUse = firstName;
                        fontWidth = 650;
                        if(firstName.length > 10 && firstName.length < 16) {
                            fontSize = constantsObj.FONT_SFCOMPACT_MEDIUM_40;
                            fontWidth = 750;
                        } else if (firstName.length >= 16) {
                            fontSize = constantsObj.FONT_SFCOMPACT_MEDIUM_35;
                            fontWidth = 850;
                        }
                    }

                    if(constantsObj.GAME_ID == 8 || constantsObj.GAME_ID == 17) {
                        fontWidth -= 50;
                    }

                    var invite_msg = xpromoStringsObj.adwall_game_invite.replace("%VALUE%", nameToUse);
                    invite_msg = invite_msg.replace("%game%", gameData["d"]);
                    utilObj.getBMFontLabel(invite_msg, fontSize, constantsObj.COLOR_BLACK, fontWidth, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), centerXPos + 355, 410 + markersOffset, 0.5, 0.5, labelArr, gameCarousel, zOrder+1);

                    var btnText = xpromoStringsObj.play_now.toUpperCase();
                    var btnTextXOffset = 0;
                    if(moreGamesLayer.incentiveAdWall) { // Show the coin surfacing in case of incentivized variant
                        var coinIconOffset = amount > 99 ? 230 : 250;
                        var coinIcon = utilObj.getSprite(questConfigObj.getConfigData("icon_coins"), centerXPos + coinIconOffset , 110 + markersOffset, 0.5, 0.5, constantsObj.SCALE_SPRITE_2, spriteArr, gameCarousel, zOrder+1, xpromoConstantsObj.SPRITE_QUESTS);
                        btnText = xpromoStringsObj.adwall_coin_free;
                        btnText = btnText.replace("%VALUE%", amount.toString());
                        btnTextXOffset = 40;
                    }
                    utilObj.getBMFontLabel(btnText, constantsObj.FONT_SFCOMPACT_MEDIUM_80, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), centerXPos + 400 + btnTextXOffset, 115 + markersOffset, 0.5, 0.5, labelArr, gameCarousel, zOrder+1);

                }

                var carousel_image = utilObj.getSprite(xpromoUtilObj.getAbsFilePath(gameIconName), centerXPos, markersOffset, 0.5, 0, 1, spriteArr,
                    gameCarousel, zOrder, constantsObj.SPRITE_LOCAL);

                var contentSize = carousel_image.getContentSize();
                if(!isInSocialVariant) {
                    carousel_image.setContentSize(canvasWidth, contentSize.height);
                }

                moreGamesLayer.images.push(gameCarousel);
                gameCarousel.setContentSize(canvasWidth, contentSize.height);
                gameCarousel.setPosition(xPos, 0);
                gameCarousel.setAnchorPoint(0.5, 0);

                utilObj.addChild(carouselLayer, gameCarousel, zOrder);
                xPos += canvasWidth;

                // Except for the first images, set all images invisible to save up on GL calls
                curImg += 1;
                if(curImg > 1) {
                    gameCarousel.setVisible(false);
                }
            }

            var markerAnchorY = 0;
            var markerOpacity = constantsObj.OPACITY_100;
            var markerYPos = 20;
            var markerScale = 0.17;
            var markerBaseX = 75;

            if(isInSocialVariant) {
                markerAnchorY = 0.5;
                markerOpacity = constantsObj.OPACITY_40;
                markerYPos = 30;
                markerScale = 0.1;
                markerBaseX = 60
            }

            // Add the navigation representation of the carousel only if there is more than 1 pane.
            if (moreGamesLayer.images.length > 1) {
                var xDiff = markerBaseX;
                xPos = centerXPos - (xDiff * (moreGamesLayer.images.length - 1) / 2);

                var horizontalBar = utilObj.getScale9Sprite(constantsObj.tile_white, centerXPos, 0, 0.5, 0, canvasWidth + xPromoGameSpecificObj.getWidthErr(), 60, spriteArr, carouselLayer, 2, constantsObj.SPRITE_MISC);
                horizontalBar.setColor(constantsObj.COLOR_BLACK);
                horizontalBar.setOpacity(constantsObj.OPACITY_80);

                for (var i = 0; i < moreGamesLayer.images.length; i++) {
                    var b1 = utilObj.getSprite(xpromoConstantsObj.circle_player, xPos, markerYPos, 0.5, markerAnchorY, xpromoUtilObj.getSpriteScale(markerScale), spriteArr, carouselLayer, 2, constantsObj.SPRITE_QUESTS);
                    b1.setContentSize(194, 194);
                    b1.setColor(constantsObj.COLOR_WHITE);
                    b1.setOpacity(markerOpacity);
                    moreGamesLayer.imageDots.push(b1);
                    xPos += xDiff;
                }
                if (moreGamesLayer.images.length > 0) {
                    if(isInSocialVariant) {
                        utilObj.setOpacity(moreGamesLayer.imageDots[moreGamesLayer.curImage], constantsObj.OPACITY_100);
                        utilObj.setScale(moreGamesLayer.imageDots[moreGamesLayer.curImage], 0.17);
                    } else {
                        moreGamesLayer.imageDots[moreGamesLayer.curImage].setColor(constantsObj.COLOR_YELLOW_01);
                    }
                }

                var delayAction = animationObj.getDelayTime(adWallObj.getCarouselAutoSwipeDuration());
                var fnCallAction = utilObj.getCallFunc(moreGamesLayer.animateCarouselLeft, moreGamesLayer);
                var actions = [delayAction, fnCallAction];
                var seq = animationObj.getSequence(actions);
                moreGamesLayer.carouselSeqRepeat = new cc.RepeatForever(seq);
                animationObj.runAction(moreGamesLayer, moreGamesLayer.carouselSeqRepeat);
            }

            utilObj.addSprites(spriteArr);
            utilObj.addLabels(labelArr);
            moreGamesLayer.carouselShown = true;

            var point = moreGamesLayer.getCarouseSectionPositions();
            carouselLayer.setPosition(point.x, point.y);
            carouselLayer.setAnchorPoint(0.5, 0);
            utilObj.addChild(moreGamesLayer, carouselLayer, zOrder+1);

        }
    },

    /**
     * Method to provide the section height of the back to game section on the more games layer
     * @returns {number} - returned numerical height
     */
    getBackToGameSectionHeight: function () {
        return 340;
    },

    /**
     * Method to provide the position of the back to game section within the more games layer
     * @returns {cc.Point}
     */
    getBackToGameSectionPositions: function() {
        var xPos = centerXPos;
        var yPos = xPromoGameSpecificObj.getFullDesignHeight() - moreGamesLayer.getBackToGameSectionHeight();

        var adWallNewUI = adWallObj.getAdWallNewUIExperiment();
        if(adWallNewUI == constantsObj.EXP_VAR_4) {
            yPos = 0;
        }

        return cc.p(xPos, yPos);
    },

    /**
     * Method to provide the section height of the recommendation section on the more games layer
     * @returns {number} - returned numerical height
     */
    getRecommendedSectionHeight: function () {
        return xPromoGameSpecificObj.getFullDesignHeight() - moreGamesLayer.getBackToGameSectionHeight() - moreGamesLayer.getCarouseSectionHeight();
    },

    /**
     * Method to provide the position of the recommendation section within the more games layer
     * @returns {cc.Point}
     */
    getRecommendedSectionPositions: function() {
        var xPos = centerXPos;
        var yPos = xPromoGameSpecificObj.getFullDesignHeight() - moreGamesLayer.getBackToGameSectionHeight();

        var adWallNewUI = adWallObj.getAdWallNewUIExperiment();
        if(adWallObj.isInAdWallNewSocialVariant()) {
            yPos = xPromoGameSpecificObj.getFullDesignHeight() - moreGamesLayer.getBackToGameSectionHeight() - moreGamesLayer.getCarouseSectionHeight();
        } else if(adWallNewUI == constantsObj.EXP_VAR_4) {
            yPos = xPromoGameSpecificObj.getFullDesignHeight();
        }

        return cc.p(xPos, yPos);
    },

    /**
     * Method to provide the section height of the carousel section on the more games layer
     * @returns {number} - returned numerical height
     */
    getCarouseSectionHeight: function () {
        if(adWallObj.isInAdWallNewSocialVariant()) {
            return 1050;
        }
        return 810;
    },

    /**
     * Method to provide the position of the carousel section within the more games layer
     * @returns {cc.Point}
     */
    getCarouseSectionPositions: function ()  {
        var xPos = centerXPos;
        var yPos = constantsObj.BEZEL_SAFE_AREA / 2;

        var adWallNewUI = adWallObj.getAdWallNewUIExperiment();
        if(adWallObj.isInAdWallNewSocialVariant()) {
            yPos = xPromoGameSpecificObj.getFullDesignHeight() - moreGamesLayer.getBackToGameSectionHeight() - moreGamesLayer.getCarouseSectionHeight();
        } else if(adWallNewUI == constantsObj.EXP_VAR_4) {
            yPos = moreGamesLayer.getBackToGameSectionHeight();
        }

        return cc.p(xPos, yPos);
    },

    /**
     * Handler for layer hide which remove all children
     */
    hide: function () {
        moreGamesLayer.removeAllChildren();
    },

    /**
     * Handler for back button
     */
    back: function () {
        moreGamesLayer.backToHome();
    },

    /**
     * Handler for back button
     */
    backToHome: function () {
        moreGamesLayer.backToPuzzle();
    },

    /**
     * Touch begin handler
     * @param touch
     * @param event
     * @returns {boolean}
     */
    onTouchesBegan: function (touch, event) {
        moreGamesLayer.touchStartX = touch.getLocation().x;
        return true;
    },

    /**
     * Touch moved handler
     * @param touch
     * @param event
     * @returns {boolean}
     */
    onTouchMoved: function (touch, event) {
        return true;
    },

    /**
     * Touch ended handler
     * @param touch
     * @param event
     * @returns {boolean}
     */
    onTouchEnded: function (touch, event) {
        if (curScreen != constantsObj.SCREEN_MORE_GAMES_LAYER) {
            return true;
        }
        moreGamesLayer.touchEndX = touch.getLocation().x;
        var swipeDir = utilObj.wasHorizontalSwipe(moreGamesLayer.touchStartX, moreGamesLayer.touchEndX);

        var yPos = moreGamesLayer.getCarouseSectionPositions().y;
        var height = moreGamesLayer.getCarouseSectionHeight();

        // Check if the touch is within the carousel area, if not return.
        if (touch.getLocation().y > (yPos + height) || touch.getLocation().y < yPos || !moreGamesLayer.carouselShown) {
            return true;
        }

        // Check the lenght of the touch and decide if we need to swipe or force a click
        // Note: Always force a click when there is only a single image in the carousel
        if (Math.abs(moreGamesLayer.touchEndX - moreGamesLayer.touchStartX) < 150 || moreGamesLayer.images.length == 1) {
            if(moreGamesLayer.carouselMap[moreGamesLayer.curImage].xpromo == true) {
                moreGamesLayer.actionOnIncentivisedBtn(moreGamesLayer.carouselMap[moreGamesLayer.curImage]);
            } else {
                moreGamesLayer.actionOnBtn(moreGamesLayer.carouselMap[moreGamesLayer.curImage].gameId, 1);
            }
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xpromoConstantsObj.TRACK_ADWALL, moreGamesLayer.carouselMap[moreGamesLayer.curImage].gameId, xpromoConstantsObj.TRACK_ADWALL, constantsObj.TRACK_CLICK, moreGamesLayer.carouselMap[moreGamesLayer.curImage].trackName, "", "", "");
        } else if (moreGamesLayer.animRunning == false) {
            moreGamesLayer.stopAllActions();
            trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xpromoConstantsObj.TRACK_ADWALL, moreGamesLayer.carouselMap[moreGamesLayer.curImage].gameId, xpromoConstantsObj.TRACK_ADWALL, xpromoConstantsObj.TRACK_SWIPE, moreGamesLayer.carouselMap[moreGamesLayer.curImage].trackName, "", "", "");
            if (swipeDir == 1) {
                moreGamesLayer.animateCarouselLeft();
            } else if (swipeDir == 2) {
                moreGamesLayer.animateCarouselRight();
            }
            var delayAction = animationObj.getDelayTime(adWallObj.getCarouselAutoSwipeDuration());
            var fnCallAction = utilObj.getCallFunc(moreGamesLayer.animateCarouselLeft, moreGamesLayer);
            var actions = [delayAction, fnCallAction];
            var seq = animationObj.getSequence(actions);
            moreGamesLayer.carouselSeqRepeat = new cc.RepeatForever(seq);
            animationObj.runAction(moreGamesLayer, moreGamesLayer.carouselSeqRepeat);
        }
        return true;
    },

    /**
     * Touch cancelled handler
     * @param touch
     * @param event
     * @returns {boolean}
     */
    onTouchCancelled: function (touch, event) {
        return true;
    },

    /**
     * Helper method to animate the carousel to the left of the screen and bring in the new pane
     */
    animateCarouselLeft: function () {
        if (curScreen != constantsObj.SCREEN_MORE_GAMES_LAYER) {
            return;
        }
        var nextImageIndex = (moreGamesLayer.curImage + 1) % moreGamesLayer.images.length;
        moreGamesLayer.animRunning = true;
        for (var i = 0; i < moreGamesLayer.images.length; i++) {
            var moveBy;
            if(i == nextImageIndex) {
                moreGamesLayer.images[i].setVisible(true);
            }
            var curPosition = Math.round(utilObj.getPositionX(moreGamesLayer.images[i]));
            if (curPosition >= 0) {
                moveBy = animationObj.getMoveBy(0.25, -canvasWidth, 0);
                animationObj.runAction(moreGamesLayer.images[i], moveBy);
            }
        }
        moreGamesLayer.lastImage = moreGamesLayer.curImage;

        if(!adWallObj.isInAdWallNewSocialVariant()) {
            utilObj.setColor(moreGamesLayer.imageDots[moreGamesLayer.curImage], constantsObj.COLOR_WHITE);
            moreGamesLayer.curImage = nextImageIndex;
            utilObj.setColor(moreGamesLayer.imageDots[moreGamesLayer.curImage], constantsObj.COLOR_YELLOW_01);
        } else {
            utilObj.setOpacity(moreGamesLayer.imageDots[moreGamesLayer.curImage], constantsObj.OPACITY_40);
            utilObj.setScale(moreGamesLayer.imageDots[moreGamesLayer.curImage], 0.1);
            moreGamesLayer.curImage = nextImageIndex;
            utilObj.setOpacity(moreGamesLayer.imageDots[moreGamesLayer.curImage], constantsObj.OPACITY_100);
            utilObj.setScale(moreGamesLayer.imageDots[moreGamesLayer.curImage], 0.17);
        }
        utilObj.delayedCallback(moreGamesLayer, moreGamesLayer.carouselAnimOver, 0.3, null, false);
    },

    /**
     * Helper method to animate the carousel to the right of the screen and bring in the new pane
     */
    animateCarouselRight: function () {
        if (curScreen != constantsObj.SCREEN_MORE_GAMES_LAYER) {
            return;
        }
        var nextImageIndex = (moreGamesLayer.curImage - 1 + moreGamesLayer.images.length) % moreGamesLayer.images.length;
        moreGamesLayer.animRunning = true;
        for (var i = 0; i < moreGamesLayer.images.length; i++) {
            var moveBy;
            if(i == nextImageIndex) {
                moreGamesLayer.images[i].setVisible(true);
            }

            var curPosition = Math.round(utilObj.getPositionX(moreGamesLayer.images[i]));
            if (curPosition >= (moreGamesLayer.images.length - 1) * canvasWidth) {
                utilObj.setPositionX(moreGamesLayer.images[i], centerXPos - canvasWidth);
            }
            moveBy = animationObj.getMoveBy(0.25, canvasWidth, 0);
            animationObj.runAction(moreGamesLayer.images[i], moveBy);
        }

        moreGamesLayer.lastImage = moreGamesLayer.curImage;
        if(!adWallObj.isInAdWallNewSocialVariant()) {
            moreGamesLayer.imageDots[moreGamesLayer.curImage].setColor(constantsObj.COLOR_WHITE);
            moreGamesLayer.curImage = nextImageIndex;
            moreGamesLayer.imageDots[moreGamesLayer.curImage].setColor(constantsObj.COLOR_YELLOW_01);
        } else {
            utilObj.setOpacity(moreGamesLayer.imageDots[moreGamesLayer.curImage], constantsObj.OPACITY_40);
            utilObj.setScale(moreGamesLayer.imageDots[moreGamesLayer.curImage], 0.1);
            moreGamesLayer.curImage = nextImageIndex;
            utilObj.setOpacity(moreGamesLayer.imageDots[moreGamesLayer.curImage], constantsObj.OPACITY_100);
            utilObj.setScale(moreGamesLayer.imageDots[moreGamesLayer.curImage], 0.17);
        }

        utilObj.delayedCallback(moreGamesLayer, moreGamesLayer.carouselAnimOver, 0.3, null, false);
    },

    /**
     * Callback method when a pane-transition animation on the carousel is completed
     */
    carouselAnimOver: function () {
        for (var i = 0; i < moreGamesLayer.images.length; i++) {
            var curPosition = Math.round(utilObj.getPositionX(moreGamesLayer.images[i]));
            if (curPosition < 0) {
                utilObj.setPositionX(moreGamesLayer.images[i], (moreGamesLayer.images.length - 1) * canvasWidth + centerXPos);
            }
        }

        moreGamesLayer.animRunning = false;
        if(moreGamesLayer.lastImage < moreGamesLayer.images.length) {
            moreGamesLayer.images[moreGamesLayer.lastImage].setVisible(false);
        }
    },

    /**
     * Method that ensures all images that are required for the more games layer are downloaded
     * If new images are added, those need to be added here as well.
     * @returns {boolean} - true if all are downloaded, false if it is in progress
     */
    downloadReqdImages: function () {
        if (utilObj.isWeb()) {
            return true;
        }

        var allImagesDownloaded = true;
        var imageName = constantsObj.moreGamesIcon;
        var psIconLink = xpromoUtilObj.getImageLinkFromName(imageName);
        // loggerObj.debugLog("this is image name", imageName, psIconLink);
        if (!utilObj.fileExists(imageName)) {
            xPromoGameSpecificObj.downloadFileV2(psIconLink, imageName);
            allImagesDownloaded = false;
        }

        imageName = "icon_" + constantsObj.GAME_ID + ".png";
        psIconLink = xpromoUtilObj.getImageLinkFromName(imageName);
        if (!utilObj.fileExists(imageName)) {
            xPromoGameSpecificObj.downloadFileV2(psIconLink, imageName);
            allImagesDownloaded = false;
        }

        if(pspnFlutterObj.isFlutterEnabled()) {
            // For Flutter Adwall, the only image required is the adwall icon shown on the game side,
            // hence skipping the check for rest of the images
            return allImagesDownloaded;
        }

        // carousel images needs to be added
        questsObj.getAvailableCrossGameQuests();
        moreGamesLayer.gameMap = questsObj.getAdWallQuests();
        if (moreGamesLayer.gameMap.length == 0 || utilObj.getStringifiedJson(moreGamesLayer.gameMap) == "{}") {
            return false;
        }
        var adWallNewUI = adWallObj.getAdWallNewUIExperiment();
        for (var game in moreGamesLayer.gameMap) {
            var gameData = moreGamesLayer.gameMap[game];
            var gameId = gameData["gh"];

            var gameIconName = "icon_" + gameId + ".png";
            var imageMaps = adWallObj.getImagesForGameId(gameId);
            if(imageMaps.icon != undefined) {
                gameIconName = imageMaps.icon;
            }

            var gameIconLink = xpromoUtilObj.getImageLinkFromName(gameIconName);
            if (!utilObj.fileExists(gameIconName)) {
                xPromoGameSpecificObj.downloadFileV2(gameIconLink, gameIconName);
                allImagesDownloaded = false;
            }
            if (gameData["link"] != "1") {
                continue;
            }

            var gameCarouselName = "carousel_" + gameId + ".jpg";
            if(imageMaps.carousel != undefined) {
                gameCarouselName = imageMaps.carousel;
            }

            if(adWallObj.isInAdWallNewSocialVariant()) {
                gameCarouselName = "adwall_social_" + gameId + ".jpg";
                if(imageMaps.social_carousel != undefined) {
                    gameCarouselName = imageMaps.social_carousel;
                }
            }

            gameIconLink = xpromoUtilObj.getImageLinkFromName(gameCarouselName);
            if (!utilObj.fileExists(gameCarouselName)) {
                xPromoGameSpecificObj.downloadFileV2(gameIconLink, gameCarouselName);
                allImagesDownloaded = false;
                // loggerObj.debugLog("this is not downloaded", gameCarouselName, gameIconLink);
            }

            if(adWallNewUI == constantsObj.EXP_VAR_3) {
                var gameThumbnailIcon = "thumb_" + gameId + ".jpg";
                if(imageMaps.thumbnail != undefined) {
                    gameThumbnailIcon = imageMaps.thumbnail;
                }
                var gameThumbnailLink = xpromoUtilObj.getImageLinkFromName(gameThumbnailIcon);
                if (!utilObj.fileExists(gameThumbnailIcon)) {
                    xPromoGameSpecificObj.downloadFileV2(gameThumbnailLink, gameThumbnailIcon);
                    allImagesDownloaded = false;
                }
            }
        }
        imageName = constantsObj.ps_logo;
        psIconLink = xpromoUtilObj.getImageLinkFromName(imageName);
        if (!utilObj.fileExists(imageName)) {
            xPromoGameSpecificObj.downloadFileV2(psIconLink, imageName);
            allImagesDownloaded = false;
        }

        adWallObj.areFriendImagesDownloaded();
        return allImagesDownloaded;
    },

    /**
     * Click handler on an game icon / carousel to direct to the app store / game
     * @param game_id
     * @param src
     */
    actionOnBtn: function (game_id, src) {
        if (game_id == undefined) {
            return;
        }
        var params = moreGamesLayer.gameMapWithId[game_id]["pa"].split(":");
        var success = xpromoUtilObj.isAppInstalled(params[0]);
        questsObj.resetRedirectDataForAdjust();
        if (success) {
            utilObj.launchAnotherApp(params[0]);
        } else {
            if (utilObj.isWeb()) {
                xpromoUtilObj.openPlayStoreUrl(params[0]);
            } else if (utilObj.isNative()) {
                var campaignName = "adwall_" + constantsObj.GAME_ID + "_carousel";
                var screenName = constantsObj.SCREEN_MORE_GAMES_LAYER;
                if (src == undefined) {
                    campaignName = "adwall_" + constantsObj.GAME_ID;
                }
                var link;
                var expVar = epmObj.getChosenVariant(xpromoConstantsObj.EXP_REDIRECT_TO_APP_STORE);
                if (utilObj.isAndroid()) {
                    if (expVar == constantsObj.EXP_VAR_CONTROL) {
                        link = questsObj.getGameShareLink(params[1], screenName, campaignName);
                        utilObj.openStore(null, link, screenName);
                    } else if (utilObj.isAmazonDevice() && game_id == 12) {
                        utilObj.openUrl("market://details?id=in.playsimple.guessup_emoji_amzn");
                    } else {
                        //xpromoUtilObj.hitUrl(moreGamesLayer.trackAndroidMap[game_id], campaignName);
                        utilObj.openUrl("market://details?id=" + params[0]);
                    }
                } else if (utilObj.isIos()) {
                    if (expVar == constantsObj.EXP_VAR_CONTROL) {
                        link = questsObj.getGameShareLink(params[3], screenName, campaignName);
                        utilObj.openStore(null, link, screenName);
                    } else if (expVar == constantsObj.EXP_VAR_1) {
                        //xpromoUtilObj.hitUrl(moreGamesLayer.trackIosMap[game_id], campaignName);
                        utilObj.openUrl("itms-apps://itunes.apple.com/app/id" + params[2]);
                    } else if (expVar == constantsObj.EXP_VAR_2) {
                        //xpromoUtilObj.hitUrl(moreGamesLayer.trackIosMap[game_id], campaignName);
                        link = questsObj.getGameShareLink(params[3], screenName, campaignName);
                        utilObj.openStore(params[2], link, screenName);
                    }
                }
            }
            if (moreGamesLayer.installQuestData[game_id] != null && moreGamesLayer.installQuestData[game_id] != undefined && moreGamesLayer.installQuestData[game_id]["status"] == xpromoConstantsObj.QUEST_NOT_STARTED) {
                questsObj.updateCrossGameQuestStatus(moreGamesLayer.installQuestData[game_id]["id"], xpromoConstantsObj.QUEST_STARTED);
            }
        }
        pspnUserTracking.trackUserSnapshot(constantsObj.SCREEN_MORE_GAMES_LAYER, constantsObj.TRACK_CLICK);
    },

    /**
     * Method to handle click in the incentivised variant
     * @param params - consists an object of gameId, questId and coins
     */
    actionOnIncentivisedBtn: function (params) {
        if(params != undefined && params.gameId != undefined && params.questId != undefined && params.coins != undefined) {
            adWallObj.markXpromoInteraction(params.gameId, params.questId, params.coins);
            moreGamesLayer.actionOnBtn(params.gameId)
        }
    },

    /**
     * Method to implement a back navigation or default to the game screen
     * To be used when exiting the current carousel
     */
    backToPuzzle: function () {
        moreGamesLayer.stopAllActions();
        moreGamesLayer.hide();
        var lastScreen = adWallObj.getAdWallLastScreen();

        if (lastLayer != undefined && utilObj.isFunction(lastLayer.redraw)) {
            lastLayer.redraw();
        } else {
            xPromoGameSpecificObj.showLayerOnBack();
        }

        trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xpromoConstantsObj.TRACK_ADWALL, "", xpromoConstantsObj.TRACK_ADWALL, constantsObj.TRACK_CLOSE, constantsObj.TRACK_HOME, "", "", "");
    },

    /**
     * Utility method to store the previous layer
     * @param layer - layer last navigated
     */
    savePrevLayer: function (layer) {
        moreGamesLayer.prevLayer = layer;
    },

    /**
     * Helper method to show the more games layer given all conditions are satisfied
     * @param allImagesDownloaded - flag indicating if all images are downloaded
     * @param shouldShowAdWall - boolean holding whether adwall is shown or not
     * @param expVar - experiment variant of ad wall
     * @returns {*} - boolean based on whether the adwall is shown or not.
     */
    canShowAddWallQuests: function (allImagesDownloaded, shouldShowAdWall, expVar) {
        var allAppsInstalled = xpromoUtilObj.areAllAppsInstalled();
        var curTimeStamp = utilObj.getCurrentTimestamp();
        if (allImagesDownloaded && !allAppsInstalled) {
            if (adWallObj.getAdwallSessionCountInDay() < xpromoQuestRuntimesObj.getRuntime(xpromoConstantsObj.RUNTIME_ADWALL)) {
                return;
            }
            if (expVar == constantsObj.EXP_VAR_CONTROL) {
                return false;
            }
            if ((expVar == constantsObj.EXP_VAR_1 || expVar == constantsObj.EXP_VAR_2) && !adWallObj.checkPayerRuntime()) {
                if (curScreen == constantsObj.SCREEN_SETTINGS) {
                    return false;
                }
                return shouldShowAdWall;
            }
            var timeSinceLastShown = curTimeStamp - adWallObj.getLastShownAdWallTimeStamp();
            var condition1 = (timeSinceLastShown > xpromoGameRuntimeObj.getAdwallCooldownTime() * 60);
            var condition2 = ((curTimeStamp - adWallObj.getAdwallFirstShown() > constantsObj.SECONDS_IN_DAY) || (adWallObj.getAdwallShownCount() < xpromoGameRuntimeObj.getAdwallLimit()));
            var condition3 = (curTimeStamp - xPromoGameSpecificObj.getInstallTimeStamp() > xpromoGameRuntimeObj.getAdWallDays() * constantsObj.SECONDS_IN_DAY);
            var var2Condition = (expVar == constantsObj.EXP_VAR_2 && (timeSinceLastShown > xpromoGameRuntimeObj.getAdwallCooldownTime() * 60));
            var CombinedVarCondition = ((expVar == constantsObj.EXP_VAR_1 || expVar == constantsObj.EXP_VAR_3) && (timeSinceLastShown > constantsObj.SECONDS_IN_DAY));
            loggerObj.debugLog("in can show adwall quests 2", CombinedVarCondition, condition1, condition2, condition3);
            if ((CombinedVarCondition || var2Condition) && condition1 && condition2 && condition3){ //&& condition3) {
                if (curTimeStamp - adWallObj.getAdwallFirstShown() > constantsObj.SECONDS_IN_DAY) {
                    adWallObj.setAdwallFirstShown(curTimeStamp);
                    adWallObj.setAdwallShownCount(0);
                }
                adWallObj.setAdwallShownCount(adWallObj.getAdwallShownCount() + 1);
                var adWallLastScreen = adWallObj.getAdWallLastScreen();

                var checkScreens = [
                    constantsObj.SCREEN_HOME,
                    constantsObj.SCREEN_COUNTRY_LIST,
                    constantsObj.SCREEN_LEVEL_PACK,
                    constantsObj.SCREEN_GAME,
                    constantsObj.SCREEN_LINEAR_PROGRESSION,
                    constantsObj.SCREEN_STORE_LAYER_V1,
                    constantsObj.SCREEN_QUEST_CENTER,
                    constantsObj.SCREEN_SPELLING_BEE,
                    constantsObj.SCREEN_STORE_LAYER
                ];

                for (var i = 0; i <= checkScreens.length; i++) {
                    if (adWallLastScreen == checkScreens[i]) {
                        utilObj.setVisible(utilObj.getLayerForScreen(checkScreens[i]), false);
                        break;
                    }
                }

                shouldShowAdWall = true;
                moreGamesLayer.show(constantsObj.TRACK_AUTO_STRING);
                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xpromoConstantsObj.TRACK_ADWALL, "", xpromoConstantsObj.TRACK_ADWALL, constantsObj.TRACK_VIEW, constantsObj.TRACK_AUTO_STRING, "", "", "");
                adWallObj.setLastShownAdWallTimeStamp(utilObj.getCurrentTimestamp());
            }
        } else {
            if (!allImagesDownloaded) {
                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(), xpromoConstantsObj.TRACK_ADWALL, xpromoConstantsObj.TRACK_ADWALL_FAILED, xpromoConstantsObj.TRACK_IMAGES_NOT_DOWNLOADED, expVar, "", "");
            } else if (allAppsInstalled) {
                trackObj.trackCounter(xpromoConstantsObj.TRACK_XPROMO, xPromoGameSpecificObj.getPhylumForTracking(), xPromoGameSpecificObj.getClassForTracking(), xpromoConstantsObj.TRACK_ADWALL, xpromoConstantsObj.TRACK_ADWALL_FAILED, xpromoConstantsObj.TRACK_ALL_GAMES_INSTALLED, expVar, "", "");
            }
        }
        if (curScreen == constantsObj.SCREEN_SETTINGS) {
            return false;
        }
        return shouldShowAdWall;

    },
});

