#import <Foundation/Foundation.h>

#import "Quests.h"
#import "Util.h"
#import "Constants.h"
#import "FlutterBridge.h"

#define SHARED_DATA_PREFIX @"sharedData_"

NSString * const sharedDataGameDataKey = @"allGamesData";
NSString * const sharedDataTsKey = @"lastUpdate";
NSString * const sharedDataCrossDataKey = @"crossGameProgress";


@implementation Quests
Quests *quests;

+(Quests*) get{
    if(quests==nil){
        quests=[[Quests alloc] init];
        [quests load];
    }
    return quests;
}

-(id) init;{
    database = [SqliteHelper get:READWRITE];
    return self;
}

-(void) load;
{
    @try {
        // 1. First lets try and read the data from Flutter maintained QUESTS_NOTIF_DATA
        NSString *game_data = [database getData:QUESTS_NOTIF_DATA_FILE];
        
        // 2. Incase QUESTS_NOTIF_DATA_FILE is blank, lets query QUESTS_DATA_FILE (Format maintained by cocos
        if(!game_data || [game_data isEqualToString:@""]) {
            game_data = [database getData:QUESTS_DATA_FILE];
        }

        if(game_data){
            NSError *e = nil;
            NSDictionary *gameObj = [Util stringToJson:game_data];
            if(e){
                ENError(@"No data for user yet %@", e.description);
                return;
            }
            if([gameObj objectForKey:@"shouldShowDailyQuestNotif"] != nil){
                _shouldShowDailyQuestNotif = [[gameObj objectForKey:@"shouldShowDailyQuestNotif"] boolValue];
            }
            if([gameObj objectForKey:@"isDailyQuestCompleted"] != nil){
                _isDailyQuestCompleted = [[gameObj objectForKey:@"isDailyQuestCompleted"] boolValue];
            }
            if([gameObj objectForKey:@"dailyQuestUnlockTime"] != nil){
                _dailyQuestUnlockTime = [[gameObj objectForKey:@"dailyQuestUnlockTime"] longValue];
            }
        }
    }
    @catch (NSException *exception) {
        ENError(@"%@", [exception description]);
    }
}

-(void) saveNotifData:(long)dailyQuestUnlockTime shouldShowDailyQuestNotif:(bool) shouldShowDailyQuestNotif isDailyQuestCompleted:(bool) isDailyQuestCompleted {
    @try {
        NSMutableDictionary* notifInfo = [[NSMutableDictionary alloc] init];
        notifInfo[@"shouldShowDailyQuestNotif"] = [NSNumber numberWithBool:shouldShowDailyQuestNotif];
        notifInfo[@"isDailyQuestCompleted"] = [NSNumber numberWithBool:isDailyQuestCompleted];
        notifInfo[@"dailyQuestUnlockTime"] = [NSNumber numberWithLong:dailyQuestUnlockTime];
        [database setItem:QUESTS_NOTIF_DATA_FILE value:[Util JSONToString:notifInfo]];
    }
    @catch (NSException *exception) {
        ENError(@"%@", [exception description]);
    }
}

+(void) shareDataBetweenGames:(NSString *)allGamesData crossGame:(NSString *) crossGame {
    @try {
        NSString *ts = [NSString stringWithFormat:@"%i", [Util getCurrentTimeStampSeconds]];
        NSDictionary *sharedData =  [NSDictionary dictionaryWithObjects:@[allGamesData, ts, crossGame] forKeys:@[sharedDataGameDataKey, sharedDataTsKey, sharedDataCrossDataKey]];

        NSUserDefaults *sharedDefaults = [[NSUserDefaults alloc] initWithSuiteName:@"group.in.playsimple"];
        NSString *gameIdKey = SHARED_DATA_PREFIX;
        gameIdKey = [gameIdKey stringByAppendingString:[NSString stringWithFormat:@"%d", GAME_ID_VAL]];
        [sharedDefaults setObject:sharedData forKey:gameIdKey];
        [sharedDefaults synchronize];
    }
    @catch (NSException *exception) {
        ENError(@"%@", [exception description]);
    }
}

+(void) gettingSharedData:(NSString *)iosGameIds {
    @try {
        NSUserDefaults *sharedDefaults = [[NSUserDefaults alloc] initWithSuiteName:@"group.in.playsimple"];
        NSString *allGamesDataFinal = @"";
        NSString *crossGameProgressFinal = @"";
        long lastUpdateValue = 0;
        long gameIdFinal = 0;

        NSArray *gameIds = [iosGameIds componentsSeparatedByString:@","];
        for (int index = 0; index < [gameIds count]; index++) {
            NSString *gameId = gameIds[index];
            NSString *gameIdKey = SHARED_DATA_PREFIX;
            gameIdKey = [gameIdKey stringByAppendingString:[NSString stringWithFormat:@"%@", gameId]];
            NSDictionary *sharedData = [sharedDefaults objectForKey:gameIdKey];

            if(sharedData != nil && [sharedData objectForKey:sharedDataGameDataKey] != nil && [sharedData objectForKey:sharedDataTsKey] != nil && [sharedData objectForKey:sharedDataCrossDataKey] != nil) {
                long lastUpdate = [sharedData[sharedDataTsKey] integerValue];
                if(lastUpdate > lastUpdateValue) {
                    gameIdFinal = [gameId integerValue];
                    allGamesDataFinal = sharedData[sharedDataGameDataKey];
                    crossGameProgressFinal = sharedData[sharedDataCrossDataKey];
                    lastUpdateValue = lastUpdate;
                }
            }
        }

        NSMutableDictionary* resp = [[NSMutableDictionary alloc] init];
        resp[@"allGamesData"] = allGamesDataFinal;
        resp[@"crossGameProgress"] = crossGameProgressFinal;
        resp[@"game_id"] = [NSNumber numberWithLong:gameIdFinal];
        resp[@"last_update"] = [NSNumber numberWithLong:lastUpdateValue];
        [Util sendJSCallBack:@"questsObj.executingFunctionsAfterGettingSharedData" msg:[Util JSONToString:resp]];
        
        NSMutableDictionary* flutterData = [[NSMutableDictionary alloc] init];
        flutterData[@"route"] = @"pspn";
        flutterData[@"action"] = @"processSharedData";
        flutterData[@"controller"] = @"quests";
        flutterData[@"params"] = [Util JSONToString:resp];
        [FlutterBridge sendDataToFlutterModule:[Util JSONToString:flutterData] isActivity:false];

    }
    @catch (NSException *exception) {
        ENError(@"%@", [exception description]);
    }
}

@end
