/**
 * This is a trans layer, it won't have a screen name, will work on functions similar to layers
 * @constructor
 */

BlitzFtueIntro = function() {

    // status for the current screen
    var isScreenActive = false;

    // array which will have all the sprites to remove
    var spritesToRemove = [];

    /**
     * this will work similar to the layer hide functions
     * it will remove all the elements on the screen pushed to arr spritesToRemove
     */
    this.hide = function () {
      for (var i in spritesToRemove) {
          if (utilObj.isNodeValid(spritesToRemove[i])) {
              utilObj.removeChild(curLayer, spritesToRemove[i]);
          }
      }
      spritesToRemove = [];
    };

    /**
     * here all the elements will be shown on the screen, all the elements will have the parentLayer as currentScreen
     */
    this.show = function () {

        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        var deviceHeightRatio = (constantsObj.DESIGN_HEIGHT + topBarOffset) / 2560;
        var deviceWidthRatio = canvasWidth / 1440;

        // stopping timer for zen
        zenModeObj.setTransLayerOpen(true);
        utilObj.hideBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));

        // making screen active
        isScreenActive = true;
        var delay = 0;

        // trans button to prevent background screen checks
        var transBg = utilObj.getScale9Sprite(constantsObj.tile_white, xPos, 0, 0.5, 0, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset);
        utilObj.setColor(transBg, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(transBg, constantsObj.OPACITY_0);
        utilObj.addChild(curLayer, transBg, zOrder);
        spritesToRemove.push(transBg);

        var transBtn = utilObj.getButton(canvasWidth,  constantsObj.DESIGN_HEIGHT + topBarOffset, 0.5, 0, xPos , 0,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzFtueIntro',
                fn: 'preventBackgroundScreenTouch',
                screen: zenModeLayer.screen,
                params : {sound : false},
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, transBtn, zOrder);
        spritesToRemove.push(transBtn);

        // ftue bg image
        var ftue_bg = utilObj.getSprite(constantsObj.blitz_ftue_intro, xPos, constantsObj.DESIGN_HEIGHT + topBarOffset, 0.5, 1, constantsObj.SCALE_ONE * 2);
        utilObj.addChild(curLayer, ftue_bg, zOrder);
        utilObj.setOpacity(ftue_bg, constantsObj.OPACITY_0);
        spritesToRemove.push(ftue_bg);

        // taking accurate height from the zeplin
        yPos -= 504 * deviceHeightRatio;

        // blitz intro header
        var blitz_header = utilObj.getSprite(constantsObj.blitz_ftue_intro_header, xPos, yPos, 0.5, 1, constantsObj.SCALE_SPRITE_NEW_1);
        utilObj.addChild(curLayer, blitz_header, zOrder);
        utilObj.setOpacity(blitz_header, constantsObj.OPACITY_0);
        spritesToRemove.push(blitz_header);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [transBg, ftue_bg, blitz_header],
            opacityArr: [constantsObj.OPACITY_90, constantsObj.OPACITY_100, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        delay += 0.15;

        // text level header for the blitz
        var currentLevel = zenModeObj.getCurrentLevel();
        yPos -= 419 * deviceHeightRatio;
        var currentLvlTxt = currentLevel;
        currentLvlTxt--;

        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_INTRO, constantsObj.TRACK_VIEW, currentLevel + "", zenModeObj.getBlitzSpeciesTracking(), "", "");

        var subtext = "Level " + currentLvlTxt;
        if (currentLvlTxt == 0) {
            subtext = "Ready for take-off!";
        }
        var headerTxt = utilObj.getBMFontLabel(subtext, constantsObj.FONT_HELVETICA_MEDIUM_110, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1);
        utilObj.addChild(curLayer, headerTxt, zOrder);
        utilObj.setOpacity(headerTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(headerTxt);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [headerTxt],
            opacityArr: [constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        delay += 0.15;

        // target variable text
        yPos -= 532 * deviceHeightRatio;
        var offset = 0;
        var offsetYPos = 0;
        if (currentLevel < constantsObj.MAX_LEVEL + 1) {

            var TextTgt = "Each letter gives a point.\n\n\n               Score:";
            offset = 90;
            offsetYPos = 80;
            if (currentLevel > constantsObj.FTUE_LEVEL_1) {
                var blitzInfo = zenModeObj.getBlitzInfo();
                var targetScore = zenModeObj.getLevelTargetPointBasedOnExp(currentLevel);
                TextTgt = "Level " + currentLevel + " Target:";
                offset = 0;
                offsetYPos = 0;
            }
            var targetTxt = utilObj.getBMFontLabel(TextTgt, constantsObj.FONT_HELVETICA_80, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos + offset + offsetYPos, 0.5, 1);
            utilObj.addChild(curLayer, targetTxt, zOrder);
            utilObj.setOpacity(targetTxt, constantsObj.OPACITY_0);
            spritesToRemove.push(targetTxt);
        }

        yPos -= 121 * deviceHeightRatio;
        if (currentLevel < constantsObj.MAX_LEVEL + 1) {
            var blitzInfo = zenModeObj.getBlitzInfo();
            var targetScore = zenModeObj.getLevelTargetPointBasedOnExp(currentLevel);
            var pointsScore = utilObj.getBMFontLabel(targetScore + " points", constantsObj.FONT_HELVETICA_MEDIUM_80, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos - offset, 0.5, 1);
            utilObj.addChild(curLayer, pointsScore, zOrder);
            utilObj.setOpacity(pointsScore, constantsObj.OPACITY_0);
            spritesToRemove.push(pointsScore);
        }

        // play button for the intro
        yPos -= 354 * deviceHeightRatio;
        var playButton = utilObj.getButton(650 * deviceWidthRatio, 208 * deviceHeightRatio, 0.5, 1, xPos , yPos,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzFtueIntro',
                fn: 'playZenMode',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, playButton, zOrder);
        spritesToRemove.push(playButton);

        var btnImage = utilObj.getScale9Sprite(constantsObj.newButton_cutup,  xPos, yPos - (104 * deviceHeightRatio), 0.5, 0.5, 650 * deviceWidthRatio, 208 * deviceHeightRatio);
        utilObj.addChild(curLayer, btnImage, zOrder);
        utilObj.setOpacity(btnImage, constantsObj.OPACITY_0);
        utilObj.setColor(btnImage, constantsObj.COLOR_INTRO_PLAY_BUTTON);
        spritesToRemove.push(btnImage);

        // pulse action for the btn
        utilObj.pulseButton(btnImage, constantsObj.PULSE_ACTION_TAG, 0);

        // play button text for intro
        yPos -= 104 * deviceHeightRatio;
        var playTxt =  utilObj.getBMFontLabel("Let's Play!", constantsObj.FONT_HELVETICA_MEDIUM_100, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 0.5);
        utilObj.addChild(curLayer, playTxt, zOrder);
        utilObj.setOpacity(playTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(playTxt);

        // pulse action for the btn txt
        utilObj.pulseButton(playTxt, constantsObj.PULSE_ACTION_TAG, 0);

        // back button UI
        var backBtn = utilObj.getButton(150 * deviceWidthRatio, 150 * deviceHeightRatio, 0, 0, 0, 0,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzFtueIntro',
                fn: 'backToGame',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, backBtn, zOrder);
        utilObj.setOpacity(backBtn, constantsObj.OPACITY_0);
        spritesToRemove.push(backBtn);

        // back arrow image
        var arrow = utilObj.getSprite(constantsObj.arrow_01, zenModeLayer.arrow.getPositionX(), zenModeLayer.arrow.getPositionY(), 0.5, 0.5, constantsObj.SCALE_SPRITE_24);
        utilObj.setColor(arrow, constantsObj.COLOR_WHITE);
        utilObj.addChild(curLayer, arrow, zOrder);
        utilObj.setOpacity(arrow, constantsObj.OPACITY_0);
        spritesToRemove.push(arrow);

        var backTxt =  utilObj.getBMFontLabel("Back", constantsObj.FONT_HELVETICA_MEDIUM_40, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), zenModeLayer.arrow.getPositionX(), zenModeLayer.arrow.getPositionY() - (arrow.getContentSize().height * constantsObj.SCALE_SPRITE_24) / 2 - 5, 0.5, 1);
        utilObj.addChild(curLayer, backTxt, zOrder);
        utilObj.setOpacity(backTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(backTxt);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [targetTxt, pointsScore, btnImage, playTxt, backBtn, arrow, backTxt],
            opacityArr: [constantsObj.OPACITY_70, constantsObj.OPACITY_90, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        // triggers the fade in animation for the UI, not used now
        // blitzFtueIntro.playFadeInAnimation(transBg, targetTxt);

    };

    /**
     * starts the gamePlay for the blitz
     */
    this.playZenMode = function () {
        var currentLevel = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_INTRO, constantsObj.TRACK_PLAY, currentLevel + "", zenModeObj.getBlitzSpeciesTracking(), "", "");

        zenModeObj.setTransLayerOpen(false);
        blitzFtueIntro.hide();
        zenModeObj.playStart();
        utilObj.showBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));
    };

    /**
     * returns if the cur screen is active
     */
    this.getIsScreenActive = function () {
        return isScreenActive;
    };

    /**
     * triggers fade in animation for the UI
     */
    this.playFadeInAnimation = function (transBg, targetTxt) {
        isScreenActive = false;
        for(var i in spritesToRemove) {
            var fadeIn = animObj.getFadeIn(0.15).easing(new cc.easeIn(1.0));
            var fadeIn90 = animObj.getFadeTo(0.15, constantsObj.OPACITY_90).easing(new cc.easeIn(1.0));;
            var fadeIn50 = animObj.getFadeTo(0.15, constantsObj.OPACITY_50).easing(new cc.easeIn(1.0));;
            if (utilObj.isNodeValid(spritesToRemove[i])) {
                if (spritesToRemove[i] == transBg) {
                    animObj.runAction(spritesToRemove[i], fadeIn90);
                } else if (spritesToRemove[i] == targetTxt) {
                    animObj.runAction(spritesToRemove[i], fadeIn50);
                } else {
                    animObj.runAction(spritesToRemove[i], fadeIn);
                }
            }
        }
    };

    /**
     * triggers fade in animation for the UI
     */
    this.playFadeInAnimation = function (sender, params) {
        var arr = params.arr;
        var opacityArr = params.opacityArr;
        var fadeInDur = params.fadeInDur;
        for(var i in arr) {
            var fadeIn = animObj.getFadeTo(fadeInDur, opacityArr[i]).easing(new cc.easeIn(1.0));
            if (utilObj.isNodeValid(arr[i])) {
                animObj.runAction(arr[i], fadeIn);
            }
        }
    };

    /**
     * returns to back to the home
     */
    this.backToGame = function () {
        var currentLevel = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_INTRO, constantsObj.TRACK_BACK, currentLevel + "", zenModeObj.getBlitzSpeciesTracking(), "", "");

        isScreenActive = false;
        zenModeObj.setTransLayerOpen(false);
        zenModeLayer.hide();
        zenModeObj.setStopTimer(false,false);
        homeLayer.show();
    };

    // function to do nothing
    this.preventBackgroundScreenTouch = function () {
        // do nothing -> rest here xD
    };



};