/**
 * This is a trans layer, it won't have a screen name, will work on functions similar to layers
 * @constructor
 */

BlitzIntro = function() {

    // status for the current screen
    var isScreenActive = false;

    // array which will have all the sprites to remove
    var spritesToRemove = [];

    /**
     * this will work similar to the layer hide functions
     * it will remove all the elements on the screen pushed to arr spritesToRemove
     */
    this.hide = function () {
        for (var i in spritesToRemove) {
            if (utilObj.isNodeValid(spritesToRemove[i])) {
                utilObj.removeChild(curLayer, spritesToRemove[i]);
            }
        }
        spritesToRemove = [];
    };

    /**
     * here all the elements will be shown on the screen, all the elements will have the parentLayer as currentScreen
     */
    this.show = function () {

        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        var deviceHeightRatio = (constantsObj.DESIGN_HEIGHT + topBarOffset) / 2560;
        var deviceWidthRatio = canvasWidth / 1440;

        var spriteArr = [];
        var labelArr = [];

        // stopping timer for zen
        zenModeObj.setTransLayerOpen(true);
        utilObj.hideBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));

        // making screen active
        isScreenActive = true;
        var delay = 0;

        // trans button to prevent background screen checks
        var transBg = utilObj.getScale9Sprite(constantsObj.tile_white, xPos, 0, 0.5, 0, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset, spriteArr, curLayer, zOrder - 1, constantsObj.SPRITE_MISC);
        utilObj.setColor(transBg, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(transBg, constantsObj.OPACITY_0);
        // utilObj.addChild(curLayer, transBg, zOrder);
        spritesToRemove.push(transBg);

        var transBtn = utilObj.getButton(canvasWidth,  constantsObj.DESIGN_HEIGHT + topBarOffset, 0.5, 0, xPos , 0,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzFtueIntro',
                fn: 'preventBackgroundScreenTouch',
                screen: zenModeLayer.screen,
                params : {sound : false},
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, transBtn, zOrder);
        spritesToRemove.push(transBtn);

        // ftue bg image
        var ftue_bg = utilObj.getSprite(constantsObj.blitz_ftue_intro, xPos, constantsObj.DESIGN_HEIGHT + topBarOffset, 0.5, 1, constantsObj.SCALE_ONE * 2, spriteArr, curLayer, zOrder - 1, constantsObj.SPRITE_LOCAL);
        // utilObj.addChild(curLayer, ftue_bg, zOrder);
        utilObj.setOpacity(ftue_bg, constantsObj.OPACITY_0);
        spritesToRemove.push(ftue_bg);

        // taking accurate height from the zeplin
        yPos -= 53 * deviceHeightRatio;

        // blitz intro header
        var blitz_header = utilObj.getSprite(constantsObj.blitz_ftue_intro_header, xPos, yPos, 0.5, 1, constantsObj.SCALE_SPRITE_NEW_1, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        // utilObj.addChild(curLayer, blitz_header, zOrder);
        utilObj.setOpacity(blitz_header, constantsObj.OPACITY_0);
        spritesToRemove.push(blitz_header);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [transBg, ftue_bg, blitz_header],
            opacityArr: [constantsObj.OPACITY_90, constantsObj.OPACITY_100, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);



        delay += 0.15;

        // text level header for the blitz
        var currentLevel = zenModeObj.getCurrentLevel();
        yPos -= 293 * deviceHeightRatio;
        var currentLvlTxt = currentLevel;
        currentLvlTxt--;
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_INTRO, constantsObj.TRACK_VIEW, currentLevel + "", zenModeObj.getBlitzSpeciesTracking(), "", "");

        var headerTxt = utilObj.getBMFontLabel("Level " + currentLvlTxt, constantsObj.FONT_HELVETICA_MEDIUM_110, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, curLayer, zOrder);
        // utilObj.addChild(curLayer, headerTxt, zOrder);
        utilObj.setOpacity(headerTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(headerTxt);


        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [headerTxt],
            opacityArr: [constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        delay += 0.15;

        // Badge Image

        var circleSprite_badge1 = utilObj.getSprite(constantsObj.white_circle_150, xPos, yPos - (550 * deviceHeightRatio), 0.5, 0.5, constantsObj.SCALE_SPRITE_3 * 2.22, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        circleSprite_badge1.setOpacity(constantsObj.OPACITY_0);
        circleSprite_badge1.setColor(constantsObj.BLITZ_LEVEL_INFO[zenModeObj.getCurrentLevel() - 1]["badgeColour"]);
        // utilObj.addChild(curLayer, circleSprite_badge1, zOrder);
        spritesToRemove.push(circleSprite_badge1);

        var circleSprite_badge2 = utilObj.getSprite(constantsObj.white_circle_150, xPos, yPos - (550 * deviceHeightRatio), 0.5, 0.5, constantsObj.SCALE_SPRITE_3 * 2.75, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        circleSprite_badge2.setOpacity(constantsObj.OPACITY_0);
        circleSprite_badge2.setColor(constantsObj.BLITZ_LEVEL_INFO[zenModeObj.getCurrentLevel() - 1]["badgeColour"]);
        // utilObj.addChild(curLayer, circleSprite_badge2, zOrder);
        spritesToRemove.push(circleSprite_badge2);



        var badgeImage = utilObj.getSprite(constantsObj.BLITZ_LEVEL_INFO[currentLvlTxt]["badgeImage"], xPos, yPos - (550 * deviceHeightRatio), 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_112, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        utilObj.setOpacity(badgeImage, constantsObj.OPACITY_0);
        // utilObj.addChild(curLayer, badgeImage, zOrder);
        spritesToRemove.push(badgeImage);


        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [circleSprite_badge1, circleSprite_badge2, badgeImage],
            opacityArr: [constantsObj.OPACITY_10, constantsObj.OPACITY_10, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        delay += 0.15;

        // ------------------------


        // target variable text
        yPos -= 980 * deviceHeightRatio;
        if (currentLevel < constantsObj.MAX_LEVEL + 1) {
            var blitzInfo = zenModeObj.getBlitzInfo();
            var targetScore = zenModeObj.getLevelTargetPointBasedOnExp(currentLevel);
            var targetLevel = currentLvlTxt + 1;

            var targetTxt = utilObj.getBMFontLabel("Level " + targetLevel + " Target:", constantsObj.FONT_HELVETICA_REGULAR_65, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, curLayer, zOrder);
            // utilObj.addChild(curLayer, targetTxt, zOrder);
            utilObj.setOpacity(targetTxt, constantsObj.OPACITY_0);
            spritesToRemove.push(targetTxt);
        } else {
            var targetTxt = utilObj.getBMFontLabel("Best Score:", constantsObj.FONT_HELVETICA_REGULAR_65, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, curLayer, zOrder);
            // utilObj.addChild(curLayer, targetTxt, zOrder);
            utilObj.setOpacity(targetTxt, constantsObj.OPACITY_0);
            spritesToRemove.push(targetTxt);
        }

        yPos -= 100 * deviceHeightRatio;
        if (currentLevel < constantsObj.MAX_LEVEL + 1) {
            var pointsScore = utilObj.getBMFontLabel("Score " + targetScore + " points!", constantsObj.FONT_HELVETICA_MEDIUM_65, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, curLayer, zOrder);
            // utilObj.addChild(curLayer, pointsScore, zOrder);
            utilObj.setOpacity(pointsScore, constantsObj.OPACITY_0);
            spritesToRemove.push(pointsScore);
        } else {
            var pointsScore = utilObj.getBMFontLabel(zenModeObj.getMaxScore() + "", constantsObj.FONT_HELVETICA_MEDIUM_80, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, curLayer, zOrder);
            // utilObj.addChild(curLayer, pointsScore, zOrder);
            utilObj.setOpacity(pointsScore, constantsObj.OPACITY_0);
            spritesToRemove.push(pointsScore);
        }


        // ------------------------

        // level scroll UI

        var scrollViewWidth = blitzLevelHit.calcScrollViewWidth(deviceWidthRatio);

        var scrollView = new ccui.ScrollView();
        scrollView.setDirection(ccui.ScrollView.DIR_HORIZONTAL);
        scrollView.setTouchEnabled(true);
        scrollView.setContentSize(cc.size(canvasWidth, 800 * deviceHeightRatio));
        scrollView.setPosition(canvasWidth, yPos - (225 * deviceHeightRatio));
        scrollView.setAnchorPoint(0, 0.5);
        scrollView.setBackGroundColor(constantsObj.COLOR_RED_01);
        scrollView.setInnerContainerSize(cc.size(scrollViewWidth, 800 * deviceHeightRatio));
        scrollView.setBounceEnabled(true);
        scrollView.setScrollBarEnabled(false);
        scrollView.setOpacity(constantsObj.OPACITY_50);

        utilObj.addChild(curLayer, scrollView, zOrder);
        spritesToRemove.push(scrollView);


        var xPosScroll = canvasWidth/2;
        var yPosScroll = 400 * deviceHeightRatio;


        // setting the scrollView UI
        var numberOfLevelsToMake = constantsObj.TOTAL_LEVELS_FOR_SCROLL;
        var startLevel = constantsObj.STARTING_LEVEL_FOR_SCROLL;
        var currentLevel = zenModeObj.getCurrentLevel();
        var maxLevel = constantsObj.MAX_LEVEL;

        // store position of the badges
        var levelBadgesArr = [];

        for (var i = 0; i < numberOfLevelsToMake; i++) {

            // init with the image for the badge if completed or not
            var imageToUseForUse = constantsObj.BLITZ_LEVEL_INFO[startLevel]["badgeImage_grey"];
            if (startLevel < currentLevel) {
                imageToUseForUse  = constantsObj.BLITZ_LEVEL_INFO[startLevel]["badgeImage"];
            }

            // badge image for the levels
            // its locked or is opened according to current level
            var badge = utilObj.getSprite(imageToUseForUse, xPosScroll, yPosScroll, 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_79, spriteArr, scrollView, zOrder, constantsObj.SPRITE_MISC);
            // utilObj.addChild(scrollView, badge, zOrder);

            levelBadgesArr.push(badge);
            var badgeHalfWidth = (badge.getContentSize().width * constantsObj.SCALE_SPRITE_NEW_79)/2;
            var badgeHalfHeight = (badge.getContentSize().height * constantsObj.SCALE_SPRITE_NEW_79)/2;

            loggerObj.debugLog("xPos is", startLevel, xPosScroll, deviceWidthRatio, badgeHalfWidth);

            // lock image
            // if (startLevel >= currentLevel) {
            //     var lock = utilObj.getSprite(constantsObj.blitz_lock, badge.getContentSize().width/2, badge.getContentSize().height/2 + 10, 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_85, spriteArr, badge, zOrder, constantsObj.SPRITE_MISC);
            //     utilObj.setColor(lock, constantsObj.COLOR_GREY_07);
            //     // utilObj.addChild(badge, lock, zOrder);
            // }

            // level number
            var yPosLevel = yPosScroll - (140 * deviceHeightRatio);
            var levelN = utilObj.getBMFontLabel("Level " + startLevel, constantsObj.FONT_HELVETICA_60, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPosScroll, yPosLevel, 0.5, 1, labelArr, scrollView, zOrder);
            // utilObj.addChild(scrollView, levelN, zOrder);

            if (startLevel < currentLevel) {
                levelN.setPosition(xPosScroll - (50 * deviceWidthRatio), yPosLevel);
                var circleBg = utilObj.getSprite(constantsObj.white_bg, xPosScroll + (135 * deviceWidthRatio), yPosLevel - (32.5 * deviceHeightRatio), 0.5, 0.5, 0.22, spriteArr, scrollView, zOrder, constantsObj.SPRITE_MISC);
                // utilObj.addChild(scrollView, circleBg, zOrder);
                utilObj.setColor(circleBg, constantsObj.COLOR_GREEN_11);

                var tick =  utilObj.getSprite(constantsObj.tick, circleBg.getContentSize().width / 2, circleBg.getContentSize().height / 2, 0.5, 0.5, constantsObj.SCALE_SPRITE_22 * 0.95, spriteArr, circleBg, zOrder, constantsObj.SPRITE_MISC);
                // utilObj.addChild(circleBg, tick, zOrder);
            }

            // progress line in between
            if (startLevel != maxLevel) {
                var progressBar = null;

                // if..else check for the width and the animation part of the progress bar
                // this will be used to resize the width of the progress bar also afterwards

                progressBar = utilObj.getScale9Sprite(constantsObj.full_bar, xPosScroll, yPosScroll, 0, 0.5, (668.5 + badgeHalfWidth) * deviceWidthRatio, 6 * deviceHeightRatio, spriteArr, scrollView, zOrder - 1, constantsObj.SPRITE_MISC);
                // utilObj.addChild(scrollView, progressBar, zOrder - 1);
                progressBar.setColor(constantsObj.COLOR_GREY_14);

                if (startLevel < currentLevel - 1) {
                    progressBar.setColor(constantsObj.COLOR_GREEN_08);
                }
            }

            xPosScroll += (badge.getContentSize().width * constantsObj.SCALE_SPRITE_NEW_79)/2;
            xPosScroll += 585 * deviceWidthRatio;
            xPosScroll += 84 * deviceWidthRatio;

            // incrementing as the levels are done
            startLevel++;

        }

        // scroll it to the exact pos initially always
        var badgeToScrollTo = currentLevel - constantsObj.STARTING_LEVEL_FOR_SCROLL;
        var xPosToScrollInScrollView = levelBadgesArr[badgeToScrollTo - 1].getPositionX();
        if (!(currentLevel == constantsObj.MAX_LEVEL + 1)) {
            xPosToScrollInScrollView = levelBadgesArr[badgeToScrollTo - 1].getPositionX() + (levelBadgesArr[badgeToScrollTo].getPositionX() - levelBadgesArr[badgeToScrollTo - 1].getPositionX()) /2;
        }
        xPosToScrollInScrollView -= canvasWidth/2;

        var percentScroll = ( xPosToScrollInScrollView / (scrollViewWidth - canvasWidth)) * 100;

        if (percentScroll < 0) {
            percentScroll = 0;
        }
        scrollView.jumpToPercentHorizontal(percentScroll);

        // ---------------------------------------

        // scrollview fade in and move in animation to the screen
        var moveByAnimDur = 0.35;
        var delayT = utilObj.getDelayTime(delay);
        var moveBy = utilObj.getMoveBy(moveByAnimDur, -canvasWidth, 0).easing(cc.easeIn(1.0));
        var fadeIn = utilObj.getFadeIn(moveByAnimDur).easing(cc.easeIn(1.0));
        var spawn = new cc.Spawn(moveBy, fadeIn);
        animObj.runAction(scrollView, utilObj.getSequence([delayT, spawn]));

        delay += moveByAnimDur;

        // ------------------------

        // play button for the intro
        yPos -= 525 * deviceHeightRatio;
        var playButton = utilObj.getButton(650 * deviceWidthRatio, 208 * deviceHeightRatio, 0.5, 1, xPos , yPos,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzIntro',
                fn: 'playZenMode',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, playButton, zOrder);
        spritesToRemove.push(playButton);

        var btnImage = utilObj.getScale9Sprite(constantsObj.newButton_cutup,  xPos, yPos - (104 * deviceHeightRatio), 0.5, 0.5, 650 * deviceWidthRatio, 208 * deviceHeightRatio, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        // utilObj.addChild(curLayer, btnImage, zOrder);
        utilObj.setOpacity(btnImage, constantsObj.OPACITY_0);
        utilObj.setColor(btnImage, constantsObj.COLOR_INTRO_PLAY_BUTTON);
        spritesToRemove.push(btnImage);

        // pulse action for the btn
        utilObj.pulseButton(btnImage, constantsObj.PULSE_ACTION_TAG, 0);

        // play button text for intro
        yPos -= 104 * deviceHeightRatio;
        var playTxt =  utilObj.getBMFontLabel("Let's Play!", constantsObj.FONT_HELVETICA_MEDIUM_100, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 0.5, labelArr, curLayer, zOrder );
        // utilObj.addChild(curLayer, playTxt, zOrder);
        utilObj.setOpacity(playTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(playTxt);

        // pulse action for the btn txt
        utilObj.pulseButton(playTxt, constantsObj.PULSE_ACTION_TAG, 0);

        // back button UI
        var backBtn = utilObj.getButton(150 * deviceWidthRatio, 150 * deviceHeightRatio, 0, 0, 0, 0,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzIntro',
                fn: 'backToGame',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, backBtn, zOrder);
        utilObj.setOpacity(backBtn, constantsObj.OPACITY_0);
        spritesToRemove.push(backBtn);

        // back arrow image
        var arrow = utilObj.getSprite(constantsObj.arrow_01, zenModeLayer.arrow.getPositionX(), zenModeLayer.arrow.getPositionY(), 0.5, 0.5, constantsObj.SCALE_SPRITE_24, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        utilObj.setColor(arrow, constantsObj.COLOR_WHITE);
        // utilObj.addChild(curLayer, arrow, zOrder);
        utilObj.setOpacity(arrow, constantsObj.OPACITY_0);
        spritesToRemove.push(arrow);

        var backTxt =  utilObj.getBMFontLabel("Back", constantsObj.FONT_HELVETICA_MEDIUM_40, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), zenModeLayer.arrow.getPositionX(), zenModeLayer.arrow.getPositionY() - (arrow.getContentSize().height * constantsObj.SCALE_SPRITE_24) / 2  - 5, 0.5, 1, labelArr, curLayer, zOrder );
        // utilObj.addChild(curLayer, backTxt, zOrder);
        utilObj.setOpacity(backTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(backTxt);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [targetTxt, pointsScore, btnImage, playTxt, backBtn, arrow, backTxt],
            opacityArr: [constantsObj.OPACITY_70, constantsObj.OPACITY_90, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);

        // triggers the fade in animation for the UI, not used now
        // blitzFtueIntro.playFadeInAnimation(transBg, targetTxt);

    };

    /**
     * starts the gamePlay for the blitz
     */
    this.playZenMode = function () {
        var currentLevel = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_INTRO, constantsObj.TRACK_PLAY, currentLevel + "", zenModeObj.getBlitzSpeciesTracking(), "", "");
        zenModeObj.setTransLayerOpen(false);
        blitzIntro.hide();
        zenModeObj.playStart();

        utilObj.showBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));
    };

    /**
     * returns if the cur screen is active
     */
    this.getIsScreenActive = function () {
        return isScreenActive;
    };

    /**
     * triggers fade in animation for the UI
     */
    this.playFadeInAnimation = function (transBg, targetTxt) {
        isScreenActive = false;
        for(var i in spritesToRemove) {
            var fadeIn = animObj.getFadeIn(0.15).easing(new cc.easeIn(1.0));
            var fadeIn90 = animObj.getFadeTo(0.15, constantsObj.OPACITY_90).easing(new cc.easeIn(1.0));;
            var fadeIn50 = animObj.getFadeTo(0.15, constantsObj.OPACITY_50).easing(new cc.easeIn(1.0));;
            if (utilObj.isNodeValid(spritesToRemove[i])) {
                if (spritesToRemove[i] == transBg) {
                    animObj.runAction(spritesToRemove[i], fadeIn90);
                } else if (spritesToRemove[i] == targetTxt) {
                    animObj.runAction(spritesToRemove[i], fadeIn50);
                } else {
                    animObj.runAction(spritesToRemove[i], fadeIn);
                }
            }
        }
    };

    /**
     * triggers fade in animation for the UI
     */
    this.playFadeInAnimation = function (sender, params) {
        var arr = params.arr;
        var opacityArr = params.opacityArr;
        var fadeInDur = params.fadeInDur;
        for(var i in arr) {
            var fadeIn = animObj.getFadeTo(fadeInDur, opacityArr[i]).easing(new cc.easeIn(1.0));
            if (utilObj.isNodeValid(arr[i])) {
                animObj.runAction(arr[i], fadeIn);
            }
        }
    };

    /**
     * returns to back to the home
     */
    this.backToGame = function () {
        var currentLevel = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_INTRO, constantsObj.TRACK_BACK, currentLevel + "", zenModeObj.getBlitzSpeciesTracking(), "", "");
        isScreenActive = false;
        zenModeObj.setTransLayerOpen(false);
        zenModeLayer.hide();
        zenModeObj.setStopTimer(false,false);
        homeLayer.show();
    };

    // function to do nothing
    this.preventBackgroundScreenTouch = function () {
        // do nothing -> rest here xD
    };



};