/**
 * called whenever a level is hit by the user
 * will handle ftue outro, badges outro and milestone animation at the time of level hit
 * @constructor
 */

BlitzLevelHit = function () {

    // status for the current screen
    var isScreenActive = false;
    var isAnimRunning = false;

    // array which will have all the sprites to remove
    var spritesToRemove = [];
    var transBg = null;

    /**
     * this will work similar to the layer hide functions
     * it will remove all the elements on the screen pushed to arr spritesToRemove
     */
    this.hide = function () {
        for (var i in spritesToRemove) {
            if (utilObj.isNodeValid(spritesToRemove[i])) {
                utilObj.removeChild(curLayer, spritesToRemove[i]);
            }
        }
        spritesToRemove = [];
    };

    /**
     * here all the elements will be shown on the screen, all the elements will have the parentLayer as currentScreen
     */
    this.show = function () {

        var zOrder = constantsObj.FTUE_Z_ORDER;
        var currentLevel = zenModeObj.getCurrentLevel();
        if (currentLevel != constantsObj.FTUE_LEVEL_1 && currentLevel != constantsObj.FTUE_LEVEL_2 && currentLevel != constantsObj.FTUE_LEVEL_3) {
            zenModeObj.addLife(1,"targetHit");
        }
        utilObj.hideBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));
        zenModeObj.setTransLayerOpen(true);
        blitzLevelHit.setCheckAnimRunning(false);

        var bonusTimeAdded = zenModeObj.getBonusTimeAdded();
        var startTime = zenModeObj.getMaxBufferTimer();
        var bufferTime = zenModeObj.getCurrentBufferTime();
        var highestStreakAchieved = zenModeObj.getHighestStreakAchieved();
        var currentLives = zenModeObj.getLivesCount();
        var currentScore = zenModeObj.getCurrentScore();
        var totalRewardCount = zenModeObj.getTotalRewardCount();
        var animDelay = 0;

        // fetches the current level of the user
        // Level 2 -> ftue outro
        // Level 3 -> reward milestone , ftue outro
        // Level 4 -> reward milestone, badge animation
        var currentLevel = zenModeObj.getCurrentLevel();

        animDelay =  blitzLevelHit.AmazingGratification(currentLevel - 1, zOrder);

        var totalTime = startTime - bufferTime + bonusTimeAdded;
        trackObj.trackCounter(constantsObj.TRACK_PUZZLE, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_CLEAR, totalTime + "_" + bonusTimeAdded + "_" + highestStreakAchieved, currentScore + "_" + currentLives + "_" + (currentLevel - 1), zenModeObj.getBlitzSpeciesTracking(), "", "");

        // background which is same for every levels
        // trans bg for the background
        transBg = utilObj.getScale9Sprite(constantsObj.tile_white, centerXPos, 0, 0.5, 0, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset);
        utilObj.setColor(transBg, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(transBg, constantsObj.OPACITY_0);
        utilObj.addChild(curLayer, transBg, zOrder - 2);
        spritesToRemove.push(transBg);

        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.REWARD_OVERLAY_FADEIN,
            animDelay: animDelay,
            node: transBg,
            animDur: 0.15,
            opacity: constantsObj.OPACITY_90
        });
        animDelay += 0.15;

        // trans button to prevent background screen checks
        var transBtn = utilObj.getButton(canvasWidth,  constantsObj.DESIGN_HEIGHT + topBarOffset, 0.5, 0, centerXPos , 0,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzFtueIntro',
                fn: 'preventBackgroundScreenTouch',
                screen: zenModeLayer.screen,
                params : {sound : false},
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, transBtn, zOrder);
        spritesToRemove.push(transBtn);

        if (currentLevel == constantsObj.FTUE_LEVEL_2) {
            /* triggers the ftue outro flow
             @args -> passing initial delay as it can be triggered after any other functions
             after clearing level 1
            */
            utilObj.delayedCallback(curLayer, blitzLevelHit.showFtueLevelHitOutro, animDelay)
        } else if (currentLevel == constantsObj.FTUE_LEVEL_3) {
            /*
            level drop animation after clearing level 2 and 3 (TODO for the 3 level clear anim)
             */
            animDelay = blitzLevelHit.rewardUnlockAnim(animDelay);
            totalRewardCount += currentScore;
            zenModeObj.setTotalRewardCount(totalRewardCount);
            utilObj.delayedCallback(curLayer, blitzLevelHit.showFtueLevelHitOutro, animDelay)
        } else {
            /*
            after ftue levels animation
             */
            animDelay = blitzRetryLayer.rewardAnimation(animDelay);
            totalRewardCount += currentScore;
            if (currentLevel == 4 ) {
                utilObj.delayedCallback(curLayer, blitzLevelHit.showLevelScrollAnimation, animDelay, {introToBadge: true})
            } else {
                utilObj.delayedCallback(curLayer, blitzLevelHit.showLevelScrollAnimation, animDelay, {});
            }
        }
    };

    /**
     * this will start the level start animation
     */
    this.showLevelScrollAnimation = function (sender, params) {

        // intro to badge case
        var introToBadge = false;
        if (params.introToBadge) {
            introToBadge = true;
        }

        var spriteArr = [];
        var labelArr = [];

        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_BADGE_ANIM, constantsObj.TRACK_VIEW, "", zenModeObj.getBlitzSpeciesTracking(), "", "");

        var zOrder = constantsObj.FTUE_Z_ORDER;
        var deviceHeightRatio = (constantsObj.DESIGN_HEIGHT + topBarOffset) / 2560;
        var deviceWidthRatio = (canvasWidth / 1440).toFixed(2);
        deviceWidthRatio = Math.round(deviceWidthRatio * 10)/ 10;
        var scrollViewWidth = blitzLevelHit.calcScrollViewWidth(deviceWidthRatio);
        var screenHeight = constantsObj.DESIGN_HEIGHT + topBarOffset;
        blitzLevelHit.setCheckAnimRunning(true);

        /* for locked level badge anim
        * ripple animation sprites declaration
        */
        var circleSprite = utilObj.getSprite(constantsObj.white_circle_150, centerXPos, centerYPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_9, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        circleSprite.setOpacity(constantsObj.OPACITY_0);
        // utilObj.addChild(curLayer, circleSprite, zOrder);
        spritesToRemove.push(circleSprite);

        var circleSprite1 = utilObj.getSprite(constantsObj.white_circle_150, centerXPos, centerYPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_9, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        circleSprite1.setOpacity(constantsObj.OPACITY_0);
        // utilObj.addChild(curLayer, circleSprite1, zOrder);
        spritesToRemove.push(circleSprite1);

        var circleSprite2 = utilObj.getSprite(constantsObj.white_circle_150, centerXPos, centerYPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_9, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        circleSprite2.setOpacity(constantsObj.OPACITY_0);
        // utilObj.addChild(curLayer, circleSprite2, zOrder);
        spritesToRemove.push(circleSprite2);


        // for unlocked level badge anim
        var circleSprite_badge = utilObj.getSprite(constantsObj.white_circle_150, centerXPos, 3 * screenHeight/ 4, 0.5, 0.5, constantsObj.SCALE_SPRITE_9 * 1.5, spriteArr, curLayer, zOrder - 2, constantsObj.SPRITE_MISC);
        circleSprite_badge.setOpacity(constantsObj.OPACITY_0);
        circleSprite_badge.setColor(constantsObj.BLITZ_LEVEL_INFO[zenModeObj.getCurrentLevel() - 1]["badgeColour"]);
        // utilObj.addChild(curLayer, circleSprite_badge, zOrder - 2);
        spritesToRemove.push(circleSprite_badge);

        var circleSprite_badge1 = utilObj.getSprite(constantsObj.white_circle_150, centerXPos, 3 * screenHeight/ 4, 0.5, 0.5, constantsObj.SCALE_SPRITE_9 * 1.5, spriteArr, curLayer, zOrder - 2, constantsObj.SPRITE_MISC);
        circleSprite_badge1.setOpacity(constantsObj.OPACITY_0);
        circleSprite_badge1.setColor(constantsObj.BLITZ_LEVEL_INFO[zenModeObj.getCurrentLevel() - 1]["badgeColour"]);
        // utilObj.addChild(curLayer, circleSprite_badge1, zOrder - 2);
        spritesToRemove.push(circleSprite_badge1);

        var circleSprite_badge2 = utilObj.getSprite(constantsObj.white_circle_150, centerXPos, 3 * screenHeight/ 4, 0.5, 0.5, constantsObj.SCALE_SPRITE_9 * 1.5, spriteArr, curLayer, zOrder - 2, constantsObj.SPRITE_MISC);
        circleSprite_badge2.setOpacity(constantsObj.OPACITY_0);
        circleSprite_badge2.setColor(constantsObj.BLITZ_LEVEL_INFO[zenModeObj.getCurrentLevel() - 2]["badgeColour"]);
        // utilObj.addChild(curLayer, circleSprite_badge2, zOrder - 2);
        spritesToRemove.push(circleSprite_badge2);

        // ----------------------

        // header weel done text
        var WellDoneTxt = utilObj.getBMFontLabel("Well done!", constantsObj.FONT_HELVETICA_MEDIUM_120, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, screenHeight - (191 * deviceHeightRatio) , 0.5, 0.5, labelArr, curLayer, zOrder);
        WellDoneTxt.setOpacity(constantsObj.OPACITY_0);
        // utilObj.addChild(curLayer, WellDoneTxt, zOrder);

        // animate well done text
        var fadeInWellDone = utilObj.getFadeTo(0.25, constantsObj.OPACITY_90).easing(new cc.easeIn(1.0));
        var scaleUpWellDoneTxt = new cc.ScaleBy(0.25, 1.2, 1.2).easing(new cc.easeIn(1.0));
        var scaleDownDoneTxt = new cc.ScaleBy(0.1, 0.84, 0.84).easing(new cc.easeIn(1.0));
        var spawn = new cc.Spawn(fadeInWellDone, scaleUpWellDoneTxt);
        animObj.runAction(WellDoneTxt, utilObj.getSequence([spawn, scaleDownDoneTxt]));

        // -----------------------

        // cross button
        var back_btn = utilObj.getSprite(constantsObj.close_icon2, xRightPos - (50 * deviceWidthRatio), screenHeight - (52 * deviceHeightRatio), 1, 1, constantsObj.SCALE_SPRITE_43, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        // utilObj.addChild(curLayer, back_btn, zOrder);
        utilObj.setOpacity(back_btn, constantsObj.OPACITY_0);
        spritesToRemove.push(back_btn);

        var closeBtn = utilObj.getButton(100 * deviceWidthRatio, 100 * deviceHeightRatio, 1, 1, xRightPos - (35 * deviceWidthRatio) , screenHeight - (36 * deviceHeightRatio),
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzLevelHit',
                fn: 'close',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, closeBtn, zOrder);
        utilObj.setOpacity(closeBtn, constantsObj.OPACITY_0);
        spritesToRemove.push(closeBtn);

         // -----------------------


        /**
         * scrollview which will have all the elements on it and is scrollable after the animation
         */
        var scrollView = new ccui.ScrollView();
        scrollView.setDirection(ccui.ScrollView.DIR_HORIZONTAL);
        scrollView.setTouchEnabled(false);
        scrollView.setContentSize(cc.size(canvasWidth, 800 * deviceHeightRatio));
        scrollView.setPosition(canvasWidth, centerYPos);
        scrollView.setAnchorPoint(0, 0.5);
        scrollView.setBackGroundColor(constantsObj.COLOR_RED_01);
        scrollView.setInnerContainerSize(cc.size(scrollViewWidth, 800 * deviceHeightRatio));
        scrollView.setBounceEnabled(false);
        scrollView.setScrollBarEnabled(false);
        scrollView.setOpacity(constantsObj.OPACITY_50);

        utilObj.addChild(curLayer, scrollView, zOrder);
        spritesToRemove.push(scrollView);

        var xPos = canvasWidth/2;
        var yPos = 400 * deviceHeightRatio;


        // setting the scrollView UI
        var numberOfLevelsToMake = constantsObj.TOTAL_LEVELS_FOR_SCROLL;
        var startLevel = constantsObj.STARTING_LEVEL_FOR_SCROLL;
        var currentLevel = zenModeObj.getCurrentLevel();
        currentLevel--;
        var maxLevel = constantsObj.MAX_LEVEL;

        // store position of the badges
        var levelBadgesArr = [];

        //progress Bar green one to animate
        var progressBarBg = null;
        var progressBarGreen = null;
        var levelBadgeImageForAnim = null;
        var levelNumberForAnim = null;
        var circleBgForAnim = null;
        var levelbadgeAtCenter = null;

        for (var i = 0; i < numberOfLevelsToMake; i++) {

            // init with the image for the badge if completed or not
            var imageToUseForUse = constantsObj.BLITZ_LEVEL_INFO[startLevel]["badgeImage_grey"];
            if (startLevel < currentLevel) {
                imageToUseForUse  = constantsObj.BLITZ_LEVEL_INFO[startLevel]["badgeImage"];
            }

            // badge image for the levels
            // its locked or is opened according to current level
            var badge = utilObj.getSprite(imageToUseForUse, xPos, yPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_79, spriteArr, scrollView, zOrder, constantsObj.SPRITE_MISC);
            // utilObj.addChild(scrollView, badge, zOrder);

            levelBadgesArr.push(badge);
            var badgeHalfWidth = (badge.getContentSize().width * constantsObj.SCALE_SPRITE_NEW_79)/2;
            var badgeHalfHeight = (badge.getContentSize().height * constantsObj.SCALE_SPRITE_NEW_79)/2;

            loggerObj.debugLog("xPos is", startLevel, xPos, deviceWidthRatio, badgeHalfWidth);

            // lock image
            // if (startLevel >= currentLevel) {
            //     var lock = utilObj.getSprite(constantsObj.blitz_lock, badge.getContentSize().width/2, badge.getContentSize().height/2 + 10, 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_85, spriteArr, badge, zOrder, constantsObj.SPRITE_MISC);
            //     utilObj.setColor(lock, constantsObj.COLOR_GREY_07);
            //     // utilObj.addChild(badge, lock, zOrder);
            // }

            // level number
            var yPosLevel = yPos - (140 * deviceHeightRatio);
            var levelN = utilObj.getBMFontLabel("Level " + startLevel, constantsObj.FONT_HELVETICA_60, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPosLevel, 0.5, 1, labelArr, scrollView, zOrder);
            // utilObj.addChild(scrollView, levelN, zOrder);

            if (startLevel < currentLevel) {
                levelN.setPosition(xPos - (50 * deviceWidthRatio), yPosLevel);
                var circleBg = utilObj.getSprite(constantsObj.white_bg, xPos + (135 * deviceWidthRatio), yPosLevel - (32.5 * deviceHeightRatio), 0.5, 0.5, 0.22, spriteArr, scrollView, zOrder, constantsObj.SPRITE_MISC);
                // utilObj.addChild(scrollView, circleBg, zOrder);
                utilObj.setColor(circleBg, constantsObj.COLOR_GREEN_11);

                var tick =  utilObj.getSprite(constantsObj.tick, circleBg.getContentSize().width / 2, circleBg.getContentSize().height / 2, 0.5, 0.5, constantsObj.SCALE_SPRITE_22 * 0.95, spriteArr, circleBg, zOrder, constantsObj.SPRITE_MISC);
                // utilObj.addChild(circleBg, tick, zOrder);
            }

            // for the aniamtion of the tick to appear after badge is unlocked
            if (startLevel == currentLevel) {
                circleBgForAnim = utilObj.getSprite(constantsObj.white_bg, xPos + (135 * deviceWidthRatio), yPosLevel - (32.5 * deviceHeightRatio), 0.5, 0.5, 0.22, spriteArr, scrollView, zOrder, constantsObj.SPRITE_MISC);
                // utilObj.addChild(scrollView, circleBgForAnim, zOrder);
                utilObj.setColor(circleBgForAnim, constantsObj.COLOR_GREEN_11);
                circleBgForAnim.setCascadeOpacityEnabled(true);

                var tick =  utilObj.getSprite(constantsObj.tick, circleBgForAnim.getContentSize().width / 2, circleBgForAnim.getContentSize().height / 2, 0.5, 0.5, constantsObj.SCALE_SPRITE_22* 0.95, spriteArr, circleBgForAnim, zOrder, constantsObj.SPRITE_MISC);
                // utilObj.addChild(circleBgForAnim, tick, zOrder);

                circleBgForAnim.setOpacity(constantsObj.OPACITY_0);

                levelNumberForAnim = levelN;
            }

            // progress line in between
            if (startLevel != maxLevel) {
                var progressBar = null;

                // if..else check for the width and the animation part of the progress bar
                // this will be used to resize the width of the progress bar also afterwards

                if (startLevel == currentLevel -1) {
                    progressBarBg = utilObj.getScale9Sprite(constantsObj.full_bar, xPos, yPos, 0, 0.5, (668.5 + badgeHalfWidth) * deviceWidthRatio, 25 * deviceHeightRatio, spriteArr, scrollView, zOrder - 1, constantsObj.SPRITE_MISC);
                    // utilObj.addChild(scrollView, progressBarBg, zOrder - 1);
                    progressBarBg.setColor(constantsObj.COLOR_GREY_14);

                    progressBar = progressBarBg;

                    progressBarGreen = utilObj.getScale9Sprite(constantsObj.full_bar, xPos, yPos, 0, 0.5, (20 + badgeHalfWidth) * deviceWidthRatio, 25 * deviceHeightRatio, spriteArr, scrollView, zOrder - 1, constantsObj.SPRITE_MISC);
                    // utilObj.addChild(scrollView, progressBarGreen, zOrder - 1);
                    progressBarGreen.setColor(constantsObj.COLOR_GREEN_08);
                } else {
                    progressBar = utilObj.getScale9Sprite(constantsObj.full_bar, xPos, yPos, 0, 0.5, (668.5 + badgeHalfWidth) * deviceWidthRatio, 6 * deviceHeightRatio, spriteArr, scrollView, zOrder - 1, constantsObj.SPRITE_MISC);
                    // utilObj.addChild(scrollView, progressBar, zOrder - 1);
                    progressBar.setColor(constantsObj.COLOR_GREY_14);
                }

                if (startLevel < currentLevel - 1) {
                    progressBar.setColor(constantsObj.COLOR_GREEN_08);
                }
            }

            // image for the currentLevel Badge for the animation
            if (startLevel == currentLevel) {
                levelBadgeImageForAnim = utilObj.getSprite(constantsObj.BLITZ_LEVEL_INFO[startLevel]["badgeImage"], xPos, yPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_79 * 3.5, spriteArr, scrollView, zOrder, constantsObj.SPRITE_MISC);
                levelBadgeImageForAnim.setScale(0, constantsObj.SCALE_SPRITE_NEW_79 * 3.5);
                levelBadgeImageForAnim.setOpacity(constantsObj.OPACITY_0);
                // utilObj.addChild(scrollView, levelBadgeImageForAnim, zOrder);

                levelbadgeAtCenter = utilObj.getSprite(constantsObj.BLITZ_LEVEL_INFO[startLevel]["badgeImage"], centerXPos, centerYPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_79, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
                levelbadgeAtCenter.setOpacity(constantsObj.OPACITY_0);
                // utilObj.addChild(curLayer, levelbadgeAtCenter, zOrder);
                spritesToRemove.push(levelbadgeAtCenter);
            }

            xPos += (badge.getContentSize().width * constantsObj.SCALE_SPRITE_NEW_79)/2;
            xPos += 585 * deviceWidthRatio;
            xPos += 84 * deviceWidthRatio;

            // incrementing as the levels are done
            startLevel++;

        }

        // scroll it to the exact pos initially always
        var badgeToScrollTo = currentLevel - constantsObj.STARTING_LEVEL_FOR_SCROLL;
        var xPosToScrollInScrollView = levelBadgesArr[badgeToScrollTo].getPositionX();
        if (!introToBadge) {
            xPosToScrollInScrollView = levelBadgesArr[badgeToScrollTo - 1].getPositionX() + (levelBadgesArr[badgeToScrollTo].getPositionX() - levelBadgesArr[badgeToScrollTo - 1].getPositionX()) /2;
        }
        xPosToScrollInScrollView -= canvasWidth/2;

        var percentScroll = ( xPosToScrollInScrollView / (scrollViewWidth - canvasWidth)) * 100;

        if (percentScroll < 0) {
            percentScroll = 0;
        }
        scrollView.jumpToPercentHorizontal(percentScroll);

        // ---------------------------------------

        // scrollview fade in and move in animation to the screen
        var delay = 0;
        var moveByAnimDur = 0.35;
        var moveBy = utilObj.getMoveBy(moveByAnimDur, -canvasWidth, 0).easing(cc.easeIn(1.0));
        var fadeIn = utilObj.getFadeIn(moveByAnimDur).easing(cc.easeIn(1.0));
        var spawn = new cc.Spawn(moveBy, fadeIn);
        animObj.runAction(scrollView, spawn);

        delay += moveByAnimDur;

        // --------------------------

        // level unlocked text animation start

        var levelUnlockedBadge = utilObj.getBMFontLabel("Level " + currentLevel + " badge:", constantsObj.FONT_HELVETICA_REGULAR_75, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, centerYPos + (184.5 * deviceHeightRatio), 0.5, 0, labelArr, curLayer, zOrder);
        // utilObj.addChild(curLayer, levelUnlockedBadge, zOrder);
        utilObj.setOpacity(levelUnlockedBadge, constantsObj.OPACITY_0);
        spritesToRemove.push(levelUnlockedBadge);

        var Unlocked = utilObj.getBMFontLabel("Unlocked", constantsObj.FONT_HELVETICA_MEDIUM_75, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, centerYPos + (93.5 * deviceHeightRatio), 0.5, 0, labelArr, curLayer, zOrder);
        utilObj.setColor(Unlocked, constantsObj.COLOR_GREEN_08);
        utilObj.setOpacity(Unlocked, constantsObj.OPACITY_0);
        // utilObj.addChild(curLayer, Unlocked, zOrder);
        spritesToRemove.push(Unlocked);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay - 0.1, {
            arr: [levelUnlockedBadge, Unlocked],
            opacityArr: [constantsObj.OPACITY_60, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        // ---------------------------

        var scrollAnimTime = 0.25;
        if (!introToBadge) {

            // bar fill in animation
            var barFillAnim = 0.5;
            var scaleAction = animObj.getScaleDifferentTo(barFillAnim, ((668.5 + badgeHalfWidth) * deviceWidthRatio) / ((20 + badgeHalfWidth) * deviceWidthRatio), 1).easing(cc.easeIn(1.0));
            var animArr = [];
            var delayT = utilObj.getDelayTime(delay);
            animArr.push(delayT);
            animArr.push(scaleAction);

            animObj.runAction(progressBarGreen, utilObj.getSequence(animArr));

            delay += barFillAnim;

            // ---------------------------

            // scroll animation to the badge at center
            var xPosToMoveToForBadge = levelBadgesArr[badgeToScrollTo].getPositionX();
            xPosToMoveToForBadge -= canvasWidth / 2;

            var percentScrollBadge = ( xPosToMoveToForBadge / (scrollViewWidth - canvasWidth)) * 100;

            utilObj.delayedCallback(curLayer, blitzLevelHit.scrollToPercentHorizontal, delay, {
                scrollview: scrollView,
                percentScroll: percentScrollBadge,
                animTime: scrollAnimTime
            }, false);

            delay += scrollAnimTime;

            // ----------------------------

        }

        //ripple animation behind locked badge
        var animTimeRipple = 0.4;
        utilObj.delayedCallback(curLayer, blitzLevelHit.rippleAnim, delay, {circleSprite: circleSprite, animTime: animTimeRipple, scale: 4.5}, false);

        delay += 0.15;
        utilObj.delayedCallback(curLayer, blitzLevelHit.rippleAnim, delay, {circleSprite: circleSprite1, animTime: animTimeRipple, scale: 4.5}, false);

        delay += 0.15;

        utilObj.delayedCallback(curLayer, blitzLevelHit.rippleAnim, delay, {circleSprite: circleSprite2, animTime: animTimeRipple, scale: 4.5}, false);

        delay += 0.1;

        // -----------------------------

        // scale up the Badge and squeeze it to 0 scale on y axis
        var animTimeScaleBadge = 0.16;
        var animTimeSqueeze = 0.06;
        var badgeToScale = levelBadgesArr[badgeToScrollTo];

        var delayT1 = utilObj.getDelayTime(delay);
        var scaleBy = new cc.ScaleBy(animTimeScaleBadge, 3.5, 3.5).easing(cc.easeIn(1.0));
        var scaleBy2 = new cc.ScaleBy(animTimeSqueeze, 0, 1).easing(cc.easeIn(1.0));

        animObj.runAction(badgeToScale, utilObj.getSequence([delayT1, scaleBy, scaleBy2]));

        // fading out the level unlocked text
        /**
         * play fade out animation
         * node arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeOutAnimation, delay, {
            arr: [levelUnlockedBadge, Unlocked],
            fadeOutArr: 0.1
        }, false);

        delay += animTimeScaleBadge;
        delay += animTimeSqueeze;

        // -----------------------------

        /*
        * show up the new level badge and expand it to the original size and settle it to the center
        * tick appearing on the level complete
        * bar width decreases
        * */
        var animTimeExpand = 0.4;
        var animTimeScaleOut = 0.2;

        var scaleTo = utilObj.getScaleTo(animTimeExpand, constantsObj.SCALE_SPRITE_NEW_79 * 3.5, constantsObj.SCALE_SPRITE_NEW_79 * 3.5).easing(cc.easeIn(1.0));
        var scaleToIn = utilObj.getScaleTo(animTimeScaleOut, constantsObj.SCALE_SPRITE_NEW_79 , constantsObj.SCALE_SPRITE_NEW_79).easing(cc.easeIn(1.0));
        var delayT2 = utilObj.getDelayTime(delay);
        var fadeIn = utilObj.getFadeIn(0.01);
        var spawn = new cc.Spawn(fadeIn, scaleTo);

        animObj.runAction(levelBadgeImageForAnim, utilObj.getSequence([delayT2, spawn, scaleToIn]));

        // tick appear
        var moveByTime = 0.2;
        var moveBy = animObj.getMoveBy(moveByTime, - (50 * deviceWidthRatio), 0).easing(cc.easeIn(1.0));
        var delayT3 = utilObj.getDelayTime(delay + animTimeExpand);
        animObj.runAction(levelNumberForAnim, utilObj.getSequence([delayT3, moveBy]));

        var fadeInTime = 0.1;
        var scaleUpTime = 0.15;
        var scaleInTime = 0.05;
        var fadeIn = utilObj.getFadeIn(fadeInTime).easing(cc.easeIn(1.0));
        var scaleUp = utilObj.getScaleTo(scaleUpTime, 0.22 * 1.5).easing(cc.easeIn(1.0));
        var scaleIn = utilObj.getScaleTo(scaleInTime, 0.22).easing(cc.easeIn(1.0));

        var spawn = new cc.Spawn(fadeIn, scaleUp);

        animObj.runAction(circleBgForAnim, utilObj.getSequence([delayT3, spawn, scaleIn]));

        // bg width reduce
        if (!introToBadge) {
            var scaleDownBg = new cc.ScaleBy(moveByTime, 1, 6 / 25).easing(cc.easeIn(1.0));
            var scaleDownBg1 = new cc.ScaleBy(moveByTime, 1, 6 / 25).easing(cc.easeIn(1.0));

            animObj.runAction(progressBarBg, utilObj.getSequence([delayT3, scaleDownBg]));
            animObj.runAction(progressBarGreen, utilObj.getSequence([delayT3, scaleDownBg1]));
        }

        delay += animTimeExpand;
        delay += animTimeScaleOut;

        delay += 0.4; // resting delay

        // -----------------------------

        // move the badge down and at the center of the screen

        var timeTaken = 0;
        utilObj.delayedCallback(curLayer, function (sender, params) {
            timeTaken = blitzLevelHit.badgeMoveToCenter(curLayer, params.p);
        }, delay, {p : {node: levelbadgeAtCenter, deviceHeightRatio: deviceHeightRatio, circleNode1: circleSprite_badge, circleNode2: circleSprite_badge1, circleNode3: circleSprite_badge2}}, false);

        // -----------------------------

        delay += 1.65;

        // scroll to next level
        // only if max level is not reached
        var currentLevel1 = zenModeObj.getCurrentLevel();
        if (currentLevel1 < maxLevel + 1) {
            var xPosToScrollTo = levelBadgesArr[badgeToScrollTo].getPositionX() + (levelBadgesArr[badgeToScrollTo + 1].getPositionX() - levelBadgesArr[badgeToScrollTo].getPositionX()) /2;
            xPosToScrollTo -= canvasWidth/2;

            var percentScroll = ( xPosToScrollTo / (scrollViewWidth - canvasWidth)) * 100;

            if (percentScroll < 0) {
                percentScroll = 0;
            }
            utilObj.delayedCallback(curLayer, blitzLevelHit.scrollToPercentHorizontal, delay, {scrollview: scrollView, percentScroll: percentScroll, animTime: scrollAnimTime}, false);

            delay += scrollAnimTime;
        }

        // ------------------------------

        /**
         * Txt and the play button which appears after animation is over
         */
        var subText = "New Target:";
        if (currentLevel1 == constantsObj.MAX_LEVEL + 1) {
            subText = "Best Score:";
        }
        var NewTargetTxt = utilObj.getBMFontLabel(subText, constantsObj.FONT_HELVETICA_REGULAR_65, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, centerYPos + (194 * deviceHeightRatio), 0.5, 0, labelArr, curLayer, zOrder);
        // utilObj.addChild(curLayer, NewTargetTxt, zOrder);
        utilObj.setOpacity(NewTargetTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(NewTargetTxt);


        var scoreTxtLabel = zenModeObj.getMaxScore() + "";
        if (currentLevel1 < constantsObj.MAX_LEVEL + 1) {
            var blitzInfo = zenModeObj.getBlitzInfo();
            var scoreNextTarget = zenModeObj.getLevelTargetPointBasedOnExp(currentLevel1);
            scoreTxtLabel = "Score " + scoreNextTarget + " points!";
        }

        var scoreTxt = utilObj.getBMFontLabel(scoreTxtLabel, constantsObj.FONT_HELVETICA_MEDIUM_65, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, centerYPos + (102.5 * deviceHeightRatio), 0.5, 0, labelArr, curLayer, zOrder);
        // utilObj.addChild(curLayer, scoreTxt, zOrder);
        utilObj.setOpacity(scoreTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(scoreTxt);

        // play button
        var playButton = utilObj.getButton(650 * deviceWidthRatio, 208 * deviceHeightRatio, 0.5, 0.5, centerXPos , centerYPos - (480.5 * deviceHeightRatio),
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzLevelHit',
                fn: 'playZenModeAgain',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, playButton, zOrder);
        utilObj.setOpacity(playButton, constantsObj.OPACITY_0);
        spritesToRemove.push(playButton);

        var BtnImage = utilObj.getScale9Sprite(constantsObj.newButton_cutup,  centerXPos, centerYPos - (480.5 * deviceHeightRatio), 0.5, 0.5, 650 * deviceWidthRatio, 208 * deviceHeightRatio, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        // utilObj.addChild(curLayer, BtnImage, zOrder);
        utilObj.setOpacity(BtnImage, constantsObj.OPACITY_0);
        utilObj.setColor(BtnImage, constantsObj.COLOR_INTRO_PLAY_BUTTON);
        spritesToRemove.push(BtnImage);

        // pulse action for the btn
        utilObj.pulseButton(BtnImage, constantsObj.PULSE_ACTION_TAG, 0);

        // play button text
        yPos -= 104 * deviceHeightRatio;
        var playTxt =  utilObj.getBMFontLabel("Play", constantsObj.FONT_HELVETICA_MEDIUM_100, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, centerYPos - (480.5 * deviceHeightRatio), 0.5, 0.5, labelArr, curLayer, zOrder + 1);
        // utilObj.addChild(curLayer, playTxt, zOrder);
        utilObj.setOpacity(playTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(playTxt);

        // pulse action for the btn txt
        utilObj.pulseButton(playTxt, constantsObj.PULSE_ACTION_TAG, 0);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [NewTargetTxt, scoreTxt, playButton, playTxt,BtnImage, back_btn, closeBtn],
            opacityArr: [constantsObj.OPACITY_60, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        utilObj.addLabels(labelArr);
        utilObj.addSprites(spriteArr);

        // ---------------------------------
        var  level = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_OUTRO, constantsObj.TRACK_VIEW, level - 1, zenModeObj.getLivesCount() + "", "", "");


        // do the necessary func while anim is finished
        utilObj.delayedCallback(curLayer, blitzLevelHit.resumeFunctionsAfterAnim, delay, {scrollView: scrollView}, false)

    };

    /**
     * resume the necessary blocked functions
     */
    this.resumeFunctionsAfterAnim = function (sender, params) {
        var scrollView = params.scrollView;
        blitzLevelHit.setCheckAnimRunning(false);
        scrollView.setTouchEnabled(true);
        scrollView.setBounceEnabled(true);
    };

    /**
     * moves the badge to center and the plist animation
     * @param sender -> curLayer
     * @param params -> node, deviceHeightRatio
     */
    this.badgeMoveToCenter = function (sender, params) {
        var node = params.node;
        var heightRatio = params.deviceHeightRatio;
        node.setOpacity(constantsObj.OPACITY_100);
        var screenHeight = constantsObj.DESIGN_HEIGHT + topBarOffset;

        var totalDel = 0;

        var animTimeDown = 0.25;
        var animTimeUp = 0.20;
        var animTimeDown1 = 0.1;
        var moveDown = utilObj.getMoveBy(animTimeDown, 0, -100 * heightRatio ).easing(cc.easeIn(1.0));
        var moveUp = utilObj.getMoveTo(animTimeUp - 0.02, centerXPos, 3 * screenHeight / 4).easing(cc.easeIn(1.0));
        var scaleTo = utilObj.getScaleTo(animTimeUp, constantsObj.SCALE_SPRITE_NEW_79 * 4, constantsObj.SCALE_SPRITE_NEW_79 * 4).easing(cc.easeIn(1.0));
        var scaleDown = utilObj.getScaleTo(animTimeDown1, constantsObj.SCALE_SPRITE_NEW_79 * 2 , constantsObj.SCALE_SPRITE_NEW_79 * 2 ).easing(cc.easeIn(1.0));

        var spawn = new cc.Spawn(moveUp, scaleTo);

        animObj.runAction(node, utilObj.getSequence([moveDown, spawn, scaleDown]));

        totalDel += animTimeDown;
        totalDel += animTimeUp;
        totalDel += animTimeDown1;

        // circle ripple animation
        var animTimeRipple = 0.5;

        utilObj.delayedCallback(curLayer, blitzLevelHit.rippleAnim, totalDel, {circleSprite: params.circleNode1, animTime: animTimeRipple, scale: 4.5}, false);

        totalDel += 0.3;
        utilObj.delayedCallback(curLayer, blitzLevelHit.rippleAnim, totalDel, {circleSprite: params.circleNode2, animTime: animTimeRipple, scale: 4.5, plist: true}, false);

        totalDel += 0.3;
        utilObj.delayedCallback(curLayer, blitzLevelHit.rippleAnim, totalDel, {circleSprite: params.circleNode3, animTime: animTimeRipple, scale: 4.5}, false);

        totalDel += animTimeRipple;

        return totalDel;

    };

    /**
     * ripple animation
     * @param sender -> curLayer
     * @param params -> node, animTime, scale
     */
    this.rippleAnim = function (sender, params) {
        var node = params.circleSprite;
        var animTime = params.animTime;
        var scale = params.scale;
        var screenHeight = constantsObj.DESIGN_HEIGHT + topBarOffset;

        node.setOpacity(constantsObj.OPACITY_50);
        var fadeOut = utilObj.getFadeOut(animTime).easing(cc.easeIn(1.0));
        var scaleTo = new cc.ScaleBy(animTime, scale, scale).easing(cc.easeIn(1.0));

        var spawn = new cc.Spawn(fadeOut, scaleTo);
        animObj.runAction(node, spawn);

        if (params.plist) {
            var params = {
                x: centerXPos,
                y: 3 * screenHeight / 4,
                layer: curLayer,
                scale: 2.5,
                zOrder: constantsObj.FTUE_Z_ORDER - 1,
                gravity: true
            };
            utilObj.delayedCallback(curLayer, effectsObj.playStarOutroEffect, 0, params, false);
        }

    };

    /**
     * scroll animation to the percentScroll mentioned in the params
     * @param sender -> layer
     * @param params -> percentScroll to scroll to on scrollview, animTime
     */
    this.scrollToPercentHorizontal = function (sender, params) {
        var scrollView = params.scrollview;
        var percentScroll = params.percentScroll;
        var animTime = params.animTime;

        scrollView.scrollToPercentHorizontal(percentScroll, animTime, false)
    };

    /**
     * to calculate the exact height of the scrollview
     * @param deviceWidthRatio -> the ratio of the device width
     */
    this.calcScrollViewWidth = function (deviceWidthRatio) {
        var numberOfLevelsToMake = constantsObj.TOTAL_LEVELS_FOR_SCROLL;
        var startLevel = constantsObj.STARTING_LEVEL_FOR_SCROLL;
        var currentLevel = 5;//zenModeObj.getCurrentLevel();
        var maxLevel = constantsObj.MAX_LEVEL;

        var width = canvasWidth/2;

        for (var i = 0; i < numberOfLevelsToMake - 1; i++) {
            width += 101.91;
            width += 585 * deviceWidthRatio;
            width += 84 * deviceWidthRatio;
        }

        width += canvasWidth/2;

        return width;
    };

    /**
     * this will show the ftue outro after Level 1 and Level 2 hit
     */
    this.showFtueLevelHitOutro = function () {
        var spritesForFtueOutro = [];
        var delay = 0;
        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        var deviceHeightRatio = (constantsObj.DESIGN_HEIGHT + topBarOffset) / 2560;
        var deviceWidthRatio = canvasWidth / 1440;

        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_OUTRO, constantsObj.TRACK_VIEW, zenModeObj.getCurrentLevel() - 1, zenModeObj.getBlitzSpeciesTracking(), "", "");

        // cross button
        var back_btn = utilObj.getSprite(constantsObj.close_icon2, xRightPos - (50 * deviceWidthRatio), yPos - (52 * deviceHeightRatio), 1, 1, constantsObj.SCALE_SPRITE_43);
        utilObj.addChild(curLayer, back_btn, zOrder);
        utilObj.setOpacity(back_btn, constantsObj.OPACITY_0);
        spritesToRemove.push(back_btn);

        var closeBtn = utilObj.getButton(100 * deviceWidthRatio, 100 * deviceHeightRatio, 1, 1, xRightPos - (35 * deviceWidthRatio) , yPos - (36 * deviceHeightRatio),
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzLevelHit',
                fn: 'close',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, closeBtn, zOrder);
        utilObj.setOpacity(closeBtn, constantsObj.OPACITY_0);
        spritesToRemove.push(closeBtn);

        // target achieved text
        yPos -= 131 * deviceHeightRatio;
        var headerTxt = utilObj.getBMFontLabel("Target Achieved!", constantsObj.FONT_HELVETICA_MEDIUM_110, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1);
        utilObj.addChild(curLayer, headerTxt, zOrder);
        utilObj.setOpacity(headerTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(headerTxt);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [transBg, headerTxt, back_btn, closeBtn],
            opacityArr: [constantsObj.OPACITY_90, constantsObj.OPACITY_100, constantsObj.OPACITY_100, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

        delay += 0.15;

        // smiley images
        yPos -= 744 * deviceHeightRatio;
        var smileyImage = utilObj.getSprite(constantsObj.blitz_smiley, xPos, yPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_53);
        utilObj.addChild(curLayer, smileyImage, zOrder);
        utilObj.setOpacity(smileyImage, constantsObj.OPACITY_0);
        spritesToRemove.push(smileyImage);

        /**
         * play fade in animation
         * node is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.smileyAnimation, delay, {node: smileyImage});

        delay += 0.25;

        // target header text
        yPos -= 647 * deviceHeightRatio;
        var currentLevel = zenModeObj.getCurrentLevel();

        // shift for reward unlock tile when some one is at ftue level 3
        if (currentLevel == constantsObj.FTUE_LEVEL_3) {
            yPos += 200;
        }

        if (currentLevel < constantsObj.MAX_LEVEL + 1) {
            var targetHeader = utilObj.getBMFontLabel("New Target:", constantsObj.FONT_HELVETICA_80, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1);
            // utilObj.setOpacity(targetHeader, constantsObj.OPACITY_70);
            utilObj.setOpacity(targetHeader, constantsObj.OPACITY_0);
            utilObj.addChild(curLayer, targetHeader, zOrder);
            spritesToRemove.push(targetHeader);
        }

        // Target Level points
        yPos -= 114 * deviceHeightRatio;
        if (currentLevel < constantsObj.MAX_LEVEL + 1) {
            var blitzInfo = zenModeObj.getBlitzInfo();
            var targetP = zenModeObj.getLevelTargetPointBasedOnExp(currentLevel);
            var targetPoints = utilObj.getBMFontLabel(targetP + " points", constantsObj.FONT_HELVETICA_MEDIUM_80, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1);
            // utilObj.setOpacity(targetPoints, constantsObj.OPACITY_90);
            utilObj.setOpacity(targetPoints, constantsObj.OPACITY_0);
            utilObj.addChild(curLayer, targetPoints, zOrder);
            spritesToRemove.push(targetPoints);
        }

        // play button for the intro
        yPos -= 293 * deviceHeightRatio;
        var playButton = utilObj.getButton(650 * deviceWidthRatio, 208 * deviceHeightRatio, 0.5, 1, xPos , yPos,
            constantsObj.button_cutup, constantsObj.button_cutup, constantsObj.button_cutup, {
                obj: 'blitzLevelHit',
                fn: 'playZenModeAgain',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, playButton, zOrder);
        utilObj.setOpacity(playButton, constantsObj.OPACITY_0);
        utilObj.setColor(playButton, constantsObj.COLOR_INTRO_PLAY_BUTTON);
        spritesToRemove.push(playButton);

        // pulse action for the btn
        utilObj.pulseButton(playButton, constantsObj.PULSE_ACTION_TAG, 0);

        // play button text for intro
        yPos -= 104 * deviceHeightRatio;
        var playTxt =  utilObj.getBMFontLabel("Play", constantsObj.FONT_HELVETICA_MEDIUM_100, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 0.5);
        utilObj.addChild(curLayer, playTxt, zOrder);
        utilObj.setOpacity(playTxt, constantsObj.OPACITY_0);
        spritesToRemove.push(playTxt);

        // pulse action for the btn txt
        utilObj.pulseButton(playTxt, constantsObj.PULSE_ACTION_TAG, 0);

        /**
         * play fade in animation
         * node arr and opacity arr is send as params
         */
        utilObj.delayedCallback(curLayer, blitzLevelHit.playFadeInAnimation, delay, {
            arr: [targetHeader, targetPoints, playButton, playTxt],
            opacityArr: [constantsObj.OPACITY_70, constantsObj.OPACITY_90, constantsObj.OPACITY_100, constantsObj.OPACITY_100],
            fadeInDur: 0.15
        }, false);

    };

    /**
     * triggers fade in animation for the UI
     */
    this.playFadeInAnimation = function (sender, params) {
        var arr = params.arr;
        var opacityArr = params.opacityArr;
        var fadeInDur = params.fadeInDur;
        for(var i in arr) {
            var fadeIn = animObj.getFadeTo(fadeInDur, opacityArr[i]).easing(new cc.easeIn(1.0));
            if (utilObj.isNodeValid(arr[i])) {
                    animObj.runAction(arr[i], fadeIn);
            }
        }
    };

    /**
     * triggers fade out animation for the UI
     * node arr as the nodes to fade out
     */
    this.playFadeOutAnimation = function (sender, params) {
        var arr = params.arr;
        var fadeInDur = params.fadeOutArr;
        for(var i in arr) {
            var fadeOut = utilObj.getFadeOut(fadeInDur).easing(new cc.easeIn(1.0));
            if (utilObj.isNodeValid(arr[i])) {
                animObj.runAction(arr[i], fadeOut);
            }
        }
    };

    /**
     * smiley animation for the ftue outro
     */
    this.smileyAnimation = function (sender ,params) {
        var node = params.node;
        var fadeIn = utilObj.getFadeIn(0.05);
        var zoomOut = utilObj.getScaleTo(0.15, constantsObj.SCALE_SPRITE_53 * 1.2);
        var zoomIn = utilObj.getScaleTo(0.1, constantsObj.SCALE_SPRITE_53);

        var animArr = [];
        var spawn = new cc.Spawn(fadeIn, zoomOut);
        animArr.push(spawn);
        animArr.push(zoomIn);

        animObj.runAction(node, utilObj.getSequence(animArr));
    };

    /**
     * starts the zen mode again with new target
     */
    this.playZenModeAgain = function () {

        // stopping button actions while anim is running
        if (isAnimRunning) {
            return;
        }
        var  level = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_OUTRO, constantsObj.TRACK_PLAY, level - 1, zenModeObj.getLivesCount() + "", "", "");

        zenModeObj.setTransLayerOpen(false);
        zenModeObj.setStartAfterPopup(true);
        blitzLevelHit.hide();
        zenModeLayer.hide();
        zenModeLayer.loadBlitzMode();
        utilObj.showBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));
    };


    /**
     * returns to back to the home
     */
    this.close = function () {

        // stopping button actions while anim is running
        if (isAnimRunning) {
            return;
        }
        var  level = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BZ_LEVEL_OUTRO, constantsObj.TRACK_BACK, level - 1, zenModeObj.getLivesCount() + "", "", "");

        isScreenActive = false;
        zenModeObj.setTransLayerOpen(false);
        zenModeLayer.hide();
        zenModeObj.setStopTimer(false,false);
        homeLayer.show();
    };

    // Reward introduction animation
    this.rewardUnlockAnim = function (animDelay) {
        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        var rewardIncrement = zenModeObj.getCurrentScore();
        var totalRewardCount = zenModeObj.getTotalRewardCount();
        var rewardMilestones = blitzRetryLayer.getRewardMilestone();
        yPos -= 600;
        var rewardText = "Reward Unlocked!";

        var headerText = utilObj.getBMFontLabel(rewardText, constantsObj.FONT_HELVETICA_MEDIUM_110, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 0.5);
        utilObj.setOpacity(headerText, constantsObj.OPACITY_0);
        utilObj.addChild(curLayer, headerText, zOrder);
        spritesToRemove.push(headerText);

        // reward header text fade in
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.HEADER_TEXT_FADEIN,
            node: headerText,
            animDur: 0.2,
            opacity: constantsObj.OPACITY_90
        });


        yPos -= 380;
        var tileWidth = 1300;
        var tileHeight = 380;
        var bigTileHeight = 480;


        var milestoneTile = utilObj.getScale9Sprite(constantsObj.trans_button, xPos, yPos - bigTileHeight / 2, 0.5, 0.5, tileWidth, bigTileHeight);
        utilObj.setColor(milestoneTile, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(milestoneTile, constantsObj.OPACITY_0);
        utilObj.addChild(curLayer, milestoneTile, zOrder);
        var milestoneHeaderText = "%X% Points added to your Blitz Chest!";
        milestoneHeaderText = milestoneHeaderText.replace("%X%", rewardIncrement);
        var milestoneText = utilObj.getBMFontLabel(milestoneHeaderText, constantsObj.FONT_HELVETICA_75, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), tileWidth / 2, tileHeight + 50, 0.5, 0.5);
        utilObj.addChild(milestoneTile, milestoneText, zOrder);
        milestoneTile.setCascadeOpacityEnabled(true);
        spritesToRemove.push(milestoneTile);

        var milestoneSubTile = utilObj.getScale9Sprite(constantsObj.white_rounded_bg, tileWidth / 2, tileHeight / 2, 0.5, 0.5, tileWidth, tileHeight);
        utilObj.setColor(milestoneSubTile, constantsObj.COLOR_BLITZ_MILESTONE);
        utilObj.setOpacity(milestoneSubTile, constantsObj.OPACITY_90);
        utilObj.addChild(milestoneTile, milestoneSubTile, zOrder);
        milestoneSubTile.setCascadeOpacityEnabled(true);

        var progressBarWidth = 940;
        var progressBarHeight = 30;
        var progressBarStart = (tileWidth - progressBarWidth) / 2;
        var lastMilestoneValue = rewardMilestones[2];
        var progressPointer = [];
        var showProgressPointer = false;
        var progressPointerXPos = 0;
        var progressPointerYPos = 0;
        var progressPointerText = "";
        var index = -1;
        var progressPointerWidth = 290;
        var progressPointerHeight = 95;
        var progressBarBg = utilObj.getScale9Sprite(constantsObj.full_bar, progressBarStart, tileHeight / 2 + 10, 0, 0.5, progressBarWidth, progressBarHeight);
        utilObj.setColor(progressBarBg, constantsObj.COLOR_BLACK);
        utilObj.addChild(milestoneSubTile, progressBarBg, zOrder);


        var len = (totalRewardCount / lastMilestoneValue) * progressBarWidth;
        var finalLen = 0;
        if (len == 0) {
            finalLen = (0.1 / lastMilestoneValue) * progressBarWidth;
        }
        var progressBar = utilObj.getScale9Sprite(constantsObj.tile_white, progressBarStart, tileHeight / 2 + 10, 0, 0.5, len + finalLen, progressBarHeight);
        utilObj.addChild(milestoneSubTile, progressBar, zOrder);
        utilObj.setColor(progressBar, constantsObj.COLOR_MILESTORE_PROGRESS);
        
        var giftBoxArr = [constantsObj.blitz_yellow_gift_box, constantsObj.blitz_pink_gift_box, constantsObj.blitz_blue_gift_box];
        var giftCapArr = [constantsObj.blitz_yellow_gift_cap, constantsObj.blitz_pink_gift_cap, constantsObj.blitz_blue_gift_cap];
        var scalingArr = [constantsObj.SCALE_ONE * 1.08 * 0.8 / 2, constantsObj.SCALE_ONE * 1.08 / 2, constantsObj.SCALE_ONE * 1.08 * 1.2 / 2];
        for (var i = 0; i < 3; i++) {
            var giftBoxIcon = utilObj.getSprite(giftBoxArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20, 0.5, 0.5, scalingArr[i]);
            var positionParams = utilObj.getSpriteDimensions(giftBoxIcon);
            var nodeCoordinates = utilObj.getSpriteCoordinates(giftBoxIcon);
            utilObj.setPosition(giftBoxIcon, nodeCoordinates.xPos, nodeCoordinates.yPos + (positionParams.height / 2));
            utilObj.addChild(milestoneSubTile, giftBoxIcon, zOrder);
            var giftCapIcon = utilObj.getSprite(giftCapArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20 + positionParams.height, 0.5, 0.5, scalingArr[i]);
            var capPositionParams = utilObj.getSpriteDimensions(giftCapIcon);
            var capNodeCoordinates = utilObj.getSpriteCoordinates(giftCapIcon);
            utilObj.setPosition(giftCapIcon, capNodeCoordinates.xPos, capNodeCoordinates.yPos + (capPositionParams.height / 2));
            utilObj.addChild(milestoneSubTile, giftCapIcon, zOrder);
            if (totalRewardCount + rewardIncrement >= rewardMilestones[i] && totalRewardCount < rewardMilestones[i]) {
                showProgressPointer = true;
                progressPointerXPos = progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth;
                progressPointerYPos = tileHeight / 2 - 20;
                progressPointerText = "FULL!";
                index = i;
            } else if (totalRewardCount + rewardIncrement < rewardMilestones[i] && index == -1) {
                showProgressPointer = true;
                progressPointerXPos = progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth;
                progressPointerYPos = tileHeight / 2 - 20;
                progressPointerText = rewardMilestones[i] - (totalRewardCount + rewardIncrement) + " points to go!";
                index = i;
                progressPointerWidth = 600;
            }
        }

        if (showProgressPointer) {
            var progressPointerIcon = utilObj.getSprite(constantsObj.themed_quest_progress_pointer, progressPointerXPos, 125, 0.5, 0, constantsObj.SCALE_SPRITE_11);
            utilObj.addChild(milestoneSubTile, progressPointerIcon, zOrder);
            utilObj.setOpacity(progressPointerIcon, constantsObj.OPACITY_0);
            progressPointer.push(progressPointerIcon);

            var progressPointerTile = utilObj.getScale9Sprite(constantsObj.white_rounded_bg, progressPointerXPos, 125 - progressPointerHeight / 2, 0.5, 0.5, progressPointerWidth, progressPointerHeight);
            utilObj.addChild(milestoneSubTile, progressPointerTile, zOrder);
            utilObj.setOpacity(progressPointerTile, constantsObj.OPACITY_0);
            progressPointer.push(progressPointerTile);
            progressPointerTile.setCascadeOpacityEnabled(true);
            var progressPointerTextValue = utilObj.getBMFontLabel(progressPointerText, constantsObj.FONT_HELVETICA_MEDIUM_70, constantsObj.COLOR_BLACK,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), progressPointerWidth / 2, progressPointerHeight / 2, 0.5, 0.5);
            utilObj.addChild(progressPointerTile, progressPointerTextValue, zOrder);

        }

        // milestone tile fade in anim
        var scaleFrom = 0;
        var scaleTo = milestoneTile.getScale();
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.MILESTONE_TILE_ANIM,
            node: milestoneTile,
            animDelay: animDelay,
            animFadeDur: 0.02,
            animScaleUpDur: 0.2,
            animScaleDownDur: 0.28,
            scaleFrom: scaleFrom,
            scaleTo: scaleTo,
            opacity: constantsObj.OPACITY_100
        });
        animDelay += 0.5;

        // milestone progress anim
        animDelay = blitzRetryLayer.showMilestoneProgressAnim(len + finalLen, rewardIncrement, progressBar, totalRewardCount, lastMilestoneValue, animDelay + 0.1);


        var animDur = 0;
        var animDelayTime = animDelay;

        // progress pointer anim
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.PROGRESS_POINTER_ANIM,
            node: progressPointer,
            animDelay: animDelayTime,
            animDur: animDur,
            opacity: constantsObj.OPACITY_100
        });

        animDelay += 0.5;

        // move down milestone tile anim
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.MILESTONE_MOVE_DOWN,
            node: milestoneTile,
            animDelay: animDelay,
            animDur: 0.25,
            opacity: constantsObj.OPACITY_100,
            yShift: 200,
            scaleDown: 0.75
        });
        animDelay += 0.25;

        // fade out milestone anim
        // animObj.blitzRewardAnimation(curLayer, {
        //     case: constantsObj.MILESTONE_TILE_FADE_OUT,
        //     node: milestoneTile,
        //     animDelay: animDelay + 0.6,
        //     animDur: 0.2,
        //     opacity: constantsObj.OPACITY_0
        // });

        // header text fade out
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.HEADER_TEXT_FADE_OUT,
            node: headerText,
            animDelay: animDelay + 0.6,
            animDur: 0.2,
            opacity: constantsObj.OPACITY_0
        });

        animDelay += 0.6 + 0.2;
        return animDelay;
    };

    // function to do nothing
    this.preventBackgroundScreenTouch = function () {
        // do nothing -> rest here xD
    };

    //check for the close Btn disablity func while anim is running
    this.getCheckAnimRunning = function () {
        return isAnimRunning;
    };

    this.setCheckAnimRunning = function (val) {
        isAnimRunning = val;
    };

    this.AmazingGratification = function (currentLevel, zOrder) {
        var mainTextImage = constantsObj.blitztastic;
        var animDelay = 0;
        var sub_text = "Level " + currentLevel + " completed!";
        var arrToRemove = [];

        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), "bz_gratif", constantsObj.TRACK_VIEW, zenModeObj.getCurrentLevel() - 1, "", "", "");

        var tempTransBg = utilObj.getScale9Sprite(constantsObj.tile_white, centerXPos, 0, 0.5, 0, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset);
        utilObj.setColor(tempTransBg, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(tempTransBg, constantsObj.OPACITY_0);
        utilObj.addChild(curLayer, tempTransBg, zOrder - 2);
        spritesToRemove.push(tempTransBg);
        arrToRemove.push(tempTransBg);

        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.REWARD_OVERLAY_FADEIN,
            animDelay: animDelay,
            node: tempTransBg,
            animDur: 0.15,
            opacity: constantsObj.OPACITY_80
        });
        animDelay += 0.15;

        var showMainText = utilObj.getSprite(mainTextImage, centerXPos, centerYPos + 50, 0.5, 0.5, constantsObj.SCALE_SPRITE_NEW_1 * 2.25 * 0.75 * 1.5);
        utilObj.addChild(curLayer, showMainText, zOrder);
        showMainText.setScale(0);
        arrToRemove.push(showMainText);
        var showMainTextDelay = 0.2;
        animObj.outroAnimations(curLayer, {
            case: constantsObj.OUTRO_MAIN_TEXT,
            node: showMainText,
            animDelay: animDelay,
            scale: constantsObj.SCALE_SPRITE_NEW_1 * 2.25 * 0.75 * 1.5,
            delay: showMainTextDelay
        });

        var showSubtext = utilObj.getBMFontLabel(sub_text, constantsObj.FONT_HELVETICA_MEDIUM_80, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), centerXPos, centerYPos - 150, 0.5, 0.5);
        utilObj.addChild(curLayer, showSubtext, zOrder);
        arrToRemove.push(showSubtext);
        showSubtext.setOpacity(0);
        animObj.outroAnimations(curLayer, {
            case: constantsObj.OUTRO_SUB_TEXT,
            node: showSubtext,
            animDelay: animDelay,
            delay: showMainTextDelay,
            opacity: constantsObj.OPACITY_100
        });
        animDelay += showMainTextDelay;

        var params1 = {
            case: constantsObj.OUTRO_CONFETTI_PLIST1,
            animDelay: animDelay - (showMainTextDelay / 2),
            x: xLeftPos - 100,
            y: centerYPos - 400,
            layer: curLayer,
            scale: 1,
            zOrder: zOrder - 1,
            duration: 0.25,
            angle: 65
        };
        animObj.outroAnimations(curLayer, params1);

        var params2 = {
            case: constantsObj.OUTRO_CONFETTI_PLIST2,
            animDelay: animDelay - (showMainTextDelay / 2),
            x: xRightPos + 100,
            y: centerYPos - 400,
            layer: curLayer,
            scale: 1,
            zOrder: zOrder - 1,
            duration: 0.25,
            angle: 115
        };
        animObj.outroAnimations(curLayer, params2);
        animDelay += 2;

        //scaling down first text here
        animObj.outroAnimations(curLayer, {
            case: constantsObj.OUTRO_MAIN_TEXT_FADE_OUT,
            node: showMainText,
            animDelay: animDelay,
            scale: constantsObj.SCALE_SPRITE_3,
            delay: showMainTextDelay
        });

        animObj.outroAnimations(curLayer, {
            case: constantsObj.OUTRO_SUB_TEXT_FADE_OUT,
            node: showSubtext,
            animDelay: animDelay,
            delay: showMainTextDelay
        });
        animDelay += showMainTextDelay;

        // outro overlay fade out
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.REWARD_OVERLAY_FADEIN,
            animDelay: animDelay,
            node: tempTransBg,
            animDur: 0,
            opacity: constantsObj.OPACITY_0
        });

        for (var i = 0; i < arrToRemove.length; i++) {
            utilObj.delayedCallback(curLayer, blitzLevelHit.hideAmazingGratificationVar, animDelay + 0.5, {
                node:arrToRemove[i]
            }, false);
        }

        return animDelay + 0.5;
    };

    this.hideAmazingGratificationVar = function (sender, params) {
        if (utilObj.isNodeValid(params.node)) {
            utilObj.removeChild(curLayer, params.node);
        }
    };

};