/**
 * This is a trans layer, it won't have a screen name, will work on functions similar to layers
 * @constructor
 */

BlitzRetryLayer = function() {
    var rewardMilestones = [100, 200, 300];
    var coinAmount = 10;
    var spritesToRemove = [];
    var blitzRetryVisible = false;


    /**
     * this will work similar to the layer hide functions
     * it will remove all the elements on the screen pushed to arr spritesToRemove
     */
    this.hide = function () {
        for (var i in spritesToRemove) {
            if (utilObj.isNodeValid(spritesToRemove[i])) {
                utilObj.removeChild(curLayer, spritesToRemove[i]);
            }
        }
        spritesToRemove = [];
        blitzRetryVisible = false;
    };

    /**
     * here all the elements will be shown on the screen, all the elements will have the parentLayer as currentScreen
     */
    this.show = function () {
        if (blitzRetryVisible) {
            return;
        }
        zenModeObj.setStartAfterPopup(true);
        zenModeObj.setSoundPlayed(false);
        zenModeObj.setTransLayerOpen(true);
        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        var deviceHeightRatio = (constantsObj.DESIGN_HEIGHT + topBarOffset) / 2560;
        var deviceWidthRatio = canvasWidth / 1440;
        blitzRetryVisible = true;
        rewardMilestones = blitzRetryLayer.getRewardMilestone();
        utilObj.hideBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));

        // trans bg for the background
        var transBg = utilObj.getScale9Sprite(constantsObj.tile_white, xPos, 0, 0.5, 0, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset);
        utilObj.setColor(transBg, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(transBg, constantsObj.OPACITY_0);
        utilObj.addChild(curLayer, transBg, zOrder);
        spritesToRemove.push(transBg);

        // tracking
        var bonusTimeAdded = zenModeObj.getBonusTimeAdded();
        var startTime = zenModeObj.getMaxBufferTimer();
        var bufferTime = zenModeObj.getCurrentBufferTime();
        var highestStreakAchieved = zenModeObj.getHighestStreakAchieved();
        var currentLives = zenModeObj.getLivesCount();
        var currentScore = zenModeObj.getCurrentScore();

        var totalTime = startTime - bufferTime + bonusTimeAdded;
        trackObj.trackCounter(constantsObj.TRACK_PUZZLE, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_FAIL, totalTime + "_" + bonusTimeAdded + "_" + highestStreakAchieved, currentScore + "_" + currentLives, zenModeObj.getBlitzSpeciesTracking(), "", "");


        var transBtn = utilObj.getButton(canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset, 0.5, 0, xPos, 0,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzFtueOutro',
                fn: 'preventBackgroundScreenTouch',
                screen: zenModeLayer.screen,
                params : {sound : false},
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, transBtn, zOrder);
        spritesToRemove.push(transBtn);

        var animDelay = 0.15;
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.REWARD_OVERLAY_FADEIN,
            node: transBg,
            animDur: animDelay,
            opacity: constantsObj.OPACITY_90
        });

        var rewardIncrement = zenModeObj.getCurrentScore();
        var totalRewardCount = zenModeObj.getTotalRewardCount();

        /**
         * Reward and milestone animation condition
         */
        if (rewardIncrement >= 0) {
            animDelay = blitzRetryLayer.rewardAnimation(animDelay);//blitzLevelHit.rewardUnlockAnim(animDelay);
            totalRewardCount += rewardIncrement;
        }
        //final outro calling func.
        utilObj.delayedCallback(zenModeLayer , blitzRetryLayer.showFinalOutro , animDelay , null , false);

    };

    /**
     * Reward and milestone animation
     */
    this.rewardAnimation = function (animDelay) {
        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        var spriteArr = [];
        var labelArr = [];
        var rewardIncrement = zenModeObj.getCurrentScore();
        var totalRewardCount = zenModeObj.getTotalRewardCount();
        rewardMilestones = blitzRetryLayer.getRewardMilestone();
        var lastMilestoneValue = rewardMilestones[2];
        //reset total reward value if it is more than the final milestone value
        while (totalRewardCount > lastMilestoneValue) {
            totalRewardCount = totalRewardCount - lastMilestoneValue;
        }

        if (rewardIncrement >= 0) {
            yPos -= 250;
            var rewardText = "Bravo!";
            var milestoneHeaderText = "%X% Points added to your Blitz Chest!";
            milestoneHeaderText = milestoneHeaderText.replace("%X%", rewardIncrement);
            for (var  i = 0; i < 3; i++) {
                if (totalRewardCount + rewardIncrement >= rewardMilestones[i] && totalRewardCount < rewardMilestones[i]) {
                    rewardText = "Cha-ching!";
                }
            }
            var headerText = utilObj.getBMFontLabel(rewardText, constantsObj.FONT_HELVETICA_MEDIUM_110, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 0.5, labelArr, curLayer, zOrder);
            utilObj.setOpacity(headerText, constantsObj.OPACITY_0);
            spritesToRemove.push(headerText);

            // reward header text fade in
            animObj.blitzRewardAnimation(curLayer, {
                case: constantsObj.HEADER_TEXT_FADEIN,
                node: headerText,
                animDur: 0.2,
                opacity: constantsObj.OPACITY_90
            });


            yPos -= 330;
            var tileWidth = 1300;
            var tileHeight = 380;
            var bigTileHeight = 480;


            var milestoneTile = utilObj.getScale9Sprite(constantsObj.trans_button, xPos, yPos - bigTileHeight / 2, 0.5, 0.5, tileWidth, bigTileHeight, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
            utilObj.setColor(milestoneTile, constantsObj.COLOR_BLACK);
            utilObj.setOpacity(milestoneTile, constantsObj.OPACITY_0);
            var milestoneText = utilObj.getBMFontLabel(milestoneHeaderText, constantsObj.FONT_HELVETICA_75, constantsObj.COLOR_WHITE,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), tileWidth / 2, tileHeight + 50, 0.5, 0.5, labelArr, milestoneTile, zOrder);
            milestoneTile.setCascadeOpacityEnabled(true);
            spritesToRemove.push(milestoneTile);

            var milestoneSubTile = utilObj.getScale9Sprite(constantsObj.white_rounded_bg, tileWidth / 2, tileHeight / 2, 0.5, 0.5, tileWidth, tileHeight, spriteArr, milestoneTile, zOrder, constantsObj.SPRITE_MISC);
            utilObj.setColor(milestoneSubTile, constantsObj.COLOR_BLITZ_MILESTONE);
            utilObj.setOpacity(milestoneSubTile, constantsObj.OPACITY_90);
            milestoneSubTile.setCascadeOpacityEnabled(true);

            var progressBarWidth = 940;
            var progressBarHeight = 30;
            var progressBarStart = (tileWidth - progressBarWidth) / 2;
            var giftBoxAnim = [];
            var progressPointer = [];
            var showProgressPointer = false;
            var progressPointerXPos = 0;
            var progressPointerYPos = 0;
            var progressPointerText = "";
            var tickIconAnim = null;
            var showGiftBoxAnim = false;
            var index = -1;
            var progressPointerWidth = 290;
            var progressPointerHeight = 95;
            var progressBarBg = utilObj.getScale9Sprite(constantsObj.full_bar, progressBarStart, tileHeight / 2 + 10, 0, 0.5, progressBarWidth, progressBarHeight, spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
            utilObj.setColor(progressBarBg, constantsObj.COLOR_BLACK);

            var var_scores = totalRewardCount + "_" + (totalRewardCount + rewardIncrement) + "_" + lastMilestoneValue;
            trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_REWARD_ANIM, constantsObj.TRACK_VIEW, var_scores, "", "", "");


            var len = (totalRewardCount / lastMilestoneValue) * progressBarWidth;
            var finalLen = 0;
            if (len == 0) {
                finalLen = (0.1 / lastMilestoneValue) * progressBarWidth;
            }
            var progressBar = utilObj.getScale9Sprite(constantsObj.tile_white, progressBarStart, tileHeight / 2 + 10, 0, 0.5, len + finalLen, progressBarHeight, spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
            utilObj.setColor(progressBar, constantsObj.COLOR_MILESTORE_PROGRESS);

            var giftBoxArr = [constantsObj.blitz_yellow_gift_box, constantsObj.blitz_pink_gift_box, constantsObj.blitz_blue_gift_box];
            var giftCapArr = [constantsObj.blitz_yellow_gift_cap, constantsObj.blitz_pink_gift_cap, constantsObj.blitz_blue_gift_cap];
            var scalingArr = [constantsObj.SCALE_ONE * 1.08 * 0.8 / 2, constantsObj.SCALE_ONE * 1.08 / 2, constantsObj.SCALE_ONE * 1.08 * 1.2 / 2];
            for (var i = 0; i < 3; i++) {
                if (totalRewardCount > rewardMilestones[i]) {
                    var giftBoxIcon = utilObj.getSprite(giftBoxArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20, 0.5, 0.5, scalingArr[i], spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                    var positionParams = utilObj.getSpriteDimensions(giftBoxIcon);
                    var nodeCoordinates = utilObj.getSpriteCoordinates(giftBoxIcon);
                    utilObj.setPosition(giftBoxIcon, nodeCoordinates.xPos, nodeCoordinates.yPos + (positionParams.height / 2));
                    var giftCapIcon = utilObj.getSprite(giftCapArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20, 0.5, 0.5, scalingArr[i], spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                    var capPositionParams = utilObj.getSpriteDimensions(giftCapIcon);
                    var capNodeCoordinates = utilObj.getSpriteCoordinates(giftCapIcon);
                    utilObj.setPosition(giftCapIcon, capNodeCoordinates.xPos - capPositionParams.width / 2 - 20, capNodeCoordinates.yPos + (capPositionParams.height));
                    giftCapIcon.setRotation(-60);
                    var tickIcon = utilObj.getSprite(constantsObj.tick, progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 + 150, 0.5, 0.5, constantsObj.SCALE_SPRITE_18 * 1.5, spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                    utilObj.setColor(tickIcon, constantsObj.COLOR_MILESTORE_PROGRESS);

                } else {
                    var giftBoxIcon = utilObj.getSprite(giftBoxArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20, 0.5, 0.5, scalingArr[i], spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                    var positionParams = utilObj.getSpriteDimensions(giftBoxIcon);
                    var nodeCoordinates = utilObj.getSpriteCoordinates(giftBoxIcon);
                    utilObj.setPosition(giftBoxIcon, nodeCoordinates.xPos, nodeCoordinates.yPos + (positionParams.height / 2));
                    var giftCapIcon = utilObj.getSprite(giftCapArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20 + positionParams.height, 0.5, 0.5, scalingArr[i], spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                    var capPositionParams = utilObj.getSpriteDimensions(giftCapIcon);
                    var capNodeCoordinates = utilObj.getSpriteCoordinates(giftCapIcon);
                    utilObj.setPosition(giftCapIcon, capNodeCoordinates.xPos, capNodeCoordinates.yPos + (capPositionParams.height / 2));
                    if (totalRewardCount + rewardIncrement >= rewardMilestones[i] && totalRewardCount < rewardMilestones[i]) {
                        giftBoxAnim.push(giftBoxIcon);
                        giftBoxAnim.push(giftCapIcon);
                        showProgressPointer = true;
                        progressPointerXPos = progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth;
                        progressPointerYPos = tileHeight / 2 - 20;
                        progressPointerText = "FULL!";
                        index = i;
                        var tickIcon = utilObj.getSprite(constantsObj.tick, progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 + 150, 0.5, 0.5, constantsObj.SCALE_SPRITE_18 * 1.5, spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                        utilObj.setColor(tickIcon, constantsObj.COLOR_MILESTORE_PROGRESS);
                        utilObj.setOpacity(tickIcon, constantsObj.OPACITY_0);
                        tickIconAnim = tickIcon;
                        showGiftBoxAnim = true;
                    } else if (totalRewardCount + rewardIncrement < rewardMilestones[i] && index == -1) {
                        showProgressPointer = true;
                        progressPointerXPos = progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth;
                        progressPointerYPos = tileHeight / 2 - 20;
                        progressPointerText = rewardMilestones[i] - (totalRewardCount + rewardIncrement) + " points to go!";
                        index = i;
                        progressPointerWidth = 600;
                    }
                }
            }

            if (showProgressPointer) {
                var progressPointerIcon = utilObj.getSprite(constantsObj.themed_quest_progress_pointer, progressPointerXPos, 125, 0.5, 0, constantsObj.SCALE_SPRITE_11, spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                utilObj.setOpacity(progressPointerIcon, constantsObj.OPACITY_0);
                progressPointer.push(progressPointerIcon);

                var anchorX = 0.5;
                if (index == 2 && progressPointerText != "FULL!") {
                    anchorX = 0.75;
                }
                var progressPointerTile = utilObj.getScale9Sprite(constantsObj.white_rounded_bg, progressPointerXPos, 125 - progressPointerHeight / 2, anchorX, 0.5, progressPointerWidth, progressPointerHeight, spriteArr, milestoneSubTile, zOrder, constantsObj.SPRITE_MISC);
                utilObj.setOpacity(progressPointerTile, constantsObj.OPACITY_0);
                progressPointer.push(progressPointerTile);
                progressPointerTile.setCascadeOpacityEnabled(true);
                var progressPointerTextValue = utilObj.getBMFontLabel(progressPointerText, constantsObj.FONT_HELVETICA_MEDIUM_70, constantsObj.COLOR_BLACK,
                    constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), progressPointerWidth / 2, progressPointerHeight / 2, 0.5, 0.5, labelArr, progressPointerTile, zOrder);
            }

            // milestone tile fade in anim
            var scaleFrom = milestoneTile.getScale();
            var scaleTo = milestoneTile.getScale();
            animObj.blitzRewardAnimation(curLayer, {
                case: constantsObj.MILESTONE_TILE_ANIM,
                node: milestoneTile,
                animDelay: animDelay,
                animFadeDur: 0.02,
                animScaleUpDur: 0.2,
                animScaleDownDur: 0.28,
                scaleFrom: scaleFrom,
                scaleTo: scaleTo,
                opacity: constantsObj.OPACITY_100
            });

            if (rewardIncrement > 0) {
                var params1 = {
                    x: xPos,
                    y: yPos - bigTileHeight + tileHeight,
                    layer: curLayer,
                    zOrder: zOrder
                };
                utilObj.delayedCallback(curLayer, effectsObj.blitzStarScratch, animDelay + 0.02 + 0.2, params1, false);

                var params2 = {
                    x: xPos,
                    y: yPos - bigTileHeight,
                    layer: curLayer,
                    zOrder: zOrder
                };
                utilObj.delayedCallback(curLayer, effectsObj.blitzStarScratch, animDelay + 0.02 + 0.2, params2, false);

                animDelay += 0.5;

                // milestone progress anim
                var finalVal = rewardIncrement;
                if (totalRewardCount + rewardIncrement >= lastMilestoneValue && totalRewardCount < lastMilestoneValue) {
                    finalVal = lastMilestoneValue - totalRewardCount;
                }
                animDelay = blitzRetryLayer.showMilestoneProgressAnim(len + finalLen, finalVal, progressBar, totalRewardCount, lastMilestoneValue, animDelay + 0.1, giftBoxAnim);
            }

            var animDur = 0.2;
            var animDelayTime = animDelay - 0.3;

            // progress pointer fade in anim
            animObj.blitzRewardAnimation(curLayer, {
                case: constantsObj.PROGRESS_POINTER_ANIM,
                node: progressPointer,
                animDelay: animDelayTime,
                animDur: animDur,
                opacity: constantsObj.OPACITY_100
            });
            animDelay += 0.5;

            // progress pointer fade out anim
            if (showGiftBoxAnim) {
                animObj.blitzRewardAnimation(curLayer, {
                    case: constantsObj.PROGRESS_POINTER_FADE_OUT_ANIM,
                    node: progressPointer,
                    animDelay: animDelay,
                    animDur: 0.2
                });
            }

            // move down milestone tile anim
            animObj.blitzRewardAnimation(curLayer, {
                case: constantsObj.MILESTONE_MOVE_DOWN,
                node: milestoneTile,
                animDelay: animDelay,
                animDur: 0.25,
                opacity: constantsObj.OPACITY_100,
                yShift: 200,
                scaleDown: 0.75
            });
            animDelay += 0.25;

            //tick anim to appear
            if (tickIconAnim != null) {
                animObj.blitzRewardAnimation(curLayer, {
                    case: constantsObj.TICK_ICON_FADE_IN,
                    node: tickIconAnim,
                    animDelay: animDelay,
                    animDur: 0.2,
                    opacity: constantsObj.OPACITY_100
                });
                animDelay += 0.2;
            }

            if (showGiftBoxAnim) {
                animDelay = blitzRetryLayer.showGiftBoxCoinGrantAnim(milestoneTile, milestoneSubTile, progressPointerXPos, progressPointerYPos, index, zOrder, animDelay, giftBoxAnim);
            }

            animDelay += 0.5;
            if (totalRewardCount + rewardIncrement >= lastMilestoneValue && totalRewardCount < lastMilestoneValue) {
                var diff = (totalRewardCount + rewardIncrement) - lastMilestoneValue;
                totalRewardCount = 0;
                rewardIncrement = diff;
                animDelay = blitzRetryLayer.showNewMilestoneTile(milestoneTile, animDelay, totalRewardCount, rewardIncrement);
                zenModeObj.setTotalRewardCount(rewardIncrement);
            } else {
                zenModeObj.setTotalRewardCount(totalRewardCount + rewardIncrement);
            }

            // header text fade out
            animObj.blitzRewardAnimation(curLayer, {
                case: constantsObj.HEADER_TEXT_FADE_OUT,
                node: headerText,
                animDelay: animDelay,
                animDur: 0.3,
                opacity: constantsObj.OPACITY_0
            });
        }
        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);

        return animDelay + 0.5;
    };

    /**
     * Milestone tile after refreshed
     */
    this.showNewMilestoneTile = function (milestoneTile1, animDelay, totalRewardCount, rewardIncrement) {
        //scale down milestone tile
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.SCALE_DOWN_MILESTONE_TILE,
            node: milestoneTile1,
            animDelay: animDelay,
            animDur: 0.3
        });
        animDelay += 0.3;

        var xPos = centerXPos;
        var yPos = 200 + 60;
        var zOrder = constantsObj.FTUE_Z_ORDER;
        var tileWidth = 1300;
        var tileHeight = 380;
        var bigTileHeight = 480;
        yPos += bigTileHeight;
        var scaleValue = 1;
        yPos -= bigTileHeight / 2;

        var totalSessionCount = zenModeObj.getTotalSessionCount();
        var trackTotalSessionCount = totalSessionCount;
        var blitzSessionLow = runtimeObj.getBlitzRewardFillSessionLow();
        var blitzSessionHigh = runtimeObj.getBlitzRewardFillSessionHigh();
        var finalMilestoneValue = zenModeObj.getFinalMilestoneValue();
        if (totalSessionCount < blitzSessionHigh) {
            zenModeObj.setFinalMilestoneValue(finalMilestoneValue + (0.5 * finalMilestoneValue));
        } else if (totalSessionCount > blitzSessionLow && totalSessionCount <= blitzSessionHigh) {
            zenModeObj.setFinalMilestoneValue(finalMilestoneValue);
        } else {
            zenModeObj.setFinalMilestoneValue(finalMilestoneValue - (0.5 * finalMilestoneValue));
        }
        rewardMilestones = blitzRetryLayer.getRewardMilestone();
        zenModeObj.setTotalSessionCount(0);
        var milestoneTile = utilObj.getScale9Sprite(constantsObj.trans_button, xPos, yPos - bigTileHeight / 2, 0.5, 0.5, tileWidth, bigTileHeight);
        utilObj.setColor(milestoneTile, constantsObj.COLOR_BLACK);
        scaleValue = milestoneTile.getScale();
        utilObj.setScale(milestoneTile, 0);
        //utilObj.setOpacity(milestoneTile, constantsObj.OPACITY_0);
        utilObj.addChild(curLayer, milestoneTile, zOrder);
        var milestoneText = utilObj.getBMFontLabel("Blitz reward refreshed!", constantsObj.FONT_HELVETICA_80, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), tileWidth / 2, tileHeight + 50, 0.5, 0.5);
        utilObj.addChild(milestoneTile, milestoneText, zOrder);
        milestoneTile.setCascadeOpacityEnabled(true);
        spritesToRemove.push(milestoneTile);

        var milestoneSubTile = utilObj.getScale9Sprite(constantsObj.white_rounded_bg, tileWidth / 2, tileHeight / 2, 0.5, 0.5, tileWidth, tileHeight);
        utilObj.setColor(milestoneSubTile, constantsObj.COLOR_BLITZ_MILESTONE);
        utilObj.setOpacity(milestoneSubTile, constantsObj.OPACITY_90);
        utilObj.addChild(milestoneTile, milestoneSubTile, zOrder);

        var progressBarWidth = 940;
        var progressBarHeight = 30;
        var progressBarStart = (tileWidth - progressBarWidth) / 2;
        var lastMilestoneValue = rewardMilestones[2];
        var giftBoxAnim = [];
        var progressPointer = [];
        var showProgressPointer = false;
        var progressPointerXPos = 0;
        var progressPointerYPos = 0;
        var progressPointerText = "";
        var tickIconAnim = null;
        var showGiftBoxAnim = false;
        var index = -1;
        var progressPointerWidth = 290;
        var progressPointerHeight = 95;
        var progressBarBg = utilObj.getScale9Sprite(constantsObj.full_bar, progressBarStart, tileHeight / 2 + 10, 0, 0.5, progressBarWidth, progressBarHeight);
        utilObj.setColor(progressBarBg, constantsObj.COLOR_BLACK);
        utilObj.addChild(milestoneSubTile, progressBarBg, zOrder);

        var var_scores = rewardIncrement + "_" + lastMilestoneValue + "_" + trackTotalSessionCount;
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_REWARD_RESET, constantsObj.TRACK_VIEW, var_scores, "", "", "");


        var len = (totalRewardCount / lastMilestoneValue) * progressBarWidth;
        var finalLen = 0;
        if (len == 0) {
            finalLen = (0.1 / lastMilestoneValue) * progressBarWidth;
        }
        var progressBar = utilObj.getScale9Sprite(constantsObj.tile_white, progressBarStart, tileHeight / 2 + 10, 0, 0.5, len + finalLen, progressBarHeight);
        utilObj.addChild(milestoneSubTile, progressBar, zOrder);
        utilObj.setColor(progressBar, constantsObj.COLOR_MILESTORE_PROGRESS);

        var giftBoxArr = [constantsObj.blitz_yellow_gift_box, constantsObj.blitz_pink_gift_box, constantsObj.blitz_blue_gift_box];
        var giftCapArr = [constantsObj.blitz_yellow_gift_cap, constantsObj.blitz_pink_gift_cap, constantsObj.blitz_blue_gift_cap];
        var scalingArr = [constantsObj.SCALE_ONE * 1.08 * 0.8 / 2, constantsObj.SCALE_ONE * 1.08 / 2, constantsObj.SCALE_ONE * 1.08 * 1.2 / 2];
        for (var i = 0; i < 3; i++) {
            var giftBoxIcon = utilObj.getSprite(giftBoxArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20, 0.5, 0.5, scalingArr[i]);
            var positionParams = utilObj.getSpriteDimensions(giftBoxIcon);
            var nodeCoordinates = utilObj.getSpriteCoordinates(giftBoxIcon);
            utilObj.setPosition(giftBoxIcon, nodeCoordinates.xPos, nodeCoordinates.yPos + (positionParams.height / 2));
            utilObj.addChild(milestoneSubTile, giftBoxIcon, zOrder);
            var giftCapIcon = utilObj.getSprite(giftCapArr[i], progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth, tileHeight / 2 - 20 + positionParams.height, 0.5, 0.5, scalingArr[i]);
            var capPositionParams = utilObj.getSpriteDimensions(giftCapIcon);
            var capNodeCoordinates = utilObj.getSpriteCoordinates(giftCapIcon);
            utilObj.setPosition(giftCapIcon, capNodeCoordinates.xPos, capNodeCoordinates.yPos + (capPositionParams.height / 2));
            utilObj.addChild(milestoneSubTile, giftCapIcon, zOrder);
            if (totalRewardCount + rewardIncrement < rewardMilestones[i] && index == -1) {
                showProgressPointer = true;
                progressPointerXPos = progressBarStart + (rewardMilestones[i] / lastMilestoneValue) * progressBarWidth;
                progressPointerYPos = tileHeight / 2 - 20;
                progressPointerText = rewardMilestones[i] - (totalRewardCount + rewardIncrement) + " points to go!";
                index = i;
                progressPointerWidth = 600;
            }
        }

        if (showProgressPointer) {
            var progressPointerIcon = utilObj.getSprite(constantsObj.themed_quest_progress_pointer, progressPointerXPos, 125, 0.5, 0, constantsObj.SCALE_SPRITE_11);
            utilObj.addChild(milestoneSubTile, progressPointerIcon, zOrder);
            //utilObj.setOpacity(progressPointerIcon, constantsObj.OPACITY_0);
            progressPointer.push(progressPointerIcon);

            var anchorX = 0.5;
            if (index == 2) {
                anchorX = 0.75;
            }
            var progressPointerTile = utilObj.getScale9Sprite(constantsObj.white_rounded_bg, progressPointerXPos, 125 - progressPointerHeight / 2, anchorX, 0.5, progressPointerWidth, progressPointerHeight);
            utilObj.addChild(milestoneSubTile, progressPointerTile, zOrder);
            //utilObj.setOpacity(progressPointerTile, constantsObj.OPACITY_0);
            progressPointer.push(progressPointerTile);
            progressPointerTile.setCascadeOpacityEnabled(true);
            var progressPointerTextValue = utilObj.getBMFontLabel(progressPointerText, constantsObj.FONT_HELVETICA_MEDIUM_70, constantsObj.COLOR_BLACK,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), progressPointerWidth / 2, progressPointerHeight / 2, 0.5, 0.5);
            utilObj.addChild(progressPointerTile, progressPointerTextValue, zOrder);
        }

        //scale up new milestone tile
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.NEW_MILESTNE_TILE_APPEAR,
            node: milestoneTile,
            animDelay: animDelay,
            animDur: 0.6,
            scale: scaleValue
        });
        animDelay += 0.6;

        animDelay = blitzRetryLayer.showMilestoneProgressAnim(len + finalLen, rewardIncrement, progressBar, totalRewardCount, lastMilestoneValue, animDelay + 0.1, giftBoxAnim);

        return animDelay;

    };

    /**
     * Milestone bar progress Animation
     */
    this.showMilestoneProgressAnim = function (len, rewardIncrement, progressBar, totalRewardCount, lastMilestoneValue, initDelay, giftBoxAnim) {
        var progressBar = progressBar;
        if (!utilObj.isNodeValid(progressBar)) {
            return initDelay;
        }
        var originalWidth = len;
        var totalWidth = 940;
        if (originalWidth < 0) {
            return initDelay;
        }
        if (rewardIncrement <= 0) {
            return initDelay;
        }
        var newWidth = ((totalRewardCount + rewardIncrement) / lastMilestoneValue) * totalWidth;
        var sy = progressBar.getScaleY();
        if (originalWidth == 0) {
            originalWidth = 0.1;
            newWidth += 0.1;
        }

        // for progress bar anim
        var delayAction = animObj.getDelayTime(initDelay);
        var scaleAction = animObj.getScaleDifferentTo(0.5, newWidth / originalWidth, sy);
        var actions = [delayAction, scaleAction];
        animObj.runAction(progressBar, animObj.getSequence(actions));

        initDelay += 0.5;
        var giftBoxDelay = 0;
        if (giftBoxAnim != undefined && giftBoxAnim.length > 0) {
            for (var i = 0; i < giftBoxAnim.length; i++) {
                if (giftBoxAnim[i] != null) {
                    giftBoxDelay = 0.3;
                    var origScale = giftBoxAnim[i].getScale();
                    var delayAction = animObj.getDelayTime(initDelay);
                    var scaleUp = new cc.ScaleTo(giftBoxDelay / 2, (origScale * 1.5)).easing((cc.easeOut(1)));
                    var scaleDown = new cc.ScaleTo(giftBoxDelay / 2, (origScale)).easing((cc.easeIn(1)));
                    var actions = [delayAction, scaleUp, scaleDown];
                    animObj.runAction(giftBoxAnim[i], animObj.getSequence(actions));
                }
            }
        }
        return initDelay + giftBoxDelay;
    };

    /**
     * Gift box animation
     */
    this.showGiftBoxCoinGrantAnim = function (milestoneTile, milestoneSubTile, xPos, yPos, index, zOrder, delay, giftBoxAnim) {
        var scaleDown = 0.75;
        var tileHeight = milestoneTile.getContentSize().height * scaleDown;
        var tileWidth = milestoneTile.getContentSize().width * scaleDown;
        var tileXPos = milestoneTile.getPositionX();
        var tileYPos = 60 + 200 * scalingArr;
        var tileHeightSubTile = milestoneSubTile.getContentSize().height * scaleDown;
        var heightDiff = tileHeight - tileHeightSubTile;
        var oldGiftYPos = 20 * scaleDown;
        var giftScale = 1;
        var giftTopScale = 1.2;
        var time = 0.5;
        var fadeOutArr = [];

        var startPosX =  xPos;//tileXPos + xPos;
        var startPosY = yPos;//tileYPos - heightDiff / 2 - oldGiftYPos;
        if (giftBoxAnim != undefined && giftBoxAnim.length > 0 && giftBoxAnim[0] != null) {
            startPosX = giftBoxAnim[0].getPositionX();
            startPosY = 60 + 200;//giftBoxAnim[0].getPositionY();
            if (index == 0) {
                startPosX += 50;
            }
        }
        var endPositionX = centerXPos;
        var endPositionY = centerYPos - 100;
        var giftBoxArr = [constantsObj.blitz_yellow_gift_box, constantsObj.blitz_pink_gift_box, constantsObj.blitz_blue_gift_box];
        var giftCapArr = [constantsObj.blitz_yellow_gift_cap, constantsObj.blitz_pink_gift_cap, constantsObj.blitz_blue_gift_cap];
        var scalingFactor = 4;
        var scalingArr = [constantsObj.SCALE_ONE * scalingFactor * 1.08 * 1.2 * 0.75 / 2, constantsObj.SCALE_ONE * scalingFactor * 1.08 * 1.2 * 0.75 / 2, constantsObj.SCALE_ONE * scalingFactor * 1.08 * 1.2 * 0.75 / 2];
        rewardMilestones = blitzRetryLayer.getRewardMilestone();
        var track_bz = "bz_reward_";
        track_bz += index;
        var track_milestone = index + "_" + rewardMilestones[index];
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), track_bz, constantsObj.TRACK_VIEW, track_milestone, "", "", "");

        // gift box cardinal spline anim
        var giftBot = utilObj.getSprite(giftBoxArr[index], startPosX, startPosY, 0.5, 0.5, scalingArr[index]);
        giftScale = scalingArr[index];
        utilObj.addChild(curLayer, giftBot, zOrder);
        fadeOutArr.push(giftBot);
        var giftTop = utilObj.getSprite(giftCapArr[index], giftBot.getContentSize().width / 2, giftBot.getContentSize().height, 0.5, 0, (scalingArr[index] * 2 / scalingFactor));
        utilObj.addChild(giftBot, giftTop, zOrder);
        utilObj.setScale(giftBot, 0);
        giftBot.setCascadeOpacityEnabled(true);
        spritesToRemove.push(giftBot);

        var start = cc.p(startPosX, startPosY);
        var end = cc.p(endPositionX, endPositionY);
        var path1Mid = cc.p(endPositionX + 50, endPositionY + endPositionY / 4);
        if (index == 0) {
            path1Mid = cc.p(endPositionX - 100, endPositionY + endPositionY / 4);
        } else if (index == 2) {
            path1Mid = cc.p(endPositionX + 200, endPositionY + endPositionY / 4);
        }
        //var path2Mid = cc.p(startPosX - 100, endPositionY + endPositionY / 2);
        var path1 = [start, path1Mid, end];
        var moveAnim1 = utilObj.getCardinalSpline(time, path1, 0);
        var easeOutMove1 = moveAnim1.easing(cc.easeOut(2));

        var animNew = [];
        animNew.push(animObj.getDelayTime(delay));
        animNew.push(utilObj.getScaleTo(0, giftScale / scalingFactor));
        var spawn = new cc.Spawn(utilObj.getScaleTo(time, giftScale), easeOutMove1);
        animNew.push(spawn);
        animObj.runAction(giftBot, animObj.getSequence(animNew));

        //opacity drop for milestone
        // if (giftBoxAnim != undefined && giftBoxAnim.length > 0 && giftBoxAnim[0] != null) {
        //     for (var i = 0; i < giftBoxAnim.length; i++) {
        //         animObj.blitzRewardAnimation(curLayer, {
        //             case: constantsObj.FADE_OUT_GIFT_ITEM,
        //             node: giftBoxAnim[i],
        //             animDelay: delay,
        //             animDur: 0,
        //             opacity: constantsObj.OPACITY_60
        //         });
        //     }
        // }

        var anims = [];
        anims.push(animObj.getDelayTime(delay));
        anims.push(animObj.getDelayTime(time));
        anims.push(animObj.getScaleDifferentTo(0.1, giftScale, giftScale * 0.9));
        anims.push(animObj.getScaleDifferentTo(0.1, giftScale, giftScale * 1.1));
        anims.push(utilObj.getScaleTo(0.1, giftScale));
        animObj.runAction(giftBot, animObj.getSequence(anims));

        var giftBot2 = utilObj.getSprite(giftBoxArr[index], centerXPos, centerYPos - (giftBot.getContentSize().height * giftScale) / 2 - 100, 0.5, 0, scalingArr[index]);
        utilObj.addChild(curLayer, giftBot2, zOrder);
        fadeOutArr.push(giftBot2);
        var giftTop2 = utilObj.getSprite(giftCapArr[index], centerXPos, giftBot2.getPositionY() + giftBot2.getContentSize().height * giftScale, 0.5, 0, scalingArr[index]);
        utilObj.addChild(curLayer, giftTop2, zOrder);
        utilObj.setVisible(giftBot2, false);
        utilObj.setVisible(giftTop2, false);
        fadeOutArr.push(giftTop2);
        spritesToRemove.push(giftBot2);
        spritesToRemove.push(giftTop2);


        delay += 0.3;
        utilObj.delayedCallback(curLayer, function () {
            utilObj.setVisible(giftBot, false);
            utilObj.setVisible(giftBot2, true);
            utilObj.setScale(giftBot2, giftScale);
            utilObj.setVisible(giftTop2, true);
        }, delay + time, null, false);

        // gift top open
        var animTopOpen = [];
        animTopOpen.push(animObj.getDelayTime(delay));
        animTopOpen.push(animObj.getDelayTime(time + 0.6));
        animTopOpen.push(animObj.getCardinalSplineTo(0.25, [{
            x: centerXPos - 100,
            y: centerYPos + 300 - 100
        }, {x: centerXPos - 250, y: centerYPos + 600}, {
            x: centerXPos - 350,
            y: giftBot2.getPositionY() - 30
        }], 0));
        animTopOpen.push(animObj.getScaleDifferentTo(0.05, giftScale * giftTopScale, giftScale * giftTopScale * 0.8));
        animTopOpen.push(animObj.getScaleDifferentTo(0.05, giftScale * giftTopScale, giftScale * giftTopScale));
        animObj.runAction(giftTop2, animObj.getSequence(animTopOpen));

        // reward anim after opening
        var rewardIconLocal = constantsObj.coin_png_2;
        coinAmount = blitzRetryLayer.getBlitzCoinAmount();
        var coins = coinAmount;
        var rewardIconLocalScale = 1.3;
        var rewardStrLocal = "+" + coins;
        var rewardStrLocalScale = 1.5;
        var rewardLbl = utilObj.getBMFontLabel(rewardStrLocal, constantsObj.FONT_ROBOTO_BOLD_40, constantsObj.COLOR_WHITE, constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), centerXPos, centerYPos + 110 + 40 + 50, 0.5, 0.5);
        utilObj.addChild(curLayer, rewardLbl, zOrder);
        fadeOutArr.push(rewardLbl);
        //utilObj.setScale(rewardLbl, 0);
        utilObj.setOpacity(rewardLbl, constantsObj.OPACITY_0);
        spritesToRemove.push(rewardLbl);
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.REWARD_LBL_FADE_IN,
            node: rewardLbl,
            animDelay: delay + time + 0.6 + 0.25 + 0.1 + 0.15,
            animFadeDur: 0.15,
            animMoveDur: 0.15,
            scaleUp: 3,
            opacity: constantsObj.OPACITY_100
        });

        // handling coin move animation delay here
        var zenModeCoinImgClone = utilObj.getSpriteClone(zenModeLayer.cashIcon);
        utilObj.addChild(curLayer, zenModeCoinImgClone, zOrder);
        var zenModeCoinLblClone = utilObj.getLabelClone(zenModeLayer.cashLbl, constantsObj.FONT_BM);
        utilObj.addChild(curLayer, zenModeCoinLblClone, zOrder);
        economyObj.addToAdditionalCashLabels(zenModeCoinLblClone);
        fadeOutArr.push(zenModeCoinImgClone);
        fadeOutArr.push(zenModeCoinLblClone);
        //

        var rewardCoin = utilObj.getSprite(rewardIconLocal, centerXPos, centerYPos + 110 + 40, 0.5, 0.5, 0.75);
        utilObj.addChild(curLayer, rewardCoin, zOrder);
        utilObj.setOpacity(rewardCoin, 0);
        fadeOutArr.push(rewardCoin);

        var zenModeCoinImgClonePos = utilObj.getSpriteCoordinates(zenModeCoinImgClone);
        var zenModeCoinImgCloneScale = utilObj.getScale(zenModeCoinImgClone);
        var zenModeCoinImgCoinDimensions = utilObj.getSpriteDimensions(zenModeCoinImgClone);

        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.REWARD_COIN_FADE_IN,
            node: rewardCoin,
            animDelay: delay + time + 0.6 + 0.25 + 0.1,
            animFadeDur: 0.15,
            animMoveDur: 0.3,
            scaleUp: 3,
            opacity: constantsObj.OPACITY_100
        });


        var rewardRay = utilObj.getSprite(constantsObj.rays, centerXPos, centerYPos - 100 - 100 + 225, 0.5, 0, 1);
        utilObj.addChild(curLayer, rewardRay, zOrder);
        spritesToRemove.push(rewardRay);
        utilObj.setOpacity(rewardRay, 0);
        utilObj.setScale(rewardRay, 0);
        fadeOutArr.push(rewardRay);
        animObj.blitzRewardAnimation(curLayer, {
            case: constantsObj.REWARD_RAY_ANIM,
            node: rewardRay,
            animDelay: delay + time + 0.6 + 0.1,
            animDur: 0.2,
            scaleUp: 1.75,
            opacity: constantsObj.OPACITY_80
        });

        var params1 = {
            x: centerXPos,
            y:  centerYPos + 60,
            layer: curLayer,
            zOrder: zOrder,
            duration: 0.15 + 0.3,
            scale: 0.75
        };
        utilObj.delayedCallback(curLayer, effectsObj.playWeeklyStarEffect, delay + time + 0.6 + 0.1, params1, false);
        delay += time + 0.6 + 0.25 + 0.1 + 0.15 + 0.3;

        //
        var numCoins = 5;
        var moveDuration = 0.5;
        var fadeInDuration = 0.5;
        var timeGap = 0.1;
        var timeOffset = 0.10;
        var targetPosition = {xPos: zenModeCoinImgClonePos.xPos, yPos: zenModeCoinImgClonePos.yPos + (zenModeCoinImgCoinDimensions.height/2)};
        utilObj.delayedCallback(curLayer, function () {
            // generate numCoins clone nodes here and run animation on each of them using a loop
            var cloneNode = null;
            for(var i = 0; i < numCoins; i++){
                cloneNode = utilObj.getSpriteClone(rewardCoin);
                utilObj.setOpacity(cloneNode, constantsObj.OPACITY_0);
                utilObj.addChild(curLayer, cloneNode, zOrder);

                animObj.blitzRewardAnimation(curLayer, {
                    case: constantsObj.REWARD_COIN_MOVE_DOWN_AND_FADE_OUT,
                    node: cloneNode,
                    animDelay: timeGap * (numCoins - (i + 1)),
                    animFadeDur: fadeInDuration,
                    animMoveDur: moveDuration,
                    scaleUp: 1,
                    opacity: constantsObj.OPACITY_100,
                    moveToPos: targetPosition,
                    scaleTo: zenModeCoinImgCloneScale
                });
            }
            //
        }, delay + timeOffset, null, false);
        delay += ((numCoins - 1) * timeGap + fadeInDuration + moveDuration + timeOffset);
        //


        utilObj.delayedCallback(curLayer, function () {
            economyObj.addCash(coinAmount, constantsObj.TRACK_EARN, track_bz, "bz");
        }, delay - 0.15, null, false);


        // fade out gift items
        for (var i = 0; i < fadeOutArr.length; i++) {
            var delayTime = 0.8;
            if (fadeOutArr[i] == rewardRay || fadeOutArr[i] == rewardCoin || fadeOutArr[i] == rewardLbl) {
                delayTime = 0.6;
            }
            animObj.blitzRewardAnimation(curLayer, {
                case: constantsObj.FADE_OUT_GIFT_ITEM,
                node: fadeOutArr[i],
                animDelay: delay + delayTime,
                animDur: 0.2,
                opacity: constantsObj.OPACITY_0
            });
        }
        delay += 0.8 + 0.2;
        return delay;
    };

    this.showFinalOutro = function(){
        var level = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BLITZ_OUTRO, constantsObj.TRACK_VIEW, level, zenModeObj.getLivesCount(), "", "");

        utilObj.hideBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));

        var spriteArr = [];
        var labelArr = [];
        var arrayToAnimate = [];
        var delayForAnim = [0.1, 0.4, 0.6 ,0.6, 1 , 1, 1];

        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset - constantsObj.BEZEL_SAFE_AREA;

        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        var deviceHeightRatio = (constantsObj.DESIGN_HEIGHT + topBarOffset) / 2560;
        var deviceWidthRatio = canvasWidth / 1440;


        // cross button
        var back_btn = utilObj.getSprite(constantsObj.close_icon2, xRightPos - (50 * deviceWidthRatio), yPos - (52 * deviceHeightRatio), 1, 1, constantsObj.SCALE_SPRITE_43);
        utilObj.addChild(curLayer, back_btn, zOrder);
        utilObj.setOpacity(back_btn, 0);
        spritesToRemove.push(back_btn);

        var closeBtn = utilObj.getButton(100 * deviceWidthRatio, 100 * deviceHeightRatio, 1, 1, xRightPos - (35 * deviceWidthRatio) , yPos - (36 * deviceHeightRatio),
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'blitzRetryLayer',
                fn: 'close',
                screen: zenModeLayer.screen
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(curLayer, closeBtn, zOrder);
        utilObj.setOpacity(closeBtn, 0);
        spritesToRemove.push(closeBtn);

        yPos -= 211;

        var levelText = "";

        var currentLevel = zenModeObj.getCurrentLevel();
        if (currentLevel >= constantsObj.MAX_LEVEL + 1) {
            levelText = "Well played!";
        }else{
            levelText = "Level " + currentLevel;
        }
        var loseLifeBlitz = utilObj.getBMFontLabel(levelText, constantsObj.FONT_HELVETICA_MEDIUM_120, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, curLayer, zOrder);
        //utilObj.setOpacity(loseLifeBlitz, constantsObj.OPACITY_90);
        utilObj.setOpacity(loseLifeBlitz, 0);
        spritesToRemove.push(loseLifeBlitz);
        arrayToAnimate.push(loseLifeBlitz);

        yPos -= 419;

        //var brokenHeart = utilObj.getSprite(constantsObj.broken_heart, xPos, yPos, 0.5, 1.0, constantsObj.SCALE_SPRITE_1, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        //spritesToRemove.push(brokenHeart);
        //utilObj.setOpacity(brokenHeart, 0);

        var yourScoreMessage = utilObj.getBMFontLabel("Your Score", constantsObj.FONT_HELVETICA_MEDIUM_100, constantsObj.COLOR_BACK,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), xPos  , yPos, 0.5, 1, labelArr, curLayer, zOrder);
        spritesToRemove.push(yourScoreMessage);
        utilObj.setOpacity(yourScoreMessage, 0);

        yPos -= 120;
        var scoreLabel = zenModeObj.getCurrentScore() + "";

        var score = utilObj.getBMFontLabel(scoreLabel, constantsObj.FONT_HELVETICA_230, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), xPos  , yPos, 0.5, 1, labelArr, curLayer, zOrder);
        spritesToRemove.push(score);
        utilObj.setOpacity(score, 0);


        yPos -= 630;

        var timerText = utilObj.getBMFontLabel("", constantsObj.FONT_HELVETICA_75, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, curLayer, zOrder);

        zenModeObj.setTimerText2(timerText);
        spritesToRemove.push(timerText);
        utilObj.setOpacity(timerText, 0);
        arrayToAnimate.push(timerText);




        yPos -= 100;

        var hearts_array = [];
        var maxLive = runtimeObj.getMaxLivesForBlitz();
        var startLife = Math.floor(maxLive/2);
        var startingPos = centerXPos - ((startLife) * (110 * canvasWidth / 1440));
        for (var i = 0; i < maxLive; i++) {
            var heart_empty = utilObj.getSprite(constantsObj.blitz_white_heart, (startingPos * canvasWidth / 1440) + i * (110 * canvasWidth / 1440), yPos - 40, 0.5, 0.5, constantsObj.SCALE_SPRITE_19, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
            heart_empty.setVisible(false);
            hearts_array.push(heart_empty);
        }
        var params = {
            hearts_array : hearts_array
        };

        zenModeObj.setLivesOnRetryLayer(params);

        var lives = zenModeObj.getLivesCount();
        if(lives >= maxLive){
            lives = maxLive;
        }

        var heart_emptyForAnim = utilObj.getSprite(constantsObj.blitz_white_heart, (startingPos * canvasWidth / 1440) + lives * (110 * canvasWidth / 1440), yPos - 40, 0.5, 0.5, constantsObj.SCALE_SPRITE_19, spriteArr, curLayer, zOrder, constantsObj.SPRITE_MISC);
        utilObj.setColor(heart_emptyForAnim, constantsObj.COLOR_BLITZ_RED_HEART);
        heart_emptyForAnim.setOpacity(0);

        for (var i = 0; i < lives; ++i) {
            if (utilObj.isNodeValid(hearts_array[i])) {
                utilObj.setColor(hearts_array[i], constantsObj.COLOR_BLITZ_RED_HEART);
                hearts_array[i].setVisible(true);
            }
        }
        for (var i = lives; i < maxLive; ++i) {
            if (utilObj.isNodeValid(hearts_array[i])) {
                utilObj.setColor(hearts_array[i], constantsObj.COLOR_GREY_10);
                hearts_array[i].setVisible(true);
            }
        }



        //arrayToAnimate.push(brokenHeart);
        arrayToAnimate.push(yourScoreMessage);
        arrayToAnimate.push(score);



        yPos -= 300;


        var fn = 'retry';
        var text = "Retry";
        if(zenModeObj.getLivesCount()<= 0){
            fn = 'close';
            text = "Back";
        }
        //retry button or back btn
        var retryButton = utilObj.getButton(650, 225, 0.5, 1.0, centerXPos, yPos,
            constantsObj.newButton_cutup, constantsObj.newButton_cutup, constantsObj.newButton_cutup, {
                obj: 'blitzRetryLayer',
                fn: fn,
                screen: zenModeLayer.screen,
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.setColor(retryButton, constantsObj.COLOR_GREEN_11);
        utilObj.addChild(curLayer, retryButton, zOrder);
        utilObj.pulseButton(retryButton, 34, 0);
        var retryText = text;
        spritesToRemove.push(retryButton);

        var retryButtonTxt = utilObj.getBMFontLabel(retryText, constantsObj.FONT_HELVETICA_MEDIUM_100, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_CENTER, cc.p(0, 0), 325, 112.5, 0.5, 0.5, labelArr, retryButton, zOrder);
        utilObj.pulseButton(retryButtonTxt, 35, 0);
        spritesToRemove.push(retryButtonTxt);

        retryButton.setCascadeOpacityEnabled(true);
        utilObj.setOpacity(retryButton, 0);
        arrayToAnimate.push(retryButton);

        arrayToAnimate.push(back_btn);
        arrayToAnimate.push(closeBtn);


        var totalDelay = 2.4;
        for (var i = 0; i < arrayToAnimate.length; ++i) {
            if (utilObj.isNodeValid(arrayToAnimate[i])) {
                blitzRetryLayer.animateOutro(arrayToAnimate[i] , delayForAnim[i]);
            }
        }
        for (var i = 0; i < hearts_array.length; ++i) {
            if (utilObj.isNodeValid(hearts_array[i])) {
                blitzRetryLayer.animateOutro(hearts_array[i], 0.4);
            }
            if(i == 0){
                if(lives < maxLive){
                    if(utilObj.isNodeValid(heart_emptyForAnim)){
                        blitzRetryLayer.animateOutro(heart_emptyForAnim, 0);
                    }
                }
            }


        }
        var param = {
            node : heart_emptyForAnim
        };

        utilObj.delayedCallback(zenModeLayer, blitzRetryLayer.animateHeart , totalDelay , param , false);

        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);

    };

    this.animateHeart = function(sender , params){
        var heart = params.node;
        if(!utilObj.isNodeValid(heart)){
            return ;
        }
        var scaleNormal = heart.getScale();
        var scaleUp = utilObj.getScaleTo(0.5 , 1.5 * scaleNormal);
        var scaleDown = utilObj.getScaleTo(0.5 , 0);
        var animArr = [scaleUp , scaleDown];
        animObj.runAction(heart , utilObj.getSequence(animArr));
    };


    this.animateOutro = function(element , delay){
        var delay = utilObj.getDelayTime(delay);
        var fadeIn = utilObj.getFadeIn(0.3).easing(new cc.easeIn(1.0));
        var animArr = [];
        animArr.push(delay);
        animArr.push(fadeIn);
        animObj.runAction(element , utilObj.getSequence(animArr));
    };

    this.close = function(){
        var level = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BLITZ_OUTRO, constantsObj.TRACK_BACK, level, zenModeObj.getLivesCount(), "", "");

        zenModeObj.setBlitzModeOn(false);
        zenModeObj.setStartAfterPopup(false);
        zenModeObj.setStartAfterPopup(false);
        zenModeObj.setTransLayerOpen(false);
        blitzRetryLayer.hide();
        zenModeLayer.hide();
        homeLayer.show();
    };

    this.retry = function(){
        var level = zenModeObj.getCurrentLevel();
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, zenModeObj.getBlitzPuzzleTracking(), zenModeObj.getBlitzPuzzleTheme(), constantsObj.TRACK_BLITZ_OUTRO, constantsObj.TRACK_BLITZ_RETRY, level, zenModeObj.getLivesCount(), "", "");

        utilObj.showBannerAd(utilObj.getScreenName(constantsObj.SCREEN_ZEN_MODE));
        blitzRetryLayer.hide();
        zenModeLayer.hide();
        zenModeLayer.loadBlitzMode();
        zenModeObj.setTransLayerOpen(false);
    };

    /**
     * blitz coin amount logic
     */
    this.getBlitzCoinAmount = function () {
        var revValue = spinnerObj.getSumOfRevStatus();
        var blitzRevStatusHigh = runtimeObj.getBlitzRewardRevStatusHigh();
        var blitzRevStatusLow = runtimeObj.getBlitzRewardRevStatusLow();
        var blitzRewardPayer = runtimeObj.getBlitzRewardValuePayer();
        var blitzRewardHigh = runtimeObj.getBlitzRewardValueHigh();
        var blitzRewardMed = runtimeObj.getBlitzRewardValueMed();
        var blitzRewardLow = runtimeObj.getBlitzRewardValueLow();
        if (gameObj.getIsPayer()) {
            return blitzRewardPayer;
        } else {
            if (revValue <= blitzRevStatusLow) {
                return blitzRewardHigh;
            } else if (revValue > blitzRevStatusLow && revValue <= blitzRevStatusHigh) {
                return blitzRewardMed;
            } else {
                return blitzRewardLow;
            }
        }
        return blitzRewardHigh;
    };

    /**
     * to get the next nearest multiplier of fifty
     */
    this.getNextMultipleFifty = function (val) {
        var nextVal = val
        if ((val % 50) != 0) {
            nextVal = val - (val % 50);// taking the smaller nearest multiple of 50.
        }
        return nextVal;
    };

    /**
     * reward milestone value logic
     */
    this.getRewardMilestone = function () {
        for (var i = 2; i >= 0; i--) {
            if (i == 2) {
                if (zenModeObj.getFinalMilestoneValue() == 0 || zenModeObj.getFinalMilestoneValue() < runtimeObj.getBlitzFinalMilestoneCap()) {
                    if (zenModeObj.getFinalMilestoneValue() == 0) {
                        rewardMilestones[i] = runtimeObj.getBlitzFirstTimeMilestoneValue();
                    } else {
                        rewardMilestones[i] = runtimeObj.getBlitzFinalMilestoneCap();
                    }
                    zenModeObj.setFinalMilestoneValue(rewardMilestones[i]);
                }else {
                    rewardMilestones[i] = zenModeObj.getFinalMilestoneValue();
                }
                rewardMilestones[i] = blitzRetryLayer.getNextMultipleFifty(rewardMilestones[i]);
            } else {
                rewardMilestones[i] = rewardMilestones[2] * (i + 1) / 3;
                rewardMilestones[i] = blitzRetryLayer.getNextMultipleFifty(rewardMilestones[i]);
            }
        }
        return rewardMilestones;
    };
};