QuestMapTransLayer = function () {
    var spritesToRemove = [];
    var spriteArr = [];
    var labelArr = [];
    var spriteDict = {};
    var questMapTransLayerVisible = false;

    /**
     * this will work similar to the layer hide functions
     * it will remove all the elements on the screen pushed to arr spritesToRemove
     */
    this.hide = function () {
        for (var i in spritesToRemove) {
            if (utilObj.isNodeValid(spritesToRemove[i])) {
                utilObj.removeChild(mainScene, spritesToRemove[i]);
            }
        }
        spritesToRemove = [];
    };

    /**
     * shows the questMapTransLayer with animation and opens questMapLayer
     */
    this.showAndChangeLayer = function () {
        if(questMapTransLayerVisible){
            return;
        }
        questMapTransLayerVisible = true;
        var animDelay = questMapTransLayer.show();
        questMapObj.setExtraDelayForSurfacingTodaysChallengeSection(animDelay/2);
        utilObj.delayedCallback(curLayer, questMapTransLayer.showQuestMapLayer, animDelay/2, null, false);
        utilObj.delayedCallback(mainScene, questMapTransLayer.hide, animDelay, null, false);
        utilObj.delayedCallback(mainScene, questMapTransLayer.resetQuestMapTransLayerVisible, animDelay, null, false);
    };

    /**
     * here all the elements will be shown on the screen, all the elements will have the parentLayer as currentScreen
     */
    this.show = function () {
        trackObj.trackCounter(constantsObj.TRACK_DIALOG, "" , "", "wq_icon_overlay", constantsObj.TRACK_VIEW, "", "", "", "");
        var zOrder = constantsObj.FTUE_Z_ORDER;
        var xPos = centerXPos;
        var yPos = constantsObj.DESIGN_HEIGHT + topBarOffset;
        spritesToRemove = [];
        spriteArr = [];
        labelArr = [];
        spriteDict = {};

        // creating a transBg to which all the screen elements will be added
        var transBg = utilObj.getScale9Sprite(constantsObj.trans_button, xPos, 0, 0.5, 0, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset, spriteArr, mainScene, zOrder, constantsObj.SPRITE_MISC);
        spritesToRemove.push(transBg);
        // creating a button covering the entire screen
        var transBtn = utilObj.getButton(canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset, 0.5, 0, xPos, 0,
            constantsObj.trans_button, constantsObj.trans_button, constantsObj.trans_button, {
                obj: 'questMapTransLayer',
                fn: 'doNothing',
                screen: mainScene,
                params: {sound: false},
            }, false, ccui.Widget.PLIST_TEXTURE, 1);
        utilObj.addChild(transBg, transBtn, zOrder);

        // black overlay
        var overlayBg = utilObj.getScale9Sprite(constantsObj.tile_white, xPos, 0, 0.5, 0, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset, spriteArr, transBg, zOrder, constantsObj.SPRITE_MISC);
        utilObj.setColor(overlayBg, constantsObj.COLOR_BLACK);
        utilObj.setOpacity(overlayBg, constantsObj.OPACITY_0);
        spriteDict["overlayBg"] = overlayBg;

        // parent section for containing all the ui elements of the screen
        spriteDict["parentSection"] = utilObj.getScale9Sprite(constantsObj.trans_button, xPos, (constantsObj.DESIGN_HEIGHT + topBarOffset)/2, 0.5, 0.5, canvasWidth, constantsObj.DESIGN_HEIGHT + topBarOffset, spriteArr, transBg, zOrder, constantsObj.SPRITE_MISC);
        utilObj.setCascadeOpacityEnabled(spriteDict["parentSection"], true);

        // calculating year, weekNum, experiment values here for getting weeklyQuest data
        var experiment = gameObj.getExpNewWeeklyQuestVariant();
        var now = new Date();
        var year = questMapObj.y2k(now.getYear());
        var month = now.getMonth();
        var date = now.getDate();
        var weekNum = questMapObj.getWeek(year, month, date);
        //
        // header here
        yPos -= 720;
        spriteDict["header"] = utilObj.getBMFontLabel(questMapObj.getWeeklyThemeText(year, weekNum, experiment), constantsObj.FONT_HELVETICA_BOLD_80, constantsObj.COLOR_WHITE,
            constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, spriteDict["parentSection"], zOrder);
        utilObj.setOpacity(spriteDict["header"], constantsObj.OPACITY_0);
        //

        // image here
        yPos -= 512;

        spriteDict["themeImage"] = utilObj.getSprite(questMapObj.getWeeklyThemeImage(year, weekNum, experiment), xPos, yPos, 0.5, 0.5, constantsObj.SCALE_SPRITE_48, spriteArr, spriteDict["parentSection"], zOrder, constantsObj.SPRITE_MISC);
        utilObj.setOpacity(spriteDict["themeImage"], constantsObj.OPACITY_0);
        //

        // sub headers here
        yPos -= 470;
        spriteDict["subHeaders"] = [];
        var descriptionText = stringsObj.quest_transition_desc;
        var txtArr = utilObj.getSplitStringsArr(descriptionText, 40);
        var numLines = txtArr.length;
        for(var i = 0; i < numLines; i++){
            var subHeader = utilObj.getBMFontLabel(txtArr[i], constantsObj.FONT_HELVETICA_60, constantsObj.COLOR_YELLOW,
                constantsObj.LABEL_AUTO_WIDTH, cc.TEXT_ALIGNMENT_LEFT, cc.p(0, 0), xPos, yPos, 0.5, 1, labelArr, spriteDict["parentSection"], zOrder);
            utilObj.setOpacity(subHeader, constantsObj.OPACITY_0);
            yPos -= 70;
            spriteDict["subHeaders"].push(subHeader);
        }
        //

        // transition animation run here
        var animDelay = questMapTransLayer.runTransitionAnimation(spriteDict);
        //

        utilObj.addSprites(spriteArr);
        utilObj.addLabels(labelArr);
        return animDelay;
    };

    this.showQuestMapLayer = function () {
        homeLayer.hide();
        questMapLayer.show();
    };

    /**
     * this function does nothing
     * added to prevent background button presses
     */
    this.doNothing = function () {
        // does nothing
    };

    /**
     * this function runs transition animation for quest map tarns Layer
     * @param spriteDictionary
     */
    this.runTransitionAnimation = function (spriteDictionary) {
        if(!spriteDictionary){
            loggerObj.debugLog("questMapTransLayerLog: spriteDictionary is null or undefined");
            return;
        }

        var overlayBg = spriteDictionary["overlayBg"];
        var parentSection = spriteDictionary["parentSection"];
        var header = spriteDictionary["header"];
        var themeImage = spriteDictionary["themeImage"];
        var subHeaders = spriteDictionary["subHeaders"];

        var animDelay = 0;
        // black Overlay animation here
        animDelay = Math.max(animDelay, animObj.runOverlayAnimation(overlayBg));

        // alpha change animation here for header, themeImage and subHeaders
        animDelay = Math.max(animDelay, animObj.fadeInImageAndTextElements(header, themeImage, subHeaders));

        // parentSection animation here
        animDelay = Math.max(animDelay, animObj.runParentSectionAnimation(parentSection));

        return animDelay;
    };
    /**
     * base conditions check for showing home to quest map transition animation
     * @returns {boolean}
     */
    this.shouldShowHomeToQuestMapTransition = function () {
        if(!questMapLayer.areAllQuestPuzzlesCompleted() && questMapObj.isQuestMapAutoOn()){
            return true;
        }
        return false;
    };

    /**
     * resetting questMapTransLayerVisible variable here
     */
    this.resetQuestMapTransLayerVisible = function () {
        questMapTransLayerVisible = false;
    };

    /**
     * gets the value of the variable questMapTransLayerVisible
     * @returns {boolean}
     */
    this.getQuestMapTransLayerVisible = function () {
        return questMapTransLayerVisible;
    };
};

